
//#define DEBUG_VS   // Uncomment this line to debug vertex shaders 
//#define DEBUG_PS   // Uncomment this line to debug pixel shaders 

using System;
using System.Collections.Generic;
using H2UtilityToolkit;
using H2UtilityToolkit.Structures;
using MDXUtilityToolkit;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Prophet
{
    public class Form1 : IFrameworkCallback, IDeviceCreation
    {
        #region Instance Data

        #region Constants

        private const string Controls_F1_to_hide_string = "Controls (F1 to hide):";
        private const string Open_another_map_string = "Open another map: Ctrl + O";
        private const string Select_different_BSP_string = "Select different BSP: Ctrl + Q";
        private const string Change_camera_speed_templateString = "Change Camera speed: +/- (Current: {0})";
        private const string Quit_string = "Quit: Esc";
        private const string Hide_help_string = "Hide help: F1";
        private const string Press_F1_for_help_string = "Press F1 for help";

        #endregion

        private Map __map__;
        private Font __statsFont__;
        private Sprite __textSprite__;
        //private Effect __effect__;
        //private EffectHandle __eh_wvp__;
        //private EffectHandle __eh_world__;
        //private EffectHandle __eh_time__;
        //private EffectHandle __eh_matAmbient__;
        //private EffectHandle __eh_matDiffuse__;
        private FlyCamera __camera__;
        private bool __isHelpShowing__;
        private Dialog __hud__;
        private string __statusText__;
        private string __errorMsg__;
        private ShaderEditPanel __shaderEditPanel__;
        private ViewerState __lastViewState__;
        private List<int> __bspPathsIndices__;
        private Dialog __bspPathsDialog__;
        private ListBox __bspPathsList__;
        private string __change_camera_speed_string__;
        private ViewerState __vs__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// StatsFont
        /// </summary>
        private Font StatsFont { get { return __statsFont__; } set { __statsFont__ = value; } }
        /// <summary>
        /// TextSprite
        /// </summary>
        private Sprite TextSprite { get { return __textSprite__; } set { __textSprite__ = value; } }
        /// <summary>
        /// Effect
        /// </summary>
        //private Effect Effect { get { return __effect__; } set { __effect__ = value; } }
        /// <summary>
        /// Eh_wvp
        /// </summary>
        //private EffectHandle Eh_wvp { get { return __eh_wvp__; } set { __eh_wvp__ = value; } }
        /// <summary>
        /// Eh_world
        /// </summary>
        //private EffectHandle Eh_world { get { return __eh_world__; } set { __eh_world__ = value; } }
        /// <summary>
        /// Eh_time
        /// </summary>
        //private EffectHandle Eh_time { get { return __eh_time__; } set { __eh_time__ = value; } }
        /// <summary>
        /// Eh_matAmbient
        /// </summary>
        //private EffectHandle Eh_matAmbient { get { return __eh_matAmbient__; } set { __eh_matAmbient__ = value; } }
        /// <summary>
        /// Eh_matDiffuse
        /// </summary>
        //private EffectHandle Eh_matDiffuse { get { return __eh_matDiffuse__; } set { __eh_matDiffuse__ = value; } }
        /// <summary>
        /// Camera
        /// </summary>
        private FlyCamera Camera { get { return __camera__; } set { __camera__ = value; } }
        /// <summary>
        /// IsHelpShowing
        /// </summary>
        private bool IsHelpShowing { get { return __isHelpShowing__; } set { __isHelpShowing__ = value; } }
        /// <summary>
        /// Hud
        /// </summary>
        private Dialog Hud { get { return __hud__; } set { __hud__ = value; } }
        /// <summary>
        /// StatusText
        /// </summary>
        private string StatusText { get { return __statusText__; } set { __statusText__ = value; } }
        /// <summary>
        /// ErrorMsg
        /// </summary>
        private string ErrorMsg { get { return __errorMsg__; } set { __errorMsg__ = value; } }
        /// <summary>
        /// ShaderEditPanel
        /// </summary>
        public ShaderEditPanel ShaderEditPanel { get { return __shaderEditPanel__; } set { __shaderEditPanel__ = value; } }
        /// <summary>
        /// LastViewState
        /// </summary>
        private ViewerState LastViewState { get { return __lastViewState__; } set { __lastViewState__ = value; } }
        /// <summary>
        /// BspPathsIndices
        /// </summary>
        private List<int> BspPathsIndices { get { return __bspPathsIndices__; } set { __bspPathsIndices__ = value; } }
        /// <summary>
        /// BspPathsDialog
        /// </summary>
        private Dialog BspPathsDialog { get { return __bspPathsDialog__; } set { __bspPathsDialog__ = value; } }
        /// <summary>
        /// BspPathsList
        /// </summary>
        private ListBox BspPathsList { get { return __bspPathsList__; } set { __bspPathsList__ = value; } }
        /// <summary>
        /// TxtHelper
        /// </summary>
        private TextHelper TxtHelper;
        /// <summary>
        /// Change_camera_speed_string
        /// </summary>
        private string Change_camera_speed_string { get { return __change_camera_speed_string__; } set { __change_camera_speed_string__ = value; } }
        /// <summary>
        /// Vs
        /// </summary>
        private ViewerState ViewState
        {
            get { return __vs__; }
            set
            {
                switch (__vs__ = value)
                {
                    case ViewerState.Started:
                        {
                            StatusText = "Press any key to begin";
                            break;
                        }
                    case ViewerState.Error:
                        {
                            StatusText = "Error!\n\"" + ErrorMsg + "\"\nPress any key to retry";
                            break;
                        }
                    case ViewerState.Loading:
                        {
                            StatusText = "Loading...";
                            break;
                        }
                    case ViewerState.NoBSP:
                        {
                            //font.RenderText = renderTexts[4];
                            //font.DestinationRect = fontPositions[0];
                            //font.Color = Color.Goldenrod;
                            StatusText = "No sbsp tags found!\nPress any key to retry";
                            break;
                        }
                    case ViewerState.SelectorScreen:
                        {
                            StatusText = "To select press [Enter] \nTo return press [Backspace]";
                            break;
                        }
                    case ViewerState.Viewing:
                        {
                            StatusText = string.Empty;

                            if (ShaderEditPanel == null)
                            {
                                ShaderEditPanel = new ShaderEditPanel(Map.MDXFrameWork);
                                ShaderEditPanel.SEP_ReloadComboBoxes();
                                int x = 10;
                                int y = 10;
                                ShaderEditPanel.SetLocation(x, y);
                                ShaderEditPanel.SetSize(
                                    200,
                                    Map.MDXFrameWork.BackBufferSurfaceDescription.Height - (y * 2));
                            }

                            break;
                        }
                }

                HideShaderEditPanel();
            }
        }

        #endregion

        public Form1(Map map)
        {
            this.Map = map;
            this.IsHelpShowing = true;
            this.Camera = new FlyCamera(map, 1.0f);
            this.Hud = new Dialog(map.MDXFrameWork);
            this.TxtHelper = new TextHelper(15);
            this.Change_camera_speed_string = string.Format(Change_camera_speed_templateString, Camera.Speed);
            this.BspPathsIndices = new List<int>();
            map.Prefs.Read();
        }

        #region Device IsAcceptable, Modifier, Creation, Reset, Lost, Destroy

        /// <summary>
        /// Called during device initialization, this code checks the device for some 
        /// minimum set of capabilities, and rejects those that don't pass by returning false.
        /// </summary>
        public bool IsDeviceAcceptable(Caps caps, Format adapterFormat, Format backBufferFormat, bool windowed)
        {
            // Skip back buffer formats that don't support alpha blending
            if (!Manager.CheckDeviceFormat(caps.AdapterOrdinal, caps.DeviceType, adapterFormat,
                Usage.QueryPostPixelShaderBlending, ResourceType.Textures, backBufferFormat))
                return false;

            return true;
        }

        /// <summary>
        /// This callback function is called immediately before a device is created to allow the 
        /// application to modify the device settings. The supplied settings parameter 
        /// contains the settings that the Map.MDXFrameWork has selected for the new device, and the 
        /// application can make any desired changes directly to this structure.  Note however that 
        /// the sample Map.MDXFrameWork will not correct invalid device settings so care must be taken 
        /// to return valid device settings, otherwise creating the Device will fail.  
        /// </summary>
        public void ModifyDeviceSettings(DeviceSettings settings, Caps caps)
        {
            // If device doesn't support HW T&L or doesn't support 1.1 vertex shaders in HW 
            // then switch to SWVP.
            if ((!caps.DeviceCaps.SupportsHardwareTransformAndLight) ||
                (caps.VertexShaderVersion < new Version(1, 1)))
            {
                settings.BehaviorFlags = CreateFlags.SoftwareVertexProcessing;
            }
            else
            {
                settings.BehaviorFlags = CreateFlags.HardwareVertexProcessing;
            }

            // This application is designed to work on a pure device by not using 
            // any get methods, so create a pure device if supported and using HWVP.
            if ((caps.DeviceCaps.SupportsPureDevice) &&
                ((settings.BehaviorFlags & CreateFlags.HardwareVertexProcessing) != 0))
                settings.BehaviorFlags |= CreateFlags.PureDevice;

            // Debugging vertex shaders requires either REF or software vertex processing 
            // and debugging pixel shaders requires REF.  
#if(DEBUG_VS)
            if (settings.DeviceType != DeviceType.Reference )
            {
                settings.BehaviorFlags &= ~CreateFlags.HardwareVertexProcessing;
                settings.BehaviorFlags |= CreateFlags.SoftwareVertexProcessing;
            }
#endif
#if(DEBUG_PS)
            settings.DeviceType = DeviceType.Reference;
#endif
            // For the first device created if its a REF device, optionally display a warning dialog box
            if (settings.DeviceType == DeviceType.Reference)
            {
                Utility.DisplaySwitchingToRefWarning(Map.MDXFrameWork, "Prophet");
            }

        }

        /// <summary>
        /// This event will be fired immediately after the Direct3D device has been 
        /// created, which will happen during application initialization and windowed/full screen 
        /// toggles. This is the best location to create Pool.Managed resources since these 
        /// resources need to be reloaded whenever the device is destroyed. Resources created  
        /// here should be released in the Disposing event. 
        /// </summary>
        private void OnCreateDevice(object sender, DeviceEventArgs e)
        {
            // Initialize the stats font
            StatsFont = ResourceCache.GetGlobalInstance().CreateFont(e.Device, 15, 0, FontWeight.Bold, 1, false, CharacterSet.Default,
                Precision.Default, FontQuality.Default, PitchAndFamily.FamilyDoNotCare | PitchAndFamily.DefaultPitch
                , "Arial");

            TxtHelper.textFont = StatsFont;

            /*
            // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the 
            // shader debugger. Debugging vertex shaders requires either REF or software vertex 
            // processing, and debugging pixel shaders requires REF.  The 
            // ShaderFlags.Force*SoftwareNoOptimizations flag improves the debug experience in the 
            // shader debugger.  It enables source level debugging, prevents instruction 
            // reordering, prevents dead code elimination, and forces the compiler to compile 
            // against the next higher available software target, which ensures that the 
            // unoptimized shaders do not exceed the shader model limitations.  Setting these 
            // flags will cause slower rendering since the shaders will be unoptimized and 
            // forced into software.  See the DirectX documentation for more information about 
            // using the shader debugger.
            ShaderFlags shaderFlags = ShaderFlags.NotCloneable;
#if(DEBUG_VS)
            shaderFlags |= ShaderFlags.ForceVertexShaderSoftwareNoOptimizations;
#endif
#if(DEBUG_PS)
            shaderFlags |= ShaderFlags.ForcePixelShaderSoftwareNoOptimizations;
#endif
            // Read the D3DX effect file
            string path = Utility.FindMediaFile("fx");
            effect = ResourceCache.GetGlobalInstance().CreateEffectFromFile(e.Device,
                path, null, null, shaderFlags, null);

            effect.Technique = "RenderScene";
            eh_wvp = effect.GetParameter(null, "g_mWorldViewProjection");
            eh_world = effect.GetParameter(null, "g_mWorld");
            eh_time = effect.GetParameter(null, "g_fTime");
            eh_matAmbient = effect.GetParameter(null, "g_MaterialAmbientColor");
            eh_matDiffuse = effect.GetParameter(null, "g_MaterialDiffuseColor");
            */
        }

        /// <summary>
        /// This event will be fired immediately after the Direct3D device has been 
        /// reset, which will happen after a lost device scenario. This is the best location to 
        /// create Pool.Default resources since these resources need to be reloaded whenever 
        /// the device is lost. Resources created here should be released in the OnLostDevice 
        /// event. 
        /// </summary>
        private void OnResetDevice(object sender, DeviceEventArgs e)
        {
            SurfaceDescription desc = e.BackBufferDescription;
            // Create a sprite to help batch calls when drawing many lines of text
            TextSprite = new Sprite(e.Device);
            TxtHelper.textSprite = TextSprite;

            // Setup the Camera's projection parameters
            float aspectRatio = (float)desc.Width / (float)desc.Height;
            Camera.SetProjectionParameters((float)Math.PI / 4, aspectRatio, 0.1f, 10000.0f);
            //Camera.SetWindow(desc.Width, desc.Height);

            // Setup UI locations
            Hud.SetLocation(desc.Width - 170, 0);
            Hud.SetSize(170, 170);

            //Camera
            Camera.OnDeviceReset((float)Map.MDXFrameWork.WindowForm.Width / (float)Map.MDXFrameWork.WindowForm.Height);

            //set some properties
            //e.Device.RenderState.Lighting = false;
            e.Device.RenderState.Ambient = System.Drawing.Color.White;

            //cull
            switch (Map.MDXFrameWork.RenderPage)
            {
                case RenderingPages.DeviceSettings:
                    {
                        e.Device.RenderState.CullMode = Cull.CounterClockwise;
                        break;
                    }

                case RenderingPages.Preferences:
                    {
                        e.Device.RenderState.CullMode = Cull.CounterClockwise;
                        break;
                    }

                case RenderingPages.Default:
                    {
                        e.Device.RenderState.CullMode = Cull.Clockwise;
                        break;
                    }
            }

            //Texture filtering
            if (e.Device.DeviceCaps.SourceBlendCaps.SupportsSourceAlpha)
            {
                e.Device.RenderState.SourceBlend = Blend.SourceAlpha;
            }
            if (e.Device.DeviceCaps.SourceBlendCaps.SupportsInverseSourceAlpha)
            {
                e.Device.RenderState.DestinationBlend = Blend.InvSourceAlpha;
            }
            if (e.Device.DeviceCaps.TextureFilterCaps.SupportsMinifyAnisotropic)
            {
                e.Device.SamplerState[0].MinFilter = TextureFilter.Anisotropic;
                e.Device.SamplerState[1].MinFilter = TextureFilter.Anisotropic;
                e.Device.SamplerState[2].MinFilter = TextureFilter.Anisotropic;
                e.Device.SamplerState[3].MinFilter = TextureFilter.Anisotropic;
            }
            if (e.Device.DeviceCaps.TextureFilterCaps.SupportsMagnifyAnisotropic)
            {
                e.Device.SamplerState[0].MagFilter = TextureFilter.Anisotropic;
                e.Device.SamplerState[1].MagFilter = TextureFilter.Anisotropic;
                e.Device.SamplerState[2].MagFilter = TextureFilter.Anisotropic;
                e.Device.SamplerState[3].MagFilter = TextureFilter.Anisotropic;
            }
            if (e.Device.DeviceCaps.TextureFilterCaps.SupportsMipMapLinear)
            {
                e.Device.SamplerState[0].MipFilter = TextureFilter.Linear;
                e.Device.SamplerState[1].MipFilter = TextureFilter.Linear;
                e.Device.SamplerState[2].MipFilter = TextureFilter.Linear;
                e.Device.SamplerState[3].MipFilter = TextureFilter.Linear;
            }

            if (ShaderEditPanel != null)
            {
                ShaderEditPanel.Height = Map.MDXFrameWork.BackBufferSurfaceDescription.Height - 20;
            }
        }

        /// <summary>
        /// This event function will be called fired after the Direct3D device has 
        /// entered a lost state and before Device.Reset() is called. Resources created
        /// in the OnResetDevice callback should be released here, which generally includes all 
        /// Pool.Default resources. See the "Lost Devices" section of the documentation for 
        /// information about lost devices.
        /// </summary>
        private void OnLostDevice(object sender, EventArgs e)
        {
            if (TextSprite != null)
            {
                TextSprite.Dispose();
                TextSprite = null;
            }
        }

        /// <summary>
        /// This callback function will be called immediately after the Direct3D device has 
        /// been destroyed, which generally happens as a result of application termination or 
        /// windowed/full screen toggles. Resources created in the OnCreateDevice callback 
        /// should be released here, which generally includes all Pool.Managed resources. 
        /// </summary>
        private void OnDestroyDevice(object sender, EventArgs e)
        {
        }

        #endregion

        #region Update, Render

        /// <summary>
        /// This callback function will be called once at the beginning of every frame. This is the
        /// best location for your application to handle updates to the scene, but is not 
        /// intended to contain actual rendering calls, which should instead be placed in the 
        /// OnFrameRender callback.  
        /// </summary>
        public void OnFrameMove(Device device, double appTime, float elapsedTime)
        {
            switch (ViewState)
            {
                /*
            case ViewerState.Started:
                {

                    break;
                }

            case ViewerState.Error:
                {

                    break;
                }

            case ViewerState.Loading:
                {

                    break;
                }

            case ViewerState.NoBSP:
                {

                    break;
                }

            case ViewerState.SelectorScreen:
                {

                    break;
                }
                */
                case ViewerState.Viewing:
                    {
                        Camera.FrameMove(elapsedTime);
                        //Map.SBSP.Update();

                        break;
                    }
            }
        }

        /// <summary>
        /// This callback function will be called at the end of every frame to perform all the 
        /// rendering calls for the scene, and it will also be called if the window needs to be 
        /// repainted. After this function has returned, the sample Map.MDXFrameWork will call 
        /// Device.Present to display the contents of the next buffer in the swap chain
        /// </summary>
        public void OnFrameRender(Device device, double appTime, float elapsedTime)
        {
            bool beginSceneCalled = false;

            // Clear the render target and the zbuffer 
            device.Clear(ClearFlags.ZBuffer | ClearFlags.Target, 0x002D32AA, 1.0f, 0);

            try
            {
                device.BeginScene();
                beginSceneCalled = true;

                switch (ViewState)
                {
                    /*
                case ViewerState.Started:
                    {

                        break;
                    }

                case ViewerState.Error:
                    {

                        break;
                    }

                case ViewerState.Loading:
                    {

                        break;
                    }

                case ViewerState.NoBSP:
                    {

                        break;
                    }

                case ViewerState.SelectorScreen:
                    {
                        BspPathsDialog.OnRender(elapsedTime);
                        break;
                    }
                    */
                    case ViewerState.Viewing:
                        {
                            //counters
                            //bsp_rend_cnt = 0;
                            //perm_rend_cnt = 0;

                            /*
                            //effect
                            effect.SetValue(eh_time, (float)appTime);

                            int numPasses = effect.Begin(FX.None);
                            for (int pass = 0; pass < numPasses; pass++)
                            {
                            */

                            //Map.SBSP.SpawnInfo.Render();
                            Map.SBSP.Render();
                            if (ShaderEditPanel != null && ShaderEditPanel.Visible) ShaderEditPanel.SEP_OnRender(elapsedTime);
                            /*
                            }
                            effect.End();
                            */

                            break;
                        }
                }

                // Show frame rate
                if (ShaderEditPanel == null || !ShaderEditPanel.Visible) RenderText();

                // Show UI
                Hud.OnRender(elapsedTime);
            }
            finally
            {
                if (beginSceneCalled)
                    device.EndScene();
            }
        }

        /// <summary>
        /// Render the help and statistics text. This function uses the Font object for 
        /// efficient text rendering.
        /// </summary>
        private void RenderText()
        {
            TxtHelper.Begin();

            // Output statistics
            TxtHelper.SetInsertionPoint(5, 5);
            TxtHelper.SetForegroundColor(System.Drawing.Color.Yellow);
            TxtHelper.DrawTextLine(Map.MDXFrameWork.FrameStats);
            TxtHelper.DrawTextLine(Map.MDXFrameWork.DeviceStats);

            TxtHelper.SetForegroundColor(System.Drawing.Color.White);
            TxtHelper.DrawTextLine(StatusText);

            if (ViewState == ViewerState.Viewing)
            {
                //TxtHelper.DrawTextLine(Environment.NewLine);
                //TxtHelper.DrawTextLine(Environment.NewLine);
                //UpdateDebugString();
                //TxtHelper.DrawTextLine(debugString);
                //TxtHelper.DrawTextLine(Map.SBSP.bsp_rend_cnt.ToString());
                //TxtHelper.DrawTextLine(Map.SBSP.perm_rend_cnt.ToString());

                // Draw help
                if (IsHelpShowing)
                {
                    if (BspPathsIndices.Count == 1)
                        TxtHelper.SetInsertionPoint(10, Map.MDXFrameWork.BackBufferSurfaceDescription.Height - 15 * 6);
                    else
                        TxtHelper.SetInsertionPoint(10, Map.MDXFrameWork.BackBufferSurfaceDescription.Height - 15 * 7);

                    TxtHelper.SetForegroundColor(System.Drawing.Color.DarkOrange);
                    TxtHelper.DrawTextLine(Controls_F1_to_hide_string);

                    if (BspPathsIndices.Count == 1)
                        TxtHelper.SetInsertionPoint(40, Map.MDXFrameWork.BackBufferSurfaceDescription.Height - 15 * 5);
                    else
                        TxtHelper.SetInsertionPoint(40, Map.MDXFrameWork.BackBufferSurfaceDescription.Height - 15 * 6);

                    TxtHelper.DrawTextLine(Open_another_map_string);
                    if (BspPathsIndices.Count != 1) TxtHelper.DrawTextLine(Select_different_BSP_string);
                    TxtHelper.DrawTextLine(Change_camera_speed_string);
                    TxtHelper.DrawTextLine(Quit_string);
                    TxtHelper.DrawTextLine(Hide_help_string);
                }
                else
                {
                    TxtHelper.SetInsertionPoint(10, Map.MDXFrameWork.BackBufferSurfaceDescription.Height - 15 * 2);
                    TxtHelper.SetForegroundColor(System.Drawing.Color.White);
                    TxtHelper.DrawTextLine(Press_F1_for_help_string);
                }
            }

            TxtHelper.End();
        }

        #endregion

        #region Debug Stuffz - Commented Out

        /*
        private string debugString;

        private void UpdateDebugString()
        {
            //debugString =
            //string.Format("\nPosition:\n{0}\nTarget:\n{1}\n\nBsp_count: {2}\nPerm_count: {3}",
            //Camera.EyeLocation, Camera.LookAtPoint, bsp_rend_cnt, perm_rend_cnt);

            debugString =
               string.Format("\nPosition:\n{0}\nTarget:\n{1}",
               Camera.EyeLocation, Camera.LookAtPoint);

            //debugString = ("\nPosition:\n" + Camera.EyeLocation.ToString() +
            //     "\nTarget:\n" + Camera.LookAtPoint.ToString() +
            //     "\n\nBSPcount: " + bsp_rend_cnt.ToString() +
            //     "\nPERMcount: " + perm_rend_cnt.ToString());
            
        }
        */

        #endregion

        #region Map I/O Code

        public void ShowOfdForMapIO()
        {
            Map.MDXFrameWork.Pause(false, true);

            try
            {
                ViewerState nextGameState = ViewState;

                using (System.Windows.Forms.OpenFileDialog o = new System.Windows.Forms.OpenFileDialog())
                {
                    o.Title = "Open File Dialog";
                    o.Filter = "Map (*.map)|*.map|All Files (*.*)|*.*";
                    if (o.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
                    {
                        ViewState = ViewerState.Loading;
                        Map.MDXFrameWork.Pause(false, false);
                        Map.MDXFrameWork.Render3DEnvironment();
                        Map.MDXFrameWork.Pause(false, true);

                        Map.Load(o.FileName);

                        BspPathsIndices.Clear();
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "sbsp")
                                BspPathsIndices.Add(x);
                        }

                        switch (BspPathsIndices.Count)
                        {
                            case 0:
                                {
                                    Map.SBSP.BSPIndex = -1;

                                    nextGameState = ViewerState.NoBSP;
                                    Map.MDXFrameWork.WindowForm.Text = "Prophet";

                                    break;
                                }
                            case 1:
                                {
                                    Map.SBSP.BSPIndex = 0;
                                    LoadForRenderWrapper(true, false);

                                    nextGameState = ViewerState.Viewing;
                                    Map.MDXFrameWork.WindowForm.Text = "Prophet - [" + o.FileName + "]";

                                    break;
                                }

                            default: // > 1
                                {
                                    if (BspPathsDialog == null)
                                    {
                                        BspPathsDialog = new Dialog(Map.MDXFrameWork);
                                        BspPathsList = BspPathsDialog.AddListBox(0, 0, 0, 400, 400);

                                        System.Drawing.Size pSize = BspPathsDialog.GetPreferredSize();
                                        BspPathsDialog.SetSize(pSize.Width, pSize.Height);

                                        BspPathsDialog.SetLocation(
                                            (Map.MDXFrameWork.BackBufferSurfaceDescription.Width / 2) - (BspPathsDialog.Width / 2),
                                            (Map.MDXFrameWork.BackBufferSurfaceDescription.Height / 2) - (BspPathsDialog.Height / 2));
                                    }

                                    for (int x = BspPathsList.NumberItems - 1; x >= 0; x--)
                                        BspPathsList.RemoveAt(x);

                                    for (int x = 0; x < BspPathsIndices.Count; x++)
                                    {
                                        BspPathsList.AddItem(
                                            Map.Tags.TagsList[BspPathsIndices[x]].Path, null);
                                    }

                                    LastViewState = ViewState;
                                    nextGameState = ViewerState.SelectorScreen;
                                    Map.MDXFrameWork.WindowForm.Text = "Prophet - [" + o.FileName + "]";

                                    break;
                                }
                        }
                    }
                }

                ViewState = nextGameState;
            }
            catch (Exception ex)
            {
                ErrorMsg = ex.Message;
                ViewState = ViewerState.Error;
            }

            Map.MDXFrameWork.Pause(false, false);
        }

        #endregion

        #region Render Load

        public void LoadForRenderWrapper(bool isNewMap, bool doPaintOps)
        {
            if (!Map.Loaded) return;

            try
            {
                if (doPaintOps)
                {
                    ViewState = ViewerState.Loading;
                    Map.MDXFrameWork.Render3DEnvironment();
                    Map.MDXFrameWork.Pause(false, true);
                }

                Map.SBSP.LoadForRender();

                if (doPaintOps)
                {
                    ViewState = ViewerState.Viewing;
                    Map.MDXFrameWork.Pause(false, false);
                }

                Camera.Refresh(isNewMap);
                if (ShaderEditPanel != null && !doPaintOps && isNewMap) ShaderEditPanel.SEP_ReloadComboBoxes();

            }
            catch (Exception ex)
            {
                ErrorMsg = ex.Message;
                ViewState = ViewerState.Error;
            }
        }

        #endregion

        #region Input Handlers

        /// <summary>
        /// As a convenience, the sample Map.MDXFrameWork inspects the incoming windows messages for
        /// keystroke messages and decodes the message parameters to pass relevant keyboard
        /// messages to the application.  The Map.MDXFrameWork does not remove the underlying keystroke 
        /// messages, which are still passed to the application's MsgProc callback.
        /// </summary>
        private void OnKeyEvent(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (Map.MDXFrameWork.RenderPage != RenderingPages.Default) return;

            switch (ViewState)
            {
                case ViewerState.Started:
                case ViewerState.Error:
                case ViewerState.NoBSP:
                    {
                        ShowOfdForMapIO();
                        break;
                    }
                /*
            case ViewerState.Error:
                {

                    break;
                }

            case ViewerState.Loading:
                {

                    break;
                }

            case ViewerState.NoBSP:
                {

                    break;
                }
                */
                case ViewerState.SelectorScreen:
                    {
                        switch (e.KeyCode)
                        {
                            case System.Windows.Forms.Keys.Enter:
                                Map.SBSP.BSPIndex = BspPathsList.GetSelectedIndex(-1);
                                LoadForRenderWrapper(true, true);
                                break;

                            case System.Windows.Forms.Keys.Back:
                                if (LastViewState != ViewerState.Viewing) LastViewState = ViewerState.Started;
                                ViewState = LastViewState;
                                break;
                        }
                        break;
                    }
                case ViewerState.Viewing:
                    {
                        switch (e.KeyCode)
                        {
                            case System.Windows.Forms.Keys.F1:
                                IsHelpShowing = !IsHelpShowing;
                                break;

                            case System.Windows.Forms.Keys.O: // Ctrl + O
                                if (e.Control) ShowOfdForMapIO();
                                break;

                            case System.Windows.Forms.Keys.Q: // Ctrl + Q
                                if (BspPathsIndices.Count != 1)
                                {
                                    LastViewState = ViewState;
                                    ViewState = ViewerState.SelectorScreen;
                                }
                                break;

                            case System.Windows.Forms.Keys.OemMinus:
                                Camera.Speed -= 0.02f;

                                Change_camera_speed_string = string.Format(
                                    Change_camera_speed_templateString, Camera.Speed);
                                break;

                            case System.Windows.Forms.Keys.Oemplus:
                                Camera.Speed += 0.02f;

                                Change_camera_speed_string = string.Format(
                                    Change_camera_speed_templateString, Camera.Speed);
                                break;
                        }
                        break;
                    }
            }
        }

        /// <summary>
        /// Before handling window messages, the sample Map.MDXFrameWork passes incoming windows 
        /// messages to the application through this callback function. If the application sets 
        /// noFurtherProcessing to true, the sample Map.MDXFrameWork will not process the message
        /// </summary>
        public IntPtr OnMsgProc(IntPtr hWnd, NativeMethods.WindowMessage msg, IntPtr wParam, IntPtr lParam, ref bool noFurtherProcessing)
        {
            switch (ViewState)
            {
                case ViewerState.Started:
                    {
                        // Give the dialog a chance to handle the message first
                        if (noFurtherProcessing = Hud.MessageProc(hWnd, msg, wParam, lParam))
                            return IntPtr.Zero;

                        break;
                    }

                case ViewerState.Error:
                    {
                        // Give the dialog a chance to handle the message first
                        if (noFurtherProcessing = Hud.MessageProc(hWnd, msg, wParam, lParam))
                            return IntPtr.Zero;

                        break;
                    }

                case ViewerState.Loading:
                    {

                        break;
                    }

                case ViewerState.NoBSP:
                    {
                        // Give the dialog a chance to handle the message first
                        if (noFurtherProcessing = Hud.MessageProc(hWnd, msg, wParam, lParam))
                            return IntPtr.Zero;

                        break;
                    }

                case ViewerState.SelectorScreen:
                    {
                        // Give the dialog a chance to handle the message first
                        if (noFurtherProcessing = Hud.MessageProc(hWnd, msg, wParam, lParam))
                        {
                            return IntPtr.Zero;
                        }
                        else if (noFurtherProcessing = BspPathsDialog.MessageProc(hWnd, msg, wParam, lParam))
                        {
                            return IntPtr.Zero;
                        }
                        break;
                    }


                case ViewerState.Viewing:
                    {
                        // Give the dialog a chance to handle the message first
                        if (noFurtherProcessing = Hud.MessageProc(hWnd, msg, wParam, lParam))
                        {
                            return IntPtr.Zero;
                        }
                        else if (ShaderEditPanel != null && ShaderEditPanel.Visible &&
                            (noFurtherProcessing = ShaderEditPanel.MessageProc(hWnd, msg, wParam, lParam)))
                        {
                            return IntPtr.Zero;
                        }
                        /*
                        else if (msg == NativeMethods.WindowMessage.MouseMove)
                        {
                            short mouseX = NativeMethods.LoWord((uint)lParam.ToInt32());
                            short mouseY = NativeMethods.HiWord((uint)lParam.ToInt32());

                            if (Map.Prefs.Perms_Load && Map.SBSP.sPermChunk_TEMP != -1 && Map.SBSP.PermPlacements[Map.SBSP.sPermChunk_TEMP].SelectedSubset != -1)
                            {
                                Map.SBSP.PermPlacements[Map.SBSP.sPermChunk_TEMP].PerformMove(mouseX, mouseY);
                                break;
                            }
                        }
                        */
                        else if (msg == NativeMethods.WindowMessage.LeftButtonDown)
                        {
                            // perform picking/intersection test
                            short mouseX = NativeMethods.LoWord((uint)lParam.ToInt32());
                            short mouseY = NativeMethods.HiWord((uint)lParam.ToInt32());

                            // Check to make sure mouse is not over SEP
                            if (ShaderEditPanel != null && ShaderEditPanel.Visible)
                            {
                                if (mouseX >= ShaderEditPanel.Location.X &&
                                    mouseY >= ShaderEditPanel.Location.Y &&
                                    mouseX <= ShaderEditPanel.Location.X + ShaderEditPanel.Width &&
                                    mouseY <= ShaderEditPanel.Location.Y + ShaderEditPanel.Height)
                                {
                                    return IntPtr.Zero;
                                }
                            }

                            #region Check BSP for Click

                            int bspSubset = -1;
                            float bspDist = -1;
                            int bspChunk = -1;

                            for (int x = 0; x < Map.SBSP.RawBSPChunks.Length; x++)
                            {
                                if (Map.SBSP.RawBSPChunks[x].RawInfoChunks.Length == 0
                                    || !Map.SBSP.RawBSPChunks[x].InFrustum)
                                    continue;

                                int sSubset;
                                float dist;
                                MeshSubsetClicked(mouseX, mouseY, Map.SBSP.RawBSPChunks[x].Mesh, Matrix.Identity, out sSubset, out dist);

                                if (sSubset != -1 && (bspDist == -1 || dist < bspDist))
                                {
                                    bspSubset = sSubset;
                                    bspDist = dist;
                                    bspChunk = x;
                                }
                            }

                            #endregion

                            #region Check Perms for Click

                            int permSubset = -1;
                            float permDist = -1;
                            int permChunk = -1;

                            if (Map.Prefs.Perms_Load)
                            {
                                for (int x = 0; x < Map.SBSP.PermPlacements.Length; x++)
                                {
                                    int rawIndx = Map.SBSP.PermPlacements[x].RawPermChunkIndex;

                                    if (Map.SBSP.RawPermChunks[rawIndx].RawInfoChunks.Length == 0
                                        || !Map.SBSP.PermPlacements[x].InFrustum)
                                        continue;

                                    int sSubset;
                                    float dist;
                                    MeshSubsetClicked(mouseX, mouseY, Map.SBSP.RawPermChunks[rawIndx].Mesh, Map.SBSP.PermPlacements[x].Matrix, out sSubset, out dist);

                                    if (sSubset != -1 && (permDist == -1 || dist < permDist))
                                    {
                                        permSubset = sSubset;
                                        permDist = dist;
                                        permChunk = x;
                                    }
                                }
                            }

                            #endregion

                            #region Find Which has been Clicked

                            if (bspDist != -1 || permDist != -1)
                            {
                                if (bspDist == -1 || (permDist != -1 && permDist < bspDist)) // Perm clicked
                                {
                                    // Already selected
                                    if (Map.SBSP.RawPermChunks[Map.SBSP.PermPlacements[permChunk].RawPermChunkIndex].SelectedSubsets.Contains(permSubset))
                                    {
                                        ClearHighlightedGeometry();
                                        StatusText = string.Empty;
                                    }
                                    // Not yet selected
                                    else
                                    {
                                        int shadIndx = Map.SBSP.RawPermChunks[Map.SBSP.PermPlacements[permChunk].RawPermChunkIndex].SubMeshInfo[permSubset].ShaderIndex;
                                        HighlightGeometryUsingShader(shadIndx);
                                        Map.SBSP.RawPermChunks[Map.SBSP.PermPlacements[permChunk].RawPermChunkIndex].IsClickedObject = true;
                                    }
                                }
                                else if (permDist == -1 || (bspDist != -1 && bspDist < permDist)) // BSP clicked
                                {
                                    // Already Selected
                                    if (Map.SBSP.RawBSPChunks[bspChunk].SelectedSubsets.Contains(bspSubset))
                                    {
                                        ClearHighlightedGeometry();
                                        StatusText = string.Empty;
                                    }
                                    else
                                    {
                                        int shadIndx = Map.SBSP.RawBSPChunks[bspChunk].SubMeshInfo[bspSubset].ShaderIndex;
                                        HighlightGeometryUsingShader(shadIndx);
                                        Map.SBSP.RawBSPChunks[bspChunk].IsClickedObject = true;
                                    }
                                }
                            }

                            #endregion
                        }
                        else
                        {
                            // Pass all remaining windows messages to Camera so it can respond to user input
                            Camera.HandleMessages(hWnd, msg, wParam, lParam);
                        }

                        break;
                    }
            }

            return IntPtr.Zero;
        }

        #endregion

        #region Mesh Selection Functions

        public void MeshSubsetClicked(int mX, int mY, Mesh mesh, Matrix mat, out int sSubset, out float dist)
        {
            Vector3 near = Vector3.Unproject(new Vector3(mX, mY, 0), Map.MDXFrameWork.Device.Viewport,
                Camera.ProjectionMatrix, Camera.ViewMatrix, mat);

            Vector3 far = Vector3.Unproject(new Vector3(mX, mY, 1), Map.MDXFrameWork.Device.Viewport,
                Camera.ProjectionMatrix, Camera.ViewMatrix, mat);

            Vector3 rDir = Vector3.Normalize(far - near);

            IntersectInformation coll_info = new IntersectInformation();
            sSubset = -1;
            dist = -1;

            for (int x = 0; x < mesh.NumberAttributes; x++)
            {
                if (mesh.IntersectSubset(x, near, rDir, out coll_info))
                {
                    //if (sIndx == -1 || (dist == -1 || coll_info.Dist < dist))
                    if (sSubset == -1 || coll_info.Dist < dist)
                    {
                        sSubset = x;
                        dist = coll_info.Dist;
                    }
                }
            }
        }

        #endregion

        #region Shader De/Select

        private void HighlightGeometryUsingShader(int shadIndx)
        {
            // Perms.
            if (Map.Prefs.Perms_Load)
            {
                // Loop Placements.
                for (int x = 0; x < Map.SBSP.RawPermChunks.Length; x++)
                {
                    if (Map.SBSP.RawPermChunks[x].RawInfoChunks.Length == 0)
                        continue;

                    // Loop subs.
                    Map.SBSP.RawPermChunks[x].IsClickedObject = false;
                    Map.SBSP.RawPermChunks[x].SelectedSubsets.Clear();
                    for (int y = 0; y < Map.SBSP.RawPermChunks[x].SubMeshInfo.Length; y++)
                    {
                        if (Map.SBSP.Shaders[Map.SBSP.RawPermChunks[x].SubMeshInfo[y].ShaderIndex].Identity == 
                            Map.SBSP.Shaders[shadIndx].Identity)
                        {
                            Map.SBSP.RawPermChunks[x].SelectedSubsets.Add(y);
                        }
                    }
                }
            }

            // BSP.
            // Loop clusters.
            for (int x = 0; x < Map.SBSP.RawBSPChunks.Length; x++)
            {
                if (Map.SBSP.RawBSPChunks[x].RawInfoChunks.Length == 0)
                    continue;

                // Loop subclusters.
                Map.SBSP.RawBSPChunks[x].IsClickedObject = false;
                Map.SBSP.RawBSPChunks[x].SelectedSubsets.Clear();
                for (int y = 0; y < Map.SBSP.RawBSPChunks[x].SubMeshInfo.Length; y++)
                {
                    if (Map.SBSP.Shaders[Map.SBSP.RawBSPChunks[x].SubMeshInfo[y].ShaderIndex].Identity ==
                        Map.SBSP.Shaders[shadIndx].Identity)
                    {
                        Map.SBSP.RawBSPChunks[x].SelectedSubsets.Add(y);
                    }
                }
            }

            ShowShaderEditPanel(shadIndx);
        }

        private void ClearHighlightedGeometry()
        {
            // Perms.
            if (Map.Prefs.Perms_Load)
            {
                // Loop Placements.
                for (int x = 0; x < Map.SBSP.RawPermChunks.Length; x++)
                {
                    if (Map.SBSP.RawPermChunks[x].RawInfoChunks.Length == 0)
                        continue;

                    Map.SBSP.RawPermChunks[x].IsClickedObject = false;
                    Map.SBSP.RawPermChunks[x].SelectedSubsets.Clear();
                }
            }

            // BSP
            // Loop clusters.
            for (int x = 0; x < Map.SBSP.RawBSPChunks.Length; x++)
            {
                if (Map.SBSP.RawBSPChunks[x].RawInfoChunks.Length == 0)
                    continue;

                Map.SBSP.RawBSPChunks[x].IsClickedObject = false;
                Map.SBSP.RawBSPChunks[x].SelectedSubsets.Clear();
            }

            HideShaderEditPanel();
        }

        #endregion

        #region ShaderEditPanel

        public void ShowShaderEditPanel(int shadIndx)
        {
            ShaderEditPanel.SEP_SelectShader(shadIndx);
            ShaderEditPanel.Visible = true;
        }

        public void HideShaderEditPanel()
        {
            if (ShaderEditPanel != null)
            {
                ShaderEditPanel.Visible = false;
            }
        }

        #endregion

        #region Application Initialization

        /// <summary>
        /// Initializes the application
        /// </summary>
        public void InitializeApplication()
        {
            int y = 10;

            // Initialize the HUD
            Button fullScreen = Hud.AddButton(0, "Toggle full screen", 35, y, 125, 22);
            Button changeDevice = Hud.AddButton(0, "Change Device (F2)", 35, y += 24, 125, 22);
            Button prefs = Hud.AddButton(0, "Preferences (F3)", 35, y += 24, 125, 22);

            // Hook the button events for when these items are clicked
            fullScreen.Click += new EventHandler(OnFullscreenClicked);
            changeDevice.Click += new EventHandler(OnChangeDeviceClicked);
            prefs.Click += new EventHandler(OnPrefsClicked);

            ViewState = ViewerState.Started;
        }

        #endregion

        #region UI Events

        /// <summary>Called when the full screen button is clicked</summary>
        private void OnFullscreenClicked(object sender, EventArgs e)
        {
            Map.MDXFrameWork.ToggleFullscreen();
        }

        /// <summary>Called when the change device button is clicked</summary>
        private void OnChangeDeviceClicked(object sender, EventArgs e)
        {
            Map.MDXFrameWork.RenderPage = RenderingPages.DeviceSettings;
        }

        /// <summary>Called when the prefs button is clicked</summary>
        private void OnPrefsClicked(object sender, EventArgs e)
        {
            Map.MDXFrameWork.RenderPage = RenderingPages.Preferences;
        }

        #endregion

        #region Main (Entry Point)

        /// <summary>
        /// Entry point to the program. Initializes everything and goes into a message processing 
        /// loop. Idle time is used to render the scene.
        /// </summary>
        [STAThread]
        static int Main()
        {
            System.Windows.Forms.Application.EnableVisualStyles();
            using (Map map = new Map())
            {
                Form1 sample = new Form1(map);
                map.MDXFrameWork.ActualForm = sample;
                // Set the callback functions. These functions allow the sample map.MDXFrameWork to notify
                // the application about device changes, user input, and windows messages.  The 
                // callbacks are optional so you need only set callbacks for events you're interested 
                // in. However, if you don't handle the device reset/lost callbacks then the sample 
                // map.MDXFrameWork won't be able to reset your device since the application must first 
                // release all device resources before resetting.  Likewise, if you don't handle the 
                // device created/destroyed callbacks then the sample map.MDXFrameWork won't be able to 
                // recreate your device resources.
                map.MDXFrameWork.Disposing += new EventHandler(sample.OnDestroyDevice);
                map.MDXFrameWork.DeviceLost += new EventHandler(sample.OnLostDevice);
                map.MDXFrameWork.DeviceCreated += new DeviceEventHandler(sample.OnCreateDevice);
                map.MDXFrameWork.DeviceReset += new DeviceEventHandler(sample.OnResetDevice);

                map.MDXFrameWork.SetWndProcCallback(new WndProcCallback(sample.OnMsgProc));

                map.MDXFrameWork.SetCallbackInterface(sample);
                try
                {

                    // Show the cursor and clip it when in full screen
                    map.MDXFrameWork.SetCursorSettings(true, true);

                    // Initialize
                    sample.InitializeApplication();

                    // Initialize the sample map.MDXFrameWork and create the desired window and Direct3D 
                    // device for the application. Calling each of these functions is optional, but they
                    // allow you to set several options which control the behavior of the sampleFramework.
                    map.MDXFrameWork.Initialize(true, true, true); // Parse the command line, handle the default hotkeys, and show msgboxes
                    map.MDXFrameWork.CreateWindow("Prophet - By Prey");
                    map.MDXFrameWork.WindowForm.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
                    // Hook the keyboard event
                    map.MDXFrameWork.WindowForm.KeyDown += new System.Windows.Forms.KeyEventHandler(sample.OnKeyEvent);
                    map.MDXFrameWork.WindowForm.FormClosed += new System.Windows.Forms.FormClosedEventHandler(sample.OnFormClosed);
                    map.MDXFrameWork.CreateDevice(0, true, MDXFramework.DefaultSizeWidth, MDXFramework.DefaultSizeHeight, sample);

                    // Pass control to the sample map.MDXFrameWork for handling the message pump and 
                    // dispatching render calls. The sample map.MDXFrameWork will call your FrameMove 
                    // and FrameRender callback when there is idle time between handling window messages.
                    map.MDXFrameWork.MainLoop();

                }
#if(DEBUG)
                catch (Exception e)
                {
                    // In debug mode show this error (maybe - depending on settings)
                    map.MDXFrameWork.DisplayErrorMessage(e);
#else
                catch
                {
                    // In release mode fail silently
#endif
                    // Ignore any exceptions here, they would have been handled by other areas
                    return (map.MDXFrameWork.ExitCode == 0) ? 1 : map.MDXFrameWork.ExitCode; // Return an error code here
                }

                // Perform any application-level cleanup here. Direct3D device resources are released within the
                // appropriate callback functions and therefore don't require any cleanup code here.
                return map.MDXFrameWork.ExitCode;
            }
        }

        private void OnFormClosed(object sender, System.Windows.Forms.FormClosedEventArgs e)
        {
            if (Map.Loaded && Map.SBSP.LoadedForRender) Map.SBSP.Unload();
            Map.Prefs.Write();
        }

        #endregion
    }

    public enum ViewerState
    {
        Started,
        Error,
        Loading,
        NoBSP,
        SelectorScreen,
        Viewing
    }
}