using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Extended;

namespace H2UtilityToolkit.Essentials
{
    /// <summary>
    /// 
    /// </summary>
    public class Index
    {
        #region Instance Data

        private Map __map__;
        private int __primaryMagicConstant__;
        private int __primaryMagic__;
        private int __classRelationshipCount__;
        private int __tagInfoIndexPointer__;
        private int __tagInfoIndexOffset__;
        private int __scenariosID__;
        private int __globalsID__;
        private int __tagInfoCount__;
        private string __tags__;
        private ClassRelationship[] __classRelationships__;
        private int __secondaryMagicConstant__;
        private int __secondaryMagic__;

        #endregion

        #region Accessors

        ///<summary>
        /// Map
        ///</summary>
        protected Map Map { get { return __map__; } set { __map__ = value; } }
        ///<summary>
        /// PrimaryMagicConstant
        ///</summary>
        public int PrimaryMagicConstant { get { return __primaryMagicConstant__; } set { __primaryMagicConstant__ = value; PrimaryMagic = value - (Map.Header.IndexOffset + 32); } }
        ///<summary>
        /// PrimaryMagic
        ///</summary>
        public int PrimaryMagic { get { return __primaryMagic__; } set { __primaryMagic__ = value; } }
        /// <summary>
        /// ClassRelationshipCount
        /// </summary>
        public int ClassRelationshipCount { get { return __classRelationshipCount__; } set { __classRelationshipCount__ = value; } }
        ///<summary>
        /// TagInfoIndexPointer
        ///</summary>
        public int TagInfoIndexPointer { get { return __tagInfoIndexPointer__; } set { __tagInfoIndexPointer__ = value; TagInfoIndexOffset = value - PrimaryMagic; } }
        ///<summary>
        /// TagInfoIndexOffset
        ///</summary>
        public int TagInfoIndexOffset { get { return __tagInfoIndexOffset__; } set { __tagInfoIndexOffset__ = value; } }
        ///<summary>
        /// ScenariosID
        ///</summary>
        public int ScenariosID { get { return __scenariosID__; } set { __scenariosID__ = value; } }
        ///<summary>
        /// GlobalsID
        ///</summary>
        public int GlobalsID { get { return __globalsID__; } set { __globalsID__ = value; } }
        ///<summary>
        /// TagInfoCount
        ///</summary>
        public int TagInfoCount { get { return __tagInfoCount__; } set { __tagInfoCount__ = value; } }
        ///<summary>
        /// Tags
        ///</summary>
        public string Tags { get { return __tags__; } set { if (value.Length <= 4) __tags__ = value; } }
        /// <summary>
        /// ClassRelationships
        /// </summary>
        public ClassRelationship[] ClassRelationships { get { return __classRelationships__; } set { __classRelationships__ = value; } }
        ///<summary>
        /// SecondaryMagicConstant
        ///</summary>
        public int SecondaryMagicConstant { get { return __secondaryMagicConstant__; } set { __secondaryMagicConstant__ = value; SecondaryMagic = value - Map.Header.MetaStart; } }
        ///<summary>
        /// SecondaryMagic
        ///</summary>
        public int SecondaryMagic { get { return __secondaryMagic__; } set { __secondaryMagic__ = value; } }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="map"></param>
        public Index(Map map)
        {
            this.Map = map;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="map"></param>
        /// <param name="br"></param>
        public Index(Map map, BitReader br)
        {
            this.Map = map;
            this.Read(br);
        }

        /// <summary>
        /// 
        /// </summary>
        public void Read()
        {
            BitReader br = new BitReader(Map.FileName, Endian.Little);
            this.Read(br);
            br.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="br"></param>
        public void Read(BitReader br)
        {
            br.BaseStream.Position = Map.Header.IndexOffset;

            PrimaryMagicConstant = br.ReadInt32();   //
            ClassRelationshipCount = br.ReadInt32(); //
            TagInfoIndexPointer = br.ReadInt32();    //
            ScenariosID = br.ReadInt32();            // scnr tag identity
            GlobalsID = br.ReadInt32();              // matg tag identity
            br.BaseStream.Position += 4;             //
            TagInfoCount = br.ReadInt32();           //
            Tags = br.ReadBTFString(4);              // 'sgat'

            ClassRelationships = new ClassRelationship[ClassRelationshipCount]; // Wildcards
            for (int x = 0; x < ClassRelationshipCount; x++)                    //
            {
                ClassRelationships[x].ObjectClass = br.ReadBTFString(4);        //
                ClassRelationships[x].ParentClass = br.ReadBTFString(4);        //
                ClassRelationships[x].GrandparentClass = br.ReadBTFString(4);   //
            }

            br.BaseStream.Position += 8;             // TagInfoIndexOffset + 8
            SecondaryMagicConstant = br.ReadInt32(); // matg raw meta offset
        }

        /// <summary>
        /// Shifts the PrimaryMagic, TagInfoIndexOffset and SecondaryMagic by specified value.
        /// </summary>
        /// <param name="shiftBy"></param>
        public void ShiftValuesInApp(int shiftBy)
        {
            PrimaryMagic -= shiftBy;
            TagInfoIndexOffset += shiftBy;
            SecondaryMagic -= shiftBy;
        }

        /// <summary>
        /// 
        /// </summary>
        public void Write()
        {
            BitWriter bw = new BitWriter(Map.FileName, Endian.Little);
            this.Write(bw);
            bw.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="bw"></param>
        public void Write(BitWriter bw)
        {
            bw.BaseStream.Position = Map.Header.IndexOffset;

            bw.Write(PrimaryMagicConstant);
            bw.BaseStream.Position += 4;
            bw.Write(TagInfoIndexPointer);
            bw.Write(ScenariosID);
            bw.Write(GlobalsID);
            bw.BaseStream.Position += 4;
            bw.Write(TagInfoCount);
            bw.WriteBTFString(Tags, 4);
            //bw.BaseStream.Position = TagInfoIndexOffset + 8;
            //bw.Write(SecondaryMagicConstant);
        }
    }

    public struct ClassRelationship
    {
        #region Instance Data

        private string __objectClass__;
        private string __parentClass__;
        private string __grandparentClass__;

        #endregion

        #region Accessors

        /// <summary>
        /// ObjectClass
        /// </summary>
        public string ObjectClass { get { return __objectClass__; } set { __objectClass__ = value; } }
        /// <summary>
        /// ParentClass
        /// </summary>
        public string ParentClass { get { return __parentClass__; } set { __parentClass__ = value; } }
        /// <summary>
        /// GrandparentClass
        /// </summary>
        public string GrandparentClass { get { return __grandparentClass__; } set { __grandparentClass__ = value; } }

        #endregion
    }
}