using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Extended;
using H2UtilityToolkit.Misc;

namespace H2UtilityToolkit.Essentials
{
    public class Tags
    {
        #region Instance Data

        #region Constants

        /// <summary>
        /// [Const] NullTagType
        /// </summary>
        public const string NullTagType = "";
        /// <summary>
        /// [Const] NullTagPath
        /// </summary>
        public const string NullTagPath = "Nulled Out";

        #endregion

        private Map __map__;
        private List<string> __tagTypes__;
        private Tag[] __tagsList__;
        private SortTagsBy __sortTagsBy__;
        private int __selectedIndex__;

        #endregion

        #region Accessors

        ///<summary>
        /// Map
        ///</summary>
        protected Map Map { get { return __map__; } set { __map__ = value; } }
        ///<summary>
        /// TagTypes
        ///</summary>
        public List<string> TagTypes { get { return __tagTypes__; } protected set { __tagTypes__ = value; } }
        ///<summary>
        /// TagsList
        ///</summary>
        public Tag[] TagsList { get { return __tagsList__; } protected set { __tagsList__ = value; } }
        ///<summary>
        /// SortTagsBy
        ///</summary>
        public SortTagsBy SortTagsBy { get { return __sortTagsBy__; } set { __sortTagsBy__ = value; } }
        ///<summary>
        /// SelectedIndex
        ///</summary>
        public int SelectedIndex { get { return __selectedIndex__; } set { __selectedIndex__ = value; } }



        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="map"></param>
        public Tags(Map map)
        {
            this.Map = map;
            this.SelectedIndex = -1;
            this.SortTagsBy = SortTagsBy.TagType;
            this.TagTypes = new List<string>();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="map"></param>
        /// <param name="br"></param>
        public Tags(Map map, BitReader br)
        {
            this.Map = map;
            this.SelectedIndex = -1;
            this.SortTagsBy = SortTagsBy.TagType;
            this.TagTypes = new List<string>();
            this.Read(br);
        }

        /// <summary>
        /// 
        /// </summary>
        public void Read()
        {
            BitReader br = new BitReader(Map.FileName, Endian.Little);
            this.Read(br);
            br.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="br"></param>
        public void Read(BitReader br)
        {
            TagTypes.Clear();
            TagsList = new Tag[Map.Index.TagInfoCount];

            // Index - Tag Info
            br.BaseStream.Position = Map.Index.TagInfoIndexOffset;
            for (int x = 0; x < Map.Index.TagInfoCount; x++)
            {
                string tt = br.ReadBTFString(4);
                if (!TagTypes.Contains(tt)) TagTypes.Add(tt);
                TagsList[x] = new Tag();
                TagsList[x].Type = tt;
                TagsList[x].Identity = br.ReadInt32();
                TagsList[x].RawMetaOffset = br.ReadInt32();
                TagsList[x].MetaOffset = TagsList[x].RawMetaOffset - Map.Index.SecondaryMagic;
                TagsList[x].MetaSize = br.ReadInt32();
            }

            if (Map.Header.FileTableOffset == 0) // Map is protected
            {
                const string wordNull = "Null";
                for (int x = 0; x < Map.Index.TagInfoCount; x++)
                {
                    TagsList[x].PathOffset = 0;
                    TagsList[x].PathLength = wordNull.Length;
                    TagsList[x].Path = wordNull;
                }
            }
            else
            {
                // File Index
                br.BaseStream.Position = Map.Header.FileTableIndexOffset;
                int[] offsets = new int[Map.Index.TagInfoCount + 1];
                int indx;
                for (indx = 0; indx < Map.Index.TagInfoCount; indx++)
                {
                    offsets[indx] = br.ReadInt32();
                }
                offsets[indx] = Map.Header.FileTableSize;

                // File Table
                br.BaseStream.Position = Map.Header.FileTableOffset;
                for (int x = 0; x < Map.Index.TagInfoCount; x++)
                {
                    TagsList[x].PathOffset = offsets[x];
                    TagsList[x].PathLength = offsets[x + 1] - offsets[x] - 1;
                    br.BaseStream.Position = Map.Header.FileTableOffset + offsets[x];
                    TagsList[x].Path = br.ReadString(TagsList[x].PathLength);
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ID"></param>
        public int IndexOf(int ID)
        {
            for (int x = 0; x < TagsList.Length; x++)
            {
                if (TagsList[x].Identity == ID)
                    return x;
            }
            return -1;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="type"></param>
        /// <param name="path"></param>
        public int IndexOf(string type, string path)
        {
            for (int x = 0; x < TagsList.Length; x++)
            {
                if (TagsList[x].Type == type && TagsList[x].Path == path)
                    return x;
            }
            return -1;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="index"></param>
        public void Write(int index)
        {
            BitWriter bw = new BitWriter(Map.FileName, Endian.Little);
            this.Write(bw, index);
            bw.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="bw"></param>
        /// <param name="index"></param>
        public void Write(BitWriter bw, int index)
        {
            bw.BaseStream.Position = Map.Index.TagInfoIndexOffset + (index * 16);

            bw.WriteBTFString(TagsList[index].Type);
            bw.Write(TagsList[index].Identity);
            bw.Write(TagsList[index].RawMetaOffset);
            bw.Write(TagsList[index].MetaSize);

            // If new path length is same as old, just write it.
            if (TagsList[index].Path.Length == TagsList[index].PathLength)
            {
                bw.BaseStream.Position = Map.Header.FileTableOffset + TagsList[index].PathOffset;
                bw.Write(TagsList[index].Path);
            }
            else // We'll need todo some shifting.
            {
                BitReader br = new BitReader(bw.BaseStream, Endian.Little);

                // Goto just after string in filetable.
                br.BaseStream.Position = Map.Header.FileTableOffset + TagsList[index].PathOffset + TagsList[index].PathLength;

                // Store filetable from position.
                byte[] FileTableEnd = br.ReadBytes((Map.Header.FileTableOffset + Map.Header.FileTableSize) - (int)br.BaseStream.Position);
                int oldPaddingLen = Map.Header.FileTableIndexOffset - (int)br.BaseStream.Position;

                br.BaseStream.Position = Map.Header.FileTableIndexOffset; // or += oldPaddingLen;
                byte[] MapHalf = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

                // Goto path offset.
                bw.BaseStream.Position = Map.Header.FileTableOffset + TagsList[index].PathOffset;
                bw.Write(TagsList[index].Path);
                bw.Write(FileTableEnd);

                int shift = TagsList[index].Path.Length - TagsList[index].PathLength;
                TagsList[index].PathLength = TagsList[index].Path.Length;

                // Fix other path offsets
                for (int x = index + 1; x < TagsList.Length; x++)
                    TagsList[x].PathOffset += shift;

                //--[Header]
                bw.BaseStream.Position = 712;
                bw.Write(Map.Header.FileTableSize += shift);

                //--[FileTable - FileIndex Padding]

                // Get new padding length.
                int newPaddingLen = 512 - (Map.Header.FileTableSize % 512);

                // Goto end of filetable.
                bw.BaseStream.Position = Map.Header.FileTableOffset + Map.Header.FileTableSize;

                if (newPaddingLen == 512) newPaddingLen = 0;
                else
                {
                    byte[] padding = new byte[newPaddingLen];
                    bw.Write(padding);
                }
                Map.Header.FileTableIndexOffset = (int)bw.BaseStream.Position;
                bw.Write(MapHalf);

                shift += newPaddingLen - oldPaddingLen;

                //--[Header]
                bw.BaseStream.Position = 716;
                bw.Write(Map.Header.FileTableIndexOffset);

                //--[FileIndex]

                // Only continue if we're not at the end of the index.
                if (index != TagsList.Length - 1)
                {
                    br.BaseStream.Position = Map.Header.FileTableIndexOffset + (index * 4);
                    int len = br.ReadInt32();

                    for (int x = index; x < TagsList.Length - 1; x++)
                    {
                        len += TagsList[x].PathLength + 1;
                        bw.Write(len);
                    }
                }

                // The padding hasn't re-aligned the rest of the map.. so we'll have to.
                if (shift != 0)
                {
                    //--[Header]
                    bw.BaseStream.Position = 16;
                    bw.Write(Map.Header.IndexOffset += shift);

                    br.BaseStream.Position = 344;
                    bw.Write(Map.Header.CrazyOffset += shift);

                    //--[Index]
                    Map.Index.ShiftValuesInApp(shift);

                    //--[Meta Offsets]
                    for (int x = 0; x < TagsList.Length; x++)
                    {
                        if (TagsList[x].Type != "sbsp" && TagsList[x].Type != "ltmp")
                            TagsList[x].MetaOffset += shift;
                    }

                    //--[Unicode]
                    int matgOffset400 = TagsList[0].MetaOffset + 400;
                    br.BaseStream.Position = matgOffset400 + 8;

                    // Read, shift, write.
                    for (int x = 0; x < 8; x++)
                    {
                        int newLangIndexOffset = br.ReadInt32() + shift;
                        int newLangTableOffset = br.ReadInt32() + shift;

                        br.BaseStream.Position -= 8;
                        bw.Write(newLangIndexOffset);
                        bw.Write(newLangTableOffset);

                        bw.BaseStream.Position += 20;
                    }

                    //--[Bitmaps]
                    for (int x = 0; x < TagsList.Length; x++)
                    {
                        if (TagsList[x].Type == "bitm")
                        {
                            br.BaseStream.Position = TagsList[x].MetaOffset + 68;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < cc; y++)
                            {
                                for (int z = 0; z < 6; z++)
                                {
                                    long ptr = tr + (y * 116) + 28 + (z * 4);
                                    br.BaseStream.Position = ptr;

                                    int Offset = br.ReadInt32();
                                    if (Offset == -1) break;

                                    RawLocation rawLoc = RawLocation.Internal;
                                    Map.Misc.ParsePointer(ref Offset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal)
                                    {
                                        bw.BaseStream.Position = ptr;
                                        bw.Write(Offset + shift);
                                    }
                                }
                            }
                        }
                    }

                    //--[End]
                    bw.BaseStream.Position = TagsList[TagsList.Length - 1].MetaOffset +
                        TagsList[TagsList.Length - 1].MetaSize;

                    newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
                    newPaddingLen += (newPaddingLen == 512 ? 512 : 1024);

                    long eof = bw.BaseStream.Position + newPaddingLen;
                    byte eofpad = 205; // CD
                    while (bw.BaseStream.Position < eof) bw.Write(eofpad);
                    bw.BaseStream.SetLength(eof);

                    //--[Header]
                    bw.BaseStream.Position = 8;
                    bw.Write(eof);
                }
                br.Close();
            }
            bw.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        public void WriteAll()
        {
            BitWriter bw = new BitWriter(Map.FileName, Endian.Little);
            this.WriteAll(bw);
            bw.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="bw"></param>
        public void WriteAll(BitWriter bw)
        {
            for (int x = 0; x < Map.Index.TagInfoCount; x++)
            {
                this.Write(bw, x);
            }
        }

        /// <summary>
        /// Pass in index of existing tag, clone is made.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="includeRawData"></param>
        public Tag Append(int index, bool includeRawData, bool appendRawData)
        {
            FileStream fs = new FileStream(Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
            Tag newTag = this.Append(fs, TagsList[index], includeRawData, appendRawData);
            fs.Close();
            return newTag;
        }

        /// <summary>
        /// Pass in index of existing tag, clone is made.
        /// </summary>
        /// <param name="fs"></param>
        /// <param name="index"></param>
        /// <param name="includeRawData"></param>
        public Tag Append(FileStream fs, int index, bool includeRawData, bool appendRawData)
        {
            return this.Append(fs, TagsList[index], includeRawData, appendRawData);
        }

        /// <summary>
        /// Pass in tag reference, clone is made.
        /// </summary>
        /// <param name="tag"></param>
        /// <param name="includeRawData"></param>
        public Tag Append(Tag tag, bool includeRawData, bool appendRawData)
        {
            FileStream fs = new FileStream(Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
            Tag newTag = this.Append(fs, tag, includeRawData, appendRawData);
            fs.Close();
            return newTag;
        }

        /// <summary>
        /// Pass in tag reference, clone is made. - 'appendRawData' ignored for now.
        /// 
        /// Dups non raw tags fine in both MP and SP.
        /// Dups bitm tags fine in both MP and SP.
        /// Dups mode tags fine in MP, but not in SP.
        /// All other raw tags untested.
        /// </summary>
        /// <param name="fs"></param>
        /// <param name="tag"></param>
        /// <param name="includeRawData"></param>
        public Tag Append(FileStream fs, Tag tag, bool includeRawData, bool appendRawData)
        {
            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            #region Get Info - Initialization Stage

            int ughIndx = TagsList.Length - 1;
            Tag newTag = tag.Clone();

            newTag.Identity = TagsList[ughIndx].Identity;
            newTag.RawMetaOffset = TagsList[ughIndx].RawMetaOffset;
            newTag.MetaOffset = TagsList[ughIndx].MetaOffset;

            TagsList[ughIndx].Identity += 65537; // Each ident is 65537 apart.
            TagsList[ughIndx].RawMetaOffset += newTag.MetaSize;
            TagsList[ughIndx].MetaOffset += newTag.MetaSize;

            // Get tag meta.
            br.BaseStream.Position = tag.MetaOffset;
            byte[] newTagMeta = br.ReadBytes(tag.MetaSize);

            // Raw.
            RawType rawType = RawType.None;
            if (includeRawData) rawType = Map.Misc.GetRawType(newTag.Type);
            List<RawInfo> rawInfoList = null;

            if (rawType != RawType.None)
            {
                rawInfoList = new List<RawInfo>();

                #region Raw Switch Statement
                switch (rawType)
                {
                    case RawType.Sound:
                        {

                            break;
                        }
                    case RawType.Model:
                        {
                            // Raw1
                            br.BaseStream.Position = tag.MetaOffset + 36;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int x = 0; x < cc; x++)
                            {
                                RawInfo rawInfo = new RawInfo();
                                br.BaseStream.Position = tr + (x * 92) + 56;
                                rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                                rawInfo.RawOffset = br.ReadInt32();
                                rawInfo.RawSize = br.ReadInt32();

                                if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                                {
                                    Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                        out rawInfo.RawLocation);

                                    rawInfoList.Add(rawInfo);
                                }
                            }

                            // Raw2.
                            br.BaseStream.Position = tag.MetaOffset + 116;

                            cc = br.ReadInt32();
                            tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int x = 0; x < cc; x++)
                            {
                                RawInfo rawInfo = new RawInfo();
                                br.BaseStream.Position = tr + (x * 88) + 52;
                                rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                                rawInfo.RawOffset = br.ReadInt32();
                                rawInfo.RawSize = br.ReadInt32();

                                if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                                {
                                    Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                        out rawInfo.RawLocation);

                                    rawInfoList.Add(rawInfo);
                                }
                            }
                            break;
                        }
                    case RawType.BSP:
                        {

                            break;
                        }
                    case RawType.Lightmap:
                        {

                            break;
                        }
                    case RawType.Weather:
                        {
                            br.BaseStream.Position = tag.MetaOffset;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int x = 0; x < cc; x++)
                            {
                                RawInfo rawInfo = new RawInfo();
                                br.BaseStream.Position = tr + (x * 140) + 64;
                                rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                                rawInfo.RawOffset = br.ReadInt32();
                                rawInfo.RawSize = br.ReadInt32();

                                if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                                {
                                    Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                        out rawInfo.RawLocation);

                                    rawInfoList.Add(rawInfo);
                                }
                            }
                            break;
                        }
                    case RawType.Decorator:
                        {
                            br.BaseStream.Position = tag.MetaOffset + 56;

                            RawInfo rawInfo = new RawInfo();
                            rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                            rawInfo.RawOffset = br.ReadInt32();
                            rawInfo.RawSize = br.ReadInt32();

                            if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                            {
                                Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                    out rawInfo.RawLocation);

                                rawInfoList.Add(rawInfo);
                            }
                            break;
                        }
                    case RawType.ParticleModel:
                        {
                            br.BaseStream.Position = tag.MetaOffset + 160;

                            RawInfo rawInfo = new RawInfo();
                            rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                            rawInfo.RawOffset = br.ReadInt32();
                            rawInfo.RawSize = br.ReadInt32();

                            if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                            {
                                Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                    out rawInfo.RawLocation);

                                rawInfoList.Add(rawInfo);
                            }
                            break;
                        }
                    case RawType.CoconutsModel:
                        {
                            br.BaseStream.Position = tag.MetaOffset + 80;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int x = 0; x < cc; x++)
                            {
                                RawInfo rawInfo = new RawInfo();
                                br.BaseStream.Position = tr + (x * 44) + 8;
                                rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                                rawInfo.RawOffset = br.ReadInt32();
                                rawInfo.RawSize = br.ReadInt32();

                                if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                                {
                                    Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                        out rawInfo.RawLocation);

                                    rawInfoList.Add(rawInfo);
                                }
                            }
                            break;
                        }
                    case RawType.Animation:
                        {
                            br.BaseStream.Position = tag.MetaOffset + 172;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int x = 0; x < cc; x++)
                            {
                                RawInfo rawInfo = new RawInfo();
                                br.BaseStream.Position = tr + (x * 20) + 4;
                                rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                                rawInfo.RawSize = br.ReadInt32();
                                rawInfo.RawOffset = br.ReadInt32();

                                if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                                {
                                    Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                        out rawInfo.RawLocation);

                                    rawInfoList.Add(rawInfo);
                                }
                            }
                            break;
                        }
                    case RawType.Bitmap:
                        {
                            br.BaseStream.Position = tag.MetaOffset + 68;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int x = 0; x < cc; x++)
                            {
                                for (int y = 0; y < 6; y++)
                                {
                                    RawInfo rawInfo = new RawInfo();
                                    br.BaseStream.Position = tr + (x * 116) + 28 + (y * 4);
                                    rawInfo.RawOffsetDeclaration = (int)br.BaseStream.Position - tag.MetaOffset;

                                    rawInfo.RawOffset = br.ReadInt32();
                                    br.BaseStream.Position += 20;
                                    rawInfo.RawSize = br.ReadInt32();

                                    if (rawInfo.RawOffset != -1 && rawInfo.RawSize != 0)
                                    {
                                        Map.Misc.ParsePointer(ref rawInfo.RawOffset,
                                            out rawInfo.RawLocation);

                                        rawInfoList.Add(rawInfo);
                                    }
                                }
                            }
                            break;
                        }
                }
                #endregion
            }

            // We need the offset of the first unicode index for later on.
            br.BaseStream.Position = TagsList[0].MetaOffset + 408;
            int firstUnicodeIndexOffset = br.ReadInt32();

            int oldPaddingLen = Map.Header.FileTableIndexOffset - (Map.Header.FileTableOffset + Map.Header.FileTableSize);
            int oldIndexOffset = Map.Header.IndexOffset;

            #endregion

            #region Store needed part of map in memory, write in raw if needed - Beginning Stage

            int bufferStartInMap = -1;
            byte[] buffer = null;
            int mhIndx = 0;
            int mhCount = 0;

            // Raw.
            int iShift = 0;
            int rawInjectOffset = 2048;
            if (rawType != RawType.None && rawType != RawType.Bitmap)
            {
                // Inject Raw
                rawInjectOffset = Map.Misc.GetNextAvailableInjectOffset(br, rawType);
                bufferStartInMap = rawInjectOffset;
                br.BaseStream.Position = bufferStartInMap;

                buffer = br.ReadBytes(
                    (Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaOffset +
                    Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaSize) - bufferStartInMap);

                br.BaseStream.Position = bufferStartInMap;
                for (int x = 0; x < rawInfoList.Count; x++)
                {
                    rawInfoList[x].NewRawOffset = (int)br.BaseStream.Position;

                    if (rawInfoList[x].RawLocation == RawLocation.Internal)
                    {
                        br.BaseStream.Position = rawInfoList[x].RawOffset;
                        byte[] rawBuffer = br.ReadBytes(rawInfoList[x].RawSize);

                        bw.BaseStream.Position = rawInfoList[x].NewRawOffset;
                        bw.Write(rawBuffer);
                    }
                    else
                    {
                        FileStream rawFs = Map.Prefs.GetRawStream(rawInfoList[x].RawLocation);
                        if (rawFs == null)
                        {
                            bw.Write(new byte[rawInfoList[x].RawSize]);
                        }
                        else
                        {
                            BitReader rawBr = new BitReader(rawFs, Endian.Little);
                            rawBr.BaseStream.Position = rawInfoList[x].RawOffset;
                            bw.Write(rawBr.ReadBytes(rawInfoList[x].RawSize));
                            rawBr.Close();
                        }
                    }

                    iShift += rawInfoList[x].RawSize;

                    int paddingLen = 512 - (rawInfoList[x].RawSize % 512);
                    if (paddingLen != 512)
                    {
                        bw.Write(new byte[paddingLen]);
                        iShift += paddingLen;
                    }
                }

                if (iShift != 0) // Unlikely, but anyways..
                {
                    mhCount = (Map.Header.FileTableOffset + TagsList[ughIndx].PathOffset) - bufferStartInMap;
                    bw.Write(buffer, mhIndx, mhCount);
                    mhIndx = Map.Header.FileTableIndexOffset - bufferStartInMap;

                    //--[Header]
                    bw.BaseStream.Position = 352;
                    bw.Write(Map.Header.StringTable128Offset += iShift);
                    bw.BaseStream.Position = 364;
                    bw.Write(Map.Header.StringTableIndexOffset += iShift);
                    bw.Write(Map.Header.StringTableOffset += iShift);
                    bw.BaseStream.Position = 708;
                    bw.Write(Map.Header.FileTableOffset += iShift);
                }
            }
            else // Store map downward from file index.
            {
                bufferStartInMap = Map.Header.FileTableIndexOffset;
                br.BaseStream.Position = bufferStartInMap;

                buffer = br.ReadBytes(
                    (Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaOffset +
                    Map.Tags.TagsList[Map.Tags.TagsList.Length - 1].MetaSize) - bufferStartInMap);
            }

            #endregion

            #region Write Info back in, Inserting new Information where Needed - Middle Stage

            //--[FileTable]
            bw.BaseStream.Position = Map.Header.FileTableOffset + TagsList[ughIndx].PathOffset;

            // Make new tag path for new tag.
            newTag.Path += "2";
            int num = 3;
            while (IndexOf(newTag.Type, newTag.Path) != -1)
                newTag.Path = tag.Path + (num++).ToString();

            newTag.PathLength = newTag.Path.Length;
            newTag.PathOffset = (int)bw.BaseStream.Position - Map.Header.FileTableOffset;

            // Write new path in.
            bw.Write(newTag.Path);
            bw.Write((byte)0);

            // Write ugh! path back in.
            TagsList[ughIndx].PathOffset = (int)bw.BaseStream.Position - Map.Header.FileTableOffset;
            bw.Write(TagsList[ughIndx].Path);
            bw.Write((byte)0);

            int shift = newTag.PathLength + 1;
            Map.Header.FileTableSize += shift;
            Map.Header.FileTableCount += 1;

            // Padding.
            int newPaddingLen = 512 - (Map.Header.FileTableSize % 512);
            if (newPaddingLen == 512) newPaddingLen = 0;
            else
            {
                byte[] padding = new byte[newPaddingLen];
                bw.Write(padding);
            }
            shift += (newPaddingLen - oldPaddingLen);

            //--[FileIndex]
            // New file index offset.
            Map.Header.FileTableIndexOffset = (int)bw.BaseStream.Position;

            // Write file index in.
            mhCount = TagsList.Length * 4;
            bw.Write(buffer, mhIndx, mhCount);

            oldPaddingLen = firstUnicodeIndexOffset - (bufferStartInMap + mhIndx + mhCount);

            // Write ugh!'s new path index in.
            bw.Write(TagsList[ughIndx].PathOffset);
            shift += 4;

            // mhCount + 4 is new file index size.
            newPaddingLen = 512 - ((mhCount + 4) % 512);
            if (newPaddingLen == 512) newPaddingLen = 0;
            else
            {
                byte[] padding = new byte[newPaddingLen];
                bw.Write(padding);
            }
            shift += (newPaddingLen - oldPaddingLen);
            shift += iShift;
            int uShift = shift;

            // Write unicode.
            mhIndx += mhCount + oldPaddingLen;
            mhCount = Map.Header.CrazyOffset - (bufferStartInMap + mhIndx);
            bw.Write(buffer, mhIndx, mhCount);

            // Write crazy and bitmap raw.
            Map.Header.CrazyOffset = (int)bw.BaseStream.Position;
            mhIndx += mhCount;
            mhCount = Map.Header.IndexOffset - (bufferStartInMap + mhIndx);
            bw.Write(buffer, mhIndx, mhCount);

            // Raw.
            if (rawType == RawType.Bitmap)
            {
                rawInjectOffset = (int)bw.BaseStream.Position;
                for (int x = 0; x < rawInfoList.Count; x++)
                {
                    rawInfoList[x].NewRawOffset = (int)bw.BaseStream.Position;

                    if (rawInfoList[x].RawLocation == RawLocation.Internal)
                    {
                        bw.Write(buffer,
                            rawInfoList[x].RawOffset - bufferStartInMap,
                            rawInfoList[x].RawSize);
                    }
                    else
                    {
                        FileStream rawFs = Map.Prefs.GetRawStream(rawInfoList[x].RawLocation);
                        if (rawFs == null)
                        {
                            bw.Write(new byte[rawInfoList[x].RawSize]);
                        }
                        else
                        {
                            BitReader rawBr = new BitReader(rawFs, Endian.Little);
                            rawBr.BaseStream.Position = rawInfoList[x].RawOffset;
                            bw.Write(rawBr.ReadBytes(rawInfoList[x].RawSize));
                            rawBr.Close();
                        }
                    }

                    iShift += rawInfoList[x].RawSize;

                    int paddingLen = 512 - (rawInfoList[x].RawSize % 512);
                    if (paddingLen != 512)
                    {
                        bw.Write(new byte[paddingLen]);
                        iShift += paddingLen;
                    }
                }
                shift += iShift;
            }

            // Write index header and wildcards in.
            Map.Header.IndexOffset = (int)bw.BaseStream.Position;
            mhIndx += mhCount;
            mhCount = Map.Index.TagInfoIndexOffset - (bufferStartInMap + mhIndx);
            bw.Write(buffer, mhIndx, mhCount);

            // Write tag info in, but not ugh!'s yet.
            mhIndx += mhCount;
            mhCount = (TagsList.Length - 1) * 16;
            bw.Write(buffer, mhIndx, mhCount);

            //--[TagInfo]
            // Write new tag info in.
            bw.WriteBTFString(newTag.Type);
            bw.Write(newTag.Identity);
            bw.Write(newTag.RawMetaOffset);
            bw.Write(newTag.MetaSize);

            // Write ugh tag info in.
            bw.WriteBTFString(TagsList[ughIndx].Type);
            bw.Write(TagsList[ughIndx].Identity);
            bw.Write(TagsList[ughIndx].RawMetaOffset);
            bw.Write(TagsList[ughIndx].MetaSize);

            shift += 16;
            Map.Index.TagInfoCount += 1;

            // Index padding.
            int indexDepth = (int)bw.BaseStream.Position - Map.Header.IndexOffset;
            if (indexDepth > Map.Header.IndexSize) // MINUS SOMETHING..?
            {
                // Meh.. we've run out of index space.. now wtf do we do...
                // Really must research this..
                // Map.header.indexSize += 16;
                // Map.header.nonRawSize += 16;
            }
            else
            {
                shift -= 16;
                mhIndx += mhCount + 16 + 16;
            }

            // Write in the rest of the index space if there is any, and
            // the meta, but not the ugh!'s yet again.
            mhCount = newTag.MetaOffset - (bufferStartInMap + mhIndx);
            bw.Write(buffer, mhIndx, mhCount);

            // Write in the new tag meta.
            bw.Write(newTagMeta);

            // Write in the ugh! tag meta.
            mhIndx += mhCount;
            mhCount = TagsList[ughIndx].MetaSize;
            bw.Write(buffer, mhIndx, mhCount);

            //--[End]
            newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
            newPaddingLen += (newPaddingLen == 512 ? 512 : 1024);

            int eof = (int)bw.BaseStream.Position + newPaddingLen;
            byte eofPadd = 205; // CD
            while (bw.BaseStream.Position < eof) bw.Write(eofPadd);
            bw.BaseStream.SetLength(eof);

            #endregion

            #region Update Values and Pointers - End Stage

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);

            bw.BaseStream.Position = 16;
            bw.Write(Map.Header.IndexOffset);
            bw.Write(Map.Header.IndexSize);

            int diff = (eof - Map.Header.MetaStart) - Map.Header.MetaSize;
            bw.Write(Map.Header.MetaSize += diff);
            bw.Write(Map.Header.NonRawSize += diff);

            bw.BaseStream.Position = 704;
            bw.Write(Map.Header.FileTableCount);
            bw.BaseStream.Position += 4;
            bw.Write(Map.Header.FileTableSize);
            bw.Write(Map.Header.FileTableIndexOffset);

            //--[Index]
            bw.BaseStream.Position = Map.Header.IndexOffset + 24;
            bw.Write(Map.Index.TagInfoCount);

            //--[New Tag Meta Reflexives]
            br.BaseStream.Position = (newTag.MetaOffset + shift) + 4;
            int mShift = newTag.MetaOffset - tag.MetaOffset;
            for (int x = 4; x < newTag.MetaSize; x += 4)
            {
                int rawReflx = br.ReadInt32();
                int transReflx = rawReflx - Map.Index.SecondaryMagic;

                // If within the old tag's bounds..
                if (transReflx >= tag.MetaOffset &&
                    transReflx <= tag.MetaOffset + newTag.MetaSize)
                {
                    // We have a reflexive in need of a fix ;x
                    bw.BaseStream.Position -= 4;
                    bw.Write(rawReflx + mShift);

                    bw.BaseStream.Position += 4;
                    x += 4;
                }
            }

            //--[UGH! Tag Meta Reflexives]
            br.BaseStream.Position = (TagsList[ughIndx].MetaOffset + shift) + 4;
            for (int x = 4; x < TagsList[ughIndx].MetaSize; x += 4)
            {
                int rawReflx = br.ReadInt32();
                int transReflx = rawReflx - Map.Index.SecondaryMagic;

                // If within the old ugh!'s bounds..
                if (transReflx >= newTag.MetaOffset &&
                    transReflx <= newTag.MetaOffset + TagsList[ughIndx].MetaSize)
                {
                    // We have a reflexive in need of a fix ;x
                    bw.BaseStream.Position -= 4;
                    bw.Write(rawReflx + newTag.MetaSize);

                    bw.BaseStream.Position += 4;
                    x += 4;
                }
            }

            //--[UGH! LoneIDs]   
            br.BaseStream.Position = (TagsList[ughIndx].MetaOffset + shift);
            for (int x = 0; x < TagsList[ughIndx].MetaSize; x += 4)
            {
                if (br.ReadInt32() == newTag.Identity)
                {
                    bw.BaseStream.Position -= 4;
                    bw.Write(TagsList[ughIndx].Identity);
                }
            }

            //--[Matg Ugh! Tag Reference]
            bw.BaseStream.Position = (TagsList[0].MetaOffset + shift) + 756;
            bw.Write(TagsList[ughIndx].Identity);

            if (shift != 0)
            {
                //--[Meta Offsets]
                for (int x = 0; x < TagsList.Length; x++)
                {
                    if (TagsList[x].Type != "sbsp" && TagsList[x].Type != "ltmp")
                        TagsList[x].MetaOffset += shift;
                }
                newTag.MetaOffset += shift;

                Map.Index.ShiftValuesInApp(shift);
            }

            if (uShift != 0)
            {
                //--[Unicode]
                br.BaseStream.Position = TagsList[0].MetaOffset + 408;
                // Read, shift, write.
                for (int x = 0; x < 8; x++)
                {
                    int newLangIndexOffset = br.ReadInt32() + uShift;
                    int newLangTableOffset = br.ReadInt32() + uShift;

                    br.BaseStream.Position -= 8;
                    bw.Write(newLangIndexOffset);
                    bw.Write(newLangTableOffset);

                    bw.BaseStream.Position += 20;
                }

                //--[Header]
                bw.BaseStream.Position = 344;
                bw.Write(Map.Header.CrazyOffset);

                //--[Bitmaps]
                Map.Misc.ShiftBitmapRawPointers(fs, uShift, 0);
            }

            // Raw Pointers.
            if (rawType != RawType.None)
            {
                for (int x = 0; x < rawInfoList.Count; x++)
                {
                    bw.BaseStream.Position = newTag.MetaOffset + rawInfoList[x].RawOffsetDeclaration;
                    bw.Write(rawInfoList[x].NewRawOffset);
                }
                Map.Misc.ShiftRawPointers(fs, iShift, rawInjectOffset);
            }

            #endregion

            // Add tag to the array.
            Tag[] tmpList = new Tag[TagsList.Length + 1];
            TagsList.CopyTo(tmpList, 0);
            tmpList[tmpList.Length - 1] = tmpList[ughIndx];
            tmpList[ughIndx] = newTag;
            TagsList = tmpList;

            return newTag;
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class Tag
    {
        #region Instance Data

        private int __identity__;
        private string __type__;
        private string __path__;
        private int __pathOffset__;
        private int __pathLength__;
        private int __rawMetaOffset__;
        private int __metaOffset__;
        private int __metaSize__;

        #endregion

        #region Accessors

        ///<summary>
        /// Identity
        ///</summary>
        public int Identity { get { return __identity__; } set { __identity__ = value; } }
        ///<summary>
        /// Type
        ///</summary>
        public string Type { get { return __type__; } set { if (value.Length <= 4) __type__ = value; } }
        ///<summary>
        /// Path
        ///</summary>
        public string Path { get { return __path__; } set { __path__ = value; } }
        ///<summary>
        /// PathOffset
        ///</summary>
        public int PathOffset { get { return __pathOffset__; } set { __pathOffset__ = value; } }
        ///<summary>
        /// PathLength
        ///</summary>
        public int PathLength { get { return __pathLength__; } set { __pathLength__ = value; } }
        ///<summary>
        /// RawMetaOffset
        ///</summary>
        public int RawMetaOffset { get { return __rawMetaOffset__; } set { __rawMetaOffset__ = value; } }
        ///<summary>
        /// MetaOffset
        ///</summary>
        public int MetaOffset { get { return __metaOffset__; } set { __metaOffset__ = value; } }
        ///<summary>
        /// MetaSize
        ///</summary>
        public int MetaSize { get { return __metaSize__; } set { __metaSize__ = value; } }

        #endregion

        /// <summary>
        /// Creates an exact replica of the current H2UtilityToolkit.Essentials.Tag.
        /// </summary>
        public Tag Clone()
        {
            Tag tag = new Tag();
            tag.Identity = this.Identity;
            tag.Type = this.Type;
            tag.Path = this.Path;
            tag.PathOffset = this.PathOffset;
            tag.PathLength = this.PathLength;
            tag.RawMetaOffset = this.RawMetaOffset;
            tag.MetaOffset = this.MetaOffset;
            tag.MetaSize = this.MetaSize;
            return tag;
        }
    }

    public enum SortTagsBy
    {
        TagType,
        FolderHierarchy
    }
}