using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Extended;

using H2UtilityToolkit.Essentials;
using H2UtilityToolkit.Misc;
//using H2UtilityToolkit.PluginStuffs;
using H2UtilityToolkit.Structures;

using MDXUtilityToolkit;

namespace H2UtilityToolkit
{
    public class Map : IDisposable
    {
        #region Instance Data

        private string __fileName__;
        private bool __loaded__;
        private MDXFramework __mDXFrameWork__;
        private Header __header__;
        private Index __index__;
        private Tags __tags__;
        private Sbsp __sBSP__;
        private Strings __strings__;
        private MiscUtilities __misc__;
        private Preferences __prefs__;

        #endregion

        #region Accessors

        ///<summary>
        /// FileName
        ///</summary>
        public string FileName { get { return __fileName__; } set { __fileName__ = value; } }
        /// <summary>
        /// Loaded
        /// </summary>
        public bool Loaded { get { return __loaded__; } set { __loaded__ = value; } }
        /// <summary>
        /// MDXFrameWork
        /// </summary>
        public MDXFramework MDXFrameWork { get { return __mDXFrameWork__; } set { __mDXFrameWork__ = value; } }
        ///<summary>
        /// Header
        ///</summary>
        public Header Header { get { return __header__; } set { __header__ = value; } }
        ///<summary>
        /// Index
        ///</summary>
        public Index Index { get { return __index__; } set { __index__ = value; } }
        ///<summary>
        /// Tags
        ///</summary>
        public Tags Tags { get { return __tags__; } set { __tags__ = value; } }
        ///<summary>
        /// SBSP
        ///</summary>
        public Sbsp SBSP { get { return __sBSP__; } set { __sBSP__ = value; } }
        ///<summary>
        /// Strings
        ///</summary>
        public Strings Strings { get { return __strings__; } set { __strings__ = value; } }
        /// <summary>
        /// Misc
        /// </summary>
        public MiscUtilities Misc { get { return __misc__; } set { __misc__ = value; } }
        /// <summary>
        /// Prefs
        /// </summary>
        public Preferences Prefs { get { return __prefs__; } set { __prefs__ = value; } }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="statusProgressBar"></param>
        /// <param name="statusReporter"></param>
        public Map()
        {
            this.MDXFrameWork = new MDXFramework(this);
            this.Prefs = new Preferences(this);
            this.Misc = new MiscUtilities(this);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="statusProgressBar"></param>
        /// <param name="statusReporter"></param>
        /// <param name="filename"></param>
        public Map(string fileName)
        {
            this.MDXFrameWork = new MDXFramework(this);
            this.Prefs = new Preferences(this);
            this.Misc = new MiscUtilities(this);
            this.Load(fileName);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="statusProgressBar"></param>
        /// <param name="statusReporter"></param>
        /// <param name="br"></param>
        public Map(BitReader br)
        {
            this.MDXFrameWork = new MDXFramework(this);
            this.Prefs = new Preferences(this);
            this.Misc = new MiscUtilities(this);
            this.Load(br);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="filename"></param>
        public void Load(string fileName)
        {
            BitReader br = new BitReader(fileName, Endian.Little);
            this.Load(br);
            br.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="br"></param>
        public void Load(BitReader br)
        {
            this.Loaded = false;
            this.FileName = (br.BaseStream as System.IO.FileStream).Name;

            this.Header = new Header(this, br);
            this.Index = new Index(this, br);
            this.Tags = new Tags(this, br);
            this.SBSP = new Sbsp(this, -1, br);
            this.Strings = new Strings(this, br);

            this.Loaded = true;
        }

        /// <summary>
        /// 
        /// </summary>
        public void Resign()
        {
            BitReader br = new BitReader(FileName, Endian.Little);
            BitWriter bw = new BitWriter(br.BaseStream, Endian.Little);

            int sig = 0;
            long length = br.BaseStream.Length;

            br.BaseStream.Position = 2048;

            for (long x = 2048; x < length; x += 4)
                sig ^= br.ReadInt32();

            br.BaseStream.Position = 720;
            bw.Write(sig);

            bw.Close();
            br.Close();

            Header.Checksum = sig;
        }

        #region IDisposable Members

        public void Dispose()
        {
            MDXFrameWork.Dispose();
        }

        /// <summary>In case they forget to dispose</summary>
        ~Map()
        {
            Dispose();
        }

        #endregion
    }
}