using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Extended;
using H2UtilityToolkit.Essentials;

namespace H2UtilityToolkit.Misc
{
    /// <summary>
    /// 
    /// </summary>
    public class MiscUtilities
    {
        #region Instance Data

        private Map __map__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        public MiscUtilities(Map map)
        {
            this.Map = map;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ptr"></param>
        /// <param name="rawLoc"></param>
        public void ParsePointer(ref int ptr, out RawLocation rawLoc)
        {
            // ptr
            long ptrLoc = ptr & 0xC0000000;
            ptr &= 0x3FFFFFFF;

            // rawloc
            switch (ptrLoc)
            {
                case 0x80000000:
                    rawLoc = RawLocation.Shared;
                    break;

                case 0xC0000000:
                    rawLoc = RawLocation.Single_Player_Shared;
                    break;

                case 0x40000000:
                    rawLoc = RawLocation.MainMenu;
                    break;

                default:
                    rawLoc = RawLocation.Internal;
                    break;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="type"></param>
        public RawType GetRawType(string type)
        {
            switch (type)
            {
                case "snd!": return RawType.Sound;
                case "mode": return RawType.Model;
                case "sbsp": return RawType.BSP;
                case "ltmp": return RawType.Lightmap;
                case "weat": return RawType.Weather;
                case "DECR": return RawType.Decorator;
                case "PRTM": return RawType.ParticleModel;
                case "ugh!": return RawType.CoconutsModel;
                case "jmad": return RawType.Animation;
                case "bitm": return RawType.Bitmap;
                default: return RawType.None;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="br"></param>
        /// <param name="rawType"></param>
        public int GetNextAvailableInjectOffset(BitReader br, RawType rawType)
        {
            int latterOffset;
            int latterSize;

        Beginning:
            latterOffset = 2048;
            latterSize = 0;

            switch (rawType)
            {
                case RawType.Sound:
                    {
                        Tag ughTag = Map.Tags.TagsList[Map.Tags.IndexOf("ugh!",
                            "i've got a lovely bunch of coconuts")];

                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "snd!")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 8;
                                int sndPermIndex = br.ReadInt16();
                                int sndPermChunkCount = br.ReadByte();

                                // 5th reflex in ugh tag; sound permutations
                                br.BaseStream.Position = ughTag.MetaOffset + 32 + 4;
                                int sndPermReflex = br.ReadInt32() - Map.Index.SecondaryMagic;

                                br.BaseStream.Position = sndPermReflex + (sndPermIndex * 12) + 8;
                                int sndChoiceIndex = br.ReadInt16();
                                int sndChoiceChunkCount = br.ReadInt16();

                                // 6th reflex in ugh tag; sound choices
                                br.BaseStream.Position = ughTag.MetaOffset + 40 + 4;
                                int sndChoiceReflex = br.ReadInt32() - Map.Index.SecondaryMagic;

                                br.BaseStream.Position = sndChoiceReflex + (sndChoiceIndex * 16) + 12;
                                int sndRawIndex = br.ReadInt16();
                                int sndRawChunkCount = br.ReadInt16();

                                // 9th reflex in ugh tag; sound raw
                                br.BaseStream.Position = ughTag.MetaOffset + 64 + 4;
                                int sndRawReflex = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < sndRawChunkCount; y++)
                                {
                                    br.BaseStream.Position =
                                        sndRawReflex + (sndRawIndex * 12) + (y * 12);

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                    }
                case RawType.Model:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "mode")
                            {
                                // Raw1.
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 36;

                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < cc; y++)
                                {
                                    br.BaseStream.Position = tr + (y * 92) + 56;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }

                                // Raw2.
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 116;

                                cc = br.ReadInt32();
                                tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < cc; y++)
                                {
                                    br.BaseStream.Position = tr + (y * 88) + 52;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.Sound;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.BSP:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "sbsp")
                            {
                                int y = Map.SBSP.IndexOfSBSP(Map.Tags.TagsList[x].Identity);
                                if (y == -1) continue;

                                // Raw1.
                                br.BaseStream.Position = Map.SBSP.BSPDeclarations[y].MetaOffset + 172;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic;

                                for (int z = 0; z < cc; z++)
                                {
                                    br.BaseStream.Position = tr + (z * 176) + 40;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }

                                // Raw2
                                br.BaseStream.Position = Map.SBSP.BSPDeclarations[y].MetaOffset + 328;

                                cc = br.ReadInt32();
                                tr = br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic;

                                for (int z = 0; z < cc; z++)
                                {
                                    br.BaseStream.Position = tr + (z * 200) + 40;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }

                                // Raw3.
                                br.BaseStream.Position = Map.SBSP.BSPDeclarations[y].MetaOffset + 580;

                                cc = br.ReadInt32();
                                tr = br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic;

                                br.BaseStream.Position = tr + 16;
                                cc = br.ReadInt32();
                                tr = br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic;

                                for (int z = 0; z < cc; z++)
                                {
                                    br.BaseStream.Position = tr + (z * 44);

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }

                                // Raw4.
                                br.BaseStream.Position = Map.SBSP.BSPDeclarations[y].MetaOffset + 548;

                                cc = br.ReadInt32();
                                tr = br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic;

                                for (int z = 0; z < cc; z++)
                                {
                                    br.BaseStream.Position = tr + (z * 172) + 16;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.Model;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.Lightmap:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "ltmp")
                            {
                                int y = Map.SBSP.IndexOfSBSP(Map.Tags.TagsList[x].Identity);
                                if (y == -1) continue;

                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 128 + 4;
                                br.BaseStream.Position = (br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic) + 64;

                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.SBSP.BSPDeclarations[y].Magic;

                                for (int z = 0; z < cc; z++)
                                {
                                    br.BaseStream.Position = tr + (z * 56) + 12;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.BSP;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.Weather:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "weat")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset;

                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < cc; y++)
                                {
                                    br.BaseStream.Position = tr + (y * 140) + 64;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.Lightmap;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.Decorator:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "DECR")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 56;

                                int rOffset = br.ReadInt32();
                                int rSize = br.ReadInt32();

                                if (rOffset != -1 && rSize != 0)
                                {
                                    RawLocation rLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                    if (rLoc == RawLocation.Internal)
                                    {
                                        if (rOffset > latterOffset)
                                        {
                                            latterOffset = rOffset;
                                            latterSize = rSize;
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.Weather;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.ParticleModel:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "PRTM")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 160;

                                int rOffset = br.ReadInt32();
                                int rSize = br.ReadInt32();

                                if (rOffset != -1 && rSize != 0)
                                {
                                    RawLocation rLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                    if (rLoc == RawLocation.Internal)
                                    {
                                        if (rOffset > latterOffset)
                                        {
                                            latterOffset = rOffset;
                                            latterSize = rSize;
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.Decorator;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.CoconutsModel:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "ugh!")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 80;

                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < cc; y++)
                                {
                                    br.BaseStream.Position = tr + (y * 44) + 8;

                                    int rOffset = br.ReadInt32();
                                    int rSize = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.ParticleModel;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.Animation:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "jmad")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 172;

                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < cc; y++)
                                {
                                    br.BaseStream.Position = tr + (y * 20) + 4;

                                    int rSize = br.ReadInt32();
                                    int rOffset = br.ReadInt32();

                                    if (rOffset != -1 && rSize != 0)
                                    {
                                        RawLocation rLoc;
                                        Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                        if (rLoc == RawLocation.Internal)
                                        {
                                            if (rOffset > latterOffset)
                                            {
                                                latterOffset = rOffset;
                                                latterSize = rSize;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.CoconutsModel;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                case RawType.Bitmap:
                    {
                        for (int x = 0; x < Map.Tags.TagsList.Length; x++)
                        {
                            if (Map.Tags.TagsList[x].Type == "bitm")
                            {
                                br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 68;

                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                                for (int y = 0; y < cc; y++)
                                {
                                    for (int z = 0; z < 6; z++)
                                    {
                                        br.BaseStream.Position =
                                            tr + (y * 116) + 28 + (z * 4);

                                        int rOffset = br.ReadInt32();
                                        br.BaseStream.Position += 20;
                                        int rSize = br.ReadInt32();

                                        if (rOffset != -1 && rSize != 0)
                                        {
                                            RawLocation rLoc;
                                            Map.Misc.ParsePointer(ref rOffset, out rLoc);

                                            if (rLoc == RawLocation.Internal)
                                            {
                                                if (rOffset > latterOffset)
                                                {
                                                    latterOffset = rOffset;
                                                    latterSize = rSize;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        if (latterOffset == 2048)
                        {
                            rawType = RawType.Animation;
                            goto Beginning;
                        }
                        else
                        {
                            return latterOffset + latterSize + ((512 - (latterSize % 512)) % 512);
                        }
                    }
                default:
                    {
                        return 2048;
                    }
            }
        }

        /// <summary>
        /// Does not include bitmap raw
        /// </summary>
        /// <param name="br"></param>
        /// <param name="bw"></param>
        /// <param name="shift"></param>
        /// <param name="minOffset"></param>
        public void ShiftRawPointers(FileStream fs, int shift, int minOffset)
        {
            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            Map.SBSP.ShiftPointers(fs, shift, minOffset);

            for (int x = 0; x < Map.Tags.TagsList.Length; x++)
            {
                #region Switch

                switch (Map.Tags.TagsList[x].Type)
                {
                    case "snd!":
                        {
                            Tag ughTag = Map.Tags.TagsList[Map.Tags.IndexOf("ugh!",
                                "i've got a lovely bunch of coconuts")];

                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 8;
                            int sndPermIndex = br.ReadInt16();
                            int sndPermChunkCount = br.ReadByte();

                            // 5th reflex in ugh tag; sound permutations
                            br.BaseStream.Position = ughTag.MetaOffset + 32 + 4;
                            int sndPermReflex = br.ReadInt32() - Map.Index.SecondaryMagic;

                            br.BaseStream.Position = sndPermReflex + (sndPermIndex * 12) + 8;
                            int sndChoiceIndex = br.ReadInt16();
                            int sndChoiceChunkCount = br.ReadInt16();

                            // 6th reflex in ugh tag; sound choices
                            br.BaseStream.Position = ughTag.MetaOffset + 40 + 4;
                            int sndChoiceReflex = br.ReadInt32() - Map.Index.SecondaryMagic;

                            br.BaseStream.Position = sndChoiceReflex + (sndChoiceIndex * 16) + 12;
                            int sndRawIndex = br.ReadInt16();
                            int sndRawChunkCount = br.ReadInt16();

                            // 9th reflex in ugh tag; sound raw
                            br.BaseStream.Position = ughTag.MetaOffset + 64 + 4;
                            int sndRawReflex = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < sndRawChunkCount; y++)
                            {
                                br.BaseStream.Position =
                                    sndRawReflex + (sndRawIndex * 12) + (y * 12);

                                int rOffset = br.ReadInt32();
                                if (rOffset != -1)
                                {
                                    RawLocation rawLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                    {
                                        bw.BaseStream.Position -= 4;
                                        bw.Write(rOffset + shift);
                                    }
                                }
                            }
                            break;
                        }
                    case "mode":
                        {
                            // Raw1
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 36;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < cc; y++)
                            {
                                br.BaseStream.Position = tr + (y * 92) + 56;

                                int rOffset = br.ReadInt32();
                                if (rOffset != -1)
                                {
                                    RawLocation rawLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                    {
                                        bw.BaseStream.Position -= 4;
                                        bw.Write(rOffset + shift);
                                    }
                                }
                            }

                            // Raw2.
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 116;

                            cc = br.ReadInt32();
                            tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < cc; y++)
                            {
                                br.BaseStream.Position = tr + (y * 88) + 52;

                                int rOffset = br.ReadInt32();
                                if (rOffset != -1)
                                {
                                    RawLocation rawLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                    {
                                        bw.BaseStream.Position -= 4;
                                        bw.Write(rOffset + shift);
                                    }
                                }
                            }

                            break;
                        }
                    //case "sbsp": break;
                    //case "ltmp": break;
                    case "weat":
                        {
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < cc; y++)
                            {
                                br.BaseStream.Position = tr + (y * 140) + 64;

                                int rOffset = br.ReadInt32();
                                if (rOffset != -1)
                                {
                                    RawLocation rawLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                    {
                                        bw.BaseStream.Position -= 4;
                                        bw.Write(rOffset + shift);
                                    }
                                }
                            }
                            break;
                        }
                    case "DECR":
                        {
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 56;

                            int rOffset = br.ReadInt32();
                            if (rOffset != -1)
                            {
                                RawLocation rawLoc;
                                Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                {
                                    bw.BaseStream.Position -= 4;
                                    bw.Write(rOffset + shift);
                                }
                            }
                            break;
                        }
                    case "PRTM":
                        {
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 160;

                            int rOffset = br.ReadInt32();
                            if (rOffset != -1)
                            {
                                RawLocation rawLoc;
                                Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                {
                                    bw.BaseStream.Position -= 4;
                                    bw.Write(rOffset + shift);
                                }
                            }
                            break;
                        }
                    case "ugh!":
                        {
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 80;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < cc; y++)
                            {
                                br.BaseStream.Position = tr + (y * 44) + 8;

                                int rOffset = br.ReadInt32();
                                if (rOffset != -1)
                                {
                                    RawLocation rawLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                    {
                                        bw.BaseStream.Position -= 4;
                                        bw.Write(rOffset + shift);
                                    }
                                }
                            }
                            break;
                        }
                    case "jmad":
                        {
                            br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 172;

                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                            for (int y = 0; y < cc; y++)
                            {
                                br.BaseStream.Position = tr + (y * 20) + 4 + 4;

                                int rOffset = br.ReadInt32();
                                if (rOffset != -1)
                                {
                                    RawLocation rawLoc;
                                    Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                    if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                    {
                                        bw.BaseStream.Position -= 4;
                                        bw.Write(rOffset + shift);
                                    }
                                }
                            }
                            break;
                        }
                    //case "bitm": break;
                }

                #endregion
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="br"></param>
        /// <param name="bw"></param>
        /// <param name="shift"></param>
        /// <param name="minOffset"></param>
        public void ShiftBitmapRawPointers(FileStream fs, int shift, int minOffset)
        {
            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            for (int x = 0; x < Map.Tags.TagsList.Length; x++)
            {
                if (Map.Tags.TagsList[x].Type == "bitm")
                {
                    br.BaseStream.Position = Map.Tags.TagsList[x].MetaOffset + 68;

                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

                    for (int y = 0; y < cc; y++)
                    {
                        for (int z = 0; z < 6; z++)
                        {
                            br.BaseStream.Position =
                                tr + (y * 116) + 28 + (z * 4);

                            int rOffset = br.ReadInt32();
                            if (rOffset != -1)
                            {
                                RawLocation rawLoc;
                                Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                                if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                                {
                                    bw.BaseStream.Position -= 4;
                                    bw.Write(rOffset + shift);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public enum RawLocation
    {
        Internal,
        MainMenu,
        Shared,
        Single_Player_Shared
    }

    public enum RawType
    {
        None,
        Sound,
        Model,
        BSP,
        Lightmap,
        Weather,
        Decorator,
        ParticleModel,
        CoconutsModel,
        Animation,
        Bitmap
    }

    public class RawInfo
    {
        public int RawOffsetDeclaration;
        public int RawOffset;
        public int RawSize;
        public RawLocation RawLocation;
        public int NewRawOffset;
    }

    public class H2TagSorter : System.Collections.IComparer
    {
        public int Compare(object x, object y)
        {
            return ((Tag)x).Path.CompareTo(((Tag)y).Path);
        }
    }
}