using System;
using System.Collections.Generic;
using System.Text;
using System.Extended;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Prophet
{
    public class BipedSpawns : SpawnsContainer
    {
        public BipedSpawns()
        {
        }

        public override void Load(BitReader hr, int scnrMetaOffset)
        {
            // Bipeds - Models
            hr.BaseStream.Position = scnrMetaOffset + 104;
            int cc = hr.ReadInt32();
            int tr = hr.ReadInt32() - Map.index.secondaryMagic;

            SpawnModels = new SpawnModel[cc];
            int[] tagIndices = new int[cc];

            for (int x = 0; x < cc; x++)
            {
                hr.BaseStream.Position = tr + (x * 40) + 4;
                int tagIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                tagIndices[x] = tagIndx;
                SpawnModels[x].TagIndex = Map.tags.FindModeTagIndexByBaseClass(hr, tagIndx);

                if (SpawnModels[x].TagIndex == -1) continue;
                SpawnModels[x].TagPath = Map.tags.tagsList[SpawnModels[x].TagIndex].Path;
            }

            // Bipeds - Placement
            hr.BaseStream.Position = scnrMetaOffset + 96;
            cc = hr.ReadInt32();
            tr = hr.ReadInt32() - Map.index.secondaryMagic;

            Spawns = new BipedSpawn[cc];

            for (int x = 0; x < cc; x++)
            {
                hr.BaseStream.Position = tr + (x * 84);

                short modeIndx = hr.ReadInt16();
                if (modeIndx == -1 || SpawnModels[modeIndx].TagIndex == -1) continue;

                hr.BaseStream.Position += 6;

                BipedSpawn vs = new BipedSpawn();
                vs.MapOffset = (int)hr.BaseStream.Position;
                vs.X = hr.ReadSingle();
                vs.Y = hr.ReadSingle();
                vs.Z = hr.ReadSingle();
                vs.DirRotation = hr.ReadSingle();
                vs.CreateMatrix();

                vs.ModelIndex = modeIndx;// SpawnModels[modeIndx];
                vs.TagPath = Map.tags.tagsList[tagIndices[modeIndx]].Path;

                Spawns[x] = vs;
            }
        }
    }

    public class BipedSpawn : DirectionalSpawn
    {
        public BipedSpawn()
        {
            this.SpawnType = SpawnTypes.Biped;
            this.RotationType = SpawnRotationTypes.Directional;
        }
    }
}
