using System;
using System.Collections.Generic;
using System.Text;
using System.Extended;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Prophet
{
    public class ScenerySpawns : SpawnsContainer
    {
        public ScenerySpawns()
        {
        }

        public override void Load(BitReader hr, int scnrMetaOffset)
        {
            // Scenery - Models
            hr.BaseStream.Position = scnrMetaOffset + 88;

            int cc = hr.ReadInt32();
            int tr = hr.ReadInt32() - Map.index.secondaryMagic;

            SpawnModels = new SpawnModel[cc];
            int[] tagIndices = new int[cc];

            for (int x = 0; x < cc; x++)
            {
                hr.BaseStream.Position = tr + (x * 40) + 4;
                int tagIndx = Map.tags.FindTagIndexByID(hr.ReadInt32());

                tagIndices[x] = tagIndx;
                SpawnModels[x].TagIndex = Map.tags.FindModeTagIndexByBaseClass(hr, tagIndx);

                if (SpawnModels[x].TagIndex == -1) continue;
                SpawnModels[x].TagPath = Map.tags.tagsList[SpawnModels[x].TagIndex].Path;
            }

            // Scenery - Placement
            hr.BaseStream.Position = scnrMetaOffset + 80;
            cc = hr.ReadInt32();
            tr = hr.ReadInt32() - Map.index.secondaryMagic;

            Spawns = new ScenerySpawn[cc];

            for (int x = 0; x < cc; x++)
            {
                hr.BaseStream.Position = tr + (x * 92);

                short modeIndx = hr.ReadInt16();
                if (modeIndx == -1 || SpawnModels[modeIndx].TagIndex == -1) continue;

                hr.BaseStream.Position += 6;

                ScenerySpawn vs = new ScenerySpawn();
                vs.MapOffset = (int)hr.BaseStream.Position;
                vs.X = hr.ReadSingle();
                vs.Y = hr.ReadSingle();
                vs.Z = hr.ReadSingle();
                vs.Roll = hr.ReadSingle();
                vs.Pitch = hr.ReadSingle();
                vs.Yaw = hr.ReadSingle();
                vs.CreateMatrix();

                vs.ModelIndex = modeIndx;// SpawnModels[modeIndx];
                vs.TagPath = Map.tags.tagsList[tagIndices[modeIndx]].Path;

                Spawns[x] = vs;
            }
        }
    }

    public class ScenerySpawn : YawPitchRollSpawn
    {
        public ScenerySpawn()
        {
            this.SpawnType = SpawnTypes.Scenery;
            this.RotationType = SpawnRotationTypes.YawPitchRoll;
        }
    }
}
