using System;
using System.Collections.Generic;
using System.Text;
using System.Extended;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Prophet
{
    public class SpawnInfo
    {
        public BipedSpawns BipedSpawns;
        public CameraSpawns CameraSpawns;
        public CollectionSpawns CollectionSpawns;
        public ControlSpawns ControlSpawns;
        public DeathZoneSpawns DeathZoneSpawns;
        public EquipmentSpawns EquipmentSpawns;
        public LightSpawns LightSpawns;
        public MachineSpawns MachineSpawns;
        public ObjectiveSpawns ObjectiveSpawns;
        public ObstacleSpawns ObstacleSpawns;
        public PlayerSpawns PlayerSpawns;
        public ScenerySpawns ScenerySpawns;
        public SpecialSpawns SpecialSpawns;
        public VehicleSpawns VehicleSpawns;
        public WeaponSpawns WeaponSpawns;

        public SpawnInfo()
        {
            BipedSpawns = new BipedSpawns();
            CameraSpawns = new CameraSpawns();
            CollectionSpawns = new CollectionSpawns();
            ControlSpawns = new ControlSpawns();
            DeathZoneSpawns = new DeathZoneSpawns();
            EquipmentSpawns = new EquipmentSpawns();
            LightSpawns = new LightSpawns();
            MachineSpawns = new MachineSpawns();
            ObjectiveSpawns = new ObjectiveSpawns();
            ObstacleSpawns = new ObstacleSpawns();
            PlayerSpawns = new PlayerSpawns();
            ScenerySpawns = new ScenerySpawns();
            SpecialSpawns = new SpecialSpawns();
            VehicleSpawns = new VehicleSpawns();
            WeaponSpawns = new WeaponSpawns();
        }

        public void Load(BitReader hr)
        {
            // Get the scnr index
            int scnrIndx = Map.tags.FindTagIndexByID(Map.index.scnrID);
            if (scnrIndx == -1)
            {
                if (Map.tags.tagsList.Length < 3 || Map.tags.tagsList[3].Type != "scnr") return;
                else scnrIndx = 3;
            }
            int scnrMetaOffset = Map.tags.tagsList[scnrIndx].MetaOffset;

            // Load
            BipedSpawns.Load(hr, scnrMetaOffset);
            CameraSpawns.Load(hr, scnrMetaOffset);
            CollectionSpawns.Load(hr, scnrMetaOffset);
            ControlSpawns.Load(hr, scnrMetaOffset);
            DeathZoneSpawns.Load(hr, scnrMetaOffset);
            EquipmentSpawns.Load(hr, scnrMetaOffset);
            LightSpawns.Load(hr, scnrMetaOffset);
            MachineSpawns.Load(hr, scnrMetaOffset);
            ObjectiveSpawns.Load(hr, scnrMetaOffset);
            ObstacleSpawns.Load(hr, scnrMetaOffset);
            PlayerSpawns.Load(hr, scnrMetaOffset);
            ScenerySpawns.Load(hr, scnrMetaOffset);
            SpecialSpawns.Load(hr, scnrMetaOffset);
            VehicleSpawns.Load(hr, scnrMetaOffset);
            WeaponSpawns.Load(hr, scnrMetaOffset);

            // Load Models
            BipedSpawns.LoadModels();
            CameraSpawns.LoadModels();
            CollectionSpawns.LoadModels();
            ControlSpawns.LoadModels();
            DeathZoneSpawns.LoadModels();
            EquipmentSpawns.LoadModels();
            LightSpawns.LoadModels();
            MachineSpawns.LoadModels();
            ObjectiveSpawns.LoadModels();
            ObstacleSpawns.LoadModels();
            PlayerSpawns.LoadModels();
            ScenerySpawns.LoadModels();
            SpecialSpawns.LoadModels();
            VehicleSpawns.LoadModels();
            WeaponSpawns.LoadModels();

            //find default mc model
            //hr.BaseStream.Position = Map.tags.tagsList[0].MetaOffset + 308;
            //int tempr = hr.ReadInt32() - secondaryMagic;
            //hr.BaseStream.Position = tempr + 4;
            //int tempbipdtag = Meta.FindMetaByID(hr.ReadInt32(), Map.tags.tagsList);
            //int bipdmodeltag = tag.FindModelByBaseClass(tempbipdtag, mapLoc);
            /*
            //Find objective models
            hr.BaseStream.Position = Map.tags.tagsList[tag.FindByTagAndTagType("mulg", "multiplayer\\multiplayer_globals")].MetaOffset + 12;
            tempr = hr.ReadInt32();
            int ctfmodeltag = -1;
            int ballmodeltag = -1;
            int juggernautdmodeltag = -1;
            int assultmodeltag = -1;
            if (tempr != 0)
            {
                tempr -= secondaryMagic;
                hr.BaseStream.Position = tempr + 4;
                int tempCtftag = Meta.FindMetaByID(hr.ReadInt32(), Map.tags.tagsList);
                hr.BaseStream.Position = tempr + 12;
                int tempBalltag = Meta.FindMetaByID(hr.ReadInt32(), Map.tags.tagsList);
                hr.BaseStream.Position = tempr + 52;
                int tempJuggernauttag = Meta.FindMetaByID(hr.ReadInt32(), Map.tags.tagsList);
                hr.BaseStream.Position = tempr + 60;
                int tempAssaulttag = Meta.FindMetaByID(hr.ReadInt32(), Map.tags.tagsList);

                ctfmodeltag = tag.FindModelByBaseClass(tempCtftag, mapLoc);
                ballmodeltag = tag.FindModelByBaseClass(tempBalltag, mapLoc);
                juggernautdmodeltag = tag.FindModelByBaseClass(tempJuggernauttag, mapLoc);
                assultmodeltag = tag.FindModelByBaseClass(tempAssaulttag, mapLoc);
            }
            */
        }

        public void Render()
        {
            if (Map.prefs.ShowBipedSpawns) BipedSpawns.Render();
            if (Map.prefs.ShowCameraSpawns) CameraSpawns.Render();
            if (Map.prefs.ShowCollectionSpawns) CollectionSpawns.Render();
            if (Map.prefs.ShowControlSpawns) ControlSpawns.Render();
            if (Map.prefs.ShowDeathZoneSpawns) DeathZoneSpawns.Render();
            if (Map.prefs.ShowEquipmentSpawns) EquipmentSpawns.Render();
            if (Map.prefs.ShowLightSpawnsSpawns) LightSpawns.Render();
            if (Map.prefs.ShowMachineSpawns) MachineSpawns.Render();
            if (Map.prefs.ShowObjectiveSpawns) ObjectiveSpawns.Render();
            if (Map.prefs.ShowObstacleSpawns) ObstacleSpawns.Render();
            if (Map.prefs.ShowPlayerSpawns) PlayerSpawns.Render();
            if (Map.prefs.ShowScenerySpawns) ScenerySpawns.Render();
            if (Map.prefs.ShowSpecialSpawns) SpecialSpawns.Render();
            if (Map.prefs.ShowVehicleSpawns) VehicleSpawns.Render();
            if (Map.prefs.ShowWeaponSpawns) WeaponSpawns.Render();
        }
    }

    public struct SpawnModel
    {
        public int TagIndex;
        public string TagPath;

        public Mesh Mesh;
        public Mesh BoundingBox;

        public void LoadModel()
        {
            // Load Model

            // Load BoundingBox
            //Vector3 bbmin, bbmax;
            //GraphicsStream gs = Mesh.LockVertexBuffer(LockFlags.None);
            //Geometry.ComputeBoundingBox(gs, VerticeCount, HVertexFormat.Format, out bbmin, bbmax);
            //Mesh.UnlockVertexBuffer();
            //BoundingBox = Mesh.Box(Map.dev, bbmax.X - bbmin.X, bbmax.Y - bbmin.Y, bbmax.Z - bbmin.Z);            
        }
    }

    public class SpawnsContainer
    {
        public SpawnModel[] SpawnModels;
        public BaseSpawn[] Spawns;

        public SpawnsContainer()
        {
        }
        
        public virtual void Load(BitReader hr, int scnrMetaOffset)
        {

        }

        public virtual void LoadModels()
        {
            for (int x = 0; x < SpawnModels.Length; x++)
            {
                SpawnModels[x].LoadModel();
            }
        }

        public virtual void Render()
        {
            for (int x = 0; x < Spawns.Length; x++)
            {
                //TEMP
                if (SpawnModels[Spawns[x].ModelIndex].Mesh == null) continue;
                //TEMP

                Map.dev.Transform.World = Spawns[x].Matrix;

                if (Spawns[x].IsSelected)
                {

                }
                else
                {

                }

                
                SpawnModels[Spawns[x].ModelIndex].Mesh.DrawSubset(0);
            }
        }
    }

    public class BaseSpawn
    {
        public SpawnTypes SpawnType;
        public SpawnRotationTypes RotationType;

        public int MapOffset;
        
        public float X;
        public float Y;
        public float Z;
        public Matrix Matrix;

        public int ModelIndex;
        public string TagPath;

        public bool IsSelected;
    }

    public class DirectionalSpawn : BaseSpawn
    {
        public float DirRotation;

        public void CreateMatrix()
        {
            this.Matrix = Matrix.Translation(this.X, this.Y, this.Z) * Matrix.RotationYawPitchRoll(0, 0, DirRotation);
        }
    }

    public class YawPitchRollSpawn : BaseSpawn
    {
        public float Yaw;
        public float Pitch;
        public float Roll;

        public void CreateMatrix()
        {
            this.Matrix = Matrix.Translation(this.X, this.Y, this.Z) * Matrix.RotationYawPitchRoll(Yaw, Pitch, Roll);
        }
    }

    public class BoundingBoxSpawn : BaseSpawn
    {
        public float Width;
        public float Height;
        public float Depth;

        public void CreateMatrix()
        {
            this.Matrix = Matrix.Translation(this.X, this.Y, this.Z);
        }
    }
}