using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Contrail
    /// </summary>
    public class Cont
    {
        #region Constants

        /// <summary>
        /// Size of the cont meta header.
        /// </summary>
        public const int HeaderSize = 240;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private ContLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v1.0 DiscoBob: Completed plugin.
        /// v0.8 Shalted: Added Xbox7887's known values.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public ContLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class ContLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public Flags1Bitmask32 Flags1 { get { return __flags1__; } set { __flags1__ = value; } }
            private Flags1Bitmask32 __flags1__;
            public class Flags1Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool FirstPointUnfaded { get { return __firstPointUnfaded__; } set { __firstPointUnfaded__ = value; } }
                private bool __firstPointUnfaded__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool LastPointUnfaded { get { return __lastPointUnfaded__; } set { __lastPointUnfaded__ = value; } }
                private bool __lastPointUnfaded__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool PointsStartPinnedToMedia { get { return __pointsStartPinnedToMedia__; } set { __pointsStartPinnedToMedia__ = value; } }
                private bool __pointsStartPinnedToMedia__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool PointsStartPinnedToGround { get { return __pointsStartPinnedToGround__; } set { __pointsStartPinnedToGround__ = value; } }
                private bool __pointsStartPinnedToGround__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool PointsAlwaysPinnedToMedia { get { return __pointsAlwaysPinnedToMedia__; } set { __pointsAlwaysPinnedToMedia__ = value; } }
                private bool __pointsAlwaysPinnedToMedia__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool PointsAlwaysPinnedToGround { get { return __pointsAlwaysPinnedToGround__; } set { __pointsAlwaysPinnedToGround__ = value; } }
                private bool __pointsAlwaysPinnedToGround__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool EdgeEffectsFadeSlowly { get { return __edgeEffectsFadeSlowly__; } set { __edgeEffectsFadeSlowly__ = value; } }
                private bool __edgeEffectsFadeSlowly__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    FirstPointUnfaded = ((bit & mask) == bit);
                    bit <<= 1;
                    LastPointUnfaded = ((bit & mask) == bit);
                    bit <<= 1;
                    PointsStartPinnedToMedia = ((bit & mask) == bit);
                    bit <<= 1;
                    PointsStartPinnedToGround = ((bit & mask) == bit);
                    bit <<= 1;
                    PointsAlwaysPinnedToMedia = ((bit & mask) == bit);
                    bit <<= 1;
                    PointsAlwaysPinnedToGround = ((bit & mask) == bit);
                    bit <<= 1;
                    EdgeEffectsFadeSlowly = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float PointGenerationRate { get { return __pointGenerationRate__; } set { __pointGenerationRate__ = value; } }
            private float __pointGenerationRate__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float MinPointVelocity { get { return __minPointVelocity__; } set { __minPointVelocity__ = value; } }
            private float __minPointVelocity__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float MaxPointVelocity { get { return __maxPointVelocity__; } set { __maxPointVelocity__ = value; } }
            private float __maxPointVelocity__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float PointVelocityConeAngle { get { return __pointVelocityConeAngle__; } set { __pointVelocityConeAngle__ = value; } }
            private float __pointVelocityConeAngle__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float InheritedVelocityFraction { get { return __inheritedVelocityFraction__; } set { __inheritedVelocityFraction__ = value; } }
            private float __inheritedVelocityFraction__;
            /// <summary>
            /// 7. Enum16 :: Offset: 0x18 (24)
            /// </summary>
            public RenderTypeEnum16 RenderType { get { return __renderType__; } set { __renderType__ = value; } }
            private RenderTypeEnum16 __renderType__;
            public class RenderTypeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool VerticalOrientation { get { return __verticalOrientation__; } set { __verticalOrientation__ = value; } }
                private bool __verticalOrientation__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool HorizontalOrientation { get { return __horizontalOrientation__; } set { __horizontalOrientation__ = value; } }
                private bool __horizontalOrientation__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool MediaMapped { get { return __mediaMapped__; } set { __mediaMapped__ = value; } }
                private bool __mediaMapped__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool GroundMapped { get { return __groundMapped__; } set { __groundMapped__ = value; } }
                private bool __groundMapped__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool ViewerFacing { get { return __viewerFacing__; } set { __viewerFacing__ = value; } }
                private bool __viewerFacing__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool DoubleMarkerLinked { get { return __doubleMarkerLinked__; } set { __doubleMarkerLinked__ = value; } }
                private bool __doubleMarkerLinked__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    VerticalOrientation = (cur == value);
                    cur++;
                    HorizontalOrientation = (cur == value);
                    cur++;
                    MediaMapped = (cur == value);
                    cur++;
                    GroundMapped = (cur == value);
                    cur++;
                    ViewerFacing = (cur == value);
                    cur++;
                    DoubleMarkerLinked = (cur == value);
                }
            }
            /// <summary>
            /// 8. ByteArray :: Offset: 0x1A (26)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 9. Float :: Offset: 0x1C (28)
            /// </summary>
            public float TextureRepeatsU { get { return __textureRepeatsU__; } set { __textureRepeatsU__ = value; } }
            private float __textureRepeatsU__;
            /// <summary>
            /// 10. Float :: Offset: 0x20 (32)
            /// </summary>
            public float TextureRepeatsV { get { return __textureRepeatsV__; } set { __textureRepeatsV__ = value; } }
            private float __textureRepeatsV__;
            /// <summary>
            /// 11. Float :: Offset: 0x24 (36)
            /// </summary>
            public float TextureAnimationU { get { return __textureAnimationU__; } set { __textureAnimationU__ = value; } }
            private float __textureAnimationU__;
            /// <summary>
            /// 12. Float :: Offset: 0x28 (40)
            /// </summary>
            public float TextureAnimationV { get { return __textureAnimationV__; } set { __textureAnimationV__ = value; } }
            private float __textureAnimationV__;
            /// <summary>
            /// 13. Float :: Offset: 0x2C (44)
            /// </summary>
            public float TextureAnimationRate { get { return __textureAnimationRate__; } set { __textureAnimationRate__ = value; } }
            private float __textureAnimationRate__;
            /// <summary>
            /// 14. Tag :: Offset: 0x30 (48)
            /// </summary>
            public TagRef ContrailBitmap { get { return __contrailBitmap__; } set { __contrailBitmap__ = value; } }
            private TagRef __contrailBitmap__;
            /// <summary>
            /// 15. Int16 :: Offset: 0x38 (56)
            /// </summary>
            public short FirstSequence { get { return __firstSequence__; } set { __firstSequence__ = value; } }
            private short __firstSequence__;
            /// <summary>
            /// 16. Int16 :: Offset: 0x3A (58)
            /// </summary>
            public short SequenceCount { get { return __sequenceCount__; } set { __sequenceCount__ = value; } }
            private short __sequenceCount__;
            /// <summary>
            /// 17. ByteArray :: Offset: 0x3C (60)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 18. Enum16 :: Offset: 0x64 (100)
            /// </summary>
            public FramebufferBlendFunctionEnum16 FramebufferBlendFunction { get { return __framebufferBlendFunction__; } set { __framebufferBlendFunction__ = value; } }
            private FramebufferBlendFunctionEnum16 __framebufferBlendFunction__;
            public class FramebufferBlendFunctionEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool AlphaBlend { get { return __alphaBlend__; } set { __alphaBlend__ = value; } }
                private bool __alphaBlend__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Multiply { get { return __multiply__; } set { __multiply__ = value; } }
                private bool __multiply__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool DoubleMultiply { get { return __doubleMultiply__; } set { __doubleMultiply__ = value; } }
                private bool __doubleMultiply__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Add { get { return __add__; } set { __add__ = value; } }
                private bool __add__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Subtract { get { return __subtract__; } set { __subtract__ = value; } }
                private bool __subtract__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool ComponentMin { get { return __componentMin__; } set { __componentMin__ = value; } }
                private bool __componentMin__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool ComponentMax { get { return __componentMax__; } set { __componentMax__ = value; } }
                private bool __componentMax__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool AlphaMultiplyAdd { get { return __alphaMultiplyAdd__; } set { __alphaMultiplyAdd__ = value; } }
                private bool __alphaMultiplyAdd__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    AlphaBlend = (cur == value);
                    cur++;
                    Multiply = (cur == value);
                    cur++;
                    DoubleMultiply = (cur == value);
                    cur++;
                    Add = (cur == value);
                    cur++;
                    Subtract = (cur == value);
                    cur++;
                    ComponentMin = (cur == value);
                    cur++;
                    ComponentMax = (cur == value);
                    cur++;
                    AlphaMultiplyAdd = (cur == value);
                }
            }
            /// <summary>
            /// 19. Enum16 :: Offset: 0x66 (102)
            /// </summary>
            public FramebufferFadeModeEnum16 FramebufferFadeMode { get { return __framebufferFadeMode__; } set { __framebufferFadeMode__ = value; } }
            private FramebufferFadeModeEnum16 __framebufferFadeMode__;
            public class FramebufferFadeModeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool None { get { return __none__; } set { __none__ = value; } }
                private bool __none__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool FadeWhenPerpendicular { get { return __fadeWhenPerpendicular__; } set { __fadeWhenPerpendicular__ = value; } }
                private bool __fadeWhenPerpendicular__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool FadeWhenParallel { get { return __fadeWhenParallel__; } set { __fadeWhenParallel__ = value; } }
                private bool __fadeWhenParallel__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool FadeAfterDuration { get { return __fadeAfterDuration__; } set { __fadeAfterDuration__ = value; } }
                private bool __fadeAfterDuration__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    None = (cur == value);
                    cur++;
                    FadeWhenPerpendicular = (cur == value);
                    cur++;
                    FadeWhenParallel = (cur == value);
                    cur++;
                    FadeAfterDuration = (cur == value);
                }
            }
            /// <summary>
            /// 20. Bitmask32 :: Offset: 0x68 (104)
            /// </summary>
            public MapFlagsBitmask32 MapFlags { get { return __mapFlags__; } set { __mapFlags__ = value; } }
            private MapFlagsBitmask32 __mapFlags__;
            public class MapFlagsBitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Unfiltered { get { return __unfiltered__; } set { __unfiltered__ = value; } }
                private bool __unfiltered__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    Unfiltered = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 21. ByteArray :: Offset: 0x6C (108)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;
            /// <summary>
            /// 22. Tag :: Offset: 0x88 (136)
            /// </summary>
            public TagRef SecondaryMap { get { return __secondaryMap__; } set { __secondaryMap__ = value; } }
            private TagRef __secondaryMap__;
            /// <summary>
            /// 23. ByteArray :: Offset: 0x90 (144)
            /// </summary>
            public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
            private byte[] __unused4__;
            /// <summary>
            /// 24. Enum16 :: Offset: 0xB4 (180)
            /// </summary>
            public AnchorEnum16 Anchor { get { return __anchor__; } set { __anchor__ = value; } }
            private AnchorEnum16 __anchor__;
            public class AnchorEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool WithPrimary { get { return __withPrimary__; } set { __withPrimary__ = value; } }
                private bool __withPrimary__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool WithScreenSpace { get { return __withScreenSpace__; } set { __withScreenSpace__ = value; } }
                private bool __withScreenSpace__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool ZSprite { get { return __zSprite__; } set { __zSprite__ = value; } }
                private bool __zSprite__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    WithPrimary = (cur == value);
                    cur++;
                    WithScreenSpace = (cur == value);
                    cur++;
                    ZSprite = (cur == value);
                }
            }
            /// <summary>
            /// 25. Bitmask16 :: Offset: 0xB6 (182)
            /// </summary>
            public Flags2Bitmask16 Flags2 { get { return __flags2__; } set { __flags2__ = value; } }
            private Flags2Bitmask16 __flags2__;
            public class Flags2Bitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Unfiltered { get { return __unfiltered__; } set { __unfiltered__ = value; } }
                private bool __unfiltered__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    Unfiltered = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 26. ByteArray :: Offset: 0xB8 (184)
            /// </summary>
            public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
            private byte[] __unused5__;
            /// <summary>
            /// 27. TagBlock :: Offset: 0xE8 (232)
            /// </summary>
            public _PointStatesTagBlock PointStates { get { return __pointStates__; } set { __pointStates__ = value; } }
            private _PointStatesTagBlock __pointStates__;
            public class _PointStatesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public ContLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private ContLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float DurationFrom { get { return __durationFrom__; } set { __durationFrom__ = value; } }
                private float __durationFrom__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float DurationTo { get { return __durationTo__; } set { __durationTo__ = value; } }
                private float __durationTo__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float TransitionFrom { get { return __transitionFrom__; } set { __transitionFrom__ = value; } }
                private float __transitionFrom__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float TransitionTo { get { return __transitionTo__; } set { __transitionTo__ = value; } }
                private float __transitionTo__;
                /// <summary>
                /// 5. Tag :: Offset: 0x10 (16)
                /// </summary>
                public TagRef ContrailSmokePhysics { get { return __contrailSmokePhysics__; } set { __contrailSmokePhysics__ = value; } }
                private TagRef __contrailSmokePhysics__;
                /// <summary>
                /// 6. Float :: Offset: 0x18 (24)
                /// </summary>
                public float TrailWidth { get { return __trailWidth__; } set { __trailWidth__ = value; } }
                private float __trailWidth__;
                /// <summary>
                /// 7. Float :: Offset: 0x1C (28)
                /// </summary>
                public float LowerAlpha { get { return __lowerAlpha__; } set { __lowerAlpha__ = value; } }
                private float __lowerAlpha__;
                /// <summary>
                /// 8. Float :: Offset: 0x20 (32)
                /// </summary>
                public float LowerRed { get { return __lowerRed__; } set { __lowerRed__ = value; } }
                private float __lowerRed__;
                /// <summary>
                /// 9. Float :: Offset: 0x24 (36)
                /// </summary>
                public float LowerGreen { get { return __lowerGreen__; } set { __lowerGreen__ = value; } }
                private float __lowerGreen__;
                /// <summary>
                /// 10. Float :: Offset: 0x28 (40)
                /// </summary>
                public float LowerBlue { get { return __lowerBlue__; } set { __lowerBlue__ = value; } }
                private float __lowerBlue__;
                /// <summary>
                /// 11. Float :: Offset: 0x2C (44)
                /// </summary>
                public float UpperAlpha { get { return __upperAlpha__; } set { __upperAlpha__ = value; } }
                private float __upperAlpha__;
                /// <summary>
                /// 12. Float :: Offset: 0x30 (48)
                /// </summary>
                public float UpperRed { get { return __upperRed__; } set { __upperRed__ = value; } }
                private float __upperRed__;
                /// <summary>
                /// 13. Float :: Offset: 0x34 (52)
                /// </summary>
                public float UpperGreen { get { return __upperGreen__; } set { __upperGreen__ = value; } }
                private float __upperGreen__;
                /// <summary>
                /// 14. Float :: Offset: 0x38 (56)
                /// </summary>
                public float UpperBlue { get { return __upperBlue__; } set { __upperBlue__ = value; } }
                private float __upperBlue__;
                /// <summary>
                /// 15. Bitmask32 :: Offset: 0x3C (60)
                /// </summary>
                public ScaleBitmask32 Scale { get { return __scale__; } set { __scale__ = value; } }
                private ScaleBitmask32 __scale__;
                public class ScaleBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool Duration { get { return __duration__; } set { __duration__ = value; } }
                    private bool __duration__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool DurationDelta { get { return __durationDelta__; } set { __durationDelta__ = value; } }
                    private bool __durationDelta__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool TransitionDuration { get { return __transitionDuration__; } set { __transitionDuration__ = value; } }
                    private bool __transitionDuration__;
                    /// <summary>
                    /// 4. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool TransitionDurationDelta { get { return __transitionDurationDelta__; } set { __transitionDurationDelta__ = value; } }
                    private bool __transitionDurationDelta__;
                    /// <summary>
                    /// 5. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool Width { get { return __width__; } set { __width__ = value; } }
                    private bool __width__;
                    /// <summary>
                    /// 6. Option :: Value: 0x5 (5)
                    /// </summary>
                    public bool Color { get { return __color__; } set { __color__ = value; } }
                    private bool __color__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        Duration = ((bit & mask) == bit);
                        bit <<= 1;
                        DurationDelta = ((bit & mask) == bit);
                        bit <<= 1;
                        TransitionDuration = ((bit & mask) == bit);
                        bit <<= 1;
                        TransitionDurationDelta = ((bit & mask) == bit);
                        bit <<= 1;
                        Width = ((bit & mask) == bit);
                        bit <<= 1;
                        Color = ((bit & mask) == bit);
                    }
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PointStatesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PointStatesTagBlock(ContLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Scale = new ScaleBitmask32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PointStatesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PointStatesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the ContLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public ContLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags1 = new Flags1Bitmask32();
                RenderType = new RenderTypeEnum16();
                FramebufferBlendFunction = new FramebufferBlendFunctionEnum16();
                FramebufferFadeMode = new FramebufferFadeModeEnum16();
                MapFlags = new MapFlagsBitmask32();
                Anchor = new AnchorEnum16();
                Flags2 = new Flags2Bitmask16();
                PointStates = new _PointStatesTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags1._Initialize(br);
                PointGenerationRate = br.ReadSingle();
                MinPointVelocity = br.ReadSingle();
                MaxPointVelocity = br.ReadSingle();
                PointVelocityConeAngle = br.ReadSingle();
                InheritedVelocityFraction = br.ReadSingle();
                RenderType._Initialize(br);
                Unused1 = br.ReadBytes(2);
                TextureRepeatsU = br.ReadSingle();
                TextureRepeatsV = br.ReadSingle();
                TextureAnimationU = br.ReadSingle();
                TextureAnimationV = br.ReadSingle();
                TextureAnimationRate = br.ReadSingle();
                ContrailBitmap._Initialize(br);
                FirstSequence = br.ReadInt16();
                SequenceCount = br.ReadInt16();
                Unused2 = br.ReadBytes(40);
                FramebufferBlendFunction._Initialize(br);
                FramebufferFadeMode._Initialize(br);
                MapFlags._Initialize(br);
                Unused3 = br.ReadBytes(28);
                SecondaryMap._Initialize(br);
                Unused4 = br.ReadBytes(36);
                Anchor._Initialize(br);
                Flags2._Initialize(br);
                Unused5 = br.ReadBytes(48);
                PointStates._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Cont class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Cont(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new ContLayout(map, identity);
        }

        #region Specific Methods of cont

        #endregion
    }
}
