using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Decal
    /// </summary>
    public class Deca
    {
        #region Constants

        /// <summary>
        /// Size of the deca meta header.
        /// </summary>
        public const int HeaderSize = 172;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private DecaLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v1.0 Shalted: Completed Tag.
        /// v0.8 shade45: Named some unknowns...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public DecaLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class DecaLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
            private FlagsBitmask32 __flags__;
            public class FlagsBitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool GeometryInheritedByNextDecal { get { return __geometryInheritedByNextDecal__; } set { __geometryInheritedByNextDecal__ = value; } }
                private bool __geometryInheritedByNextDecal__;
                /// <summary>
                /// 2. Option :: Value: 0x2 (2)
                /// </summary>
                public bool InterpolateColour { get { return __interpolateColour__; } set { __interpolateColour__ = value; } }
                private bool __interpolateColour__;
                /// <summary>
                /// 3. Option :: Value: 0x4 (4)
                /// </summary>
                public bool MoreColour { get { return __moreColour__; } set { __moreColour__ = value; } }
                private bool __moreColour__;
                /// <summary>
                /// 4. Option :: Value: 0x8 (8)
                /// </summary>
                public bool NoRandomRotation { get { return __noRandomRotation__; } set { __noRandomRotation__ = value; } }
                private bool __noRandomRotation__;
                /// <summary>
                /// 5. Option :: Value: 0x10 (16)
                /// </summary>
                public bool WaterEffect { get { return __waterEffect__; } set { __waterEffect__ = value; } }
                private bool __waterEffect__;
                /// <summary>
                /// 6. Option :: Value: 0x20 (32)
                /// </summary>
                public bool SpecialEFSnapToAxis { get { return __specialEFSnapToAxis__; } set { __specialEFSnapToAxis__ = value; } }
                private bool __specialEFSnapToAxis__;
                /// <summary>
                /// 7. Option :: Value: 0x40 (64)
                /// </summary>
                public bool SpecialEFIncrementalCounter { get { return __specialEFIncrementalCounter__; } set { __specialEFIncrementalCounter__ = value; } }
                private bool __specialEFIncrementalCounter__;
                /// <summary>
                /// 8. Option :: Value: 0x80 (128)
                /// </summary>
                public bool AnimationLoop { get { return __animationLoop__; } set { __animationLoop__ = value; } }
                private bool __animationLoop__;
                /// <summary>
                /// 9. Option :: Value: 0x100 (256)
                /// </summary>
                public bool PreserveAspect { get { return __preserveAspect__; } set { __preserveAspect__ = value; } }
                private bool __preserveAspect__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    GeometryInheritedByNextDecal = ((bit & mask) == bit);
                    bit <<= 1;
                    InterpolateColour = ((bit & mask) == bit);
                    bit <<= 1;
                    MoreColour = ((bit & mask) == bit);
                    bit <<= 1;
                    NoRandomRotation = ((bit & mask) == bit);
                    bit <<= 1;
                    WaterEffect = ((bit & mask) == bit);
                    bit <<= 1;
                    SpecialEFSnapToAxis = ((bit & mask) == bit);
                    bit <<= 1;
                    SpecialEFIncrementalCounter = ((bit & mask) == bit);
                    bit <<= 1;
                    AnimationLoop = ((bit & mask) == bit);
                    bit <<= 1;
                    PreserveAspect = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Enum16 :: Offset: 0x4 (4)
            /// </summary>
            public TypeEnum16 Type { get { return __type__; } set { __type__ = value; } }
            private TypeEnum16 __type__;
            public class TypeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Scratch { get { return __scratch__; } set { __scratch__ = value; } }
                private bool __scratch__;
                /// <summary>
                /// 2. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Splatter { get { return __splatter__; } set { __splatter__ = value; } }
                private bool __splatter__;
                /// <summary>
                /// 3. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Burn { get { return __burn__; } set { __burn__ = value; } }
                private bool __burn__;
                /// <summary>
                /// 4. Option :: Value: 0x4 (4)
                /// </summary>
                public bool PaintedSign { get { return __paintedSign__; } set { __paintedSign__ = value; } }
                private bool __paintedSign__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    Scratch = (cur == value);
                    cur++;
                    Splatter = (cur == value);
                    cur++;
                    Burn = (cur == value);
                    cur++;
                    PaintedSign = (cur == value);
                }
            }
            /// <summary>
            /// 3. Enum16 :: Offset: 0x6 (6)
            /// </summary>
            public LayerEnum16 Layer { get { return __layer__; } set { __layer__ = value; } }
            private LayerEnum16 __layer__;
            public class LayerEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Primary { get { return __primary__; } set { __primary__ = value; } }
                private bool __primary__;
                /// <summary>
                /// 2. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Secondary { get { return __secondary__; } set { __secondary__ = value; } }
                private bool __secondary__;
                /// <summary>
                /// 3. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Light { get { return __light__; } set { __light__ = value; } }
                private bool __light__;
                /// <summary>
                /// 4. Option :: Value: 0x4 (4)
                /// </summary>
                public bool AlphaTested { get { return __alphaTested__; } set { __alphaTested__ = value; } }
                private bool __alphaTested__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Water { get { return __water__; } set { __water__ = value; } }
                private bool __water__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    Primary = (cur == value);
                    cur++;
                    Secondary = (cur == value);
                    cur++;
                    Light = (cur == value);
                    cur++;
                    AlphaTested = (cur == value);
                    cur++;
                    Water = (cur == value);
                }
            }
            /// <summary>
            /// 4. Tag :: Offset: 0x8 (8)
            /// </summary>
            public TagRef NextDecalInChain { get { return __nextDecalInChain__; } set { __nextDecalInChain__ = value; } }
            private TagRef __nextDecalInChain__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float Radius { get { return __radius__; } set { __radius__ = value; } }
            private float __radius__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float PowerOfTo_WorldUnits { get { return __powerOfTo_WorldUnits__; } set { __powerOfTo_WorldUnits__ = value; } }
            private float __powerOfTo_WorldUnits__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float Intensity { get { return __intensity__; } set { __intensity__ = value; } }
            private float __intensity__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float ColorLowerBoundR { get { return __colorLowerBoundR__; } set { __colorLowerBoundR__ = value; } }
            private float __colorLowerBoundR__;
            /// <summary>
            /// 9. Float :: Offset: 0x20 (32)
            /// </summary>
            public float ColorLowerBoundG { get { return __colorLowerBoundG__; } set { __colorLowerBoundG__ = value; } }
            private float __colorLowerBoundG__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float ColorLowerBoundB { get { return __colorLowerBoundB__; } set { __colorLowerBoundB__ = value; } }
            private float __colorLowerBoundB__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float ColorUpperBoundR { get { return __colorUpperBoundR__; } set { __colorUpperBoundR__ = value; } }
            private float __colorUpperBoundR__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float ColorUpperBoundG { get { return __colorUpperBoundG__; } set { __colorUpperBoundG__ = value; } }
            private float __colorUpperBoundG__;
            /// <summary>
            /// 13. Float :: Offset: 0x30 (48)
            /// </summary>
            public float ColorUpperBoundB { get { return __colorUpperBoundB__; } set { __colorUpperBoundB__ = value; } }
            private float __colorUpperBoundB__;
            /// <summary>
            /// 14. Float :: Offset: 0x34 (52)
            /// </summary>
            public float Lifetime { get { return __lifetime__; } set { __lifetime__ = value; } }
            private float __lifetime__;
            /// <summary>
            /// 15. Float :: Offset: 0x38 (56)
            /// </summary>
            public float PowerOfTo_Seconds1 { get { return __powerOfTo_Seconds1__; } set { __powerOfTo_Seconds1__ = value; } }
            private float __powerOfTo_Seconds1__;
            /// <summary>
            /// 16. Float :: Offset: 0x3C (60)
            /// </summary>
            public float DecayTime { get { return __decayTime__; } set { __decayTime__ = value; } }
            private float __decayTime__;
            /// <summary>
            /// 17. Float :: Offset: 0x40 (64)
            /// </summary>
            public float PowerOfTo_Seconds2 { get { return __powerOfTo_Seconds2__; } set { __powerOfTo_Seconds2__ = value; } }
            private float __powerOfTo_Seconds2__;
            /// <summary>
            /// 18. ByteArray :: Offset: 0x44 (68)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 19. Tag :: Offset: 0x88 (136)
            /// </summary>
            public TagRef Bitmap { get { return __bitmap__; } set { __bitmap__ = value; } }
            private TagRef __bitmap__;
            /// <summary>
            /// 20. ByteArray :: Offset: 0x90 (144)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 21. Float :: Offset: 0xA4 (164)
            /// </summary>
            public float MaximumSpriteExtent_pixels { get { return __maximumSpriteExtent_pixels__; } set { __maximumSpriteExtent_pixels__ = value; } }
            private float __maximumSpriteExtent_pixels__;
            /// <summary>
            /// 22. ByteArray :: Offset: 0xA8 (168)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the DecaLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public DecaLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags = new FlagsBitmask32();
                Type = new TypeEnum16();
                Layer = new LayerEnum16();
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags._Initialize(br);
                Type._Initialize(br);
                Layer._Initialize(br);
                NextDecalInChain._Initialize(br);
                Radius = br.ReadSingle();
                PowerOfTo_WorldUnits = br.ReadSingle();
                Intensity = br.ReadSingle();
                ColorLowerBoundR = br.ReadSingle();
                ColorLowerBoundG = br.ReadSingle();
                ColorLowerBoundB = br.ReadSingle();
                ColorUpperBoundR = br.ReadSingle();
                ColorUpperBoundG = br.ReadSingle();
                ColorUpperBoundB = br.ReadSingle();
                Lifetime = br.ReadSingle();
                PowerOfTo_Seconds1 = br.ReadSingle();
                DecayTime = br.ReadSingle();
                PowerOfTo_Seconds2 = br.ReadSingle();
                Unused1 = br.ReadBytes(68);
                Bitmap._Initialize(br);
                Unused2 = br.ReadBytes(20);
                MaximumSpriteExtent_pixels = br.ReadSingle();
                Unused3 = br.ReadBytes(4);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Deca class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Deca(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new DecaLayout(map, identity);
        }

        #region Specific Methods of deca

        #endregion
    }
}
