using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Cluster Fog
    /// </summary>
    public class Fog
    {
        #region Constants

        /// <summary>
        /// Size of the fog meta header.
        /// </summary>
        public const int HeaderSize = 96;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private FogLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.7 Shalted: Mapped most of the tag.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public FogLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class FogLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public Flags1Bitmask32 Flags1 { get { return __flags1__; } set { __flags1__ = value; } }
            private Flags1Bitmask32 __flags1__;
            public class Flags1Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool IsWaterPossibly { get { return __isWaterPossibly__; } set { __isWaterPossibly__ = value; } }
                private bool __isWaterPossibly__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool AtmosphereDominantPossibly { get { return __atmosphereDominantPossibly__; } set { __atmosphereDominantPossibly__ = value; } }
                private bool __atmosphereDominantPossibly__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool FogScreenOnlyPossibly { get { return __fogScreenOnlyPossibly__; } set { __fogScreenOnlyPossibly__ = value; } }
                private bool __fogScreenOnlyPossibly__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    IsWaterPossibly = ((bit & mask) == bit);
                    bit <<= 1;
                    AtmosphereDominantPossibly = ((bit & mask) == bit);
                    bit <<= 1;
                    FogScreenOnlyPossibly = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. StringID :: Offset: 0x4 (4)
            /// </summary>
            public StringID FogName { get { return __fogName__; } set { __fogName__ = value; } }
            private StringID __fogName__;
            /// <summary>
            /// 3. Bitmask32 :: Offset: 0x8 (8)
            /// </summary>
            public Flags2Bitmask32 Flags2 { get { return __flags2__; } set { __flags2__ = value; } }
            private Flags2Bitmask32 __flags2__;
            public class Flags2Bitmask32
            {

                public void _Initialize(BitReader br)
                {
                    // Empty..
                    br.BaseStream.Position += 4;
                }
            }
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float MaximumDensity { get { return __maximumDensity__; } set { __maximumDensity__ = value; } }
            private float __maximumDensity__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float OpaqueDistance { get { return __opaqueDistance__; } set { __opaqueDistance__ = value; } }
            private float __opaqueDistance__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float OpaqueDepth { get { return __opaqueDepth__; } set { __opaqueDepth__ = value; } }
            private float __opaqueDepth__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float DistanceToWaterPlane { get { return __distanceToWaterPlane__; } set { __distanceToWaterPlane__ = value; } }
            private float __distanceToWaterPlane__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float BrightnessPossibly { get { return __brightnessPossibly__; } set { __brightnessPossibly__ = value; } }
            private float __brightnessPossibly__;
            /// <summary>
            /// 9. ByteArray :: Offset: 0x20 (32)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float PrimaryRed { get { return __primaryRed__; } set { __primaryRed__ = value; } }
            private float __primaryRed__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float PrimaryGreen { get { return __primaryGreen__; } set { __primaryGreen__ = value; } }
            private float __primaryGreen__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float PrimaryBlue { get { return __primaryBlue__; } set { __primaryBlue__ = value; } }
            private float __primaryBlue__;
            /// <summary>
            /// 13. TagBlock :: Offset: 0x30 (48)
            /// </summary>
            public _FogPatchesTagBlock FogPatches { get { return __fogPatches__; } set { __fogPatches__ = value; } }
            private _FogPatchesTagBlock __fogPatches__;
            public class _FogPatchesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public FogLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private FogLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Red { get { return __red__; } set { __red__ = value; } }
                private float __red__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Green { get { return __green__; } set { __green__ = value; } }
                private float __green__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Blue { get { return __blue__; } set { __blue__ = value; } }
                private float __blue__;
                /// <summary>
                /// 4. ByteArray :: Offset: 0xC (12)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 5. Float :: Offset: 0x18 (24)
                /// </summary>
                public float MaximumDensity { get { return __maximumDensity__; } set { __maximumDensity__ = value; } }
                private float __maximumDensity__;
                /// <summary>
                /// 6. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Opacity { get { return __opacity__; } set { __opacity__ = value; } }
                private float __opacity__;
                /// <summary>
                /// 7. Float :: Offset: 0x20 (32)
                /// </summary>
                public float OpaqueDepth { get { return __opaqueDepth__; } set { __opaqueDepth__ = value; } }
                private float __opaqueDepth__;
                /// <summary>
                /// 8. Float :: Offset: 0x24 (36)
                /// </summary>
                public float OpaqueDistance { get { return __opaqueDistance__; } set { __opaqueDistance__ = value; } }
                private float __opaqueDistance__;
                /// <summary>
                /// 9. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private float __unknown__;
                /// <summary>
                /// 10. Tag :: Offset: 0x2C (44)
                /// </summary>
                public TagRef FogPatch { get { return __fogPatch__; } set { __fogPatch__ = value; } }
                private TagRef __fogPatch__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FogPatchesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FogPatchesTagBlock(FogLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FogPatchesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FogPatchesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 14. Tag :: Offset: 0x38 (56)
            /// </summary>
            public TagRef Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private TagRef __unused2__;
            /// <summary>
            /// 15. Tag :: Offset: 0x40 (64)
            /// </summary>
            public TagRef Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private TagRef __unused3__;
            /// <summary>
            /// 16. Float :: Offset: 0x48 (72)
            /// </summary>
            public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private float __unknown1__;
            /// <summary>
            /// 17. Float :: Offset: 0x4C (76)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 18. Tag :: Offset: 0x50 (80)
            /// </summary>
            public TagRef Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
            private TagRef __unused4__;
            /// <summary>
            /// 19. Tag :: Offset: 0x58 (88)
            /// </summary>
            public TagRef Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
            private TagRef __unused5__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the FogLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public FogLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags1 = new Flags1Bitmask32();
                Flags2 = new Flags2Bitmask32();
                FogPatches = new _FogPatchesTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags1._Initialize(br);
                FogName._Initialize(br);
                Flags2._Initialize(br);
                MaximumDensity = br.ReadSingle();
                OpaqueDistance = br.ReadSingle();
                OpaqueDepth = br.ReadSingle();
                DistanceToWaterPlane = br.ReadSingle();
                BrightnessPossibly = br.ReadSingle();
                Unused1 = br.ReadBytes(4);
                PrimaryRed = br.ReadSingle();
                PrimaryGreen = br.ReadSingle();
                PrimaryBlue = br.ReadSingle();
                FogPatches._Initialize(br);
                Unused2._Initialize(br);
                Unused3._Initialize(br);
                Unknown1 = br.ReadSingle();
                Unknown2 = br.ReadSingle();
                Unused4._Initialize(br);
                Unused5._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Fog class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Fog(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new FogLayout(map, identity);
        }

        #region Specific Methods of fog

        #endregion
    }
}
