using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Material Effect
    /// </summary>
    public class Foot
    {
        #region Constants

        /// <summary>
        /// Size of the foot meta header.
        /// </summary>
        public const int HeaderSize = 8;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private FootLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v1 DiscoBob: Completed Plugin.
        /// v0.2 DiscoBob: Named most of the unknowns.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public FootLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class FootLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. TagBlock :: Offset: 0x0 (0)
            /// </summary>
            public _ImpactOptionsTagBlock ImpactOptions { get { return __impactOptions__; } set { __impactOptions__ = value; } }
            private _ImpactOptionsTagBlock __impactOptions__;
            public class _ImpactOptionsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public FootLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private FootLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. TagBlock :: Offset: 0x8 (8)
                /// </summary>
                public _ImpactSoundsTagBlock ImpactSounds { get { return __impactSounds__; } set { __impactSounds__ = value; } }
                private _ImpactSoundsTagBlock __impactSounds__;
                public class _ImpactSoundsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public FootLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private FootLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef ImpactSound { get { return __impactSound__; } set { __impactSound__ = value; } }
                    private TagRef __impactSound__;
                    /// <summary>
                    /// 2. ByteArray :: Offset: 0x8 (8)
                    /// </summary>
                    public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                    private byte[] __unused__;
                    /// <summary>
                    /// 3. StringID :: Offset: 0x10 (16)
                    /// </summary>
                    public StringID MaterialIdentifier { get { return __materialIdentifier__; } set { __materialIdentifier__ = value; } }
                    private StringID __materialIdentifier__;
                    /// <summary>
                    /// 4. Enum16 :: Offset: 0x14 (20)
                    /// </summary>
                    public MaterialIdentityEnum16 MaterialIdentity { get { return __materialIdentity__; } set { __materialIdentity__ = value; } }
                    private MaterialIdentityEnum16 __materialIdentity__;
                    public class MaterialIdentityEnum16
                    {
                        /// <summary>
                        /// 1. Option :: Value: 0xFFFFFFFF (-1)
                        /// </summary>
                        public bool NoMaterial { get { return __noMaterial__; } set { __noMaterial__ = value; } }
                        private bool __noMaterial__;
                        /// <summary>
                        /// 2. Option :: Value: 0x0 (0)
                        /// </summary>
                        public bool Default_material { get { return __default_material__; } set { __default_material__ = value; } }
                        private bool __default_material__;
                        /// <summary>
                        /// 3. Option :: Value: 0x1 (1)
                        /// </summary>
                        public bool Liquid { get { return __liquid__; } set { __liquid__ = value; } }
                        private bool __liquid__;
                        /// <summary>
                        /// 4. Option :: Value: 0x2 (2)
                        /// </summary>
                        public bool Liquid_thin { get { return __liquid_thin__; } set { __liquid_thin__ = value; } }
                        private bool __liquid_thin__;
                        /// <summary>
                        /// 5. Option :: Value: 0x6 (6)
                        /// </summary>
                        public bool Soft { get { return __soft__; } set { __soft__ = value; } }
                        private bool __soft__;
                        /// <summary>
                        /// 6. Option :: Value: 0x7 (7)
                        /// </summary>
                        public bool Soft_organic { get { return __soft_organic__; } set { __soft_organic__ = value; } }
                        private bool __soft_organic__;
                        /// <summary>
                        /// 7. Option :: Value: 0x8 (8)
                        /// </summary>
                        public bool Soft_organic_flesh { get { return __soft_organic_flesh__; } set { __soft_organic_flesh__ = value; } }
                        private bool __soft_organic_flesh__;
                        /// <summary>
                        /// 8. Option :: Value: 0x9 (9)
                        /// </summary>
                        public bool Soft_organic_flesh_human { get { return __soft_organic_flesh_human__; } set { __soft_organic_flesh_human__ = value; } }
                        private bool __soft_organic_flesh_human__;
                        /// <summary>
                        /// 9. Option :: Value: 0xA (10)
                        /// </summary>
                        public bool Soft_organic_flesh_elite { get { return __soft_organic_flesh_elite__; } set { __soft_organic_flesh_elite__ = value; } }
                        private bool __soft_organic_flesh_elite__;
                        /// <summary>
                        /// 10. Option :: Value: 0xB (11)
                        /// </summary>
                        public bool Soft_organic_flesh_grunt { get { return __soft_organic_flesh_grunt__; } set { __soft_organic_flesh_grunt__ = value; } }
                        private bool __soft_organic_flesh_grunt__;
                        /// <summary>
                        /// 11. Option :: Value: 0xC (12)
                        /// </summary>
                        public bool Soft_organic_flesh_jackal { get { return __soft_organic_flesh_jackal__; } set { __soft_organic_flesh_jackal__ = value; } }
                        private bool __soft_organic_flesh_jackal__;
                        /// <summary>
                        /// 12. Option :: Value: 0xD (13)
                        /// </summary>
                        public bool Soft_organic_flesh_hunter { get { return __soft_organic_flesh_hunter__; } set { __soft_organic_flesh_hunter__ = value; } }
                        private bool __soft_organic_flesh_hunter__;
                        /// <summary>
                        /// 13. Option :: Value: 0xE (14)
                        /// </summary>
                        public bool Soft_organic_flesh_prophet { get { return __soft_organic_flesh_prophet__; } set { __soft_organic_flesh_prophet__ = value; } }
                        private bool __soft_organic_flesh_prophet__;
                        /// <summary>
                        /// 14. Option :: Value: 0xF (15)
                        /// </summary>
                        public bool Soft_organic_flesh_bugger { get { return __soft_organic_flesh_bugger__; } set { __soft_organic_flesh_bugger__ = value; } }
                        private bool __soft_organic_flesh_bugger__;
                        /// <summary>
                        /// 15. Option :: Value: 0x11 (17)
                        /// </summary>
                        public bool Soft_organic_plant { get { return __soft_organic_plant__; } set { __soft_organic_plant__ = value; } }
                        private bool __soft_organic_plant__;
                        /// <summary>
                        /// 16. Option :: Value: 0x12 (18)
                        /// </summary>
                        public bool Soft_organic_plant_leafy { get { return __soft_organic_plant_leafy__; } set { __soft_organic_plant_leafy__ = value; } }
                        private bool __soft_organic_plant_leafy__;
                        /// <summary>
                        /// 17. Option :: Value: 0x14 (20)
                        /// </summary>
                        public bool Soft_organic_plant_grassy { get { return __soft_organic_plant_grassy__; } set { __soft_organic_plant_grassy__ = value; } }
                        private bool __soft_organic_plant_grassy__;
                        /// <summary>
                        /// 18. Option :: Value: 0x15 (21)
                        /// </summary>
                        public bool Soft_inorganic { get { return __soft_inorganic__; } set { __soft_inorganic__ = value; } }
                        private bool __soft_inorganic__;
                        /// <summary>
                        /// 19. Option :: Value: 0x16 (22)
                        /// </summary>
                        public bool Soft_inorganic_vinyl { get { return __soft_inorganic_vinyl__; } set { __soft_inorganic_vinyl__ = value; } }
                        private bool __soft_inorganic_vinyl__;
                        /// <summary>
                        /// 20. Option :: Value: 0x19 (25)
                        /// </summary>
                        public bool Soft_terrain { get { return __soft_terrain__; } set { __soft_terrain__ = value; } }
                        private bool __soft_terrain__;
                        /// <summary>
                        /// 21. Option :: Value: 0x1A (26)
                        /// </summary>
                        public bool Soft_terrain_snow { get { return __soft_terrain_snow__; } set { __soft_terrain_snow__ = value; } }
                        private bool __soft_terrain_snow__;
                        /// <summary>
                        /// 22. Option :: Value: 0x1B (27)
                        /// </summary>
                        public bool Tough { get { return __tough__; } set { __tough__ = value; } }
                        private bool __tough__;
                        /// <summary>
                        /// 23. Option :: Value: 0x1C (28)
                        /// </summary>
                        public bool Tough_organic { get { return __tough_organic__; } set { __tough_organic__ = value; } }
                        private bool __tough_organic__;
                        /// <summary>
                        /// 24. Option :: Value: 0x1D (29)
                        /// </summary>
                        public bool Tough_organic_flesh { get { return __tough_organic_flesh__; } set { __tough_organic_flesh__ = value; } }
                        private bool __tough_organic_flesh__;
                        /// <summary>
                        /// 25. Option :: Value: 0x1E (30)
                        /// </summary>
                        public bool Tough_organic_flesh_bugger { get { return __tough_organic_flesh_bugger__; } set { __tough_organic_flesh_bugger__ = value; } }
                        private bool __tough_organic_flesh_bugger__;
                        /// <summary>
                        /// 26. Option :: Value: 0x1F (31)
                        /// </summary>
                        public bool Tough_organic_flesh_brute { get { return __tough_organic_flesh_brute__; } set { __tough_organic_flesh_brute__ = value; } }
                        private bool __tough_organic_flesh_brute__;
                        /// <summary>
                        /// 27. Option :: Value: 0x21 (33)
                        /// </summary>
                        public bool Tough_inorganic { get { return __tough_inorganic__; } set { __tough_inorganic__ = value; } }
                        private bool __tough_inorganic__;
                        /// <summary>
                        /// 28. Option :: Value: 0x22 (34)
                        /// </summary>
                        public bool Tough_inorganic_armor { get { return __tough_inorganic_armor__; } set { __tough_inorganic_armor__ = value; } }
                        private bool __tough_inorganic_armor__;
                        /// <summary>
                        /// 29. Option :: Value: 0x24 (36)
                        /// </summary>
                        public bool Tough_inorganic_rubber { get { return __tough_inorganic_rubber__; } set { __tough_inorganic_rubber__ = value; } }
                        private bool __tough_inorganic_rubber__;
                        /// <summary>
                        /// 30. Option :: Value: 0x27 (39)
                        /// </summary>
                        public bool Tough_inorganic_plastic { get { return __tough_inorganic_plastic__; } set { __tough_inorganic_plastic__ = value; } }
                        private bool __tough_inorganic_plastic__;
                        /// <summary>
                        /// 31. Option :: Value: 0x2A (42)
                        /// </summary>
                        public bool Tough_floodflesh { get { return __tough_floodflesh__; } set { __tough_floodflesh__ = value; } }
                        private bool __tough_floodflesh__;
                        /// <summary>
                        /// 32. Option :: Value: 0x2C (44)
                        /// </summary>
                        public bool Tough_floodflesh_carrierform { get { return __tough_floodflesh_carrierform__; } set { __tough_floodflesh_carrierform__ = value; } }
                        private bool __tough_floodflesh_carrierform__;
                        /// <summary>
                        /// 33. Option :: Value: 0x2D (45)
                        /// </summary>
                        public bool Tough_floodflesh_infectionform { get { return __tough_floodflesh_infectionform__; } set { __tough_floodflesh_infectionform__ = value; } }
                        private bool __tough_floodflesh_infectionform__;
                        /// <summary>
                        /// 34. Option :: Value: 0x2E (46)
                        /// </summary>
                        public bool Tough_organic_wood { get { return __tough_organic_wood__; } set { __tough_organic_wood__ = value; } }
                        private bool __tough_organic_wood__;
                        /// <summary>
                        /// 35. Option :: Value: 0x2F (47)
                        /// </summary>
                        public bool Tough_terrain { get { return __tough_terrain__; } set { __tough_terrain__ = value; } }
                        private bool __tough_terrain__;
                        /// <summary>
                        /// 36. Option :: Value: 0x30 (48)
                        /// </summary>
                        public bool Tough_terrain_dirt { get { return __tough_terrain_dirt__; } set { __tough_terrain_dirt__ = value; } }
                        private bool __tough_terrain_dirt__;
                        /// <summary>
                        /// 37. Option :: Value: 0x31 (49)
                        /// </summary>
                        public bool Tough_terrain_grass { get { return __tough_terrain_grass__; } set { __tough_terrain_grass__ = value; } }
                        private bool __tough_terrain_grass__;
                        /// <summary>
                        /// 38. Option :: Value: 0x32 (50)
                        /// </summary>
                        public bool Tough_terrain_mud { get { return __tough_terrain_mud__; } set { __tough_terrain_mud__ = value; } }
                        private bool __tough_terrain_mud__;
                        /// <summary>
                        /// 39. Option :: Value: 0x33 (51)
                        /// </summary>
                        public bool Tough_terrain_gravel { get { return __tough_terrain_gravel__; } set { __tough_terrain_gravel__ = value; } }
                        private bool __tough_terrain_gravel__;
                        /// <summary>
                        /// 40. Option :: Value: 0x34 (52)
                        /// </summary>
                        public bool Tough_terrain_sand { get { return __tough_terrain_sand__; } set { __tough_terrain_sand__ = value; } }
                        private bool __tough_terrain_sand__;
                        /// <summary>
                        /// 41. Option :: Value: 0x35 (53)
                        /// </summary>
                        public bool Hard { get { return __hard__; } set { __hard__ = value; } }
                        private bool __hard__;
                        /// <summary>
                        /// 42. Option :: Value: 0x36 (54)
                        /// </summary>
                        public bool Hard_metal_thin { get { return __hard_metal_thin__; } set { __hard_metal_thin__ = value; } }
                        private bool __hard_metal_thin__;
                        /// <summary>
                        /// 43. Option :: Value: 0x37 (55)
                        /// </summary>
                        public bool Hard_metal_thin_hum { get { return __hard_metal_thin_hum__; } set { __hard_metal_thin_hum__ = value; } }
                        private bool __hard_metal_thin_hum__;
                        /// <summary>
                        /// 44. Option :: Value: 0x38 (56)
                        /// </summary>
                        public bool Hard_metal_thin_hum_masterchief { get { return __hard_metal_thin_hum_masterchief__; } set { __hard_metal_thin_hum_masterchief__ = value; } }
                        private bool __hard_metal_thin_hum_masterchief__;
                        /// <summary>
                        /// 45. Option :: Value: 0x3D (61)
                        /// </summary>
                        public bool Hard_metal_thin_cov_grunt { get { return __hard_metal_thin_cov_grunt__; } set { __hard_metal_thin_cov_grunt__ = value; } }
                        private bool __hard_metal_thin_cov_grunt__;
                        /// <summary>
                        /// 46. Option :: Value: 0x3E (62)
                        /// </summary>
                        public bool Hard_metal_thin_cov_hereticleader { get { return __hard_metal_thin_cov_hereticleader__; } set { __hard_metal_thin_cov_hereticleader__ = value; } }
                        private bool __hard_metal_thin_cov_hereticleader__;
                        /// <summary>
                        /// 47. Option :: Value: 0x3F (63)
                        /// </summary>
                        public bool Hard_metal_thin_cov_elite { get { return __hard_metal_thin_cov_elite__; } set { __hard_metal_thin_cov_elite__ = value; } }
                        private bool __hard_metal_thin_cov_elite__;
                        /// <summary>
                        /// 48. Option :: Value: 0x43 (67)
                        /// </summary>
                        public bool Hard_metal_thin_for_sentinel { get { return __hard_metal_thin_for_sentinel__; } set { __hard_metal_thin_for_sentinel__ = value; } }
                        private bool __hard_metal_thin_for_sentinel__;
                        /// <summary>
                        /// 49. Option :: Value: 0x44 (68)
                        /// </summary>
                        public bool Hard_metal_thin_for_monitor { get { return __hard_metal_thin_for_monitor__; } set { __hard_metal_thin_for_monitor__ = value; } }
                        private bool __hard_metal_thin_for_monitor__;
                        /// <summary>
                        /// 50. Option :: Value: 0x45 (69)
                        /// </summary>
                        public bool Hard_metal_thick { get { return __hard_metal_thick__; } set { __hard_metal_thick__ = value; } }
                        private bool __hard_metal_thick__;
                        /// <summary>
                        /// 51. Option :: Value: 0x46 (70)
                        /// </summary>
                        public bool Hard_metal_thick_hum { get { return __hard_metal_thick_hum__; } set { __hard_metal_thick_hum__ = value; } }
                        private bool __hard_metal_thick_hum__;
                        /// <summary>
                        /// 52. Option :: Value: 0x48 (72)
                        /// </summary>
                        public bool Hard_metal_thick_hum_warthog { get { return __hard_metal_thick_hum_warthog__; } set { __hard_metal_thick_hum_warthog__ = value; } }
                        private bool __hard_metal_thick_hum_warthog__;
                        /// <summary>
                        /// 53. Option :: Value: 0x4B (75)
                        /// </summary>
                        public bool Hard_metal_thick_hum_chassis { get { return __hard_metal_thick_hum_chassis__; } set { __hard_metal_thick_hum_chassis__ = value; } }
                        private bool __hard_metal_thick_hum_chassis__;
                        /// <summary>
                        /// 54. Option :: Value: 0x4C (76)
                        /// </summary>
                        public bool Hard_metal_thick_cov { get { return __hard_metal_thick_cov__; } set { __hard_metal_thick_cov__ = value; } }
                        private bool __hard_metal_thick_cov__;
                        /// <summary>
                        /// 55. Option :: Value: 0x4D (77)
                        /// </summary>
                        public bool Hard_metal_thick_cov_hunter { get { return __hard_metal_thick_cov_hunter__; } set { __hard_metal_thick_cov_hunter__ = value; } }
                        private bool __hard_metal_thick_cov_hunter__;
                        /// <summary>
                        /// 56. Option :: Value: 0x4E (78)
                        /// </summary>
                        public bool Hard_metal_thick_cov_ghost { get { return __hard_metal_thick_cov_ghost__; } set { __hard_metal_thick_cov_ghost__ = value; } }
                        private bool __hard_metal_thick_cov_ghost__;
                        /// <summary>
                        /// 57. Option :: Value: 0x4F (79)
                        /// </summary>
                        public bool Hard_metal_thick_cov_shadow { get { return __hard_metal_thick_cov_shadow__; } set { __hard_metal_thick_cov_shadow__ = value; } }
                        private bool __hard_metal_thick_cov_shadow__;
                        /// <summary>
                        /// 58. Option :: Value: 0x52 (82)
                        /// </summary>
                        public bool Hard_metal_thick_cov_banshee { get { return __hard_metal_thick_cov_banshee__; } set { __hard_metal_thick_cov_banshee__ = value; } }
                        private bool __hard_metal_thick_cov_banshee__;
                        /// <summary>
                        /// 59. Option :: Value: 0x53 (83)
                        /// </summary>
                        public bool Hard_metal_thick_cov_creep { get { return __hard_metal_thick_cov_creep__; } set { __hard_metal_thick_cov_creep__ = value; } }
                        private bool __hard_metal_thick_cov_creep__;
                        /// <summary>
                        /// 60. Option :: Value: 0x54 (84)
                        /// </summary>
                        public bool Hard_metal_thick_cov_wraith { get { return __hard_metal_thick_cov_wraith__; } set { __hard_metal_thick_cov_wraith__ = value; } }
                        private bool __hard_metal_thick_cov_wraith__;
                        /// <summary>
                        /// 61. Option :: Value: 0x59 (89)
                        /// </summary>
                        public bool Hard_metal_solid { get { return __hard_metal_solid__; } set { __hard_metal_solid__ = value; } }
                        private bool __hard_metal_solid__;
                        /// <summary>
                        /// 62. Option :: Value: 0x5B (91)
                        /// </summary>
                        public bool Hard_metal_solid_hum_scorpion { get { return __hard_metal_solid_hum_scorpion__; } set { __hard_metal_solid_hum_scorpion__ = value; } }
                        private bool __hard_metal_solid_hum_scorpion__;
                        /// <summary>
                        /// 63. Option :: Value: 0x5F (95)
                        /// </summary>
                        public bool Hard_metal_solid_cov_hunter { get { return __hard_metal_solid_cov_hunter__; } set { __hard_metal_solid_cov_hunter__ = value; } }
                        private bool __hard_metal_solid_cov_hunter__;
                        /// <summary>
                        /// 64. Option :: Value: 0x60 (96)
                        /// </summary>
                        public bool Hard_metal_solid_cov_wraith { get { return __hard_metal_solid_cov_wraith__; } set { __hard_metal_solid_cov_wraith__ = value; } }
                        private bool __hard_metal_solid_cov_wraith__;
                        /// <summary>
                        /// 65. Option :: Value: 0x61 (97)
                        /// </summary>
                        public bool Hard_metal_solid_cov_creep { get { return __hard_metal_solid_cov_creep__; } set { __hard_metal_solid_cov_creep__ = value; } }
                        private bool __hard_metal_solid_cov_creep__;
                        /// <summary>
                        /// 66. Option :: Value: 0x64 (100)
                        /// </summary>
                        public bool Hard_terrain { get { return __hard_terrain__; } set { __hard_terrain__ = value; } }
                        private bool __hard_terrain__;
                        /// <summary>
                        /// 67. Option :: Value: 0x65 (101)
                        /// </summary>
                        public bool Hard_terrain_ice { get { return __hard_terrain_ice__; } set { __hard_terrain_ice__ = value; } }
                        private bool __hard_terrain_ice__;
                        /// <summary>
                        /// 68. Option :: Value: 0x66 (102)
                        /// </summary>
                        public bool Hard_terrain_stone { get { return __hard_terrain_stone__; } set { __hard_terrain_stone__ = value; } }
                        private bool __hard_terrain_stone__;
                        /// <summary>
                        /// 69. Option :: Value: 0x67 (103)
                        /// </summary>
                        public bool Hard_terrain_stone_rough { get { return __hard_terrain_stone_rough__; } set { __hard_terrain_stone_rough__ = value; } }
                        private bool __hard_terrain_stone_rough__;
                        /// <summary>
                        /// 70. Option :: Value: 0x68 (104)
                        /// </summary>
                        public bool Hard_terrain_concrete { get { return __hard_terrain_concrete__; } set { __hard_terrain_concrete__ = value; } }
                        private bool __hard_terrain_concrete__;
                        /// <summary>
                        /// 71. Option :: Value: 0x69 (105)
                        /// </summary>
                        public bool Hard_floodflesh { get { return __hard_floodflesh__; } set { __hard_floodflesh__ = value; } }
                        private bool __hard_floodflesh__;
                        /// <summary>
                        /// 72. Option :: Value: 0x6A (106)
                        /// </summary>
                        public bool Brittle { get { return __brittle__; } set { __brittle__ = value; } }
                        private bool __brittle__;
                        /// <summary>
                        /// 73. Option :: Value: 0x6B (107)
                        /// </summary>
                        public bool Brittle_glass { get { return __brittle_glass__; } set { __brittle_glass__ = value; } }
                        private bool __brittle_glass__;
                        /// <summary>
                        /// 74. Option :: Value: 0x70 (112)
                        /// </summary>
                        public bool Brittle_glass_for_unbreakable { get { return __brittle_glass_for_unbreakable__; } set { __brittle_glass_for_unbreakable__ = value; } }
                        private bool __brittle_glass_for_unbreakable__;
                        /// <summary>
                        /// 75. Option :: Value: 0x73 (115)
                        /// </summary>
                        public bool Brittle_elec { get { return __brittle_elec__; } set { __brittle_elec__ = value; } }
                        private bool __brittle_elec__;
                        /// <summary>
                        /// 76. Option :: Value: 0x75 (117)
                        /// </summary>
                        public bool Brittle_elec_cov { get { return __brittle_elec_cov__; } set { __brittle_elec_cov__ = value; } }
                        private bool __brittle_elec_cov__;
                        /// <summary>
                        /// 77. Option :: Value: 0x7E (126)
                        /// </summary>
                        public bool Brittle_elec_for { get { return __brittle_elec_for__; } set { __brittle_elec_for__ = value; } }
                        private bool __brittle_elec_for__;
                        /// <summary>
                        /// 78. Option :: Value: 0x80 (128)
                        /// </summary>
                        public bool Brittle_mech { get { return __brittle_mech__; } set { __brittle_mech__ = value; } }
                        private bool __brittle_mech__;
                        /// <summary>
                        /// 79. Option :: Value: 0x87 (135)
                        /// </summary>
                        public bool Brittle_mech_cov { get { return __brittle_mech_cov__; } set { __brittle_mech_cov__ = value; } }
                        private bool __brittle_mech_cov__;
                        /// <summary>
                        /// 80. Option :: Value: 0x89 (137)
                        /// </summary>
                        public bool Brittle_explosive { get { return __brittle_explosive__; } set { __brittle_explosive__ = value; } }
                        private bool __brittle_explosive__;
                        /// <summary>
                        /// 81. Option :: Value: 0x90 (144)
                        /// </summary>
                        public bool Energy { get { return __energy__; } set { __energy__ = value; } }
                        private bool __energy__;
                        /// <summary>
                        /// 82. Option :: Value: 0x91 (145)
                        /// </summary>
                        public bool Energy_shield_thin { get { return __energy_shield_thin__; } set { __energy_shield_thin__ = value; } }
                        private bool __energy_shield_thin__;
                        /// <summary>
                        /// 83. Option :: Value: 0x92 (146)
                        /// </summary>
                        public bool Energy_shield_thin_hum { get { return __energy_shield_thin_hum__; } set { __energy_shield_thin_hum__ = value; } }
                        private bool __energy_shield_thin_hum__;
                        /// <summary>
                        /// 84. Option :: Value: 0x94 (148)
                        /// </summary>
                        public bool Energy_shield_thin_cov { get { return __energy_shield_thin_cov__; } set { __energy_shield_thin_cov__ = value; } }
                        private bool __energy_shield_thin_cov__;
                        /// <summary>
                        /// 85. Option :: Value: 0x95 (149)
                        /// </summary>
                        public bool Energy_shield_thin_cov_elite { get { return __energy_shield_thin_cov_elite__; } set { __energy_shield_thin_cov_elite__ = value; } }
                        private bool __energy_shield_thin_cov_elite__;
                        /// <summary>
                        /// 86. Option :: Value: 0x98 (152)
                        /// </summary>
                        public bool Energy_shield_thick { get { return __energy_shield_thick__; } set { __energy_shield_thick__ = value; } }
                        private bool __energy_shield_thick__;
                        /// <summary>
                        /// 87. Option :: Value: 0x9A (154)
                        /// </summary>
                        public bool Energy_shield_thick_cov { get { return __energy_shield_thick_cov__; } set { __energy_shield_thick_cov__ = value; } }
                        private bool __energy_shield_thick_cov__;
                        /// <summary>
                        /// 88. Option :: Value: 0x9B (155)
                        /// </summary>
                        public bool Energy_shield_thick_cov_jackal { get { return __energy_shield_thick_cov_jackal__; } set { __energy_shield_thick_cov_jackal__ = value; } }
                        private bool __energy_shield_thick_cov_jackal__;
                        /// <summary>
                        /// 89. Option :: Value: 0x9D (157)
                        /// </summary>
                        public bool Energy_holo { get { return __energy_holo__; } set { __energy_holo__ = value; } }
                        private bool __energy_holo__;
                        /// <summary>
                        /// 90. Option :: Value: 0xA1 (161)
                        /// </summary>
                        public bool Liquid_thin_water_puddle { get { return __liquid_thin_water_puddle__; } set { __liquid_thin_water_puddle__ = value; } }
                        private bool __liquid_thin_water_puddle__;
                        /// <summary>
                        /// 91. Option :: Value: 0xA2 (162)
                        /// </summary>
                        public bool Liquid_thin_water_shallow { get { return __liquid_thin_water_shallow__; } set { __liquid_thin_water_shallow__ = value; } }
                        private bool __liquid_thin_water_shallow__;
                        /// <summary>
                        /// 92. Option :: Value: 0xA7 (167)
                        /// </summary>
                        public bool Hard_metal_thin_hum_grating { get { return __hard_metal_thin_hum_grating__; } set { __hard_metal_thin_hum_grating__ = value; } }
                        private bool __hard_metal_thin_hum_grating__;
                        /// <summary>
                        /// 93. Option :: Value: 0xAB (171)
                        /// </summary>
                        public bool Tough_terrain_woodfloor { get { return __tough_terrain_woodfloor__; } set { __tough_terrain_woodfloor__ = value; } }
                        private bool __tough_terrain_woodfloor__;
                        /// <summary>
                        /// 94. Option :: Value: 0xB0 (176)
                        /// </summary>
                        public bool Tough_terrain_carpet { get { return __tough_terrain_carpet__; } set { __tough_terrain_carpet__ = value; } }
                        private bool __tough_terrain_carpet__;
                        /// <summary>
                        /// 95. Option :: Value: 0xB1 (177)
                        /// </summary>
                        public bool Hard_metal_thin_cov_brute { get { return __hard_metal_thin_cov_brute__; } set { __hard_metal_thin_cov_brute__ = value; } }
                        private bool __hard_metal_thin_cov_brute__;

                        public void _Initialize(BitReader br)
                        {
                            ushort value = br.ReadUInt16();
                            ushort cur = 0;

                            NoMaterial = (cur == value);
                            cur++;
                            Default_material = (cur == value);
                            cur++;
                            Liquid = (cur == value);
                            cur++;
                            Liquid_thin = (cur == value);
                            cur++;
                            Soft = (cur == value);
                            cur++;
                            Soft_organic = (cur == value);
                            cur++;
                            Soft_organic_flesh = (cur == value);
                            cur++;
                            Soft_organic_flesh_human = (cur == value);
                            cur++;
                            Soft_organic_flesh_elite = (cur == value);
                            cur++;
                            Soft_organic_flesh_grunt = (cur == value);
                            cur++;
                            Soft_organic_flesh_jackal = (cur == value);
                            cur++;
                            Soft_organic_flesh_hunter = (cur == value);
                            cur++;
                            Soft_organic_flesh_prophet = (cur == value);
                            cur++;
                            Soft_organic_flesh_bugger = (cur == value);
                            cur++;
                            Soft_organic_plant = (cur == value);
                            cur++;
                            Soft_organic_plant_leafy = (cur == value);
                            cur++;
                            Soft_organic_plant_grassy = (cur == value);
                            cur++;
                            Soft_inorganic = (cur == value);
                            cur++;
                            Soft_inorganic_vinyl = (cur == value);
                            cur++;
                            Soft_terrain = (cur == value);
                            cur++;
                            Soft_terrain_snow = (cur == value);
                            cur++;
                            Tough = (cur == value);
                            cur++;
                            Tough_organic = (cur == value);
                            cur++;
                            Tough_organic_flesh = (cur == value);
                            cur++;
                            Tough_organic_flesh_bugger = (cur == value);
                            cur++;
                            Tough_organic_flesh_brute = (cur == value);
                            cur++;
                            Tough_inorganic = (cur == value);
                            cur++;
                            Tough_inorganic_armor = (cur == value);
                            cur++;
                            Tough_inorganic_rubber = (cur == value);
                            cur++;
                            Tough_inorganic_plastic = (cur == value);
                            cur++;
                            Tough_floodflesh = (cur == value);
                            cur++;
                            Tough_floodflesh_carrierform = (cur == value);
                            cur++;
                            Tough_floodflesh_infectionform = (cur == value);
                            cur++;
                            Tough_organic_wood = (cur == value);
                            cur++;
                            Tough_terrain = (cur == value);
                            cur++;
                            Tough_terrain_dirt = (cur == value);
                            cur++;
                            Tough_terrain_grass = (cur == value);
                            cur++;
                            Tough_terrain_mud = (cur == value);
                            cur++;
                            Tough_terrain_gravel = (cur == value);
                            cur++;
                            Tough_terrain_sand = (cur == value);
                            cur++;
                            Hard = (cur == value);
                            cur++;
                            Hard_metal_thin = (cur == value);
                            cur++;
                            Hard_metal_thin_hum = (cur == value);
                            cur++;
                            Hard_metal_thin_hum_masterchief = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_grunt = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_hereticleader = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_elite = (cur == value);
                            cur++;
                            Hard_metal_thin_for_sentinel = (cur == value);
                            cur++;
                            Hard_metal_thin_for_monitor = (cur == value);
                            cur++;
                            Hard_metal_thick = (cur == value);
                            cur++;
                            Hard_metal_thick_hum = (cur == value);
                            cur++;
                            Hard_metal_thick_hum_warthog = (cur == value);
                            cur++;
                            Hard_metal_thick_hum_chassis = (cur == value);
                            cur++;
                            Hard_metal_thick_cov = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_hunter = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_ghost = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_shadow = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_banshee = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_creep = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_wraith = (cur == value);
                            cur++;
                            Hard_metal_solid = (cur == value);
                            cur++;
                            Hard_metal_solid_hum_scorpion = (cur == value);
                            cur++;
                            Hard_metal_solid_cov_hunter = (cur == value);
                            cur++;
                            Hard_metal_solid_cov_wraith = (cur == value);
                            cur++;
                            Hard_metal_solid_cov_creep = (cur == value);
                            cur++;
                            Hard_terrain = (cur == value);
                            cur++;
                            Hard_terrain_ice = (cur == value);
                            cur++;
                            Hard_terrain_stone = (cur == value);
                            cur++;
                            Hard_terrain_stone_rough = (cur == value);
                            cur++;
                            Hard_terrain_concrete = (cur == value);
                            cur++;
                            Hard_floodflesh = (cur == value);
                            cur++;
                            Brittle = (cur == value);
                            cur++;
                            Brittle_glass = (cur == value);
                            cur++;
                            Brittle_glass_for_unbreakable = (cur == value);
                            cur++;
                            Brittle_elec = (cur == value);
                            cur++;
                            Brittle_elec_cov = (cur == value);
                            cur++;
                            Brittle_elec_for = (cur == value);
                            cur++;
                            Brittle_mech = (cur == value);
                            cur++;
                            Brittle_mech_cov = (cur == value);
                            cur++;
                            Brittle_explosive = (cur == value);
                            cur++;
                            Energy = (cur == value);
                            cur++;
                            Energy_shield_thin = (cur == value);
                            cur++;
                            Energy_shield_thin_hum = (cur == value);
                            cur++;
                            Energy_shield_thin_cov = (cur == value);
                            cur++;
                            Energy_shield_thin_cov_elite = (cur == value);
                            cur++;
                            Energy_shield_thick = (cur == value);
                            cur++;
                            Energy_shield_thick_cov = (cur == value);
                            cur++;
                            Energy_shield_thick_cov_jackal = (cur == value);
                            cur++;
                            Energy_holo = (cur == value);
                            cur++;
                            Liquid_thin_water_puddle = (cur == value);
                            cur++;
                            Liquid_thin_water_shallow = (cur == value);
                            cur++;
                            Hard_metal_thin_hum_grating = (cur == value);
                            cur++;
                            Tough_terrain_woodfloor = (cur == value);
                            cur++;
                            Tough_terrain_carpet = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_brute = (cur == value);
                        }
                    }
                    /// <summary>
                    /// 5. Enum16 :: Offset: 0x16 (22)
                    /// </summary>
                    public SoundTypeEnum16 SoundType { get { return __soundType__; } set { __soundType__ = value; } }
                    private SoundTypeEnum16 __soundType__;
                    public class SoundTypeEnum16
                    {
                        /// <summary>
                        /// 1. Option :: Value: 0x1 (1)
                        /// </summary>
                        public bool Unique { get { return __unique__; } set { __unique__ = value; } }
                        private bool __unique__;
                        /// <summary>
                        /// 2. Option :: Value: 0x2 (2)
                        /// </summary>
                        public bool Standard { get { return __standard__; } set { __standard__ = value; } }
                        private bool __standard__;

                        public void _Initialize(BitReader br)
                        {
                            ushort value = br.ReadUInt16();
                            ushort cur = 0;

                            Unique = (cur == value);
                            cur++;
                            Standard = (cur == value);
                        }
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _ImpactSoundsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _ImpactSoundsTagBlock(FootLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        MaterialIdentity = new MaterialIdentityEnum16();
                        SoundType = new SoundTypeEnum16();
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_ImpactSoundsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _ImpactSoundsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 3. TagBlock :: Offset: 0x10 (16)
                /// </summary>
                public _ImpactEffectsTagBlock ImpactEffects { get { return __impactEffects__; } set { __impactEffects__ = value; } }
                private _ImpactEffectsTagBlock __impactEffects__;
                public class _ImpactEffectsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public FootLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private FootLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef ImpactEffectNum1 { get { return __impactEffectNum1__; } set { __impactEffectNum1__ = value; } }
                    private TagRef __impactEffectNum1__;
                    /// <summary>
                    /// 2. Tag :: Offset: 0x8 (8)
                    /// </summary>
                    public TagRef ImpactEffectNum2 { get { return __impactEffectNum2__; } set { __impactEffectNum2__ = value; } }
                    private TagRef __impactEffectNum2__;
                    /// <summary>
                    /// 3. StringID :: Offset: 0x10 (16)
                    /// </summary>
                    public StringID MaterialIdentifier { get { return __materialIdentifier__; } set { __materialIdentifier__ = value; } }
                    private StringID __materialIdentifier__;
                    /// <summary>
                    /// 4. Enum16 :: Offset: 0x14 (20)
                    /// </summary>
                    public MaterialIdentityEnum16 MaterialIdentity { get { return __materialIdentity__; } set { __materialIdentity__ = value; } }
                    private MaterialIdentityEnum16 __materialIdentity__;
                    public class MaterialIdentityEnum16
                    {
                        /// <summary>
                        /// 1. Option :: Value: 0xFFFFFFFF (-1)
                        /// </summary>
                        public bool NoMaterial { get { return __noMaterial__; } set { __noMaterial__ = value; } }
                        private bool __noMaterial__;
                        /// <summary>
                        /// 2. Option :: Value: 0x0 (0)
                        /// </summary>
                        public bool Default_material { get { return __default_material__; } set { __default_material__ = value; } }
                        private bool __default_material__;
                        /// <summary>
                        /// 3. Option :: Value: 0x1 (1)
                        /// </summary>
                        public bool Liquid { get { return __liquid__; } set { __liquid__ = value; } }
                        private bool __liquid__;
                        /// <summary>
                        /// 4. Option :: Value: 0x2 (2)
                        /// </summary>
                        public bool Liquid_thin { get { return __liquid_thin__; } set { __liquid_thin__ = value; } }
                        private bool __liquid_thin__;
                        /// <summary>
                        /// 5. Option :: Value: 0x6 (6)
                        /// </summary>
                        public bool Soft { get { return __soft__; } set { __soft__ = value; } }
                        private bool __soft__;
                        /// <summary>
                        /// 6. Option :: Value: 0x7 (7)
                        /// </summary>
                        public bool Soft_organic { get { return __soft_organic__; } set { __soft_organic__ = value; } }
                        private bool __soft_organic__;
                        /// <summary>
                        /// 7. Option :: Value: 0x8 (8)
                        /// </summary>
                        public bool Soft_organic_flesh { get { return __soft_organic_flesh__; } set { __soft_organic_flesh__ = value; } }
                        private bool __soft_organic_flesh__;
                        /// <summary>
                        /// 8. Option :: Value: 0x9 (9)
                        /// </summary>
                        public bool Soft_organic_flesh_human { get { return __soft_organic_flesh_human__; } set { __soft_organic_flesh_human__ = value; } }
                        private bool __soft_organic_flesh_human__;
                        /// <summary>
                        /// 9. Option :: Value: 0xA (10)
                        /// </summary>
                        public bool Soft_organic_flesh_elite { get { return __soft_organic_flesh_elite__; } set { __soft_organic_flesh_elite__ = value; } }
                        private bool __soft_organic_flesh_elite__;
                        /// <summary>
                        /// 10. Option :: Value: 0xB (11)
                        /// </summary>
                        public bool Soft_organic_flesh_grunt { get { return __soft_organic_flesh_grunt__; } set { __soft_organic_flesh_grunt__ = value; } }
                        private bool __soft_organic_flesh_grunt__;
                        /// <summary>
                        /// 11. Option :: Value: 0xC (12)
                        /// </summary>
                        public bool Soft_organic_flesh_jackal { get { return __soft_organic_flesh_jackal__; } set { __soft_organic_flesh_jackal__ = value; } }
                        private bool __soft_organic_flesh_jackal__;
                        /// <summary>
                        /// 12. Option :: Value: 0xD (13)
                        /// </summary>
                        public bool Soft_organic_flesh_hunter { get { return __soft_organic_flesh_hunter__; } set { __soft_organic_flesh_hunter__ = value; } }
                        private bool __soft_organic_flesh_hunter__;
                        /// <summary>
                        /// 13. Option :: Value: 0xE (14)
                        /// </summary>
                        public bool Soft_organic_flesh_prophet { get { return __soft_organic_flesh_prophet__; } set { __soft_organic_flesh_prophet__ = value; } }
                        private bool __soft_organic_flesh_prophet__;
                        /// <summary>
                        /// 14. Option :: Value: 0xF (15)
                        /// </summary>
                        public bool Soft_organic_flesh_bugger { get { return __soft_organic_flesh_bugger__; } set { __soft_organic_flesh_bugger__ = value; } }
                        private bool __soft_organic_flesh_bugger__;
                        /// <summary>
                        /// 15. Option :: Value: 0x11 (17)
                        /// </summary>
                        public bool Soft_organic_plant { get { return __soft_organic_plant__; } set { __soft_organic_plant__ = value; } }
                        private bool __soft_organic_plant__;
                        /// <summary>
                        /// 16. Option :: Value: 0x12 (18)
                        /// </summary>
                        public bool Soft_organic_plant_leafy { get { return __soft_organic_plant_leafy__; } set { __soft_organic_plant_leafy__ = value; } }
                        private bool __soft_organic_plant_leafy__;
                        /// <summary>
                        /// 17. Option :: Value: 0x14 (20)
                        /// </summary>
                        public bool Soft_organic_plant_grassy { get { return __soft_organic_plant_grassy__; } set { __soft_organic_plant_grassy__ = value; } }
                        private bool __soft_organic_plant_grassy__;
                        /// <summary>
                        /// 18. Option :: Value: 0x15 (21)
                        /// </summary>
                        public bool Soft_inorganic { get { return __soft_inorganic__; } set { __soft_inorganic__ = value; } }
                        private bool __soft_inorganic__;
                        /// <summary>
                        /// 19. Option :: Value: 0x16 (22)
                        /// </summary>
                        public bool Soft_inorganic_vinyl { get { return __soft_inorganic_vinyl__; } set { __soft_inorganic_vinyl__ = value; } }
                        private bool __soft_inorganic_vinyl__;
                        /// <summary>
                        /// 20. Option :: Value: 0x19 (25)
                        /// </summary>
                        public bool Soft_terrain { get { return __soft_terrain__; } set { __soft_terrain__ = value; } }
                        private bool __soft_terrain__;
                        /// <summary>
                        /// 21. Option :: Value: 0x1A (26)
                        /// </summary>
                        public bool Soft_terrain_snow { get { return __soft_terrain_snow__; } set { __soft_terrain_snow__ = value; } }
                        private bool __soft_terrain_snow__;
                        /// <summary>
                        /// 22. Option :: Value: 0x1B (27)
                        /// </summary>
                        public bool Tough { get { return __tough__; } set { __tough__ = value; } }
                        private bool __tough__;
                        /// <summary>
                        /// 23. Option :: Value: 0x1C (28)
                        /// </summary>
                        public bool Tough_organic { get { return __tough_organic__; } set { __tough_organic__ = value; } }
                        private bool __tough_organic__;
                        /// <summary>
                        /// 24. Option :: Value: 0x1D (29)
                        /// </summary>
                        public bool Tough_organic_flesh { get { return __tough_organic_flesh__; } set { __tough_organic_flesh__ = value; } }
                        private bool __tough_organic_flesh__;
                        /// <summary>
                        /// 25. Option :: Value: 0x1E (30)
                        /// </summary>
                        public bool Tough_organic_flesh_bugger { get { return __tough_organic_flesh_bugger__; } set { __tough_organic_flesh_bugger__ = value; } }
                        private bool __tough_organic_flesh_bugger__;
                        /// <summary>
                        /// 26. Option :: Value: 0x1F (31)
                        /// </summary>
                        public bool Tough_organic_flesh_brute { get { return __tough_organic_flesh_brute__; } set { __tough_organic_flesh_brute__ = value; } }
                        private bool __tough_organic_flesh_brute__;
                        /// <summary>
                        /// 27. Option :: Value: 0x21 (33)
                        /// </summary>
                        public bool Tough_inorganic { get { return __tough_inorganic__; } set { __tough_inorganic__ = value; } }
                        private bool __tough_inorganic__;
                        /// <summary>
                        /// 28. Option :: Value: 0x22 (34)
                        /// </summary>
                        public bool Tough_inorganic_armor { get { return __tough_inorganic_armor__; } set { __tough_inorganic_armor__ = value; } }
                        private bool __tough_inorganic_armor__;
                        /// <summary>
                        /// 29. Option :: Value: 0x24 (36)
                        /// </summary>
                        public bool Tough_inorganic_rubber { get { return __tough_inorganic_rubber__; } set { __tough_inorganic_rubber__ = value; } }
                        private bool __tough_inorganic_rubber__;
                        /// <summary>
                        /// 30. Option :: Value: 0x27 (39)
                        /// </summary>
                        public bool Tough_inorganic_plastic { get { return __tough_inorganic_plastic__; } set { __tough_inorganic_plastic__ = value; } }
                        private bool __tough_inorganic_plastic__;
                        /// <summary>
                        /// 31. Option :: Value: 0x2A (42)
                        /// </summary>
                        public bool Tough_floodflesh { get { return __tough_floodflesh__; } set { __tough_floodflesh__ = value; } }
                        private bool __tough_floodflesh__;
                        /// <summary>
                        /// 32. Option :: Value: 0x2C (44)
                        /// </summary>
                        public bool Tough_floodflesh_carrierform { get { return __tough_floodflesh_carrierform__; } set { __tough_floodflesh_carrierform__ = value; } }
                        private bool __tough_floodflesh_carrierform__;
                        /// <summary>
                        /// 33. Option :: Value: 0x2D (45)
                        /// </summary>
                        public bool Tough_floodflesh_infectionform { get { return __tough_floodflesh_infectionform__; } set { __tough_floodflesh_infectionform__ = value; } }
                        private bool __tough_floodflesh_infectionform__;
                        /// <summary>
                        /// 34. Option :: Value: 0x2E (46)
                        /// </summary>
                        public bool Tough_organic_wood { get { return __tough_organic_wood__; } set { __tough_organic_wood__ = value; } }
                        private bool __tough_organic_wood__;
                        /// <summary>
                        /// 35. Option :: Value: 0x2F (47)
                        /// </summary>
                        public bool Tough_terrain { get { return __tough_terrain__; } set { __tough_terrain__ = value; } }
                        private bool __tough_terrain__;
                        /// <summary>
                        /// 36. Option :: Value: 0x30 (48)
                        /// </summary>
                        public bool Tough_terrain_dirt { get { return __tough_terrain_dirt__; } set { __tough_terrain_dirt__ = value; } }
                        private bool __tough_terrain_dirt__;
                        /// <summary>
                        /// 37. Option :: Value: 0x31 (49)
                        /// </summary>
                        public bool Tough_terrain_grass { get { return __tough_terrain_grass__; } set { __tough_terrain_grass__ = value; } }
                        private bool __tough_terrain_grass__;
                        /// <summary>
                        /// 38. Option :: Value: 0x32 (50)
                        /// </summary>
                        public bool Tough_terrain_mud { get { return __tough_terrain_mud__; } set { __tough_terrain_mud__ = value; } }
                        private bool __tough_terrain_mud__;
                        /// <summary>
                        /// 39. Option :: Value: 0x33 (51)
                        /// </summary>
                        public bool Tough_terrain_gravel { get { return __tough_terrain_gravel__; } set { __tough_terrain_gravel__ = value; } }
                        private bool __tough_terrain_gravel__;
                        /// <summary>
                        /// 40. Option :: Value: 0x34 (52)
                        /// </summary>
                        public bool Tough_terrain_sand { get { return __tough_terrain_sand__; } set { __tough_terrain_sand__ = value; } }
                        private bool __tough_terrain_sand__;
                        /// <summary>
                        /// 41. Option :: Value: 0x35 (53)
                        /// </summary>
                        public bool Hard { get { return __hard__; } set { __hard__ = value; } }
                        private bool __hard__;
                        /// <summary>
                        /// 42. Option :: Value: 0x36 (54)
                        /// </summary>
                        public bool Hard_metal_thin { get { return __hard_metal_thin__; } set { __hard_metal_thin__ = value; } }
                        private bool __hard_metal_thin__;
                        /// <summary>
                        /// 43. Option :: Value: 0x37 (55)
                        /// </summary>
                        public bool Hard_metal_thin_hum { get { return __hard_metal_thin_hum__; } set { __hard_metal_thin_hum__ = value; } }
                        private bool __hard_metal_thin_hum__;
                        /// <summary>
                        /// 44. Option :: Value: 0x38 (56)
                        /// </summary>
                        public bool Hard_metal_thin_hum_masterchief { get { return __hard_metal_thin_hum_masterchief__; } set { __hard_metal_thin_hum_masterchief__ = value; } }
                        private bool __hard_metal_thin_hum_masterchief__;
                        /// <summary>
                        /// 45. Option :: Value: 0x3D (61)
                        /// </summary>
                        public bool Hard_metal_thin_cov_grunt { get { return __hard_metal_thin_cov_grunt__; } set { __hard_metal_thin_cov_grunt__ = value; } }
                        private bool __hard_metal_thin_cov_grunt__;
                        /// <summary>
                        /// 46. Option :: Value: 0x3E (62)
                        /// </summary>
                        public bool Hard_metal_thin_cov_hereticleader { get { return __hard_metal_thin_cov_hereticleader__; } set { __hard_metal_thin_cov_hereticleader__ = value; } }
                        private bool __hard_metal_thin_cov_hereticleader__;
                        /// <summary>
                        /// 47. Option :: Value: 0x3F (63)
                        /// </summary>
                        public bool Hard_metal_thin_cov_elite { get { return __hard_metal_thin_cov_elite__; } set { __hard_metal_thin_cov_elite__ = value; } }
                        private bool __hard_metal_thin_cov_elite__;
                        /// <summary>
                        /// 48. Option :: Value: 0x43 (67)
                        /// </summary>
                        public bool Hard_metal_thin_for_sentinel { get { return __hard_metal_thin_for_sentinel__; } set { __hard_metal_thin_for_sentinel__ = value; } }
                        private bool __hard_metal_thin_for_sentinel__;
                        /// <summary>
                        /// 49. Option :: Value: 0x44 (68)
                        /// </summary>
                        public bool Hard_metal_thin_for_monitor { get { return __hard_metal_thin_for_monitor__; } set { __hard_metal_thin_for_monitor__ = value; } }
                        private bool __hard_metal_thin_for_monitor__;
                        /// <summary>
                        /// 50. Option :: Value: 0x45 (69)
                        /// </summary>
                        public bool Hard_metal_thick { get { return __hard_metal_thick__; } set { __hard_metal_thick__ = value; } }
                        private bool __hard_metal_thick__;
                        /// <summary>
                        /// 51. Option :: Value: 0x46 (70)
                        /// </summary>
                        public bool Hard_metal_thick_hum { get { return __hard_metal_thick_hum__; } set { __hard_metal_thick_hum__ = value; } }
                        private bool __hard_metal_thick_hum__;
                        /// <summary>
                        /// 52. Option :: Value: 0x48 (72)
                        /// </summary>
                        public bool Hard_metal_thick_hum_warthog { get { return __hard_metal_thick_hum_warthog__; } set { __hard_metal_thick_hum_warthog__ = value; } }
                        private bool __hard_metal_thick_hum_warthog__;
                        /// <summary>
                        /// 53. Option :: Value: 0x4B (75)
                        /// </summary>
                        public bool Hard_metal_thick_hum_chassis { get { return __hard_metal_thick_hum_chassis__; } set { __hard_metal_thick_hum_chassis__ = value; } }
                        private bool __hard_metal_thick_hum_chassis__;
                        /// <summary>
                        /// 54. Option :: Value: 0x4C (76)
                        /// </summary>
                        public bool Hard_metal_thick_cov { get { return __hard_metal_thick_cov__; } set { __hard_metal_thick_cov__ = value; } }
                        private bool __hard_metal_thick_cov__;
                        /// <summary>
                        /// 55. Option :: Value: 0x4D (77)
                        /// </summary>
                        public bool Hard_metal_thick_cov_hunter { get { return __hard_metal_thick_cov_hunter__; } set { __hard_metal_thick_cov_hunter__ = value; } }
                        private bool __hard_metal_thick_cov_hunter__;
                        /// <summary>
                        /// 56. Option :: Value: 0x4E (78)
                        /// </summary>
                        public bool Hard_metal_thick_cov_ghost { get { return __hard_metal_thick_cov_ghost__; } set { __hard_metal_thick_cov_ghost__ = value; } }
                        private bool __hard_metal_thick_cov_ghost__;
                        /// <summary>
                        /// 57. Option :: Value: 0x4F (79)
                        /// </summary>
                        public bool Hard_metal_thick_cov_shadow { get { return __hard_metal_thick_cov_shadow__; } set { __hard_metal_thick_cov_shadow__ = value; } }
                        private bool __hard_metal_thick_cov_shadow__;
                        /// <summary>
                        /// 58. Option :: Value: 0x52 (82)
                        /// </summary>
                        public bool Hard_metal_thick_cov_banshee { get { return __hard_metal_thick_cov_banshee__; } set { __hard_metal_thick_cov_banshee__ = value; } }
                        private bool __hard_metal_thick_cov_banshee__;
                        /// <summary>
                        /// 59. Option :: Value: 0x53 (83)
                        /// </summary>
                        public bool Hard_metal_thick_cov_creep { get { return __hard_metal_thick_cov_creep__; } set { __hard_metal_thick_cov_creep__ = value; } }
                        private bool __hard_metal_thick_cov_creep__;
                        /// <summary>
                        /// 60. Option :: Value: 0x54 (84)
                        /// </summary>
                        public bool Hard_metal_thick_cov_wraith { get { return __hard_metal_thick_cov_wraith__; } set { __hard_metal_thick_cov_wraith__ = value; } }
                        private bool __hard_metal_thick_cov_wraith__;
                        /// <summary>
                        /// 61. Option :: Value: 0x59 (89)
                        /// </summary>
                        public bool Hard_metal_solid { get { return __hard_metal_solid__; } set { __hard_metal_solid__ = value; } }
                        private bool __hard_metal_solid__;
                        /// <summary>
                        /// 62. Option :: Value: 0x5B (91)
                        /// </summary>
                        public bool Hard_metal_solid_hum_scorpion { get { return __hard_metal_solid_hum_scorpion__; } set { __hard_metal_solid_hum_scorpion__ = value; } }
                        private bool __hard_metal_solid_hum_scorpion__;
                        /// <summary>
                        /// 63. Option :: Value: 0x5F (95)
                        /// </summary>
                        public bool Hard_metal_solid_cov_hunter { get { return __hard_metal_solid_cov_hunter__; } set { __hard_metal_solid_cov_hunter__ = value; } }
                        private bool __hard_metal_solid_cov_hunter__;
                        /// <summary>
                        /// 64. Option :: Value: 0x60 (96)
                        /// </summary>
                        public bool Hard_metal_solid_cov_wraith { get { return __hard_metal_solid_cov_wraith__; } set { __hard_metal_solid_cov_wraith__ = value; } }
                        private bool __hard_metal_solid_cov_wraith__;
                        /// <summary>
                        /// 65. Option :: Value: 0x61 (97)
                        /// </summary>
                        public bool Hard_metal_solid_cov_creep { get { return __hard_metal_solid_cov_creep__; } set { __hard_metal_solid_cov_creep__ = value; } }
                        private bool __hard_metal_solid_cov_creep__;
                        /// <summary>
                        /// 66. Option :: Value: 0x64 (100)
                        /// </summary>
                        public bool Hard_terrain { get { return __hard_terrain__; } set { __hard_terrain__ = value; } }
                        private bool __hard_terrain__;
                        /// <summary>
                        /// 67. Option :: Value: 0x65 (101)
                        /// </summary>
                        public bool Hard_terrain_ice { get { return __hard_terrain_ice__; } set { __hard_terrain_ice__ = value; } }
                        private bool __hard_terrain_ice__;
                        /// <summary>
                        /// 68. Option :: Value: 0x66 (102)
                        /// </summary>
                        public bool Hard_terrain_stone { get { return __hard_terrain_stone__; } set { __hard_terrain_stone__ = value; } }
                        private bool __hard_terrain_stone__;
                        /// <summary>
                        /// 69. Option :: Value: 0x67 (103)
                        /// </summary>
                        public bool Hard_terrain_stone_rough { get { return __hard_terrain_stone_rough__; } set { __hard_terrain_stone_rough__ = value; } }
                        private bool __hard_terrain_stone_rough__;
                        /// <summary>
                        /// 70. Option :: Value: 0x68 (104)
                        /// </summary>
                        public bool Hard_terrain_concrete { get { return __hard_terrain_concrete__; } set { __hard_terrain_concrete__ = value; } }
                        private bool __hard_terrain_concrete__;
                        /// <summary>
                        /// 71. Option :: Value: 0x69 (105)
                        /// </summary>
                        public bool Hard_floodflesh { get { return __hard_floodflesh__; } set { __hard_floodflesh__ = value; } }
                        private bool __hard_floodflesh__;
                        /// <summary>
                        /// 72. Option :: Value: 0x6A (106)
                        /// </summary>
                        public bool Brittle { get { return __brittle__; } set { __brittle__ = value; } }
                        private bool __brittle__;
                        /// <summary>
                        /// 73. Option :: Value: 0x6B (107)
                        /// </summary>
                        public bool Brittle_glass { get { return __brittle_glass__; } set { __brittle_glass__ = value; } }
                        private bool __brittle_glass__;
                        /// <summary>
                        /// 74. Option :: Value: 0x70 (112)
                        /// </summary>
                        public bool Brittle_glass_for_unbreakable { get { return __brittle_glass_for_unbreakable__; } set { __brittle_glass_for_unbreakable__ = value; } }
                        private bool __brittle_glass_for_unbreakable__;
                        /// <summary>
                        /// 75. Option :: Value: 0x73 (115)
                        /// </summary>
                        public bool Brittle_elec { get { return __brittle_elec__; } set { __brittle_elec__ = value; } }
                        private bool __brittle_elec__;
                        /// <summary>
                        /// 76. Option :: Value: 0x75 (117)
                        /// </summary>
                        public bool Brittle_elec_cov { get { return __brittle_elec_cov__; } set { __brittle_elec_cov__ = value; } }
                        private bool __brittle_elec_cov__;
                        /// <summary>
                        /// 77. Option :: Value: 0x7E (126)
                        /// </summary>
                        public bool Brittle_elec_for { get { return __brittle_elec_for__; } set { __brittle_elec_for__ = value; } }
                        private bool __brittle_elec_for__;
                        /// <summary>
                        /// 78. Option :: Value: 0x80 (128)
                        /// </summary>
                        public bool Brittle_mech { get { return __brittle_mech__; } set { __brittle_mech__ = value; } }
                        private bool __brittle_mech__;
                        /// <summary>
                        /// 79. Option :: Value: 0x87 (135)
                        /// </summary>
                        public bool Brittle_mech_cov { get { return __brittle_mech_cov__; } set { __brittle_mech_cov__ = value; } }
                        private bool __brittle_mech_cov__;
                        /// <summary>
                        /// 80. Option :: Value: 0x89 (137)
                        /// </summary>
                        public bool Brittle_explosive { get { return __brittle_explosive__; } set { __brittle_explosive__ = value; } }
                        private bool __brittle_explosive__;
                        /// <summary>
                        /// 81. Option :: Value: 0x90 (144)
                        /// </summary>
                        public bool Energy { get { return __energy__; } set { __energy__ = value; } }
                        private bool __energy__;
                        /// <summary>
                        /// 82. Option :: Value: 0x91 (145)
                        /// </summary>
                        public bool Energy_shield_thin { get { return __energy_shield_thin__; } set { __energy_shield_thin__ = value; } }
                        private bool __energy_shield_thin__;
                        /// <summary>
                        /// 83. Option :: Value: 0x92 (146)
                        /// </summary>
                        public bool Energy_shield_thin_hum { get { return __energy_shield_thin_hum__; } set { __energy_shield_thin_hum__ = value; } }
                        private bool __energy_shield_thin_hum__;
                        /// <summary>
                        /// 84. Option :: Value: 0x94 (148)
                        /// </summary>
                        public bool Energy_shield_thin_cov { get { return __energy_shield_thin_cov__; } set { __energy_shield_thin_cov__ = value; } }
                        private bool __energy_shield_thin_cov__;
                        /// <summary>
                        /// 85. Option :: Value: 0x95 (149)
                        /// </summary>
                        public bool Energy_shield_thin_cov_elite { get { return __energy_shield_thin_cov_elite__; } set { __energy_shield_thin_cov_elite__ = value; } }
                        private bool __energy_shield_thin_cov_elite__;
                        /// <summary>
                        /// 86. Option :: Value: 0x98 (152)
                        /// </summary>
                        public bool Energy_shield_thick { get { return __energy_shield_thick__; } set { __energy_shield_thick__ = value; } }
                        private bool __energy_shield_thick__;
                        /// <summary>
                        /// 87. Option :: Value: 0x9A (154)
                        /// </summary>
                        public bool Energy_shield_thick_cov { get { return __energy_shield_thick_cov__; } set { __energy_shield_thick_cov__ = value; } }
                        private bool __energy_shield_thick_cov__;
                        /// <summary>
                        /// 88. Option :: Value: 0x9B (155)
                        /// </summary>
                        public bool Energy_shield_thick_cov_jackal { get { return __energy_shield_thick_cov_jackal__; } set { __energy_shield_thick_cov_jackal__ = value; } }
                        private bool __energy_shield_thick_cov_jackal__;
                        /// <summary>
                        /// 89. Option :: Value: 0x9D (157)
                        /// </summary>
                        public bool Energy_holo { get { return __energy_holo__; } set { __energy_holo__ = value; } }
                        private bool __energy_holo__;
                        /// <summary>
                        /// 90. Option :: Value: 0xA1 (161)
                        /// </summary>
                        public bool Liquid_thin_water_puddle { get { return __liquid_thin_water_puddle__; } set { __liquid_thin_water_puddle__ = value; } }
                        private bool __liquid_thin_water_puddle__;
                        /// <summary>
                        /// 91. Option :: Value: 0xA2 (162)
                        /// </summary>
                        public bool Liquid_thin_water_shallow { get { return __liquid_thin_water_shallow__; } set { __liquid_thin_water_shallow__ = value; } }
                        private bool __liquid_thin_water_shallow__;
                        /// <summary>
                        /// 92. Option :: Value: 0xA7 (167)
                        /// </summary>
                        public bool Hard_metal_thin_hum_grating { get { return __hard_metal_thin_hum_grating__; } set { __hard_metal_thin_hum_grating__ = value; } }
                        private bool __hard_metal_thin_hum_grating__;
                        /// <summary>
                        /// 93. Option :: Value: 0xAB (171)
                        /// </summary>
                        public bool Tough_terrain_woodfloor { get { return __tough_terrain_woodfloor__; } set { __tough_terrain_woodfloor__ = value; } }
                        private bool __tough_terrain_woodfloor__;
                        /// <summary>
                        /// 94. Option :: Value: 0xB0 (176)
                        /// </summary>
                        public bool Tough_terrain_carpet { get { return __tough_terrain_carpet__; } set { __tough_terrain_carpet__ = value; } }
                        private bool __tough_terrain_carpet__;
                        /// <summary>
                        /// 95. Option :: Value: 0xB1 (177)
                        /// </summary>
                        public bool Hard_metal_thin_cov_brute { get { return __hard_metal_thin_cov_brute__; } set { __hard_metal_thin_cov_brute__ = value; } }
                        private bool __hard_metal_thin_cov_brute__;

                        public void _Initialize(BitReader br)
                        {
                            ushort value = br.ReadUInt16();
                            ushort cur = 0;

                            NoMaterial = (cur == value);
                            cur++;
                            Default_material = (cur == value);
                            cur++;
                            Liquid = (cur == value);
                            cur++;
                            Liquid_thin = (cur == value);
                            cur++;
                            Soft = (cur == value);
                            cur++;
                            Soft_organic = (cur == value);
                            cur++;
                            Soft_organic_flesh = (cur == value);
                            cur++;
                            Soft_organic_flesh_human = (cur == value);
                            cur++;
                            Soft_organic_flesh_elite = (cur == value);
                            cur++;
                            Soft_organic_flesh_grunt = (cur == value);
                            cur++;
                            Soft_organic_flesh_jackal = (cur == value);
                            cur++;
                            Soft_organic_flesh_hunter = (cur == value);
                            cur++;
                            Soft_organic_flesh_prophet = (cur == value);
                            cur++;
                            Soft_organic_flesh_bugger = (cur == value);
                            cur++;
                            Soft_organic_plant = (cur == value);
                            cur++;
                            Soft_organic_plant_leafy = (cur == value);
                            cur++;
                            Soft_organic_plant_grassy = (cur == value);
                            cur++;
                            Soft_inorganic = (cur == value);
                            cur++;
                            Soft_inorganic_vinyl = (cur == value);
                            cur++;
                            Soft_terrain = (cur == value);
                            cur++;
                            Soft_terrain_snow = (cur == value);
                            cur++;
                            Tough = (cur == value);
                            cur++;
                            Tough_organic = (cur == value);
                            cur++;
                            Tough_organic_flesh = (cur == value);
                            cur++;
                            Tough_organic_flesh_bugger = (cur == value);
                            cur++;
                            Tough_organic_flesh_brute = (cur == value);
                            cur++;
                            Tough_inorganic = (cur == value);
                            cur++;
                            Tough_inorganic_armor = (cur == value);
                            cur++;
                            Tough_inorganic_rubber = (cur == value);
                            cur++;
                            Tough_inorganic_plastic = (cur == value);
                            cur++;
                            Tough_floodflesh = (cur == value);
                            cur++;
                            Tough_floodflesh_carrierform = (cur == value);
                            cur++;
                            Tough_floodflesh_infectionform = (cur == value);
                            cur++;
                            Tough_organic_wood = (cur == value);
                            cur++;
                            Tough_terrain = (cur == value);
                            cur++;
                            Tough_terrain_dirt = (cur == value);
                            cur++;
                            Tough_terrain_grass = (cur == value);
                            cur++;
                            Tough_terrain_mud = (cur == value);
                            cur++;
                            Tough_terrain_gravel = (cur == value);
                            cur++;
                            Tough_terrain_sand = (cur == value);
                            cur++;
                            Hard = (cur == value);
                            cur++;
                            Hard_metal_thin = (cur == value);
                            cur++;
                            Hard_metal_thin_hum = (cur == value);
                            cur++;
                            Hard_metal_thin_hum_masterchief = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_grunt = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_hereticleader = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_elite = (cur == value);
                            cur++;
                            Hard_metal_thin_for_sentinel = (cur == value);
                            cur++;
                            Hard_metal_thin_for_monitor = (cur == value);
                            cur++;
                            Hard_metal_thick = (cur == value);
                            cur++;
                            Hard_metal_thick_hum = (cur == value);
                            cur++;
                            Hard_metal_thick_hum_warthog = (cur == value);
                            cur++;
                            Hard_metal_thick_hum_chassis = (cur == value);
                            cur++;
                            Hard_metal_thick_cov = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_hunter = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_ghost = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_shadow = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_banshee = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_creep = (cur == value);
                            cur++;
                            Hard_metal_thick_cov_wraith = (cur == value);
                            cur++;
                            Hard_metal_solid = (cur == value);
                            cur++;
                            Hard_metal_solid_hum_scorpion = (cur == value);
                            cur++;
                            Hard_metal_solid_cov_hunter = (cur == value);
                            cur++;
                            Hard_metal_solid_cov_wraith = (cur == value);
                            cur++;
                            Hard_metal_solid_cov_creep = (cur == value);
                            cur++;
                            Hard_terrain = (cur == value);
                            cur++;
                            Hard_terrain_ice = (cur == value);
                            cur++;
                            Hard_terrain_stone = (cur == value);
                            cur++;
                            Hard_terrain_stone_rough = (cur == value);
                            cur++;
                            Hard_terrain_concrete = (cur == value);
                            cur++;
                            Hard_floodflesh = (cur == value);
                            cur++;
                            Brittle = (cur == value);
                            cur++;
                            Brittle_glass = (cur == value);
                            cur++;
                            Brittle_glass_for_unbreakable = (cur == value);
                            cur++;
                            Brittle_elec = (cur == value);
                            cur++;
                            Brittle_elec_cov = (cur == value);
                            cur++;
                            Brittle_elec_for = (cur == value);
                            cur++;
                            Brittle_mech = (cur == value);
                            cur++;
                            Brittle_mech_cov = (cur == value);
                            cur++;
                            Brittle_explosive = (cur == value);
                            cur++;
                            Energy = (cur == value);
                            cur++;
                            Energy_shield_thin = (cur == value);
                            cur++;
                            Energy_shield_thin_hum = (cur == value);
                            cur++;
                            Energy_shield_thin_cov = (cur == value);
                            cur++;
                            Energy_shield_thin_cov_elite = (cur == value);
                            cur++;
                            Energy_shield_thick = (cur == value);
                            cur++;
                            Energy_shield_thick_cov = (cur == value);
                            cur++;
                            Energy_shield_thick_cov_jackal = (cur == value);
                            cur++;
                            Energy_holo = (cur == value);
                            cur++;
                            Liquid_thin_water_puddle = (cur == value);
                            cur++;
                            Liquid_thin_water_shallow = (cur == value);
                            cur++;
                            Hard_metal_thin_hum_grating = (cur == value);
                            cur++;
                            Tough_terrain_woodfloor = (cur == value);
                            cur++;
                            Tough_terrain_carpet = (cur == value);
                            cur++;
                            Hard_metal_thin_cov_brute = (cur == value);
                        }
                    }
                    /// <summary>
                    /// 5. ByteArray :: Offset: 0x16 (22)
                    /// </summary>
                    public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                    private byte[] __unused__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _ImpactEffectsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _ImpactEffectsTagBlock(FootLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        MaterialIdentity = new MaterialIdentityEnum16();
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_ImpactEffectsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _ImpactEffectsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ImpactOptionsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ImpactOptionsTagBlock(FootLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    ImpactSounds = new _ImpactSoundsTagBlock(layout);
                    ImpactEffects = new _ImpactEffectsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ImpactOptionsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ImpactOptionsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the FootLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public FootLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                ImpactOptions = new _ImpactOptionsTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                ImpactOptions._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Foot class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Foot(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new FootLayout(map, identity);
        }

        #region Specific Methods of foot

        #endregion
    }
}
