using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Fog Patch
    /// </summary>
    public class Fpch
    {
        #region Constants

        /// <summary>
        /// Size of the fpch meta header.
        /// </summary>
        public const int HeaderSize = 80;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private FpchLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public FpchLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class FpchLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Float :: Offset: 0x0 (0)
            /// </summary>
            public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private float __unknown1__;
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private float __unknown3__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private float __unknown4__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private float __unknown5__;
            /// <summary>
            /// 6. Tag :: Offset: 0x14 (20)
            /// </summary>
            public TagRef Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private TagRef __unknown6__;
            /// <summary>
            /// 7. Float :: Offset: 0x1C (28)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;
            /// <summary>
            /// 8. Float :: Offset: 0x20 (32)
            /// </summary>
            public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private float __unknown8__;
            /// <summary>
            /// 9. Float :: Offset: 0x24 (36)
            /// </summary>
            public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private float __unknown9__;
            /// <summary>
            /// 10. Float :: Offset: 0x28 (40)
            /// </summary>
            public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private float __unknown10__;
            /// <summary>
            /// 11. Float :: Offset: 0x2C (44)
            /// </summary>
            public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private float __unknown11__;
            /// <summary>
            /// 12. Float :: Offset: 0x30 (48)
            /// </summary>
            public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private float __unknown12__;
            /// <summary>
            /// 13. Float :: Offset: 0x34 (52)
            /// </summary>
            public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private float __unknown13__;
            /// <summary>
            /// 14. Float :: Offset: 0x38 (56)
            /// </summary>
            public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
            private float __unknown14__;
            /// <summary>
            /// 15. Float :: Offset: 0x3C (60)
            /// </summary>
            public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
            private float __unknown15__;
            /// <summary>
            /// 16. Float :: Offset: 0x40 (64)
            /// </summary>
            public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
            private float __unknown16__;
            /// <summary>
            /// 17. Float :: Offset: 0x44 (68)
            /// </summary>
            public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
            private float __unknown17__;
            /// <summary>
            /// 18. Float :: Offset: 0x48 (72)
            /// </summary>
            public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
            private float __unknown18__;
            /// <summary>
            /// 19. Float :: Offset: 0x4C (76)
            /// </summary>
            public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
            private float __unknown19__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the FpchLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public FpchLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unknown1 = br.ReadSingle();
                Unknown2 = br.ReadSingle();
                Unknown3 = br.ReadSingle();
                Unknown4 = br.ReadSingle();
                Unknown5 = br.ReadSingle();
                Unknown6._Initialize(br);
                Unknown7 = br.ReadSingle();
                Unknown8 = br.ReadSingle();
                Unknown9 = br.ReadSingle();
                Unknown10 = br.ReadSingle();
                Unknown11 = br.ReadSingle();
                Unknown12 = br.ReadSingle();
                Unknown13 = br.ReadSingle();
                Unknown14 = br.ReadSingle();
                Unknown15 = br.ReadSingle();
                Unknown16 = br.ReadSingle();
                Unknown17 = br.ReadSingle();
                Unknown18 = br.ReadSingle();
                Unknown19 = br.ReadSingle();

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Fpch class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Fpch(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new FpchLayout(map, identity);
        }

        #region Specific Methods of fpch

        #endregion
    }
}
