using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// HUD Global
    /// </summary>
    public class Hudg
    {
        #region Constants

        /// <summary>
        /// Size of the hudg meta header.
        /// </summary>
        public const int HeaderSize = 1160;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private HudgLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.80 DiscoBob: Mapped most of the plugin...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public HudgLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class HudgLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. ByteArray :: Offset: 0x0 (0)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 2. Enum16 :: Offset: 0x24 (36)
            /// </summary>
            public MessageAnchorEnum16 MessageAnchor { get { return __messageAnchor__; } set { __messageAnchor__ = value; } }
            private MessageAnchorEnum16 __messageAnchor__;
            public class MessageAnchorEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool TopLeft { get { return __topLeft__; } set { __topLeft__ = value; } }
                private bool __topLeft__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool TopRight { get { return __topRight__; } set { __topRight__ = value; } }
                private bool __topRight__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool BottomLeft { get { return __bottomLeft__; } set { __bottomLeft__ = value; } }
                private bool __bottomLeft__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool BottomRight { get { return __bottomRight__; } set { __bottomRight__ = value; } }
                private bool __bottomRight__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Center { get { return __center__; } set { __center__ = value; } }
                private bool __center__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    TopLeft = (cur == value);
                    cur++;
                    TopRight = (cur == value);
                    cur++;
                    BottomLeft = (cur == value);
                    cur++;
                    BottomRight = (cur == value);
                    cur++;
                    Center = (cur == value);
                }
            }
            /// <summary>
            /// 3. Byte :: Offset: 0x26 (38)
            /// </summary>
            public byte AnchorOffset_y { get { return __anchorOffset_y__; } set { __anchorOffset_y__ = value; } }
            private byte __anchorOffset_y__;
            /// <summary>
            /// 4. Byte :: Offset: 0x27 (39)
            /// </summary>
            public byte AnchorOffset_x { get { return __anchorOffset_x__; } set { __anchorOffset_x__ = value; } }
            private byte __anchorOffset_x__;
            /// <summary>
            /// 5. ByteArray :: Offset: 0x28 (40)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 6. Tag :: Offset: 0x48 (72)
            /// </summary>
            public TagRef Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private TagRef __unused3__;
            /// <summary>
            /// 7. Tag :: Offset: 0x50 (80)
            /// </summary>
            public TagRef Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
            private TagRef __unused4__;
            /// <summary>
            /// 8. Float :: Offset: 0x58 (88)
            /// </summary>
            public float MessageUptime { get { return __messageUptime__; } set { __messageUptime__ = value; } }
            private float __messageUptime__;
            /// <summary>
            /// 9. Float :: Offset: 0x5C (92)
            /// </summary>
            public float MessageFadeTime { get { return __messageFadeTime__; } set { __messageFadeTime__ = value; } }
            private float __messageFadeTime__;
            /// <summary>
            /// 10. Float :: Offset: 0x60 (96)
            /// </summary>
            public float IconColor_alpha { get { return __iconColor_alpha__; } set { __iconColor_alpha__ = value; } }
            private float __iconColor_alpha__;
            /// <summary>
            /// 11. Float :: Offset: 0x64 (100)
            /// </summary>
            public float IconColor_red { get { return __iconColor_red__; } set { __iconColor_red__ = value; } }
            private float __iconColor_red__;
            /// <summary>
            /// 12. Float :: Offset: 0x68 (104)
            /// </summary>
            public float IconColor_green { get { return __iconColor_green__; } set { __iconColor_green__ = value; } }
            private float __iconColor_green__;
            /// <summary>
            /// 13. Float :: Offset: 0x6C (108)
            /// </summary>
            public float IconColor_blue { get { return __iconColor_blue__; } set { __iconColor_blue__ = value; } }
            private float __iconColor_blue__;
            /// <summary>
            /// 14. Float :: Offset: 0x70 (112)
            /// </summary>
            public float TextColor_alpha { get { return __textColor_alpha__; } set { __textColor_alpha__ = value; } }
            private float __textColor_alpha__;
            /// <summary>
            /// 15. Float :: Offset: 0x74 (116)
            /// </summary>
            public float TextColor_red { get { return __textColor_red__; } set { __textColor_red__ = value; } }
            private float __textColor_red__;
            /// <summary>
            /// 16. Float :: Offset: 0x78 (120)
            /// </summary>
            public float TextColor_green { get { return __textColor_green__; } set { __textColor_green__ = value; } }
            private float __textColor_green__;
            /// <summary>
            /// 17. Float :: Offset: 0x7C (124)
            /// </summary>
            public float TextColor_blue { get { return __textColor_blue__; } set { __textColor_blue__ = value; } }
            private float __textColor_blue__;
            /// <summary>
            /// 18. Float :: Offset: 0x80 (128)
            /// </summary>
            public float TextSpacing { get { return __textSpacing__; } set { __textSpacing__ = value; } }
            private float __textSpacing__;
            /// <summary>
            /// 19. Tag :: Offset: 0x84 (132)
            /// </summary>
            public TagRef Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
            private TagRef __unused5__;
            /// <summary>
            /// 20. Tag :: Offset: 0x8C (140)
            /// </summary>
            public TagRef IconBitmap { get { return __iconBitmap__; } set { __iconBitmap__ = value; } }
            private TagRef __iconBitmap__;
            /// <summary>
            /// 21. Tag :: Offset: 0x94 (148)
            /// </summary>
            public TagRef Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
            private TagRef __unused6__;
            /// <summary>
            /// 22. TagBlock :: Offset: 0x9C (156)
            /// </summary>
            public _ButtonIconsTagBlock ButtonIcons { get { return __buttonIcons__; } set { __buttonIcons__ = value; } }
            private _ButtonIconsTagBlock __buttonIcons__;
            public class _ButtonIconsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Int16 :: Offset: 0x0 (0)
                /// </summary>
                public short SequenceIndex { get { return __sequenceIndex__; } set { __sequenceIndex__ = value; } }
                private short __sequenceIndex__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x2 (2)
                /// </summary>
                public short WidthOffset { get { return __widthOffset__; } set { __widthOffset__ = value; } }
                private short __widthOffset__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x4 (4)
                /// </summary>
                public short OffsetFromReferenceCorner_x { get { return __offsetFromReferenceCorner_x__; } set { __offsetFromReferenceCorner_x__ = value; } }
                private short __offsetFromReferenceCorner_x__;
                /// <summary>
                /// 4. Int16 :: Offset: 0x6 (6)
                /// </summary>
                public short OffsetFromReferenceCorner_y { get { return __offsetFromReferenceCorner_y__; } set { __offsetFromReferenceCorner_y__ = value; } }
                private short __offsetFromReferenceCorner_y__;
                /// <summary>
                /// 5. Byte :: Offset: 0x8 (8)
                /// </summary>
                public byte OverrideIconColor_blue { get { return __overrideIconColor_blue__; } set { __overrideIconColor_blue__ = value; } }
                private byte __overrideIconColor_blue__;
                /// <summary>
                /// 6. Byte :: Offset: 0x9 (9)
                /// </summary>
                public byte OverrideIconColor_green { get { return __overrideIconColor_green__; } set { __overrideIconColor_green__ = value; } }
                private byte __overrideIconColor_green__;
                /// <summary>
                /// 7. Byte :: Offset: 0xA (10)
                /// </summary>
                public byte OverrideIconColor_red { get { return __overrideIconColor_red__; } set { __overrideIconColor_red__ = value; } }
                private byte __overrideIconColor_red__;
                /// <summary>
                /// 8. Byte :: Offset: 0xB (11)
                /// </summary>
                public byte OverrideIconColor_alpha { get { return __overrideIconColor_alpha__; } set { __overrideIconColor_alpha__ = value; } }
                private byte __overrideIconColor_alpha__;
                /// <summary>
                /// 9. Byte :: Offset: 0xC (12)
                /// </summary>
                public byte FrameRate_ { get { return __frameRate___; } set { __frameRate___ = value; } }
                private byte __frameRate___;
                /// <summary>
                /// 10. Bitmask8 :: Offset: 0xD (13)
                /// </summary>
                public FlagsBitmask8 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask8 __flags__;
                public class FlagsBitmask8
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool UseTextFromString_listInstead { get { return __useTextFromString_listInstead__; } set { __useTextFromString_listInstead__ = value; } }
                    private bool __useTextFromString_listInstead__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool OverrideDefaultColor { get { return __overrideDefaultColor__; } set { __overrideDefaultColor__ = value; } }
                    private bool __overrideDefaultColor__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool WidthOffsetIsAbsoluteIconWidth { get { return __widthOffsetIsAbsoluteIconWidth__; } set { __widthOffsetIsAbsoluteIconWidth__ = value; } }
                    private bool __widthOffsetIsAbsoluteIconWidth__;

                    public void _Initialize(BitReader br)
                    {
                        byte mask = br.ReadByte();
                        byte bit = 1;

                        UseTextFromString_listInstead = ((bit & mask) == bit);
                        bit <<= 1;
                        OverrideDefaultColor = ((bit & mask) == bit);
                        bit <<= 1;
                        WidthOffsetIsAbsoluteIconWidth = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 11. Int16 :: Offset: 0xE (14)
                /// </summary>
                public short TextIndex { get { return __textIndex__; } set { __textIndex__ = value; } }
                private short __textIndex__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ButtonIconsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ButtonIconsTagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask8();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ButtonIconsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ButtonIconsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 23. Byte :: Offset: 0xA4 (164)
            /// </summary>
            public byte HudHelpDefaultColor_blue { get { return __hudHelpDefaultColor_blue__; } set { __hudHelpDefaultColor_blue__ = value; } }
            private byte __hudHelpDefaultColor_blue__;
            /// <summary>
            /// 24. Byte :: Offset: 0xA5 (165)
            /// </summary>
            public byte HudHelpDefaultColor_green { get { return __hudHelpDefaultColor_green__; } set { __hudHelpDefaultColor_green__ = value; } }
            private byte __hudHelpDefaultColor_green__;
            /// <summary>
            /// 25. Byte :: Offset: 0xA6 (166)
            /// </summary>
            public byte HudHelpDefaultColor_red { get { return __hudHelpDefaultColor_red__; } set { __hudHelpDefaultColor_red__ = value; } }
            private byte __hudHelpDefaultColor_red__;
            /// <summary>
            /// 26. Byte :: Offset: 0xA7 (167)
            /// </summary>
            public byte HudHelpDefaultColor_alpha { get { return __hudHelpDefaultColor_alpha__; } set { __hudHelpDefaultColor_alpha__ = value; } }
            private byte __hudHelpDefaultColor_alpha__;
            /// <summary>
            /// 27. Byte :: Offset: 0xA8 (168)
            /// </summary>
            public byte HudHelpFlashingColor_blue { get { return __hudHelpFlashingColor_blue__; } set { __hudHelpFlashingColor_blue__ = value; } }
            private byte __hudHelpFlashingColor_blue__;
            /// <summary>
            /// 28. Byte :: Offset: 0xA9 (169)
            /// </summary>
            public byte HudHelpFlashingColor_green { get { return __hudHelpFlashingColor_green__; } set { __hudHelpFlashingColor_green__ = value; } }
            private byte __hudHelpFlashingColor_green__;
            /// <summary>
            /// 29. Byte :: Offset: 0xAA (170)
            /// </summary>
            public byte HudHelpFlashingColor_red { get { return __hudHelpFlashingColor_red__; } set { __hudHelpFlashingColor_red__ = value; } }
            private byte __hudHelpFlashingColor_red__;
            /// <summary>
            /// 30. Byte :: Offset: 0xAB (171)
            /// </summary>
            public byte HudHelpFlashingColor_alpha { get { return __hudHelpFlashingColor_alpha__; } set { __hudHelpFlashingColor_alpha__ = value; } }
            private byte __hudHelpFlashingColor_alpha__;
            /// <summary>
            /// 31. Float :: Offset: 0xAC (172)
            /// </summary>
            public float HudHelpFlashPeriod { get { return __hudHelpFlashPeriod__; } set { __hudHelpFlashPeriod__ = value; } }
            private float __hudHelpFlashPeriod__;
            /// <summary>
            /// 32. Float :: Offset: 0xB0 (176)
            /// </summary>
            public float HudHelpFlashDelay { get { return __hudHelpFlashDelay__; } set { __hudHelpFlashDelay__ = value; } }
            private float __hudHelpFlashDelay__;
            /// <summary>
            /// 33. Int16 :: Offset: 0xB4 (180)
            /// </summary>
            public short NumberOfFlashes1 { get { return __numberOfFlashes1__; } set { __numberOfFlashes1__ = value; } }
            private short __numberOfFlashes1__;
            /// <summary>
            /// 34. Bitmask16 :: Offset: 0xB6 (182)
            /// </summary>
            public HudHelpFlashFlagsBitmask16 HudHelpFlashFlags { get { return __hudHelpFlashFlags__; } set { __hudHelpFlashFlags__ = value; } }
            private HudHelpFlashFlagsBitmask16 __hudHelpFlashFlags__;
            public class HudHelpFlashFlagsBitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool ReverseDefault_FlashingColors { get { return __reverseDefault_FlashingColors__; } set { __reverseDefault_FlashingColors__ = value; } }
                private bool __reverseDefault_FlashingColors__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    ReverseDefault_FlashingColors = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 35. Float :: Offset: 0xB8 (184)
            /// </summary>
            public float HudHelpFlashLength { get { return __hudHelpFlashLength__; } set { __hudHelpFlashLength__ = value; } }
            private float __hudHelpFlashLength__;
            /// <summary>
            /// 36. ByteArray :: Offset: 0xBC (188)
            /// </summary>
            public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
            private byte[] __unused7__;
            /// <summary>
            /// 37. ByteArray :: Offset: 0xC4 (196)
            /// </summary>
            public byte[] Unused8 { get { return __unused8__; } set { __unused8__ = value; } }
            private byte[] __unused8__;
            /// <summary>
            /// 38. ByteArray :: Offset: 0xCC (204)
            /// </summary>
            public byte[] Unused9 { get { return __unused9__; } set { __unused9__ = value; } }
            private byte[] __unused9__;
            /// <summary>
            /// 39. Float :: Offset: 0xEC (236)
            /// </summary>
            public float WaypointOffset_top { get { return __waypointOffset_top__; } set { __waypointOffset_top__ = value; } }
            private float __waypointOffset_top__;
            /// <summary>
            /// 40. Float :: Offset: 0xF0 (240)
            /// </summary>
            public float WaypointOffset_bottom { get { return __waypointOffset_bottom__; } set { __waypointOffset_bottom__ = value; } }
            private float __waypointOffset_bottom__;
            /// <summary>
            /// 41. Float :: Offset: 0xF4 (244)
            /// </summary>
            public float WaypointOffset_left { get { return __waypointOffset_left__; } set { __waypointOffset_left__ = value; } }
            private float __waypointOffset_left__;
            /// <summary>
            /// 42. Float :: Offset: 0xF8 (248)
            /// </summary>
            public float WaypointOffset_right { get { return __waypointOffset_right__; } set { __waypointOffset_right__ = value; } }
            private float __waypointOffset_right__;
            /// <summary>
            /// 43. ByteArray :: Offset: 0xFC (252)
            /// </summary>
            public byte[] Unused10 { get { return __unused10__; } set { __unused10__ = value; } }
            private byte[] __unused10__;
            /// <summary>
            /// 44. Tag :: Offset: 0x11C (284)
            /// </summary>
            public TagRef WaypointBitmap { get { return __waypointBitmap__; } set { __waypointBitmap__ = value; } }
            private TagRef __waypointBitmap__;
            /// <summary>
            /// 45. TagBlock :: Offset: 0x124 (292)
            /// </summary>
            public _WaypointsTagBlock Waypoints { get { return __waypoints__; } set { __waypoints__ = value; } }
            private _WaypointsTagBlock __waypoints__;
            public class _WaypointsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. String32 :: Offset: 0x0 (0)
                /// </summary>
                public string Name { get { return __name__; } set { __name__ = value; } }
                private string __name__;
                /// <summary>
                /// 2. ByteArray :: Offset: 0x20 (32)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 3. Byte :: Offset: 0x28 (40)
                /// </summary>
                public byte Blue { get { return __blue__; } set { __blue__ = value; } }
                private byte __blue__;
                /// <summary>
                /// 4. Byte :: Offset: 0x29 (41)
                /// </summary>
                public byte Green { get { return __green__; } set { __green__ = value; } }
                private byte __green__;
                /// <summary>
                /// 5. Byte :: Offset: 0x2A (42)
                /// </summary>
                public byte Red { get { return __red__; } set { __red__ = value; } }
                private byte __red__;
                /// <summary>
                /// 6. Byte :: Offset: 0x2B (43)
                /// </summary>
                public byte Translucency { get { return __translucency__; } set { __translucency__ = value; } }
                private byte __translucency__;
                /// <summary>
                /// 7. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Opacity { get { return __opacity__; } set { __opacity__ = value; } }
                private float __opacity__;
                /// <summary>
                /// 8. ByteArray :: Offset: 0x30 (48)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 9. Int16 :: Offset: 0x34 (52)
                /// </summary>
                public short OnScreenSequenceIndex { get { return __onScreenSequenceIndex__; } set { __onScreenSequenceIndex__ = value; } }
                private short __onScreenSequenceIndex__;
                /// <summary>
                /// 10. Int16 :: Offset: 0x36 (54)
                /// </summary>
                public short OffScreenSequenceIndex { get { return __offScreenSequenceIndex__; } set { __offScreenSequenceIndex__ = value; } }
                private short __offScreenSequenceIndex__;
                /// <summary>
                /// 11. Int16 :: Offset: 0x38 (56)
                /// </summary>
                public short UnusedSequenceIndex { get { return __unusedSequenceIndex__; } set { __unusedSequenceIndex__ = value; } }
                private short __unusedSequenceIndex__;
                /// <summary>
                /// 12. Bitmask16 :: Offset: 0x3A (58)
                /// </summary>
                public FlagsBitmask16 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask16 __flags__;
                public class FlagsBitmask16
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool DoNotRotateWhenPointingOffScreen { get { return __doNotRotateWhenPointingOffScreen__; } set { __doNotRotateWhenPointingOffScreen__ = value; } }
                    private bool __doNotRotateWhenPointingOffScreen__;

                    public void _Initialize(BitReader br)
                    {
                        ushort mask = br.ReadUInt16();
                        ushort bit = 1;

                        DoNotRotateWhenPointingOffScreen = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 13. ByteArray :: Offset: 0x3C (60)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WaypointsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WaypointsTagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask16();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WaypointsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WaypointsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 46. ByteArray :: Offset: 0x12C (300)
            /// </summary>
            public byte[] Unused11 { get { return __unused11__; } set { __unused11__ = value; } }
            private byte[] __unused11__;
            /// <summary>
            /// 47. Float :: Offset: 0x17C (380)
            /// </summary>
            public float MultiplayerHudScale { get { return __multiplayerHudScale__; } set { __multiplayerHudScale__ = value; } }
            private float __multiplayerHudScale__;
            /// <summary>
            /// 48. ByteArray :: Offset: 0x180 (384)
            /// </summary>
            public byte[] Unused12 { get { return __unused12__; } set { __unused12__ = value; } }
            private byte[] __unused12__;
            /// <summary>
            /// 49. Float :: Offset: 0x290 (656)
            /// </summary>
            public float MotionSensorRange { get { return __motionSensorRange__; } set { __motionSensorRange__ = value; } }
            private float __motionSensorRange__;
            /// <summary>
            /// 50. Float :: Offset: 0x294 (660)
            /// </summary>
            public float MotionSensorVelocitySensitivity { get { return __motionSensorVelocitySensitivity__; } set { __motionSensorVelocitySensitivity__ = value; } }
            private float __motionSensorVelocitySensitivity__;
            /// <summary>
            /// 51. Float :: Offset: 0x298 (664)
            /// </summary>
            public float MotionSensorScale { get { return __motionSensorScale__; } set { __motionSensorScale__ = value; } }
            private float __motionSensorScale__;
            /// <summary>
            /// 52. Int16 :: Offset: 0x29C (668)
            /// </summary>
            public short DefaultTitleChapterBounds_top { get { return __defaultTitleChapterBounds_top__; } set { __defaultTitleChapterBounds_top__ = value; } }
            private short __defaultTitleChapterBounds_top__;
            /// <summary>
            /// 53. Int16 :: Offset: 0x29E (670)
            /// </summary>
            public short DefaultTitleChapterBounds_left { get { return __defaultTitleChapterBounds_left__; } set { __defaultTitleChapterBounds_left__ = value; } }
            private short __defaultTitleChapterBounds_left__;
            /// <summary>
            /// 54. Int16 :: Offset: 0x2A0 (672)
            /// </summary>
            public short DefaultTitleChapterBounds_bottom { get { return __defaultTitleChapterBounds_bottom__; } set { __defaultTitleChapterBounds_bottom__ = value; } }
            private short __defaultTitleChapterBounds_bottom__;
            /// <summary>
            /// 55. Int16 :: Offset: 0x2A2 (674)
            /// </summary>
            public short DefaultTitleChapterBounds_right { get { return __defaultTitleChapterBounds_right__; } set { __defaultTitleChapterBounds_right__ = value; } }
            private short __defaultTitleChapterBounds_right__;
            /// <summary>
            /// 56. ByteArray :: Offset: 0x2A4 (676)
            /// </summary>
            public byte[] Unused13 { get { return __unused13__; } set { __unused13__ = value; } }
            private byte[] __unused13__;
            /// <summary>
            /// 57. Int16 :: Offset: 0x2D0 (720)
            /// </summary>
            public short HudDamageIndicatorOffset_top { get { return __hudDamageIndicatorOffset_top__; } set { __hudDamageIndicatorOffset_top__ = value; } }
            private short __hudDamageIndicatorOffset_top__;
            /// <summary>
            /// 58. Int16 :: Offset: 0x2D2 (722)
            /// </summary>
            public short HudDamageIndicatorOffset_bottom { get { return __hudDamageIndicatorOffset_bottom__; } set { __hudDamageIndicatorOffset_bottom__ = value; } }
            private short __hudDamageIndicatorOffset_bottom__;
            /// <summary>
            /// 59. Int16 :: Offset: 0x2D4 (724)
            /// </summary>
            public short HudDamageIndicatorOffset_left { get { return __hudDamageIndicatorOffset_left__; } set { __hudDamageIndicatorOffset_left__ = value; } }
            private short __hudDamageIndicatorOffset_left__;
            /// <summary>
            /// 60. Int16 :: Offset: 0x2D6 (726)
            /// </summary>
            public short HudDamageIndicatorOffset_right { get { return __hudDamageIndicatorOffset_right__; } set { __hudDamageIndicatorOffset_right__ = value; } }
            private short __hudDamageIndicatorOffset_right__;
            /// <summary>
            /// 61. ByteArray :: Offset: 0x2D8 (728)
            /// </summary>
            public byte[] Unused14 { get { return __unused14__; } set { __unused14__ = value; } }
            private byte[] __unused14__;
            /// <summary>
            /// 62. Tag :: Offset: 0x2F8 (760)
            /// </summary>
            public TagRef DamageIndicatorBitmap { get { return __damageIndicatorBitmap__; } set { __damageIndicatorBitmap__ = value; } }
            private TagRef __damageIndicatorBitmap__;
            /// <summary>
            /// 63. Int16 :: Offset: 0x300 (768)
            /// </summary>
            public short SequenceIndex { get { return __sequenceIndex__; } set { __sequenceIndex__ = value; } }
            private short __sequenceIndex__;
            /// <summary>
            /// 64. Int16 :: Offset: 0x300 (768)
            /// </summary>
            public short MultiplayerSequenceIndex { get { return __multiplayerSequenceIndex__; } set { __multiplayerSequenceIndex__ = value; } }
            private short __multiplayerSequenceIndex__;
            /// <summary>
            /// 65. ByteArray :: Offset: 0x302 (770)
            /// </summary>
            public byte[] Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private byte[] __unknown1__;
            /// <summary>
            /// 66. Byte :: Offset: 0x304 (772)
            /// </summary>
            public byte DamageIndicatorColor_blue { get { return __damageIndicatorColor_blue__; } set { __damageIndicatorColor_blue__ = value; } }
            private byte __damageIndicatorColor_blue__;
            /// <summary>
            /// 67. Byte :: Offset: 0x305 (773)
            /// </summary>
            public byte DamageIndicatorColor_green { get { return __damageIndicatorColor_green__; } set { __damageIndicatorColor_green__ = value; } }
            private byte __damageIndicatorColor_green__;
            /// <summary>
            /// 68. Byte :: Offset: 0x306 (774)
            /// </summary>
            public byte DamageIndicatorColor_red { get { return __damageIndicatorColor_red__; } set { __damageIndicatorColor_red__ = value; } }
            private byte __damageIndicatorColor_red__;
            /// <summary>
            /// 69. Byte :: Offset: 0x307 (775)
            /// </summary>
            public byte DamageIndicatorColor_alpha { get { return __damageIndicatorColor_alpha__; } set { __damageIndicatorColor_alpha__ = value; } }
            private byte __damageIndicatorColor_alpha__;
            /// <summary>
            /// 70. ByteArray :: Offset: 0x308 (776)
            /// </summary>
            public byte[] Unused15 { get { return __unused15__; } set { __unused15__ = value; } }
            private byte[] __unused15__;
            /// <summary>
            /// 71. Byte :: Offset: 0x318 (792)
            /// </summary>
            public byte NotMuchTimeLeftDefaultColor_blue { get { return __notMuchTimeLeftDefaultColor_blue__; } set { __notMuchTimeLeftDefaultColor_blue__ = value; } }
            private byte __notMuchTimeLeftDefaultColor_blue__;
            /// <summary>
            /// 72. Byte :: Offset: 0x319 (793)
            /// </summary>
            public byte NotMuchTimeLeftDefaultColor_green { get { return __notMuchTimeLeftDefaultColor_green__; } set { __notMuchTimeLeftDefaultColor_green__ = value; } }
            private byte __notMuchTimeLeftDefaultColor_green__;
            /// <summary>
            /// 73. Byte :: Offset: 0x31A (794)
            /// </summary>
            public byte NotMuchTimeLeftDefaultColor_red { get { return __notMuchTimeLeftDefaultColor_red__; } set { __notMuchTimeLeftDefaultColor_red__ = value; } }
            private byte __notMuchTimeLeftDefaultColor_red__;
            /// <summary>
            /// 74. Byte :: Offset: 0x31B (795)
            /// </summary>
            public byte NotMuchTimeLeftDefaultColor_alpha { get { return __notMuchTimeLeftDefaultColor_alpha__; } set { __notMuchTimeLeftDefaultColor_alpha__ = value; } }
            private byte __notMuchTimeLeftDefaultColor_alpha__;
            /// <summary>
            /// 75. Byte :: Offset: 0x31C (796)
            /// </summary>
            public byte NotMuchTimeLeftFlashColor_blue { get { return __notMuchTimeLeftFlashColor_blue__; } set { __notMuchTimeLeftFlashColor_blue__ = value; } }
            private byte __notMuchTimeLeftFlashColor_blue__;
            /// <summary>
            /// 76. Byte :: Offset: 0x31D (797)
            /// </summary>
            public byte NotMuchTimeLeftFlash_green { get { return __notMuchTimeLeftFlash_green__; } set { __notMuchTimeLeftFlash_green__ = value; } }
            private byte __notMuchTimeLeftFlash_green__;
            /// <summary>
            /// 77. Byte :: Offset: 0x31E (798)
            /// </summary>
            public byte NotMuchTimeLeftFlash_red { get { return __notMuchTimeLeftFlash_red__; } set { __notMuchTimeLeftFlash_red__ = value; } }
            private byte __notMuchTimeLeftFlash_red__;
            /// <summary>
            /// 78. Byte :: Offset: 0x31F (799)
            /// </summary>
            public byte NotMuchTimeLeftFlash_alpha { get { return __notMuchTimeLeftFlash_alpha__; } set { __notMuchTimeLeftFlash_alpha__ = value; } }
            private byte __notMuchTimeLeftFlash_alpha__;
            /// <summary>
            /// 79. Float :: Offset: 0x320 (800)
            /// </summary>
            public float NotMuchTimeLeftFlashPeriod { get { return __notMuchTimeLeftFlashPeriod__; } set { __notMuchTimeLeftFlashPeriod__ = value; } }
            private float __notMuchTimeLeftFlashPeriod__;
            /// <summary>
            /// 80. Float :: Offset: 0x324 (804)
            /// </summary>
            public float NotMuchTimeLeftFlashDelay { get { return __notMuchTimeLeftFlashDelay__; } set { __notMuchTimeLeftFlashDelay__ = value; } }
            private float __notMuchTimeLeftFlashDelay__;
            /// <summary>
            /// 81. Int16 :: Offset: 0x328 (808)
            /// </summary>
            public short NumberOfFlashes2 { get { return __numberOfFlashes2__; } set { __numberOfFlashes2__ = value; } }
            private short __numberOfFlashes2__;
            /// <summary>
            /// 82. Bitmask16 :: Offset: 0x32A (810)
            /// </summary>
            public NotMuchTimeLeftFlashFlagsBitmask16 NotMuchTimeLeftFlashFlags { get { return __notMuchTimeLeftFlashFlags__; } set { __notMuchTimeLeftFlashFlags__ = value; } }
            private NotMuchTimeLeftFlashFlagsBitmask16 __notMuchTimeLeftFlashFlags__;
            public class NotMuchTimeLeftFlashFlagsBitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool ReverseDefault_FlashingColors { get { return __reverseDefault_FlashingColors__; } set { __reverseDefault_FlashingColors__ = value; } }
                private bool __reverseDefault_FlashingColors__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    ReverseDefault_FlashingColors = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 83. Float :: Offset: 0x32C (812)
            /// </summary>
            public float NotMuchTimeLeftFlashLength { get { return __notMuchTimeLeftFlashLength__; } set { __notMuchTimeLeftFlashLength__ = value; } }
            private float __notMuchTimeLeftFlashLength__;
            /// <summary>
            /// 84. ByteArray :: Offset: 0x330 (816)
            /// </summary>
            public byte[] Unused16 { get { return __unused16__; } set { __unused16__ = value; } }
            private byte[] __unused16__;
            /// <summary>
            /// 85. Byte :: Offset: 0x338 (824)
            /// </summary>
            public byte TimeOutDefaultColor_blue { get { return __timeOutDefaultColor_blue__; } set { __timeOutDefaultColor_blue__ = value; } }
            private byte __timeOutDefaultColor_blue__;
            /// <summary>
            /// 86. Byte :: Offset: 0x339 (825)
            /// </summary>
            public byte TimeOutDefaultColor_green { get { return __timeOutDefaultColor_green__; } set { __timeOutDefaultColor_green__ = value; } }
            private byte __timeOutDefaultColor_green__;
            /// <summary>
            /// 87. Byte :: Offset: 0x33A (826)
            /// </summary>
            public byte TimeOutDefaultColor_red { get { return __timeOutDefaultColor_red__; } set { __timeOutDefaultColor_red__ = value; } }
            private byte __timeOutDefaultColor_red__;
            /// <summary>
            /// 88. Byte :: Offset: 0x33B (827)
            /// </summary>
            public byte TimeOutDefaultColor_alpha { get { return __timeOutDefaultColor_alpha__; } set { __timeOutDefaultColor_alpha__ = value; } }
            private byte __timeOutDefaultColor_alpha__;
            /// <summary>
            /// 89. Byte :: Offset: 0x33C (828)
            /// </summary>
            public byte TimeOutFlashingColor_blue { get { return __timeOutFlashingColor_blue__; } set { __timeOutFlashingColor_blue__ = value; } }
            private byte __timeOutFlashingColor_blue__;
            /// <summary>
            /// 90. Byte :: Offset: 0x33D (829)
            /// </summary>
            public byte TimeOutFlashingColor_green { get { return __timeOutFlashingColor_green__; } set { __timeOutFlashingColor_green__ = value; } }
            private byte __timeOutFlashingColor_green__;
            /// <summary>
            /// 91. Byte :: Offset: 0x33E (830)
            /// </summary>
            public byte TimeOutFlashingColor_red { get { return __timeOutFlashingColor_red__; } set { __timeOutFlashingColor_red__ = value; } }
            private byte __timeOutFlashingColor_red__;
            /// <summary>
            /// 92. Byte :: Offset: 0x33F (831)
            /// </summary>
            public byte TimeOutFlashingColor_alpha { get { return __timeOutFlashingColor_alpha__; } set { __timeOutFlashingColor_alpha__ = value; } }
            private byte __timeOutFlashingColor_alpha__;
            /// <summary>
            /// 93. Float :: Offset: 0x340 (832)
            /// </summary>
            public float TimeOutFlashPeriod { get { return __timeOutFlashPeriod__; } set { __timeOutFlashPeriod__ = value; } }
            private float __timeOutFlashPeriod__;
            /// <summary>
            /// 94. Float :: Offset: 0x344 (836)
            /// </summary>
            public float TimeOutFlashDelay { get { return __timeOutFlashDelay__; } set { __timeOutFlashDelay__ = value; } }
            private float __timeOutFlashDelay__;
            /// <summary>
            /// 95. Int16 :: Offset: 0x348 (840)
            /// </summary>
            public short NumberOfFlashes3 { get { return __numberOfFlashes3__; } set { __numberOfFlashes3__ = value; } }
            private short __numberOfFlashes3__;
            /// <summary>
            /// 96. Bitmask16 :: Offset: 0x34A (842)
            /// </summary>
            public TimeOutFlashFlagsBitmask16 TimeOutFlashFlags { get { return __timeOutFlashFlags__; } set { __timeOutFlashFlags__ = value; } }
            private TimeOutFlashFlagsBitmask16 __timeOutFlashFlags__;
            public class TimeOutFlashFlagsBitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool ReverseDefault_FlashingColors { get { return __reverseDefault_FlashingColors__; } set { __reverseDefault_FlashingColors__ = value; } }
                private bool __reverseDefault_FlashingColors__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    ReverseDefault_FlashingColors = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 97. Float :: Offset: 0x34C (844)
            /// </summary>
            public float TimeOutFlashLength { get { return __timeOutFlashLength__; } set { __timeOutFlashLength__ = value; } }
            private float __timeOutFlashLength__;
            /// <summary>
            /// 98. ByteArray :: Offset: 0x350 (848)
            /// </summary>
            public byte[] Unused17 { get { return __unused17__; } set { __unused17__ = value; } }
            private byte[] __unused17__;
            /// <summary>
            /// 99. ByteArray :: Offset: 0x380 (896)
            /// </summary>
            public byte[] Unused18 { get { return __unused18__; } set { __unused18__ = value; } }
            private byte[] __unused18__;
            /// <summary>
            /// 100. Int16 :: Offset: 0x388 (904)
            /// </summary>
            public short LoadingBeginText { get { return __loadingBeginText__; } set { __loadingBeginText__ = value; } }
            private short __loadingBeginText__;
            /// <summary>
            /// 101. Int16 :: Offset: 0x38A (906)
            /// </summary>
            public short LoadingEndText { get { return __loadingEndText__; } set { __loadingEndText__ = value; } }
            private short __loadingEndText__;
            /// <summary>
            /// 102. Int16 :: Offset: 0x38C (908)
            /// </summary>
            public short CheckpointBeginText { get { return __checkpointBeginText__; } set { __checkpointBeginText__ = value; } }
            private short __checkpointBeginText__;
            /// <summary>
            /// 103. Int16 :: Offset: 0x38E (910)
            /// </summary>
            public short CheckpointEndText { get { return __checkpointEndText__; } set { __checkpointEndText__ = value; } }
            private short __checkpointEndText__;
            /// <summary>
            /// 104. ByteArray :: Offset: 0x390 (912)
            /// </summary>
            public byte[] Unused19 { get { return __unused19__; } set { __unused19__ = value; } }
            private byte[] __unused19__;
            /// <summary>
            /// 105. ByteArray :: Offset: 0x398 (920)
            /// </summary>
            public byte[] Unused20 { get { return __unused20__; } set { __unused20__ = value; } }
            private byte[] __unused20__;
            /// <summary>
            /// 106. Tag :: Offset: 0x3F8 (1016)
            /// </summary>
            public TagRef HudMessages { get { return __hudMessages__; } set { __hudMessages__ = value; } }
            private TagRef __hudMessages__;
            /// <summary>
            /// 107. TagBlock :: Offset: 0x400 (1024)
            /// </summary>
            public _Unknown2TagBlock Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private _Unknown2TagBlock __unknown2__;
            public class _Unknown2TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private TagRef __unknown1__;
                /// <summary>
                /// 2. Tag :: Offset: 0x8 (8)
                /// </summary>
                public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private TagRef __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Tag :: Offset: 0x14 (20)
                /// </summary>
                public TagRef Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private TagRef __unknown4__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown2TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown2TagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown2TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown2TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 108. TagBlock :: Offset: 0x408 (1032)
            /// </summary>
            public _Unknown3TagBlock Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private _Unknown3TagBlock __unknown3__;
            public class _Unknown3TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private TagRef __unknown1__;
                /// <summary>
                /// 2. Tag :: Offset: 0x8 (8)
                /// </summary>
                public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private TagRef __unknown2__;
                /// <summary>
                /// 3. ByteArray :: Offset: 0x10 (16)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 4. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Tag :: Offset: 0x1C (28)
                /// </summary>
                public TagRef Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private TagRef __unknown5__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown3TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown3TagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown3TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown3TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 109. TagBlock :: Offset: 0x410 (1040)
            /// </summary>
            public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private _Unknown4TagBlock __unknown4__;
            public class _Unknown4TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private TagRef __unknown1__;
                /// <summary>
                /// 2. Tag :: Offset: 0x8 (8)
                /// </summary>
                public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private TagRef __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown4TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown4TagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown4TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown4TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 110. TagBlock :: Offset: 0x418 (1048)
            /// </summary>
            public _Unknown5TagBlock Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private _Unknown5TagBlock __unknown5__;
            public class _Unknown5TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private TagRef __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. ByteArray :: Offset: 0xC (12)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 4. Tag :: Offset: 0x10 (16)
                /// </summary>
                public TagRef Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private TagRef __unknown3__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown5TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown5TagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown5TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown5TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 111. TagBlock :: Offset: 0x420 (1056)
            /// </summary>
            public _Unknown6TagBlock Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private _Unknown6TagBlock __unknown6__;
            public class _Unknown6TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HudgLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HudgLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private StringID __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown6TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown6TagBlock(HudgLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown6TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown6TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 112. Tag :: Offset: 0x428 (1064)
            /// </summary>
            public TagRef PrimaryMessageSound { get { return __primaryMessageSound__; } set { __primaryMessageSound__ = value; } }
            private TagRef __primaryMessageSound__;
            /// <summary>
            /// 113. Tag :: Offset: 0x430 (1072)
            /// </summary>
            public TagRef SecondaryMessageSound { get { return __secondaryMessageSound__; } set { __secondaryMessageSound__ = value; } }
            private TagRef __secondaryMessageSound__;
            /// <summary>
            /// 114. StringID :: Offset: 0x438 (1080)
            /// </summary>
            public StringID Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private StringID __unknown7__;
            /// <summary>
            /// 115. StringID :: Offset: 0x43C (1084)
            /// </summary>
            public StringID Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private StringID __unknown8__;
            /// <summary>
            /// 116. Tag :: Offset: 0x440 (1088)
            /// </summary>
            public TagRef TrainingShader { get { return __trainingShader__; } set { __trainingShader__ = value; } }
            private TagRef __trainingShader__;
            /// <summary>
            /// 117. Tag :: Offset: 0x448 (1096)
            /// </summary>
            public TagRef HumanTopRightTrainingBitmap { get { return __humanTopRightTrainingBitmap__; } set { __humanTopRightTrainingBitmap__ = value; } }
            private TagRef __humanTopRightTrainingBitmap__;
            /// <summary>
            /// 118. Tag :: Offset: 0x450 (1104)
            /// </summary>
            public TagRef HumanTopCenterTrainingBitmap { get { return __humanTopCenterTrainingBitmap__; } set { __humanTopCenterTrainingBitmap__ = value; } }
            private TagRef __humanTopCenterTrainingBitmap__;
            /// <summary>
            /// 119. Tag :: Offset: 0x458 (1112)
            /// </summary>
            public TagRef HumanTopLeftTrainingBitmap { get { return __humanTopLeftTrainingBitmap__; } set { __humanTopLeftTrainingBitmap__ = value; } }
            private TagRef __humanTopLeftTrainingBitmap__;
            /// <summary>
            /// 120. Tag :: Offset: 0x460 (1120)
            /// </summary>
            public TagRef HumanCenterTrainingBitmap { get { return __humanCenterTrainingBitmap__; } set { __humanCenterTrainingBitmap__ = value; } }
            private TagRef __humanCenterTrainingBitmap__;
            /// <summary>
            /// 121. Tag :: Offset: 0x468 (1128)
            /// </summary>
            public TagRef CovenantTopRightTrainingBitmap { get { return __covenantTopRightTrainingBitmap__; } set { __covenantTopRightTrainingBitmap__ = value; } }
            private TagRef __covenantTopRightTrainingBitmap__;
            /// <summary>
            /// 122. Tag :: Offset: 0x470 (1136)
            /// </summary>
            public TagRef CovenantTopCenterTrainingBitmap { get { return __covenantTopCenterTrainingBitmap__; } set { __covenantTopCenterTrainingBitmap__ = value; } }
            private TagRef __covenantTopCenterTrainingBitmap__;
            /// <summary>
            /// 123. Tag :: Offset: 0x478 (1144)
            /// </summary>
            public TagRef CovenantTopLeftTrainingBitmap { get { return __covenantTopLeftTrainingBitmap__; } set { __covenantTopLeftTrainingBitmap__ = value; } }
            private TagRef __covenantTopLeftTrainingBitmap__;
            /// <summary>
            /// 124. Tag :: Offset: 0x480 (1152)
            /// </summary>
            public TagRef CovenantCenterTrainingBitmap { get { return __covenantCenterTrainingBitmap__; } set { __covenantCenterTrainingBitmap__ = value; } }
            private TagRef __covenantCenterTrainingBitmap__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the HudgLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public HudgLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                MessageAnchor = new MessageAnchorEnum16();
                ButtonIcons = new _ButtonIconsTagBlock(this);
                HudHelpFlashFlags = new HudHelpFlashFlagsBitmask16();
                Waypoints = new _WaypointsTagBlock(this);
                NotMuchTimeLeftFlashFlags = new NotMuchTimeLeftFlashFlagsBitmask16();
                TimeOutFlashFlags = new TimeOutFlashFlagsBitmask16();
                Unknown2 = new _Unknown2TagBlock(this);
                Unknown3 = new _Unknown3TagBlock(this);
                Unknown4 = new _Unknown4TagBlock(this);
                Unknown5 = new _Unknown5TagBlock(this);
                Unknown6 = new _Unknown6TagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unused1 = br.ReadBytes(36);
                MessageAnchor._Initialize(br);
                AnchorOffset_y = br.ReadByte();
                AnchorOffset_x = br.ReadByte();
                Unused2 = br.ReadBytes(32);
                Unused3._Initialize(br);
                Unused4._Initialize(br);
                MessageUptime = br.ReadSingle();
                MessageFadeTime = br.ReadSingle();
                IconColor_alpha = br.ReadSingle();
                IconColor_red = br.ReadSingle();
                IconColor_green = br.ReadSingle();
                IconColor_blue = br.ReadSingle();
                TextColor_alpha = br.ReadSingle();
                TextColor_red = br.ReadSingle();
                TextColor_green = br.ReadSingle();
                TextColor_blue = br.ReadSingle();
                TextSpacing = br.ReadSingle();
                Unused5._Initialize(br);
                IconBitmap._Initialize(br);
                Unused6._Initialize(br);
                ButtonIcons._Initialize(br);
                HudHelpDefaultColor_blue = br.ReadByte();
                HudHelpDefaultColor_green = br.ReadByte();
                HudHelpDefaultColor_red = br.ReadByte();
                HudHelpDefaultColor_alpha = br.ReadByte();
                HudHelpFlashingColor_blue = br.ReadByte();
                HudHelpFlashingColor_green = br.ReadByte();
                HudHelpFlashingColor_red = br.ReadByte();
                HudHelpFlashingColor_alpha = br.ReadByte();
                HudHelpFlashPeriod = br.ReadSingle();
                HudHelpFlashDelay = br.ReadSingle();
                NumberOfFlashes1 = br.ReadInt16();
                HudHelpFlashFlags._Initialize(br);
                HudHelpFlashLength = br.ReadSingle();
                Unused7 = br.ReadBytes(8);
                Unused8 = br.ReadBytes(8);
                Unused9 = br.ReadBytes(32);
                WaypointOffset_top = br.ReadSingle();
                WaypointOffset_bottom = br.ReadSingle();
                WaypointOffset_left = br.ReadSingle();
                WaypointOffset_right = br.ReadSingle();
                Unused10 = br.ReadBytes(32);
                WaypointBitmap._Initialize(br);
                Waypoints._Initialize(br);
                Unused11 = br.ReadBytes(80);
                MultiplayerHudScale = br.ReadSingle();
                Unused12 = br.ReadBytes(272);
                MotionSensorRange = br.ReadSingle();
                MotionSensorVelocitySensitivity = br.ReadSingle();
                MotionSensorScale = br.ReadSingle();
                DefaultTitleChapterBounds_top = br.ReadInt16();
                DefaultTitleChapterBounds_left = br.ReadInt16();
                DefaultTitleChapterBounds_bottom = br.ReadInt16();
                DefaultTitleChapterBounds_right = br.ReadInt16();
                Unused13 = br.ReadBytes(44);
                HudDamageIndicatorOffset_top = br.ReadInt16();
                HudDamageIndicatorOffset_bottom = br.ReadInt16();
                HudDamageIndicatorOffset_left = br.ReadInt16();
                HudDamageIndicatorOffset_right = br.ReadInt16();
                Unused14 = br.ReadBytes(32);
                DamageIndicatorBitmap._Initialize(br);
                SequenceIndex = br.ReadInt16();
                MultiplayerSequenceIndex = br.ReadInt16();
                Unknown1 = br.ReadBytes(2);
                DamageIndicatorColor_blue = br.ReadByte();
                DamageIndicatorColor_green = br.ReadByte();
                DamageIndicatorColor_red = br.ReadByte();
                DamageIndicatorColor_alpha = br.ReadByte();
                Unused15 = br.ReadBytes(16);
                NotMuchTimeLeftDefaultColor_blue = br.ReadByte();
                NotMuchTimeLeftDefaultColor_green = br.ReadByte();
                NotMuchTimeLeftDefaultColor_red = br.ReadByte();
                NotMuchTimeLeftDefaultColor_alpha = br.ReadByte();
                NotMuchTimeLeftFlashColor_blue = br.ReadByte();
                NotMuchTimeLeftFlash_green = br.ReadByte();
                NotMuchTimeLeftFlash_red = br.ReadByte();
                NotMuchTimeLeftFlash_alpha = br.ReadByte();
                NotMuchTimeLeftFlashPeriod = br.ReadSingle();
                NotMuchTimeLeftFlashDelay = br.ReadSingle();
                NumberOfFlashes2 = br.ReadInt16();
                NotMuchTimeLeftFlashFlags._Initialize(br);
                NotMuchTimeLeftFlashLength = br.ReadSingle();
                Unused16 = br.ReadBytes(8);
                TimeOutDefaultColor_blue = br.ReadByte();
                TimeOutDefaultColor_green = br.ReadByte();
                TimeOutDefaultColor_red = br.ReadByte();
                TimeOutDefaultColor_alpha = br.ReadByte();
                TimeOutFlashingColor_blue = br.ReadByte();
                TimeOutFlashingColor_green = br.ReadByte();
                TimeOutFlashingColor_red = br.ReadByte();
                TimeOutFlashingColor_alpha = br.ReadByte();
                TimeOutFlashPeriod = br.ReadSingle();
                TimeOutFlashDelay = br.ReadSingle();
                NumberOfFlashes3 = br.ReadInt16();
                TimeOutFlashFlags._Initialize(br);
                TimeOutFlashLength = br.ReadSingle();
                Unused17 = br.ReadBytes(48);
                Unused18 = br.ReadBytes(8);
                LoadingBeginText = br.ReadInt16();
                LoadingEndText = br.ReadInt16();
                CheckpointBeginText = br.ReadInt16();
                CheckpointEndText = br.ReadInt16();
                Unused19 = br.ReadBytes(8);
                Unused20 = br.ReadBytes(96);
                HudMessages._Initialize(br);
                Unknown2._Initialize(br);
                Unknown3._Initialize(br);
                Unknown4._Initialize(br);
                Unknown5._Initialize(br);
                Unknown6._Initialize(br);
                PrimaryMessageSound._Initialize(br);
                SecondaryMessageSound._Initialize(br);
                Unknown7._Initialize(br);
                Unknown8._Initialize(br);
                TrainingShader._Initialize(br);
                HumanTopRightTrainingBitmap._Initialize(br);
                HumanTopCenterTrainingBitmap._Initialize(br);
                HumanTopLeftTrainingBitmap._Initialize(br);
                HumanCenterTrainingBitmap._Initialize(br);
                CovenantTopRightTrainingBitmap._Initialize(br);
                CovenantTopCenterTrainingBitmap._Initialize(br);
                CovenantTopLeftTrainingBitmap._Initialize(br);
                CovenantCenterTrainingBitmap._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Hudg class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Hudg(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new HudgLayout(map, identity);
        }

        #region Specific Methods of hudg

        #endregion
    }
}
