using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Physics
    /// </summary>
    public class Phys
    {
        #region Constants

        /// <summary>
        /// Size of the phys meta header.
        /// </summary>
        public const int HeaderSize = 116;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private PhysLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v1.0 Shalted: Completed Layout.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public PhysLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class PhysLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Float :: Offset: 0x0 (0)
            /// </summary>
            public float Radius { get { return __radius__; } set { __radius__ = value; } }
            private float __radius__;
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float MomentScale { get { return __momentScale__; } set { __momentScale__ = value; } }
            private float __momentScale__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float Mass { get { return __mass__; } set { __mass__ = value; } }
            private float __mass__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float CenterOfMassX { get { return __centerOfMassX__; } set { __centerOfMassX__ = value; } }
            private float __centerOfMassX__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float CenterOfMassY { get { return __centerOfMassY__; } set { __centerOfMassY__ = value; } }
            private float __centerOfMassY__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float CenterOfMassZ { get { return __centerOfMassZ__; } set { __centerOfMassZ__ = value; } }
            private float __centerOfMassZ__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float Density { get { return __density__; } set { __density__ = value; } }
            private float __density__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float GravityScale { get { return __gravityScale__; } set { __gravityScale__ = value; } }
            private float __gravityScale__;
            /// <summary>
            /// 9. Float :: Offset: 0x20 (32)
            /// </summary>
            public float GroundFriction { get { return __groundFriction__; } set { __groundFriction__ = value; } }
            private float __groundFriction__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float GroundDepth { get { return __groundDepth__; } set { __groundDepth__ = value; } }
            private float __groundDepth__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float GroundDampFunction { get { return __groundDampFunction__; } set { __groundDampFunction__ = value; } }
            private float __groundDampFunction__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float NormalK1 { get { return __normalK1__; } set { __normalK1__ = value; } }
            private float __normalK1__;
            /// <summary>
            /// 13. Float :: Offset: 0x30 (48)
            /// </summary>
            public float NormalK2 { get { return __normalK2__; } set { __normalK2__ = value; } }
            private float __normalK2__;
            /// <summary>
            /// 14. ByteArray :: Offset: 0x34 (52)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 15. Float :: Offset: 0x38 (56)
            /// </summary>
            public float WaterFriction { get { return __waterFriction__; } set { __waterFriction__ = value; } }
            private float __waterFriction__;
            /// <summary>
            /// 16. Float :: Offset: 0x3C (60)
            /// </summary>
            public float WaterDepth { get { return __waterDepth__; } set { __waterDepth__ = value; } }
            private float __waterDepth__;
            /// <summary>
            /// 17. Float :: Offset: 0x40 (64)
            /// </summary>
            public float WaterDensity { get { return __waterDensity__; } set { __waterDensity__ = value; } }
            private float __waterDensity__;
            /// <summary>
            /// 18. ByteArray :: Offset: 0x44 (68)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 19. Float :: Offset: 0x48 (72)
            /// </summary>
            public float AirFriction { get { return __airFriction__; } set { __airFriction__ = value; } }
            private float __airFriction__;
            /// <summary>
            /// 20. ByteArray :: Offset: 0x4C (76)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;
            /// <summary>
            /// 21. Float :: Offset: 0x50 (80)
            /// </summary>
            public float XxMovement { get { return __xxMovement__; } set { __xxMovement__ = value; } }
            private float __xxMovement__;
            /// <summary>
            /// 22. Float :: Offset: 0x54 (84)
            /// </summary>
            public float YyMovement { get { return __yyMovement__; } set { __yyMovement__ = value; } }
            private float __yyMovement__;
            /// <summary>
            /// 23. Float :: Offset: 0x58 (88)
            /// </summary>
            public float ZzMovement { get { return __zzMovement__; } set { __zzMovement__ = value; } }
            private float __zzMovement__;
            /// <summary>
            /// 24. TagBlock :: Offset: 0x5C (92)
            /// </summary>
            public _InternalMatrixTagBlock InternalMatrix { get { return __internalMatrix__; } set { __internalMatrix__ = value; } }
            private _InternalMatrixTagBlock __internalMatrix__;
            public class _InternalMatrixTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public PhysLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private PhysLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float YyZz { get { return __yyZz__; } set { __yyZz__ = value; } }
                private float __yyZz__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Xy1 { get { return __xy1__; } set { __xy1__ = value; } }
                private float __xy1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Zx1 { get { return __zx1__; } set { __zx1__ = value; } }
                private float __zx1__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Xy2 { get { return __xy2__; } set { __xy2__ = value; } }
                private float __xy2__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float ZzXx { get { return __zzXx__; } set { __zzXx__ = value; } }
                private float __zzXx__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Yz1 { get { return __yz1__; } set { __yz1__ = value; } }
                private float __yz1__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Zx2 { get { return __zx2__; } set { __zx2__ = value; } }
                private float __zx2__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Yz2 { get { return __yz2__; } set { __yz2__ = value; } }
                private float __yz2__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float XxYy { get { return __xxYy__; } set { __xxYy__ = value; } }
                private float __xxYy__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _InternalMatrixTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _InternalMatrixTagBlock(PhysLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_InternalMatrixTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _InternalMatrixTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 25. TagBlock :: Offset: 0x64 (100)
            /// </summary>
            public _MassPoints1TagBlock MassPoints1 { get { return __massPoints1__; } set { __massPoints1__ = value; } }
            private _MassPoints1TagBlock __massPoints1__;
            public class _MassPoints1TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public PhysLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private PhysLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. String32 :: Offset: 0x0 (0)
                /// </summary>
                public string Name { get { return __name__; } set { __name__ = value; } }
                private string __name__;
                /// <summary>
                /// 2. Bitmask32 :: Offset: 0x20 (32)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool GroundFriction { get { return __groundFriction__; } set { __groundFriction__ = value; } }
                    private bool __groundFriction__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool WaterFriction { get { return __waterFriction__; } set { __waterFriction__ = value; } }
                    private bool __waterFriction__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool AirFriction { get { return __airFriction__; } set { __airFriction__ = value; } }
                    private bool __airFriction__;
                    /// <summary>
                    /// 4. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool WaterLift { get { return __waterLift__; } set { __waterLift__ = value; } }
                    private bool __waterLift__;
                    /// <summary>
                    /// 5. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool AirLift { get { return __airLift__; } set { __airLift__ = value; } }
                    private bool __airLift__;
                    /// <summary>
                    /// 6. Option :: Value: 0x5 (5)
                    /// </summary>
                    public bool Thrust { get { return __thrust__; } set { __thrust__ = value; } }
                    private bool __thrust__;
                    /// <summary>
                    /// 7. Option :: Value: 0x6 (6)
                    /// </summary>
                    public bool Antigrav { get { return __antigrav__; } set { __antigrav__ = value; } }
                    private bool __antigrav__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        GroundFriction = ((bit & mask) == bit);
                        bit <<= 1;
                        WaterFriction = ((bit & mask) == bit);
                        bit <<= 1;
                        AirFriction = ((bit & mask) == bit);
                        bit <<= 1;
                        WaterLift = ((bit & mask) == bit);
                        bit <<= 1;
                        AirLift = ((bit & mask) == bit);
                        bit <<= 1;
                        Thrust = ((bit & mask) == bit);
                        bit <<= 1;
                        Antigrav = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 3. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Strength { get { return __strength__; } set { __strength__ = value; } }
                private float __strength__;
                /// <summary>
                /// 4. Float :: Offset: 0x28 (40)
                /// </summary>
                public float OffsetReal { get { return __offsetReal__; } set { __offsetReal__ = value; } }
                private float __offsetReal__;
                /// <summary>
                /// 5. Float :: Offset: 0x2C (44)
                /// </summary>
                public float HeightReal { get { return __heightReal__; } set { __heightReal__ = value; } }
                private float __heightReal__;
                /// <summary>
                /// 6. Float :: Offset: 0x30 (48)
                /// </summary>
                public float DampFraction { get { return __dampFraction__; } set { __dampFraction__ = value; } }
                private float __dampFraction__;
                /// <summary>
                /// 7. Float :: Offset: 0x34 (52)
                /// </summary>
                public float NormalK1 { get { return __normalK1__; } set { __normalK1__ = value; } }
                private float __normalK1__;
                /// <summary>
                /// 8. Float :: Offset: 0x38 (56)
                /// </summary>
                public float NormalK2 { get { return __normalK2__; } set { __normalK2__ = value; } }
                private float __normalK2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _MassPoints1TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _MassPoints1TagBlock(PhysLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_MassPoints1TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _MassPoints1TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 26. TagBlock :: Offset: 0x6C (108)
            /// </summary>
            public _MassPoints2TagBlock MassPoints2 { get { return __massPoints2__; } set { __massPoints2__ = value; } }
            private _MassPoints2TagBlock __massPoints2__;
            public class _MassPoints2TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public PhysLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private PhysLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. String32 :: Offset: 0x0 (0)
                /// </summary>
                public string Name { get { return __name__; } set { __name__ = value; } }
                private string __name__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x20 (32)
                /// </summary>
                public short ModelNode { get { return __modelNode__; } set { __modelNode__ = value; } }
                private short __modelNode__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x22 (34)
                /// </summary>
                public short PoweredMassPoint { get { return __poweredMassPoint__; } set { __poweredMassPoint__ = value; } }
                private short __poweredMassPoint__;
                /// <summary>
                /// 4. Bitmask32 :: Offset: 0x24 (36)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool Metallic { get { return __metallic__; } set { __metallic__ = value; } }
                    private bool __metallic__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        Metallic = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 5. Float :: Offset: 0x28 (40)
                /// </summary>
                public float RelativeMass { get { return __relativeMass__; } set { __relativeMass__ = value; } }
                private float __relativeMass__;
                /// <summary>
                /// 6. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Mass { get { return __mass__; } set { __mass__ = value; } }
                private float __mass__;
                /// <summary>
                /// 7. Float :: Offset: 0x30 (48)
                /// </summary>
                public float RelativeDensity { get { return __relativeDensity__; } set { __relativeDensity__ = value; } }
                private float __relativeDensity__;
                /// <summary>
                /// 8. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Density { get { return __density__; } set { __density__ = value; } }
                private float __density__;
                /// <summary>
                /// 9. Float :: Offset: 0x38 (56)
                /// </summary>
                public float PosX { get { return __posX__; } set { __posX__ = value; } }
                private float __posX__;
                /// <summary>
                /// 10. Float :: Offset: 0x3C (60)
                /// </summary>
                public float PosY { get { return __posY__; } set { __posY__ = value; } }
                private float __posY__;
                /// <summary>
                /// 11. Float :: Offset: 0x40 (64)
                /// </summary>
                public float PosZ { get { return __posZ__; } set { __posZ__ = value; } }
                private float __posZ__;
                /// <summary>
                /// 12. Float :: Offset: 0x44 (68)
                /// </summary>
                public float ForwardI { get { return __forwardI__; } set { __forwardI__ = value; } }
                private float __forwardI__;
                /// <summary>
                /// 13. Float :: Offset: 0x48 (72)
                /// </summary>
                public float ForwardJ { get { return __forwardJ__; } set { __forwardJ__ = value; } }
                private float __forwardJ__;
                /// <summary>
                /// 14. Float :: Offset: 0x4C (76)
                /// </summary>
                public float ForwardK { get { return __forwardK__; } set { __forwardK__ = value; } }
                private float __forwardK__;
                /// <summary>
                /// 15. Float :: Offset: 0x50 (80)
                /// </summary>
                public float UpI { get { return __upI__; } set { __upI__ = value; } }
                private float __upI__;
                /// <summary>
                /// 16. Float :: Offset: 0x54 (84)
                /// </summary>
                public float UpJ { get { return __upJ__; } set { __upJ__ = value; } }
                private float __upJ__;
                /// <summary>
                /// 17. Float :: Offset: 0x58 (88)
                /// </summary>
                public float UpK { get { return __upK__; } set { __upK__ = value; } }
                private float __upK__;
                /// <summary>
                /// 18. Enum32 :: Offset: 0x5C (92)
                /// </summary>
                public FrictionTypeEnum32 FrictionType { get { return __frictionType__; } set { __frictionType__ = value; } }
                private FrictionTypeEnum32 __frictionType__;
                public class FrictionTypeEnum32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool Point { get { return __point__; } set { __point__ = value; } }
                    private bool __point__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool Forward { get { return __forward__; } set { __forward__ = value; } }
                    private bool __forward__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool Left { get { return __left__; } set { __left__ = value; } }
                    private bool __left__;
                    /// <summary>
                    /// 4. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool Up { get { return __up__; } set { __up__ = value; } }
                    private bool __up__;

                    public void _Initialize(BitReader br)
                    {
                        uint value = br.ReadUInt32();
                        uint cur = 0;

                        Point = (cur == value);
                        cur++;
                        Forward = (cur == value);
                        cur++;
                        Left = (cur == value);
                        cur++;
                        Up = (cur == value);
                    }
                }
                /// <summary>
                /// 19. Float :: Offset: 0x60 (96)
                /// </summary>
                public float FrictionParallelScale { get { return __frictionParallelScale__; } set { __frictionParallelScale__ = value; } }
                private float __frictionParallelScale__;
                /// <summary>
                /// 20. Float :: Offset: 0x64 (100)
                /// </summary>
                public float FrictionPerpendicularScale { get { return __frictionPerpendicularScale__; } set { __frictionPerpendicularScale__ = value; } }
                private float __frictionPerpendicularScale__;
                /// <summary>
                /// 21. Float :: Offset: 0x68 (104)
                /// </summary>
                public float Radius { get { return __radius__; } set { __radius__ = value; } }
                private float __radius__;
                /// <summary>
                /// 22. ByteArray :: Offset: 0x6C (108)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _MassPoints2TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _MassPoints2TagBlock(PhysLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                    FrictionType = new FrictionTypeEnum32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_MassPoints2TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _MassPoints2TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the PhysLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public PhysLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                InternalMatrix = new _InternalMatrixTagBlock(this);
                MassPoints1 = new _MassPoints1TagBlock(this);
                MassPoints2 = new _MassPoints2TagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Radius = br.ReadSingle();
                MomentScale = br.ReadSingle();
                Mass = br.ReadSingle();
                CenterOfMassX = br.ReadSingle();
                CenterOfMassY = br.ReadSingle();
                CenterOfMassZ = br.ReadSingle();
                Density = br.ReadSingle();
                GravityScale = br.ReadSingle();
                GroundFriction = br.ReadSingle();
                GroundDepth = br.ReadSingle();
                GroundDampFunction = br.ReadSingle();
                NormalK1 = br.ReadSingle();
                NormalK2 = br.ReadSingle();
                Unused1 = br.ReadBytes(4);
                WaterFriction = br.ReadSingle();
                WaterDepth = br.ReadSingle();
                WaterDensity = br.ReadSingle();
                Unused2 = br.ReadBytes(4);
                AirFriction = br.ReadSingle();
                Unused3 = br.ReadBytes(4);
                XxMovement = br.ReadSingle();
                YyMovement = br.ReadSingle();
                ZzMovement = br.ReadSingle();
                InternalMatrix._Initialize(br);
                MassPoints1._Initialize(br);
                MassPoints2._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Phys class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Phys(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new PhysLayout(map, identity);
        }

        #region Specific Methods of phys

        #endregion
    }
}
