using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Point Physics
    /// </summary>
    public class Pphy
    {
        #region Constants

        /// <summary>
        /// Size of the pphy meta header.
        /// </summary>
        public const int HeaderSize = 64;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private PphyLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.9 Shalted: Named Unknowns.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public PphyLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class PphyLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
            private FlagsBitmask32 __flags__;
            public class FlagsBitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool SpecialParticleCollision { get { return __specialParticleCollision__; } set { __specialParticleCollision__ = value; } }
                private bool __specialParticleCollision__;
                /// <summary>
                /// 2. Option :: Value: 0x2 (2)
                /// </summary>
                public bool CollidesWithStructures { get { return __collidesWithStructures__; } set { __collidesWithStructures__ = value; } }
                private bool __collidesWithStructures__;
                /// <summary>
                /// 3. Option :: Value: 0x4 (4)
                /// </summary>
                public bool CollidesWithWaterSurfaces { get { return __collidesWithWaterSurfaces__; } set { __collidesWithWaterSurfaces__ = value; } }
                private bool __collidesWithWaterSurfaces__;
                /// <summary>
                /// 4. Option :: Value: 0x8 (8)
                /// </summary>
                public bool UsesSimpleWind { get { return __usesSimpleWind__; } set { __usesSimpleWind__ = value; } }
                private bool __usesSimpleWind__;
                /// <summary>
                /// 5. Option :: Value: 0x10 (16)
                /// </summary>
                public bool UsesDampenedWind { get { return __usesDampenedWind__; } set { __usesDampenedWind__ = value; } }
                private bool __usesDampenedWind__;
                /// <summary>
                /// 6. Option :: Value: 0x20 (32)
                /// </summary>
                public bool NoGravity { get { return __noGravity__; } set { __noGravity__ = value; } }
                private bool __noGravity__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    SpecialParticleCollision = ((bit & mask) == bit);
                    bit <<= 1;
                    CollidesWithStructures = ((bit & mask) == bit);
                    bit <<= 1;
                    CollidesWithWaterSurfaces = ((bit & mask) == bit);
                    bit <<= 1;
                    UsesSimpleWind = ((bit & mask) == bit);
                    bit <<= 1;
                    UsesDampenedWind = ((bit & mask) == bit);
                    bit <<= 1;
                    NoGravity = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float Density { get { return __density__; } set { __density__ = value; } }
            private float __density__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float AirFriction { get { return __airFriction__; } set { __airFriction__ = value; } }
            private float __airFriction__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float WaterFriction { get { return __waterFriction__; } set { __waterFriction__ = value; } }
            private float __waterFriction__;
            /// <summary>
            /// 5. ByteArray :: Offset: 0x10 (16)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 6. Float :: Offset: 0x20 (32)
            /// </summary>
            public float SurfaceFriction { get { return __surfaceFriction__; } set { __surfaceFriction__ = value; } }
            private float __surfaceFriction__;
            /// <summary>
            /// 7. Float :: Offset: 0x24 (36)
            /// </summary>
            public float ElasticityPossibly { get { return __elasticityPossibly__; } set { __elasticityPossibly__ = value; } }
            private float __elasticityPossibly__;
            /// <summary>
            /// 8. Float :: Offset: 0x28 (40)
            /// </summary>
            public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
            private float __unknown__;
            /// <summary>
            /// 9. ByteArray :: Offset: 0x2C (44)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the PphyLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public PphyLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags = new FlagsBitmask32();
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags._Initialize(br);
                Density = br.ReadSingle();
                AirFriction = br.ReadSingle();
                WaterFriction = br.ReadSingle();
                Unused1 = br.ReadBytes(16);
                SurfaceFriction = br.ReadSingle();
                ElasticityPossibly = br.ReadSingle();
                Unknown = br.ReadSingle();
                Unused2 = br.ReadBytes(20);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Pphy class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Pphy(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new PphyLayout(map, identity);
        }

        #region Specific Methods of pphy

        #endregion
    }
}
