using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Scenario
    /// </summary>
    public class Scnr
    {
        #region Constants

        /// <summary>
        /// Size of the scnr meta header.
        /// </summary>
        public const int HeaderSize = 992;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        //private ScnrLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.9 XZodia: Corrected Many Values and Added a Bunch More.
        /// v0.85 DiscoBob: Added TheTyckoMan's AI Findings...
        /// v0.82 DiscoBob: Named the Script Objects Reflexive...
        /// v0.8 TunesRus90: A couple of Placement reflexives updated...
        /// v0.77 Shade45: Added Script and Globals info...
        /// v0.75 TheTyckoMan: Some AI Stuff added...
        /// v0.7 Soldier of Light: Death Zones and a few more knowns added...
        /// v0.65 Soldier of Light: Lots of spawn locations found (player, objective)...
        /// v0.6 Soldier of Light: Netgame and Starting equipment values found...
        /// v0.55 Toabn: Combined Cavemaneca and my additions...
        /// v0.5 Cavemaneca: Added some guessed values...
        /// v0.3 Iron_Forge: Added some known values...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        //public ScnrLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        // public class ScnrLayout...

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Scnr class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Scnr(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            //this.Layout = new ScnrLayout(map, identity);
        }

        #region Specific Methods of scnr

        [FlagsAttribute]
        public enum SpawnTypes : int
        {
            Player = 1,
            Weapon = 2,
            Collection = 4,
            Vehicle = 8,
            Obstacle = 16,
            Machine = 32,
            Scenery = 64,
            Objective = 128,
            Equipment = 256,
            ItemCollection = 512,
            Biped = 1024,
            Control = 2048,
            Special = 4096,
            DeathZone = 8192,
            Camera = 16384,
            Light = 32768
        }

        public enum SpawnRotationTypes
        {
            BoundingBox,
            Directional,
            YawPitchRoll
        }

        #endregion
    }
}