using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Sound
    /// </summary>
    public class Snd
    {
        #region Constants

        /// <summary>
        /// Size of the snd! meta header.
        /// </summary>
        public const int HeaderSize = 20;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private SndLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.5 Shalted: No Comment.
        /// v0.2 TfAv1228: Named Some Unknowns.
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public SndLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class SndLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask16 :: Offset: 0x0 (0)
            /// </summary>
            public FlagsBitmask16 Flags { get { return __flags__; } set { __flags__ = value; } }
            private FlagsBitmask16 __flags__;
            public class FlagsBitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool SplitIntoPermutations { get { return __splitIntoPermutations__; } set { __splitIntoPermutations__ = value; } }
                private bool __splitIntoPermutations__;
                /// <summary>
                /// 2. Option :: Value: 0xA (10)
                /// </summary>
                public bool N1 { get { return __n1__; } set { __n1__ = value; } }
                private bool __n1__;
                /// <summary>
                /// 3. Option :: Value: 0xB (11)
                /// </summary>
                public bool N2 { get { return __n2__; } set { __n2__ = value; } }
                private bool __n2__;
                /// <summary>
                /// 4. Option :: Value: 0xC (12)
                /// </summary>
                public bool Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private bool __unknown__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    SplitIntoPermutations = ((bit & mask) == bit);
                    bit <<= 1;
                    N1 = ((bit & mask) == bit);
                    bit <<= 1;
                    N2 = ((bit & mask) == bit);
                    bit <<= 1;
                    Unknown = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Int16 :: Offset: 0x2 (2)
            /// </summary>
            public short SoundClass { get { return __soundClass__; } set { __soundClass__ = value; } }
            private short __soundClass__;
            /// <summary>
            /// 3. Enum8 :: Offset: 0x4 (4)
            /// </summary>
            public FormatEnum8 Format { get { return __format__; } set { __format__ = value; } }
            private FormatEnum8 __format__;
            public class FormatEnum8
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Mono { get { return __mono__; } set { __mono__ = value; } }
                private bool __mono__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Stereo { get { return __stereo__; } set { __stereo__ = value; } }
                private bool __stereo__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool WMASpecific { get { return __wMASpecific__; } set { __wMASpecific__ = value; } }
                private bool __wMASpecific__;

                public void _Initialize(BitReader br)
                {
                    byte value = br.ReadByte();
                    byte cur = 0;

                    Mono = (cur == value);
                    cur++;
                    Stereo = (cur == value);
                    cur++;
                    WMASpecific = (cur == value);
                }
            }
            /// <summary>
            /// 4. Enum8 :: Offset: 0x4 (4)
            /// </summary>
            public CompressionEnum8 Compression { get { return __compression__; } set { __compression__ = value; } }
            private CompressionEnum8 __compression__;
            public class CompressionEnum8
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool None { get { return __none__; } set { __none__ = value; } }
                private bool __none__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool XboxAdpcm { get { return __xboxAdpcm__; } set { __xboxAdpcm__ = value; } }
                private bool __xboxAdpcm__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool ImaAdpcmPossibly { get { return __imaAdpcmPossibly__; } set { __imaAdpcmPossibly__ = value; } }
                private bool __imaAdpcmPossibly__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool OggPossibly { get { return __oggPossibly__; } set { __oggPossibly__ = value; } }
                private bool __oggPossibly__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool WMA { get { return __wMA__; } set { __wMA__ = value; } }
                private bool __wMA__;

                public void _Initialize(BitReader br)
                {
                    byte value = br.ReadByte();
                    byte cur = 0;

                    None = (cur == value);
                    cur++;
                    XboxAdpcm = (cur == value);
                    cur++;
                    ImaAdpcmPossibly = (cur == value);
                    cur++;
                    OggPossibly = (cur == value);
                    cur++;
                    WMA = (cur == value);
                }
            }
            /// <summary>
            /// 5. ByteArray :: Offset: 0x5 (5)
            /// </summary>
            public byte[] Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private byte[] __unknown1__;
            /// <summary>
            /// 6. Int16 :: Offset: 0x6 (6)
            /// </summary>
            public short IndexToSoundProperties { get { return __indexToSoundProperties__; } set { __indexToSoundProperties__ = value; } }
            private short __indexToSoundProperties__;
            /// <summary>
            /// 7. Int16 :: Offset: 0x8 (8)
            /// </summary>
            public short IndexToSoundPermutations { get { return __indexToSoundPermutations__; } set { __indexToSoundPermutations__ = value; } }
            private short __indexToSoundPermutations__;
            /// <summary>
            /// 8. Byte :: Offset: 0xA (10)
            /// </summary>
            public byte PermutationChunkCount { get { return __permutationChunkCount__; } set { __permutationChunkCount__ = value; } }
            private byte __permutationChunkCount__;
            /// <summary>
            /// 9. Byte :: Offset: 0xB (11)
            /// </summary>
            public byte IndexToSoundUnknown { get { return __indexToSoundUnknown__; } set { __indexToSoundUnknown__ = value; } }
            private byte __indexToSoundUnknown__;
            /// <summary>
            /// 10. Int16 :: Offset: 0x6 (6)
            /// </summary>
            public short NotIndexD1 { get { return __notIndexD1__; } set { __notIndexD1__ = value; } }
            private short __notIndexD1__;
            /// <summary>
            /// 11. Int16 :: Offset: 0x8 (8)
            /// </summary>
            public short IndexToCombatDialogRaw { get { return __indexToCombatDialogRaw__; } set { __indexToCombatDialogRaw__ = value; } }
            private short __indexToCombatDialogRaw__;
            /// <summary>
            /// 12. ByteArray :: Offset: 0xA (10)
            /// </summary>
            public byte[] Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private byte[] __unknown2__;
            /// <summary>
            /// 13. Int32 :: Offset: 0x10 (16)
            /// </summary>
            public int NotIndexD2 { get { return __notIndexD2__; } set { __notIndexD2__ = value; } }
            private int __notIndexD2__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the SndLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public SndLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags = new FlagsBitmask16();
                Format = new FormatEnum8();
                Compression = new CompressionEnum8();
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags._Initialize(br);
                SoundClass = br.ReadInt16();
                Format._Initialize(br);
                Compression._Initialize(br);
                Unknown1 = br.ReadBytes(1);
                IndexToSoundProperties = br.ReadInt16();
                IndexToSoundPermutations = br.ReadInt16();
                PermutationChunkCount = br.ReadByte();
                IndexToSoundUnknown = br.ReadByte();
                NotIndexD1 = br.ReadInt16();
                IndexToCombatDialogRaw = br.ReadInt16();
                Unknown2 = br.ReadBytes(6);
                NotIndexD2 = br.ReadInt32();

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Snd class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Snd(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new SndLayout(map, identity);
        }

        #region Specific Methods of snd!

        #endregion
    }
}
