using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Unicode String
    /// </summary>
    public class Unic
    {
        #region Constants

        /// <summary>
        /// Size of the unic meta header.
        /// </summary>
        public const int HeaderSize = 52;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private UnicLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v1.0 Iron_Forge: Completed...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public UnicLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class UnicLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. ByteArray :: Offset: 0x0 (0)
            /// </summary>
            public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
            private byte[] __unused__;
            /// <summary>
            /// 2. UInt16 :: Offset: 0x10 (16)
            /// </summary>
            public ushort EnglishStringOffset { get { return __englishStringOffset__; } set { __englishStringOffset__ = value; } }
            private ushort __englishStringOffset__;
            /// <summary>
            /// 3. UInt16 :: Offset: 0x12 (18)
            /// </summary>
            public ushort EnglishStringCount { get { return __englishStringCount__; } set { __englishStringCount__ = value; } }
            private ushort __englishStringCount__;
            /// <summary>
            /// 4. UInt16 :: Offset: 0x14 (20)
            /// </summary>
            public ushort JapaneseStringOffset { get { return __japaneseStringOffset__; } set { __japaneseStringOffset__ = value; } }
            private ushort __japaneseStringOffset__;
            /// <summary>
            /// 5. UInt16 :: Offset: 0x16 (22)
            /// </summary>
            public ushort JapaneseStringCount { get { return __japaneseStringCount__; } set { __japaneseStringCount__ = value; } }
            private ushort __japaneseStringCount__;
            /// <summary>
            /// 6. UInt16 :: Offset: 0x18 (24)
            /// </summary>
            public ushort GermanStringOffset { get { return __germanStringOffset__; } set { __germanStringOffset__ = value; } }
            private ushort __germanStringOffset__;
            /// <summary>
            /// 7. UInt16 :: Offset: 0x1A (26)
            /// </summary>
            public ushort GermanStringCount { get { return __germanStringCount__; } set { __germanStringCount__ = value; } }
            private ushort __germanStringCount__;
            /// <summary>
            /// 8. UInt16 :: Offset: 0x1C (28)
            /// </summary>
            public ushort FrenchStringOffset { get { return __frenchStringOffset__; } set { __frenchStringOffset__ = value; } }
            private ushort __frenchStringOffset__;
            /// <summary>
            /// 9. UInt16 :: Offset: 0x1E (30)
            /// </summary>
            public ushort FrenchStringCount { get { return __frenchStringCount__; } set { __frenchStringCount__ = value; } }
            private ushort __frenchStringCount__;
            /// <summary>
            /// 10. UInt16 :: Offset: 0x20 (32)
            /// </summary>
            public ushort SpanishStringOffset { get { return __spanishStringOffset__; } set { __spanishStringOffset__ = value; } }
            private ushort __spanishStringOffset__;
            /// <summary>
            /// 11. UInt16 :: Offset: 0x22 (34)
            /// </summary>
            public ushort SpanishStringCount { get { return __spanishStringCount__; } set { __spanishStringCount__ = value; } }
            private ushort __spanishStringCount__;
            /// <summary>
            /// 12. UInt16 :: Offset: 0x24 (36)
            /// </summary>
            public ushort ItalianStringOffset { get { return __italianStringOffset__; } set { __italianStringOffset__ = value; } }
            private ushort __italianStringOffset__;
            /// <summary>
            /// 13. UInt16 :: Offset: 0x26 (38)
            /// </summary>
            public ushort ItalianStringCount { get { return __italianStringCount__; } set { __italianStringCount__ = value; } }
            private ushort __italianStringCount__;
            /// <summary>
            /// 14. UInt16 :: Offset: 0x28 (40)
            /// </summary>
            public ushort KoreanStringOffset { get { return __koreanStringOffset__; } set { __koreanStringOffset__ = value; } }
            private ushort __koreanStringOffset__;
            /// <summary>
            /// 15. UInt16 :: Offset: 0x2A (42)
            /// </summary>
            public ushort KoreanStringCount { get { return __koreanStringCount__; } set { __koreanStringCount__ = value; } }
            private ushort __koreanStringCount__;
            /// <summary>
            /// 16. UInt16 :: Offset: 0x2C (44)
            /// </summary>
            public ushort ChineseStringOffset { get { return __chineseStringOffset__; } set { __chineseStringOffset__ = value; } }
            private ushort __chineseStringOffset__;
            /// <summary>
            /// 17. UInt16 :: Offset: 0x2E (46)
            /// </summary>
            public ushort ChineseStringCount { get { return __chineseStringCount__; } set { __chineseStringCount__ = value; } }
            private ushort __chineseStringCount__;
            /// <summary>
            /// 18. UInt16 :: Offset: 0x30 (48)
            /// </summary>
            public ushort PortugueseStringOffset { get { return __portugueseStringOffset__; } set { __portugueseStringOffset__ = value; } }
            private ushort __portugueseStringOffset__;
            /// <summary>
            /// 19. UInt16 :: Offset: 0x32 (50)
            /// </summary>
            public ushort PortugueseStringCount { get { return __portugueseStringCount__; } set { __portugueseStringCount__ = value; } }
            private ushort __portugueseStringCount__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the UnicLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public UnicLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unused = br.ReadBytes(16);
                EnglishStringOffset = br.ReadUInt16();
                EnglishStringCount = br.ReadUInt16();
                JapaneseStringOffset = br.ReadUInt16();
                JapaneseStringCount = br.ReadUInt16();
                GermanStringOffset = br.ReadUInt16();
                GermanStringCount = br.ReadUInt16();
                FrenchStringOffset = br.ReadUInt16();
                FrenchStringCount = br.ReadUInt16();
                SpanishStringOffset = br.ReadUInt16();
                SpanishStringCount = br.ReadUInt16();
                ItalianStringOffset = br.ReadUInt16();
                ItalianStringCount = br.ReadUInt16();
                KoreanStringOffset = br.ReadUInt16();
                KoreanStringCount = br.ReadUInt16();
                ChineseStringOffset = br.ReadUInt16();
                ChineseStringCount = br.ReadUInt16();
                PortugueseStringOffset = br.ReadUInt16();
                PortugueseStringCount = br.ReadUInt16();

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Unic class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Unic(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new UnicLayout(map, identity);
        }

        #region Specific Methods of unic

        #endregion
    }
}
