using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Extended;
using H2UtilityToolkit;
using H2UtilityToolkit.Misc;
using MDXUtilityToolkit;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Structure BSP
    /// </summary>
    public class Sbsp
    {
        #region Instance Data

        #region Constants

        /// <summary>
        /// Size of the sbsp meta header.
        /// </summary>
        public const int HeaderSize = 588;

        #endregion

        //public int bsp_rend_cnt;
        //public int perm_rend_cnt;

        private Map __map__;
        private int __identity__;
        private BSPDeclaration[] __bSPDeclarations__;
        private bool __loadedForRender__;
        private int __bSPIndex__;
        private List<RawBSPModelChunk> __rawBSPModelChunks__;
        private RawBSPChunk[] __rawBSPChunks__;
        private RawPermChunk[] __rawPermChunks__;
        private PermPlacement[] __permPlacements__;
        private Shad[] __shaders__;
        private Bitmap[] __bSPLtmpBitmaps__;
        private Texture[] __bSPLtmpTextures__;
        private Bitmap[] __permLtmpBitmaps__;
        private Texture[] __permLtmpTextures__;
        //private SpawnInfo __spawnInfo__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return __identity__; } set { __identity__ = value; } }
        /// <summary>
        /// BSPDeclarations
        /// </summary>
        public BSPDeclaration[] BSPDeclarations { get { return __bSPDeclarations__; } set { __bSPDeclarations__ = value; } }
        /// <summary>
        /// LoadedForRender
        /// </summary>
        public bool LoadedForRender { get { return __loadedForRender__; } set { __loadedForRender__ = value; } }
        /// <summary>
        /// BSPIndex
        /// </summary>
        public int BSPIndex { get { return __bSPIndex__; } set { __bSPIndex__ = value; } }
        /// <summary>
        /// RawBSPModelChunks
        /// </summary>
        public List<RawBSPModelChunk> RawBSPModelChunks { get { return __rawBSPModelChunks__; } set { __rawBSPModelChunks__ = value; } }
        /// <summary>
        /// RawBSPChunks
        /// </summary>
        public RawBSPChunk[] RawBSPChunks { get { return __rawBSPChunks__; } set { __rawBSPChunks__ = value; } }
        /// <summary>
        /// RawPermChunks
        /// </summary>
        public RawPermChunk[] RawPermChunks { get { return __rawPermChunks__; } set { __rawPermChunks__ = value; } }
        /// <summary>
        /// PermPlacements
        /// </summary>
        public PermPlacement[] PermPlacements { get { return __permPlacements__; } set { __permPlacements__ = value; } }
        /// <summary>
        /// Shaders
        /// </summary>
        public Shad[] Shaders { get { return __shaders__; } set { __shaders__ = value; } }
        /// <summary>
        /// BSPLtmpBitmaps
        /// </summary>
        public Bitmap[] BSPLtmpBitmaps { get { return __bSPLtmpBitmaps__; } set { __bSPLtmpBitmaps__ = value; } }
        /// <summary>
        /// BSPLtmpTextures
        /// </summary>
        public Texture[] BSPLtmpTextures { get { return __bSPLtmpTextures__; } set { __bSPLtmpTextures__ = value; } }
        /// <summary>
        /// PermLtmpBitmaps
        /// </summary>
        public Bitmap[] PermLtmpBitmaps { get { return __permLtmpBitmaps__; } set { __permLtmpBitmaps__ = value; } }
        /// <summary>
        /// PermLtmpTextures
        /// </summary>
        public Texture[] PermLtmpTextures { get { return __permLtmpTextures__; } set { __permLtmpTextures__ = value; } }
        /// <summary>
        /// DevMaterial
        /// </summary>
        private Material DevMaterial;
        /// <summary>
        /// SpawnInfo
        /// </summary>
        //public SpawnInfo SpawnInfo { get { return __spawnInfo__; } set { __spawnInfo__ = value; } }

        #endregion

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Sbsp class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Sbsp(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.RawBSPModelChunks = new List<RawBSPModelChunk>();
            //this.SpawnInfo = new SpawnInfo();
        }

        #region Specific Methods of sbsp

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Sbsp class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        /// <param name="br"></param>
        public Sbsp(Map map, int identity, BitReader br)
        {
            this.Map = map;
            this.Identity = identity;
            this.RawBSPModelChunks = new List<RawBSPModelChunk>();
            //this.SpawnInfo = new SpawnInfo();
            this.Read(br);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="br"></param>
        public void Read(BitReader br)
        {
            int scnrIndx = Map.Tags.IndexOf(Map.Index.ScenariosID);
            if (scnrIndx == -1)
            {
                if (Map.Tags.TagsList.Length < 3 || Map.Tags.TagsList[3].Type != "scnr") return;
                else scnrIndx = 3;
            }
            int scnrOffset = Map.Tags.TagsList[scnrIndx].MetaOffset;

            br.BaseStream.Position = scnrOffset + 528;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            BSPDeclarations = new BSPDeclaration[cc];
            for (int x = 0; x < cc; x++)
            {
                br.BaseStream.Position = tr + (x * 68);
                BSPDeclarations[x] = new BSPDeclaration();
                BSPDeclarations[x].DeclarationOffset = (int)br.BaseStream.Position - scnrOffset;

                // SBSP.
                BSPDeclarations[x].MetaOffset = br.ReadInt32(); // & (int)0x3FFFFFFF;
                BSPDeclarations[x].MetaSize = br.ReadInt32();
                BSPDeclarations[x].Magic = br.ReadInt32() - BSPDeclarations[x].MetaOffset;
                br.BaseStream.Position += 8;
                BSPDeclarations[x].Identity = br.ReadInt32();
                int sbspIndx = Map.Tags.IndexOf(BSPDeclarations[x].Identity);
                // Fix.
                Map.Tags.TagsList[sbspIndx].MetaOffset = BSPDeclarations[x].MetaOffset;
                Map.Tags.TagsList[sbspIndx].MetaSize = BSPDeclarations[x].MetaSize;

                // LTMP.
                br.BaseStream.Position += 4;
                BSPDeclarations[x].Lightmap.Identity = br.ReadInt32();
                int ltmpIndx = Map.Tags.IndexOf(BSPDeclarations[x].Lightmap.Identity);
                // Broken ident.
                if (ltmpIndx == -1) continue;
                br.BaseStream.Position = BSPDeclarations[x].MetaOffset + 8;
                BSPDeclarations[x].Lightmap.MetaOffset = br.ReadInt32();
                // May mean bsp is not linked to a lightmap.
                if (BSPDeclarations[x].Lightmap.MetaOffset == 0) continue;
                BSPDeclarations[x].Lightmap.MetaOffset -= BSPDeclarations[x].Magic;
                BSPDeclarations[x].Lightmap.MetaSize = (BSPDeclarations[x].MetaOffset + BSPDeclarations[x].MetaSize) - BSPDeclarations[x].Lightmap.MetaOffset;
                // Fix.
                Map.Tags.TagsList[ltmpIndx].MetaOffset = BSPDeclarations[x].Lightmap.MetaOffset;
                Map.Tags.TagsList[ltmpIndx].MetaSize = BSPDeclarations[x].Lightmap.MetaSize;
            }
        }

        public void ShiftPointers(FileStream fs, int shift, int minOffset)
        {
            BitReader br = new BitReader(fs, Endian.Little);
            BitWriter bw = new BitWriter(fs, Endian.Little);

            // Get scnr
            int scnrIndx = Map.Tags.IndexOf(Map.Index.ScenariosID);
            if (scnrIndx == -1)
            {
                if (Map.Tags.TagsList.Length < 3 || Map.Tags.TagsList[3].Type != "scnr") return;
                else scnrIndx = 3;
            }
            int scnrOffset = Map.Tags.TagsList[scnrIndx].MetaOffset;


            br.BaseStream.Position = scnrOffset + 528;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.Index.SecondaryMagic;

            for (int x = 0; x < cc; x++)
            {
                if (BSPDeclarations[x].MetaOffset >= minOffset)
                {
                    br.BaseStream.Position = tr + (x * 68);
                    BSPDeclarations[x].DeclarationOffset = (int)br.BaseStream.Position - scnrOffset;

                    // SBSP.
                    bw.Write(BSPDeclarations[x].MetaOffset += shift);
                    br.BaseStream.Position += 4;
                    BSPDeclarations[x].Magic = br.ReadInt32() - BSPDeclarations[x].MetaOffset;
                    br.BaseStream.Position += 12;
                    int sbspIndx = Map.Tags.IndexOf(BSPDeclarations[x].Identity);
                    // Fix.
                    Map.Tags.TagsList[sbspIndx].MetaOffset = BSPDeclarations[x].MetaOffset;
                }

                if (BSPDeclarations[x].Lightmap.MetaOffset >= minOffset)
                {
                    // LTMP.
                    // May mean bsp is not linked to a lightmap.
                    if (BSPDeclarations[x].Lightmap.MetaOffset == 0) continue;
                    br.BaseStream.Position += 8;
                    int ltmpIndx = Map.Tags.IndexOf(BSPDeclarations[x].Lightmap.Identity);
                    // Broken ident.
                    if (ltmpIndx == -1) continue;
                    br.BaseStream.Position = BSPDeclarations[x].MetaOffset + 8;
                    BSPDeclarations[x].Lightmap.MetaOffset = br.ReadInt32();
                    // May mean bsp is not linked to a lightmap.
                    if (BSPDeclarations[x].Lightmap.MetaOffset == 0) continue;
                    BSPDeclarations[x].Lightmap.MetaOffset -= BSPDeclarations[x].Magic;
                    // Fix.
                    Map.Tags.TagsList[ltmpIndx].MetaOffset = BSPDeclarations[x].Lightmap.MetaOffset;
                }

                // Shift raw pointers
                // Raw1.
                br.BaseStream.Position = BSPDeclarations[x].MetaOffset + 172;
                cc = br.ReadInt32();
                tr = br.ReadInt32() - BSPDeclarations[x].Magic;

                for (int z = 0; z < cc; z++)
                {
                    br.BaseStream.Position = tr + (z * 176) + 40;

                    int rOffset = br.ReadInt32();
                    if (rOffset != -1)
                    {
                        RawLocation rawLoc;
                        Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                        if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                        {
                            bw.BaseStream.Position -= 4;
                            bw.Write(rOffset + shift);
                        }
                    }
                }

                // Raw2
                br.BaseStream.Position = BSPDeclarations[x].MetaOffset + 328;

                cc = br.ReadInt32();
                tr = br.ReadInt32() - BSPDeclarations[x].Magic;

                for (int z = 0; z < cc; z++)
                {
                    br.BaseStream.Position = tr + (z * 200) + 40;

                    int rOffset = br.ReadInt32();
                    if (rOffset != -1)
                    {
                        RawLocation rawLoc;
                        Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                        if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                        {
                            bw.BaseStream.Position -= 4;
                            bw.Write(rOffset + shift);
                        }
                    }
                }

                // Lightmap Raw.
                br.BaseStream.Position = BSPDeclarations[x].Lightmap.MetaOffset + 128 + 4;
                br.BaseStream.Position = (br.ReadInt32() - BSPDeclarations[x].Magic) + 64;

                cc = br.ReadInt32();
                tr = br.ReadInt32() - BSPDeclarations[x].Magic;

                for (int z = 0; z < cc; z++)
                {
                    br.BaseStream.Position = tr + (z * 56) + 12;

                    int rOffset = br.ReadInt32();
                    if (rOffset != -1)
                    {
                        RawLocation rawLoc;
                        Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                        if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                        {
                            bw.BaseStream.Position -= 4;
                            bw.Write(rOffset + shift);
                        }
                    }
                }

                // Raw3.
                br.BaseStream.Position = BSPDeclarations[x].MetaOffset + 580;
                cc = br.ReadInt32();
                tr = br.ReadInt32() - BSPDeclarations[x].Magic;

                br.BaseStream.Position = tr + 16;
                cc = br.ReadInt32();
                tr = br.ReadInt32() - BSPDeclarations[x].Magic;

                for (int z = 0; z < cc; z++)
                {
                    br.BaseStream.Position = tr + (z * 44);

                    int rOffset = br.ReadInt32();
                    if (rOffset != -1)
                    {
                        RawLocation rawLoc;
                        Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                        if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                        {
                            bw.BaseStream.Position -= 4;
                            bw.Write(rOffset + shift);
                        }
                    }
                }

                // Raw4.
                br.BaseStream.Position = BSPDeclarations[x].MetaOffset + 548;

                cc = br.ReadInt32();
                tr = br.ReadInt32() - BSPDeclarations[x].Magic;

                for (int z = 0; z < cc; z++)
                {
                    br.BaseStream.Position = tr + (z * 172) + 16;

                    int rOffset = br.ReadInt32();
                    if (rOffset != -1)
                    {
                        RawLocation rawLoc;
                        Map.Misc.ParsePointer(ref rOffset, out rawLoc);

                        if (rawLoc == RawLocation.Internal && rOffset >= minOffset)
                        {
                            bw.BaseStream.Position -= 4;
                            bw.Write(rOffset + shift);
                        }
                    }
                }
            }
        }

        #region Functions

        public int IndexOfSBSP(int ID)
        {
            for (int x = 0; x < BSPDeclarations.Length; x++)
            {
                if (BSPDeclarations[x].Identity == ID)
                    return x;
            }
            return -1;
        }

        public int IndexOfLTMP(int ID)
        {
            for (int x = 0; x < BSPDeclarations.Length; x++)
            {
                if (BSPDeclarations[x].Lightmap.Identity == ID)
                    return x;
            }
            return -1;
        }

        public bool IsBSPShader(int shadIndx)
        {
            for (int x = 0; x < RawBSPChunks.Length; x++)
            {
                if (RawBSPChunks[x].RawInfoChunks.Length == 0)
                    continue;

                for (int y = 0; y < RawBSPChunks[x].SubMeshInfo.Length; y++)
                {
                    if (RawBSPChunks[x].SubMeshInfo[y].ShaderIndex == shadIndx)
                        return true;
                }
            }

            return false;
        }

        public bool IsPermShader(int shadIndx)
        {
            for (int x = 0; x < RawPermChunks.Length; x++)
            {

                if (RawPermChunks[x].RawInfoChunks.Length == 0)
                    continue;

                for (int y = 0; y < RawPermChunks[x].SubMeshInfo.Length; y++)
                {
                    if (RawPermChunks[x].SubMeshInfo[y].ShaderIndex == shadIndx)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        #endregion

        #region Render Load

        public void LoadForRender()
        {
            if (LoadedForRender) Unload();
            LoadedForRender = false;

            LoadFromMap();
            LoadBSPMeshes();
            LoadPermMeshes();
            FindAlphaTextures();
            LoadLightmapTextures();

            LoadedForRender = true;
        }

        public void Unload()
        {
            RawBSPModelChunks.Clear();
            if (RawBSPChunks != null) Array.Clear(RawBSPChunks, 0, RawBSPChunks.Length);
            if (RawPermChunks != null) Array.Clear(RawPermChunks, 0, RawPermChunks.Length);
            if (PermPlacements != null) Array.Clear(PermPlacements, 0, PermPlacements.Length);
            if (Shaders != null)
            {
                for (int x = 0; x < Shaders.Length; x++)
                    Shaders[x].DisposeOf(true, true, true);

                Array.Clear(Shaders, 0, Shaders.Length);
            }
            if (BSPLtmpBitmaps != null) Array.Clear(BSPLtmpBitmaps, 0, BSPLtmpBitmaps.Length);
            if (PermLtmpBitmaps != null) Array.Clear(PermLtmpBitmaps, 0, PermLtmpBitmaps.Length);
            if (BSPLtmpTextures != null) Array.Clear(BSPLtmpTextures, 0, BSPLtmpTextures.Length);
            if (PermLtmpTextures != null) Array.Clear(PermLtmpTextures, 0, PermLtmpTextures.Length);
        }

        private void LoadFromMap()
        {
            BitReader br = new BitReader(Map.FileName, Endian.Little);
            BSPDeclaration bspD = BSPDeclarations[BSPIndex];

            #region Identify Raw Models

            // BSP Model 1
            br.BaseStream.Position = bspD.MetaOffset + 172;
            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - bspD.Magic;

            for (int x = 0; x < cc; x++)
            {
                RawBSPModelChunk raw = new RawBSPModelChunk();

                br.BaseStream.Position = tr + (x * 176) + 40;
                raw.Offset = br.ReadInt32();
                raw.Size = br.ReadInt32();

                RawBSPModelChunks.Add(raw);
            }
            int firstBSPModelChunkIndx = RawBSPModelChunks.Count;

            // BSP Model 2
            br.BaseStream.Position = bspD.MetaOffset + 328;
            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            for (int x = 0; x < cc; x++)
            {
                RawBSPModelChunk raw = new RawBSPModelChunk();

                br.BaseStream.Position = tr + (x * 200) + 40;
                raw.Offset = br.ReadInt32();
                raw.Size = br.ReadInt32();

                RawBSPModelChunks.Add(raw);
            }
            if (firstBSPModelChunkIndx == RawBSPModelChunks.Count) firstBSPModelChunkIndx = 0;

            // BSP Model 3
            br.BaseStream.Position = bspD.MetaOffset + 580;
            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            br.BaseStream.Position = tr + 16;
            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            for (int x = 0; x < cc; x++)
            {
                RawBSPModelChunk raw = new RawBSPModelChunk();

                br.BaseStream.Position = tr + (x * 44);
                raw.Offset = br.ReadInt32();
                raw.Size = br.ReadInt32();

                RawBSPModelChunks.Add(raw);
            }

            // BSP Model 4
            br.BaseStream.Position = bspD.MetaOffset + 548;
            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            for (int x = 0; x < cc; x++)
            {
                RawBSPModelChunk raw = new RawBSPModelChunk();

                br.BaseStream.Position = tr + (x * 172) + 16;
                raw.Offset = br.ReadInt32();
                raw.Size = br.ReadInt32();

                RawBSPModelChunks.Add(raw);
            }

            #endregion

            #region Load Model Structure

            br.BaseStream.Position = bspD.MetaOffset + 172;

            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            RawBSPChunks = new RawBSPChunk[cc];

            for (int x = 0; x < cc; x++)
            {
                br.BaseStream.Position = tr + (x * 176);
                RawBSPChunks[x] = new RawBSPChunk(br, bspD, RawBSPModelChunks[x]);
            }

            #endregion

            #region Load Permutations

            if (Map.Prefs.Perms_Load)
            {
                br.BaseStream.Position = bspD.MetaOffset + 328;
                cc = br.ReadInt32();
                tr = br.ReadInt32() - bspD.Magic;

                RawPermChunks = new RawPermChunk[cc];

                for (int x = 0; x < cc; x++)
                {
                    br.BaseStream.Position = tr + (x * 200);

                    RawPermChunks[x] = new RawPermChunk(br, bspD,
                        this.RawBSPModelChunks[firstBSPModelChunkIndx + x]);
                }

                br.BaseStream.Position = bspD.MetaOffset + 336;
                cc = br.ReadInt32();
                tr = br.ReadInt32() - bspD.Magic;

                PermPlacements = new PermPlacement[cc];
                for (int x = 0; x < cc; x++)
                {
                    br.BaseStream.Position = tr + (x * 88);
                    PermPlacements[x] = new PermPlacement(br);
                }
            }

            #endregion

            #region Load Shaders

            br.BaseStream.Position = bspD.MetaOffset + 180;
            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            Shaders = new Shad[cc];
            for (int x = 0; x < cc; x++)
            {
                bool loadTextures = true;

                if (IsBSPShader(x))
                {
                    if (!Map.Prefs.BSP_LoadShaders)
                    {
                        loadTextures = false;
                    }
                }
                else
                {
                    if (!Map.Prefs.Perms_Load || !Map.Prefs.Perms_LoadShaders)
                    {
                        loadTextures = false;
                    }
                }

                // Load Shader
                br.BaseStream.Position = tr + (x * 32) + 12;
                Shaders[x] = new Shad(Map, br.ReadInt32(), br, loadTextures);
            }

            #endregion

            #region Load Lightmap

            br.BaseStream.Position = bspD.Lightmap.MetaOffset + 128;
            cc = br.ReadInt32();
            tr = br.ReadInt32() - bspD.Magic;

            br.BaseStream.Position = tr + 28;
            int ltmpBitmID = br.ReadInt32();

            if ((Map.Prefs.BSP_LoadLightmaps || Map.Prefs.Perms_LoadLightmaps))
            {
                // Lightmap palettes
                br.BaseStream.Position = tr + 8;
                int cc2 = br.ReadInt32();
                int tr2 = br.ReadInt32() - bspD.Magic;

                if (cc2 != 0)
                {
                    bspD.Lightmap.PalettesOffset = tr2;
                    br.BaseStream.Position = tr2;

                    bspD.Lightmap.Palettes.Clear();
                    for (int y = 0; y < cc2; y++)
                    {
                        Bitm.RGBA_ByteColor[] pc = new Bitm.RGBA_ByteColor[256];
                        for (int z = 0; z < 256; z++)
                        {
                            pc[z].R = br.ReadByte();
                            pc[z].G = br.ReadByte();
                            pc[z].B = br.ReadByte();
                            pc[z].A = br.ReadByte();
                        }
                        bspD.Lightmap.Palettes.Add(pc);
                    }
                }

                Bitm bitmTag = new Bitm(Map, ltmpBitmID);

                if (Map.Prefs.BSP_LoadLightmaps)
                {
                    br.BaseStream.Position = tr + 40;
                    cc2 = br.ReadInt32();
                    tr2 = br.ReadInt32() - bspD.Magic;

                    if (cc2 != 0)
                    {
                        BSPLtmpBitmaps = new Bitmap[cc2];

                        for (int y = 0; y < cc2; y++)
                        {
                            br.BaseStream.Position = tr2 + (y * 4);
                            BSPLtmpBitmaps[y] = bitmTag.GetBitmap(br, br.ReadInt16(), 0, br.ReadInt16(), BSPIndex);
                        }
                    }
                }

                if (Map.Prefs.Perms_LoadLightmaps)
                {
                    br.BaseStream.Position = tr + 72;
                    cc2 = br.ReadInt32();
                    tr2 = br.ReadInt32() - bspD.Magic;

                    if (cc2 != 0)
                    {
                        PermLtmpBitmaps = new Bitmap[cc2];

                        for (int y = 0; y < cc2; y++)
                        {
                            br.BaseStream.Position = tr2 + (y * 4);
                            PermLtmpBitmaps[y] = bitmTag.GetBitmap(br, br.ReadInt16(), 0, br.ReadInt16(), BSPIndex);
                        }
                    }
                }
            }

            #endregion

            // #region Load Spawns
            // SpawnInfo.Load(br);
            // #endregion

            br.Close();
        }

        private void LoadBSPMeshes()
        {
            for (int x = 0; x < RawBSPChunks.Length; x++)
            {
                if (RawBSPChunks[x].RawInfoChunks.Length == 0)
                    continue;

                if (RawBSPChunks[x].Mesh != null) RawBSPChunks[x].Mesh.Dispose();
                RawBSPChunks[x].Mesh = new Mesh(RawBSPChunks[x].IndiceCount / 3, RawBSPChunks[x].VerticeCount,
                    MeshFlags.Managed, H2VertexFormat.Format, Map.MDXFrameWork.Device);

                H2VertexFormat[] meshVerts = LoadBSPVerts(x);

                RawBSPChunks[x].Mesh.VertexBuffer.SetData(meshVerts, 0, LockFlags.None);
                RawBSPChunks[x].Mesh.IndexBuffer.SetData(RawBSPChunks[x].Indices, 0, LockFlags.None);

                // Set attribute buffer data, setting the attribute  
                // ID (subset #) for each face in the attribute buffer 
                int[] attribBuffer = RawBSPChunks[x].Mesh.LockAttributeBufferArray(LockFlags.None);

                for (int y = 0; y < RawBSPChunks[x].SubMeshInfo.Length; y++)
                {
                    for (int z = RawBSPChunks[x].SubMeshInfo[y].IndiceStart / 3; z < (RawBSPChunks[x].SubMeshInfo[y].IndiceStart / 3) + (RawBSPChunks[x].SubMeshInfo[y].IndiceCount / 3); z++)
                    {
                        attribBuffer[z] = y;
                    }
                }

                // unlock and update the buffer 
                RawBSPChunks[x].Mesh.UnlockAttributeBuffer(attribBuffer);

                // Updating the attribute table can also be done automatically  
                // by optimizing the mesh in place, like this: 
                int[] adj = new int[RawBSPChunks[x].IndiceCount];
                RawBSPChunks[x].Mesh.GenerateAdjacency(0.1f, adj);
                RawBSPChunks[x].Mesh.OptimizeInPlace(MeshFlags.OptimizeAttributeSort, adj);

                //calculate bounding spheres
                GraphicsStream gs = RawBSPChunks[x].Mesh.LockVertexBuffer(LockFlags.None);

                Vector3 centerVector;
                RawBSPChunks[x].BoundingRadius = Geometry.ComputeBoundingSphere(gs, RawBSPChunks[x].VerticeCount, H2VertexFormat.Format, out centerVector);
                RawBSPChunks[x].BoundingCenter = centerVector;

                RawBSPChunks[x].Mesh.UnlockVertexBuffer();
            }
        }

        private H2VertexFormat[] LoadBSPVerts(int indx)
        {
            H2VertexFormat[] verts = new H2VertexFormat[RawBSPChunks[indx].VerticeCount];

            for (int y = 0; y < RawBSPChunks[indx].VerticeCount; y++)
            {
                verts[y] = new H2VertexFormat(
                    RawBSPChunks[indx].Vertices[y],
                    RawBSPChunks[indx].Normals[y],
                    1,
                    1,
                    RawBSPChunks[indx].UVs[y],
                    RawBSPChunks[indx].LightMapUVs == null ?
                    RawBSPChunks[indx].UVs[y] : RawBSPChunks[indx].LightMapUVs[y]);
            }

            return verts;
        }

        private void LoadPermMeshes()
        {
            if (Map.Prefs.Perms_Load)
            {
                for (int x = 0; x < RawPermChunks.Length; x++)
                {
                    if (RawPermChunks[x].RawInfoChunks.Length == 0)
                        continue;

                    if (RawPermChunks[x].Mesh != null) RawPermChunks[x].Mesh.Dispose();
                    RawPermChunks[x].Mesh = new Mesh(RawPermChunks[x].IndiceCount / 3,
                        RawPermChunks[x].VerticeCount, MeshFlags.Managed, H2VertexFormat.Format, Map.MDXFrameWork.Device);

                    RawPermChunks[x].Mesh.VertexBuffer.SetData(LoadPermVerts(x), 0, LockFlags.None);
                    RawPermChunks[x].Mesh.IndexBuffer.SetData(RawPermChunks[x].Indices, 0, LockFlags.None);

                    // Set attribute buffer data, setting the attribute  
                    // ID (subset #) for each face in the attribute buffer 
                    int[] attribBuffer = RawPermChunks[x].Mesh.LockAttributeBufferArray(LockFlags.None);

                    for (int y = 0; y < RawPermChunks[x].SubMeshInfo.Length; y++)
                    {
                        for (int z = (RawPermChunks[x].SubMeshInfo[y].IndiceStart / 3); z < (RawPermChunks[x].SubMeshInfo[y].IndiceStart / 3) + (RawPermChunks[x].SubMeshInfo[y].IndiceCount / 3); z++)
                        {
                            attribBuffer[z] = y;
                        }
                    }

                    // unlock and update the buffer 
                    RawPermChunks[x].Mesh.UnlockAttributeBuffer(attribBuffer);

                    // Updating the attribute table can also be done automatically  
                    // by optimizing the mesh in place, like this: 

                    int[] adj = new int[RawPermChunks[x].IndiceCount];
                    RawPermChunks[x].Mesh.GenerateAdjacency(0.1f, adj);
                    RawPermChunks[x].Mesh.OptimizeInPlace(MeshFlags.OptimizeAttributeSort, adj);

                    //calculate bounding sphere
                    GraphicsStream gs = RawPermChunks[x].Mesh.LockVertexBuffer(LockFlags.None);

                    Vector3 centerVector;
                    RawPermChunks[x].BoundingRadius = Geometry.ComputeBoundingSphere(gs, RawPermChunks[x].VerticeCount, H2VertexFormat.Format, out centerVector);
                    RawPermChunks[x].BoundingCenter = centerVector;
                    
                    RawPermChunks[x].Mesh.UnlockVertexBuffer();
                }
            }
        }

        private H2VertexFormat[] LoadPermVerts(int indx)
        {
            H2VertexFormat[] verts = new H2VertexFormat[RawPermChunks[indx].VerticeCount];

            for (int y = 0; y < RawPermChunks[indx].VerticeCount; y++)
            {
                verts[y] = new H2VertexFormat(
                    RawPermChunks[indx].Vertices[y],
                    RawPermChunks[indx].Normals[y],
                    1,
                    1,
                    RawPermChunks[indx].UVs[y],
                    RawPermChunks[indx].LightMapUVs == null ?
                    RawPermChunks[indx].UVs[y] : RawPermChunks[indx].LightMapUVs[y]);
            }

            return verts;
        }

        private void FindAlphaTextures()
        {
            //perms
            if (Map.Prefs.Perms_Load && Map.Prefs.Perms_LoadShaders)
            {
                //loop placements
                for (int x = 0; x < RawPermChunks.Length; x++)
                {
                    if (RawPermChunks[x].RawInfoChunks.Length == 0)
                        continue;

                    RawPermChunks[x].AlphaSubsets.Clear();

                    //loop sub clusters
                    for (int y = 0; y < RawPermChunks[x].SubMeshInfo.Length; y++)
                    {
                        int shadIndx = RawPermChunks[x].SubMeshInfo[y].ShaderIndex;
                        if (Shaders[shadIndx].Alpha != Shad.AlphaType.None)
                        {
                            RawPermChunks[x].AlphaSubsets.Add(y);
                        }
                    }
                }
            }

            if (Map.Prefs.BSP_LoadShaders)
            {
                //bsp
                for (int x = 0; x < RawBSPChunks.Length; x++)
                {
                    if (RawBSPChunks[x].RawInfoChunks.Length == 0)
                        continue;

                    RawBSPChunks[x].AlphaSubsets.Clear();

                    //loop sub clusters
                    for (int y = 0; y < RawBSPChunks[x].SubMeshInfo.Length; y++)
                    {
                        int shadIndx = RawBSPChunks[x].SubMeshInfo[y].ShaderIndex;
                        if (Shaders[shadIndx].Alpha != Shad.AlphaType.None)
                        {
                            RawBSPChunks[x].AlphaSubsets.Add(y);
                        }
                    }
                }
            }
        }

        private void LoadLightmapTextures()
        {
            if (Map.Prefs.BSP_LoadLightmaps && BSPLtmpBitmaps != null)
            {
                BSPLtmpTextures = new Texture[BSPLtmpBitmaps.Length];
                for (int x = 0; x < BSPLtmpBitmaps.Length; x++)
                {
                    BSPLtmpTextures[x] = Shad.CreateTexture(Map.MDXFrameWork.Device, BSPLtmpBitmaps[x]);
                }
            }

            if (Map.Prefs.Perms_LoadLightmaps && PermLtmpBitmaps != null)
            {
                PermLtmpTextures = new Texture[PermLtmpBitmaps.Length];
                for (int x = 0; x < PermLtmpBitmaps.Length; x++)
                {
                    PermLtmpTextures[x] = Shad.CreateTexture(Map.MDXFrameWork.Device, PermLtmpBitmaps[x]);
                }
            }
        }

        #endregion

        #region Update, Render

        public void Render()
        {
            //bsp_rend_cnt = 0;
            //perm_rend_cnt = 0;

            #region Render Permutations

            if (Map.Prefs.Perms_Load)
            {
                //loop placements
                for (int x = 0; x < PermPlacements.Length; x++)
                {
                    int rawIndx = PermPlacements[x].RawPermChunkIndex;

                    if (RawPermChunks[rawIndx].RawInfoChunks.Length == 0
                        || !PermPlacements[x].InFrustum)
                        continue;

                    Map.MDXFrameWork.Device.Transform.World = PermPlacements[x].Matrix;
                    //effect.SetValue(eh_world, Map.sbsp.permPlacements[x].mat);
                    //effect.SetValue(eh_wvp, Map.sbsp.permPlacements[x].mat * camera.ViewMatrix * camera.ProjectionMatrix);

                    //loop sub clusters
                    for (int y = 0; y < RawPermChunks[rawIndx].SubMeshInfo.Length; y++)
                    {
                        if (!RawPermChunks[rawIndx].AlphaSubsets.Contains(y))
                        {
                            RenderPermutation(x, y, rawIndx);
                            ResetDevTextureStates();
                        }

                        //perm_rend_cnt++;
                    }

                    for (int y = 0; y < RawPermChunks[rawIndx].AlphaSubsets.Count; y++)
                    {
                        RenderPermutation(x, RawPermChunks[rawIndx].AlphaSubsets[y], rawIndx);
                        ResetDevTextureStates();
                    }
                }
            }

            #endregion

            #region Render BSP

            Map.MDXFrameWork.Device.Transform.World = Matrix.Identity;
            //effect.SetValue(eh_world, Matrix.Identity);
            //effect.SetValue(eh_wvp, Matrix.Identity * camera.ViewMatrix * camera.ProjectionMatrix);

            // Loop clusters.
            for (int x = 0; x < RawBSPChunks.Length; x++)
            {
                if (RawBSPChunks[x].RawInfoChunks.Length == 0
                    || !RawBSPChunks[x].InFrustum)
                    continue;

                // Loop subclusters.
                for (int y = 0; y < RawBSPChunks[x].SubMeshInfo.Length; y++)
                {
                    if (!RawBSPChunks[x].AlphaSubsets.Contains(y))
                    {
                        RenderBSP(x, y);
                        ResetDevTextureStates();
                    }

                    //bsp_rend_cnt++;
                }

                for (int y = 0; y < RawBSPChunks[x].AlphaSubsets.Count; y++)
                {
                    RenderBSP(x, RawBSPChunks[x].AlphaSubsets[y]);
                    ResetDevTextureStates();
                }
            }

            #endregion
        }

        private void RenderPermutation(int x, int y, int rawIndx)
        {
            //load bitmap
            if (Map.Prefs.Perms_LoadShaders)
            {
                int shadIndx = RawPermChunks[rawIndx].SubMeshInfo[y].ShaderIndex;
                SetDevAlphaBlending(Shaders[shadIndx].Alpha);

                //effect.SetValue("g_MeshTexture", Map.sbsp.shaders[shadIndx].MainBitm.Texture);
                if (Shaders[shadIndx].MainBitm != null) Map.MDXFrameWork.Device.SetTexture(0, Shaders[shadIndx].MainBitm.Texture);
                Map.MDXFrameWork.Device.TextureState[0].ColorOperation = TextureOperation.Modulate;
                Map.MDXFrameWork.Device.TextureState[0].ColorArgument1 = TextureArgument.TextureColor;
                Map.MDXFrameWork.Device.TextureState[0].ColorArgument2 = TextureArgument.Current;
                Map.MDXFrameWork.Device.TextureState[0].TextureCoordinateIndex = 0;
            }

            //load lightmap
            if (Map.Prefs.Perms_LoadLightmaps && PermLtmpTextures != null)
            {
                //effect.SetValue("g_LightTexture", Map.sbsp.permLtmpTextures[x]);
                Map.MDXFrameWork.Device.SetTexture(1, PermLtmpTextures[x]);
                Map.MDXFrameWork.Device.TextureState[1].ColorOperation = TextureOperation.Modulate2X;
                Map.MDXFrameWork.Device.TextureState[1].ColorArgument1 = TextureArgument.TextureColor;
                Map.MDXFrameWork.Device.TextureState[1].ColorArgument2 = TextureArgument.Current;
                Map.MDXFrameWork.Device.TextureState[1].TextureCoordinateIndex = 2;
            }

            //choose material colour
            if (RawPermChunks[rawIndx].SelectedSubsets.Contains(y))
            {
                if (RawPermChunks[rawIndx].IsClickedObject)
                {
                    DevMaterial.Diffuse = Color.Blue;
                    DevMaterial.Ambient = Color.Blue;
                }
                else
                {
                    //effect.SetValue(eh_matAmbient, new Vector4(255, 255, 0, 0));
                    //effect.SetValue(eh_matDiffuse, new Vector4(255, 255, 0, 0));
                    DevMaterial.Diffuse = Color.Red;
                    DevMaterial.Ambient = Color.Red;
                }
            }
            else
            {
                //effect.SetValue(eh_matAmbient, new Vector4(1, 1, 1, 1));
                //effect.SetValue(eh_matDiffuse, new Vector4(1, 1, 1, 1));
                DevMaterial.Diffuse = System.Drawing.Color.White;
                DevMaterial.Ambient = System.Drawing.Color.White;
            }
            Map.MDXFrameWork.Device.Material = DevMaterial;

            //draw
            //effect.BeginPass(pass);

            RawPermChunks[rawIndx].Mesh.DrawSubset(y);
            //perm_rend_cnt++;

            //effect.EndPass();

            //draw again in wireframe if bitmaps have not been loaded
            if (!Map.Prefs.Perms_LoadShaders)
            {
                Map.MDXFrameWork.Device.RenderState.FillMode = FillMode.WireFrame;

                DevMaterial.Diffuse = System.Drawing.Color.Black;
                DevMaterial.Ambient = System.Drawing.Color.Black;
                Map.MDXFrameWork.Device.Material = DevMaterial;

                //effect.BeginPass(pass);
                RawPermChunks[rawIndx].Mesh.DrawSubset(y);
                //effect.EndPass();

                Map.MDXFrameWork.Device.RenderState.FillMode = FillMode.Solid;
            }
        }

        private void RenderBSP(int x, int yy)
        {
            bool loadLtmp = true;
            int shadIndx = RawBSPChunks[x].SubMeshInfo[yy].ShaderIndex;

            //load bitmap
            if (Map.Prefs.BSP_LoadShaders)
            {
                SetDevAlphaBlending(Shaders[shadIndx].Alpha);

                if (Shaders[shadIndx].MainBitm != null)
                {
                    Map.MDXFrameWork.Device.SetTexture(0, Shaders[shadIndx].MainBitm.Texture);

                    Map.MDXFrameWork.Device.TextureState[0].ColorOperation = TextureOperation.Modulate;
                    Map.MDXFrameWork.Device.TextureState[0].ColorArgument1 = TextureArgument.TextureColor;
                    Map.MDXFrameWork.Device.TextureState[0].ColorArgument2 = TextureArgument.Current;
                    //Map.MDXFrameWork.Device.TextureState[0].AlphaOperation = TextureOperation.BlendTextureAlpha;
                    //Map.MDXFrameWork.Device.TextureState[0].AlphaArgument1 = TextureArgument.TextureColor;
                    //Map.MDXFrameWork.Device.TextureState[0].AlphaArgument2 = TextureArgument.Diffuse;
                    Map.MDXFrameWork.Device.TextureState[0].AlphaOperation = TextureOperation.SelectArg1;
                    Map.MDXFrameWork.Device.TextureState[0].AlphaArgument1 = TextureArgument.TextureColor;

                    Map.MDXFrameWork.Device.TextureState[0].TextureCoordinateIndex = 0;
                }

                /*
                if (Shaders[shadIndx].BumpBitm != null)
                {
                    Map.MDXFrameWork.Device.SetTexture(1, Shaders[shadIndx].BumpBitm.Texture);
                    //device.Transform.Texture2 = Map.sbsp.shaderInfos[shadIndx].bitmUVWScales[1].mat;
                    //device.TextureState[0].TextureTransform = TextureTransform.Count3;
                    Map.MDXFrameWork.Device.TextureState[1].ColorOperation = TextureOperation.BumpEnvironmentMap; //(TextureOperation)OMG;
                    Map.MDXFrameWork.Device.TextureState[1].ColorArgument1 = TextureArgument.TextureColor;
                    Map.MDXFrameWork.Device.TextureState[1].ColorArgument2 = TextureArgument.TFactor;
                    //device.RenderState.TextureFactor = Color.FromArgb(0, 0, 1).ToArgb();
                    Map.MDXFrameWork.Device.TextureState[1].BumpEnvironmentMaterial00 = 1.0f;
                    Map.MDXFrameWork.Device.TextureState[1].BumpEnvironmentMaterial01 = 0.0f;
                    Map.MDXFrameWork.Device.TextureState[1].BumpEnvironmentMaterial10 = 0.0f;
                    Map.MDXFrameWork.Device.TextureState[1].BumpEnvironmentMaterial11 = 1.0f;
                    //Map.MDXFrameWork.Device.TextureState[1].BumpEnvironmentLuminanceScale = 1.0f;
                    //Map.MDXFrameWork.Device.TextureState[1].BumpEnvironmentLuminanceOffset = 0.0f;
                    Map.MDXFrameWork.Device.TextureState[1].TextureCoordinateIndex = 0;
                }
                */

                /*
                if (Map.sbsp.shaders[shadIndx].CubeBitm != null)
                {
                    device.SetTexture(3, Map.sbsp.shaders[shadIndx].CubeBitm.CubeTexture);
                    //device.Transform.Texture1 = Map.sbsp.shaderInfos[shadIndx].bitmUVWScales[1].mat;
                    device.TextureState[3].TextureTransform = TextureTransform.Count2;
                    device.TextureState[3].ColorOperation = TextureOperation.Modulate;
                    device.TextureState[3].ColorArgument1 = TextureArgument.TextureColor;
                    device.TextureState[3].ColorArgument2 = TextureArgument.Current;
                    //device.RenderState.TextureFactor = Color.FromArgb(0, 0, 1).ToArgb();
                }
                */
            }

            // load lightmap
            if (Map.Prefs.BSP_LoadLightmaps && loadLtmp && BSPLtmpTextures != null)
            {
                //effect.SetValue("g_LightTexture", Map.sbsp.ltmpTextures[x]);
                Map.MDXFrameWork.Device.SetTexture(1, BSPLtmpTextures[x]);
                //device.Transform.Texture1 = Map.sbsp.shaderInfos[shadIndx].bitmUVWScales[1].mat;
                Map.MDXFrameWork.Device.TextureState[1].ColorOperation = TextureOperation.Modulate2X;
                Map.MDXFrameWork.Device.TextureState[1].ColorArgument1 = TextureArgument.TextureColor;
                Map.MDXFrameWork.Device.TextureState[1].ColorArgument2 = TextureArgument.Current;
                Map.MDXFrameWork.Device.TextureState[1].TextureCoordinateIndex = 2;
            }

            //choose material colour
            if (RawBSPChunks[x].SelectedSubsets.Contains(yy))
            {
                if (RawBSPChunks[x].IsClickedObject)
                {
                    DevMaterial.Diffuse = Color.Blue;
                    DevMaterial.Ambient = Color.Blue;
                }
                else
                {
                    //effect.SetValue(eh_matAmbient, new Vector4(255, 255, 0, 0));
                    //effect.SetValue(eh_matDiffuse, new Vector4(255, 255, 0, 0));
                    DevMaterial.Diffuse = Color.Red;
                    DevMaterial.Ambient = Color.Red;
                }
            }
            else
            {
                //effect.SetValue(eh_matAmbient, new Vector4(1, 1, 1, 1));
                //effect.SetValue(eh_matDiffuse, new Vector4(1, 1, 1, 1));
                DevMaterial.Diffuse = System.Drawing.Color.White;
                DevMaterial.Ambient = System.Drawing.Color.White;
            }
            Map.MDXFrameWork.Device.Material = DevMaterial;

            // Draw.
            //effect.BeginPass(pass);

            RawBSPChunks[x].Mesh.DrawSubset(yy);
            //bsp_rend_cnt++;

            //effect.EndPass();

            //draw again in wireframe if bitmaps have not been loaded
            if (!Map.Prefs.BSP_LoadShaders)
            {
                Map.MDXFrameWork.Device.RenderState.FillMode = FillMode.WireFrame;

                DevMaterial.Diffuse = System.Drawing.Color.Black;
                DevMaterial.Ambient = System.Drawing.Color.Black;
                Map.MDXFrameWork.Device.Material = DevMaterial;

                //effect.BeginPass(pass);
                RawBSPChunks[x].Mesh.DrawSubset(yy);
                //effect.EndPass();

                Map.MDXFrameWork.Device.RenderState.FillMode = FillMode.Solid;
            }
        }

        #endregion

        #region Misc

        public void ResetDevTextureStates()
        {
            Map.MDXFrameWork.Device.SamplerState[0].AddressU = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[0].AddressV = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[1].AddressU = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[1].AddressV = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[2].AddressU = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[2].AddressV = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[3].AddressU = TextureAddress.Wrap;
            Map.MDXFrameWork.Device.SamplerState[3].AddressV = TextureAddress.Wrap;

            Map.MDXFrameWork.Device.Transform.Texture0 = Matrix.Identity;
            Map.MDXFrameWork.Device.TextureState[0].TextureTransform = TextureTransform.Disable;
            Map.MDXFrameWork.Device.TextureState[0].TextureCoordinateIndex = 0;
            Map.MDXFrameWork.Device.TextureState[0].ColorOperation = TextureOperation.Disable;
            Map.MDXFrameWork.Device.SetTexture(0, null);

            Map.MDXFrameWork.Device.Transform.Texture1 = Matrix.Identity;
            Map.MDXFrameWork.Device.TextureState[1].TextureTransform = TextureTransform.Disable;
            Map.MDXFrameWork.Device.TextureState[1].TextureCoordinateIndex = 1;
            Map.MDXFrameWork.Device.TextureState[1].ColorOperation = TextureOperation.Disable;
            Map.MDXFrameWork.Device.SetTexture(1, null);

            Map.MDXFrameWork.Device.Transform.Texture2 = Matrix.Identity;
            Map.MDXFrameWork.Device.TextureState[2].TextureTransform = TextureTransform.Disable;
            Map.MDXFrameWork.Device.TextureState[2].TextureCoordinateIndex = 2;
            Map.MDXFrameWork.Device.TextureState[2].ColorOperation = TextureOperation.Disable;
            Map.MDXFrameWork.Device.SetTexture(2, null);

            Map.MDXFrameWork.Device.Transform.Texture3 = Matrix.Identity;
            Map.MDXFrameWork.Device.TextureState[3].TextureTransform = TextureTransform.Disable;
            Map.MDXFrameWork.Device.TextureState[3].TextureCoordinateIndex = 3;
            Map.MDXFrameWork.Device.TextureState[3].ColorOperation = TextureOperation.Disable;
            Map.MDXFrameWork.Device.SetTexture(3, null);
        }

        public void SetDevAlphaBlending(Shad.AlphaType alpha)
        {
            switch (alpha)
            {
                case Shad.AlphaType.AlphaBlend:
                    Map.MDXFrameWork.Device.RenderState.AlphaBlendEnable = true;
                    Map.MDXFrameWork.Device.RenderState.AlphaTestEnable = false;
                    break;

                case Shad.AlphaType.AlphaTest:
                    Map.MDXFrameWork.Device.RenderState.AlphaBlendEnable = false;
                    Map.MDXFrameWork.Device.RenderState.AlphaTestEnable = true;
                    break;

                default:
                    Map.MDXFrameWork.Device.RenderState.AlphaBlendEnable = false;
                    Map.MDXFrameWork.Device.RenderState.AlphaTestEnable = false;
                    break;
            }
        }

        #endregion

        #endregion

        public class BSPDeclaration
        {
            #region Instance Data

            private int __declarationOffset__;
            private int __identity__;
            private int __metaOffset__;
            private int __metaSize__;
            private int __magic__;
            private Lightmap __lightmap__;

            #endregion

            #region Accessors

            /// <summary>
            /// DeclarationOffset
            /// </summary>
            public int DeclarationOffset { get { return __declarationOffset__; } set { __declarationOffset__ = value; } }
            /// <summary>
            /// Identity
            /// </summary>
            public int Identity { get { return __identity__; } set { __identity__ = value; } }
            /// <summary>
            /// MetaOffset
            /// </summary>
            public int MetaOffset { get { return __metaOffset__; } set { __metaOffset__ = value; } }
            /// <summary>
            /// MetaSize
            /// </summary>
            public int MetaSize { get { return __metaSize__; } set { __metaSize__ = value; } }
            /// <summary>
            /// Magic
            /// </summary>
            public int Magic { get { return __magic__; } set { __magic__ = value; } }
            /// <summary>
            /// Lightmap
            /// </summary>
            public Lightmap Lightmap { get { return __lightmap__; } set { __lightmap__ = value; } }

            #endregion

            public BSPDeclaration()
            {
                this.Lightmap = new Lightmap();
            }
        }

        public class Lightmap
        {
            #region Instance Data

            private int __identity__;
            private int __metaOffset__;
            private int __metaSize__;
            private int __palettesOffset__;
            private List<Bitm.RGBA_ByteColor[]> __palettes__;
            private int[] __bSPSubmapIndices__;
            private int[] __bSPPaletteIndices__;
            private int[] __permSubmapIndices__;
            private int[] __permPaletteIndices__;

            #endregion

            #region Accessors

            /// <summary>
            /// Identity
            /// </summary>
            public int Identity { get { return __identity__; } set { __identity__ = value; } }
            /// <summary>
            /// MetaOffset
            /// </summary>
            public int MetaOffset { get { return __metaOffset__; } set { __metaOffset__ = value; } }
            /// <summary>
            /// MetaSize
            /// </summary>
            public int MetaSize { get { return __metaSize__; } set { __metaSize__ = value; } }
            /// <summary>
            /// PalettesOffset
            /// </summary>
            public int PalettesOffset { get { return __palettesOffset__; } set { __palettesOffset__ = value; } }
            /// <summary>
            /// Palettes
            /// </summary>
            public List<Bitm.RGBA_ByteColor[]> Palettes { get { return __palettes__; } set { __palettes__ = value; } }
            /// <summary>
            /// BSPSubmapIndices
            /// </summary>
            public int[] BSPSubmapIndices { get { return __bSPSubmapIndices__; } set { __bSPSubmapIndices__ = value; } }
            /// <summary>
            /// BSPPaletteIndices
            /// </summary>
            public int[] BSPPaletteIndices { get { return __bSPPaletteIndices__; } set { __bSPPaletteIndices__ = value; } }
            /// <summary>
            /// PermSubmapIndices
            /// </summary>
            public int[] PermSubmapIndices { get { return __permSubmapIndices__; } set { __permSubmapIndices__ = value; } }
            /// <summary>
            /// PermPaletteIndices
            /// </summary>
            public int[] PermPaletteIndices { get { return __permPaletteIndices__; } set { __permPaletteIndices__ = value; } }

            #endregion

            public Lightmap()
            {
                this.Palettes = new List<Bitm.RGBA_ByteColor[]>();
            }

            public void WritePalettes(string fileName)
            {
                BitWriter bw = new BitWriter(fileName, Endian.Little);

                bw.BaseStream.Position = this.PalettesOffset;
                foreach (Bitm.RGBA_ByteColor[] pal in Palettes)
                {
                    for (int x = 0; x < 256; x++)
                    {
                        bw.Write(pal[x].R);
                        bw.Write(pal[x].G);
                        bw.Write(pal[x].B);
                        bw.Write(pal[x].A);
                    }
                }
                bw.Close();
            }
        }

        public class RawBSPModelChunk
        {
            #region Instance Data

            private int __shaderIndex__;
            private int __offset__;
            private int __size__;

            #endregion

            #region Accessors

            /// <summary>
            /// ShaderIndex
            /// </summary>
            public int ShaderIndex { get { return __shaderIndex__; } set { __shaderIndex__ = value; } }
            /// <summary>
            /// Offset
            /// </summary>
            public int Offset { get { return __offset__; } set { __offset__ = value; } }
            /// <summary>
            /// Size
            /// </summary>
            public int Size { get { return __size__; } set { __size__ = value; } }

            #endregion
        }

        public class ModelSubMeshInfo
        {
            #region Instance Data

            private int __shaderIndex__;
            private int __indiceStart__;
            private int __indiceCount__;

            #endregion

            #region Accessors

            /// <summary>
            /// ShaderIndex
            /// </summary>
            public int ShaderIndex { get { return __shaderIndex__; } set { __shaderIndex__ = value; } }
            /// <summary>
            /// IndiceStart
            /// </summary>
            public int IndiceStart { get { return __indiceStart__; } set { __indiceStart__ = value; } }
            /// <summary>
            /// IndiceCount
            /// </summary>
            public int IndiceCount { get { return __indiceCount__; } set { __indiceCount__ = value; } }

            #endregion

            public ModelSubMeshInfo(BitReader hr)
            {
                this.ShaderIndex = hr.ReadUInt16();
                this.IndiceStart = hr.ReadUInt16();
                this.IndiceCount = hr.ReadUInt16();
            }
        }

        public class RawInfoChunk
        {
            #region Instance Data

            private int __offset__;
            private int __size__;
            private int __chunkSize__;
            private int __chunkCount__;

            #endregion

            #region Accessors

            /// <summary>
            /// Offset
            /// </summary>
            public int Offset { get { return __offset__; } set { __offset__ = value; } }
            /// <summary>
            /// Size
            /// </summary>
            public int Size { get { return __size__; } set { __size__ = value; } }
            /// <summary>
            /// ChunkSize
            /// </summary>
            public int ChunkSize { get { return __chunkSize__; } set { __chunkSize__ = value; } }
            /// <summary>
            /// ChunkCount
            /// </summary>
            public int ChunkCount { get { return __chunkCount__; } set { __chunkCount__ = value; } }

            #endregion

            public RawInfoChunk(BitReader hr)
            {
                this.ChunkSize = hr.ReadUInt16();
                this.Size = hr.ReadInt32();
                this.Offset = hr.ReadInt32();
                if (this.ChunkSize == 0) this.ChunkSize = this.Size;
                this.ChunkCount = this.Size / this.ChunkSize;
            }
        }

        public class BaseRawDataChunk
        {
            #region Instance Data

            private ushort __verticeCount__;
            private ushort __faceCount__;
            private int __headerSize__;
            private RawInfoChunk[] __rawInfoChunks__;
            private ModelSubMeshInfo[] __subMeshInfo__;
            private ushort __indiceCount__;
            private short[] __indices__;
            private Vector3[] __vertices__;
            private Vector2[] __uVs__;
            private Vector2[] __lightMapUVs__;
            private Vector3[] __normals__;
            private Vector3[] __binormals__;
            private Vector3[] __tangents__;
            private Mesh __mesh__;
            private bool __inFrustum__;
            private float __boundingRadius__;
            private Vector3 __boundingCenter__;
            private List<int> __alphaSubsets__;

            #endregion

            #region Accessors

            /// <summary>
            /// VerticeCount
            /// </summary>
            public ushort VerticeCount { get { return __verticeCount__; } set { __verticeCount__ = value; } }
            /// <summary>
            /// FaceCount
            /// </summary>
            public ushort FaceCount { get { return __faceCount__; } set { __faceCount__ = value; } }
            /// <summary>
            /// HeaderSize
            /// </summary>
            public int HeaderSize { get { return __headerSize__; } set { __headerSize__ = value; } }
            /// <summary>
            /// RawInfoChunks
            /// </summary>
            public RawInfoChunk[] RawInfoChunks { get { return __rawInfoChunks__; } set { __rawInfoChunks__ = value; } }
            /// <summary>
            /// SubMeshInfo
            /// </summary>
            public ModelSubMeshInfo[] SubMeshInfo { get { return __subMeshInfo__; } set { __subMeshInfo__ = value; } }
            /// <summary>
            /// IndiceCount
            /// </summary>
            public ushort IndiceCount { get { return __indiceCount__; } set { __indiceCount__ = value; } }
            /// <summary>
            /// Indices
            /// </summary>
            public short[] Indices { get { return __indices__; } set { __indices__ = value; } }
            /// <summary>
            /// Vertices
            /// </summary>
            public Vector3[] Vertices { get { return __vertices__; } set { __vertices__ = value; } }
            /// <summary>
            /// UVs
            /// </summary>
            public Vector2[] UVs { get { return __uVs__; } set { __uVs__ = value; } }
            /// <summary>
            /// LightMapUVs
            /// </summary>
            public Vector2[] LightMapUVs { get { return __lightMapUVs__; } set { __lightMapUVs__ = value; } }
            /// <summary>
            /// Normals
            /// </summary>
            public Vector3[] Normals { get { return __normals__; } set { __normals__ = value; } }
            /// <summary>
            /// Binormals
            /// </summary>
            public Vector3[] Binormals { get { return __binormals__; } set { __binormals__ = value; } }
            /// <summary>
            /// Tangents
            /// </summary>
            public Vector3[] Tangents { get { return __tangents__; } set { __tangents__ = value; } }
            /// <summary>
            /// Mesh
            /// </summary>
            public Mesh Mesh { get { return __mesh__; } set { __mesh__ = value; } }
            /// <summary>
            /// InFrustum
            /// </summary>
            public bool InFrustum { get { return __inFrustum__; } set { __inFrustum__ = value; } }
            /// <summary>
            /// BoundingRadius
            /// </summary>
            public float BoundingRadius { get { return __boundingRadius__; } set { __boundingRadius__ = value; } }
            /// <summary>
            /// BoundingCenter
            /// </summary>
            public Vector3 BoundingCenter { get { return __boundingCenter__; } set { __boundingCenter__ = value; } }
            /// <summary>
            /// AlphaSubsets
            /// </summary>
            public List<int> AlphaSubsets { get { return __alphaSubsets__; } set { __alphaSubsets__ = value; } }

            #endregion

            #region De/Compression Functions

            public Vector3 DecompressNormal(int compressedNormal)
            {
                int xx = compressedNormal & 0x3ff;
                float xxx = Convert.ToSingle(xx) / Convert.ToSingle(0x3ff);
                int bitx = ((compressedNormal >> 10) & 0x1);
                if (bitx == 1)
                {
                    xxx = 1 - xxx;
                    xxx = -xxx;
                }

                int yy = ((compressedNormal >> 11) & 0x3ff);
                float yyy = Convert.ToSingle(yy) / Convert.ToSingle(0x3ff);
                int bity = ((compressedNormal >> 21) & 0x1);
                if (bity == 1)
                {
                    yyy = 1 - yyy;
                    yyy = -yyy;
                }

                int zz = ((compressedNormal >> 22) & 0x1ff);
                float zzz = Convert.ToSingle(zz) / Convert.ToSingle(0x1ff);
                int bitz = ((compressedNormal >> 31) & 0x1);
                if (bitz == 1)
                {
                    zzz = 1 - zzz;
                    zzz = -zzz;
                }

                return new Vector3(xxx, yyy, zzz);
            }

            public int CompressNormal(Vector3 normal)
            {
                bool negx = false;
                if (normal.X < 0) { negx = true; normal.X = -normal.X; }
                float bbb = normal.X * Convert.ToSingle(0x3ff);
                int b = Convert.ToInt32(bbb);
                if (negx) b = -b + 1 | 0x400;

                bool negy = false;
                if (normal.Y < 0) { negy = true; normal.Y = -normal.Y; }
                float bbb2 = normal.Y * Convert.ToSingle(0x3ff);
                int b2 = Convert.ToInt32(bbb2);
                if (negy) b2 = -b2 + 1 | 0x400;
                b2 <<= 11;

                bool negz = false;
                if (normal.Z < 0) { negz = true; normal.Z = -normal.Z; }
                float bbb3 = normal.Z * Convert.ToSingle(0x1ff);
                int b3 = Convert.ToInt32(bbb3);
                if (negz) b3 = -b3 + 1 | 0x200;
                b3 <<= 22;

                int result = b3 | b2 | b;
                return result;
            }

            public float DecompressVertice(float input, float min, float max)
            {
                float percent = (input + 32768) / 65535;
                float result = (percent * (max - min)) + min;
                return result;
            }

            public float CompressVertice(float input, float min, float max)
            {
                float result = input - min;
                result = (result / (max - min));
                result = result * 65535;
                result = result - 32768;
                return result;
            }

            #endregion
        }

        public class RawBSPChunk : BaseRawDataChunk
        {
            public bool IsClickedObject;

            #region Instance Data

            private List<int> __selectedSubsets__;

            #endregion

            #region Accessors

            /// <summary>
            /// SelectedSubsets
            /// </summary>
            public List<int> SelectedSubsets { get { return __selectedSubsets__; } set { __selectedSubsets__ = value; } }

            #endregion

            public RawBSPChunk(BitReader br, BSPDeclaration bspD, RawBSPModelChunk rawBSPModelChunk)
            {
                this.SelectedSubsets = new List<int>();
                this.AlphaSubsets = new List<int>();

                this.VerticeCount = br.ReadUInt16();
                this.FaceCount = br.ReadUInt16();

                //48
                br.BaseStream.Position += 44;
                int HeaderSize = br.ReadInt32() + 8;

                //52
                br.BaseStream.Position += 4;
                int cc = br.ReadInt32();
                int tr = br.ReadInt32() - bspD.Magic;

                RawInfoChunks = new RawInfoChunk[cc];
                if (cc == 0) return;

                for (int x = 0; x < cc; x++)
                {
                    br.BaseStream.Position = tr + (x * 16) + 6;
                    RawInfoChunks[x] = new RawInfoChunk(br);
                }

                // Mesh
                this.SubMeshInfo = new ModelSubMeshInfo[RawInfoChunks[0].ChunkCount];

                for (int x = 0; x < RawInfoChunks[0].ChunkCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize +
                        RawInfoChunks[0].Offset + (x * 72) + 4;

                    SubMeshInfo[x] = new ModelSubMeshInfo(br);
                }

                int indicechunk = 0;

                for (int x = 0; x < RawInfoChunks.Length; x++)
                {
                    if (RawInfoChunks[x].ChunkSize == 2)
                    {
                        indicechunk = x;
                        break;
                    }
                }

                int normalchunk = 0;
                int verticechunk = 0;
                int uvchunk = 0;

                for (int x = indicechunk; x < RawInfoChunks.Length; x++)
                {
                    if (RawInfoChunks[x].ChunkCount == 1)
                    {
                        verticechunk = x;
                        uvchunk = x + 1;
                        normalchunk = x + 2;
                        break;
                    }
                }

                br.BaseStream.Position = rawBSPModelChunk.Offset + 40;
                this.IndiceCount = br.ReadUInt16();

                this.Indices = new short[this.IndiceCount];
                br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize + this.RawInfoChunks[indicechunk].Offset;

                for (int x = 0; x < this.IndiceCount; x++)
                    this.Indices[x] = (short)br.ReadUInt16();

                this.Vertices = new Vector3[this.VerticeCount];
                this.RawInfoChunks[verticechunk].ChunkSize = this.RawInfoChunks[verticechunk].Size / VerticeCount;

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize + this.RawInfoChunks[verticechunk].Offset
                        + (this.RawInfoChunks[verticechunk].ChunkSize * x);

                    Vertices[x] = new Vector3(br.ReadSingle(), br.ReadSingle(), br.ReadSingle());
                }

                this.UVs = new Vector2[this.VerticeCount];
                this.RawInfoChunks[uvchunk].ChunkSize = 8;

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize + this.RawInfoChunks[uvchunk].Offset
                        + (this.RawInfoChunks[uvchunk].ChunkSize * x);

                    this.UVs[x] = new Vector2();
                    this.UVs[x].X = br.ReadSingle();
                    this.UVs[x].Y = br.ReadSingle();
                    //this.UVs[x].X = DecompressVertice(hr.ReadSingle(), -1, 1);
                    //this.UVs[x].Y = DecompressVertice(hr.ReadSingle(), -1, 1);
                }

                this.RawInfoChunks[normalchunk].ChunkSize = 12;

                this.Normals = new Vector3[this.VerticeCount];
                this.Binormals = new Vector3[this.VerticeCount];
                this.Tangents = new Vector3[this.VerticeCount];

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize +
                        this.RawInfoChunks[normalchunk].Offset + (this.RawInfoChunks[normalchunk].ChunkSize * x);

                    this.Normals[x] = DecompressNormal(br.ReadInt32());
                    this.Binormals[x] = DecompressNormal(br.ReadInt32());
                    this.Tangents[x] = DecompressNormal(br.ReadInt32());
                }

                int lightmapuvchunk = -1;
                for (int x = normalchunk + 1; x < RawInfoChunks.Length; x++)
                {
                    if (RawInfoChunks[x].ChunkSize == 3)
                    {
                        lightmapuvchunk = x;
                        break;
                    }
                }
                if (lightmapuvchunk == -1) return;

                this.LightMapUVs = new Vector2[this.VerticeCount];
                RawInfoChunks[lightmapuvchunk].ChunkSize = 4;

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize +
                        this.RawInfoChunks[lightmapuvchunk].Offset + (this.RawInfoChunks[lightmapuvchunk].ChunkSize * x);

                    this.LightMapUVs[x] = new Vector2(DecompressVertice(Convert.ToSingle(br.ReadInt16()), -1, 1),
                        DecompressVertice(Convert.ToSingle(br.ReadInt16()), -1, 1));
                }
            }
        }

        public class RawPermChunk : BaseRawDataChunk
        {
            public bool IsClickedObject;

            #region Instance Data

            private List<int> __selectedSubsets__;

            #endregion

            #region Accessors

            /// <summary>
            /// SelectedSubsets
            /// </summary>
            public List<int> SelectedSubsets { get { return __selectedSubsets__; } set { __selectedSubsets__ = value; } }

            #endregion

            public RawPermChunk(BitReader br, BSPDeclaration bspD, RawBSPModelChunk rawBSPModelChunk)
            {
                this.SelectedSubsets = new List<int>();
                this.AlphaSubsets = new List<int>();

                this.VerticeCount = br.ReadUInt16();
                this.FaceCount = br.ReadUInt16();

                //48
                br.BaseStream.Position += 44;
                this.HeaderSize = br.ReadInt32() + 8;

                //52
                br.BaseStream.Position += 4;
                int cc = br.ReadInt32();
                int tr = br.ReadInt32() - bspD.Magic;

                this.RawInfoChunks = new RawInfoChunk[cc];
                for (int x = 0; x < cc; x++)
                {
                    br.BaseStream.Position = tr + (x * 16) + 6;
                    this.RawInfoChunks[x] = new RawInfoChunk(br);
                }
                if (this.RawInfoChunks.Length == 0) return;

                // Mesh
                this.SubMeshInfo = new ModelSubMeshInfo[this.RawInfoChunks[0].ChunkCount];

                for (int x = 0; x < this.RawInfoChunks[0].ChunkCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize +
                        this.RawInfoChunks[0].Offset + (x * 72) + 4;

                    this.SubMeshInfo[x] = new ModelSubMeshInfo(br);
                }

                int indicechunk = 0;

                for (int x = 0; x < RawInfoChunks.Length; x++)
                {
                    if (RawInfoChunks[x].ChunkSize == 2)
                    {
                        indicechunk = x;
                        break;
                    }
                }

                int normalchunk = 0;
                int verticechunk = 0;
                int uvchunk = 0;

                for (int x = indicechunk; x < RawInfoChunks.Length; x++)
                {
                    if (RawInfoChunks[x].ChunkCount == 1)
                    {
                        verticechunk = x;
                        uvchunk = x + 1;
                        normalchunk = x + 2;
                        break;
                    }
                }

                br.BaseStream.Position = rawBSPModelChunk.Offset + 40;
                this.IndiceCount = br.ReadUInt16();

                this.Indices = new short[this.IndiceCount];
                br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize + this.RawInfoChunks[indicechunk].Offset;

                for (int x = 0; x < this.IndiceCount; x++)
                    this.Indices[x] = (short)br.ReadUInt16();

                this.Vertices = new Vector3[this.VerticeCount];
                this.RawInfoChunks[verticechunk].ChunkSize = this.RawInfoChunks[verticechunk].Size / VerticeCount;

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize + this.RawInfoChunks[verticechunk].Offset
                        + (this.RawInfoChunks[verticechunk].ChunkSize * x);

                    Vertices[x] = new Vector3(br.ReadSingle(), br.ReadSingle(), br.ReadSingle());
                }

                this.UVs = new Vector2[this.VerticeCount];
                this.RawInfoChunks[uvchunk].ChunkSize = 8;

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize + this.RawInfoChunks[uvchunk].Offset
                        + (this.RawInfoChunks[uvchunk].ChunkSize * x);

                    this.UVs[x] = new Vector2(br.ReadSingle(), br.ReadSingle());
                }

                this.RawInfoChunks[normalchunk].ChunkSize = 12;

                this.Normals = new Vector3[this.VerticeCount];
                this.Binormals = new Vector3[this.VerticeCount];
                this.Tangents = new Vector3[this.VerticeCount];

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize +
                        this.RawInfoChunks[normalchunk].Offset + (this.RawInfoChunks[normalchunk].ChunkSize * x);

                    this.Normals[x] = DecompressNormal(br.ReadInt32());
                    this.Binormals[x] = DecompressNormal(br.ReadInt32());
                    this.Tangents[x] = DecompressNormal(br.ReadInt32());
                }

                int lightmapuvchunk = -1;
                for (int x = normalchunk + 1; x < RawInfoChunks.Length; x++)
                {
                    if (RawInfoChunks[x].ChunkSize == 3)
                    {
                        lightmapuvchunk = x;
                        break;
                    }
                }
                if (lightmapuvchunk == -1) return;

                this.LightMapUVs = new Vector2[this.VerticeCount];
                RawInfoChunks[lightmapuvchunk].ChunkSize = 4;

                for (int x = 0; x < this.VerticeCount; x++)
                {
                    br.BaseStream.Position = rawBSPModelChunk.Offset + HeaderSize +
                        this.RawInfoChunks[lightmapuvchunk].Offset + (this.RawInfoChunks[lightmapuvchunk].ChunkSize * x);

                    this.LightMapUVs[x] = new Vector2(DecompressVertice(Convert.ToSingle(br.ReadInt16()), -1, 1),
                        DecompressVertice(Convert.ToSingle(br.ReadInt16()), -1, 1));
                }
            }
        }

        public class PermPlacement
        {
            #region Instance Data

            //private long __startInMeta__;
            private float __scale__;
            private Vector3 __position__;
            private bool __inFrustum__;
            private short __rawPermChunkIndex__;
            //private bool __mDown__;
            //private Point __old_pos__;

            #endregion

            #region Accessors

            /// <summary>
            /// StartInMeta
            /// </summary>
            //public long StartInMeta { get { return __startInMeta__; } set { __startInMeta__ = value; } }
            /// <summary>
            /// Scale
            /// </summary>
            public float Scale { get { return __scale__; } set { __scale__ = value; } }
            /// <summary>
            /// 
            /// </summary>
            public Matrix Matrix;
            /// <summary>
            /// Position
            /// </summary>
            public Vector3 Position { get { return __position__; } set { __position__ = value; } }
            /// <summary>
            /// InFrustum
            /// </summary>
            public bool InFrustum { get { return __inFrustum__; } set { __inFrustum__ = value; } }
            /// <summary>
            /// RawPermChunkIndx
            /// </summary>
            public short RawPermChunkIndex { get { return __rawPermChunkIndex__; } set { __rawPermChunkIndex__ = value; } }
            /// <summary>
            /// MDown
            /// </summary>
            //public bool MDown { get { return __mDown__; } set { __mDown__ = value; } }
            /// <summary>
            /// Old_pos
            /// </summary>
            //public Point Old_pos { get { return __old_pos__; } set { __old_pos__ = value; } }
            /// <summary>
            /// PositionX
            /// </summary>
            public float PositionX { get { return Position.X; } set { Matrix.M41 = value; __position__.X = value; } }
            /// <summary>
            /// PositionY
            /// </summary>
            public float PositionY { get { return Position.Y; } set { Matrix.M42 = value; __position__.Y = value; } }
            /// <summary>
            /// PositionZ
            /// </summary>
            public float PositionZ { get { return Position.Z; } set { Matrix.M43 = value; __position__.Z = value; } }

            #endregion

            public PermPlacement(BitReader br)
            {
                //StartInMeta = br.BaseStream.Position;

                Scale = br.ReadSingle();

                Matrix.M11 = br.ReadSingle();
                Matrix.M12 = br.ReadSingle();
                Matrix.M13 = br.ReadSingle();
                Matrix.M14 = 0;

                Matrix.M21 = br.ReadSingle();
                Matrix.M22 = br.ReadSingle();
                Matrix.M23 = br.ReadSingle();
                Matrix.M24 = 0;

                Matrix.M31 = br.ReadSingle();
                Matrix.M32 = br.ReadSingle();
                Matrix.M33 = br.ReadSingle();
                Matrix.M34 = 0;

                Matrix.M41 = br.ReadSingle();
                Matrix.M42 = br.ReadSingle();
                Matrix.M43 = br.ReadSingle();
                Matrix.M44 = 1;

                Position = new Vector3(Matrix.M41, Matrix.M42, Matrix.M43);

                RawPermChunkIndex = br.ReadInt16();
            }

            public void PerformMove(int x, int y)
            {
                /*
                if (mDown)
                {
                    x -= old_pos.X;
                    y -= old_pos.Y;

                    if (x != 0)
                    {
                        old_pos.X += x;
                        PositionX += (x < 0 ? -0.5f : 0.5f);
                    }

                    if (y != 0)
                    {
                        old_pos.Y += y;
                        PositionY += (y < 0 ? -0.5f : 0.5f);
                    }
                }
                else
                {
                    old_pos.X = x;
                    old_pos.Y = y;
                    mDown = true;
                }
                */
            }

            public void MovePosition(byte dir)
            {
                /*
                float Speed = 0.05f;

                switch (dir)
                {
                    case 1: //forward
                        {
                            //PositionX += Cam.i * Speed;
                            //PositionY += Cam.j * Speed;
                            //PositionZ += Cam.k * Speed;
                        }
                        break;

                    case 2: //backward
                        {
                            //PositionX -= Cam.i * Speed;
                            //PositionY -= Cam.j * Speed;
                            //PositionZ -= Cam.k * Speed;
                        }
                        break;

                    case 3: //strafe left
                        {
                            //float xLeft = Cam.Radius * Map.MathCos(Cam.lrRotation - Cam.PiOver2) * Map.MathCos(Cam.udRotation) * Speed;
                            //float yLeft = Cam.Radius * Map.MathSin(Cam.lrRotation - Cam.PiOver2) * Map.MathCos(Cam.udRotation) * Speed;

                            //if (Cam.UpVector.Z == 1)
                            //{
                            //    PositionX -= xLeft;
                            //}
                            //else
                            //{
                            //    PositionX += xLeft;
                            //}

                            //PositionY -= yLeft;
                        }
                        break;

                    case 4: //strafe right
                        {
                            
                            float xRight = Cam.Radius * Map.MathCos(Cam.lrRotation - Cam.PiOver2) * Map.MathCos(Cam.udRotation) * Speed;
                            float yRight = Cam.Radius * Map.MathSin(Cam.lrRotation - Cam.PiOver2) * Map.MathCos(Cam.udRotation) * Speed;

                            if (Cam.UpVector.Z == 1)
                            {
                                PositionX += xRight;
                            }
                            else
                            {
                                PositionX -= xRight;
                            }

                            PositionY += yRight;
                            
                        }
                        break;

                    case 5: //up
                        {
                            PositionZ += Speed;
                        }
                        break;

                    case 6: //down
                        {
                            PositionZ -= Speed;
                        }
                        break;
                }
                */
            }
        }

        public struct H2VertexFormat
        {
            #region Instance Data

            #region Static

            public static readonly VertexFormats Format = VertexFormats.PositionNormal |
                VertexFormats.Diffuse | VertexFormats.Texture4;

            #endregion

            /// <summary>
            /// Position
            /// </summary>
            public Vector3 Position;
            /// <summary>
            /// Normal
            /// </summary>
            public Vector3 Normal;
            /// <summary>
            /// Diffuse
            /// </summary>
            public int diffuse;
            /// <summary>
            /// Specular
            /// </summary>
            //public int Specular;
            /// <summary>
            /// Tu0
            /// </summary>
            public float Tu0;
            /// <summary>
            /// Tv0
            /// </summary>
            public float Tv0;
            /// <summary>
            /// Tu1
            /// </summary>
            public float Tu1;
            /// <summary>
            /// Tv1
            /// </summary>
            public float Tv1;
            /// <summary>
            /// Tu2
            /// </summary>
            public float Tu2;
            /// <summary>
            /// Tv2
            /// </summary>
            public float Tv2;
            /// <summary>
            /// Tu3
            /// </summary>
            public float Tu3;
            /// <summary>
            /// Tv3
            /// </summary>
            public float Tv3;

            #endregion

            public H2VertexFormat(Vector3 pos, Vector3 nor, int dif, int spec, Vector2 uv, Vector2 ltmpUV)
            {
                Position = pos;
                Normal = nor;
                diffuse = dif;
                //Specular = spec;
                Tu0 = uv.X;
                Tv0 = uv.Y;
                Tu1 = uv.X;
                Tv1 = uv.Y;
                Tu2 = ltmpUV.X;
                Tv2 = ltmpUV.Y;
                Tu3 = uv.X;
                Tv3 = uv.Y;
            }
        }
    }
}