using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace System.Extended
{
    ///<summary>
    /// 
    ///</summary>
    public enum Endian
    {
        ///<summary>
        /// 
        ///</summary>
        Little,
        ///<summary>
        /// 
        ///</summary>
        Big
    }

    ///<summary>
    /// 
    ///</summary>
    public class BitReader : BinaryReader
    {
        #region Instance Data

        private Endian endian;

        #endregion

        #region Accessors

        ///<summary>
        /// Endian
        ///</summary>
        public Endian Endian { get { return endian; } set { endian = value; } }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="endian"></param>
        public BitReader(string fileName, Endian endian)
            : base(new FileStream(fileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite))
        {
            this.Endian = endian;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="endian"></param>
        public BitReader(Stream stream, Endian endian)
            : base(stream)
        {
            this.Endian = endian;
        }

        ///<summary>
        /// 
        ///</summary>
        public override char ReadChar()
        {
            return Convert.ToChar(this.ReadByte());
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        public override char[] ReadChars(int count)
        {
            return Encoding.ASCII.GetChars(this.ReadBytes(count));
            /*
            char[] chars = new char[count];
            for (int x = 0; x < count; x++)
            {
                chars[x] = Convert.ToChar(this.ReadByte());
            }
            return chars;
            */
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="count"></param>
        /// <returns></returns>
        public char[] ReadBTFChars(int count)
        {
            char[] chars = Encoding.ASCII.GetChars(this.ReadBytes(count));
            Array.Reverse(chars);
            return chars;
            /*
            char[] chars = new char[count];
            for (int x = 0; x < count; x++)
            {
                chars[x] = Convert.ToChar(this.ReadByte());
            }
            Array.Reverse(chars);
            return chars;
            */
        }

        ///<summary>
        /// 
        ///</summary>
        public override string ReadString()
        {
            StringBuilder sb = new StringBuilder();
            char ch;
            long pos = this.BaseStream.Position;
            long len = this.BaseStream.Length;
            while (pos != len && (ch = Convert.ToChar(this.ReadByte())) >= char.MinValue && ch <= char.MaxValue)
            {
                sb.Append(ch);
                pos++;
            }
            return sb.ToString();
        }

        ///<summary>
        /// 
        ///</summary>
        public string ReadBTFString()
        {
            char ch;
            long oldpos = this.BaseStream.Position;
            long pos = oldpos;
            long len = this.BaseStream.Length;
            int readlen = 0;
            while (pos != len && (ch = Convert.ToChar(this.ReadByte())) >= char.MinValue && ch <= char.MaxValue)
            {
                readlen++;
                pos++;
            }
            if (readlen == 0 && pos == len) readlen = (int)(len - pos);
            this.BaseStream.Position = oldpos;
            char[] chars = Encoding.ASCII.GetChars(this.ReadBytes(readlen));
            Array.Reverse(chars);
            return new string(chars);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadString(int length)
        {
            return ReadString(length, Encoding.ASCII);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <param name="encoding"></param>
        /// <returns></returns>
        public string ReadString(int length, Encoding encoding)
        {
            if (length <= 0) return null;
            else return encoding.GetString(this.ReadBytes(length));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadBTFString(int length)
        {
            return ReadBTFString(length, Encoding.ASCII);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <param name="encoding"></param>
        /// <returns></returns>
        public string ReadBTFString(int length, Encoding encoding)
        {
            if (length <= 0) return null;
            else
            {
                char[] chars = encoding.GetChars(this.ReadBytes(length));
                Array.Reverse(chars);
                return new string(chars);
            }
        }

        ///<summary>
        /// 
        ///</summary>
        public string ReadCString()
        {
            return ReadCString('\0');
        }

        ///<summary>
        /// 
        ///</summary>
        public string ReadBTFCString()
        {
            return ReadBTFCString('\0');
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="nullTerminator"></param>
        /// <returns></returns>
        public string ReadCString(char nullTerminator)
        {
            StringBuilder sb = new StringBuilder();
            char ch;
            long pos = this.BaseStream.Position;
            long len = this.BaseStream.Length;
            while (pos != len && (ch = Convert.ToChar(this.ReadByte())) != nullTerminator)
            {
                sb.Append(ch);
            }
            return sb.ToString();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="nullTerminator"></param>
        /// <returns></returns>
        public string ReadBTFCString(char nullTerminator)
        {
            char ch;
            long oldpos = this.BaseStream.Position;
            long pos = oldpos;
            long len = this.BaseStream.Length;
            int readlen = 0;
            while (pos != len && (ch = Convert.ToChar(this.ReadByte())) != nullTerminator)
            {
                readlen++;
                pos++;
            }
            if (readlen == 0 && pos == len) readlen = (int)(len - pos);
            this.BaseStream.Position = oldpos;
            char[] chars = Encoding.ASCII.GetChars(this.ReadBytes(readlen));
            Array.Reverse(chars);
            return new string(chars);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadCString(int length)
        {
            return ReadCString(length, Encoding.ASCII, '\0');
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadBTFCString(int length)
        {
            return ReadBTFCString(length, Encoding.ASCII, '\0');
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadCString(int length, Encoding encoding)
        {
            return ReadCString(length, encoding, '\0');
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <returns></returns>
        public string ReadBTFCString(int length, Encoding encoding)
        {
            return ReadBTFCString(length, encoding, '\0');
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <param name="nullTerminator"></param>
        /// <returns></returns>
        public string ReadCString(int length, Encoding encoding, char nullTerminator)
        {
            if (length <= 0) return null;
            else
            {
                byte[] buffer = this.ReadBytes(length);
                for (int x = 0; x < length; x++)
                {
                    if (buffer[x] == 0)
                    {
                        return encoding.GetString(buffer, 0, x);
                    }
                }
                return encoding.GetString(buffer);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="length"></param>
        /// <param name="nullTerminator"></param>
        /// <returns></returns>
        public string ReadBTFCString(int length, Encoding encoding, char nullTerminator)
        {
            if (length <= 0) return null;
            else
            {
                byte[] buffer = this.ReadBytes(length);
                Array.Reverse(buffer);
                for (int x = length - 1; x >= 0; x--)
                {
                    if (buffer[x] == 0)
                    {
                        return encoding.GetString(buffer, x + 1, length - x - 1);
                    }
                }
                return encoding.GetString(buffer);
            }
        }

        ///<summary>
        /// 
        ///</summary>
        public override short ReadInt16()
        {
            if (Endian == Endian.Little) return base.ReadInt16();
            else
            {
                byte[] buffer = this.ReadBytes(2);
                Array.Reverse(buffer);
                return BitConverter.ToInt16(buffer, 0);
            }
        }

        ///<summary>
        /// 
        ///</summary>
        public override int ReadInt32()
        {
            if (Endian == Endian.Little) return base.ReadInt32();
            else
            {
                byte[] buffer = this.ReadBytes(4);
                Array.Reverse(buffer);
                return BitConverter.ToInt32(buffer, 0);
            }
        }

        ///<summary>
        /// 
        ///</summary>
        public override float ReadSingle()
        {
            if (Endian == Endian.Little) return base.ReadSingle();
            else
            {
                byte[] buffer = this.ReadBytes(4);
                Array.Reverse(buffer);
                return BitConverter.ToSingle(buffer, 0);
            }
        }

        ///<summary>
        /// 
        ///</summary>
        public override double ReadDouble()
        {
            if (Endian == Endian.Little) return base.ReadDouble();
            else
            {
                byte[] buffer = this.ReadBytes(8);
                Array.Reverse(buffer);
                return BitConverter.ToDouble(buffer, 0);
            }
        }
    }
}

