using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Biped
    /// </summary>
    public class Bipd
    {
        #region Constants

        /// <summary>
        /// Size of the bipd meta header.
        /// </summary>
        public const int HeaderSize = 788;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private BipdLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.9 XZodia: added a load of stuff.
        /// v0.82 turk645: added the ragdoll info.
        /// v0.8 XZodia: added unit header.
        /// v0.7 XZodia: added standard header empty biped weapon.
        /// v0.5 turk645: added the color info.
        /// v0.41 shade45: named some unknowns.
        /// v0.3 XD: Added some known values (with some help from Aequitas)...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public BipdLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class BipdLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public Flags1Bitmask32 Flags1 { get { return __flags1__; } set { __flags1__ = value; } }
            private Flags1Bitmask32 __flags1__;
            public class Flags1Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool DoesNotCastShadow { get { return __doesNotCastShadow__; } set { __doesNotCastShadow__ = value; } }
                private bool __doesNotCastShadow__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool SearchCardinalDirectionLightmaps { get { return __searchCardinalDirectionLightmaps__; } set { __searchCardinalDirectionLightmaps__ = value; } }
                private bool __searchCardinalDirectionLightmaps__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Unused { get { return __unused__; } set { __unused__ = value; } }
                private bool __unused__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool NotAPathfindingObstacle { get { return __notAPathfindingObstacle__; } set { __notAPathfindingObstacle__ = value; } }
                private bool __notAPathfindingObstacle__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool ExtensionOfParent { get { return __extensionOfParent__; } set { __extensionOfParent__ = value; } }
                private bool __extensionOfParent__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool DoesNotCauseCollisionDamage { get { return __doesNotCauseCollisionDamage__; } set { __doesNotCauseCollisionDamage__ = value; } }
                private bool __doesNotCauseCollisionDamage__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool EarlyMover { get { return __earlyMover__; } set { __earlyMover__ = value; } }
                private bool __earlyMover__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool EarlyMoverLocalizedPhysics { get { return __earlyMoverLocalizedPhysics__; } set { __earlyMoverLocalizedPhysics__ = value; } }
                private bool __earlyMoverLocalizedPhysics__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool UseStaticMassiveLightmapSample { get { return __useStaticMassiveLightmapSample__; } set { __useStaticMassiveLightmapSample__ = value; } }
                private bool __useStaticMassiveLightmapSample__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool ObjectScalesAttachments { get { return __objectScalesAttachments__; } set { __objectScalesAttachments__ = value; } }
                private bool __objectScalesAttachments__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool InheritsPlayersAppearance { get { return __inheritsPlayersAppearance__; } set { __inheritsPlayersAppearance__ = value; } }
                private bool __inheritsPlayersAppearance__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool DeadBipedsCantLocalize { get { return __deadBipedsCantLocalize__; } set { __deadBipedsCantLocalize__ = value; } }
                private bool __deadBipedsCantLocalize__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool AttachToClustersByDynamicSphere { get { return __attachToClustersByDynamicSphere__; } set { __attachToClustersByDynamicSphere__ = value; } }
                private bool __attachToClustersByDynamicSphere__;
                /// <summary>
                /// 14. Option :: Value: 0xD (13)
                /// </summary>
                public bool EffectsCreatedByThisObjectDoNotBlah { get { return __effectsCreatedByThisObjectDoNotBlah__; } set { __effectsCreatedByThisObjectDoNotBlah__ = value; } }
                private bool __effectsCreatedByThisObjectDoNotBlah__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    DoesNotCastShadow = ((bit & mask) == bit);
                    bit <<= 1;
                    SearchCardinalDirectionLightmaps = ((bit & mask) == bit);
                    bit <<= 1;
                    Unused = ((bit & mask) == bit);
                    bit <<= 1;
                    NotAPathfindingObstacle = ((bit & mask) == bit);
                    bit <<= 1;
                    ExtensionOfParent = ((bit & mask) == bit);
                    bit <<= 1;
                    DoesNotCauseCollisionDamage = ((bit & mask) == bit);
                    bit <<= 1;
                    EarlyMover = ((bit & mask) == bit);
                    bit <<= 1;
                    EarlyMoverLocalizedPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    UseStaticMassiveLightmapSample = ((bit & mask) == bit);
                    bit <<= 1;
                    ObjectScalesAttachments = ((bit & mask) == bit);
                    bit <<= 1;
                    InheritsPlayersAppearance = ((bit & mask) == bit);
                    bit <<= 1;
                    DeadBipedsCantLocalize = ((bit & mask) == bit);
                    bit <<= 1;
                    AttachToClustersByDynamicSphere = ((bit & mask) == bit);
                    bit <<= 1;
                    EffectsCreatedByThisObjectDoNotBlah = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float BoundingRadius { get { return __boundingRadius__; } set { __boundingRadius__ = value; } }
            private float __boundingRadius__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float BoundingOffsetX { get { return __boundingOffsetX__; } set { __boundingOffsetX__ = value; } }
            private float __boundingOffsetX__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float BoundingOffsetY { get { return __boundingOffsetY__; } set { __boundingOffsetY__ = value; } }
            private float __boundingOffsetY__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float BoundingOffsetZ { get { return __boundingOffsetZ__; } set { __boundingOffsetZ__ = value; } }
            private float __boundingOffsetZ__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float AccelerationScale { get { return __accelerationScale__; } set { __accelerationScale__ = value; } }
            private float __accelerationScale__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float LightmapShadowMode { get { return __lightmapShadowMode__; } set { __lightmapShadowMode__ = value; } }
            private float __lightmapShadowMode__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float SweetenerSize { get { return __sweetenerSize__; } set { __sweetenerSize__ = value; } }
            private float __sweetenerSize__;
            /// <summary>
            /// 9. Float :: Offset: 0x20 (32)
            /// </summary>
            public float DynamicLightSphereRadius { get { return __dynamicLightSphereRadius__; } set { __dynamicLightSphereRadius__ = value; } }
            private float __dynamicLightSphereRadius__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float DynamicLightSphereOffsetX { get { return __dynamicLightSphereOffsetX__; } set { __dynamicLightSphereOffsetX__ = value; } }
            private float __dynamicLightSphereOffsetX__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float DynamicLightSphereOffsetY { get { return __dynamicLightSphereOffsetY__; } set { __dynamicLightSphereOffsetY__ = value; } }
            private float __dynamicLightSphereOffsetY__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float DynamicLightSphereOffsetZ { get { return __dynamicLightSphereOffsetZ__; } set { __dynamicLightSphereOffsetZ__ = value; } }
            private float __dynamicLightSphereOffsetZ__;
            /// <summary>
            /// 13. StringID :: Offset: 0x30 (48)
            /// </summary>
            public StringID DefaultModelVariant { get { return __defaultModelVariant__; } set { __defaultModelVariant__ = value; } }
            private StringID __defaultModelVariant__;
            /// <summary>
            /// 14. Tag :: Offset: 0x34 (52)
            /// </summary>
            public TagRef Model { get { return __model__; } set { __model__ = value; } }
            private TagRef __model__;
            /// <summary>
            /// 15. Tag :: Offset: 0x3C (60)
            /// </summary>
            public TagRef CrateObject { get { return __crateObject__; } set { __crateObject__ = value; } }
            private TagRef __crateObject__;
            /// <summary>
            /// 16. Tag :: Offset: 0x44 (68)
            /// </summary>
            public TagRef ModifierShader { get { return __modifierShader__; } set { __modifierShader__ = value; } }
            private TagRef __modifierShader__;
            /// <summary>
            /// 17. Tag :: Offset: 0x4C (76)
            /// </summary>
            public TagRef CreationEffect { get { return __creationEffect__; } set { __creationEffect__ = value; } }
            private TagRef __creationEffect__;
            /// <summary>
            /// 18. Tag :: Offset: 0x54 (84)
            /// </summary>
            public TagRef MaterialEffects { get { return __materialEffects__; } set { __materialEffects__ = value; } }
            private TagRef __materialEffects__;
            /// <summary>
            /// 19. TagBlock :: Offset: 0x5C (92)
            /// </summary>
            public _AiPropertiesTagBlock AiProperties { get { return __aiProperties__; } set { __aiProperties__ = value; } }
            private _AiPropertiesTagBlock __aiProperties__;
            public class _AiPropertiesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Flags { get { return __flags__; } set { __flags__ = value; } }
                private float __flags__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID TypeName { get { return __typeName__; } set { __typeName__ = value; } }
                private StringID __typeName__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Size { get { return __size__; } set { __size__ = value; } }
                private float __size__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float LeapJumpSpeed { get { return __leapJumpSpeed__; } set { __leapJumpSpeed__ = value; } }
                private float __leapJumpSpeed__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AiPropertiesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AiPropertiesTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AiPropertiesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AiPropertiesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 20. TagBlock :: Offset: 0x64 (100)
            /// </summary>
            public _FunctionsTagBlock Functions { get { return __functions__; } set { __functions__ = value; } }
            private _FunctionsTagBlock __functions__;
            public class _FunctionsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Flags { get { return __flags__; } set { __flags__ = value; } }
                private float __flags__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Function { get { return __function__; } set { __function__ = value; } }
                private StringID __function__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID ScaleFunctionBy { get { return __scaleFunctionBy__; } set { __scaleFunctionBy__ = value; } }
                private StringID __scaleFunctionBy__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID WobbleFunction { get { return __wobbleFunction__; } set { __wobbleFunction__ = value; } }
                private StringID __wobbleFunction__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float WobblePeriod { get { return __wobblePeriod__; } set { __wobblePeriod__ = value; } }
                private float __wobblePeriod__;
                /// <summary>
                /// 6. TagBlock :: Offset: 0x14 (20)
                /// </summary>
                public _Inputs_realNumbersTagBlock Inputs_realNumbers { get { return __inputs_realNumbers__; } set { __inputs_realNumbers__ = value; } }
                private _Inputs_realNumbersTagBlock __inputs_realNumbers__;
                public class _Inputs_realNumbersTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private BipdLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Byte :: Offset: 0x0 (0)
                    /// </summary>
                    public byte Data { get { return __data__; } set { __data__ = value; } }
                    private byte __data__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Inputs_realNumbersTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Inputs_realNumbersTagBlock(BipdLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Inputs_realNumbersTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Inputs_realNumbersTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 7. StringID :: Offset: 0x1C (28)
                /// </summary>
                public StringID TurnOffWith { get { return __turnOffWith__; } set { __turnOffWith__ = value; } }
                private StringID __turnOffWith__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FunctionsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FunctionsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Inputs_realNumbers = new _Inputs_realNumbersTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FunctionsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FunctionsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 21. Float :: Offset: 0x6C (108)
            /// </summary>
            public float RenderBoundingRadius { get { return __renderBoundingRadius__; } set { __renderBoundingRadius__ = value; } }
            private float __renderBoundingRadius__;
            /// <summary>
            /// 22. Float :: Offset: 0x70 (112)
            /// </summary>
            public float Lower1 { get { return __lower1__; } set { __lower1__ = value; } }
            private float __lower1__;
            /// <summary>
            /// 23. Float :: Offset: 0x74 (116)
            /// </summary>
            public float Upper1 { get { return __upper1__; } set { __upper1__ = value; } }
            private float __upper1__;
            /// <summary>
            /// 24. Float :: Offset: 0x78 (120)
            /// </summary>
            public float Lower2 { get { return __lower2__; } set { __lower2__ = value; } }
            private float __lower2__;
            /// <summary>
            /// 25. Float :: Offset: 0x7C (124)
            /// </summary>
            public float Upper2 { get { return __upper2__; } set { __upper2__ = value; } }
            private float __upper2__;
            /// <summary>
            /// 26. Float :: Offset: 0x80 (128)
            /// </summary>
            public float Lower3 { get { return __lower3__; } set { __lower3__ = value; } }
            private float __lower3__;
            /// <summary>
            /// 27. Float :: Offset: 0x84 (132)
            /// </summary>
            public float Upper3 { get { return __upper3__; } set { __upper3__ = value; } }
            private float __upper3__;
            /// <summary>
            /// 28. Float :: Offset: 0x88 (136)
            /// </summary>
            public float Lower4 { get { return __lower4__; } set { __lower4__ = value; } }
            private float __lower4__;
            /// <summary>
            /// 29. Float :: Offset: 0x8C (140)
            /// </summary>
            public float Upper4 { get { return __upper4__; } set { __upper4__ = value; } }
            private float __upper4__;
            /// <summary>
            /// 30. Int16 :: Offset: 0x90 (144)
            /// </summary>
            public short HudTextMessageIndex { get { return __hudTextMessageIndex__; } set { __hudTextMessageIndex__ = value; } }
            private short __hudTextMessageIndex__;
            /// <summary>
            /// 31. Int16 :: Offset: 0x92 (146)
            /// </summary>
            public short ForcedShaderPermutationIndex { get { return __forcedShaderPermutationIndex__; } set { __forcedShaderPermutationIndex__ = value; } }
            private short __forcedShaderPermutationIndex__;
            /// <summary>
            /// 32. TagBlock :: Offset: 0x94 (148)
            /// </summary>
            public _AttachmentsTagBlock Attachments { get { return __attachments__; } set { __attachments__ = value; } }
            private _AttachmentsTagBlock __attachments__;
            public class _AttachmentsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Type { get { return __type__; } set { __type__ = value; } }
                private TagRef __type__;
                /// <summary>
                /// 2. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 3. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID ChangeColor { get { return __changeColor__; } set { __changeColor__ = value; } }
                private StringID __changeColor__;
                /// <summary>
                /// 4. StringID :: Offset: 0x10 (16)
                /// </summary>
                public StringID PrimaryScale { get { return __primaryScale__; } set { __primaryScale__ = value; } }
                private StringID __primaryScale__;
                /// <summary>
                /// 5. StringID :: Offset: 0x14 (20)
                /// </summary>
                public StringID SecondaryScale { get { return __secondaryScale__; } set { __secondaryScale__ = value; } }
                private StringID __secondaryScale__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AttachmentsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AttachmentsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AttachmentsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AttachmentsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 33. TagBlock :: Offset: 0x9C (156)
            /// </summary>
            public _WidgetsTagBlock Widgets { get { return __widgets__; } set { __widgets__ = value; } }
            private _WidgetsTagBlock __widgets__;
            public class _WidgetsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Reference { get { return __reference__; } set { __reference__ = value; } }
                private TagRef __reference__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WidgetsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WidgetsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WidgetsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WidgetsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 34. ByteArray :: Offset: 0xA4 (164)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 35. TagBlock :: Offset: 0xAC (172)
            /// </summary>
            public _ChangeColorsTagBlock ChangeColors { get { return __changeColors__; } set { __changeColors__ = value; } }
            private _ChangeColorsTagBlock __changeColors__;
            public class _ChangeColorsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. TagBlock :: Offset: 0x0 (0)
                /// </summary>
                public _PermutationsTagBlock Permutations { get { return __permutations__; } set { __permutations__ = value; } }
                private _PermutationsTagBlock __permutations__;
                public class _PermutationsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private BipdLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Weight { get { return __weight__; } set { __weight__ = value; } }
                    private float __weight__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float ColorLowerBoundR { get { return __colorLowerBoundR__; } set { __colorLowerBoundR__ = value; } }
                    private float __colorLowerBoundR__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float ColorLowerBoundG { get { return __colorLowerBoundG__; } set { __colorLowerBoundG__ = value; } }
                    private float __colorLowerBoundG__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float ColorLowerBoundB { get { return __colorLowerBoundB__; } set { __colorLowerBoundB__ = value; } }
                    private float __colorLowerBoundB__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float ColorUpperBoundR { get { return __colorUpperBoundR__; } set { __colorUpperBoundR__ = value; } }
                    private float __colorUpperBoundR__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float ColorUpperBoundG { get { return __colorUpperBoundG__; } set { __colorUpperBoundG__ = value; } }
                    private float __colorUpperBoundG__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float ColorUpperBoundB { get { return __colorUpperBoundB__; } set { __colorUpperBoundB__ = value; } }
                    private float __colorUpperBoundB__;
                    /// <summary>
                    /// 8. StringID :: Offset: 0x1C (28)
                    /// </summary>
                    public StringID Name { get { return __name__; } set { __name__ = value; } }
                    private StringID __name__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PermutationsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PermutationsTagBlock(BipdLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PermutationsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PermutationsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 2. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ChangeColorsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ChangeColorsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Permutations = new _PermutationsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ChangeColorsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ChangeColorsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 36. TagBlock :: Offset: 0xB4 (180)
            /// </summary>
            public _PredictedResourceTagBlock PredictedResource { get { return __predictedResource__; } set { __predictedResource__ = value; } }
            private _PredictedResourceTagBlock __predictedResource__;
            public class _PredictedResourceTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Int16 :: Offset: 0x0 (0)
                /// </summary>
                public short Type { get { return __type__; } set { __type__ = value; } }
                private short __type__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x2 (2)
                /// </summary>
                public short ResourceIndex { get { return __resourceIndex__; } set { __resourceIndex__ = value; } }
                private short __resourceIndex__;
                /// <summary>
                /// 3. ID :: Offset: 0x4 (4)
                /// </summary>
                public LoneID TagIndex { get { return __tagIndex__; } set { __tagIndex__ = value; } }
                private LoneID __tagIndex__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PredictedResourceTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PredictedResourceTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PredictedResourceTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PredictedResourceTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 37. Bitmask32 :: Offset: 0xBC (188)
            /// </summary>
            public Flags2Bitmask32 Flags2 { get { return __flags2__; } set { __flags2__ = value; } }
            private Flags2Bitmask32 __flags2__;
            public class Flags2Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool CircularAiming { get { return __circularAiming__; } set { __circularAiming__ = value; } }
                private bool __circularAiming__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool DestroyedAfterDying { get { return __destroyedAfterDying__; } set { __destroyedAfterDying__ = value; } }
                private bool __destroyedAfterDying__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool HalfSpeedInterpolation { get { return __halfSpeedInterpolation__; } set { __halfSpeedInterpolation__ = value; } }
                private bool __halfSpeedInterpolation__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool FiresFromCamera { get { return __firesFromCamera__; } set { __firesFromCamera__ = value; } }
                private bool __firesFromCamera__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool EntranceInsideBoundingSphere { get { return __entranceInsideBoundingSphere__; } set { __entranceInsideBoundingSphere__ = value; } }
                private bool __entranceInsideBoundingSphere__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool WeaponNotVisibleIn { get { return __weaponNotVisibleIn__; } set { __weaponNotVisibleIn__ = value; } }
                private bool __weaponNotVisibleIn__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool CausesPassengerDialogue { get { return __causesPassengerDialogue__; } set { __causesPassengerDialogue__ = value; } }
                private bool __causesPassengerDialogue__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool ResistsPings { get { return __resistsPings__; } set { __resistsPings__ = value; } }
                private bool __resistsPings__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool MeleeAttackIsFatal { get { return __meleeAttackIsFatal__; } set { __meleeAttackIsFatal__ = value; } }
                private bool __meleeAttackIsFatal__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool DontRefaceDuringPings { get { return __dontRefaceDuringPings__; } set { __dontRefaceDuringPings__ = value; } }
                private bool __dontRefaceDuringPings__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool HasNoAiming { get { return __hasNoAiming__; } set { __hasNoAiming__ = value; } }
                private bool __hasNoAiming__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool SimpleCreature { get { return __simpleCreature__; } set { __simpleCreature__ = value; } }
                private bool __simpleCreature__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool ImpactMeleeAttachesToUnit { get { return __impactMeleeAttachesToUnit__; } set { __impactMeleeAttachesToUnit__ = value; } }
                private bool __impactMeleeAttachesToUnit__;
                /// <summary>
                /// 14. Option :: Value: 0xD (13)
                /// </summary>
                public bool ImpactMeleeDiesOnShield { get { return __impactMeleeDiesOnShield__; } set { __impactMeleeDiesOnShield__ = value; } }
                private bool __impactMeleeDiesOnShield__;
                /// <summary>
                /// 15. Option :: Value: 0xE (14)
                /// </summary>
                public bool CannotOpenDoorsAutomatically { get { return __cannotOpenDoorsAutomatically__; } set { __cannotOpenDoorsAutomatically__ = value; } }
                private bool __cannotOpenDoorsAutomatically__;
                /// <summary>
                /// 16. Option :: Value: 0xF (15)
                /// </summary>
                public bool MeleeAttackersCannotAttack { get { return __meleeAttackersCannotAttack__; } set { __meleeAttackersCannotAttack__ = value; } }
                private bool __meleeAttackersCannotAttack__;
                /// <summary>
                /// 17. Option :: Value: 0x10 (16)
                /// </summary>
                public bool NotInstantlyKilledByMelee { get { return __notInstantlyKilledByMelee__; } set { __notInstantlyKilledByMelee__ = value; } }
                private bool __notInstantlyKilledByMelee__;
                /// <summary>
                /// 18. Option :: Value: 0x11 (17)
                /// </summary>
                public bool ShieldSapping { get { return __shieldSapping__; } set { __shieldSapping__ = value; } }
                private bool __shieldSapping__;
                /// <summary>
                /// 19. Option :: Value: 0x12 (18)
                /// </summary>
                public bool RunsAroundFlaming { get { return __runsAroundFlaming__; } set { __runsAroundFlaming__ = value; } }
                private bool __runsAroundFlaming__;
                /// <summary>
                /// 20. Option :: Value: 0x13 (19)
                /// </summary>
                public bool Inconsequential { get { return __inconsequential__; } set { __inconsequential__ = value; } }
                private bool __inconsequential__;
                /// <summary>
                /// 21. Option :: Value: 0x14 (20)
                /// </summary>
                public bool SpecialCinematicUnit { get { return __specialCinematicUnit__; } set { __specialCinematicUnit__ = value; } }
                private bool __specialCinematicUnit__;
                /// <summary>
                /// 22. Option :: Value: 0x15 (21)
                /// </summary>
                public bool IgnoredByAutoAiming { get { return __ignoredByAutoAiming__; } set { __ignoredByAutoAiming__ = value; } }
                private bool __ignoredByAutoAiming__;
                /// <summary>
                /// 23. Option :: Value: 0x16 (22)
                /// </summary>
                public bool ShieldsFryInfectionForms { get { return __shieldsFryInfectionForms__; } set { __shieldsFryInfectionForms__ = value; } }
                private bool __shieldsFryInfectionForms__;
                /// <summary>
                /// 24. Option :: Value: 0x17 (23)
                /// </summary>
                public bool IntegratedLightControlsWeapon { get { return __integratedLightControlsWeapon__; } set { __integratedLightControlsWeapon__ = value; } }
                private bool __integratedLightControlsWeapon__;
                /// <summary>
                /// 25. Option :: Value: 0x18 (24)
                /// </summary>
                public bool IntegratedLightLastsForever { get { return __integratedLightLastsForever__; } set { __integratedLightLastsForever__ = value; } }
                private bool __integratedLightLastsForever__;
                /// <summary>
                /// 26. Option :: Value: 0x1C (28)
                /// </summary>
                public bool HasBoost { get { return __hasBoost__; } set { __hasBoost__ = value; } }
                private bool __hasBoost__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    CircularAiming = ((bit & mask) == bit);
                    bit <<= 1;
                    DestroyedAfterDying = ((bit & mask) == bit);
                    bit <<= 1;
                    HalfSpeedInterpolation = ((bit & mask) == bit);
                    bit <<= 1;
                    FiresFromCamera = ((bit & mask) == bit);
                    bit <<= 1;
                    EntranceInsideBoundingSphere = ((bit & mask) == bit);
                    bit <<= 1;
                    WeaponNotVisibleIn = ((bit & mask) == bit);
                    bit <<= 1;
                    CausesPassengerDialogue = ((bit & mask) == bit);
                    bit <<= 1;
                    ResistsPings = ((bit & mask) == bit);
                    bit <<= 1;
                    MeleeAttackIsFatal = ((bit & mask) == bit);
                    bit <<= 1;
                    DontRefaceDuringPings = ((bit & mask) == bit);
                    bit <<= 1;
                    HasNoAiming = ((bit & mask) == bit);
                    bit <<= 1;
                    SimpleCreature = ((bit & mask) == bit);
                    bit <<= 1;
                    ImpactMeleeAttachesToUnit = ((bit & mask) == bit);
                    bit <<= 1;
                    ImpactMeleeDiesOnShield = ((bit & mask) == bit);
                    bit <<= 1;
                    CannotOpenDoorsAutomatically = ((bit & mask) == bit);
                    bit <<= 1;
                    MeleeAttackersCannotAttack = ((bit & mask) == bit);
                    bit <<= 1;
                    NotInstantlyKilledByMelee = ((bit & mask) == bit);
                    bit <<= 1;
                    ShieldSapping = ((bit & mask) == bit);
                    bit <<= 1;
                    RunsAroundFlaming = ((bit & mask) == bit);
                    bit <<= 1;
                    Inconsequential = ((bit & mask) == bit);
                    bit <<= 1;
                    SpecialCinematicUnit = ((bit & mask) == bit);
                    bit <<= 1;
                    IgnoredByAutoAiming = ((bit & mask) == bit);
                    bit <<= 1;
                    ShieldsFryInfectionForms = ((bit & mask) == bit);
                    bit <<= 1;
                    IntegratedLightControlsWeapon = ((bit & mask) == bit);
                    bit <<= 1;
                    IntegratedLightLastsForever = ((bit & mask) == bit);
                    bit <<= 1;
                    HasBoost = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 38. Enum16 :: Offset: 0xC0 (192)
            /// </summary>
            public DefaultTeamEnum16 DefaultTeam { get { return __defaultTeam__; } set { __defaultTeam__ = value; } }
            private DefaultTeamEnum16 __defaultTeam__;
            public class DefaultTeamEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool None { get { return __none__; } set { __none__ = value; } }
                private bool __none__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Player { get { return __player__; } set { __player__ = value; } }
                private bool __player__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Human { get { return __human__; } set { __human__ = value; } }
                private bool __human__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Covenant { get { return __covenant__; } set { __covenant__ = value; } }
                private bool __covenant__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Flood { get { return __flood__; } set { __flood__ = value; } }
                private bool __flood__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool Sentinel { get { return __sentinel__; } set { __sentinel__ = value; } }
                private bool __sentinel__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool Heretic { get { return __heretic__; } set { __heretic__ = value; } }
                private bool __heretic__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool NewCovenant { get { return __newCovenant__; } set { __newCovenant__ = value; } }
                private bool __newCovenant__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    None = (cur == value);
                    cur++;
                    Player = (cur == value);
                    cur++;
                    Human = (cur == value);
                    cur++;
                    Covenant = (cur == value);
                    cur++;
                    Flood = (cur == value);
                    cur++;
                    Sentinel = (cur == value);
                    cur++;
                    Heretic = (cur == value);
                    cur++;
                    NewCovenant = (cur == value);
                }
            }
            /// <summary>
            /// 39. Enum16 :: Offset: 0xC2 (194)
            /// </summary>
            public ConstantSoundVolumeEnum16 ConstantSoundVolume { get { return __constantSoundVolume__; } set { __constantSoundVolume__ = value; } }
            private ConstantSoundVolumeEnum16 __constantSoundVolume__;
            public class ConstantSoundVolumeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Silent { get { return __silent__; } set { __silent__ = value; } }
                private bool __silent__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Medium { get { return __medium__; } set { __medium__ = value; } }
                private bool __medium__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Loud { get { return __loud__; } set { __loud__ = value; } }
                private bool __loud__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Shout { get { return __shout__; } set { __shout__ = value; } }
                private bool __shout__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Quiet { get { return __quiet__; } set { __quiet__ = value; } }
                private bool __quiet__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    Silent = (cur == value);
                    cur++;
                    Medium = (cur == value);
                    cur++;
                    Loud = (cur == value);
                    cur++;
                    Shout = (cur == value);
                    cur++;
                    Quiet = (cur == value);
                }
            }
            /// <summary>
            /// 40. Tag :: Offset: 0xC4 (196)
            /// </summary>
            public TagRef IntegratedLightToggle { get { return __integratedLightToggle__; } set { __integratedLightToggle__ = value; } }
            private TagRef __integratedLightToggle__;
            /// <summary>
            /// 41. Float :: Offset: 0xCC (204)
            /// </summary>
            public float CameraFieldOfView { get { return __cameraFieldOfView__; } set { __cameraFieldOfView__ = value; } }
            private float __cameraFieldOfView__;
            /// <summary>
            /// 42. Float :: Offset: 0xD0 (208)
            /// </summary>
            public float CameraStiffness { get { return __cameraStiffness__; } set { __cameraStiffness__ = value; } }
            private float __cameraStiffness__;
            /// <summary>
            /// 43. StringID :: Offset: 0xD4 (212)
            /// </summary>
            public StringID CameraMarkerName { get { return __cameraMarkerName__; } set { __cameraMarkerName__ = value; } }
            private StringID __cameraMarkerName__;
            /// <summary>
            /// 44. StringID :: Offset: 0xD8 (216)
            /// </summary>
            public StringID CameraSubmergedMarkerName { get { return __cameraSubmergedMarkerName__; } set { __cameraSubmergedMarkerName__ = value; } }
            private StringID __cameraSubmergedMarkerName__;
            /// <summary>
            /// 45. Float :: Offset: 0xDC (220)
            /// </summary>
            public float PitchAutoLevel { get { return __pitchAutoLevel__; } set { __pitchAutoLevel__ = value; } }
            private float __pitchAutoLevel__;
            /// <summary>
            /// 46. Float :: Offset: 0xE0 (224)
            /// </summary>
            public float PitchRange1 { get { return __pitchRange1__; } set { __pitchRange1__ = value; } }
            private float __pitchRange1__;
            /// <summary>
            /// 47. Float :: Offset: 0xE4 (228)
            /// </summary>
            public float PitchRange2 { get { return __pitchRange2__; } set { __pitchRange2__ = value; } }
            private float __pitchRange2__;
            /// <summary>
            /// 48. TagBlock :: Offset: 0xE8 (232)
            /// </summary>
            public _CameraTracksTagBlock CameraTracks { get { return __cameraTracks__; } set { __cameraTracks__ = value; } }
            private _CameraTracksTagBlock __cameraTracks__;
            public class _CameraTracksTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Track { get { return __track__; } set { __track__ = value; } }
                private TagRef __track__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _CameraTracksTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _CameraTracksTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_CameraTracksTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _CameraTracksTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 49. Float :: Offset: 0xF0 (240)
            /// </summary>
            public float SeatAccelerationScaleI { get { return __seatAccelerationScaleI__; } set { __seatAccelerationScaleI__ = value; } }
            private float __seatAccelerationScaleI__;
            /// <summary>
            /// 50. Float :: Offset: 0xF4 (244)
            /// </summary>
            public float SeatAccelerationScaleJ { get { return __seatAccelerationScaleJ__; } set { __seatAccelerationScaleJ__ = value; } }
            private float __seatAccelerationScaleJ__;
            /// <summary>
            /// 51. Float :: Offset: 0xF8 (248)
            /// </summary>
            public float SeatAccelerationScaleK { get { return __seatAccelerationScaleK__; } set { __seatAccelerationScaleK__ = value; } }
            private float __seatAccelerationScaleK__;
            /// <summary>
            /// 52. ByteArray :: Offset: 0xFC (252)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 53. Float :: Offset: 0x104 (260)
            /// </summary>
            public float SoftPingThreshold { get { return __softPingThreshold__; } set { __softPingThreshold__ = value; } }
            private float __softPingThreshold__;
            /// <summary>
            /// 54. Float :: Offset: 0x108 (264)
            /// </summary>
            public float SoftPingInterruptTime { get { return __softPingInterruptTime__; } set { __softPingInterruptTime__ = value; } }
            private float __softPingInterruptTime__;
            /// <summary>
            /// 55. Float :: Offset: 0x10C (268)
            /// </summary>
            public float HardPingThreshold { get { return __hardPingThreshold__; } set { __hardPingThreshold__ = value; } }
            private float __hardPingThreshold__;
            /// <summary>
            /// 56. Float :: Offset: 0x110 (272)
            /// </summary>
            public float HardPingInterruptTime { get { return __hardPingInterruptTime__; } set { __hardPingInterruptTime__ = value; } }
            private float __hardPingInterruptTime__;
            /// <summary>
            /// 57. Float :: Offset: 0x114 (276)
            /// </summary>
            public float HardPingDeathThreshold { get { return __hardPingDeathThreshold__; } set { __hardPingDeathThreshold__ = value; } }
            private float __hardPingDeathThreshold__;
            /// <summary>
            /// 58. Float :: Offset: 0x118 (280)
            /// </summary>
            public float FeignDeathThreshold { get { return __feignDeathThreshold__; } set { __feignDeathThreshold__ = value; } }
            private float __feignDeathThreshold__;
            /// <summary>
            /// 59. Float :: Offset: 0x11C (284)
            /// </summary>
            public float FeignDeathTime { get { return __feignDeathTime__; } set { __feignDeathTime__ = value; } }
            private float __feignDeathTime__;
            /// <summary>
            /// 60. Float :: Offset: 0x120 (288)
            /// </summary>
            public float DistOfEvadeAnim { get { return __distOfEvadeAnim__; } set { __distOfEvadeAnim__ = value; } }
            private float __distOfEvadeAnim__;
            /// <summary>
            /// 61. Float :: Offset: 0x124 (292)
            /// </summary>
            public float DistOfDiveAnim { get { return __distOfDiveAnim__; } set { __distOfDiveAnim__ = value; } }
            private float __distOfDiveAnim__;
            /// <summary>
            /// 62. Float :: Offset: 0x128 (296)
            /// </summary>
            public float StunnedMovementThreshold { get { return __stunnedMovementThreshold__; } set { __stunnedMovementThreshold__ = value; } }
            private float __stunnedMovementThreshold__;
            /// <summary>
            /// 63. Float :: Offset: 0x12C (300)
            /// </summary>
            public float FeignDeathChance { get { return __feignDeathChance__; } set { __feignDeathChance__ = value; } }
            private float __feignDeathChance__;
            /// <summary>
            /// 64. Float :: Offset: 0x130 (304)
            /// </summary>
            public float FeignRepeatChance { get { return __feignRepeatChance__; } set { __feignRepeatChance__ = value; } }
            private float __feignRepeatChance__;
            /// <summary>
            /// 65. Tag :: Offset: 0x134 (308)
            /// </summary>
            public TagRef SpawnedActor { get { return __spawnedActor__; } set { __spawnedActor__ = value; } }
            private TagRef __spawnedActor__;
            /// <summary>
            /// 66. Int16 :: Offset: 0x13C (316)
            /// </summary>
            public short SpawnedActorCountLower { get { return __spawnedActorCountLower__; } set { __spawnedActorCountLower__ = value; } }
            private short __spawnedActorCountLower__;
            /// <summary>
            /// 67. Int16 :: Offset: 0x13E (318)
            /// </summary>
            public short SpawnedActorCountUpper { get { return __spawnedActorCountUpper__; } set { __spawnedActorCountUpper__ = value; } }
            private short __spawnedActorCountUpper__;
            /// <summary>
            /// 68. Float :: Offset: 0x140 (320)
            /// </summary>
            public float SpawnedVelocity { get { return __spawnedVelocity__; } set { __spawnedVelocity__ = value; } }
            private float __spawnedVelocity__;
            /// <summary>
            /// 69. Float :: Offset: 0x144 (324)
            /// </summary>
            public float AimingVelocityMax { get { return __aimingVelocityMax__; } set { __aimingVelocityMax__ = value; } }
            private float __aimingVelocityMax__;
            /// <summary>
            /// 70. Float :: Offset: 0x148 (328)
            /// </summary>
            public float AimingAccelMax { get { return __aimingAccelMax__; } set { __aimingAccelMax__ = value; } }
            private float __aimingAccelMax__;
            /// <summary>
            /// 71. Float :: Offset: 0x14C (332)
            /// </summary>
            public float CasualAimingModifier { get { return __casualAimingModifier__; } set { __casualAimingModifier__ = value; } }
            private float __casualAimingModifier__;
            /// <summary>
            /// 72. Float :: Offset: 0x150 (336)
            /// </summary>
            public float LookingVelocityMax { get { return __lookingVelocityMax__; } set { __lookingVelocityMax__ = value; } }
            private float __lookingVelocityMax__;
            /// <summary>
            /// 73. Float :: Offset: 0x154 (340)
            /// </summary>
            public float LookingAccelMax { get { return __lookingAccelMax__; } set { __lookingAccelMax__ = value; } }
            private float __lookingAccelMax__;
            /// <summary>
            /// 74. StringID :: Offset: 0x158 (344)
            /// </summary>
            public StringID RightHand1 { get { return __rightHand1__; } set { __rightHand1__ = value; } }
            private StringID __rightHand1__;
            /// <summary>
            /// 75. StringID :: Offset: 0x15C (348)
            /// </summary>
            public StringID LeftHand { get { return __leftHand__; } set { __leftHand__ = value; } }
            private StringID __leftHand__;
            /// <summary>
            /// 76. StringID :: Offset: 0x160 (352)
            /// </summary>
            public StringID RightHand2 { get { return __rightHand2__; } set { __rightHand2__ = value; } }
            private StringID __rightHand2__;
            /// <summary>
            /// 77. Tag :: Offset: 0x164 (356)
            /// </summary>
            public TagRef MeleeDamage { get { return __meleeDamage__; } set { __meleeDamage__ = value; } }
            private TagRef __meleeDamage__;
            /// <summary>
            /// 78. Tag :: Offset: 0x16C (364)
            /// </summary>
            public TagRef BoardingMeleeDamage { get { return __boardingMeleeDamage__; } set { __boardingMeleeDamage__ = value; } }
            private TagRef __boardingMeleeDamage__;
            /// <summary>
            /// 79. Tag :: Offset: 0x174 (372)
            /// </summary>
            public TagRef BoardingMeleeResponseDamage { get { return __boardingMeleeResponseDamage__; } set { __boardingMeleeResponseDamage__ = value; } }
            private TagRef __boardingMeleeResponseDamage__;
            /// <summary>
            /// 80. Tag :: Offset: 0x17C (380)
            /// </summary>
            public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private TagRef __unknown1__;
            /// <summary>
            /// 81. Tag :: Offset: 0x184 (388)
            /// </summary>
            public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private TagRef __unknown2__;
            /// <summary>
            /// 82. Tag :: Offset: 0x18C (396)
            /// </summary>
            public TagRef Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private TagRef __unknown3__;
            /// <summary>
            /// 83. Enum32 :: Offset: 0x194 (404)
            /// </summary>
            public MotionSensorBlipSizeEnum32 MotionSensorBlipSize { get { return __motionSensorBlipSize__; } set { __motionSensorBlipSize__ = value; } }
            private MotionSensorBlipSizeEnum32 __motionSensorBlipSize__;
            public class MotionSensorBlipSizeEnum32
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Medium { get { return __medium__; } set { __medium__ = value; } }
                private bool __medium__;
                /// <summary>
                /// 2. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Small { get { return __small__; } set { __small__ = value; } }
                private bool __small__;
                /// <summary>
                /// 3. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Large { get { return __large__; } set { __large__ = value; } }
                private bool __large__;

                public void _Initialize(BitReader br)
                {
                    uint value = br.ReadUInt32();
                    uint cur = 0;

                    Medium = (cur == value);
                    cur++;
                    Small = (cur == value);
                    cur++;
                    Large = (cur == value);
                }
            }
            /// <summary>
            /// 84. TagBlock :: Offset: 0x198 (408)
            /// </summary>
            public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private _Unknown4TagBlock __unknown4__;
            public class _Unknown4TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private StringID __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown4TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown4TagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown4TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown4TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 85. TagBlock :: Offset: 0x1A0 (416)
            /// </summary>
            public _HudInterfacesTagBlock HudInterfaces { get { return __hudInterfaces__; } set { __hudInterfaces__ = value; } }
            private _HudInterfacesTagBlock __hudInterfaces__;
            public class _HudInterfacesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef UnitHudInterface { get { return __unitHudInterface__; } set { __unitHudInterface__ = value; } }
                private TagRef __unitHudInterface__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _HudInterfacesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _HudInterfacesTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_HudInterfacesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _HudInterfacesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 86. TagBlock :: Offset: 0x1A8 (424)
            /// </summary>
            public _Unknown5TagBlock Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private _Unknown5TagBlock __unknown5__;
            public class _Unknown5TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. ByteArray :: Offset: 0x4 (4)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 3. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 4. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 5. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown5TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown5TagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown5TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown5TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 87. Float :: Offset: 0x1B0 (432)
            /// </summary>
            public float GrenadeVelocity { get { return __grenadeVelocity__; } set { __grenadeVelocity__ = value; } }
            private float __grenadeVelocity__;
            /// <summary>
            /// 88. Enum16 :: Offset: 0x1B4 (436)
            /// </summary>
            public GrenadeTypeEnum16 GrenadeType { get { return __grenadeType__; } set { __grenadeType__ = value; } }
            private GrenadeTypeEnum16 __grenadeType__;
            public class GrenadeTypeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool HumanFragmentation { get { return __humanFragmentation__; } set { __humanFragmentation__ = value; } }
                private bool __humanFragmentation__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool CovenantPlasma { get { return __covenantPlasma__; } set { __covenantPlasma__ = value; } }
                private bool __covenantPlasma__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    HumanFragmentation = (cur == value);
                    cur++;
                    CovenantPlasma = (cur == value);
                }
            }
            /// <summary>
            /// 89. UInt16 :: Offset: 0x1B6 (438)
            /// </summary>
            public ushort GrenadeCount { get { return __grenadeCount__; } set { __grenadeCount__ = value; } }
            private ushort __grenadeCount__;
            /// <summary>
            /// 90. TagBlock :: Offset: 0x1B8 (440)
            /// </summary>
            public _PoweredSeatsTagBlock PoweredSeats { get { return __poweredSeats__; } set { __poweredSeats__ = value; } }
            private _PoweredSeatsTagBlock __poweredSeats__;
            public class _PoweredSeatsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float DriverPowerupTime { get { return __driverPowerupTime__; } set { __driverPowerupTime__ = value; } }
                private float __driverPowerupTime__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float DriverPowerdownTime { get { return __driverPowerdownTime__; } set { __driverPowerdownTime__ = value; } }
                private float __driverPowerdownTime__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PoweredSeatsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PoweredSeatsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PoweredSeatsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PoweredSeatsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 91. TagBlock :: Offset: 0x1C0 (448)
            /// </summary>
            public _WeaponTagBlock Weapon { get { return __weapon__; } set { __weapon__ = value; } }
            private _WeaponTagBlock __weapon__;
            public class _WeaponTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Weapon { get { return __weapon__; } set { __weapon__ = value; } }
                private TagRef __weapon__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WeaponTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WeaponTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WeaponTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WeaponTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 92. TagBlock :: Offset: 0x1C8 (456)
            /// </summary>
            public _SeatsTagBlock Seats { get { return __seats__; } set { __seats__ = value; } }
            private _SeatsTagBlock __seats__;
            public class _SeatsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x40000 (262144)
                    /// </summary>
                    public bool FireWeapon { get { return __fireWeapon__; } set { __fireWeapon__ = value; } }
                    private bool __fireWeapon__;
                    /// <summary>
                    /// 2. Option :: Value: 0x10000 (65536)
                    /// </summary>
                    public bool DisableSeat { get { return __disableSeat__; } set { __disableSeat__ = value; } }
                    private bool __disableSeat__;
                    /// <summary>
                    /// 3. Option :: Value: 0x8000 (32768)
                    /// </summary>
                    public bool CloseDoor { get { return __closeDoor__; } set { __closeDoor__ = value; } }
                    private bool __closeDoor__;
                    /// <summary>
                    /// 4. Option :: Value: 0x2000 (8192)
                    /// </summary>
                    public bool TakeADrive_G { get { return __takeADrive_G__; } set { __takeADrive_G__ = value; } }
                    private bool __takeADrive_G__;
                    /// <summary>
                    /// 5. Option :: Value: 0x800 (2048)
                    /// </summary>
                    public bool HideBoardingSeat { get { return __hideBoardingSeat__; } set { __hideBoardingSeat__ = value; } }
                    private bool __hideBoardingSeat__;
                    /// <summary>
                    /// 6. Option :: Value: 0x200 (512)
                    /// </summary>
                    public bool DriverReqIn_w_DF { get { return __driverReqIn_w_DF__; } set { __driverReqIn_w_DF__ = value; } }
                    private bool __driverReqIn_w_DF__;
                    /// <summary>
                    /// 7. Option :: Value: 0x80 (128)
                    /// </summary>
                    public bool TurretSeat { get { return __turretSeat__; } set { __turretSeat__ = value; } }
                    private bool __turretSeat__;
                    /// <summary>
                    /// 8. Option :: Value: 0x20 (32)
                    /// </summary>
                    public bool EnableHUD { get { return __enableHUD__; } set { __enableHUD__ = value; } }
                    private bool __enableHUD__;
                    /// <summary>
                    /// 9. Option :: Value: 0x10 (16)
                    /// </summary>
                    public bool N { get { return __n__; } set { __n__ = value; } }
                    private bool __n__;
                    /// <summary>
                    /// 10. Option :: Value: 0x8 (8)
                    /// </summary>
                    public bool TurretFire_HW { get { return __turretFire_HW__; } set { __turretFire_HW__ = value; } }
                    private bool __turretFire_HW__;
                    /// <summary>
                    /// 11. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool DriverSeat { get { return __driverSeat__; } set { __driverSeat__ = value; } }
                    private bool __driverSeat__;
                    /// <summary>
                    /// 12. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool VehicleWeaponEnabled { get { return __vehicleWeaponEnabled__; } set { __vehicleWeaponEnabled__ = value; } }
                    private bool __vehicleWeaponEnabled__;
                    /// <summary>
                    /// 13. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool DriveSystemEnabled { get { return __driveSystemEnabled__; } set { __driveSystemEnabled__ = value; } }
                    private bool __driveSystemEnabled__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        FireWeapon = ((bit & mask) == bit);
                        bit <<= 1;
                        DisableSeat = ((bit & mask) == bit);
                        bit <<= 1;
                        CloseDoor = ((bit & mask) == bit);
                        bit <<= 1;
                        TakeADrive_G = ((bit & mask) == bit);
                        bit <<= 1;
                        HideBoardingSeat = ((bit & mask) == bit);
                        bit <<= 1;
                        DriverReqIn_w_DF = ((bit & mask) == bit);
                        bit <<= 1;
                        TurretSeat = ((bit & mask) == bit);
                        bit <<= 1;
                        EnableHUD = ((bit & mask) == bit);
                        bit <<= 1;
                        N = ((bit & mask) == bit);
                        bit <<= 1;
                        TurretFire_HW = ((bit & mask) == bit);
                        bit <<= 1;
                        DriverSeat = ((bit & mask) == bit);
                        bit <<= 1;
                        VehicleWeaponEnabled = ((bit & mask) == bit);
                        bit <<= 1;
                        DriveSystemEnabled = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Name { get { return __name__; } set { __name__ = value; } }
                private StringID __name__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID SittingPositionMarker { get { return __sittingPositionMarker__; } set { __sittingPositionMarker__ = value; } }
                private StringID __sittingPositionMarker__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID EnterMarker { get { return __enterMarker__; } set { __enterMarker__ = value; } }
                private StringID __enterMarker__;
                /// <summary>
                /// 5. StringID :: Offset: 0x10 (16)
                /// </summary>
                public StringID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private StringID __unknown1__;
                /// <summary>
                /// 6. StringID :: Offset: 0x14 (20)
                /// </summary>
                public StringID Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private StringID __unknown2__;
                /// <summary>
                /// 7. StringID :: Offset: 0x18 (24)
                /// </summary>
                public StringID Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private StringID __unknown3__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float AccelScaleI { get { return __accelScaleI__; } set { __accelScaleI__ = value; } }
                private float __accelScaleI__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float AccelScaleJ { get { return __accelScaleJ__; } set { __accelScaleJ__ = value; } }
                private float __accelScaleJ__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float AccelScaleK { get { return __accelScaleK__; } set { __accelScaleK__ = value; } }
                private float __accelScaleK__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float YawRate { get { return __yawRate__; } set { __yawRate__ = value; } }
                private float __yawRate__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float PitchRate { get { return __pitchRate__; } set { __pitchRate__ = value; } }
                private float __pitchRate__;
                /// <summary>
                /// 15. Int16 :: Offset: 0x38 (56)
                /// </summary>
                public short Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private short __unknown6__;
                /// <summary>
                /// 16. Int16 :: Offset: 0x3A (58)
                /// </summary>
                public short Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private short __unknown7__;
                /// <summary>
                /// 17. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 18. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 19. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 20. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 21. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 22. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 23. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 24. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 25. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 26. StringID :: Offset: 0x60 (96)
                /// </summary>
                public StringID CameraMarkerName { get { return __cameraMarkerName__; } set { __cameraMarkerName__ = value; } }
                private StringID __cameraMarkerName__;
                /// <summary>
                /// 27. StringID :: Offset: 0x64 (100)
                /// </summary>
                public StringID CameraSubmergedMarkerName { get { return __cameraSubmergedMarkerName__; } set { __cameraSubmergedMarkerName__ = value; } }
                private StringID __cameraSubmergedMarkerName__;
                /// <summary>
                /// 28. Float :: Offset: 0x68 (104)
                /// </summary>
                public float PitchAutoLevel { get { return __pitchAutoLevel__; } set { __pitchAutoLevel__ = value; } }
                private float __pitchAutoLevel__;
                /// <summary>
                /// 29. Float :: Offset: 0x6C (108)
                /// </summary>
                public float PitchRangeLower { get { return __pitchRangeLower__; } set { __pitchRangeLower__ = value; } }
                private float __pitchRangeLower__;
                /// <summary>
                /// 30. Float :: Offset: 0x70 (112)
                /// </summary>
                public float PitchRangeUpper { get { return __pitchRangeUpper__; } set { __pitchRangeUpper__ = value; } }
                private float __pitchRangeUpper__;
                /// <summary>
                /// 31. TagBlock :: Offset: 0x74 (116)
                /// </summary>
                public _CameraTracksTagBlock CameraTracks { get { return __cameraTracks__; } set { __cameraTracks__ = value; } }
                private _CameraTracksTagBlock __cameraTracks__;
                public class _CameraTracksTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private BipdLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef Camera { get { return __camera__; } set { __camera__ = value; } }
                    private TagRef __camera__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _CameraTracksTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _CameraTracksTagBlock(BipdLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_CameraTracksTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _CameraTracksTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 32. TagBlock :: Offset: 0x7C (124)
                /// </summary>
                public _UnitHudInterfaceTagBlock UnitHudInterface { get { return __unitHudInterface__; } set { __unitHudInterface__ = value; } }
                private _UnitHudInterfaceTagBlock __unitHudInterface__;
                public class _UnitHudInterfaceTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private BipdLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef Hud { get { return __hud__; } set { __hud__ = value; } }
                    private TagRef __hud__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _UnitHudInterfaceTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _UnitHudInterfaceTagBlock(BipdLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_UnitHudInterfaceTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _UnitHudInterfaceTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 33. StringID :: Offset: 0x84 (132)
                /// </summary>
                public StringID HudTextMessageIndex { get { return __hudTextMessageIndex__; } set { __hudTextMessageIndex__ = value; } }
                private StringID __hudTextMessageIndex__;
                /// <summary>
                /// 34. Float :: Offset: 0x88 (136)
                /// </summary>
                public float YawMin { get { return __yawMin__; } set { __yawMin__ = value; } }
                private float __yawMin__;
                /// <summary>
                /// 35. Float :: Offset: 0x8C (140)
                /// </summary>
                public float YawMax { get { return __yawMax__; } set { __yawMax__ = value; } }
                private float __yawMax__;
                /// <summary>
                /// 36. Tag :: Offset: 0x90 (144)
                /// </summary>
                public TagRef BuiltInGunner { get { return __builtInGunner__; } set { __builtInGunner__ = value; } }
                private TagRef __builtInGunner__;
                /// <summary>
                /// 37. Float :: Offset: 0x98 (152)
                /// </summary>
                public float SeatEnterRadius { get { return __seatEnterRadius__; } set { __seatEnterRadius__ = value; } }
                private float __seatEnterRadius__;
                /// <summary>
                /// 38. Float :: Offset: 0x9C (156)
                /// </summary>
                public float SomeAngle { get { return __someAngle__; } set { __someAngle__ = value; } }
                private float __someAngle__;
                /// <summary>
                /// 39. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float SeatEnterAngle { get { return __seatEnterAngle__; } set { __seatEnterAngle__ = value; } }
                private float __seatEnterAngle__;
                /// <summary>
                /// 40. Float :: Offset: 0xA4 (164)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 41. StringID :: Offset: 0xA8 (168)
                /// </summary>
                public StringID Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private StringID __unknown18__;
                /// <summary>
                /// 42. ID :: Offset: 0xAC (172)
                /// </summary>
                public LoneID Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private LoneID __unknown19__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _SeatsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _SeatsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                    CameraTracks = new _CameraTracksTagBlock(layout);
                    UnitHudInterface = new _UnitHudInterfaceTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_SeatsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _SeatsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 93. Float :: Offset: 0x1D0 (464)
            /// </summary>
            public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private float __unknown6__;
            /// <summary>
            /// 94. Float :: Offset: 0x1D4 (468)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;
            /// <summary>
            /// 95. Float :: Offset: 0x1D8 (472)
            /// </summary>
            public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private float __unknown8__;
            /// <summary>
            /// 96. Float :: Offset: 0x1DC (476)
            /// </summary>
            public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private float __unknown9__;
            /// <summary>
            /// 97. Float :: Offset: 0x1E0 (480)
            /// </summary>
            public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private float __unknown10__;
            /// <summary>
            /// 98. ByteArray :: Offset: 0x1E4 (484)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;
            /// <summary>
            /// 99. Float :: Offset: 0x1EC (492)
            /// </summary>
            public float MovingTurningSpeed { get { return __movingTurningSpeed__; } set { __movingTurningSpeed__ = value; } }
            private float __movingTurningSpeed__;
            /// <summary>
            /// 100. Bitmask32 :: Offset: 0x1F0 (496)
            /// </summary>
            public RagdollBitmasksBitmask32 RagdollBitmasks { get { return __ragdollBitmasks__; } set { __ragdollBitmasks__ = value; } }
            private RagdollBitmasksBitmask32 __ragdollBitmasks__;
            public class RagdollBitmasksBitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool TurnsWithoutAnimating { get { return __turnsWithoutAnimating__; } set { __turnsWithoutAnimating__ = value; } }
                private bool __turnsWithoutAnimating__;
                /// <summary>
                /// 2. Option :: Value: 0x3 (3)
                /// </summary>
                public bool PhysicsPillCenteredAtOrigin { get { return __physicsPillCenteredAtOrigin__; } set { __physicsPillCenteredAtOrigin__ = value; } }
                private bool __physicsPillCenteredAtOrigin__;
                /// <summary>
                /// 3. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Spherical { get { return __spherical__; } set { __spherical__ = value; } }
                private bool __spherical__;
                /// <summary>
                /// 4. Option :: Value: 0x7 (7)
                /// </summary>
                public bool ImmuneToFallingDamage { get { return __immuneToFallingDamage__; } set { __immuneToFallingDamage__ = value; } }
                private bool __immuneToFallingDamage__;
                /// <summary>
                /// 5. Option :: Value: 0x8 (8)
                /// </summary>
                public bool RotateWhileAirborne { get { return __rotateWhileAirborne__; } set { __rotateWhileAirborne__ = value; } }
                private bool __rotateWhileAirborne__;
                /// <summary>
                /// 6. Option :: Value: 0x9 (9)
                /// </summary>
                public bool UsesLimpBodyPhysics_RD { get { return __usesLimpBodyPhysics_RD__; } set { __usesLimpBodyPhysics_RD__ = value; } }
                private bool __usesLimpBodyPhysics_RD__;
                /// <summary>
                /// 7. Option :: Value: 0xA (10)
                /// </summary>
                public bool HasNoDyingAirborne_RD { get { return __hasNoDyingAirborne_RD__; } set { __hasNoDyingAirborne_RD__ = value; } }
                private bool __hasNoDyingAirborne_RD__;
                /// <summary>
                /// 8. Option :: Value: 0xB (11)
                /// </summary>
                public bool RandomSpeedIncrease_RD { get { return __randomSpeedIncrease_RD__; } set { __randomSpeedIncrease_RD__ = value; } }
                private bool __randomSpeedIncrease_RD__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    TurnsWithoutAnimating = ((bit & mask) == bit);
                    bit <<= 1;
                    PhysicsPillCenteredAtOrigin = ((bit & mask) == bit);
                    bit <<= 1;
                    Spherical = ((bit & mask) == bit);
                    bit <<= 1;
                    ImmuneToFallingDamage = ((bit & mask) == bit);
                    bit <<= 1;
                    RotateWhileAirborne = ((bit & mask) == bit);
                    bit <<= 1;
                    UsesLimpBodyPhysics_RD = ((bit & mask) == bit);
                    bit <<= 1;
                    HasNoDyingAirborne_RD = ((bit & mask) == bit);
                    bit <<= 1;
                    RandomSpeedIncrease_RD = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 101. Float :: Offset: 0x1F4 (500)
            /// </summary>
            public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private float __unknown11__;
            /// <summary>
            /// 102. Float :: Offset: 0x1F8 (504)
            /// </summary>
            public float JumpVelocity { get { return __jumpVelocity__; } set { __jumpVelocity__ = value; } }
            private float __jumpVelocity__;
            /// <summary>
            /// 103. Float :: Offset: 0x1FC (508)
            /// </summary>
            public float MaxSoftLandingTime { get { return __maxSoftLandingTime__; } set { __maxSoftLandingTime__ = value; } }
            private float __maxSoftLandingTime__;
            /// <summary>
            /// 104. Float :: Offset: 0x200 (512)
            /// </summary>
            public float MaxHardLandingTime { get { return __maxHardLandingTime__; } set { __maxHardLandingTime__ = value; } }
            private float __maxHardLandingTime__;
            /// <summary>
            /// 105. Float :: Offset: 0x204 (516)
            /// </summary>
            public float MinSoftLandingVelocity { get { return __minSoftLandingVelocity__; } set { __minSoftLandingVelocity__ = value; } }
            private float __minSoftLandingVelocity__;
            /// <summary>
            /// 106. Float :: Offset: 0x208 (520)
            /// </summary>
            public float MinHardLandingVelocity { get { return __minHardLandingVelocity__; } set { __minHardLandingVelocity__ = value; } }
            private float __minHardLandingVelocity__;
            /// <summary>
            /// 107. Float :: Offset: 0x20C (524)
            /// </summary>
            public float MaxHardLandingVelocity { get { return __maxHardLandingVelocity__; } set { __maxHardLandingVelocity__ = value; } }
            private float __maxHardLandingVelocity__;
            /// <summary>
            /// 108. Float :: Offset: 0x210 (528)
            /// </summary>
            public float DeathHardLandingVelocity { get { return __deathHardLandingVelocity__; } set { __deathHardLandingVelocity__ = value; } }
            private float __deathHardLandingVelocity__;
            /// <summary>
            /// 109. Float :: Offset: 0x214 (532)
            /// </summary>
            public float RagdollTimer { get { return __ragdollTimer__; } set { __ragdollTimer__ = value; } }
            private float __ragdollTimer__;
            /// <summary>
            /// 110. Float :: Offset: 0x218 (536)
            /// </summary>
            public float StandingCameraHeight { get { return __standingCameraHeight__; } set { __standingCameraHeight__ = value; } }
            private float __standingCameraHeight__;
            /// <summary>
            /// 111. Float :: Offset: 0x21C (540)
            /// </summary>
            public float CrouchingCameraHeight { get { return __crouchingCameraHeight__; } set { __crouchingCameraHeight__ = value; } }
            private float __crouchingCameraHeight__;
            /// <summary>
            /// 112. Float :: Offset: 0x220 (544)
            /// </summary>
            public float CrouchTransitionTime { get { return __crouchTransitionTime__; } set { __crouchTransitionTime__ = value; } }
            private float __crouchTransitionTime__;
            /// <summary>
            /// 113. Float :: Offset: 0x224 (548)
            /// </summary>
            public float StandingCollisionHeight { get { return __standingCollisionHeight__; } set { __standingCollisionHeight__ = value; } }
            private float __standingCollisionHeight__;
            /// <summary>
            /// 114. Float :: Offset: 0x228 (552)
            /// </summary>
            public float CollisionRadius { get { return __collisionRadius__; } set { __collisionRadius__ = value; } }
            private float __collisionRadius__;
            /// <summary>
            /// 115. Float :: Offset: 0x22C (556)
            /// </summary>
            public float LeanForward_lookingDown { get { return __leanForward_lookingDown__; } set { __leanForward_lookingDown__ = value; } }
            private float __leanForward_lookingDown__;
            /// <summary>
            /// 116. Float :: Offset: 0x230 (560)
            /// </summary>
            public float LeanRight_lookingDown { get { return __leanRight_lookingDown__; } set { __leanRight_lookingDown__ = value; } }
            private float __leanRight_lookingDown__;
            /// <summary>
            /// 117. Float :: Offset: 0x234 (564)
            /// </summary>
            public float LeanBackward_lookingDown { get { return __leanBackward_lookingDown__; } set { __leanBackward_lookingDown__ = value; } }
            private float __leanBackward_lookingDown__;
            /// <summary>
            /// 118. Float :: Offset: 0x238 (568)
            /// </summary>
            public float LeanPossibly_verySimilarToLeanForward { get { return __leanPossibly_verySimilarToLeanForward__; } set { __leanPossibly_verySimilarToLeanForward__ = value; } }
            private float __leanPossibly_verySimilarToLeanForward__;
            /// <summary>
            /// 119. Float :: Offset: 0x23C (572)
            /// </summary>
            public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private float __unknown12__;
            /// <summary>
            /// 120. Float :: Offset: 0x240 (576)
            /// </summary>
            public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private float __unknown13__;
            /// <summary>
            /// 121. Float :: Offset: 0x244 (580)
            /// </summary>
            public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
            private float __unknown14__;
            /// <summary>
            /// 122. Float :: Offset: 0x248 (584)
            /// </summary>
            public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
            private float __unknown15__;
            /// <summary>
            /// 123. Float :: Offset: 0x24C (588)
            /// </summary>
            public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
            private float __unknown16__;
            /// <summary>
            /// 124. Float :: Offset: 0x250 (592)
            /// </summary>
            public float CrouchSpeed { get { return __crouchSpeed__; } set { __crouchSpeed__ = value; } }
            private float __crouchSpeed__;
            /// <summary>
            /// 125. Float :: Offset: 0x254 (596)
            /// </summary>
            public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
            private float __unknown17__;
            /// <summary>
            /// 126. Float :: Offset: 0x258 (600)
            /// </summary>
            public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
            private float __unknown18__;
            /// <summary>
            /// 127. Tag :: Offset: 0x25C (604)
            /// </summary>
            public TagRef Blood { get { return __blood__; } set { __blood__ = value; } }
            private TagRef __blood__;
            /// <summary>
            /// 128. Bitmask16 :: Offset: 0x264 (612)
            /// </summary>
            public MovementBitmask16 Movement { get { return __movement__; } set { __movement__ = value; } }
            private MovementBitmask16 __movement__;
            public class MovementBitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool TurnsWithoutAnimating { get { return __turnsWithoutAnimating__; } set { __turnsWithoutAnimating__ = value; } }
                private bool __turnsWithoutAnimating__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Flying { get { return __flying__; } set { __flying__ = value; } }
                private bool __flying__;
                /// <summary>
                /// 3. Option :: Value: 0x4 (4)
                /// </summary>
                public bool GlobalControlled { get { return __globalControlled__; } set { __globalControlled__ = value; } }
                private bool __globalControlled__;
                /// <summary>
                /// 4. Option :: Value: 0x5 (5)
                /// </summary>
                public bool PassesThroughOtherBipeds { get { return __passesThroughOtherBipeds__; } set { __passesThroughOtherBipeds__ = value; } }
                private bool __passesThroughOtherBipeds__;
                /// <summary>
                /// 5. Option :: Value: 0x8 (8)
                /// </summary>
                public bool CanClimbAnySurface { get { return __canClimbAnySurface__; } set { __canClimbAnySurface__ = value; } }
                private bool __canClimbAnySurface__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    TurnsWithoutAnimating = ((bit & mask) == bit);
                    bit <<= 1;
                    Flying = ((bit & mask) == bit);
                    bit <<= 1;
                    GlobalControlled = ((bit & mask) == bit);
                    bit <<= 1;
                    PassesThroughOtherBipeds = ((bit & mask) == bit);
                    bit <<= 1;
                    CanClimbAnySurface = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 129. Int16 :: Offset: 0x266 (614)
            /// </summary>
            public short Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
            private short __unknown19__;
            /// <summary>
            /// 130. Float :: Offset: 0x268 (616)
            /// </summary>
            public float CrouchBounce { get { return __crouchBounce__; } set { __crouchBounce__ = value; } }
            private float __crouchBounce__;
            /// <summary>
            /// 131. Float :: Offset: 0x26C (620)
            /// </summary>
            public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
            private float __unknown20__;
            /// <summary>
            /// 132. Float :: Offset: 0x270 (624)
            /// </summary>
            public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
            private float __unknown21__;
            /// <summary>
            /// 133. Float :: Offset: 0x274 (628)
            /// </summary>
            public float StationaryTurningThreshold { get { return __stationaryTurningThreshold__; } set { __stationaryTurningThreshold__ = value; } }
            private float __stationaryTurningThreshold__;
            /// <summary>
            /// 134. StringID :: Offset: 0x278 (632)
            /// </summary>
            public StringID MaterialPossibly1 { get { return __materialPossibly1__; } set { __materialPossibly1__ = value; } }
            private StringID __materialPossibly1__;
            /// <summary>
            /// 135. StringID :: Offset: 0x27C (636)
            /// </summary>
            public StringID MaterialPossibly2 { get { return __materialPossibly2__; } set { __materialPossibly2__ = value; } }
            private StringID __materialPossibly2__;
            /// <summary>
            /// 136. Bitmask32 :: Offset: 0x264 (612)
            /// </summary>
            public Unknown22Bitmask32 Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
            private Unknown22Bitmask32 __unknown22__;
            public class Unknown22Bitmask32
            {

                public void _Initialize(BitReader br)
                {
                    // Empty..
                    br.BaseStream.Position += 4;
                }
            }
            /// <summary>
            /// 137. ByteArray :: Offset: 0x268 (616)
            /// </summary>
            public byte[] Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
            private byte[] __unknown23__;
            /// <summary>
            /// 138. TagBlock :: Offset: 0x284 (644)
            /// </summary>
            public _Unknown24TagBlock Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
            private _Unknown24TagBlock __unknown24__;
            public class _Unknown24TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. UInt16 :: Offset: 0x4 (4)
                /// </summary>
                public ushort Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private ushort __unknown1__;
                /// <summary>
                /// 3. UInt16 :: Offset: 0x6 (6)
                /// </summary>
                public ushort Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private ushort __unknown2__;
                /// <summary>
                /// 4. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 5. UInt16 :: Offset: 0x1C (28)
                /// </summary>
                public ushort Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private ushort __unknown3__;
                /// <summary>
                /// 6. UInt16 :: Offset: 0x1E (30)
                /// </summary>
                public ushort Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private ushort __unknown4__;
                /// <summary>
                /// 7. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 8. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 9. ByteArray :: Offset: 0x28 (40)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;
                /// <summary>
                /// 10. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 11. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 12. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 13. ByteArray :: Offset: 0x38 (56)
                /// </summary>
                public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                private byte[] __unused4__;
                /// <summary>
                /// 14. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 15. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 16. ByteArray :: Offset: 0x44 (68)
                /// </summary>
                public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
                private byte[] __unused5__;
                /// <summary>
                /// 17. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 18. ByteArray :: Offset: 0x58 (88)
                /// </summary>
                public byte[] Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
                private byte[] __unused6__;
                /// <summary>
                /// 19. Float :: Offset: 0x68 (104)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 20. ByteArray :: Offset: 0x6C (108)
                /// </summary>
                public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
                private byte[] __unused7__;
                /// <summary>
                /// 21. Float :: Offset: 0x78 (120)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 22. ByteArray :: Offset: 0x7C (124)
                /// </summary>
                public byte[] Unused8 { get { return __unused8__; } set { __unused8__ = value; } }
                private byte[] __unused8__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown24TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown24TagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown24TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown24TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 139. TagBlock :: Offset: 0x28C (652)
            /// </summary>
            public _Unknown25TagBlock Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
            private _Unknown25TagBlock __unknown25__;
            public class _Unknown25TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. UInt16 :: Offset: 0x4 (4)
                /// </summary>
                public ushort Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private ushort __unknown1__;
                /// <summary>
                /// 3. UInt16 :: Offset: 0x6 (6)
                /// </summary>
                public ushort Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private ushort __unknown2__;
                /// <summary>
                /// 4. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 5. UInt16 :: Offset: 0x1C (28)
                /// </summary>
                public ushort Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private ushort __unknown3__;
                /// <summary>
                /// 6. UInt16 :: Offset: 0x1E (30)
                /// </summary>
                public ushort Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private ushort __unknown4__;
                /// <summary>
                /// 7. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 8. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 9. ByteArray :: Offset: 0x28 (40)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;
                /// <summary>
                /// 10. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 11. ByteArray :: Offset: 0x30 (48)
                /// </summary>
                public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                private byte[] __unused4__;
                /// <summary>
                /// 12. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 13. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 14. ByteArray :: Offset: 0x40 (64)
                /// </summary>
                public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
                private byte[] __unused5__;
                /// <summary>
                /// 15. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 16. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown25TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown25TagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown25TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown25TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 140. TagBlock :: Offset: 0x294 (660)
            /// </summary>
            public _Unknown26TagBlock Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
            private _Unknown26TagBlock __unknown26__;
            public class _Unknown26TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 4. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 5. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 6. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 7. ByteArray :: Offset: 0x28 (40)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;
                /// <summary>
                /// 8. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 9. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 10. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 11. ByteArray :: Offset: 0x38 (56)
                /// </summary>
                public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                private byte[] __unused4__;
                /// <summary>
                /// 12. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 13. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 14. ByteArray :: Offset: 0x44 (68)
                /// </summary>
                public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
                private byte[] __unused5__;
                /// <summary>
                /// 15. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 16. ByteArray :: Offset: 0x58 (88)
                /// </summary>
                public byte[] Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
                private byte[] __unused6__;
                /// <summary>
                /// 17. Float :: Offset: 0x68 (104)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 18. ByteArray :: Offset: 0x6C (108)
                /// </summary>
                public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
                private byte[] __unused7__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown26TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown26TagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown26TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown26TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 141. Float :: Offset: 0x29C (668)
            /// </summary>
            public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
            private float __unknown27__;
            /// <summary>
            /// 142. Float :: Offset: 0x2A0 (672)
            /// </summary>
            public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
            private float __unknown28__;
            /// <summary>
            /// 143. Float :: Offset: 0x2A4 (676)
            /// </summary>
            public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
            private float __unknown29__;
            /// <summary>
            /// 144. Float :: Offset: 0x2A8 (680)
            /// </summary>
            public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
            private float __unknown30__;
            /// <summary>
            /// 145. Float :: Offset: 0x2AC (684)
            /// </summary>
            public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
            private float __unknown31__;
            /// <summary>
            /// 146. Float :: Offset: 0x2B0 (688)
            /// </summary>
            public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
            private float __unknown32__;
            /// <summary>
            /// 147. Float :: Offset: 0x2B4 (692)
            /// </summary>
            public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
            private float __unknown33__;
            /// <summary>
            /// 148. Float :: Offset: 0x2B8 (696)
            /// </summary>
            public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
            private float __unknown34__;
            /// <summary>
            /// 149. Float :: Offset: 0x2BC (700)
            /// </summary>
            public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
            private float __unknown35__;
            /// <summary>
            /// 150. Float :: Offset: 0x2C0 (704)
            /// </summary>
            public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
            private float __unknown36__;
            /// <summary>
            /// 151. Float :: Offset: 0x2C4 (708)
            /// </summary>
            public float Unknown37 { get { return __unknown37__; } set { __unknown37__ = value; } }
            private float __unknown37__;
            /// <summary>
            /// 152. Float :: Offset: 0x2C8 (712)
            /// </summary>
            public float Unknown38 { get { return __unknown38__; } set { __unknown38__ = value; } }
            private float __unknown38__;
            /// <summary>
            /// 153. Float :: Offset: 0x2CC (716)
            /// </summary>
            public float FlyingBankAngle { get { return __flyingBankAngle__; } set { __flyingBankAngle__ = value; } }
            private float __flyingBankAngle__;
            /// <summary>
            /// 154. Float :: Offset: 0x2D0 (720)
            /// </summary>
            public float FlyingBankApplyTime { get { return __flyingBankApplyTime__; } set { __flyingBankApplyTime__ = value; } }
            private float __flyingBankApplyTime__;
            /// <summary>
            /// 155. Float :: Offset: 0x2D4 (724)
            /// </summary>
            public float FlyingPitchRatio { get { return __flyingPitchRatio__; } set { __flyingPitchRatio__ = value; } }
            private float __flyingPitchRatio__;
            /// <summary>
            /// 156. Float :: Offset: 0x2D8 (728)
            /// </summary>
            public float FlyingMaxVelocity { get { return __flyingMaxVelocity__; } set { __flyingMaxVelocity__ = value; } }
            private float __flyingMaxVelocity__;
            /// <summary>
            /// 157. Float :: Offset: 0x2DC (732)
            /// </summary>
            public float FlyingMaxSidestepVelocity { get { return __flyingMaxSidestepVelocity__; } set { __flyingMaxSidestepVelocity__ = value; } }
            private float __flyingMaxSidestepVelocity__;
            /// <summary>
            /// 158. Float :: Offset: 0x2E0 (736)
            /// </summary>
            public float FlyingAcceleration { get { return __flyingAcceleration__; } set { __flyingAcceleration__ = value; } }
            private float __flyingAcceleration__;
            /// <summary>
            /// 159. Float :: Offset: 0x2E4 (740)
            /// </summary>
            public float FlyingDeceleration { get { return __flyingDeceleration__; } set { __flyingDeceleration__ = value; } }
            private float __flyingDeceleration__;
            /// <summary>
            /// 160. Float :: Offset: 0x2E8 (744)
            /// </summary>
            public float FlyingAngularVelocityMax { get { return __flyingAngularVelocityMax__; } set { __flyingAngularVelocityMax__ = value; } }
            private float __flyingAngularVelocityMax__;
            /// <summary>
            /// 161. Float :: Offset: 0x2EC (748)
            /// </summary>
            public float FlyingAngularAccelerationMax { get { return __flyingAngularAccelerationMax__; } set { __flyingAngularAccelerationMax__ = value; } }
            private float __flyingAngularAccelerationMax__;
            /// <summary>
            /// 162. Float :: Offset: 0x2F0 (752)
            /// </summary>
            public float FlyingCrouchVelocityModifier { get { return __flyingCrouchVelocityModifier__; } set { __flyingCrouchVelocityModifier__ = value; } }
            private float __flyingCrouchVelocityModifier__;
            /// <summary>
            /// 163. Float :: Offset: 0x2F4 (756)
            /// </summary>
            public float Unknown39 { get { return __unknown39__; } set { __unknown39__ = value; } }
            private float __unknown39__;
            /// <summary>
            /// 164. TagBlock :: Offset: 0x2F8 (760)
            /// </summary>
            public _ContactPointsTagBlock ContactPoints { get { return __contactPoints__; } set { __contactPoints__ = value; } }
            private _ContactPointsTagBlock __contactPoints__;
            public class _ContactPointsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public BipdLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private BipdLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ContactPointsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ContactPointsTagBlock(BipdLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ContactPointsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ContactPointsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 165. Tag :: Offset: 0x300 (768)
            /// </summary>
            public TagRef Unknown40 { get { return __unknown40__; } set { __unknown40__ = value; } }
            private TagRef __unknown40__;
            /// <summary>
            /// 166. Tag :: Offset: 0x308 (776)
            /// </summary>
            public TagRef DeathSpawn { get { return __deathSpawn__; } set { __deathSpawn__ = value; } }
            private TagRef __deathSpawn__;
            /// <summary>
            /// 167. Int16 :: Offset: 0x310 (784)
            /// </summary>
            public short SpawnCount { get { return __spawnCount__; } set { __spawnCount__ = value; } }
            private short __spawnCount__;
            /// <summary>
            /// 168. Int16 :: Offset: 0x312 (786)
            /// </summary>
            public short Unknown41 { get { return __unknown41__; } set { __unknown41__ = value; } }
            private short __unknown41__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the BipdLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public BipdLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags1 = new Flags1Bitmask32();
                AiProperties = new _AiPropertiesTagBlock(this);
                Functions = new _FunctionsTagBlock(this);
                Attachments = new _AttachmentsTagBlock(this);
                Widgets = new _WidgetsTagBlock(this);
                ChangeColors = new _ChangeColorsTagBlock(this);
                PredictedResource = new _PredictedResourceTagBlock(this);
                Flags2 = new Flags2Bitmask32();
                DefaultTeam = new DefaultTeamEnum16();
                ConstantSoundVolume = new ConstantSoundVolumeEnum16();
                CameraTracks = new _CameraTracksTagBlock(this);
                MotionSensorBlipSize = new MotionSensorBlipSizeEnum32();
                Unknown4 = new _Unknown4TagBlock(this);
                HudInterfaces = new _HudInterfacesTagBlock(this);
                Unknown5 = new _Unknown5TagBlock(this);
                GrenadeType = new GrenadeTypeEnum16();
                PoweredSeats = new _PoweredSeatsTagBlock(this);
                Weapon = new _WeaponTagBlock(this);
                Seats = new _SeatsTagBlock(this);
                RagdollBitmasks = new RagdollBitmasksBitmask32();
                Movement = new MovementBitmask16();
                Unknown22 = new Unknown22Bitmask32();
                Unknown24 = new _Unknown24TagBlock(this);
                Unknown25 = new _Unknown25TagBlock(this);
                Unknown26 = new _Unknown26TagBlock(this);
                ContactPoints = new _ContactPointsTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags1._Initialize(br);
                BoundingRadius = br.ReadSingle();
                BoundingOffsetX = br.ReadSingle();
                BoundingOffsetY = br.ReadSingle();
                BoundingOffsetZ = br.ReadSingle();
                AccelerationScale = br.ReadSingle();
                LightmapShadowMode = br.ReadSingle();
                SweetenerSize = br.ReadSingle();
                DynamicLightSphereRadius = br.ReadSingle();
                DynamicLightSphereOffsetX = br.ReadSingle();
                DynamicLightSphereOffsetY = br.ReadSingle();
                DynamicLightSphereOffsetZ = br.ReadSingle();
                DefaultModelVariant._Initialize(br);
                Model._Initialize(br);
                CrateObject._Initialize(br);
                ModifierShader._Initialize(br);
                CreationEffect._Initialize(br);
                MaterialEffects._Initialize(br);
                AiProperties._Initialize(br);
                Functions._Initialize(br);
                RenderBoundingRadius = br.ReadSingle();
                Lower1 = br.ReadSingle();
                Upper1 = br.ReadSingle();
                Lower2 = br.ReadSingle();
                Upper2 = br.ReadSingle();
                Lower3 = br.ReadSingle();
                Upper3 = br.ReadSingle();
                Lower4 = br.ReadSingle();
                Upper4 = br.ReadSingle();
                HudTextMessageIndex = br.ReadInt16();
                ForcedShaderPermutationIndex = br.ReadInt16();
                Attachments._Initialize(br);
                Widgets._Initialize(br);
                Unused1 = br.ReadBytes(8);
                ChangeColors._Initialize(br);
                PredictedResource._Initialize(br);
                Flags2._Initialize(br);
                DefaultTeam._Initialize(br);
                ConstantSoundVolume._Initialize(br);
                IntegratedLightToggle._Initialize(br);
                CameraFieldOfView = br.ReadSingle();
                CameraStiffness = br.ReadSingle();
                CameraMarkerName._Initialize(br);
                CameraSubmergedMarkerName._Initialize(br);
                PitchAutoLevel = br.ReadSingle();
                PitchRange1 = br.ReadSingle();
                PitchRange2 = br.ReadSingle();
                CameraTracks._Initialize(br);
                SeatAccelerationScaleI = br.ReadSingle();
                SeatAccelerationScaleJ = br.ReadSingle();
                SeatAccelerationScaleK = br.ReadSingle();
                Unused2 = br.ReadBytes(8);
                SoftPingThreshold = br.ReadSingle();
                SoftPingInterruptTime = br.ReadSingle();
                HardPingThreshold = br.ReadSingle();
                HardPingInterruptTime = br.ReadSingle();
                HardPingDeathThreshold = br.ReadSingle();
                FeignDeathThreshold = br.ReadSingle();
                FeignDeathTime = br.ReadSingle();
                DistOfEvadeAnim = br.ReadSingle();
                DistOfDiveAnim = br.ReadSingle();
                StunnedMovementThreshold = br.ReadSingle();
                FeignDeathChance = br.ReadSingle();
                FeignRepeatChance = br.ReadSingle();
                SpawnedActor._Initialize(br);
                SpawnedActorCountLower = br.ReadInt16();
                SpawnedActorCountUpper = br.ReadInt16();
                SpawnedVelocity = br.ReadSingle();
                AimingVelocityMax = br.ReadSingle();
                AimingAccelMax = br.ReadSingle();
                CasualAimingModifier = br.ReadSingle();
                LookingVelocityMax = br.ReadSingle();
                LookingAccelMax = br.ReadSingle();
                RightHand1._Initialize(br);
                LeftHand._Initialize(br);
                RightHand2._Initialize(br);
                MeleeDamage._Initialize(br);
                BoardingMeleeDamage._Initialize(br);
                BoardingMeleeResponseDamage._Initialize(br);
                Unknown1._Initialize(br);
                Unknown2._Initialize(br);
                Unknown3._Initialize(br);
                MotionSensorBlipSize._Initialize(br);
                Unknown4._Initialize(br);
                HudInterfaces._Initialize(br);
                Unknown5._Initialize(br);
                GrenadeVelocity = br.ReadSingle();
                GrenadeType._Initialize(br);
                GrenadeCount = br.ReadUInt16();
                PoweredSeats._Initialize(br);
                Weapon._Initialize(br);
                Seats._Initialize(br);
                Unknown6 = br.ReadSingle();
                Unknown7 = br.ReadSingle();
                Unknown8 = br.ReadSingle();
                Unknown9 = br.ReadSingle();
                Unknown10 = br.ReadSingle();
                Unused3 = br.ReadBytes(8);
                MovingTurningSpeed = br.ReadSingle();
                RagdollBitmasks._Initialize(br);
                Unknown11 = br.ReadSingle();
                JumpVelocity = br.ReadSingle();
                MaxSoftLandingTime = br.ReadSingle();
                MaxHardLandingTime = br.ReadSingle();
                MinSoftLandingVelocity = br.ReadSingle();
                MinHardLandingVelocity = br.ReadSingle();
                MaxHardLandingVelocity = br.ReadSingle();
                DeathHardLandingVelocity = br.ReadSingle();
                RagdollTimer = br.ReadSingle();
                StandingCameraHeight = br.ReadSingle();
                CrouchingCameraHeight = br.ReadSingle();
                CrouchTransitionTime = br.ReadSingle();
                StandingCollisionHeight = br.ReadSingle();
                CollisionRadius = br.ReadSingle();
                LeanForward_lookingDown = br.ReadSingle();
                LeanRight_lookingDown = br.ReadSingle();
                LeanBackward_lookingDown = br.ReadSingle();
                LeanPossibly_verySimilarToLeanForward = br.ReadSingle();
                Unknown12 = br.ReadSingle();
                Unknown13 = br.ReadSingle();
                Unknown14 = br.ReadSingle();
                Unknown15 = br.ReadSingle();
                Unknown16 = br.ReadSingle();
                CrouchSpeed = br.ReadSingle();
                Unknown17 = br.ReadSingle();
                Unknown18 = br.ReadSingle();
                Blood._Initialize(br);
                Movement._Initialize(br);
                Unknown19 = br.ReadInt16();
                CrouchBounce = br.ReadSingle();
                Unknown20 = br.ReadSingle();
                Unknown21 = br.ReadSingle();
                StationaryTurningThreshold = br.ReadSingle();
                MaterialPossibly1._Initialize(br);
                MaterialPossibly2._Initialize(br);
                Unknown22._Initialize(br);
                Unknown23 = br.ReadBytes(28);
                Unknown24._Initialize(br);
                Unknown25._Initialize(br);
                Unknown26._Initialize(br);
                Unknown27 = br.ReadSingle();
                Unknown28 = br.ReadSingle();
                Unknown29 = br.ReadSingle();
                Unknown30 = br.ReadSingle();
                Unknown31 = br.ReadSingle();
                Unknown32 = br.ReadSingle();
                Unknown33 = br.ReadSingle();
                Unknown34 = br.ReadSingle();
                Unknown35 = br.ReadSingle();
                Unknown36 = br.ReadSingle();
                Unknown37 = br.ReadSingle();
                Unknown38 = br.ReadSingle();
                FlyingBankAngle = br.ReadSingle();
                FlyingBankApplyTime = br.ReadSingle();
                FlyingPitchRatio = br.ReadSingle();
                FlyingMaxVelocity = br.ReadSingle();
                FlyingMaxSidestepVelocity = br.ReadSingle();
                FlyingAcceleration = br.ReadSingle();
                FlyingDeceleration = br.ReadSingle();
                FlyingAngularVelocityMax = br.ReadSingle();
                FlyingAngularAccelerationMax = br.ReadSingle();
                FlyingCrouchVelocityModifier = br.ReadSingle();
                Unknown39 = br.ReadSingle();
                ContactPoints._Initialize(br);
                Unknown40._Initialize(br);
                DeathSpawn._Initialize(br);
                SpawnCount = br.ReadInt16();
                Unknown41 = br.ReadInt16();

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Bipd class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Bipd(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new BipdLayout(map, identity);
        }

        #region Specific Methods of bipd

        #endregion
    }
}
