using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Character
    /// </summary>
    public class Char
    {
        #region Constants

        /// <summary>
        /// Size of the char meta header.
        /// </summary>
        public const int HeaderSize = 236;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private CharLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public CharLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class CharLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. ByteArray :: Offset: 0x0 (0)
            /// </summary>
            public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
            private byte[] __unused__;
            /// <summary>
            /// 2. Tag :: Offset: 0x4 (4)
            /// </summary>
            public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private TagRef __unknown1__;
            /// <summary>
            /// 3. Tag :: Offset: 0xC (12)
            /// </summary>
            public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private TagRef __unknown2__;
            /// <summary>
            /// 4. Tag :: Offset: 0x14 (20)
            /// </summary>
            public TagRef Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private TagRef __unknown3__;
            /// <summary>
            /// 5. Tag :: Offset: 0x1C (28)
            /// </summary>
            public TagRef Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private TagRef __unknown4__;
            /// <summary>
            /// 6. Tag :: Offset: 0x24 (36)
            /// </summary>
            public TagRef Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private TagRef __unknown5__;
            /// <summary>
            /// 7. TagBlock :: Offset: 0x2C (44)
            /// </summary>
            public _VariationTagBlock Variation { get { return __variation__; } set { __variation__ = value; } }
            private _VariationTagBlock __variation__;
            public class _VariationTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Permutation { get { return __permutation__; } set { __permutation__ = value; } }
                private StringID __permutation__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private float __unknown__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID BasePermutation { get { return __basePermutation__; } set { __basePermutation__ = value; } }
                private StringID __basePermutation__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _VariationTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _VariationTagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_VariationTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _VariationTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 8. TagBlock :: Offset: 0x34 (52)
            /// </summary>
            public _FlagsTagBlock Flags { get { return __flags__; } set { __flags__ = value; } }
            private _FlagsTagBlock __flags__;
            public class _FlagsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool DoNotSpawn { get { return __doNotSpawn__; } set { __doNotSpawn__ = value; } }
                    private bool __doNotSpawn__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool DoNotTurn { get { return __doNotTurn__; } set { __doNotTurn__ = value; } }
                    private bool __doNotTurn__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        DoNotSpawn = ((bit & mask) == bit);
                        bit <<= 1;
                        DoNotTurn = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FlagsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FlagsTagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FlagsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FlagsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 9. TagBlock :: Offset: 0x3C (60)
            /// </summary>
            public _CharStatsTagBlock CharStats { get { return __charStats__; } set { __charStats__ = value; } }
            private _CharStatsTagBlock __charStats__;
            public class _CharStatsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float HealthOnEasy { get { return __healthOnEasy__; } set { __healthOnEasy__ = value; } }
                private float __healthOnEasy__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float ShieldsOnEasy { get { return __shieldsOnEasy__; } set { __shieldsOnEasy__ = value; } }
                private float __shieldsOnEasy__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float HealthOnLegendary { get { return __healthOnLegendary__; } set { __healthOnLegendary__ = value; } }
                private float __healthOnLegendary__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float ShieldsOnLegendary { get { return __shieldsOnLegendary__; } set { __shieldsOnLegendary__ = value; } }
                private float __shieldsOnLegendary__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 15. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 16. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 17. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 18. Float :: Offset: 0x44 (68)
                /// </summary>
                public float ShieldRechargeRateOnEasy { get { return __shieldRechargeRateOnEasy__; } set { __shieldRechargeRateOnEasy__ = value; } }
                private float __shieldRechargeRateOnEasy__;
                /// <summary>
                /// 19. Float :: Offset: 0x48 (72)
                /// </summary>
                public float ShieldRechargeRateOnLegendary { get { return __shieldRechargeRateOnLegendary__; } set { __shieldRechargeRateOnLegendary__ = value; } }
                private float __shieldRechargeRateOnLegendary__;
                /// <summary>
                /// 20. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 21. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 22. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 23. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 24. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 25. Float :: Offset: 0x60 (96)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 26. ByteArray :: Offset: 0x64 (100)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 27. Float :: Offset: 0x68 (104)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;
                /// <summary>
                /// 28. Float :: Offset: 0x6C (108)
                /// </summary>
                public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private float __unknown20__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _CharStatsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _CharStatsTagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_CharStatsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _CharStatsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 10. TagBlock :: Offset: 0x44 (68)
            /// </summary>
            public _Unknown6TagBlock Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private _Unknown6TagBlock __unknown6__;
            public class _Unknown6TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 7. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 8. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 9. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 10. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 11. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 12. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown6TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown6TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown6TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown6TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 11. TagBlock :: Offset: 0x4C (76)
            /// </summary>
            public _Unknown7TagBlock Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private _Unknown7TagBlock __unknown7__;
            public class _Unknown7TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown7TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown7TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown7TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown7TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 12. TagBlock :: Offset: 0x54 (84)
            /// </summary>
            public _Unknown8TagBlock Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private _Unknown8TagBlock __unknown8__;
            public class _Unknown8TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 15. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 16. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 17. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 18. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 19. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;
                /// <summary>
                /// 20. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private float __unknown20__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown8TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown8TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown8TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown8TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 13. TagBlock :: Offset: 0x5C (92)
            /// </summary>
            public _Unknown9TagBlock Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private _Unknown9TagBlock __unknown9__;
            public class _Unknown9TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 9. ByteArray :: Offset: 0x20 (32)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown9TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown9TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown9TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown9TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 14. TagBlock :: Offset: 0x64 (100)
            /// </summary>
            public _Unknown10TagBlock Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private _Unknown10TagBlock __unknown10__;
            public class _Unknown10TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown10TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown10TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown10TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown10TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 15. TagBlock :: Offset: 0x6C (108)
            /// </summary>
            public _Unknown11TagBlock Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private _Unknown11TagBlock __unknown11__;
            public class _Unknown11TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown11TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown11TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown11TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown11TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 16. TagBlock :: Offset: 0x74 (116)
            /// </summary>
            public _Unknown12TagBlock Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private _Unknown12TagBlock __unknown12__;
            public class _Unknown12TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown12TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown12TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown12TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown12TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 17. TagBlock :: Offset: 0x7C (124)
            /// </summary>
            public _Unknown13TagBlock Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private _Unknown13TagBlock __unknown13__;
            public class _Unknown13TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 15. Tag :: Offset: 0x38 (56)
                /// </summary>
                public TagRef Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private TagRef __unknown15__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown13TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown13TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown13TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown13TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 18. TagBlock :: Offset: 0x84 (132)
            /// </summary>
            public _Unknown14TagBlock Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
            private _Unknown14TagBlock __unknown14__;
            public class _Unknown14TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown14TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown14TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown14TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown14TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 19. TagBlock :: Offset: 0x8C (140)
            /// </summary>
            public _Unknown15TagBlock Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
            private _Unknown15TagBlock __unknown15__;
            public class _Unknown15TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. ByteArray :: Offset: 0x14 (20)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 15. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 16. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown15TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown15TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown15TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown15TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 20. TagBlock :: Offset: 0x94 (148)
            /// </summary>
            public _Unknown16TagBlock Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
            private _Unknown16TagBlock __unknown16__;
            public class _Unknown16TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 11. ByteArray :: Offset: 0x28 (40)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 15. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 16. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 17. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 18. Tag :: Offset: 0x44 (68)
                /// </summary>
                public TagRef Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private TagRef __unknown17__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown16TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown16TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown16TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown16TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 21. TagBlock :: Offset: 0x9C (156)
            /// </summary>
            public _Unknown17TagBlock Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
            private _Unknown17TagBlock __unknown17__;
            public class _Unknown17TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown17TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown17TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown17TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown17TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 22. TagBlock :: Offset: 0xA4 (164)
            /// </summary>
            public _Unknown18TagBlock Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
            private _Unknown18TagBlock __unknown18__;
            public class _Unknown18TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown18TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown18TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown18TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown18TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 23. TagBlock :: Offset: 0xAC (172)
            /// </summary>
            public _Unknown19TagBlock Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
            private _Unknown19TagBlock __unknown19__;
            public class _Unknown19TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown19TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown19TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown19TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown19TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 24. TagBlock :: Offset: 0xB4 (180)
            /// </summary>
            public _Unknown20TagBlock Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
            private _Unknown20TagBlock __unknown20__;
            public class _Unknown20TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown20TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown20TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown20TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown20TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 25. TagBlock :: Offset: 0xBC (188)
            /// </summary>
            public _Unknown21TagBlock Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
            private _Unknown21TagBlock __unknown21__;
            public class _Unknown21TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown21TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown21TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown21TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown21TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 26. TagBlock :: Offset: 0xC4 (196)
            /// </summary>
            public _Unknown22TagBlock Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
            private _Unknown22TagBlock __unknown22__;
            public class _Unknown22TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown22TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown22TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown22TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown22TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 27. TagBlock :: Offset: 0xCC (204)
            /// </summary>
            public _Unknown23TagBlock Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
            private _Unknown23TagBlock __unknown23__;
            public class _Unknown23TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Tag :: Offset: 0x4 (4)
                /// </summary>
                public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private TagRef __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 8. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 9. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 10. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 11. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 12. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 13. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 14. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 15. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 16. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 17. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 18. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 19. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;
                /// <summary>
                /// 20. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private float __unknown20__;
                /// <summary>
                /// 21. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                private float __unknown21__;
                /// <summary>
                /// 22. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                private float __unknown22__;
                /// <summary>
                /// 23. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                private float __unknown23__;
                /// <summary>
                /// 24. ByteArray :: Offset: 0x60 (96)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 25. Float :: Offset: 0x78 (120)
                /// </summary>
                public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                private float __unknown24__;
                /// <summary>
                /// 26. Float :: Offset: 0x7C (124)
                /// </summary>
                public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                private float __unknown25__;
                /// <summary>
                /// 27. Float :: Offset: 0x80 (128)
                /// </summary>
                public float Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                private float __unknown26__;
                /// <summary>
                /// 28. ByteArray :: Offset: 0x84 (132)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 29. Float :: Offset: 0x8C (140)
                /// </summary>
                public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                private float __unknown27__;
                /// <summary>
                /// 30. Float :: Offset: 0x90 (144)
                /// </summary>
                public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                private float __unknown28__;
                /// <summary>
                /// 31. Float :: Offset: 0x94 (148)
                /// </summary>
                public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                private float __unknown29__;
                /// <summary>
                /// 32. Float :: Offset: 0x98 (152)
                /// </summary>
                public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                private float __unknown30__;
                /// <summary>
                /// 33. Float :: Offset: 0x9C (156)
                /// </summary>
                public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
                private float __unknown31__;
                /// <summary>
                /// 34. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
                private float __unknown32__;
                /// <summary>
                /// 35. Float :: Offset: 0xA4 (164)
                /// </summary>
                public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
                private float __unknown33__;
                /// <summary>
                /// 36. Float :: Offset: 0xA8 (168)
                /// </summary>
                public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
                private float __unknown34__;
                /// <summary>
                /// 37. Float :: Offset: 0xAC (172)
                /// </summary>
                public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
                private float __unknown35__;
                /// <summary>
                /// 38. Float :: Offset: 0xB0 (176)
                /// </summary>
                public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
                private float __unknown36__;
                /// <summary>
                /// 39. Float :: Offset: 0xB4 (180)
                /// </summary>
                public float Unknown37 { get { return __unknown37__; } set { __unknown37__ = value; } }
                private float __unknown37__;
                /// <summary>
                /// 40. Float :: Offset: 0xB8 (184)
                /// </summary>
                public float Unknown38 { get { return __unknown38__; } set { __unknown38__ = value; } }
                private float __unknown38__;
                /// <summary>
                /// 41. ByteArray :: Offset: 0xBC (188)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;
                /// <summary>
                /// 42. Tag :: Offset: 0xC4 (196)
                /// </summary>
                public TagRef Unknown39 { get { return __unknown39__; } set { __unknown39__ = value; } }
                private TagRef __unknown39__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown23TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown23TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown23TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown23TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 28. TagBlock :: Offset: 0xD4 (212)
            /// </summary>
            public _WeaponAssociationsTagBlock WeaponAssociations { get { return __weaponAssociations__; } set { __weaponAssociations__ = value; } }
            private _WeaponAssociationsTagBlock __weaponAssociations__;
            public class _WeaponAssociationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Weapon { get { return __weapon__; } set { __weapon__ = value; } }
                private TagRef __weapon__;
                /// <summary>
                /// 2. TagBlock :: Offset: 0x8 (8)
                /// </summary>
                public _UnknownTagBlock Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private _UnknownTagBlock __unknown__;
                public class _UnknownTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private CharLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x1C (28)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;
                    /// <summary>
                    /// 9. Float :: Offset: 0x20 (32)
                    /// </summary>
                    public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                    private float __unknown9__;
                    /// <summary>
                    /// 10. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                    private float __unknown10__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                    private float __unknown11__;
                    /// <summary>
                    /// 12. Float :: Offset: 0x2C (44)
                    /// </summary>
                    public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                    private float __unknown12__;
                    /// <summary>
                    /// 13. Float :: Offset: 0x30 (48)
                    /// </summary>
                    public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                    private float __unknown13__;
                    /// <summary>
                    /// 14. Float :: Offset: 0x34 (52)
                    /// </summary>
                    public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                    private float __unknown14__;
                    /// <summary>
                    /// 15. Float :: Offset: 0x38 (56)
                    /// </summary>
                    public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                    private float __unknown15__;
                    /// <summary>
                    /// 16. Float :: Offset: 0x3C (60)
                    /// </summary>
                    public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                    private float __unknown16__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _UnknownTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _UnknownTagBlock(CharLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_UnknownTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _UnknownTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WeaponAssociationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WeaponAssociationsTagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Unknown = new _UnknownTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WeaponAssociationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WeaponAssociationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 29. TagBlock :: Offset: 0xDC (220)
            /// </summary>
            public _Unknown24TagBlock Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
            private _Unknown24TagBlock __unknown24__;
            public class _Unknown24TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 15. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown24TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown24TagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown24TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown24TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 30. TagBlock :: Offset: 0xE4 (228)
            /// </summary>
            public _VehicleAssociationsTagBlock VehicleAssociations { get { return __vehicleAssociations__; } set { __vehicleAssociations__ = value; } }
            private _VehicleAssociationsTagBlock __vehicleAssociations__;
            public class _VehicleAssociationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public CharLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private CharLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Vehicle { get { return __vehicle__; } set { __vehicle__ = value; } }
                private TagRef __vehicle__;
                /// <summary>
                /// 2. Tag :: Offset: 0x8 (8)
                /// </summary>
                public TagRef Style { get { return __style__; } set { __style__ = value; } }
                private TagRef __style__;
                /// <summary>
                /// 3. Bitmask32 :: Offset: 0x10 (16)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 4. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 5. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 6. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 7. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 8. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 9. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 10. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 11. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 12. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 13. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 14. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 15. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 16. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 17. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 18. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 19. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 20. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 21. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 22. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;
                /// <summary>
                /// 23. Float :: Offset: 0x60 (96)
                /// </summary>
                public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private float __unknown20__;
                /// <summary>
                /// 24. Float :: Offset: 0x64 (100)
                /// </summary>
                public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                private float __unknown21__;
                /// <summary>
                /// 25. Float :: Offset: 0x68 (104)
                /// </summary>
                public float Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                private float __unknown22__;
                /// <summary>
                /// 26. Float :: Offset: 0x6C (108)
                /// </summary>
                public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                private float __unknown23__;
                /// <summary>
                /// 27. Float :: Offset: 0x70 (112)
                /// </summary>
                public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                private float __unknown24__;
                /// <summary>
                /// 28. Float :: Offset: 0x74 (116)
                /// </summary>
                public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                private float __unknown25__;
                /// <summary>
                /// 29. Float :: Offset: 0x78 (120)
                /// </summary>
                public float Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                private float __unknown26__;
                /// <summary>
                /// 30. Float :: Offset: 0x7C (124)
                /// </summary>
                public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                private float __unknown27__;
                /// <summary>
                /// 31. Float :: Offset: 0x80 (128)
                /// </summary>
                public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                private float __unknown28__;
                /// <summary>
                /// 32. Float :: Offset: 0x84 (132)
                /// </summary>
                public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                private float __unknown29__;
                /// <summary>
                /// 33. Float :: Offset: 0x88 (136)
                /// </summary>
                public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                private float __unknown30__;
                /// <summary>
                /// 34. Float :: Offset: 0x8C (140)
                /// </summary>
                public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
                private float __unknown31__;
                /// <summary>
                /// 35. Float :: Offset: 0x90 (144)
                /// </summary>
                public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
                private float __unknown32__;
                /// <summary>
                /// 36. Float :: Offset: 0x94 (148)
                /// </summary>
                public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
                private float __unknown33__;
                /// <summary>
                /// 37. Float :: Offset: 0x98 (152)
                /// </summary>
                public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
                private float __unknown34__;
                /// <summary>
                /// 38. Float :: Offset: 0x9C (156)
                /// </summary>
                public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
                private float __unknown35__;
                /// <summary>
                /// 39. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
                private float __unknown36__;
                /// <summary>
                /// 40. Float :: Offset: 0xA4 (164)
                /// </summary>
                public float Unknown37 { get { return __unknown37__; } set { __unknown37__ = value; } }
                private float __unknown37__;
                /// <summary>
                /// 41. Float :: Offset: 0xA8 (168)
                /// </summary>
                public float Unknown38 { get { return __unknown38__; } set { __unknown38__ = value; } }
                private float __unknown38__;
                /// <summary>
                /// 42. Float :: Offset: 0xAC (172)
                /// </summary>
                public float Unknown39 { get { return __unknown39__; } set { __unknown39__ = value; } }
                private float __unknown39__;
                /// <summary>
                /// 43. Float :: Offset: 0xB0 (176)
                /// </summary>
                public float Unknown40 { get { return __unknown40__; } set { __unknown40__ = value; } }
                private float __unknown40__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _VehicleAssociationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _VehicleAssociationsTagBlock(CharLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_VehicleAssociationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _VehicleAssociationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the CharLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public CharLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Variation = new _VariationTagBlock(this);
                Flags = new _FlagsTagBlock(this);
                CharStats = new _CharStatsTagBlock(this);
                Unknown6 = new _Unknown6TagBlock(this);
                Unknown7 = new _Unknown7TagBlock(this);
                Unknown8 = new _Unknown8TagBlock(this);
                Unknown9 = new _Unknown9TagBlock(this);
                Unknown10 = new _Unknown10TagBlock(this);
                Unknown11 = new _Unknown11TagBlock(this);
                Unknown12 = new _Unknown12TagBlock(this);
                Unknown13 = new _Unknown13TagBlock(this);
                Unknown14 = new _Unknown14TagBlock(this);
                Unknown15 = new _Unknown15TagBlock(this);
                Unknown16 = new _Unknown16TagBlock(this);
                Unknown17 = new _Unknown17TagBlock(this);
                Unknown18 = new _Unknown18TagBlock(this);
                Unknown19 = new _Unknown19TagBlock(this);
                Unknown20 = new _Unknown20TagBlock(this);
                Unknown21 = new _Unknown21TagBlock(this);
                Unknown22 = new _Unknown22TagBlock(this);
                Unknown23 = new _Unknown23TagBlock(this);
                WeaponAssociations = new _WeaponAssociationsTagBlock(this);
                Unknown24 = new _Unknown24TagBlock(this);
                VehicleAssociations = new _VehicleAssociationsTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unused = br.ReadBytes(4);
                Unknown1._Initialize(br);
                Unknown2._Initialize(br);
                Unknown3._Initialize(br);
                Unknown4._Initialize(br);
                Unknown5._Initialize(br);
                Variation._Initialize(br);
                Flags._Initialize(br);
                CharStats._Initialize(br);
                Unknown6._Initialize(br);
                Unknown7._Initialize(br);
                Unknown8._Initialize(br);
                Unknown9._Initialize(br);
                Unknown10._Initialize(br);
                Unknown11._Initialize(br);
                Unknown12._Initialize(br);
                Unknown13._Initialize(br);
                Unknown14._Initialize(br);
                Unknown15._Initialize(br);
                Unknown16._Initialize(br);
                Unknown17._Initialize(br);
                Unknown18._Initialize(br);
                Unknown19._Initialize(br);
                Unknown20._Initialize(br);
                Unknown21._Initialize(br);
                Unknown22._Initialize(br);
                Unknown23._Initialize(br);
                WeaponAssociations._Initialize(br);
                Unknown24._Initialize(br);
                VehicleAssociations._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Char class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Char(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new CharLayout(map, identity);
        }

        #region Specific Methods of char

        #endregion
    }
}
