using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Effect
    /// </summary>
    public class Effe
    {
        #region Constants

        /// <summary>
        /// Size of the effe meta header.
        /// </summary>
        public const int HeaderSize = 48;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private EffeLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.4 DiscoBob: Named alot of values.
        /// v0.2 DiscoBob: Named a few things and fixed some values.
        /// v0.1b Snaku: Identified a few things and guessed on a few more...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public EffeLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class EffeLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Float :: Offset: 0x0 (0)
            /// </summary>
            public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private float __unknown1__;
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private float __unknown3__;
            /// <summary>
            /// 4. TagBlock :: Offset: 0xC (12)
            /// </summary>
            public _LocationsTagBlock Locations { get { return __locations__; } set { __locations__ = value; } }
            private _LocationsTagBlock __locations__;
            public class _LocationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private EffeLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _LocationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _LocationsTagBlock(EffeLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_LocationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _LocationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 5. TagBlock :: Offset: 0x14 (20)
            /// </summary>
            public _EventsTagBlock Events { get { return __events__; } set { __events__ = value; } }
            private _EventsTagBlock __events__;
            public class _EventsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private EffeLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float SkipFraction { get { return __skipFraction__; } set { __skipFraction__ = value; } }
                private float __skipFraction__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float DelayBounds_min { get { return __delayBounds_min__; } set { __delayBounds_min__ = value; } }
                private float __delayBounds_min__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float DelayBounds_max { get { return __delayBounds_max__; } set { __delayBounds_max__ = value; } }
                private float __delayBounds_max__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float DurationBounds_min { get { return __durationBounds_min__; } set { __durationBounds_min__ = value; } }
                private float __durationBounds_min__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float DurationBounds_max { get { return __durationBounds_max__; } set { __durationBounds_max__ = value; } }
                private float __durationBounds_max__;
                /// <summary>
                /// 7. TagBlock :: Offset: 0x18 (24)
                /// </summary>
                public _PartsTagBlock Parts { get { return __parts__; } set { __parts__ = value; } }
                private _PartsTagBlock __parts__;
                public class _PartsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private EffeLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private short __unknown1__;
                    /// <summary>
                    /// 2. Int16 :: Offset: 0x2 (2)
                    /// </summary>
                    public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private short __unknown2__;
                    /// <summary>
                    /// 3. Int16 :: Offset: 0x4 (4)
                    /// </summary>
                    public short Location { get { return __location__; } set { __location__ = value; } }
                    private short __location__;
                    /// <summary>
                    /// 4. Int16 :: Offset: 0x6 (6)
                    /// </summary>
                    public short Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private short __unknown3__;
                    /// <summary>
                    /// 5. Tag :: Offset: 0x8 (8)
                    /// </summary>
                    public TagRef Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                    private TagRef __unused1__;
                    /// <summary>
                    /// 6. Tag :: Offset: 0xC (12)
                    /// </summary>
                    public TagRef Type1 { get { return __type1__; } set { __type1__ = value; } }
                    private TagRef __type1__;
                    /// <summary>
                    /// 7. ID :: Offset: 0x10 (16)
                    /// </summary>
                    public LoneID Type2 { get { return __type2__; } set { __type2__ = value; } }
                    private LoneID __type2__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float VelocityBound_min { get { return __velocityBound_min__; } set { __velocityBound_min__ = value; } }
                    private float __velocityBound_min__;
                    /// <summary>
                    /// 9. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float VelocityBound_max { get { return __velocityBound_max__; } set { __velocityBound_max__ = value; } }
                    private float __velocityBound_max__;
                    /// <summary>
                    /// 10. Float :: Offset: 0x1C (28)
                    /// </summary>
                    public float VelocityConeAngle_radians { get { return __velocityConeAngle_radians__; } set { __velocityConeAngle_radians__ = value; } }
                    private float __velocityConeAngle_radians__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x20 (32)
                    /// </summary>
                    public float AngularVelocityBound_min { get { return __angularVelocityBound_min__; } set { __angularVelocityBound_min__ = value; } }
                    private float __angularVelocityBound_min__;
                    /// <summary>
                    /// 12. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float AngularVelocityBound_max { get { return __angularVelocityBound_max__; } set { __angularVelocityBound_max__ = value; } }
                    private float __angularVelocityBound_max__;
                    /// <summary>
                    /// 13. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float RadiusModifier_min { get { return __radiusModifier_min__; } set { __radiusModifier_min__ = value; } }
                    private float __radiusModifier_min__;
                    /// <summary>
                    /// 14. Float :: Offset: 0x2C (44)
                    /// </summary>
                    public float RadiusModifier_max { get { return __radiusModifier_max__; } set { __radiusModifier_max__ = value; } }
                    private float __radiusModifier_max__;
                    /// <summary>
                    /// 15. Bitmask32 :: Offset: 0x30 (48)
                    /// </summary>
                    public Flags_scaleModifiersPossiblyBitmask32 Flags_scaleModifiersPossibly { get { return __flags_scaleModifiersPossibly__; } set { __flags_scaleModifiersPossibly__ = value; } }
                    private Flags_scaleModifiersPossiblyBitmask32 __flags_scaleModifiersPossibly__;
                    public class Flags_scaleModifiersPossiblyBitmask32
                    {

                        public void _Initialize(BitReader br)
                        {
                            // Empty..
                            br.BaseStream.Position += 4;
                        }
                    }
                    /// <summary>
                    /// 16. ByteArray :: Offset: 0x34 (52)
                    /// </summary>
                    public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                    private byte[] __unused2__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PartsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PartsTagBlock(EffeLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Flags_scaleModifiersPossibly = new Flags_scaleModifiersPossiblyBitmask32();
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PartsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PartsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 8. TagBlock :: Offset: 0x20 (32)
                /// </summary>
                public _Unknown1TagBlock Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private _Unknown1TagBlock __unknown1__;
                public class _Unknown1TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private EffeLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private TagRef __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. TagBlock :: Offset: 0xC (12)
                    /// </summary>
                    public _Unknown3TagBlock Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private _Unknown3TagBlock __unknown3__;
                    public class _Unknown3TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Byte :: Offset: 0x0 (0)
                        /// </summary>
                        public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private byte __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown3TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown3TagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown3TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown3TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 4. TagBlock :: Offset: 0x14 (20)
                    /// </summary>
                    public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private _Unknown4TagBlock __unknown4__;
                    public class _Unknown4TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Byte :: Offset: 0x0 (0)
                        /// </summary>
                        public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private byte __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown4TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown4TagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown4TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown4TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 5. TagBlock :: Offset: 0x1C (28)
                    /// </summary>
                    public _Unknown5TagBlock Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private _Unknown5TagBlock __unknown5__;
                    public class _Unknown5TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Byte :: Offset: 0x0 (0)
                        /// </summary>
                        public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private byte __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown5TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown5TagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown5TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown5TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 6. TagBlock :: Offset: 0x24 (36)
                    /// </summary>
                    public _Unknown6TagBlock Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private _Unknown6TagBlock __unknown6__;
                    public class _Unknown6TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Byte :: Offset: 0x0 (0)
                        /// </summary>
                        public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private byte __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown6TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown6TagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown6TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown6TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 7. TagBlock :: Offset: 0x2C (44)
                    /// </summary>
                    public _Unknown7TagBlock Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private _Unknown7TagBlock __unknown7__;
                    public class _Unknown7TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Byte :: Offset: 0x0 (0)
                        /// </summary>
                        public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private byte __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown7TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown7TagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown7TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown7TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 8. TagBlock :: Offset: 0x34 (52)
                    /// </summary>
                    public _Unknown8TagBlock Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private _Unknown8TagBlock __unknown8__;
                    public class _Unknown8TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Byte :: Offset: 0x0 (0)
                        /// </summary>
                        public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private byte __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown8TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown8TagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown8TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown8TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown1TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown1TagBlock(EffeLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Unknown3 = new _Unknown3TagBlock(layout);
                        Unknown4 = new _Unknown4TagBlock(layout);
                        Unknown5 = new _Unknown5TagBlock(layout);
                        Unknown6 = new _Unknown6TagBlock(layout);
                        Unknown7 = new _Unknown7TagBlock(layout);
                        Unknown8 = new _Unknown8TagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown1TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown1TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 9. TagBlock :: Offset: 0x28 (40)
                /// </summary>
                public _Unknown2TagBlock Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private _Unknown2TagBlock __unknown2__;
                public class _Unknown2TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private EffeLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. ByteArray :: Offset: 0x0 (0)
                    /// </summary>
                    public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                    private byte[] __unused__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown2TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown2TagBlock(EffeLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown2TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown2TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 10. TagBlock :: Offset: 0x30 (48)
                /// </summary>
                public _Particles_PrtTagBlock Particles_Prt { get { return __particles_Prt__; } set { __particles_Prt__ = value; } }
                private _Particles_PrtTagBlock __particles_Prt__;
                public class _Particles_PrtTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private EffeLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private TagRef __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x1C (28)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x20 (32)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;
                    /// <summary>
                    /// 9. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                    private float __unknown9__;
                    /// <summary>
                    /// 10. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                    private float __unknown10__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x2C (44)
                    /// </summary>
                    public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                    private float __unknown11__;
                    /// <summary>
                    /// 12. TagBlock :: Offset: 0x30 (48)
                    /// </summary>
                    public _ParticlePhysics_pmovTagBlock ParticlePhysics_pmov { get { return __particlePhysics_pmov__; } set { __particlePhysics_pmov__ = value; } }
                    private _ParticlePhysics_pmovTagBlock __particlePhysics_pmov__;
                    public class _ParticlePhysics_pmovTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private EffeLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Tag :: Offset: 0x0 (0)
                        /// </summary>
                        public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                        private TagRef __unknown1__;
                        /// <summary>
                        /// 2. Float :: Offset: 0x8 (8)
                        /// </summary>
                        public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                        private float __unknown2__;
                        /// <summary>
                        /// 3. Float :: Offset: 0xC (12)
                        /// </summary>
                        public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                        private float __unknown3__;
                        /// <summary>
                        /// 4. TagBlock :: Offset: 0x10 (16)
                        /// </summary>
                        public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                        private _Unknown4TagBlock __unknown4__;
                        public class _Unknown4TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown4TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown4TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown4TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown4TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 5. Float :: Offset: 0x18 (24)
                        /// </summary>
                        public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                        private float __unknown5__;
                        /// <summary>
                        /// 6. Float :: Offset: 0x1C (28)
                        /// </summary>
                        public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                        private float __unknown6__;
                        /// <summary>
                        /// 7. TagBlock :: Offset: 0x20 (32)
                        /// </summary>
                        public _Unknown7TagBlock Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                        private _Unknown7TagBlock __unknown7__;
                        public class _Unknown7TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown7TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown7TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown7TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown7TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 8. Float :: Offset: 0x28 (40)
                        /// </summary>
                        public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                        private float __unknown8__;
                        /// <summary>
                        /// 9. Float :: Offset: 0x2C (44)
                        /// </summary>
                        public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                        private float __unknown9__;
                        /// <summary>
                        /// 10. TagBlock :: Offset: 0x30 (48)
                        /// </summary>
                        public _Unknown10TagBlock Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                        private _Unknown10TagBlock __unknown10__;
                        public class _Unknown10TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown10TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown10TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown10TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown10TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 11. Float :: Offset: 0x38 (56)
                        /// </summary>
                        public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                        private float __unknown11__;
                        /// <summary>
                        /// 12. Float :: Offset: 0x3C (60)
                        /// </summary>
                        public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                        private float __unknown12__;
                        /// <summary>
                        /// 13. TagBlock :: Offset: 0x40 (64)
                        /// </summary>
                        public _Unknown13TagBlock Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                        private _Unknown13TagBlock __unknown13__;
                        public class _Unknown13TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown13TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown13TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown13TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown13TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 14. Float :: Offset: 0x48 (72)
                        /// </summary>
                        public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                        private float __unknown14__;
                        /// <summary>
                        /// 15. Float :: Offset: 0x4C (76)
                        /// </summary>
                        public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                        private float __unknown15__;
                        /// <summary>
                        /// 16. TagBlock :: Offset: 0x50 (80)
                        /// </summary>
                        public _Unknown16TagBlock Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                        private _Unknown16TagBlock __unknown16__;
                        public class _Unknown16TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown16TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown16TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown16TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown16TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 17. Float :: Offset: 0x58 (88)
                        /// </summary>
                        public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                        private float __unknown17__;
                        /// <summary>
                        /// 18. Float :: Offset: 0x5C (92)
                        /// </summary>
                        public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                        private float __unknown18__;
                        /// <summary>
                        /// 19. TagBlock :: Offset: 0x60 (96)
                        /// </summary>
                        public _Unknown19TagBlock Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                        private _Unknown19TagBlock __unknown19__;
                        public class _Unknown19TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown19TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown19TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown19TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown19TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 20. Float :: Offset: 0x68 (104)
                        /// </summary>
                        public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                        private float __unknown20__;
                        /// <summary>
                        /// 21. Float :: Offset: 0x6C (108)
                        /// </summary>
                        public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                        private float __unknown21__;
                        /// <summary>
                        /// 22. TagBlock :: Offset: 0x70 (112)
                        /// </summary>
                        public _Unknown22TagBlock Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                        private _Unknown22TagBlock __unknown22__;
                        public class _Unknown22TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown22TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown22TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown22TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown22TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 23. Float :: Offset: 0x78 (120)
                        /// </summary>
                        public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                        private float __unknown23__;
                        /// <summary>
                        /// 24. Float :: Offset: 0x7C (124)
                        /// </summary>
                        public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                        private float __unknown24__;
                        /// <summary>
                        /// 25. Float :: Offset: 0x80 (128)
                        /// </summary>
                        public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                        private float __unknown25__;
                        /// <summary>
                        /// 26. TagBlock :: Offset: 0x84 (132)
                        /// </summary>
                        public _Unknown26TagBlock Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                        private _Unknown26TagBlock __unknown26__;
                        public class _Unknown26TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown26TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown26TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown26TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown26TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 27. Float :: Offset: 0x8C (140)
                        /// </summary>
                        public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                        private float __unknown27__;
                        /// <summary>
                        /// 28. Float :: Offset: 0x90 (144)
                        /// </summary>
                        public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                        private float __unknown28__;
                        /// <summary>
                        /// 29. TagBlock :: Offset: 0x94 (148)
                        /// </summary>
                        public _Unknown29TagBlock Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                        private _Unknown29TagBlock __unknown29__;
                        public class _Unknown29TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public EffeLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private EffeLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Byte :: Offset: 0x0 (0)
                            /// </summary>
                            public byte Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private byte __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown29TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown29TagBlock(EffeLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown29TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown29TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 30. Float :: Offset: 0x9C (156)
                        /// </summary>
                        public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                        private float __unknown30__;
                        /// <summary>
                        /// 31. Float :: Offset: 0xA0 (160)
                        /// </summary>
                        public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
                        private float __unknown31__;
                        /// <summary>
                        /// 32. Float :: Offset: 0xA4 (164)
                        /// </summary>
                        public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
                        private float __unknown32__;
                        /// <summary>
                        /// 33. Float :: Offset: 0xA8 (168)
                        /// </summary>
                        public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
                        private float __unknown33__;
                        /// <summary>
                        /// 34. Float :: Offset: 0xAC (172)
                        /// </summary>
                        public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
                        private float __unknown34__;
                        /// <summary>
                        /// 35. Float :: Offset: 0xB0 (176)
                        /// </summary>
                        public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
                        private float __unknown35__;
                        /// <summary>
                        /// 36. Float :: Offset: 0xB4 (180)
                        /// </summary>
                        public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
                        private float __unknown36__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _ParticlePhysics_pmovTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _ParticlePhysics_pmovTagBlock(EffeLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                            Unknown4 = new _Unknown4TagBlock(layout);
                            Unknown7 = new _Unknown7TagBlock(layout);
                            Unknown10 = new _Unknown10TagBlock(layout);
                            Unknown13 = new _Unknown13TagBlock(layout);
                            Unknown16 = new _Unknown16TagBlock(layout);
                            Unknown19 = new _Unknown19TagBlock(layout);
                            Unknown22 = new _Unknown22TagBlock(layout);
                            Unknown26 = new _Unknown26TagBlock(layout);
                            Unknown29 = new _Unknown29TagBlock(layout);
                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_ParticlePhysics_pmovTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _ParticlePhysics_pmovTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Particles_PrtTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Particles_PrtTagBlock(EffeLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        ParticlePhysics_pmov = new _ParticlePhysics_pmovTagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Particles_PrtTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Particles_PrtTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _EventsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _EventsTagBlock(EffeLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Parts = new _PartsTagBlock(layout);
                    Unknown1 = new _Unknown1TagBlock(layout);
                    Unknown2 = new _Unknown2TagBlock(layout);
                    Particles_Prt = new _Particles_PrtTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_EventsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _EventsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 6. Tag :: Offset: 0x1C (28)
            /// </summary>
            public TagRef Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private TagRef __unknown4__;
            /// <summary>
            /// 7. Float :: Offset: 0x24 (36)
            /// </summary>
            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private float __unknown5__;
            /// <summary>
            /// 8. Float :: Offset: 0x28 (40)
            /// </summary>
            public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private float __unknown6__;
            /// <summary>
            /// 9. Float :: Offset: 0x2C (44)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the EffeLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public EffeLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Locations = new _LocationsTagBlock(this);
                Events = new _EventsTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unknown1 = br.ReadSingle();
                Unknown2 = br.ReadSingle();
                Unknown3 = br.ReadSingle();
                Locations._Initialize(br);
                Events._Initialize(br);
                Unknown4._Initialize(br);
                Unknown5 = br.ReadSingle();
                Unknown6 = br.ReadSingle();
                Unknown7 = br.ReadSingle();

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Effe class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Effe(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new EffeLayout(map, identity);
        }

        #region Specific Methods of effe

        #endregion
    }
}
