using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Object Property
    /// </summary>
    public class Hlmt
    {
        #region Constants

        /// <summary>
        /// Size of the hlmt meta header.
        /// </summary>
        public const int HeaderSize = 252;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private HlmtLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.51 Turk645: Small permutation update.
        /// v0.5 XZodia: Updated.
        /// v0.3 XZodia: Name a load of stuff.
        /// v0.2 Iron_Forge: Added some basic stuff...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public HlmtLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class HlmtLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Tag :: Offset: 0x0 (0)
            /// </summary>
            public TagRef Model { get { return __model__; } set { __model__ = value; } }
            private TagRef __model__;
            /// <summary>
            /// 2. Tag :: Offset: 0x8 (8)
            /// </summary>
            public TagRef Collision { get { return __collision__; } set { __collision__ = value; } }
            private TagRef __collision__;
            /// <summary>
            /// 3. Tag :: Offset: 0x10 (16)
            /// </summary>
            public TagRef Animations { get { return __animations__; } set { __animations__ = value; } }
            private TagRef __animations__;
            /// <summary>
            /// 4. Tag :: Offset: 0x18 (24)
            /// </summary>
            public TagRef Physics { get { return __physics__; } set { __physics__ = value; } }
            private TagRef __physics__;
            /// <summary>
            /// 5. Tag :: Offset: 0x20 (32)
            /// </summary>
            public TagRef PhysicsModel { get { return __physicsModel__; } set { __physicsModel__ = value; } }
            private TagRef __physicsModel__;
            /// <summary>
            /// 6. Float :: Offset: 0x28 (40)
            /// </summary>
            public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private float __unknown1__;
            /// <summary>
            /// 7. Float :: Offset: 0x2C (44)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 8. ByteArray :: Offset: 0x30 (48)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 9. Float :: Offset: 0x34 (52)
            /// </summary>
            public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private float __unknown3__;
            /// <summary>
            /// 10. Float :: Offset: 0x38 (56)
            /// </summary>
            public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private float __unknown4__;
            /// <summary>
            /// 11. Float :: Offset: 0x3C (60)
            /// </summary>
            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private float __unknown5__;
            /// <summary>
            /// 12. Float :: Offset: 0x40 (64)
            /// </summary>
            public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private float __unknown6__;
            /// <summary>
            /// 13. Float :: Offset: 0x44 (68)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;
            /// <summary>
            /// 14. ByteArray :: Offset: 0x48 (72)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 15. Float :: Offset: 0x4C (76)
            /// </summary>
            public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private float __unknown8__;
            /// <summary>
            /// 16. TagBlock :: Offset: 0x50 (80)
            /// </summary>
            public _VariationsTagBlock Variations { get { return __variations__; } set { __variations__ = value; } }
            private _VariationsTagBlock __variations__;
            public class _VariationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Name { get { return __name__; } set { __name__ = value; } }
                private StringID __name__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. TagBlock :: Offset: 0x14 (20)
                /// </summary>
                public _PieceTagBlock Piece { get { return __piece__; } set { __piece__ = value; } }
                private _PieceTagBlock __piece__;
                public class _PieceTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Name { get { return __name__; } set { __name__ = value; } }
                    private StringID __name__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private float __unknown__;
                    /// <summary>
                    /// 3. TagBlock :: Offset: 0x8 (8)
                    /// </summary>
                    public _PermutationTagBlock Permutation { get { return __permutation__; } set { __permutation__ = value; } }
                    private _PermutationTagBlock __permutation__;
                    public class _PermutationTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private HlmtLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. StringID :: Offset: 0x0 (0)
                        /// </summary>
                        public StringID Name { get { return __name__; } set { __name__ = value; } }
                        private StringID __name__;
                        /// <summary>
                        /// 2. Int16 :: Offset: 0x4 (4)
                        /// </summary>
                        public short AppearanceChunkNum { get { return __appearanceChunkNum__; } set { __appearanceChunkNum__ = value; } }
                        private short __appearanceChunkNum__;
                        /// <summary>
                        /// 3. Int16 :: Offset: 0x6 (6)
                        /// </summary>
                        public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                        private short __unknown1__;
                        /// <summary>
                        /// 4. Float :: Offset: 0x8 (8)
                        /// </summary>
                        public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                        private float __unknown2__;
                        /// <summary>
                        /// 5. TagBlock :: Offset: 0xC (12)
                        /// </summary>
                        public __PermutationTagBlock Permutation { get { return __permutation__; } set { __permutation__ = value; } }
                        private __PermutationTagBlock __permutation__;
                        public class __PermutationTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private HlmtLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. StringID :: Offset: 0x0 (0)
                            /// </summary>
                            public StringID Name { get { return __name__; } set { __name__ = value; } }
                            private StringID __name__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x4 (4)
                            /// </summary>
                            public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                            private short __unknown1__;
                            /// <summary>
                            /// 3. Int16 :: Offset: 0x6 (6)
                            /// </summary>
                            public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                            private short __unknown2__;
                            /// <summary>
                            /// 4. Tag :: Offset: 0x8 (8)
                            /// </summary>
                            public TagRef Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                            private TagRef __unknown3__;
                            /// <summary>
                            /// 5. StringID :: Offset: 0x10 (16)
                            /// </summary>
                            public StringID Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                            private StringID __unknown4__;
                            /// <summary>
                            /// 6. Float :: Offset: 0x14 (20)
                            /// </summary>
                            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                            private float __unknown5__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the __PermutationTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public __PermutationTagBlock(HlmtLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(__PermutationTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, __PermutationTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 6. ByteArray :: Offset: 0x14 (20)
                        /// </summary>
                        public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                        private byte[] __unused__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _PermutationTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _PermutationTagBlock(HlmtLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                            Permutation = new __PermutationTagBlock(layout);
                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_PermutationTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _PermutationTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 4. ByteArray :: Offset: 0x10 (16)
                    /// </summary>
                    public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                    private byte[] __unused__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PieceTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PieceTagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Permutation = new _PermutationTagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PieceTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PieceTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 7. TagBlock :: Offset: 0x1C (28)
                /// </summary>
                public _AttachmentsTagBlock Attachments { get { return __attachments__; } set { __attachments__ = value; } }
                private _AttachmentsTagBlock __attachments__;
                public class _AttachmentsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                    private StringID __marker__;
                    /// <summary>
                    /// 2. StringID :: Offset: 0x4 (4)
                    /// </summary>
                    public StringID AttachmentPermutationPossibly { get { return __attachmentPermutationPossibly__; } set { __attachmentPermutationPossibly__ = value; } }
                    private StringID __attachmentPermutationPossibly__;
                    /// <summary>
                    /// 3. Tag :: Offset: 0x8 (8)
                    /// </summary>
                    public TagRef Attachment { get { return __attachment__; } set { __attachment__ = value; } }
                    private TagRef __attachment__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _AttachmentsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _AttachmentsTagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_AttachmentsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _AttachmentsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 8. ByteArray :: Offset: 0x24 (36)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 9. StringID :: Offset: 0x2C (44)
                /// </summary>
                public StringID Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private StringID __unknown5__;
                /// <summary>
                /// 10. Tag :: Offset: 0x30 (48)
                /// </summary>
                public TagRef Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private TagRef __unknown6__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _VariationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _VariationsTagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Piece = new _PieceTagBlock(layout);
                    Attachments = new _AttachmentsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_VariationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _VariationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 17. TagBlock :: Offset: 0x58 (88)
            /// </summary>
            public _SurfaceMaterialsTagBlock SurfaceMaterials { get { return __surfaceMaterials__; } set { __surfaceMaterials__ = value; } }
            private _SurfaceMaterialsTagBlock __surfaceMaterials__;
            public class _SurfaceMaterialsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Area { get { return __area__; } set { __area__ = value; } }
                private StringID __area__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID Type { get { return __type__; } set { __type__ = value; } }
                private StringID __type__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _SurfaceMaterialsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _SurfaceMaterialsTagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_SurfaceMaterialsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _SurfaceMaterialsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 18. TagBlock :: Offset: 0x60 (96)
            /// </summary>
            public _Health_ShieldsTagBlock Health_Shields { get { return __health_Shields__; } set { __health_Shields__ = value; } }
            private _Health_ShieldsTagBlock __health_Shields__;
            public class _Health_ShieldsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private StringID __unknown2__;
                /// <summary>
                /// 3. Int32 :: Offset: 0x8 (8)
                /// </summary>
                public int Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private int __unknown3__;
                /// <summary>
                /// 4. ByteArray :: Offset: 0xC (12)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 5. Byte :: Offset: 0x10 (16)
                /// </summary>
                public byte Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private byte __unknown4__;
                /// <summary>
                /// 6. Byte :: Offset: 0x11 (17)
                /// </summary>
                public byte Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private byte __unknown5__;
                /// <summary>
                /// 7. Int16 :: Offset: 0x12 (18)
                /// </summary>
                public short Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private short __unknown6__;
                /// <summary>
                /// 8. ByteArray :: Offset: 0x14 (20)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 9. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Health_noShields { get { return __health_noShields__; } set { __health_noShields__ = value; } }
                private float __health_noShields__;
                /// <summary>
                /// 10. ByteArray :: Offset: 0x2C (44)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;
                /// <summary>
                /// 11. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 12. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 13. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 14. ByteArray :: Offset: 0x3C (60)
                /// </summary>
                public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                private byte[] __unused4__;
                /// <summary>
                /// 15. Tag :: Offset: 0x7C (124)
                /// </summary>
                public TagRef ShieldFP { get { return __shieldFP__; } set { __shieldFP__ = value; } }
                private TagRef __shieldFP__;
                /// <summary>
                /// 16. Tag :: Offset: 0x84 (132)
                /// </summary>
                public TagRef Shield1 { get { return __shield1__; } set { __shield1__ = value; } }
                private TagRef __shield1__;
                /// <summary>
                /// 17. Float :: Offset: 0x8C (140)
                /// </summary>
                public float Shield2 { get { return __shield2__; } set { __shield2__ = value; } }
                private float __shield2__;
                /// <summary>
                /// 18. StringID :: Offset: 0x90 (144)
                /// </summary>
                public StringID ShieldNoise { get { return __shieldNoise__; } set { __shieldNoise__ = value; } }
                private StringID __shieldNoise__;
                /// <summary>
                /// 19. Float :: Offset: 0x94 (148)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 20. Float :: Offset: 0x98 (152)
                /// </summary>
                public float ShieldRechargeTime { get { return __shieldRechargeTime__; } set { __shieldRechargeTime__ = value; } }
                private float __shieldRechargeTime__;
                /// <summary>
                /// 21. Float :: Offset: 0x9C (156)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 22. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 23. Tag :: Offset: 0xA4 (164)
                /// </summary>
                public TagRef Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private TagRef __unknown13__;
                /// <summary>
                /// 24. Tag :: Offset: 0xAC (172)
                /// </summary>
                public TagRef Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private TagRef __unknown14__;
                /// <summary>
                /// 25. Tag :: Offset: 0xB4 (180)
                /// </summary>
                public TagRef Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private TagRef __unknown15__;
                /// <summary>
                /// 26. TagBlock :: Offset: 0xBC (188)
                /// </summary>
                public _DamageLocationsTagBlock DamageLocations { get { return __damageLocations__; } set { __damageLocations__ = value; } }
                private _DamageLocationsTagBlock __damageLocations__;
                public class _DamageLocationsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Name { get { return __name__; } set { __name__ = value; } }
                    private StringID __name__;
                    /// <summary>
                    /// 2. Int32 :: Offset: 0x4 (4)
                    /// </summary>
                    public int Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private int __unknown1__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float DamageToActivate { get { return __damageToActivate__; } set { __damageToActivate__ = value; } }
                    private float __damageToActivate__;
                    /// <summary>
                    /// 4. TagBlock :: Offset: 0xC (12)
                    /// </summary>
                    public _PieceTagBlock Piece { get { return __piece__; } set { __piece__ = value; } }
                    private _PieceTagBlock __piece__;
                    public class _PieceTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private HlmtLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Int16 :: Offset: 0x0 (0)
                        /// </summary>
                        public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                        private short __unknown1__;
                        /// <summary>
                        /// 2. Int16 :: Offset: 0x2 (2)
                        /// </summary>
                        public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                        private short __unknown2__;
                        /// <summary>
                        /// 3. Float :: Offset: 0x4 (4)
                        /// </summary>
                        public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                        private float __unknown3__;
                        /// <summary>
                        /// 4. Float :: Offset: 0x8 (8)
                        /// </summary>
                        public float DamageToActivate { get { return __damageToActivate__; } set { __damageToActivate__ = value; } }
                        private float __damageToActivate__;
                        /// <summary>
                        /// 5. Tag :: Offset: 0xC (12)
                        /// </summary>
                        public TagRef Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                        private TagRef __unknown4__;
                        /// <summary>
                        /// 6. Tag :: Offset: 0x14 (20)
                        /// </summary>
                        public TagRef Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                        private TagRef __unknown5__;
                        /// <summary>
                        /// 7. StringID :: Offset: 0x1C (28)
                        /// </summary>
                        public StringID Location { get { return __location__; } set { __location__ = value; } }
                        private StringID __location__;
                        /// <summary>
                        /// 8. Int16 :: Offset: 0x20 (32)
                        /// </summary>
                        public short Phase { get { return __phase__; } set { __phase__ = value; } }
                        private short __phase__;
                        /// <summary>
                        /// 9. Int16 :: Offset: 0x22 (34)
                        /// </summary>
                        public short PieceNum { get { return __pieceNum__; } set { __pieceNum__ = value; } }
                        private short __pieceNum__;
                        /// <summary>
                        /// 10. StringID :: Offset: 0x24 (36)
                        /// </summary>
                        public StringID Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                        private StringID __unknown6__;
                        /// <summary>
                        /// 11. StringID :: Offset: 0x28 (40)
                        /// </summary>
                        public StringID Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                        private StringID __unknown7__;
                        /// <summary>
                        /// 12. Float :: Offset: 0x2C (44)
                        /// </summary>
                        public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                        private float __unknown8__;
                        /// <summary>
                        /// 13. Tag :: Offset: 0x30 (48)
                        /// </summary>
                        public TagRef Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                        private TagRef __unknown9__;
                        /// <summary>
                        /// 14. StringID :: Offset: 0x38 (56)
                        /// </summary>
                        public StringID Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                        private StringID __unknown10__;
                        /// <summary>
                        /// 15. StringID :: Offset: 0x3C (60)
                        /// </summary>
                        public StringID Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                        private StringID __unknown11__;
                        /// <summary>
                        /// 16. StringID :: Offset: 0x40 (64)
                        /// </summary>
                        public StringID Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                        private StringID __unknown12__;
                        /// <summary>
                        /// 17. Float :: Offset: 0x44 (68)
                        /// </summary>
                        public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                        private float __unknown13__;
                        /// <summary>
                        /// 18. StringID :: Offset: 0x48 (72)
                        /// </summary>
                        public StringID Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                        private StringID __unknown14__;
                        /// <summary>
                        /// 19. Float :: Offset: 0x4C (76)
                        /// </summary>
                        public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                        private float __unknown15__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _PieceTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _PieceTagBlock(HlmtLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_PieceTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _PieceTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 5. ByteArray :: Offset: 0x14 (20)
                    /// </summary>
                    public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                    private byte[] __unused__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x2C (44)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 9. StringID :: Offset: 0x30 (48)
                    /// </summary>
                    public StringID Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private StringID __unknown5__;
                    /// <summary>
                    /// 10. Int16 :: Offset: 0x34 (52)
                    /// </summary>
                    public short Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private short __unknown6__;
                    /// <summary>
                    /// 11. Int16 :: Offset: 0x36 (54)
                    /// </summary>
                    public short Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private short __unknown7__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _DamageLocationsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _DamageLocationsTagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Piece = new _PieceTagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_DamageLocationsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _DamageLocationsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 27. TagBlock :: Offset: 0xC4 (196)
                /// </summary>
                public _Unknown16TagBlock Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private _Unknown16TagBlock __unknown16__;
                public class _Unknown16TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private float __unknown__;
                    /// <summary>
                    /// 2. ByteArray :: Offset: 0x4 (4)
                    /// </summary>
                    public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                    private byte[] __unused__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown16TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown16TagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown16TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown16TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 28. Int16 :: Offset: 0xCC (204)
                /// </summary>
                public short Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private short __unknown17__;
                /// <summary>
                /// 29. Int16 :: Offset: 0xCE (206)
                /// </summary>
                public short Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private short __unknown18__;
                /// <summary>
                /// 30. Float :: Offset: 0xD0 (208)
                /// </summary>
                public float ShieldRechargeRate { get { return __shieldRechargeRate__; } set { __shieldRechargeRate__ = value; } }
                private float __shieldRechargeRate__;
                /// <summary>
                /// 31. Float :: Offset: 0xD4 (212)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;
                /// <summary>
                /// 32. TagBlock :: Offset: 0xD8 (216)
                /// </summary>
                public _SeatProtectionTagBlock SeatProtection { get { return __seatProtection__; } set { __seatProtection__ = value; } }
                private _SeatProtectionTagBlock __seatProtection__;
                public class _SeatProtectionTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Type { get { return __type__; } set { __type__ = value; } }
                    private StringID __type__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float RifleDamageMultiplierPossibly { get { return __rifleDamageMultiplierPossibly__; } set { __rifleDamageMultiplierPossibly__ = value; } }
                    private float __rifleDamageMultiplierPossibly__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float PistolDamageMultiplierPossibly { get { return __pistolDamageMultiplierPossibly__; } set { __pistolDamageMultiplierPossibly__ = value; } }
                    private float __pistolDamageMultiplierPossibly__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float SupportLowDamageMultiplierPossibly { get { return __supportLowDamageMultiplierPossibly__; } set { __supportLowDamageMultiplierPossibly__ = value; } }
                    private float __supportLowDamageMultiplierPossibly__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float SupportHighDamageMultiplierPossibly { get { return __supportHighDamageMultiplierPossibly__; } set { __supportHighDamageMultiplierPossibly__ = value; } }
                    private float __supportHighDamageMultiplierPossibly__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _SeatProtectionTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _SeatProtectionTagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_SeatProtectionTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _SeatProtectionTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 33. TagBlock :: Offset: 0xE0 (224)
                /// </summary>
                public _HingesTagBlock Hinges { get { return __hinges__; } set { __hinges__ = value; } }
                private _HingesTagBlock __hinges__;
                public class _HingesTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Name { get { return __name__; } set { __name__ = value; } }
                    private StringID __name__;
                    /// <summary>
                    /// 2. StringID :: Offset: 0x4 (4)
                    /// </summary>
                    public StringID Location { get { return __location__; } set { __location__ = value; } }
                    private StringID __location__;
                    /// <summary>
                    /// 3. StringID :: Offset: 0x8 (8)
                    /// </summary>
                    public StringID ObjectAffected { get { return __objectAffected__; } set { __objectAffected__ = value; } }
                    private StringID __objectAffected__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 5. Int32 :: Offset: 0x10 (16)
                    /// </summary>
                    public int Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private int __unknown2__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _HingesTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _HingesTagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_HingesTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _HingesTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 34. Tag :: Offset: 0xE8 (232)
                /// </summary>
                public TagRef Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private TagRef __unknown20__;
                /// <summary>
                /// 35. Tag :: Offset: 0xF0 (240)
                /// </summary>
                public TagRef Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                private TagRef __unknown21__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Health_ShieldsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Health_ShieldsTagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    DamageLocations = new _DamageLocationsTagBlock(layout);
                    Unknown16 = new _Unknown16TagBlock(layout);
                    SeatProtection = new _SeatProtectionTagBlock(layout);
                    Hinges = new _HingesTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Health_ShieldsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Health_ShieldsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 19. TagBlock :: Offset: 0x68 (104)
            /// </summary>
            public _HitBoxsTagBlock HitBoxs { get { return __hitBoxs__; } set { __hitBoxs__ = value; } }
            private _HitBoxsTagBlock __hitBoxs__;
            public class _HitBoxsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float RadiusPossibly { get { return __radiusPossibly__; } set { __radiusPossibly__ = value; } }
                private float __radiusPossibly__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float DamageMultiplierPossibly { get { return __damageMultiplierPossibly__; } set { __damageMultiplierPossibly__ = value; } }
                private float __damageMultiplierPossibly__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 6. Bitmask32 :: Offset: 0x14 (20)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool RocketLockOnWithBiped { get { return __rocketLockOnWithBiped__; } set { __rocketLockOnWithBiped__ = value; } }
                    private bool __rocketLockOnWithBiped__;
                    /// <summary>
                    /// 2. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool PlasmaPistolLockOnable { get { return __plasmaPistolLockOnable__; } set { __plasmaPistolLockOnable__ = value; } }
                    private bool __plasmaPistolLockOnable__;
                    /// <summary>
                    /// 3. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool CountHitAsHeadshot { get { return __countHitAsHeadshot__; } set { __countHitAsHeadshot__ = value; } }
                    private bool __countHitAsHeadshot__;
                    /// <summary>
                    /// 4. Option :: Value: 0x6 (6)
                    /// </summary>
                    public bool DoubleDamage { get { return __doubleDamage__; } set { __doubleDamage__ = value; } }
                    private bool __doubleDamage__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        RocketLockOnWithBiped = ((bit & mask) == bit);
                        bit <<= 1;
                        PlasmaPistolLockOnable = ((bit & mask) == bit);
                        bit <<= 1;
                        CountHitAsHeadshot = ((bit & mask) == bit);
                        bit <<= 1;
                        DoubleDamage = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _HitBoxsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _HitBoxsTagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_HitBoxsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _HitBoxsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 20. TagBlock :: Offset: 0x70 (112)
            /// </summary>
            public _ModelVariationsTagBlock ModelVariations { get { return __modelVariations__; } set { __modelVariations__ = value; } }
            private _ModelVariationsTagBlock __modelVariations__;
            public class _ModelVariationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private float __unknown__;
                /// <summary>
                /// 3. TagBlock :: Offset: 0x8 (8)
                /// </summary>
                public _PermutationTagBlock Permutation { get { return __permutation__; } set { __permutation__ = value; } }
                private _PermutationTagBlock __permutation__;
                public class _PermutationTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private HlmtLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID Variation { get { return __variation__; } set { __variation__ = value; } }
                    private StringID __variation__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private float __unknown__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PermutationTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PermutationTagBlock(HlmtLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PermutationTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PermutationTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ModelVariationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ModelVariationsTagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Permutation = new _PermutationTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ModelVariationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ModelVariationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 21. TagBlock :: Offset: 0x78 (120)
            /// </summary>
            public _BonesTagBlock Bones { get { return __bones__; } set { __bones__ = value; } }
            private _BonesTagBlock __bones__;
            public class _BonesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID BoneName { get { return __boneName__; } set { __boneName__ = value; } }
                private StringID __boneName__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x4 (4)
                /// </summary>
                public short Parent_Index { get { return __parent_Index__; } set { __parent_Index__ = value; } }
                private short __parent_Index__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x6 (6)
                /// </summary>
                public short FirstChild_Index { get { return __firstChild_Index__; } set { __firstChild_Index__ = value; } }
                private short __firstChild_Index__;
                /// <summary>
                /// 4. Int16 :: Offset: 0x8 (8)
                /// </summary>
                public short NextSibling_Index { get { return __nextSibling_Index__; } set { __nextSibling_Index__ = value; } }
                private short __nextSibling_Index__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xA (10)
                /// </summary>
                public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private short __unknown1__;
                /// <summary>
                /// 6. Float :: Offset: 0xC (12)
                /// </summary>
                public float Y { get { return __y__; } set { __y__ = value; } }
                private float __y__;
                /// <summary>
                /// 7. Float :: Offset: 0x10 (16)
                /// </summary>
                public float X { get { return __x__; } set { __x__ = value; } }
                private float __x__;
                /// <summary>
                /// 8. Float :: Offset: 0x12 (18)
                /// </summary>
                public float Z { get { return __z__; } set { __z__ = value; } }
                private float __z__;
                /// <summary>
                /// 9. Float :: Offset: 0x16 (22)
                /// </summary>
                public float I { get { return __i__; } set { __i__ = value; } }
                private float __i__;
                /// <summary>
                /// 10. Float :: Offset: 0x1A (26)
                /// </summary>
                public float J { get { return __j__; } set { __j__ = value; } }
                private float __j__;
                /// <summary>
                /// 11. Float :: Offset: 0x1E (30)
                /// </summary>
                public float K { get { return __k__; } set { __k__ = value; } }
                private float __k__;
                /// <summary>
                /// 12. Float :: Offset: 0x22 (34)
                /// </summary>
                public float W { get { return __w__; } set { __w__ = value; } }
                private float __w__;
                /// <summary>
                /// 13. ByteArray :: Offset: 0x26 (38)
                /// </summary>
                public byte[] Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private byte[] __unknown2__;
                /// <summary>
                /// 14. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 15. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 16. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 17. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 18. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 19. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 20. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 21. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 22. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 23. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 24. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Y_LeaveChild { get { return __y_LeaveChild__; } set { __y_LeaveChild__ = value; } }
                private float __y_LeaveChild__;
                /// <summary>
                /// 25. Float :: Offset: 0x54 (84)
                /// </summary>
                public float X_LeaveChild { get { return __x_LeaveChild__; } set { __x_LeaveChild__ = value; } }
                private float __x_LeaveChild__;
                /// <summary>
                /// 26. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Z_LeaveChild { get { return __z_LeaveChild__; } set { __z_LeaveChild__ = value; } }
                private float __z_LeaveChild__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _BonesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _BonesTagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_BonesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _BonesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 22. ByteArray :: Offset: 0x80 (128)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;
            /// <summary>
            /// 23. TagBlock :: Offset: 0x84 (132)
            /// </summary>
            public _Unknown9TagBlock Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private _Unknown9TagBlock __unknown9__;
            public class _Unknown9TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public HlmtLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private HlmtLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown9TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown9TagBlock(HlmtLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown9TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown9TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 24. Tag :: Offset: 0x8C (140)
            /// </summary>
            public TagRef UnitDialogue { get { return __unitDialogue__; } set { __unitDialogue__ = value; } }
            private TagRef __unitDialogue__;
            /// <summary>
            /// 25. Tag :: Offset: 0x94 (148)
            /// </summary>
            public TagRef ActiveCamoShader { get { return __activeCamoShader__; } set { __activeCamoShader__ = value; } }
            private TagRef __activeCamoShader__;
            /// <summary>
            /// 26. ByteArray :: Offset: 0x9C (156)
            /// </summary>
            public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
            private byte[] __unused4__;
            /// <summary>
            /// 27. Float :: Offset: 0xA4 (164)
            /// </summary>
            public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private float __unknown10__;
            /// <summary>
            /// 28. Float :: Offset: 0xA8 (168)
            /// </summary>
            public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private float __unknown11__;
            /// <summary>
            /// 29. Float :: Offset: 0xAC (172)
            /// </summary>
            public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private float __unknown12__;
            /// <summary>
            /// 30. ByteArray :: Offset: 0xB0 (176)
            /// </summary>
            public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
            private byte[] __unused5__;
            /// <summary>
            /// 31. Float :: Offset: 0xC4 (196)
            /// </summary>
            public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private float __unknown13__;
            /// <summary>
            /// 32. Float :: Offset: 0xC8 (200)
            /// </summary>
            public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
            private float __unknown14__;
            /// <summary>
            /// 33. Float :: Offset: 0xCC (204)
            /// </summary>
            public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
            private float __unknown15__;
            /// <summary>
            /// 34. ByteArray :: Offset: 0xD0 (208)
            /// </summary>
            public byte[] Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
            private byte[] __unused6__;
            /// <summary>
            /// 35. Float :: Offset: 0xE4 (228)
            /// </summary>
            public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
            private float __unknown16__;
            /// <summary>
            /// 36. ByteArray :: Offset: 0xE8 (232)
            /// </summary>
            public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
            private byte[] __unused7__;
            /// <summary>
            /// 37. Tag :: Offset: 0xF0 (240)
            /// </summary>
            public TagRef Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
            private TagRef __unknown17__;
            /// <summary>
            /// 38. StringID :: Offset: 0xF8 (248)
            /// </summary>
            public StringID Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
            private StringID __unknown18__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the HlmtLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public HlmtLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Variations = new _VariationsTagBlock(this);
                SurfaceMaterials = new _SurfaceMaterialsTagBlock(this);
                Health_Shields = new _Health_ShieldsTagBlock(this);
                HitBoxs = new _HitBoxsTagBlock(this);
                ModelVariations = new _ModelVariationsTagBlock(this);
                Bones = new _BonesTagBlock(this);
                Unknown9 = new _Unknown9TagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Model._Initialize(br);
                Collision._Initialize(br);
                Animations._Initialize(br);
                Physics._Initialize(br);
                PhysicsModel._Initialize(br);
                Unknown1 = br.ReadSingle();
                Unknown2 = br.ReadSingle();
                Unused1 = br.ReadBytes(4);
                Unknown3 = br.ReadSingle();
                Unknown4 = br.ReadSingle();
                Unknown5 = br.ReadSingle();
                Unknown6 = br.ReadSingle();
                Unknown7 = br.ReadSingle();
                Unused2 = br.ReadBytes(4);
                Unknown8 = br.ReadSingle();
                Variations._Initialize(br);
                SurfaceMaterials._Initialize(br);
                Health_Shields._Initialize(br);
                HitBoxs._Initialize(br);
                ModelVariations._Initialize(br);
                Bones._Initialize(br);
                Unused3 = br.ReadBytes(4);
                Unknown9._Initialize(br);
                UnitDialogue._Initialize(br);
                ActiveCamoShader._Initialize(br);
                Unused4 = br.ReadBytes(8);
                Unknown10 = br.ReadSingle();
                Unknown11 = br.ReadSingle();
                Unknown12 = br.ReadSingle();
                Unused5 = br.ReadBytes(20);
                Unknown13 = br.ReadSingle();
                Unknown14 = br.ReadSingle();
                Unknown15 = br.ReadSingle();
                Unused6 = br.ReadBytes(20);
                Unknown16 = br.ReadSingle();
                Unused7 = br.ReadBytes(8);
                Unknown17._Initialize(br);
                Unknown18._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Hlmt class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Hlmt(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new HlmtLayout(map, identity);
        }

        #region Specific Methods of hlmt

        #endregion
    }
}
