using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// JMS Animation Graph
    /// </summary>
    public class Jmad
    {
        #region Constants

        /// <summary>
        /// Size of the jmad meta header.
        /// </summary>
        public const int HeaderSize = 188;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private JmadLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.5 Soldier of Light: Mapped most of the plugin...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public JmadLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class JmadLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Tag :: Offset: 0x0 (0)
            /// </summary>
            public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private TagRef __unknown1__;
            /// <summary>
            /// 2. Float :: Offset: 0x8 (8)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 3. TagBlock :: Offset: 0xC (12)
            /// </summary>
            public _BonesTagBlock Bones { get { return __bones__; } set { __bones__ = value; } }
            private _BonesTagBlock __bones__;
            public class _BonesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Bone { get { return __bone__; } set { __bone__ = value; } }
                private StringID __bone__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x4 (4)
                /// </summary>
                public short Parent { get { return __parent__; } set { __parent__ = value; } }
                private short __parent__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x6 (6)
                /// </summary>
                public short Child { get { return __child__; } set { __child__ = value; } }
                private short __child__;
                /// <summary>
                /// 4. Int16 :: Offset: 0x8 (8)
                /// </summary>
                public short NextSibling { get { return __nextSibling__; } set { __nextSibling__ = value; } }
                private short __nextSibling__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xA (10)
                /// </summary>
                public short FinalSiblingPossibly { get { return __finalSiblingPossibly__; } set { __finalSiblingPossibly__ = value; } }
                private short __finalSiblingPossibly__;
                /// <summary>
                /// 6. ByteArray :: Offset: 0xC (12)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 7. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private float __unknown__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _BonesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _BonesTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_BonesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _BonesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 4. TagBlock :: Offset: 0x14 (20)
            /// </summary>
            public _SoundsTagBlock Sounds { get { return __sounds__; } set { __sounds__ = value; } }
            private _SoundsTagBlock __sounds__;
            public class _SoundsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Sound { get { return __sound__; } set { __sound__ = value; } }
                private TagRef __sound__;
                /// <summary>
                /// 2. Int32 :: Offset: 0x8 (8)
                /// </summary>
                public int Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private int __unknown__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _SoundsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _SoundsTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_SoundsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _SoundsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 5. TagBlock :: Offset: 0x1C (28)
            /// </summary>
            public _EffectsTagBlock Effects { get { return __effects__; } set { __effects__ = value; } }
            private _EffectsTagBlock __effects__;
            public class _EffectsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Effect { get { return __effect__; } set { __effect__ = value; } }
                private TagRef __effect__;
                /// <summary>
                /// 2. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _EffectsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _EffectsTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_EffectsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _EffectsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 6. TagBlock :: Offset: 0x24 (36)
            /// </summary>
            public _ScreenBoundsTagBlock ScreenBounds { get { return __screenBounds__; } set { __screenBounds__ = value; } }
            private _ScreenBoundsTagBlock __screenBounds__;
            public class _ScreenBoundsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Label { get { return __label__; } set { __label__ = value; } }
                private StringID __label__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float RightYawPerFrame { get { return __rightYawPerFrame__; } set { __rightYawPerFrame__ = value; } }
                private float __rightYawPerFrame__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float LeftYawPerFrame { get { return __leftYawPerFrame__; } set { __leftYawPerFrame__ = value; } }
                private float __leftYawPerFrame__;
                /// <summary>
                /// 4. Int16 :: Offset: 0xC (12)
                /// </summary>
                public short RightFrameCount { get { return __rightFrameCount__; } set { __rightFrameCount__ = value; } }
                private short __rightFrameCount__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xE (14)
                /// </summary>
                public short LeftFrameCount { get { return __leftFrameCount__; } set { __leftFrameCount__ = value; } }
                private short __leftFrameCount__;
                /// <summary>
                /// 6. Float :: Offset: 0x10 (16)
                /// </summary>
                public float DownPitchPerFrame { get { return __downPitchPerFrame__; } set { __downPitchPerFrame__ = value; } }
                private float __downPitchPerFrame__;
                /// <summary>
                /// 7. Float :: Offset: 0x14 (20)
                /// </summary>
                public float UpPitchPerFrame { get { return __upPitchPerFrame__; } set { __upPitchPerFrame__ = value; } }
                private float __upPitchPerFrame__;
                /// <summary>
                /// 8. Int16 :: Offset: 0x18 (24)
                /// </summary>
                public short DownPitchFrameCount { get { return __downPitchFrameCount__; } set { __downPitchFrameCount__ = value; } }
                private short __downPitchFrameCount__;
                /// <summary>
                /// 9. Int16 :: Offset: 0x1A (26)
                /// </summary>
                public short UpPitchFrameCount { get { return __upPitchFrameCount__; } set { __upPitchFrameCount__ = value; } }
                private short __upPitchFrameCount__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ScreenBoundsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ScreenBoundsTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ScreenBoundsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ScreenBoundsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 7. TagBlock :: Offset: 0x2C (44)
            /// </summary>
            public _AnimationsTagBlock Animations { get { return __animations__; } set { __animations__ = value; } }
            private _AnimationsTagBlock __animations__;
            public class _AnimationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Name { get { return __name__; } set { __name__ = value; } }
                private StringID __name__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Byte :: Offset: 0x8 (8)
                /// </summary>
                public byte Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private byte __unknown2__;
                /// <summary>
                /// 4. Byte :: Offset: 0x9 (9)
                /// </summary>
                public byte Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private byte __unknown3__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xA (10)
                /// </summary>
                public short Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private short __unknown4__;
                /// <summary>
                /// 6. Byte :: Offset: 0xC (12)
                /// </summary>
                public byte Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private byte __unknown5__;
                /// <summary>
                /// 7. Byte :: Offset: 0xD (13)
                /// </summary>
                public byte Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private byte __unknown6__;
                /// <summary>
                /// 8. Int16 :: Offset: 0xE (14)
                /// </summary>
                public short Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private short __unknown7__;
                /// <summary>
                /// 9. Int16 :: Offset: 0x10 (16)
                /// </summary>
                public short Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private short __unknown8__;
                /// <summary>
                /// 10. Byte :: Offset: 0x12 (18)
                /// </summary>
                public byte Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private byte __unknown9__;
                /// <summary>
                /// 11. Byte :: Offset: 0x13 (19)
                /// </summary>
                public byte NumOfBones { get { return __numOfBones__; } set { __numOfBones__ = value; } }
                private byte __numOfBones__;
                /// <summary>
                /// 12. Int16 :: Offset: 0x14 (20)
                /// </summary>
                public short MaxCycleLength_milliseconds { get { return __maxCycleLength_milliseconds__; } set { __maxCycleLength_milliseconds__ = value; } }
                private short __maxCycleLength_milliseconds__;
                /// <summary>
                /// 13. Int16 :: Offset: 0x16 (22)
                /// </summary>
                public short Unknown_DoNotChange { get { return __unknown_DoNotChange__; } set { __unknown_DoNotChange__ = value; } }
                private short __unknown_DoNotChange__;
                /// <summary>
                /// 14. Int16 :: Offset: 0x18 (24)
                /// </summary>
                public short TransitionSpeedPossibly { get { return __transitionSpeedPossibly__; } set { __transitionSpeedPossibly__ = value; } }
                private short __transitionSpeedPossibly__;
                /// <summary>
                /// 15. Int16 :: Offset: 0x1A (26)
                /// </summary>
                public short Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private short __unknown10__;
                /// <summary>
                /// 16. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 17. ByteArray :: Offset: 0x20 (32)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 18. Int32 :: Offset: 0x24 (36)
                /// </summary>
                public int RawChunkNum { get { return __rawChunkNum__; } set { __rawChunkNum__ = value; } }
                private int __rawChunkNum__;
                /// <summary>
                /// 19. Int16 :: Offset: 0x28 (40)
                /// </summary>
                public short OffsetInRaw { get { return __offsetInRaw__; } set { __offsetInRaw__ = value; } }
                private short __offsetInRaw__;
                /// <summary>
                /// 20. Int16 :: Offset: 0x2A (42)
                /// </summary>
                public short Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private short __unused2__;
                /// <summary>
                /// 21. Int16 :: Offset: 0x2C (44)
                /// </summary>
                public short Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private short __unknown12__;
                /// <summary>
                /// 22. Bitmask16 :: Offset: 0x2E (46)
                /// </summary>
                public DoesntRepeatBitmask16 DoesntRepeat { get { return __doesntRepeat__; } set { __doesntRepeat__ = value; } }
                private DoesntRepeatBitmask16 __doesntRepeat__;
                public class DoesntRepeatBitmask16
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x8 (8)
                    /// </summary>
                    public bool DoesntRepeat { get { return __doesntRepeat__; } set { __doesntRepeat__ = value; } }
                    private bool __doesntRepeat__;

                    public void _Initialize(BitReader br)
                    {
                        ushort mask = br.ReadUInt16();
                        ushort bit = 1;

                        DoesntRepeat = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 23. Int16 :: Offset: 0x30 (48)
                /// </summary>
                public short Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private short __unknown13__;
                /// <summary>
                /// 24. Int16 :: Offset: 0x32 (50)
                /// </summary>
                public short Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private short __unknown14__;
                /// <summary>
                /// 25. ByteArray :: Offset: 0x34 (52)
                /// </summary>
                public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private byte[] __unused3__;
                /// <summary>
                /// 26. Byte :: Offset: 0x3C (60)
                /// </summary>
                public byte Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private byte __unknown15__;
                /// <summary>
                /// 27. Byte :: Offset: 0x3D (61)
                /// </summary>
                public byte Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private byte __unknown16__;
                /// <summary>
                /// 28. Byte :: Offset: 0x3E (62)
                /// </summary>
                public byte Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private byte __unknown17__;
                /// <summary>
                /// 29. Byte :: Offset: 0x3F (63)
                /// </summary>
                public byte Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private byte __unknown18__;
                /// <summary>
                /// 30. Int16 :: Offset: 0x40 (64)
                /// </summary>
                public short Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private short __unknown19__;
                /// <summary>
                /// 31. Int16 :: Offset: 0x42 (66)
                /// </summary>
                public short AnimationRawSomething { get { return __animationRawSomething__; } set { __animationRawSomething__ = value; } }
                private short __animationRawSomething__;
                /// <summary>
                /// 32. ByteArray :: Offset: 0x44 (68)
                /// </summary>
                public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                private byte[] __unused4__;
                /// <summary>
                /// 33. Byte :: Offset: 0x48 (72)
                /// </summary>
                public byte Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private byte __unknown20__;
                /// <summary>
                /// 34. Byte :: Offset: 0x49 (73)
                /// </summary>
                public byte Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                private byte __unknown21__;
                /// <summary>
                /// 35. Int16 :: Offset: 0x4A (74)
                /// </summary>
                public short Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                private short __unknown22__;
                /// <summary>
                /// 36. TagBlock :: Offset: 0x4C (76)
                /// </summary>
                public _Unknown23TagBlock Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                private _Unknown23TagBlock __unknown23__;
                public class _Unknown23TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private short __unknown1__;
                    /// <summary>
                    /// 2. Int16 :: Offset: 0x2 (2)
                    /// </summary>
                    public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private short __unknown2__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown23TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown23TagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown23TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown23TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 37. TagBlock :: Offset: 0x54 (84)
                /// </summary>
                public _SoundsTagBlock Sounds { get { return __sounds__; } set { __sounds__ = value; } }
                private _SoundsTagBlock __sounds__;
                public class _SoundsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short ChunkNum { get { return __chunkNum__; } set { __chunkNum__ = value; } }
                    private short __chunkNum__;
                    /// <summary>
                    /// 2. Int16 :: Offset: 0x2 (2)
                    /// </summary>
                    public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private short __unknown1__;
                    /// <summary>
                    /// 3. StringID :: Offset: 0x4 (4)
                    /// </summary>
                    public StringID Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private StringID __unknown2__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _SoundsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _SoundsTagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_SoundsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _SoundsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 38. TagBlock :: Offset: 0x5C (92)
                /// </summary>
                public _EffectsTagBlock Effects { get { return __effects__; } set { __effects__ = value; } }
                private _EffectsTagBlock __effects__;
                public class _EffectsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private short __unknown1__;
                    /// <summary>
                    /// 2. Int16 :: Offset: 0x2 (2)
                    /// </summary>
                    public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private short __unknown2__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _EffectsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _EffectsTagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_EffectsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _EffectsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 39. TagBlock :: Offset: 0x64 (100)
                /// </summary>
                public _Unknown24TagBlock Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                private _Unknown24TagBlock __unknown24__;
                public class _Unknown24TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private short __unknown1__;
                    /// <summary>
                    /// 2. Int16 :: Offset: 0x2 (2)
                    /// </summary>
                    public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private short __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown24TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown24TagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown24TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown24TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AnimationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AnimationsTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    DoesntRepeat = new DoesntRepeatBitmask16();
                    Unknown23 = new _Unknown23TagBlock(layout);
                    Sounds = new _SoundsTagBlock(layout);
                    Effects = new _EffectsTagBlock(layout);
                    Unknown24 = new _Unknown24TagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AnimationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AnimationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 8. TagBlock :: Offset: 0x34 (52)
            /// </summary>
            public _AnimationCallsTagBlock AnimationCalls { get { return __animationCalls__; } set { __animationCalls__ = value; } }
            private _AnimationCallsTagBlock __animationCalls__;
            public class _AnimationCallsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID PositionName { get { return __positionName__; } set { __positionName__ = value; } }
                private StringID __positionName__;
                /// <summary>
                /// 2. TagBlock :: Offset: 0x4 (4)
                /// </summary>
                public _ClassTagBlock Class { get { return __class__; } set { __class__ = value; } }
                private _ClassTagBlock __class__;
                public class _ClassTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID ClassName { get { return __className__; } set { __className__ = value; } }
                    private StringID __className__;
                    /// <summary>
                    /// 2. TagBlock :: Offset: 0x4 (4)
                    /// </summary>
                    public _ObjectTagBlock Object { get { return __object__; } set { __object__ = value; } }
                    private _ObjectTagBlock __object__;
                    public class _ObjectTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private JmadLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. StringID :: Offset: 0x0 (0)
                        /// </summary>
                        public StringID Type { get { return __type__; } set { __type__ = value; } }
                        private StringID __type__;
                        /// <summary>
                        /// 2. TagBlock :: Offset: 0x4 (4)
                        /// </summary>
                        public _FullObjectAnimationsTagBlock FullObjectAnimations { get { return __fullObjectAnimations__; } set { __fullObjectAnimations__ = value; } }
                        private _FullObjectAnimationsTagBlock __fullObjectAnimations__;
                        public class _FullObjectAnimationsTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private JmadLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. StringID :: Offset: 0x0 (0)
                            /// </summary>
                            public StringID Condition { get { return __condition__; } set { __condition__ = value; } }
                            private StringID __condition__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x4 (4)
                            /// </summary>
                            public short Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private short __unknown__;
                            /// <summary>
                            /// 3. Int16 :: Offset: 0x6 (6)
                            /// </summary>
                            public short Animation { get { return __animation__; } set { __animation__ = value; } }
                            private short __animation__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _FullObjectAnimationsTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _FullObjectAnimationsTagBlock(JmadLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_FullObjectAnimationsTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _FullObjectAnimationsTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 3. TagBlock :: Offset: 0xC (12)
                        /// </summary>
                        public _Half_UpperObjectAnimationsTagBlock Half_UpperObjectAnimations { get { return __half_UpperObjectAnimations__; } set { __half_UpperObjectAnimations__ = value; } }
                        private _Half_UpperObjectAnimationsTagBlock __half_UpperObjectAnimations__;
                        public class _Half_UpperObjectAnimationsTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private JmadLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. StringID :: Offset: 0x0 (0)
                            /// </summary>
                            public StringID Condition { get { return __condition__; } set { __condition__ = value; } }
                            private StringID __condition__;
                            /// <summary>
                            /// 2. Int16 :: Offset: 0x4 (4)
                            /// </summary>
                            public short Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private short __unknown__;
                            /// <summary>
                            /// 3. Int16 :: Offset: 0x6 (6)
                            /// </summary>
                            public short Animation { get { return __animation__; } set { __animation__ = value; } }
                            private short __animation__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Half_UpperObjectAnimationsTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Half_UpperObjectAnimationsTagBlock(JmadLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Half_UpperObjectAnimationsTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Half_UpperObjectAnimationsTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 4. TagBlock :: Offset: 0x14 (20)
                        /// </summary>
                        public _Death_DamageTagBlock Death_Damage { get { return __death_Damage__; } set { __death_Damage__ = value; } }
                        private _Death_DamageTagBlock __death_Damage__;
                        public class _Death_DamageTagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private JmadLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. StringID :: Offset: 0x0 (0)
                            /// </summary>
                            public StringID Condition { get { return __condition__; } set { __condition__ = value; } }
                            private StringID __condition__;
                            /// <summary>
                            /// 2. TagBlock :: Offset: 0x4 (4)
                            /// </summary>
                            public _ValuesTagBlock Values { get { return __values__; } set { __values__ = value; } }
                            private _ValuesTagBlock __values__;
                            public class _ValuesTagBlock
                            {
                                #region Class Instance Data, Accessors, Classes

                                /// <summary>
                                /// The root class.
                                /// </summary>
                                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                                private JmadLayout __layout__;
                                /// <summary>
                                /// The offset at which this reflexive is declared in the tag meta.
                                /// </summary>
                                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                                private long __declarationOffset__;
                                /// <summary>
                                /// The total amount of chunks.
                                /// </summary>
                                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                                private int __count__;
                                /// <summary>
                                /// The initial pointer read from the tag meta.
                                /// </summary>
                                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                                private int __pointer__;
                                /// <summary>
                                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                                /// </summary>
                                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                                private int __translation__;
                                /// <summary>
                                /// The currently selected chunk's index.
                                /// </summary>
                                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                                private int __selectedIndex__;
                                /// <summary>
                                /// True if the reflexive does not exist in the current tag meta.
                                /// </summary>
                                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                                private bool __null__;
                                /// <summary>
                                /// True if Initialize() has been called, and executed, without error.
                                /// </summary>
                                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                                private bool __initialized__;

                                #endregion

                                #region Reflexive Instance Data, Accessors, Classes

                                /// <summary>
                                /// 1. TagBlock :: Offset: 0x0 (0)
                                /// </summary>
                                public __ValuesTagBlock Values { get { return __values__; } set { __values__ = value; } }
                                private __ValuesTagBlock __values__;
                                public class __ValuesTagBlock
                                {
                                    #region Class Instance Data, Accessors, Classes

                                    /// <summary>
                                    /// The root class.
                                    /// </summary>
                                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                                    private JmadLayout __layout__;
                                    /// <summary>
                                    /// The offset at which this reflexive is declared in the tag meta.
                                    /// </summary>
                                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                                    private long __declarationOffset__;
                                    /// <summary>
                                    /// The total amount of chunks.
                                    /// </summary>
                                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                                    private int __count__;
                                    /// <summary>
                                    /// The initial pointer read from the tag meta.
                                    /// </summary>
                                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                                    private int __pointer__;
                                    /// <summary>
                                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                                    /// </summary>
                                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                                    private int __translation__;
                                    /// <summary>
                                    /// The currently selected chunk's index.
                                    /// </summary>
                                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                                    private int __selectedIndex__;
                                    /// <summary>
                                    /// True if the reflexive does not exist in the current tag meta.
                                    /// </summary>
                                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                                    private bool __null__;
                                    /// <summary>
                                    /// True if Initialize() has been called, and executed, without error.
                                    /// </summary>
                                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                                    private bool __initialized__;

                                    #endregion

                                    #region Reflexive Instance Data, Accessors, Classes

                                    /// <summary>
                                    /// 1. Int16 :: Offset: 0x0 (0)
                                    /// </summary>
                                    public short Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                                    private short __unknown__;
                                    /// <summary>
                                    /// 2. Int16 :: Offset: 0x2 (2)
                                    /// </summary>
                                    public short ChunkNum { get { return __chunkNum__; } set { __chunkNum__ = value; } }
                                    private short __chunkNum__;

                                    #endregion

                                    #region Initialization

                                    /// <summary>
                                    /// Initializes a new instance of the __ValuesTagBlock class based 
                                    /// on the supplied layout reference.
                                    /// </summary>
                                    /// <param name="layout">The layout reference. </param>
                                    public __ValuesTagBlock(JmadLayout layout)
                                    {
                                        this._Layout = layout;
                                        this._Initialized = false;

                                    }

                                    /// <summary>
                                    /// Parses the declaration of the reflexive into the several class members.
                                    /// </summary>
                                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                                    public void _Initialize(BitReader br)
                                    {
                                        _Initialized = false;
                                        int cc = br.ReadInt32();
                                        int tr = br.ReadInt32();

                                        if (tr == -1)
                                        {
                                            _Null = true;
                                        }
                                        else
                                        {
                                            _DeclarationOffset = br.BaseStream.Position - 8;
                                            _Count = cc;
                                            _Pointer = tr;
                                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                            _SelectedIndex = 0;
                                            _Null = false;
                                        }
                                        _Initialized = true;
                                    }

                                    #endregion

                                    #region New, Add, Delete, Remove Methods

                                    /// <summary>
                                    /// Overwrites the current reflexive with null values. If the reflexive 
                                    /// does not exist in the current tag meta, space will be made.
                                    /// </summary>
                                    public void _New()
                                    {
                                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                        this._New(fs);
                                        fs.Close();
                                    }

                                    /// <summary>
                                    /// Overwrites the current reflexive with null values. If the reflexive 
                                    /// does not exist in the current tag meta, space will be made.
                                    /// </summary>
                                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                    public void _New(FileStream fs)
                                    {
                                        BitReader br = new BitReader(fs, Endian.Little);
                                        BitWriter bw = new BitWriter(fs, Endian.Little);

                                        if (_Null)
                                        {
                                            // Make space
                                        }

                                        // Write null, -1 for pointers
                                    }

                                    /// <summary>
                                    /// Overwrites the current reflexive with the values provided
                                    /// by the template. If the reflexive does not exist in the current 
                                    /// tag meta, space will be made.
                                    /// </summary>
                                    /// <param name="template">The template containing the new values. </param>
                                    public void _New(__ValuesTagBlock template)
                                    {
                                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                        this._New(fs, template);
                                        fs.Close();
                                    }

                                    /// <summary>
                                    /// Overwrites the current reflexive with the values provided
                                    /// by the template. If the reflexive does not exist in the current 
                                    /// tag meta, space will be made.
                                    /// </summary>
                                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                    /// <param name="template">The template containing the new values. </param>
                                    public void _New(FileStream fs, __ValuesTagBlock template)
                                    {
                                        BitReader br = new BitReader(fs, Endian.Little);
                                        BitWriter bw = new BitWriter(fs, Endian.Little);

                                        if (_Null)
                                        {
                                            // Make space
                                        }

                                        // Write template
                                    }

                                    /// <summary>
                                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                    /// be made up of null values.
                                    /// </summary>
                                    public void _Add()
                                    {
                                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                        this._Add(fs);
                                        fs.Close();
                                    }

                                    /// <summary>
                                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                    /// be made up of null values.
                                    /// </summary>
                                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                    public void _Add(FileStream fs)
                                    {
                                        if (_Null) throw new Exception("Reflexive does not exist.");

                                        BitReader br = new BitReader(fs, Endian.Little);
                                        BitWriter bw = new BitWriter(fs, Endian.Little);

                                        // Make null space
                                    }

                                    /// <summary>
                                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                    /// be based off of the values found within the chunk at the given index.
                                    /// </summary>
                                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                                    public void _Add(int index)
                                    {
                                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                        this._Add(fs, index);
                                        fs.Close();
                                    }

                                    /// <summary>
                                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                    /// be based off of the values found within the chunk at the given index.
                                    /// </summary>
                                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                                    public void _Add(FileStream fs, int index)
                                    {
                                        if (_Null) throw new Exception("Reflexive does not exist.");

                                        BitReader br = new BitReader(fs, Endian.Little);
                                        BitWriter bw = new BitWriter(fs, Endian.Little);

                                        // Make space, fill with template
                                    }

                                    /// <summary>
                                    /// Removes the reflexive if it exists in the current tag meta.
                                    /// </summary>
                                    public void _Delete()
                                    {
                                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                        this._Delete(fs);
                                        fs.Close();
                                    }

                                    /// <summary>
                                    /// Removes the reflexive if it exists in the current tag meta.
                                    /// </summary>
                                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                    public void _Delete(FileStream fs)
                                    {
                                        if (_Null) return;

                                        BitReader br = new BitReader(fs, Endian.Little);
                                        BitWriter bw = new BitWriter(fs, Endian.Little);

                                        // Delete

                                        _Null = true;
                                    }

                                    /// <summary>
                                    /// Removes the chunk at the given index.
                                    /// </summary>
                                    /// <param name="index">The index of the chunk to remove. </param>
                                    public void _Remove(int index)
                                    {
                                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                        this._Remove(fs, index);
                                        fs.Close();
                                    }

                                    /// <summary>
                                    /// Removes the chunk at the given index.
                                    /// </summary>
                                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                    /// <param name="index">The index of the chunk to remove. </param>
                                    public void _Remove(FileStream fs, int index)
                                    {
                                        if (_Null) throw new Exception("Reflexive does not exist.");

                                        BitReader br = new BitReader(fs, Endian.Little);
                                        BitWriter bw = new BitWriter(fs, Endian.Little);

                                        // Remove

                                        if (--_Count == 0)
                                        {
                                            // Write -1 pointer
                                            _Null = true;
                                        }
                                    }

                                    #endregion
                                }

                                #endregion

                                #region Initialization

                                /// <summary>
                                /// Initializes a new instance of the _ValuesTagBlock class based 
                                /// on the supplied layout reference.
                                /// </summary>
                                /// <param name="layout">The layout reference. </param>
                                public _ValuesTagBlock(JmadLayout layout)
                                {
                                    this._Layout = layout;
                                    this._Initialized = false;

                                    Values = new __ValuesTagBlock(layout);
                                }

                                /// <summary>
                                /// Parses the declaration of the reflexive into the several class members.
                                /// </summary>
                                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                                public void _Initialize(BitReader br)
                                {
                                    _Initialized = false;
                                    int cc = br.ReadInt32();
                                    int tr = br.ReadInt32();

                                    if (tr == -1)
                                    {
                                        _Null = true;
                                    }
                                    else
                                    {
                                        _DeclarationOffset = br.BaseStream.Position - 8;
                                        _Count = cc;
                                        _Pointer = tr;
                                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                        _SelectedIndex = 0;
                                        _Null = false;
                                    }
                                    _Initialized = true;
                                }

                                #endregion

                                #region New, Add, Delete, Remove Methods

                                /// <summary>
                                /// Overwrites the current reflexive with null values. If the reflexive 
                                /// does not exist in the current tag meta, space will be made.
                                /// </summary>
                                public void _New()
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._New(fs);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Overwrites the current reflexive with null values. If the reflexive 
                                /// does not exist in the current tag meta, space will be made.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                public void _New(FileStream fs)
                                {
                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    if (_Null)
                                    {
                                        // Make space
                                    }

                                    // Write null, -1 for pointers
                                }

                                /// <summary>
                                /// Overwrites the current reflexive with the values provided
                                /// by the template. If the reflexive does not exist in the current 
                                /// tag meta, space will be made.
                                /// </summary>
                                /// <param name="template">The template containing the new values. </param>
                                public void _New(_ValuesTagBlock template)
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._New(fs, template);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Overwrites the current reflexive with the values provided
                                /// by the template. If the reflexive does not exist in the current 
                                /// tag meta, space will be made.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                /// <param name="template">The template containing the new values. </param>
                                public void _New(FileStream fs, _ValuesTagBlock template)
                                {
                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    if (_Null)
                                    {
                                        // Make space
                                    }

                                    // Write template
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be made up of null values.
                                /// </summary>
                                public void _Add()
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Add(fs);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be made up of null values.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                public void _Add(FileStream fs)
                                {
                                    if (_Null) throw new Exception("Reflexive does not exist.");

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Make null space
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be based off of the values found within the chunk at the given index.
                                /// </summary>
                                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                                public void _Add(int index)
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Add(fs, index);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be based off of the values found within the chunk at the given index.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                                public void _Add(FileStream fs, int index)
                                {
                                    if (_Null) throw new Exception("Reflexive does not exist.");

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Make space, fill with template
                                }

                                /// <summary>
                                /// Removes the reflexive if it exists in the current tag meta.
                                /// </summary>
                                public void _Delete()
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Delete(fs);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Removes the reflexive if it exists in the current tag meta.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                public void _Delete(FileStream fs)
                                {
                                    if (_Null) return;

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Delete

                                    _Null = true;
                                }

                                /// <summary>
                                /// Removes the chunk at the given index.
                                /// </summary>
                                /// <param name="index">The index of the chunk to remove. </param>
                                public void _Remove(int index)
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Remove(fs, index);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Removes the chunk at the given index.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                /// <param name="index">The index of the chunk to remove. </param>
                                public void _Remove(FileStream fs, int index)
                                {
                                    if (_Null) throw new Exception("Reflexive does not exist.");

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Remove

                                    if (--_Count == 0)
                                    {
                                        // Write -1 pointer
                                        _Null = true;
                                    }
                                }

                                #endregion
                            }

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Death_DamageTagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Death_DamageTagBlock(JmadLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                                Values = new _ValuesTagBlock(layout);
                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Death_DamageTagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Death_DamageTagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 5. TagBlock :: Offset: 0x1C (28)
                        /// </summary>
                        public _AITagBlock AI { get { return __aI__; } set { __aI__ = value; } }
                        private _AITagBlock __aI__;
                        public class _AITagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private JmadLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. StringID :: Offset: 0x0 (0)
                            /// </summary>
                            public StringID CurrentlyPerforming1 { get { return __currentlyPerforming1__; } set { __currentlyPerforming1__ = value; } }
                            private StringID __currentlyPerforming1__;
                            /// <summary>
                            /// 2. StringID :: Offset: 0x4 (4)
                            /// </summary>
                            public StringID CurrentlyPerforming2 { get { return __currentlyPerforming2__; } set { __currentlyPerforming2__ = value; } }
                            private StringID __currentlyPerforming2__;
                            /// <summary>
                            /// 3. Float :: Offset: 0x8 (8)
                            /// </summary>
                            public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private float __unknown__;
                            /// <summary>
                            /// 4. TagBlock :: Offset: 0xC (12)
                            /// </summary>
                            public _OptionsTagBlock Options { get { return __options__; } set { __options__ = value; } }
                            private _OptionsTagBlock __options__;
                            public class _OptionsTagBlock
                            {
                                #region Class Instance Data, Accessors, Classes

                                /// <summary>
                                /// The root class.
                                /// </summary>
                                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                                private JmadLayout __layout__;
                                /// <summary>
                                /// The offset at which this reflexive is declared in the tag meta.
                                /// </summary>
                                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                                private long __declarationOffset__;
                                /// <summary>
                                /// The total amount of chunks.
                                /// </summary>
                                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                                private int __count__;
                                /// <summary>
                                /// The initial pointer read from the tag meta.
                                /// </summary>
                                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                                private int __pointer__;
                                /// <summary>
                                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                                /// </summary>
                                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                                private int __translation__;
                                /// <summary>
                                /// The currently selected chunk's index.
                                /// </summary>
                                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                                private int __selectedIndex__;
                                /// <summary>
                                /// True if the reflexive does not exist in the current tag meta.
                                /// </summary>
                                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                                private bool __null__;
                                /// <summary>
                                /// True if Initialize() has been called, and executed, without error.
                                /// </summary>
                                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                                private bool __initialized__;

                                #endregion

                                #region Reflexive Instance Data, Accessors, Classes

                                /// <summary>
                                /// 1. StringID :: Offset: 0x0 (0)
                                /// </summary>
                                public StringID FullName { get { return __fullName__; } set { __fullName__ = value; } }
                                private StringID __fullName__;
                                /// <summary>
                                /// 2. StringID :: Offset: 0x4 (4)
                                /// </summary>
                                public StringID StateName { get { return __stateName__; } set { __stateName__ = value; } }
                                private StringID __stateName__;
                                /// <summary>
                                /// 3. StringID :: Offset: 0x8 (8)
                                /// </summary>
                                public StringID AnimationName { get { return __animationName__; } set { __animationName__ = value; } }
                                private StringID __animationName__;
                                /// <summary>
                                /// 4. Float :: Offset: 0xC (12)
                                /// </summary>
                                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                                private float __unknown1__;
                                /// <summary>
                                /// 5. Int16 :: Offset: 0x10 (16)
                                /// </summary>
                                public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                                private short __unknown2__;
                                /// <summary>
                                /// 6. Int16 :: Offset: 0x12 (18)
                                /// </summary>
                                public short ChunkNum { get { return __chunkNum__; } set { __chunkNum__ = value; } }
                                private short __chunkNum__;

                                #endregion

                                #region Initialization

                                /// <summary>
                                /// Initializes a new instance of the _OptionsTagBlock class based 
                                /// on the supplied layout reference.
                                /// </summary>
                                /// <param name="layout">The layout reference. </param>
                                public _OptionsTagBlock(JmadLayout layout)
                                {
                                    this._Layout = layout;
                                    this._Initialized = false;

                                }

                                /// <summary>
                                /// Parses the declaration of the reflexive into the several class members.
                                /// </summary>
                                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                                public void _Initialize(BitReader br)
                                {
                                    _Initialized = false;
                                    int cc = br.ReadInt32();
                                    int tr = br.ReadInt32();

                                    if (tr == -1)
                                    {
                                        _Null = true;
                                    }
                                    else
                                    {
                                        _DeclarationOffset = br.BaseStream.Position - 8;
                                        _Count = cc;
                                        _Pointer = tr;
                                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                        _SelectedIndex = 0;
                                        _Null = false;
                                    }
                                    _Initialized = true;
                                }

                                #endregion

                                #region New, Add, Delete, Remove Methods

                                /// <summary>
                                /// Overwrites the current reflexive with null values. If the reflexive 
                                /// does not exist in the current tag meta, space will be made.
                                /// </summary>
                                public void _New()
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._New(fs);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Overwrites the current reflexive with null values. If the reflexive 
                                /// does not exist in the current tag meta, space will be made.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                public void _New(FileStream fs)
                                {
                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    if (_Null)
                                    {
                                        // Make space
                                    }

                                    // Write null, -1 for pointers
                                }

                                /// <summary>
                                /// Overwrites the current reflexive with the values provided
                                /// by the template. If the reflexive does not exist in the current 
                                /// tag meta, space will be made.
                                /// </summary>
                                /// <param name="template">The template containing the new values. </param>
                                public void _New(_OptionsTagBlock template)
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._New(fs, template);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Overwrites the current reflexive with the values provided
                                /// by the template. If the reflexive does not exist in the current 
                                /// tag meta, space will be made.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                /// <param name="template">The template containing the new values. </param>
                                public void _New(FileStream fs, _OptionsTagBlock template)
                                {
                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    if (_Null)
                                    {
                                        // Make space
                                    }

                                    // Write template
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be made up of null values.
                                /// </summary>
                                public void _Add()
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Add(fs);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be made up of null values.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                public void _Add(FileStream fs)
                                {
                                    if (_Null) throw new Exception("Reflexive does not exist.");

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Make null space
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be based off of the values found within the chunk at the given index.
                                /// </summary>
                                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                                public void _Add(int index)
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Add(fs, index);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                                /// be based off of the values found within the chunk at the given index.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                                public void _Add(FileStream fs, int index)
                                {
                                    if (_Null) throw new Exception("Reflexive does not exist.");

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Make space, fill with template
                                }

                                /// <summary>
                                /// Removes the reflexive if it exists in the current tag meta.
                                /// </summary>
                                public void _Delete()
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Delete(fs);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Removes the reflexive if it exists in the current tag meta.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                public void _Delete(FileStream fs)
                                {
                                    if (_Null) return;

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Delete

                                    _Null = true;
                                }

                                /// <summary>
                                /// Removes the chunk at the given index.
                                /// </summary>
                                /// <param name="index">The index of the chunk to remove. </param>
                                public void _Remove(int index)
                                {
                                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                    this._Remove(fs, index);
                                    fs.Close();
                                }

                                /// <summary>
                                /// Removes the chunk at the given index.
                                /// </summary>
                                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                                /// <param name="index">The index of the chunk to remove. </param>
                                public void _Remove(FileStream fs, int index)
                                {
                                    if (_Null) throw new Exception("Reflexive does not exist.");

                                    BitReader br = new BitReader(fs, Endian.Little);
                                    BitWriter bw = new BitWriter(fs, Endian.Little);

                                    // Remove

                                    if (--_Count == 0)
                                    {
                                        // Write -1 pointer
                                        _Null = true;
                                    }
                                }

                                #endregion
                            }

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _AITagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _AITagBlock(JmadLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                                Options = new _OptionsTagBlock(layout);
                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_AITagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _AITagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 6. TagBlock :: Offset: 0x24 (36)
                        /// </summary>
                        public _Unknown1TagBlock Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                        private _Unknown1TagBlock __unknown1__;
                        public class _Unknown1TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private JmadLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Int32 :: Offset: 0x0 (0)
                            /// </summary>
                            public int Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private int __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown1TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown1TagBlock(JmadLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown1TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown1TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }
                        /// <summary>
                        /// 7. TagBlock :: Offset: 0x2C (44)
                        /// </summary>
                        public _Unknown2TagBlock Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                        private _Unknown2TagBlock __unknown2__;
                        public class _Unknown2TagBlock
                        {
                            #region Class Instance Data, Accessors, Classes

                            /// <summary>
                            /// The root class.
                            /// </summary>
                            public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                            private JmadLayout __layout__;
                            /// <summary>
                            /// The offset at which this reflexive is declared in the tag meta.
                            /// </summary>
                            public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                            private long __declarationOffset__;
                            /// <summary>
                            /// The total amount of chunks.
                            /// </summary>
                            public int _Count { get { return __count__; } protected set { __count__ = value; } }
                            private int __count__;
                            /// <summary>
                            /// The initial pointer read from the tag meta.
                            /// </summary>
                            public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                            private int __pointer__;
                            /// <summary>
                            /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                            /// </summary>
                            public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                            private int __translation__;
                            /// <summary>
                            /// The currently selected chunk's index.
                            /// </summary>
                            public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                            private int __selectedIndex__;
                            /// <summary>
                            /// True if the reflexive does not exist in the current tag meta.
                            /// </summary>
                            public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                            private bool __null__;
                            /// <summary>
                            /// True if Initialize() has been called, and executed, without error.
                            /// </summary>
                            public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                            private bool __initialized__;

                            #endregion

                            #region Reflexive Instance Data, Accessors, Classes

                            /// <summary>
                            /// 1. Int32 :: Offset: 0x0 (0)
                            /// </summary>
                            public int Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                            private int __unknown__;

                            #endregion

                            #region Initialization

                            /// <summary>
                            /// Initializes a new instance of the _Unknown2TagBlock class based 
                            /// on the supplied layout reference.
                            /// </summary>
                            /// <param name="layout">The layout reference. </param>
                            public _Unknown2TagBlock(JmadLayout layout)
                            {
                                this._Layout = layout;
                                this._Initialized = false;

                            }

                            /// <summary>
                            /// Parses the declaration of the reflexive into the several class members.
                            /// </summary>
                            /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                            public void _Initialize(BitReader br)
                            {
                                _Initialized = false;
                                int cc = br.ReadInt32();
                                int tr = br.ReadInt32();

                                if (tr == -1)
                                {
                                    _Null = true;
                                }
                                else
                                {
                                    _DeclarationOffset = br.BaseStream.Position - 8;
                                    _Count = cc;
                                    _Pointer = tr;
                                    _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                    _SelectedIndex = 0;
                                    _Null = false;
                                }
                                _Initialized = true;
                            }

                            #endregion

                            #region New, Add, Delete, Remove Methods

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            public void _New()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with null values. If the reflexive 
                            /// does not exist in the current tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _New(FileStream fs)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write null, -1 for pointers
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(_Unknown2TagBlock template)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._New(fs, template);
                                fs.Close();
                            }

                            /// <summary>
                            /// Overwrites the current reflexive with the values provided
                            /// by the template. If the reflexive does not exist in the current 
                            /// tag meta, space will be made.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="template">The template containing the new values. </param>
                            public void _New(FileStream fs, _Unknown2TagBlock template)
                            {
                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                if (_Null)
                                {
                                    // Make space
                                }

                                // Write template
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            public void _Add()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be made up of null values.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Add(FileStream fs)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make null space
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Add(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                            /// be based off of the values found within the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                            public void _Add(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Make space, fill with template
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            public void _Delete()
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Delete(fs);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the reflexive if it exists in the current tag meta.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            public void _Delete(FileStream fs)
                            {
                                if (_Null) return;

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Delete

                                _Null = true;
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(int index)
                            {
                                FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                                this._Remove(fs, index);
                                fs.Close();
                            }

                            /// <summary>
                            /// Removes the chunk at the given index.
                            /// </summary>
                            /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                            /// <param name="index">The index of the chunk to remove. </param>
                            public void _Remove(FileStream fs, int index)
                            {
                                if (_Null) throw new Exception("Reflexive does not exist.");

                                BitReader br = new BitReader(fs, Endian.Little);
                                BitWriter bw = new BitWriter(fs, Endian.Little);

                                // Remove

                                if (--_Count == 0)
                                {
                                    // Write -1 pointer
                                    _Null = true;
                                }
                            }

                            #endregion
                        }

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _ObjectTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _ObjectTagBlock(JmadLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                            FullObjectAnimations = new _FullObjectAnimationsTagBlock(layout);
                            Half_UpperObjectAnimations = new _Half_UpperObjectAnimationsTagBlock(layout);
                            Death_Damage = new _Death_DamageTagBlock(layout);
                            AI = new _AITagBlock(layout);
                            Unknown1 = new _Unknown1TagBlock(layout);
                            Unknown2 = new _Unknown2TagBlock(layout);
                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_ObjectTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _ObjectTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 3. TagBlock :: Offset: 0xC (12)
                    /// </summary>
                    public _LimbPositioningTagBlock LimbPositioning { get { return __limbPositioning__; } set { __limbPositioning__ = value; } }
                    private _LimbPositioningTagBlock __limbPositioning__;
                    public class _LimbPositioningTagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private JmadLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. StringID :: Offset: 0x0 (0)
                        /// </summary>
                        public StringID LocalMarker { get { return __localMarker__; } set { __localMarker__ = value; } }
                        private StringID __localMarker__;
                        /// <summary>
                        /// 2. StringID :: Offset: 0x4 (4)
                        /// </summary>
                        public StringID RemoteMarker { get { return __remoteMarker__; } set { __remoteMarker__ = value; } }
                        private StringID __remoteMarker__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _LimbPositioningTagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _LimbPositioningTagBlock(JmadLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_LimbPositioningTagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _LimbPositioningTagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _ClassTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _ClassTagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Object = new _ObjectTagBlock(layout);
                        LimbPositioning = new _LimbPositioningTagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_ClassTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _ClassTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 3. TagBlock :: Offset: 0xC (12)
                /// </summary>
                public _LimbPositioningTagBlock LimbPositioning { get { return __limbPositioning__; } set { __limbPositioning__ = value; } }
                private _LimbPositioningTagBlock __limbPositioning__;
                public class _LimbPositioningTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. StringID :: Offset: 0x0 (0)
                    /// </summary>
                    public StringID LocalMarker { get { return __localMarker__; } set { __localMarker__ = value; } }
                    private StringID __localMarker__;
                    /// <summary>
                    /// 2. StringID :: Offset: 0x4 (4)
                    /// </summary>
                    public StringID RemoteMarker { get { return __remoteMarker__; } set { __remoteMarker__ = value; } }
                    private StringID __remoteMarker__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _LimbPositioningTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _LimbPositioningTagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_LimbPositioningTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _LimbPositioningTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AnimationCallsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AnimationCallsTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Class = new _ClassTagBlock(layout);
                    LimbPositioning = new _LimbPositioningTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AnimationCallsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AnimationCallsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 9. TagBlock :: Offset: 0x3C (60)
            /// </summary>
            public _VehicleSuspensionTagBlock VehicleSuspension { get { return __vehicleSuspension__; } set { __vehicleSuspension__ = value; } }
            private _VehicleSuspensionTagBlock __vehicleSuspension__;
            public class _VehicleSuspensionTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID SuspensionPart { get { return __suspensionPart__; } set { __suspensionPart__ = value; } }
                private StringID __suspensionPart__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private StringID __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 7. StringID :: Offset: 0x18 (24)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 8. ByteArray :: Offset: 0x1C (28)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _VehicleSuspensionTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _VehicleSuspensionTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_VehicleSuspensionTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _VehicleSuspensionTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 10. TagBlock :: Offset: 0x44 (68)
            /// </summary>
            public _Unknown3TagBlock Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private _Unknown3TagBlock __unknown3__;
            public class _Unknown3TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private StringID __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private StringID __unknown3__;
                /// <summary>
                /// 5. ByteArray :: Offset: 0x10 (16)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown3TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown3TagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown3TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown3TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 11. TagBlock :: Offset: 0x4C (76)
            /// </summary>
            public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private _Unknown4TagBlock __unknown4__;
            public class _Unknown4TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private TagRef __unknown1__;
                /// <summary>
                /// 2. TagBlock :: Offset: 0x8 (8)
                /// </summary>
                public _Unknown2TagBlock Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private _Unknown2TagBlock __unknown2__;
                public class _Unknown2TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private short __unknown__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown2TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown2TagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown2TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown2TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 3. TagBlock :: Offset: 0x10 (16)
                /// </summary>
                public _Unknown3TagBlock Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private _Unknown3TagBlock __unknown3__;
                public class _Unknown3TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private JmadLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private float __unknown__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown3TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown3TagBlock(JmadLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown3TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown3TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 4. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown4TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown4TagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Unknown2 = new _Unknown2TagBlock(layout);
                    Unknown3 = new _Unknown3TagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown4TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown4TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 12. TagBlock :: Offset: 0x54 (84)
            /// </summary>
            public _WeaponAssociationsTagBlock WeaponAssociations { get { return __weaponAssociations__; } set { __weaponAssociations__ = value; } }
            private _WeaponAssociationsTagBlock __weaponAssociations__;
            public class _WeaponAssociationsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Short { get { return __short__; } set { __short__ = value; } }
                private StringID __short__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Class { get { return __class__; } set { __class__ = value; } }
                private StringID __class__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WeaponAssociationsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WeaponAssociationsTagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WeaponAssociationsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WeaponAssociationsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 13. Float :: Offset: 0x5C (92)
            /// </summary>
            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private float __unknown5__;
            /// <summary>
            /// 14. Float :: Offset: 0x60 (96)
            /// </summary>
            public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private float __unknown6__;
            /// <summary>
            /// 15. ByteArray :: Offset: 0x64 (100)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 16. Float :: Offset: 0x7C (124)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;
            /// <summary>
            /// 17. Float :: Offset: 0x80 (128)
            /// </summary>
            public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private float __unknown8__;
            /// <summary>
            /// 18. Float :: Offset: 0x84 (132)
            /// </summary>
            public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private float __unknown9__;
            /// <summary>
            /// 19. ByteArray :: Offset: 0x88 (136)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 20. TagBlock :: Offset: 0xAC (172)
            /// </summary>
            public _Unknown10TagBlock Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private _Unknown10TagBlock __unknown10__;
            public class _Unknown10TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ID :: Offset: 0x0 (0)
                /// </summary>
                public LoneID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private LoneID __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float RawSize { get { return __rawSize__; } set { __rawSize__ = value; } }
                private float __rawSize__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float RawOffset { get { return __rawOffset__; } set { __rawOffset__ = value; } }
                private float __rawOffset__;
                /// <summary>
                /// 4. ByteArray :: Offset: 0xC (12)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown10TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown10TagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown10TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown10TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 21. TagBlock :: Offset: 0xB4 (180)
            /// </summary>
            public _Unknown11TagBlock Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private _Unknown11TagBlock __unknown11__;
            public class _Unknown11TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public JmadLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private JmadLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown11TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown11TagBlock(JmadLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown11TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown11TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the JmadLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public JmadLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Bones = new _BonesTagBlock(this);
                Sounds = new _SoundsTagBlock(this);
                Effects = new _EffectsTagBlock(this);
                ScreenBounds = new _ScreenBoundsTagBlock(this);
                Animations = new _AnimationsTagBlock(this);
                AnimationCalls = new _AnimationCallsTagBlock(this);
                VehicleSuspension = new _VehicleSuspensionTagBlock(this);
                Unknown3 = new _Unknown3TagBlock(this);
                Unknown4 = new _Unknown4TagBlock(this);
                WeaponAssociations = new _WeaponAssociationsTagBlock(this);
                Unknown10 = new _Unknown10TagBlock(this);
                Unknown11 = new _Unknown11TagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unknown1._Initialize(br);
                Unknown2 = br.ReadSingle();
                Bones._Initialize(br);
                Sounds._Initialize(br);
                Effects._Initialize(br);
                ScreenBounds._Initialize(br);
                Animations._Initialize(br);
                AnimationCalls._Initialize(br);
                VehicleSuspension._Initialize(br);
                Unknown3._Initialize(br);
                Unknown4._Initialize(br);
                WeaponAssociations._Initialize(br);
                Unknown5 = br.ReadSingle();
                Unknown6 = br.ReadSingle();
                Unused1 = br.ReadBytes(24);
                Unknown7 = br.ReadSingle();
                Unknown8 = br.ReadSingle();
                Unknown9 = br.ReadSingle();
                Unused2 = br.ReadBytes(36);
                Unknown10._Initialize(br);
                Unknown11._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Jmad class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Jmad(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new JmadLayout(map, identity);
        }

        #region Specific Methods of jmad

        #endregion
    }
}
