using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Lightmap
    /// </summary>
    public class Ltmp
    {
        #region Constants

        /// <summary>
        /// Size of the ltmp meta header.
        /// </summary>
        public const int HeaderSize = 260;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private LtmpLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public LtmpLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class LtmpLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Float :: Offset: 0x0 (0)
            /// </summary>
            public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private float __unknown1__;
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private float __unknown3__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private float __unknown4__;
            /// <summary>
            /// 5. ByteArray :: Offset: 0x10 (16)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private float __unknown5__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private float __unknown6__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;
            /// <summary>
            /// 9. Float :: Offset: 0x20 (32)
            /// </summary>
            public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private float __unknown8__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private float __unknown9__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private float __unknown10__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private float __unknown11__;
            /// <summary>
            /// 13. Float :: Offset: 0x30 (48)
            /// </summary>
            public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private float __unknown12__;
            /// <summary>
            /// 14. ByteArray :: Offset: 0x34 (52)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 15. TagBlock :: Offset: 0x80 (128)
            /// </summary>
            public _Unknown13TagBlock Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private _Unknown13TagBlock __unknown13__;
            public class _Unknown13TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private LtmpLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. TagBlock :: Offset: 0x8 (8)
                /// </summary>
                public _Unknown3TagBlock Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private _Unknown3TagBlock __unknown3__;
                public class _Unknown3TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x1C (28)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;
                    /// <summary>
                    /// 9. Float :: Offset: 0x20 (32)
                    /// </summary>
                    public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                    private float __unknown9__;
                    /// <summary>
                    /// 10. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                    private float __unknown10__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                    private float __unknown11__;
                    /// <summary>
                    /// 12. Float :: Offset: 0x2C (44)
                    /// </summary>
                    public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                    private float __unknown12__;
                    /// <summary>
                    /// 13. Float :: Offset: 0x30 (48)
                    /// </summary>
                    public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                    private float __unknown13__;
                    /// <summary>
                    /// 14. Float :: Offset: 0x34 (52)
                    /// </summary>
                    public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                    private float __unknown14__;
                    /// <summary>
                    /// 15. Float :: Offset: 0x38 (56)
                    /// </summary>
                    public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                    private float __unknown15__;
                    /// <summary>
                    /// 16. Float :: Offset: 0x3C (60)
                    /// </summary>
                    public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                    private float __unknown16__;
                    /// <summary>
                    /// 17. Float :: Offset: 0x40 (64)
                    /// </summary>
                    public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                    private float __unknown17__;
                    /// <summary>
                    /// 18. Float :: Offset: 0x44 (68)
                    /// </summary>
                    public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                    private float __unknown18__;
                    /// <summary>
                    /// 19. Float :: Offset: 0x48 (72)
                    /// </summary>
                    public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                    private float __unknown19__;
                    /// <summary>
                    /// 20. Float :: Offset: 0x4C (76)
                    /// </summary>
                    public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                    private float __unknown20__;
                    /// <summary>
                    /// 21. Float :: Offset: 0x50 (80)
                    /// </summary>
                    public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                    private float __unknown21__;
                    /// <summary>
                    /// 22. Float :: Offset: 0x54 (84)
                    /// </summary>
                    public float Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                    private float __unknown22__;
                    /// <summary>
                    /// 23. Float :: Offset: 0x58 (88)
                    /// </summary>
                    public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                    private float __unknown23__;
                    /// <summary>
                    /// 24. Float :: Offset: 0x5C (92)
                    /// </summary>
                    public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                    private float __unknown24__;
                    /// <summary>
                    /// 25. Float :: Offset: 0x60 (96)
                    /// </summary>
                    public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                    private float __unknown25__;
                    /// <summary>
                    /// 26. Float :: Offset: 0x64 (100)
                    /// </summary>
                    public float Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                    private float __unknown26__;
                    /// <summary>
                    /// 27. Float :: Offset: 0x68 (104)
                    /// </summary>
                    public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                    private float __unknown27__;
                    /// <summary>
                    /// 28. Float :: Offset: 0x6C (108)
                    /// </summary>
                    public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                    private float __unknown28__;
                    /// <summary>
                    /// 29. Float :: Offset: 0x70 (112)
                    /// </summary>
                    public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                    private float __unknown29__;
                    /// <summary>
                    /// 30. Float :: Offset: 0x74 (116)
                    /// </summary>
                    public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                    private float __unknown30__;
                    /// <summary>
                    /// 31. Float :: Offset: 0x78 (120)
                    /// </summary>
                    public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
                    private float __unknown31__;
                    /// <summary>
                    /// 32. Float :: Offset: 0x7C (124)
                    /// </summary>
                    public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
                    private float __unknown32__;
                    /// <summary>
                    /// 33. Float :: Offset: 0x80 (128)
                    /// </summary>
                    public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
                    private float __unknown33__;
                    /// <summary>
                    /// 34. Float :: Offset: 0x84 (132)
                    /// </summary>
                    public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
                    private float __unknown34__;
                    /// <summary>
                    /// 35. Float :: Offset: 0x88 (136)
                    /// </summary>
                    public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
                    private float __unknown35__;
                    /// <summary>
                    /// 36. Float :: Offset: 0x8C (140)
                    /// </summary>
                    public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
                    private float __unknown36__;
                    /// <summary>
                    /// 37. Float :: Offset: 0x90 (144)
                    /// </summary>
                    public float Unknown37 { get { return __unknown37__; } set { __unknown37__ = value; } }
                    private float __unknown37__;
                    /// <summary>
                    /// 38. Float :: Offset: 0x94 (148)
                    /// </summary>
                    public float Unknown38 { get { return __unknown38__; } set { __unknown38__ = value; } }
                    private float __unknown38__;
                    /// <summary>
                    /// 39. Float :: Offset: 0x98 (152)
                    /// </summary>
                    public float Unknown39 { get { return __unknown39__; } set { __unknown39__ = value; } }
                    private float __unknown39__;
                    /// <summary>
                    /// 40. Float :: Offset: 0x9C (156)
                    /// </summary>
                    public float Unknown40 { get { return __unknown40__; } set { __unknown40__ = value; } }
                    private float __unknown40__;
                    /// <summary>
                    /// 41. Float :: Offset: 0xA0 (160)
                    /// </summary>
                    public float Unknown41 { get { return __unknown41__; } set { __unknown41__ = value; } }
                    private float __unknown41__;
                    /// <summary>
                    /// 42. Float :: Offset: 0xA4 (164)
                    /// </summary>
                    public float Unknown42 { get { return __unknown42__; } set { __unknown42__ = value; } }
                    private float __unknown42__;
                    /// <summary>
                    /// 43. Float :: Offset: 0xA8 (168)
                    /// </summary>
                    public float Unknown43 { get { return __unknown43__; } set { __unknown43__ = value; } }
                    private float __unknown43__;
                    /// <summary>
                    /// 44. Float :: Offset: 0xAC (172)
                    /// </summary>
                    public float Unknown44 { get { return __unknown44__; } set { __unknown44__ = value; } }
                    private float __unknown44__;
                    /// <summary>
                    /// 45. Float :: Offset: 0xB0 (176)
                    /// </summary>
                    public float Unknown45 { get { return __unknown45__; } set { __unknown45__ = value; } }
                    private float __unknown45__;
                    /// <summary>
                    /// 46. Float :: Offset: 0xB4 (180)
                    /// </summary>
                    public float Unknown46 { get { return __unknown46__; } set { __unknown46__ = value; } }
                    private float __unknown46__;
                    /// <summary>
                    /// 47. Float :: Offset: 0xB8 (184)
                    /// </summary>
                    public float Unknown47 { get { return __unknown47__; } set { __unknown47__ = value; } }
                    private float __unknown47__;
                    /// <summary>
                    /// 48. Float :: Offset: 0xBC (188)
                    /// </summary>
                    public float Unknown48 { get { return __unknown48__; } set { __unknown48__ = value; } }
                    private float __unknown48__;
                    /// <summary>
                    /// 49. Float :: Offset: 0xC0 (192)
                    /// </summary>
                    public float Unknown49 { get { return __unknown49__; } set { __unknown49__ = value; } }
                    private float __unknown49__;
                    /// <summary>
                    /// 50. Float :: Offset: 0xC4 (196)
                    /// </summary>
                    public float Unknown50 { get { return __unknown50__; } set { __unknown50__ = value; } }
                    private float __unknown50__;
                    /// <summary>
                    /// 51. Float :: Offset: 0xC8 (200)
                    /// </summary>
                    public float Unknown51 { get { return __unknown51__; } set { __unknown51__ = value; } }
                    private float __unknown51__;
                    /// <summary>
                    /// 52. Float :: Offset: 0xCC (204)
                    /// </summary>
                    public float Unknown52 { get { return __unknown52__; } set { __unknown52__ = value; } }
                    private float __unknown52__;
                    /// <summary>
                    /// 53. Float :: Offset: 0xD0 (208)
                    /// </summary>
                    public float Unknown53 { get { return __unknown53__; } set { __unknown53__ = value; } }
                    private float __unknown53__;
                    /// <summary>
                    /// 54. Float :: Offset: 0xD4 (212)
                    /// </summary>
                    public float Unknown54 { get { return __unknown54__; } set { __unknown54__ = value; } }
                    private float __unknown54__;
                    /// <summary>
                    /// 55. Float :: Offset: 0xD8 (216)
                    /// </summary>
                    public float Unknown55 { get { return __unknown55__; } set { __unknown55__ = value; } }
                    private float __unknown55__;
                    /// <summary>
                    /// 56. Float :: Offset: 0xDC (220)
                    /// </summary>
                    public float Unknown56 { get { return __unknown56__; } set { __unknown56__ = value; } }
                    private float __unknown56__;
                    /// <summary>
                    /// 57. Float :: Offset: 0xE0 (224)
                    /// </summary>
                    public float Unknown57 { get { return __unknown57__; } set { __unknown57__ = value; } }
                    private float __unknown57__;
                    /// <summary>
                    /// 58. Float :: Offset: 0xE4 (228)
                    /// </summary>
                    public float Unknown58 { get { return __unknown58__; } set { __unknown58__ = value; } }
                    private float __unknown58__;
                    /// <summary>
                    /// 59. Float :: Offset: 0xE8 (232)
                    /// </summary>
                    public float Unknown59 { get { return __unknown59__; } set { __unknown59__ = value; } }
                    private float __unknown59__;
                    /// <summary>
                    /// 60. Float :: Offset: 0xEC (236)
                    /// </summary>
                    public float Unknown60 { get { return __unknown60__; } set { __unknown60__ = value; } }
                    private float __unknown60__;
                    /// <summary>
                    /// 61. Float :: Offset: 0xF0 (240)
                    /// </summary>
                    public float Unknown61 { get { return __unknown61__; } set { __unknown61__ = value; } }
                    private float __unknown61__;
                    /// <summary>
                    /// 62. Float :: Offset: 0xF4 (244)
                    /// </summary>
                    public float Unknown62 { get { return __unknown62__; } set { __unknown62__ = value; } }
                    private float __unknown62__;
                    /// <summary>
                    /// 63. Float :: Offset: 0xF8 (248)
                    /// </summary>
                    public float Unknown63 { get { return __unknown63__; } set { __unknown63__ = value; } }
                    private float __unknown63__;
                    /// <summary>
                    /// 64. Float :: Offset: 0xFC (252)
                    /// </summary>
                    public float Unknown64 { get { return __unknown64__; } set { __unknown64__ = value; } }
                    private float __unknown64__;
                    /// <summary>
                    /// 65. Float :: Offset: 0x100 (256)
                    /// </summary>
                    public float Unknown65 { get { return __unknown65__; } set { __unknown65__ = value; } }
                    private float __unknown65__;
                    /// <summary>
                    /// 66. Float :: Offset: 0x104 (260)
                    /// </summary>
                    public float Unknown66 { get { return __unknown66__; } set { __unknown66__ = value; } }
                    private float __unknown66__;
                    /// <summary>
                    /// 67. Float :: Offset: 0x108 (264)
                    /// </summary>
                    public float Unknown67 { get { return __unknown67__; } set { __unknown67__ = value; } }
                    private float __unknown67__;
                    /// <summary>
                    /// 68. Float :: Offset: 0x10C (268)
                    /// </summary>
                    public float Unknown68 { get { return __unknown68__; } set { __unknown68__ = value; } }
                    private float __unknown68__;
                    /// <summary>
                    /// 69. Float :: Offset: 0x110 (272)
                    /// </summary>
                    public float Unknown69 { get { return __unknown69__; } set { __unknown69__ = value; } }
                    private float __unknown69__;
                    /// <summary>
                    /// 70. Float :: Offset: 0x114 (276)
                    /// </summary>
                    public float Unknown70 { get { return __unknown70__; } set { __unknown70__ = value; } }
                    private float __unknown70__;
                    /// <summary>
                    /// 71. Float :: Offset: 0x118 (280)
                    /// </summary>
                    public float Unknown71 { get { return __unknown71__; } set { __unknown71__ = value; } }
                    private float __unknown71__;
                    /// <summary>
                    /// 72. Float :: Offset: 0x11C (284)
                    /// </summary>
                    public float Unknown72 { get { return __unknown72__; } set { __unknown72__ = value; } }
                    private float __unknown72__;
                    /// <summary>
                    /// 73. Float :: Offset: 0x120 (288)
                    /// </summary>
                    public float Unknown73 { get { return __unknown73__; } set { __unknown73__ = value; } }
                    private float __unknown73__;
                    /// <summary>
                    /// 74. Float :: Offset: 0x124 (292)
                    /// </summary>
                    public float Unknown74 { get { return __unknown74__; } set { __unknown74__ = value; } }
                    private float __unknown74__;
                    /// <summary>
                    /// 75. Float :: Offset: 0x128 (296)
                    /// </summary>
                    public float Unknown75 { get { return __unknown75__; } set { __unknown75__ = value; } }
                    private float __unknown75__;
                    /// <summary>
                    /// 76. Float :: Offset: 0x12C (300)
                    /// </summary>
                    public float Unknown76 { get { return __unknown76__; } set { __unknown76__ = value; } }
                    private float __unknown76__;
                    /// <summary>
                    /// 77. Float :: Offset: 0x130 (304)
                    /// </summary>
                    public float Unknown77 { get { return __unknown77__; } set { __unknown77__ = value; } }
                    private float __unknown77__;
                    /// <summary>
                    /// 78. Float :: Offset: 0x134 (308)
                    /// </summary>
                    public float Unknown78 { get { return __unknown78__; } set { __unknown78__ = value; } }
                    private float __unknown78__;
                    /// <summary>
                    /// 79. Float :: Offset: 0x138 (312)
                    /// </summary>
                    public float Unknown79 { get { return __unknown79__; } set { __unknown79__ = value; } }
                    private float __unknown79__;
                    /// <summary>
                    /// 80. Float :: Offset: 0x13C (316)
                    /// </summary>
                    public float Unknown80 { get { return __unknown80__; } set { __unknown80__ = value; } }
                    private float __unknown80__;
                    /// <summary>
                    /// 81. Float :: Offset: 0x140 (320)
                    /// </summary>
                    public float Unknown81 { get { return __unknown81__; } set { __unknown81__ = value; } }
                    private float __unknown81__;
                    /// <summary>
                    /// 82. Float :: Offset: 0x144 (324)
                    /// </summary>
                    public float Unknown82 { get { return __unknown82__; } set { __unknown82__ = value; } }
                    private float __unknown82__;
                    /// <summary>
                    /// 83. Float :: Offset: 0x148 (328)
                    /// </summary>
                    public float Unknown83 { get { return __unknown83__; } set { __unknown83__ = value; } }
                    private float __unknown83__;
                    /// <summary>
                    /// 84. Float :: Offset: 0x14C (332)
                    /// </summary>
                    public float Unknown84 { get { return __unknown84__; } set { __unknown84__ = value; } }
                    private float __unknown84__;
                    /// <summary>
                    /// 85. Float :: Offset: 0x150 (336)
                    /// </summary>
                    public float Unknown85 { get { return __unknown85__; } set { __unknown85__ = value; } }
                    private float __unknown85__;
                    /// <summary>
                    /// 86. Float :: Offset: 0x154 (340)
                    /// </summary>
                    public float Unknown86 { get { return __unknown86__; } set { __unknown86__ = value; } }
                    private float __unknown86__;
                    /// <summary>
                    /// 87. Float :: Offset: 0x158 (344)
                    /// </summary>
                    public float Unknown87 { get { return __unknown87__; } set { __unknown87__ = value; } }
                    private float __unknown87__;
                    /// <summary>
                    /// 88. Float :: Offset: 0x15C (348)
                    /// </summary>
                    public float Unknown88 { get { return __unknown88__; } set { __unknown88__ = value; } }
                    private float __unknown88__;
                    /// <summary>
                    /// 89. Float :: Offset: 0x160 (352)
                    /// </summary>
                    public float Unknown89 { get { return __unknown89__; } set { __unknown89__ = value; } }
                    private float __unknown89__;
                    /// <summary>
                    /// 90. Float :: Offset: 0x164 (356)
                    /// </summary>
                    public float Unknown90 { get { return __unknown90__; } set { __unknown90__ = value; } }
                    private float __unknown90__;
                    /// <summary>
                    /// 91. Float :: Offset: 0x168 (360)
                    /// </summary>
                    public float Unknown91 { get { return __unknown91__; } set { __unknown91__ = value; } }
                    private float __unknown91__;
                    /// <summary>
                    /// 92. Float :: Offset: 0x16C (364)
                    /// </summary>
                    public float Unknown92 { get { return __unknown92__; } set { __unknown92__ = value; } }
                    private float __unknown92__;
                    /// <summary>
                    /// 93. Float :: Offset: 0x170 (368)
                    /// </summary>
                    public float Unknown93 { get { return __unknown93__; } set { __unknown93__ = value; } }
                    private float __unknown93__;
                    /// <summary>
                    /// 94. Float :: Offset: 0x174 (372)
                    /// </summary>
                    public float Unknown94 { get { return __unknown94__; } set { __unknown94__ = value; } }
                    private float __unknown94__;
                    /// <summary>
                    /// 95. Float :: Offset: 0x178 (376)
                    /// </summary>
                    public float Unknown95 { get { return __unknown95__; } set { __unknown95__ = value; } }
                    private float __unknown95__;
                    /// <summary>
                    /// 96. Float :: Offset: 0x17C (380)
                    /// </summary>
                    public float Unknown96 { get { return __unknown96__; } set { __unknown96__ = value; } }
                    private float __unknown96__;
                    /// <summary>
                    /// 97. Float :: Offset: 0x180 (384)
                    /// </summary>
                    public float Unknown97 { get { return __unknown97__; } set { __unknown97__ = value; } }
                    private float __unknown97__;
                    /// <summary>
                    /// 98. Float :: Offset: 0x184 (388)
                    /// </summary>
                    public float Unknown98 { get { return __unknown98__; } set { __unknown98__ = value; } }
                    private float __unknown98__;
                    /// <summary>
                    /// 99. Float :: Offset: 0x188 (392)
                    /// </summary>
                    public float Unknown99 { get { return __unknown99__; } set { __unknown99__ = value; } }
                    private float __unknown99__;
                    /// <summary>
                    /// 100. Float :: Offset: 0x18C (396)
                    /// </summary>
                    public float Unknown100 { get { return __unknown100__; } set { __unknown100__ = value; } }
                    private float __unknown100__;
                    /// <summary>
                    /// 101. Float :: Offset: 0x190 (400)
                    /// </summary>
                    public float Unknown101 { get { return __unknown101__; } set { __unknown101__ = value; } }
                    private float __unknown101__;
                    /// <summary>
                    /// 102. Float :: Offset: 0x194 (404)
                    /// </summary>
                    public float Unknown102 { get { return __unknown102__; } set { __unknown102__ = value; } }
                    private float __unknown102__;
                    /// <summary>
                    /// 103. Float :: Offset: 0x198 (408)
                    /// </summary>
                    public float Unknown103 { get { return __unknown103__; } set { __unknown103__ = value; } }
                    private float __unknown103__;
                    /// <summary>
                    /// 104. Float :: Offset: 0x19C (412)
                    /// </summary>
                    public float Unknown104 { get { return __unknown104__; } set { __unknown104__ = value; } }
                    private float __unknown104__;
                    /// <summary>
                    /// 105. Float :: Offset: 0x1A0 (416)
                    /// </summary>
                    public float Unknown105 { get { return __unknown105__; } set { __unknown105__ = value; } }
                    private float __unknown105__;
                    /// <summary>
                    /// 106. Float :: Offset: 0x1A4 (420)
                    /// </summary>
                    public float Unknown106 { get { return __unknown106__; } set { __unknown106__ = value; } }
                    private float __unknown106__;
                    /// <summary>
                    /// 107. Float :: Offset: 0x1A8 (424)
                    /// </summary>
                    public float Unknown107 { get { return __unknown107__; } set { __unknown107__ = value; } }
                    private float __unknown107__;
                    /// <summary>
                    /// 108. Float :: Offset: 0x1AC (428)
                    /// </summary>
                    public float Unknown108 { get { return __unknown108__; } set { __unknown108__ = value; } }
                    private float __unknown108__;
                    /// <summary>
                    /// 109. Float :: Offset: 0x1B0 (432)
                    /// </summary>
                    public float Unknown109 { get { return __unknown109__; } set { __unknown109__ = value; } }
                    private float __unknown109__;
                    /// <summary>
                    /// 110. Float :: Offset: 0x1B4 (436)
                    /// </summary>
                    public float Unknown110 { get { return __unknown110__; } set { __unknown110__ = value; } }
                    private float __unknown110__;
                    /// <summary>
                    /// 111. Float :: Offset: 0x1B8 (440)
                    /// </summary>
                    public float Unknown111 { get { return __unknown111__; } set { __unknown111__ = value; } }
                    private float __unknown111__;
                    /// <summary>
                    /// 112. Float :: Offset: 0x1BC (444)
                    /// </summary>
                    public float Unknown112 { get { return __unknown112__; } set { __unknown112__ = value; } }
                    private float __unknown112__;
                    /// <summary>
                    /// 113. Float :: Offset: 0x1C0 (448)
                    /// </summary>
                    public float Unknown113 { get { return __unknown113__; } set { __unknown113__ = value; } }
                    private float __unknown113__;
                    /// <summary>
                    /// 114. Float :: Offset: 0x1C4 (452)
                    /// </summary>
                    public float Unknown114 { get { return __unknown114__; } set { __unknown114__ = value; } }
                    private float __unknown114__;
                    /// <summary>
                    /// 115. Float :: Offset: 0x1C8 (456)
                    /// </summary>
                    public float Unknown115 { get { return __unknown115__; } set { __unknown115__ = value; } }
                    private float __unknown115__;
                    /// <summary>
                    /// 116. Float :: Offset: 0x1CC (460)
                    /// </summary>
                    public float Unknown116 { get { return __unknown116__; } set { __unknown116__ = value; } }
                    private float __unknown116__;
                    /// <summary>
                    /// 117. Float :: Offset: 0x1D0 (464)
                    /// </summary>
                    public float Unknown117 { get { return __unknown117__; } set { __unknown117__ = value; } }
                    private float __unknown117__;
                    /// <summary>
                    /// 118. Float :: Offset: 0x1D4 (468)
                    /// </summary>
                    public float Unknown118 { get { return __unknown118__; } set { __unknown118__ = value; } }
                    private float __unknown118__;
                    /// <summary>
                    /// 119. Float :: Offset: 0x1D8 (472)
                    /// </summary>
                    public float Unknown119 { get { return __unknown119__; } set { __unknown119__ = value; } }
                    private float __unknown119__;
                    /// <summary>
                    /// 120. Float :: Offset: 0x1DC (476)
                    /// </summary>
                    public float Unknown120 { get { return __unknown120__; } set { __unknown120__ = value; } }
                    private float __unknown120__;
                    /// <summary>
                    /// 121. Float :: Offset: 0x1E0 (480)
                    /// </summary>
                    public float Unknown121 { get { return __unknown121__; } set { __unknown121__ = value; } }
                    private float __unknown121__;
                    /// <summary>
                    /// 122. Float :: Offset: 0x1E4 (484)
                    /// </summary>
                    public float Unknown122 { get { return __unknown122__; } set { __unknown122__ = value; } }
                    private float __unknown122__;
                    /// <summary>
                    /// 123. Float :: Offset: 0x1E8 (488)
                    /// </summary>
                    public float Unknown123 { get { return __unknown123__; } set { __unknown123__ = value; } }
                    private float __unknown123__;
                    /// <summary>
                    /// 124. Float :: Offset: 0x1EC (492)
                    /// </summary>
                    public float Unknown124 { get { return __unknown124__; } set { __unknown124__ = value; } }
                    private float __unknown124__;
                    /// <summary>
                    /// 125. Float :: Offset: 0x1F0 (496)
                    /// </summary>
                    public float Unknown125 { get { return __unknown125__; } set { __unknown125__ = value; } }
                    private float __unknown125__;
                    /// <summary>
                    /// 126. Float :: Offset: 0x1F4 (500)
                    /// </summary>
                    public float Unknown126 { get { return __unknown126__; } set { __unknown126__ = value; } }
                    private float __unknown126__;
                    /// <summary>
                    /// 127. Float :: Offset: 0x1F8 (504)
                    /// </summary>
                    public float Unknown127 { get { return __unknown127__; } set { __unknown127__ = value; } }
                    private float __unknown127__;
                    /// <summary>
                    /// 128. Float :: Offset: 0x1FC (508)
                    /// </summary>
                    public float Unknown128 { get { return __unknown128__; } set { __unknown128__ = value; } }
                    private float __unknown128__;
                    /// <summary>
                    /// 129. Float :: Offset: 0x200 (512)
                    /// </summary>
                    public float Unknown129 { get { return __unknown129__; } set { __unknown129__ = value; } }
                    private float __unknown129__;
                    /// <summary>
                    /// 130. Float :: Offset: 0x204 (516)
                    /// </summary>
                    public float Unknown130 { get { return __unknown130__; } set { __unknown130__ = value; } }
                    private float __unknown130__;
                    /// <summary>
                    /// 131. Float :: Offset: 0x208 (520)
                    /// </summary>
                    public float Unknown131 { get { return __unknown131__; } set { __unknown131__ = value; } }
                    private float __unknown131__;
                    /// <summary>
                    /// 132. Float :: Offset: 0x20C (524)
                    /// </summary>
                    public float Unknown132 { get { return __unknown132__; } set { __unknown132__ = value; } }
                    private float __unknown132__;
                    /// <summary>
                    /// 133. Float :: Offset: 0x210 (528)
                    /// </summary>
                    public float Unknown133 { get { return __unknown133__; } set { __unknown133__ = value; } }
                    private float __unknown133__;
                    /// <summary>
                    /// 134. Float :: Offset: 0x214 (532)
                    /// </summary>
                    public float Unknown134 { get { return __unknown134__; } set { __unknown134__ = value; } }
                    private float __unknown134__;
                    /// <summary>
                    /// 135. Float :: Offset: 0x218 (536)
                    /// </summary>
                    public float Unknown135 { get { return __unknown135__; } set { __unknown135__ = value; } }
                    private float __unknown135__;
                    /// <summary>
                    /// 136. Float :: Offset: 0x21C (540)
                    /// </summary>
                    public float Unknown136 { get { return __unknown136__; } set { __unknown136__ = value; } }
                    private float __unknown136__;
                    /// <summary>
                    /// 137. Float :: Offset: 0x220 (544)
                    /// </summary>
                    public float Unknown137 { get { return __unknown137__; } set { __unknown137__ = value; } }
                    private float __unknown137__;
                    /// <summary>
                    /// 138. Float :: Offset: 0x224 (548)
                    /// </summary>
                    public float Unknown138 { get { return __unknown138__; } set { __unknown138__ = value; } }
                    private float __unknown138__;
                    /// <summary>
                    /// 139. Float :: Offset: 0x228 (552)
                    /// </summary>
                    public float Unknown139 { get { return __unknown139__; } set { __unknown139__ = value; } }
                    private float __unknown139__;
                    /// <summary>
                    /// 140. Float :: Offset: 0x22C (556)
                    /// </summary>
                    public float Unknown140 { get { return __unknown140__; } set { __unknown140__ = value; } }
                    private float __unknown140__;
                    /// <summary>
                    /// 141. Float :: Offset: 0x230 (560)
                    /// </summary>
                    public float Unknown141 { get { return __unknown141__; } set { __unknown141__ = value; } }
                    private float __unknown141__;
                    /// <summary>
                    /// 142. Float :: Offset: 0x234 (564)
                    /// </summary>
                    public float Unknown142 { get { return __unknown142__; } set { __unknown142__ = value; } }
                    private float __unknown142__;
                    /// <summary>
                    /// 143. Float :: Offset: 0x238 (568)
                    /// </summary>
                    public float Unknown143 { get { return __unknown143__; } set { __unknown143__ = value; } }
                    private float __unknown143__;
                    /// <summary>
                    /// 144. Float :: Offset: 0x23C (572)
                    /// </summary>
                    public float Unknown144 { get { return __unknown144__; } set { __unknown144__ = value; } }
                    private float __unknown144__;
                    /// <summary>
                    /// 145. Float :: Offset: 0x240 (576)
                    /// </summary>
                    public float Unknown145 { get { return __unknown145__; } set { __unknown145__ = value; } }
                    private float __unknown145__;
                    /// <summary>
                    /// 146. Float :: Offset: 0x244 (580)
                    /// </summary>
                    public float Unknown146 { get { return __unknown146__; } set { __unknown146__ = value; } }
                    private float __unknown146__;
                    /// <summary>
                    /// 147. Float :: Offset: 0x248 (584)
                    /// </summary>
                    public float Unknown147 { get { return __unknown147__; } set { __unknown147__ = value; } }
                    private float __unknown147__;
                    /// <summary>
                    /// 148. Float :: Offset: 0x24C (588)
                    /// </summary>
                    public float Unknown148 { get { return __unknown148__; } set { __unknown148__ = value; } }
                    private float __unknown148__;
                    /// <summary>
                    /// 149. Float :: Offset: 0x250 (592)
                    /// </summary>
                    public float Unknown149 { get { return __unknown149__; } set { __unknown149__ = value; } }
                    private float __unknown149__;
                    /// <summary>
                    /// 150. Float :: Offset: 0x254 (596)
                    /// </summary>
                    public float Unknown150 { get { return __unknown150__; } set { __unknown150__ = value; } }
                    private float __unknown150__;
                    /// <summary>
                    /// 151. Float :: Offset: 0x258 (600)
                    /// </summary>
                    public float Unknown151 { get { return __unknown151__; } set { __unknown151__ = value; } }
                    private float __unknown151__;
                    /// <summary>
                    /// 152. Float :: Offset: 0x25C (604)
                    /// </summary>
                    public float Unknown152 { get { return __unknown152__; } set { __unknown152__ = value; } }
                    private float __unknown152__;
                    /// <summary>
                    /// 153. Float :: Offset: 0x260 (608)
                    /// </summary>
                    public float Unknown153 { get { return __unknown153__; } set { __unknown153__ = value; } }
                    private float __unknown153__;
                    /// <summary>
                    /// 154. Float :: Offset: 0x264 (612)
                    /// </summary>
                    public float Unknown154 { get { return __unknown154__; } set { __unknown154__ = value; } }
                    private float __unknown154__;
                    /// <summary>
                    /// 155. Float :: Offset: 0x268 (616)
                    /// </summary>
                    public float Unknown155 { get { return __unknown155__; } set { __unknown155__ = value; } }
                    private float __unknown155__;
                    /// <summary>
                    /// 156. Float :: Offset: 0x26C (620)
                    /// </summary>
                    public float Unknown156 { get { return __unknown156__; } set { __unknown156__ = value; } }
                    private float __unknown156__;
                    /// <summary>
                    /// 157. Float :: Offset: 0x270 (624)
                    /// </summary>
                    public float Unknown157 { get { return __unknown157__; } set { __unknown157__ = value; } }
                    private float __unknown157__;
                    /// <summary>
                    /// 158. Float :: Offset: 0x274 (628)
                    /// </summary>
                    public float Unknown158 { get { return __unknown158__; } set { __unknown158__ = value; } }
                    private float __unknown158__;
                    /// <summary>
                    /// 159. Float :: Offset: 0x278 (632)
                    /// </summary>
                    public float Unknown159 { get { return __unknown159__; } set { __unknown159__ = value; } }
                    private float __unknown159__;
                    /// <summary>
                    /// 160. Float :: Offset: 0x27C (636)
                    /// </summary>
                    public float Unknown160 { get { return __unknown160__; } set { __unknown160__ = value; } }
                    private float __unknown160__;
                    /// <summary>
                    /// 161. Float :: Offset: 0x280 (640)
                    /// </summary>
                    public float Unknown161 { get { return __unknown161__; } set { __unknown161__ = value; } }
                    private float __unknown161__;
                    /// <summary>
                    /// 162. Float :: Offset: 0x284 (644)
                    /// </summary>
                    public float Unknown162 { get { return __unknown162__; } set { __unknown162__ = value; } }
                    private float __unknown162__;
                    /// <summary>
                    /// 163. Float :: Offset: 0x288 (648)
                    /// </summary>
                    public float Unknown163 { get { return __unknown163__; } set { __unknown163__ = value; } }
                    private float __unknown163__;
                    /// <summary>
                    /// 164. Float :: Offset: 0x28C (652)
                    /// </summary>
                    public float Unknown164 { get { return __unknown164__; } set { __unknown164__ = value; } }
                    private float __unknown164__;
                    /// <summary>
                    /// 165. Float :: Offset: 0x290 (656)
                    /// </summary>
                    public float Unknown165 { get { return __unknown165__; } set { __unknown165__ = value; } }
                    private float __unknown165__;
                    /// <summary>
                    /// 166. Float :: Offset: 0x294 (660)
                    /// </summary>
                    public float Unknown166 { get { return __unknown166__; } set { __unknown166__ = value; } }
                    private float __unknown166__;
                    /// <summary>
                    /// 167. Float :: Offset: 0x298 (664)
                    /// </summary>
                    public float Unknown167 { get { return __unknown167__; } set { __unknown167__ = value; } }
                    private float __unknown167__;
                    /// <summary>
                    /// 168. Float :: Offset: 0x29C (668)
                    /// </summary>
                    public float Unknown168 { get { return __unknown168__; } set { __unknown168__ = value; } }
                    private float __unknown168__;
                    /// <summary>
                    /// 169. Float :: Offset: 0x2A0 (672)
                    /// </summary>
                    public float Unknown169 { get { return __unknown169__; } set { __unknown169__ = value; } }
                    private float __unknown169__;
                    /// <summary>
                    /// 170. Float :: Offset: 0x2A4 (676)
                    /// </summary>
                    public float Unknown170 { get { return __unknown170__; } set { __unknown170__ = value; } }
                    private float __unknown170__;
                    /// <summary>
                    /// 171. Float :: Offset: 0x2A8 (680)
                    /// </summary>
                    public float Unknown171 { get { return __unknown171__; } set { __unknown171__ = value; } }
                    private float __unknown171__;
                    /// <summary>
                    /// 172. Float :: Offset: 0x2AC (684)
                    /// </summary>
                    public float Unknown172 { get { return __unknown172__; } set { __unknown172__ = value; } }
                    private float __unknown172__;
                    /// <summary>
                    /// 173. Float :: Offset: 0x2B0 (688)
                    /// </summary>
                    public float Unknown173 { get { return __unknown173__; } set { __unknown173__ = value; } }
                    private float __unknown173__;
                    /// <summary>
                    /// 174. Float :: Offset: 0x2B4 (692)
                    /// </summary>
                    public float Unknown174 { get { return __unknown174__; } set { __unknown174__ = value; } }
                    private float __unknown174__;
                    /// <summary>
                    /// 175. Float :: Offset: 0x2B8 (696)
                    /// </summary>
                    public float Unknown175 { get { return __unknown175__; } set { __unknown175__ = value; } }
                    private float __unknown175__;
                    /// <summary>
                    /// 176. Float :: Offset: 0x2BC (700)
                    /// </summary>
                    public float Unknown176 { get { return __unknown176__; } set { __unknown176__ = value; } }
                    private float __unknown176__;
                    /// <summary>
                    /// 177. Float :: Offset: 0x2C0 (704)
                    /// </summary>
                    public float Unknown177 { get { return __unknown177__; } set { __unknown177__ = value; } }
                    private float __unknown177__;
                    /// <summary>
                    /// 178. Float :: Offset: 0x2C4 (708)
                    /// </summary>
                    public float Unknown178 { get { return __unknown178__; } set { __unknown178__ = value; } }
                    private float __unknown178__;
                    /// <summary>
                    /// 179. Float :: Offset: 0x2C8 (712)
                    /// </summary>
                    public float Unknown179 { get { return __unknown179__; } set { __unknown179__ = value; } }
                    private float __unknown179__;
                    /// <summary>
                    /// 180. Float :: Offset: 0x2CC (716)
                    /// </summary>
                    public float Unknown180 { get { return __unknown180__; } set { __unknown180__ = value; } }
                    private float __unknown180__;
                    /// <summary>
                    /// 181. Float :: Offset: 0x2D0 (720)
                    /// </summary>
                    public float Unknown181 { get { return __unknown181__; } set { __unknown181__ = value; } }
                    private float __unknown181__;
                    /// <summary>
                    /// 182. Float :: Offset: 0x2D4 (724)
                    /// </summary>
                    public float Unknown182 { get { return __unknown182__; } set { __unknown182__ = value; } }
                    private float __unknown182__;
                    /// <summary>
                    /// 183. Float :: Offset: 0x2D8 (728)
                    /// </summary>
                    public float Unknown183 { get { return __unknown183__; } set { __unknown183__ = value; } }
                    private float __unknown183__;
                    /// <summary>
                    /// 184. Float :: Offset: 0x2DC (732)
                    /// </summary>
                    public float Unknown184 { get { return __unknown184__; } set { __unknown184__ = value; } }
                    private float __unknown184__;
                    /// <summary>
                    /// 185. Float :: Offset: 0x2E0 (736)
                    /// </summary>
                    public float Unknown185 { get { return __unknown185__; } set { __unknown185__ = value; } }
                    private float __unknown185__;
                    /// <summary>
                    /// 186. Float :: Offset: 0x2E4 (740)
                    /// </summary>
                    public float Unknown186 { get { return __unknown186__; } set { __unknown186__ = value; } }
                    private float __unknown186__;
                    /// <summary>
                    /// 187. Float :: Offset: 0x2E8 (744)
                    /// </summary>
                    public float Unknown187 { get { return __unknown187__; } set { __unknown187__ = value; } }
                    private float __unknown187__;
                    /// <summary>
                    /// 188. Float :: Offset: 0x2EC (748)
                    /// </summary>
                    public float Unknown188 { get { return __unknown188__; } set { __unknown188__ = value; } }
                    private float __unknown188__;
                    /// <summary>
                    /// 189. Float :: Offset: 0x2F0 (752)
                    /// </summary>
                    public float Unknown189 { get { return __unknown189__; } set { __unknown189__ = value; } }
                    private float __unknown189__;
                    /// <summary>
                    /// 190. Float :: Offset: 0x2F4 (756)
                    /// </summary>
                    public float Unknown190 { get { return __unknown190__; } set { __unknown190__ = value; } }
                    private float __unknown190__;
                    /// <summary>
                    /// 191. Float :: Offset: 0x2F8 (760)
                    /// </summary>
                    public float Unknown191 { get { return __unknown191__; } set { __unknown191__ = value; } }
                    private float __unknown191__;
                    /// <summary>
                    /// 192. Float :: Offset: 0x2FC (764)
                    /// </summary>
                    public float Unknown192 { get { return __unknown192__; } set { __unknown192__ = value; } }
                    private float __unknown192__;
                    /// <summary>
                    /// 193. Float :: Offset: 0x300 (768)
                    /// </summary>
                    public float Unknown193 { get { return __unknown193__; } set { __unknown193__ = value; } }
                    private float __unknown193__;
                    /// <summary>
                    /// 194. Float :: Offset: 0x304 (772)
                    /// </summary>
                    public float Unknown194 { get { return __unknown194__; } set { __unknown194__ = value; } }
                    private float __unknown194__;
                    /// <summary>
                    /// 195. Float :: Offset: 0x308 (776)
                    /// </summary>
                    public float Unknown195 { get { return __unknown195__; } set { __unknown195__ = value; } }
                    private float __unknown195__;
                    /// <summary>
                    /// 196. Float :: Offset: 0x30C (780)
                    /// </summary>
                    public float Unknown196 { get { return __unknown196__; } set { __unknown196__ = value; } }
                    private float __unknown196__;
                    /// <summary>
                    /// 197. Float :: Offset: 0x310 (784)
                    /// </summary>
                    public float Unknown197 { get { return __unknown197__; } set { __unknown197__ = value; } }
                    private float __unknown197__;
                    /// <summary>
                    /// 198. Float :: Offset: 0x314 (788)
                    /// </summary>
                    public float Unknown198 { get { return __unknown198__; } set { __unknown198__ = value; } }
                    private float __unknown198__;
                    /// <summary>
                    /// 199. Float :: Offset: 0x318 (792)
                    /// </summary>
                    public float Unknown199 { get { return __unknown199__; } set { __unknown199__ = value; } }
                    private float __unknown199__;
                    /// <summary>
                    /// 200. Float :: Offset: 0x31C (796)
                    /// </summary>
                    public float Unknown200 { get { return __unknown200__; } set { __unknown200__ = value; } }
                    private float __unknown200__;
                    /// <summary>
                    /// 201. Float :: Offset: 0x320 (800)
                    /// </summary>
                    public float Unknown201 { get { return __unknown201__; } set { __unknown201__ = value; } }
                    private float __unknown201__;
                    /// <summary>
                    /// 202. Float :: Offset: 0x324 (804)
                    /// </summary>
                    public float Unknown202 { get { return __unknown202__; } set { __unknown202__ = value; } }
                    private float __unknown202__;
                    /// <summary>
                    /// 203. Float :: Offset: 0x328 (808)
                    /// </summary>
                    public float Unknown203 { get { return __unknown203__; } set { __unknown203__ = value; } }
                    private float __unknown203__;
                    /// <summary>
                    /// 204. Float :: Offset: 0x32C (812)
                    /// </summary>
                    public float Unknown204 { get { return __unknown204__; } set { __unknown204__ = value; } }
                    private float __unknown204__;
                    /// <summary>
                    /// 205. Float :: Offset: 0x330 (816)
                    /// </summary>
                    public float Unknown205 { get { return __unknown205__; } set { __unknown205__ = value; } }
                    private float __unknown205__;
                    /// <summary>
                    /// 206. Float :: Offset: 0x334 (820)
                    /// </summary>
                    public float Unknown206 { get { return __unknown206__; } set { __unknown206__ = value; } }
                    private float __unknown206__;
                    /// <summary>
                    /// 207. Float :: Offset: 0x338 (824)
                    /// </summary>
                    public float Unknown207 { get { return __unknown207__; } set { __unknown207__ = value; } }
                    private float __unknown207__;
                    /// <summary>
                    /// 208. Float :: Offset: 0x33C (828)
                    /// </summary>
                    public float Unknown208 { get { return __unknown208__; } set { __unknown208__ = value; } }
                    private float __unknown208__;
                    /// <summary>
                    /// 209. Float :: Offset: 0x340 (832)
                    /// </summary>
                    public float Unknown209 { get { return __unknown209__; } set { __unknown209__ = value; } }
                    private float __unknown209__;
                    /// <summary>
                    /// 210. Float :: Offset: 0x344 (836)
                    /// </summary>
                    public float Unknown210 { get { return __unknown210__; } set { __unknown210__ = value; } }
                    private float __unknown210__;
                    /// <summary>
                    /// 211. Float :: Offset: 0x348 (840)
                    /// </summary>
                    public float Unknown211 { get { return __unknown211__; } set { __unknown211__ = value; } }
                    private float __unknown211__;
                    /// <summary>
                    /// 212. Float :: Offset: 0x34C (844)
                    /// </summary>
                    public float Unknown212 { get { return __unknown212__; } set { __unknown212__ = value; } }
                    private float __unknown212__;
                    /// <summary>
                    /// 213. Float :: Offset: 0x350 (848)
                    /// </summary>
                    public float Unknown213 { get { return __unknown213__; } set { __unknown213__ = value; } }
                    private float __unknown213__;
                    /// <summary>
                    /// 214. Float :: Offset: 0x354 (852)
                    /// </summary>
                    public float Unknown214 { get { return __unknown214__; } set { __unknown214__ = value; } }
                    private float __unknown214__;
                    /// <summary>
                    /// 215. Float :: Offset: 0x358 (856)
                    /// </summary>
                    public float Unknown215 { get { return __unknown215__; } set { __unknown215__ = value; } }
                    private float __unknown215__;
                    /// <summary>
                    /// 216. Float :: Offset: 0x35C (860)
                    /// </summary>
                    public float Unknown216 { get { return __unknown216__; } set { __unknown216__ = value; } }
                    private float __unknown216__;
                    /// <summary>
                    /// 217. Float :: Offset: 0x360 (864)
                    /// </summary>
                    public float Unknown217 { get { return __unknown217__; } set { __unknown217__ = value; } }
                    private float __unknown217__;
                    /// <summary>
                    /// 218. Float :: Offset: 0x364 (868)
                    /// </summary>
                    public float Unknown218 { get { return __unknown218__; } set { __unknown218__ = value; } }
                    private float __unknown218__;
                    /// <summary>
                    /// 219. Float :: Offset: 0x368 (872)
                    /// </summary>
                    public float Unknown219 { get { return __unknown219__; } set { __unknown219__ = value; } }
                    private float __unknown219__;
                    /// <summary>
                    /// 220. Float :: Offset: 0x36C (876)
                    /// </summary>
                    public float Unknown220 { get { return __unknown220__; } set { __unknown220__ = value; } }
                    private float __unknown220__;
                    /// <summary>
                    /// 221. Float :: Offset: 0x370 (880)
                    /// </summary>
                    public float Unknown221 { get { return __unknown221__; } set { __unknown221__ = value; } }
                    private float __unknown221__;
                    /// <summary>
                    /// 222. Float :: Offset: 0x374 (884)
                    /// </summary>
                    public float Unknown222 { get { return __unknown222__; } set { __unknown222__ = value; } }
                    private float __unknown222__;
                    /// <summary>
                    /// 223. Float :: Offset: 0x378 (888)
                    /// </summary>
                    public float Unknown223 { get { return __unknown223__; } set { __unknown223__ = value; } }
                    private float __unknown223__;
                    /// <summary>
                    /// 224. Float :: Offset: 0x37C (892)
                    /// </summary>
                    public float Unknown224 { get { return __unknown224__; } set { __unknown224__ = value; } }
                    private float __unknown224__;
                    /// <summary>
                    /// 225. Float :: Offset: 0x380 (896)
                    /// </summary>
                    public float Unknown225 { get { return __unknown225__; } set { __unknown225__ = value; } }
                    private float __unknown225__;
                    /// <summary>
                    /// 226. Float :: Offset: 0x384 (900)
                    /// </summary>
                    public float Unknown226 { get { return __unknown226__; } set { __unknown226__ = value; } }
                    private float __unknown226__;
                    /// <summary>
                    /// 227. Float :: Offset: 0x388 (904)
                    /// </summary>
                    public float Unknown227 { get { return __unknown227__; } set { __unknown227__ = value; } }
                    private float __unknown227__;
                    /// <summary>
                    /// 228. Float :: Offset: 0x38C (908)
                    /// </summary>
                    public float Unknown228 { get { return __unknown228__; } set { __unknown228__ = value; } }
                    private float __unknown228__;
                    /// <summary>
                    /// 229. Float :: Offset: 0x390 (912)
                    /// </summary>
                    public float Unknown229 { get { return __unknown229__; } set { __unknown229__ = value; } }
                    private float __unknown229__;
                    /// <summary>
                    /// 230. Float :: Offset: 0x394 (916)
                    /// </summary>
                    public float Unknown230 { get { return __unknown230__; } set { __unknown230__ = value; } }
                    private float __unknown230__;
                    /// <summary>
                    /// 231. Float :: Offset: 0x398 (920)
                    /// </summary>
                    public float Unknown231 { get { return __unknown231__; } set { __unknown231__ = value; } }
                    private float __unknown231__;
                    /// <summary>
                    /// 232. Float :: Offset: 0x39C (924)
                    /// </summary>
                    public float Unknown232 { get { return __unknown232__; } set { __unknown232__ = value; } }
                    private float __unknown232__;
                    /// <summary>
                    /// 233. Float :: Offset: 0x3A0 (928)
                    /// </summary>
                    public float Unknown233 { get { return __unknown233__; } set { __unknown233__ = value; } }
                    private float __unknown233__;
                    /// <summary>
                    /// 234. Float :: Offset: 0x3A4 (932)
                    /// </summary>
                    public float Unknown234 { get { return __unknown234__; } set { __unknown234__ = value; } }
                    private float __unknown234__;
                    /// <summary>
                    /// 235. Float :: Offset: 0x3A8 (936)
                    /// </summary>
                    public float Unknown235 { get { return __unknown235__; } set { __unknown235__ = value; } }
                    private float __unknown235__;
                    /// <summary>
                    /// 236. Float :: Offset: 0x3AC (940)
                    /// </summary>
                    public float Unknown236 { get { return __unknown236__; } set { __unknown236__ = value; } }
                    private float __unknown236__;
                    /// <summary>
                    /// 237. Float :: Offset: 0x3B0 (944)
                    /// </summary>
                    public float Unknown237 { get { return __unknown237__; } set { __unknown237__ = value; } }
                    private float __unknown237__;
                    /// <summary>
                    /// 238. Float :: Offset: 0x3B4 (948)
                    /// </summary>
                    public float Unknown238 { get { return __unknown238__; } set { __unknown238__ = value; } }
                    private float __unknown238__;
                    /// <summary>
                    /// 239. Float :: Offset: 0x3B8 (952)
                    /// </summary>
                    public float Unknown239 { get { return __unknown239__; } set { __unknown239__ = value; } }
                    private float __unknown239__;
                    /// <summary>
                    /// 240. Float :: Offset: 0x3BC (956)
                    /// </summary>
                    public float Unknown240 { get { return __unknown240__; } set { __unknown240__ = value; } }
                    private float __unknown240__;
                    /// <summary>
                    /// 241. Float :: Offset: 0x3C0 (960)
                    /// </summary>
                    public float Unknown241 { get { return __unknown241__; } set { __unknown241__ = value; } }
                    private float __unknown241__;
                    /// <summary>
                    /// 242. Float :: Offset: 0x3C4 (964)
                    /// </summary>
                    public float Unknown242 { get { return __unknown242__; } set { __unknown242__ = value; } }
                    private float __unknown242__;
                    /// <summary>
                    /// 243. Float :: Offset: 0x3C8 (968)
                    /// </summary>
                    public float Unknown243 { get { return __unknown243__; } set { __unknown243__ = value; } }
                    private float __unknown243__;
                    /// <summary>
                    /// 244. Float :: Offset: 0x3CC (972)
                    /// </summary>
                    public float Unknown244 { get { return __unknown244__; } set { __unknown244__ = value; } }
                    private float __unknown244__;
                    /// <summary>
                    /// 245. Float :: Offset: 0x3D0 (976)
                    /// </summary>
                    public float Unknown245 { get { return __unknown245__; } set { __unknown245__ = value; } }
                    private float __unknown245__;
                    /// <summary>
                    /// 246. Float :: Offset: 0x3D4 (980)
                    /// </summary>
                    public float Unknown246 { get { return __unknown246__; } set { __unknown246__ = value; } }
                    private float __unknown246__;
                    /// <summary>
                    /// 247. Float :: Offset: 0x3D8 (984)
                    /// </summary>
                    public float Unknown247 { get { return __unknown247__; } set { __unknown247__ = value; } }
                    private float __unknown247__;
                    /// <summary>
                    /// 248. Float :: Offset: 0x3DC (988)
                    /// </summary>
                    public float Unknown248 { get { return __unknown248__; } set { __unknown248__ = value; } }
                    private float __unknown248__;
                    /// <summary>
                    /// 249. Float :: Offset: 0x3E0 (992)
                    /// </summary>
                    public float Unknown249 { get { return __unknown249__; } set { __unknown249__ = value; } }
                    private float __unknown249__;
                    /// <summary>
                    /// 250. Float :: Offset: 0x3E4 (996)
                    /// </summary>
                    public float Unknown250 { get { return __unknown250__; } set { __unknown250__ = value; } }
                    private float __unknown250__;
                    /// <summary>
                    /// 251. Float :: Offset: 0x3E8 (1000)
                    /// </summary>
                    public float Unknown251 { get { return __unknown251__; } set { __unknown251__ = value; } }
                    private float __unknown251__;
                    /// <summary>
                    /// 252. Float :: Offset: 0x3EC (1004)
                    /// </summary>
                    public float Unknown252 { get { return __unknown252__; } set { __unknown252__ = value; } }
                    private float __unknown252__;
                    /// <summary>
                    /// 253. Float :: Offset: 0x3F0 (1008)
                    /// </summary>
                    public float Unknown253 { get { return __unknown253__; } set { __unknown253__ = value; } }
                    private float __unknown253__;
                    /// <summary>
                    /// 254. Float :: Offset: 0x3F4 (1012)
                    /// </summary>
                    public float Unknown254 { get { return __unknown254__; } set { __unknown254__ = value; } }
                    private float __unknown254__;
                    /// <summary>
                    /// 255. Float :: Offset: 0x3F8 (1016)
                    /// </summary>
                    public float Unknown255 { get { return __unknown255__; } set { __unknown255__ = value; } }
                    private float __unknown255__;
                    /// <summary>
                    /// 256. Float :: Offset: 0x3FC (1020)
                    /// </summary>
                    public float Unknown256 { get { return __unknown256__; } set { __unknown256__ = value; } }
                    private float __unknown256__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown3TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown3TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown3TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown3TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 4. ByteArray :: Offset: 0x10 (16)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 5. Tag :: Offset: 0x18 (24)
                /// </summary>
                public TagRef LightmapBitmap_shadows { get { return __lightmapBitmap_shadows__; } set { __lightmapBitmap_shadows__ = value; } }
                private TagRef __lightmapBitmap_shadows__;
                /// <summary>
                /// 6. TagBlock :: Offset: 0x20 (32)
                /// </summary>
                public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private _Unknown4TagBlock __unknown4__;
                public class _Unknown4TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. ByteArray :: Offset: 0x18 (24)
                    /// </summary>
                    public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                    private byte[] __unused1__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 9. ByteArray :: Offset: 0x28 (40)
                    /// </summary>
                    public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                    private byte[] __unused2__;
                    /// <summary>
                    /// 10. ByteArray :: Offset: 0x2C (44)
                    /// </summary>
                    public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                    private byte[] __unused3__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x30 (48)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;
                    /// <summary>
                    /// 12. Float :: Offset: 0x34 (52)
                    /// </summary>
                    public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                    private float __unknown9__;
                    /// <summary>
                    /// 13. ByteArray :: Offset: 0x38 (56)
                    /// </summary>
                    public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                    private byte[] __unused4__;
                    /// <summary>
                    /// 14. ByteArray :: Offset: 0x40 (64)
                    /// </summary>
                    public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
                    private byte[] __unused5__;
                    /// <summary>
                    /// 15. ByteArray :: Offset: 0x44 (68)
                    /// </summary>
                    public byte[] Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
                    private byte[] __unused6__;
                    /// <summary>
                    /// 16. ByteArray :: Offset: 0x48 (72)
                    /// </summary>
                    public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
                    private byte[] __unused7__;
                    /// <summary>
                    /// 17. ByteArray :: Offset: 0x4C (76)
                    /// </summary>
                    public byte[] Unused8 { get { return __unused8__; } set { __unused8__ = value; } }
                    private byte[] __unused8__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown4TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown4TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown4TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown4TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 7. TagBlock :: Offset: 0x28 (40)
                /// </summary>
                public _Unknown5TagBlock Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private _Unknown5TagBlock __unknown5__;
                public class _Unknown5TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private float __unknown__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown5TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown5TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown5TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown5TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 8. TagBlock :: Offset: 0x30 (48)
                /// </summary>
                public _Unknown6TagBlock Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private _Unknown6TagBlock __unknown6__;
                public class _Unknown6TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. ByteArray :: Offset: 0x18 (24)
                    /// </summary>
                    public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                    private byte[] __unused1__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 9. ByteArray :: Offset: 0x28 (40)
                    /// </summary>
                    public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                    private byte[] __unused2__;
                    /// <summary>
                    /// 10. ByteArray :: Offset: 0x2C (44)
                    /// </summary>
                    public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                    private byte[] __unused3__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x30 (48)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;
                    /// <summary>
                    /// 12. Float :: Offset: 0x34 (52)
                    /// </summary>
                    public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                    private float __unknown9__;
                    /// <summary>
                    /// 13. ByteArray :: Offset: 0x38 (56)
                    /// </summary>
                    public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
                    private byte[] __unused4__;
                    /// <summary>
                    /// 14. ByteArray :: Offset: 0x40 (64)
                    /// </summary>
                    public byte[] Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
                    private byte[] __unused5__;
                    /// <summary>
                    /// 15. ByteArray :: Offset: 0x44 (68)
                    /// </summary>
                    public byte[] Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
                    private byte[] __unused6__;
                    /// <summary>
                    /// 16. ByteArray :: Offset: 0x48 (72)
                    /// </summary>
                    public byte[] Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
                    private byte[] __unused7__;
                    /// <summary>
                    /// 17. ByteArray :: Offset: 0x4C (76)
                    /// </summary>
                    public byte[] Unused8 { get { return __unused8__; } set { __unused8__ = value; } }
                    private byte[] __unused8__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown6TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown6TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown6TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown6TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 9. TagBlock :: Offset: 0x38 (56)
                /// </summary>
                public _Unknown7TagBlock Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private _Unknown7TagBlock __unknown7__;
                public class _Unknown7TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private float __unknown4__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private float __unknown5__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                    private float __unknown7__;
                    /// <summary>
                    /// 8. Float :: Offset: 0x1C (28)
                    /// </summary>
                    public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                    private float __unknown8__;
                    /// <summary>
                    /// 9. Float :: Offset: 0x20 (32)
                    /// </summary>
                    public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                    private float __unknown9__;
                    /// <summary>
                    /// 10. Float :: Offset: 0x24 (36)
                    /// </summary>
                    public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                    private float __unknown10__;
                    /// <summary>
                    /// 11. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                    private float __unknown11__;
                    /// <summary>
                    /// 12. Float :: Offset: 0x2C (44)
                    /// </summary>
                    public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                    private float __unknown12__;
                    /// <summary>
                    /// 13. Float :: Offset: 0x30 (48)
                    /// </summary>
                    public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                    private float __unknown13__;
                    /// <summary>
                    /// 14. Float :: Offset: 0x34 (52)
                    /// </summary>
                    public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                    private float __unknown14__;
                    /// <summary>
                    /// 15. Float :: Offset: 0x38 (56)
                    /// </summary>
                    public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                    private float __unknown15__;
                    /// <summary>
                    /// 16. Float :: Offset: 0x3C (60)
                    /// </summary>
                    public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                    private float __unknown16__;
                    /// <summary>
                    /// 17. Float :: Offset: 0x40 (64)
                    /// </summary>
                    public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                    private float __unknown17__;
                    /// <summary>
                    /// 18. Float :: Offset: 0x44 (68)
                    /// </summary>
                    public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                    private float __unknown18__;
                    /// <summary>
                    /// 19. Float :: Offset: 0x48 (72)
                    /// </summary>
                    public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                    private float __unknown19__;
                    /// <summary>
                    /// 20. Float :: Offset: 0x4C (76)
                    /// </summary>
                    public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                    private float __unknown20__;
                    /// <summary>
                    /// 21. Float :: Offset: 0x50 (80)
                    /// </summary>
                    public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                    private float __unknown21__;
                    /// <summary>
                    /// 22. Float :: Offset: 0x54 (84)
                    /// </summary>
                    public float Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                    private float __unknown22__;
                    /// <summary>
                    /// 23. Float :: Offset: 0x58 (88)
                    /// </summary>
                    public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                    private float __unknown23__;
                    /// <summary>
                    /// 24. Float :: Offset: 0x5C (92)
                    /// </summary>
                    public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                    private float __unknown24__;
                    /// <summary>
                    /// 25. Float :: Offset: 0x60 (96)
                    /// </summary>
                    public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                    private float __unknown25__;
                    /// <summary>
                    /// 26. Float :: Offset: 0x64 (100)
                    /// </summary>
                    public float Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                    private float __unknown26__;
                    /// <summary>
                    /// 27. Float :: Offset: 0x68 (104)
                    /// </summary>
                    public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                    private float __unknown27__;
                    /// <summary>
                    /// 28. Float :: Offset: 0x6C (108)
                    /// </summary>
                    public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                    private float __unknown28__;
                    /// <summary>
                    /// 29. Float :: Offset: 0x70 (112)
                    /// </summary>
                    public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                    private float __unknown29__;
                    /// <summary>
                    /// 30. Float :: Offset: 0x74 (116)
                    /// </summary>
                    public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                    private float __unknown30__;
                    /// <summary>
                    /// 31. Float :: Offset: 0x78 (120)
                    /// </summary>
                    public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
                    private float __unknown31__;
                    /// <summary>
                    /// 32. Float :: Offset: 0x7C (124)
                    /// </summary>
                    public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
                    private float __unknown32__;
                    /// <summary>
                    /// 33. Float :: Offset: 0x80 (128)
                    /// </summary>
                    public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
                    private float __unknown33__;
                    /// <summary>
                    /// 34. Float :: Offset: 0x84 (132)
                    /// </summary>
                    public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
                    private float __unknown34__;
                    /// <summary>
                    /// 35. Float :: Offset: 0x88 (136)
                    /// </summary>
                    public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
                    private float __unknown35__;
                    /// <summary>
                    /// 36. Float :: Offset: 0x8C (140)
                    /// </summary>
                    public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
                    private float __unknown36__;
                    /// <summary>
                    /// 37. ByteArray :: Offset: 0x90 (144)
                    /// </summary>
                    public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                    private byte[] __unused1__;
                    /// <summary>
                    /// 38. ByteArray :: Offset: 0x94 (148)
                    /// </summary>
                    public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                    private byte[] __unused2__;
                    /// <summary>
                    /// 39. Float :: Offset: 0xA4 (164)
                    /// </summary>
                    public float Unknown37 { get { return __unknown37__; } set { __unknown37__ = value; } }
                    private float __unknown37__;
                    /// <summary>
                    /// 40. Float :: Offset: 0xA8 (168)
                    /// </summary>
                    public float Unknown38 { get { return __unknown38__; } set { __unknown38__ = value; } }
                    private float __unknown38__;
                    /// <summary>
                    /// 41. Float :: Offset: 0xAC (172)
                    /// </summary>
                    public float Unknown39 { get { return __unknown39__; } set { __unknown39__ = value; } }
                    private float __unknown39__;
                    /// <summary>
                    /// 42. Float :: Offset: 0xB0 (176)
                    /// </summary>
                    public float Unknown40 { get { return __unknown40__; } set { __unknown40__ = value; } }
                    private float __unknown40__;
                    /// <summary>
                    /// 43. Float :: Offset: 0xB4 (180)
                    /// </summary>
                    public float Unknown41 { get { return __unknown41__; } set { __unknown41__ = value; } }
                    private float __unknown41__;
                    /// <summary>
                    /// 44. Float :: Offset: 0xB8 (184)
                    /// </summary>
                    public float Unknown42 { get { return __unknown42__; } set { __unknown42__ = value; } }
                    private float __unknown42__;
                    /// <summary>
                    /// 45. Float :: Offset: 0xBC (188)
                    /// </summary>
                    public float Unknown43 { get { return __unknown43__; } set { __unknown43__ = value; } }
                    private float __unknown43__;
                    /// <summary>
                    /// 46. Float :: Offset: 0xC0 (192)
                    /// </summary>
                    public float Unknown44 { get { return __unknown44__; } set { __unknown44__ = value; } }
                    private float __unknown44__;
                    /// <summary>
                    /// 47. Float :: Offset: 0xC4 (196)
                    /// </summary>
                    public float Unknown45 { get { return __unknown45__; } set { __unknown45__ = value; } }
                    private float __unknown45__;
                    /// <summary>
                    /// 48. Float :: Offset: 0xC8 (200)
                    /// </summary>
                    public float Unknown46 { get { return __unknown46__; } set { __unknown46__ = value; } }
                    private float __unknown46__;
                    /// <summary>
                    /// 49. ByteArray :: Offset: 0xCC (204)
                    /// </summary>
                    public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                    private byte[] __unused3__;
                    /// <summary>
                    /// 50. Float :: Offset: 0xD4 (212)
                    /// </summary>
                    public float Unknown47 { get { return __unknown47__; } set { __unknown47__ = value; } }
                    private float __unknown47__;
                    /// <summary>
                    /// 51. Float :: Offset: 0xD8 (216)
                    /// </summary>
                    public float Unknown48 { get { return __unknown48__; } set { __unknown48__ = value; } }
                    private float __unknown48__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown7TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown7TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown7TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown7TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 10. TagBlock :: Offset: 0x40 (64)
                /// </summary>
                public _Unknown8TagBlock Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private _Unknown8TagBlock __unknown8__;
                public class _Unknown8TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. ByteArray :: Offset: 0x4 (4)
                    /// </summary>
                    public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                    private byte[] __unused1__;
                    /// <summary>
                    /// 3. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float RawOffset { get { return __rawOffset__; } set { __rawOffset__ = value; } }
                    private float __rawOffset__;
                    /// <summary>
                    /// 4. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float RawSize { get { return __rawSize__; } set { __rawSize__ = value; } }
                    private float __rawSize__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;
                    /// <summary>
                    /// 7. TagBlock :: Offset: 0x1C (28)
                    /// </summary>
                    public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                    private _Unknown4TagBlock __unknown4__;
                    public class _Unknown4TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private LtmpLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Float :: Offset: 0x0 (0)
                        /// </summary>
                        public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                        private float __unknown1__;
                        /// <summary>
                        /// 2. Float :: Offset: 0x4 (4)
                        /// </summary>
                        public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                        private float __unknown2__;
                        /// <summary>
                        /// 3. Float :: Offset: 0x8 (8)
                        /// </summary>
                        public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                        private float __unknown3__;
                        /// <summary>
                        /// 4. Float :: Offset: 0xC (12)
                        /// </summary>
                        public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                        private float __unknown4__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown4TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown4TagBlock(LtmpLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown4TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown4TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }
                    /// <summary>
                    /// 8. ID :: Offset: 0x24 (36)
                    /// </summary>
                    public LoneID Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                    private LoneID __unknown5__;
                    /// <summary>
                    /// 9. Float :: Offset: 0x28 (40)
                    /// </summary>
                    public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                    private float __unknown6__;
                    /// <summary>
                    /// 10. ByteArray :: Offset: 0x2C (44)
                    /// </summary>
                    public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                    private byte[] __unused2__;
                    /// <summary>
                    /// 11. ByteArray :: Offset: 0x30 (48)
                    /// </summary>
                    public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                    private byte[] __unused3__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown8TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown8TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Unknown4 = new _Unknown4TagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown8TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown8TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 11. TagBlock :: Offset: 0x48 (72)
                /// </summary>
                public _Unknown9TagBlock Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private _Unknown9TagBlock __unknown9__;
                public class _Unknown9TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private float __unknown__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown9TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown9TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown9TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown9TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 12. TagBlock :: Offset: 0x50 (80)
                /// </summary>
                public _Unknown10TagBlock Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private _Unknown10TagBlock __unknown10__;
                public class _Unknown10TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. TagBlock :: Offset: 0x4 (4)
                    /// </summary>
                    public _Unknown2TagBlock Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private _Unknown2TagBlock __unknown2__;
                    public class _Unknown2TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private LtmpLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Float :: Offset: 0x0 (0)
                        /// </summary>
                        public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private float __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown2TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown2TagBlock(LtmpLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown2TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown2TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown10TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown10TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Unknown2 = new _Unknown2TagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown10TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown10TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 13. TagBlock :: Offset: 0x58 (88)
                /// </summary>
                public _Unknown11TagBlock Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private _Unknown11TagBlock __unknown11__;
                public class _Unknown11TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private float __unknown2__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                    private float __unknown3__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown11TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown11TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown11TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown11TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 14. TagBlock :: Offset: 0x60 (96)
                /// </summary>
                public _Unknown12TagBlock Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private _Unknown12TagBlock __unknown12__;
                public class _Unknown12TagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private LtmpLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                    private float __unknown1__;
                    /// <summary>
                    /// 2. TagBlock :: Offset: 0x4 (4)
                    /// </summary>
                    public _Unknown2TagBlock Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                    private _Unknown2TagBlock __unknown2__;
                    public class _Unknown2TagBlock
                    {
                        #region Class Instance Data, Accessors, Classes

                        /// <summary>
                        /// The root class.
                        /// </summary>
                        public LtmpLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                        private LtmpLayout __layout__;
                        /// <summary>
                        /// The offset at which this reflexive is declared in the tag meta.
                        /// </summary>
                        public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                        private long __declarationOffset__;
                        /// <summary>
                        /// The total amount of chunks.
                        /// </summary>
                        public int _Count { get { return __count__; } protected set { __count__ = value; } }
                        private int __count__;
                        /// <summary>
                        /// The initial pointer read from the tag meta.
                        /// </summary>
                        public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                        private int __pointer__;
                        /// <summary>
                        /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                        /// </summary>
                        public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                        private int __translation__;
                        /// <summary>
                        /// The currently selected chunk's index.
                        /// </summary>
                        public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                        private int __selectedIndex__;
                        /// <summary>
                        /// True if the reflexive does not exist in the current tag meta.
                        /// </summary>
                        public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                        private bool __null__;
                        /// <summary>
                        /// True if Initialize() has been called, and executed, without error.
                        /// </summary>
                        public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                        private bool __initialized__;

                        #endregion

                        #region Reflexive Instance Data, Accessors, Classes

                        /// <summary>
                        /// 1. Int16 :: Offset: 0x0 (0)
                        /// </summary>
                        public short Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                        private short __unknown__;

                        #endregion

                        #region Initialization

                        /// <summary>
                        /// Initializes a new instance of the _Unknown2TagBlock class based 
                        /// on the supplied layout reference.
                        /// </summary>
                        /// <param name="layout">The layout reference. </param>
                        public _Unknown2TagBlock(LtmpLayout layout)
                        {
                            this._Layout = layout;
                            this._Initialized = false;

                        }

                        /// <summary>
                        /// Parses the declaration of the reflexive into the several class members.
                        /// </summary>
                        /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                        public void _Initialize(BitReader br)
                        {
                            _Initialized = false;
                            int cc = br.ReadInt32();
                            int tr = br.ReadInt32();

                            if (tr == -1)
                            {
                                _Null = true;
                            }
                            else
                            {
                                _DeclarationOffset = br.BaseStream.Position - 8;
                                _Count = cc;
                                _Pointer = tr;
                                _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                                _SelectedIndex = 0;
                                _Null = false;
                            }
                            _Initialized = true;
                        }

                        #endregion

                        #region New, Add, Delete, Remove Methods

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        public void _New()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with null values. If the reflexive 
                        /// does not exist in the current tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _New(FileStream fs)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write null, -1 for pointers
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(_Unknown2TagBlock template)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._New(fs, template);
                            fs.Close();
                        }

                        /// <summary>
                        /// Overwrites the current reflexive with the values provided
                        /// by the template. If the reflexive does not exist in the current 
                        /// tag meta, space will be made.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="template">The template containing the new values. </param>
                        public void _New(FileStream fs, _Unknown2TagBlock template)
                        {
                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            if (_Null)
                            {
                                // Make space
                            }

                            // Write template
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        public void _Add()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be made up of null values.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Add(FileStream fs)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make null space
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Add(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                        /// be based off of the values found within the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                        public void _Add(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Make space, fill with template
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        public void _Delete()
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Delete(fs);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the reflexive if it exists in the current tag meta.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        public void _Delete(FileStream fs)
                        {
                            if (_Null) return;

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Delete

                            _Null = true;
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(int index)
                        {
                            FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                            this._Remove(fs, index);
                            fs.Close();
                        }

                        /// <summary>
                        /// Removes the chunk at the given index.
                        /// </summary>
                        /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                        /// <param name="index">The index of the chunk to remove. </param>
                        public void _Remove(FileStream fs, int index)
                        {
                            if (_Null) throw new Exception("Reflexive does not exist.");

                            BitReader br = new BitReader(fs, Endian.Little);
                            BitWriter bw = new BitWriter(fs, Endian.Little);

                            // Remove

                            if (--_Count == 0)
                            {
                                // Write -1 pointer
                                _Null = true;
                            }
                        }

                        #endregion
                    }

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Unknown12TagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Unknown12TagBlock(LtmpLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                        Unknown2 = new _Unknown2TagBlock(layout);
                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Unknown12TagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Unknown12TagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown13TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown13TagBlock(LtmpLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Unknown3 = new _Unknown3TagBlock(layout);
                    Unknown4 = new _Unknown4TagBlock(layout);
                    Unknown5 = new _Unknown5TagBlock(layout);
                    Unknown6 = new _Unknown6TagBlock(layout);
                    Unknown7 = new _Unknown7TagBlock(layout);
                    Unknown8 = new _Unknown8TagBlock(layout);
                    Unknown9 = new _Unknown9TagBlock(layout);
                    Unknown10 = new _Unknown10TagBlock(layout);
                    Unknown11 = new _Unknown11TagBlock(layout);
                    Unknown12 = new _Unknown12TagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown13TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown13TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 16. ByteArray :: Offset: 0x88 (136)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the LtmpLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public LtmpLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Unknown13 = new _Unknown13TagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Unknown1 = br.ReadSingle();
                Unknown2 = br.ReadSingle();
                Unknown3 = br.ReadSingle();
                Unknown4 = br.ReadSingle();
                Unused1 = br.ReadBytes(4);
                Unknown5 = br.ReadSingle();
                Unknown6 = br.ReadSingle();
                Unknown7 = br.ReadSingle();
                Unknown8 = br.ReadSingle();
                Unknown9 = br.ReadSingle();
                Unknown10 = br.ReadSingle();
                Unknown11 = br.ReadSingle();
                Unknown12 = br.ReadSingle();
                Unused2 = br.ReadBytes(76);
                Unknown13._Initialize(br);
                Unused3 = br.ReadBytes(124);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Ltmp class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Ltmp(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new LtmpLayout(map, identity);
        }

        #region Specific Methods of ltmp

        #endregion
    }
}
