using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Vehicle
    /// </summary>
    public class Vehi
    {
        #region Constants

        /// <summary>
        /// Size of the vehi meta header.
        /// </summary>
        public const int HeaderSize = 768;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private VehiLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.71 Turk645: Fixed a few bitmasks and added more info.
        /// v0.7 XZodia: Merged With Old/New Plugin.
        /// v0.6 Shalted: Added Kornman's standard header.
        /// v0.56 Turk: fixed some values from the xml.
        /// v0.5 Shalted: Added Standard header, fixed some values and plugin errors, added some AI info.
        /// v0.4 XZodia: Lots of string ids and some unknowns named.
        /// v0.3 Toabn: Weapon located and Seat values found...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public VehiLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class VehiLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public Flags1Bitmask32 Flags1 { get { return __flags1__; } set { __flags1__ = value; } }
            private Flags1Bitmask32 __flags1__;
            public class Flags1Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool DoesNotCastShadow { get { return __doesNotCastShadow__; } set { __doesNotCastShadow__ = value; } }
                private bool __doesNotCastShadow__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool SearchCardinalDirectionLightmaps { get { return __searchCardinalDirectionLightmaps__; } set { __searchCardinalDirectionLightmaps__ = value; } }
                private bool __searchCardinalDirectionLightmaps__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Unused { get { return __unused__; } set { __unused__ = value; } }
                private bool __unused__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool NotAPathfindingObstacle { get { return __notAPathfindingObstacle__; } set { __notAPathfindingObstacle__ = value; } }
                private bool __notAPathfindingObstacle__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool ExtensionOfParent { get { return __extensionOfParent__; } set { __extensionOfParent__ = value; } }
                private bool __extensionOfParent__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool DoesNotCauseCollisionDamage { get { return __doesNotCauseCollisionDamage__; } set { __doesNotCauseCollisionDamage__ = value; } }
                private bool __doesNotCauseCollisionDamage__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool EarlyMover { get { return __earlyMover__; } set { __earlyMover__ = value; } }
                private bool __earlyMover__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool EarlyMoverLocalizedPhysics { get { return __earlyMoverLocalizedPhysics__; } set { __earlyMoverLocalizedPhysics__ = value; } }
                private bool __earlyMoverLocalizedPhysics__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool UseStaticMassiveLightmapSample { get { return __useStaticMassiveLightmapSample__; } set { __useStaticMassiveLightmapSample__ = value; } }
                private bool __useStaticMassiveLightmapSample__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool ObjectScalesAttachments { get { return __objectScalesAttachments__; } set { __objectScalesAttachments__ = value; } }
                private bool __objectScalesAttachments__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool InheritsPlayersAppearance { get { return __inheritsPlayersAppearance__; } set { __inheritsPlayersAppearance__ = value; } }
                private bool __inheritsPlayersAppearance__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool DeadBipedsCantLocalize { get { return __deadBipedsCantLocalize__; } set { __deadBipedsCantLocalize__ = value; } }
                private bool __deadBipedsCantLocalize__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool AttachToClustersByDynamicSphere { get { return __attachToClustersByDynamicSphere__; } set { __attachToClustersByDynamicSphere__ = value; } }
                private bool __attachToClustersByDynamicSphere__;
                /// <summary>
                /// 14. Option :: Value: 0xD (13)
                /// </summary>
                public bool EffectsCreatedByThisObjectDoNotBlah { get { return __effectsCreatedByThisObjectDoNotBlah__; } set { __effectsCreatedByThisObjectDoNotBlah__ = value; } }
                private bool __effectsCreatedByThisObjectDoNotBlah__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    DoesNotCastShadow = ((bit & mask) == bit);
                    bit <<= 1;
                    SearchCardinalDirectionLightmaps = ((bit & mask) == bit);
                    bit <<= 1;
                    Unused = ((bit & mask) == bit);
                    bit <<= 1;
                    NotAPathfindingObstacle = ((bit & mask) == bit);
                    bit <<= 1;
                    ExtensionOfParent = ((bit & mask) == bit);
                    bit <<= 1;
                    DoesNotCauseCollisionDamage = ((bit & mask) == bit);
                    bit <<= 1;
                    EarlyMover = ((bit & mask) == bit);
                    bit <<= 1;
                    EarlyMoverLocalizedPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    UseStaticMassiveLightmapSample = ((bit & mask) == bit);
                    bit <<= 1;
                    ObjectScalesAttachments = ((bit & mask) == bit);
                    bit <<= 1;
                    InheritsPlayersAppearance = ((bit & mask) == bit);
                    bit <<= 1;
                    DeadBipedsCantLocalize = ((bit & mask) == bit);
                    bit <<= 1;
                    AttachToClustersByDynamicSphere = ((bit & mask) == bit);
                    bit <<= 1;
                    EffectsCreatedByThisObjectDoNotBlah = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float BoundingRadius { get { return __boundingRadius__; } set { __boundingRadius__ = value; } }
            private float __boundingRadius__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float BoundingOffsetX { get { return __boundingOffsetX__; } set { __boundingOffsetX__ = value; } }
            private float __boundingOffsetX__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float BoundingOffsetY { get { return __boundingOffsetY__; } set { __boundingOffsetY__ = value; } }
            private float __boundingOffsetY__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float BoundingOffsetZ { get { return __boundingOffsetZ__; } set { __boundingOffsetZ__ = value; } }
            private float __boundingOffsetZ__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float AccelerationScale { get { return __accelerationScale__; } set { __accelerationScale__ = value; } }
            private float __accelerationScale__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float LightmapShadowMode { get { return __lightmapShadowMode__; } set { __lightmapShadowMode__ = value; } }
            private float __lightmapShadowMode__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float SweetenerSize { get { return __sweetenerSize__; } set { __sweetenerSize__ = value; } }
            private float __sweetenerSize__;
            /// <summary>
            /// 9. Float :: Offset: 0x20 (32)
            /// </summary>
            public float DynamicLightSphereRadius { get { return __dynamicLightSphereRadius__; } set { __dynamicLightSphereRadius__ = value; } }
            private float __dynamicLightSphereRadius__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float DynamicLightSphereOffsetX { get { return __dynamicLightSphereOffsetX__; } set { __dynamicLightSphereOffsetX__ = value; } }
            private float __dynamicLightSphereOffsetX__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float DynamicLightSphereOffsetY { get { return __dynamicLightSphereOffsetY__; } set { __dynamicLightSphereOffsetY__ = value; } }
            private float __dynamicLightSphereOffsetY__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float DynamicLightSphereOffsetZ { get { return __dynamicLightSphereOffsetZ__; } set { __dynamicLightSphereOffsetZ__ = value; } }
            private float __dynamicLightSphereOffsetZ__;
            /// <summary>
            /// 13. StringID :: Offset: 0x30 (48)
            /// </summary>
            public StringID DefaultModelVariant { get { return __defaultModelVariant__; } set { __defaultModelVariant__ = value; } }
            private StringID __defaultModelVariant__;
            /// <summary>
            /// 14. Tag :: Offset: 0x34 (52)
            /// </summary>
            public TagRef Model { get { return __model__; } set { __model__ = value; } }
            private TagRef __model__;
            /// <summary>
            /// 15. Tag :: Offset: 0x3C (60)
            /// </summary>
            public TagRef CrateObject { get { return __crateObject__; } set { __crateObject__ = value; } }
            private TagRef __crateObject__;
            /// <summary>
            /// 16. Tag :: Offset: 0x44 (68)
            /// </summary>
            public TagRef ModifierShader { get { return __modifierShader__; } set { __modifierShader__ = value; } }
            private TagRef __modifierShader__;
            /// <summary>
            /// 17. Tag :: Offset: 0x4C (76)
            /// </summary>
            public TagRef CreationEffect { get { return __creationEffect__; } set { __creationEffect__ = value; } }
            private TagRef __creationEffect__;
            /// <summary>
            /// 18. Tag :: Offset: 0x54 (84)
            /// </summary>
            public TagRef MaterialEffects1 { get { return __materialEffects1__; } set { __materialEffects1__ = value; } }
            private TagRef __materialEffects1__;
            /// <summary>
            /// 19. TagBlock :: Offset: 0x5C (92)
            /// </summary>
            public _AiPropertiesTagBlock AiProperties { get { return __aiProperties__; } set { __aiProperties__ = value; } }
            private _AiPropertiesTagBlock __aiProperties__;
            public class _AiPropertiesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Flags { get { return __flags__; } set { __flags__ = value; } }
                private float __flags__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID TypeName { get { return __typeName__; } set { __typeName__ = value; } }
                private StringID __typeName__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Size { get { return __size__; } set { __size__ = value; } }
                private float __size__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float LeapJumpSpeed { get { return __leapJumpSpeed__; } set { __leapJumpSpeed__ = value; } }
                private float __leapJumpSpeed__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AiPropertiesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AiPropertiesTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AiPropertiesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AiPropertiesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 20. TagBlock :: Offset: 0x64 (100)
            /// </summary>
            public _FunctionsTagBlock Functions { get { return __functions__; } set { __functions__ = value; } }
            private _FunctionsTagBlock __functions__;
            public class _FunctionsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Flags { get { return __flags__; } set { __flags__ = value; } }
                private float __flags__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Function { get { return __function__; } set { __function__ = value; } }
                private StringID __function__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID ScaleFunctionBy { get { return __scaleFunctionBy__; } set { __scaleFunctionBy__ = value; } }
                private StringID __scaleFunctionBy__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID WobbleFunction { get { return __wobbleFunction__; } set { __wobbleFunction__ = value; } }
                private StringID __wobbleFunction__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float WobblePeriod { get { return __wobblePeriod__; } set { __wobblePeriod__ = value; } }
                private float __wobblePeriod__;
                /// <summary>
                /// 6. TagBlock :: Offset: 0x14 (20)
                /// </summary>
                public _Inputs_realNumbersTagBlock Inputs_realNumbers { get { return __inputs_realNumbers__; } set { __inputs_realNumbers__ = value; } }
                private _Inputs_realNumbersTagBlock __inputs_realNumbers__;
                public class _Inputs_realNumbersTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private VehiLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Byte :: Offset: 0x0 (0)
                    /// </summary>
                    public byte Data { get { return __data__; } set { __data__ = value; } }
                    private byte __data__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Inputs_realNumbersTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Inputs_realNumbersTagBlock(VehiLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Inputs_realNumbersTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Inputs_realNumbersTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 7. StringID :: Offset: 0x1C (28)
                /// </summary>
                public StringID TurnOffWith { get { return __turnOffWith__; } set { __turnOffWith__ = value; } }
                private StringID __turnOffWith__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FunctionsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FunctionsTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Inputs_realNumbers = new _Inputs_realNumbersTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FunctionsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FunctionsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 21. Float :: Offset: 0x6C (108)
            /// </summary>
            public float RenderBoundingRadius { get { return __renderBoundingRadius__; } set { __renderBoundingRadius__ = value; } }
            private float __renderBoundingRadius__;
            /// <summary>
            /// 22. Float :: Offset: 0x70 (112)
            /// </summary>
            public float Lower1 { get { return __lower1__; } set { __lower1__ = value; } }
            private float __lower1__;
            /// <summary>
            /// 23. Float :: Offset: 0x74 (116)
            /// </summary>
            public float Upper1 { get { return __upper1__; } set { __upper1__ = value; } }
            private float __upper1__;
            /// <summary>
            /// 24. Float :: Offset: 0x78 (120)
            /// </summary>
            public float Lower2 { get { return __lower2__; } set { __lower2__ = value; } }
            private float __lower2__;
            /// <summary>
            /// 25. Float :: Offset: 0x7C (124)
            /// </summary>
            public float Upper2 { get { return __upper2__; } set { __upper2__ = value; } }
            private float __upper2__;
            /// <summary>
            /// 26. Float :: Offset: 0x80 (128)
            /// </summary>
            public float Lower3 { get { return __lower3__; } set { __lower3__ = value; } }
            private float __lower3__;
            /// <summary>
            /// 27. Float :: Offset: 0x84 (132)
            /// </summary>
            public float Upper3 { get { return __upper3__; } set { __upper3__ = value; } }
            private float __upper3__;
            /// <summary>
            /// 28. Float :: Offset: 0x88 (136)
            /// </summary>
            public float Lower4 { get { return __lower4__; } set { __lower4__ = value; } }
            private float __lower4__;
            /// <summary>
            /// 29. Float :: Offset: 0x8C (140)
            /// </summary>
            public float Upper4 { get { return __upper4__; } set { __upper4__ = value; } }
            private float __upper4__;
            /// <summary>
            /// 30. Int16 :: Offset: 0x90 (144)
            /// </summary>
            public short HudTextMessageIndex { get { return __hudTextMessageIndex__; } set { __hudTextMessageIndex__ = value; } }
            private short __hudTextMessageIndex__;
            /// <summary>
            /// 31. Int16 :: Offset: 0x92 (146)
            /// </summary>
            public short ForcedShaderPermutationIndex { get { return __forcedShaderPermutationIndex__; } set { __forcedShaderPermutationIndex__ = value; } }
            private short __forcedShaderPermutationIndex__;
            /// <summary>
            /// 32. TagBlock :: Offset: 0x94 (148)
            /// </summary>
            public _AttachmentsTagBlock Attachments { get { return __attachments__; } set { __attachments__ = value; } }
            private _AttachmentsTagBlock __attachments__;
            public class _AttachmentsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Type { get { return __type__; } set { __type__ = value; } }
                private TagRef __type__;
                /// <summary>
                /// 2. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 3. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID ChangeColor { get { return __changeColor__; } set { __changeColor__ = value; } }
                private StringID __changeColor__;
                /// <summary>
                /// 4. StringID :: Offset: 0x10 (16)
                /// </summary>
                public StringID PrimaryScale { get { return __primaryScale__; } set { __primaryScale__ = value; } }
                private StringID __primaryScale__;
                /// <summary>
                /// 5. StringID :: Offset: 0x14 (20)
                /// </summary>
                public StringID SecondaryScale { get { return __secondaryScale__; } set { __secondaryScale__ = value; } }
                private StringID __secondaryScale__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AttachmentsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AttachmentsTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AttachmentsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AttachmentsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 33. TagBlock :: Offset: 0x9C (156)
            /// </summary>
            public _WidgetsTagBlock Widgets { get { return __widgets__; } set { __widgets__ = value; } }
            private _WidgetsTagBlock __widgets__;
            public class _WidgetsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Reference { get { return __reference__; } set { __reference__ = value; } }
                private TagRef __reference__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WidgetsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WidgetsTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WidgetsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WidgetsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 34. ByteArray :: Offset: 0xA4 (164)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 35. TagBlock :: Offset: 0xAC (172)
            /// </summary>
            public _ChangeColorsTagBlock ChangeColors { get { return __changeColors__; } set { __changeColors__ = value; } }
            private _ChangeColorsTagBlock __changeColors__;
            public class _ChangeColorsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. TagBlock :: Offset: 0x0 (0)
                /// </summary>
                public _PermutationsTagBlock Permutations { get { return __permutations__; } set { __permutations__ = value; } }
                private _PermutationsTagBlock __permutations__;
                public class _PermutationsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private VehiLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Weight { get { return __weight__; } set { __weight__ = value; } }
                    private float __weight__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float ColorLowerBoundR { get { return __colorLowerBoundR__; } set { __colorLowerBoundR__ = value; } }
                    private float __colorLowerBoundR__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float ColorLowerBoundG { get { return __colorLowerBoundG__; } set { __colorLowerBoundG__ = value; } }
                    private float __colorLowerBoundG__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float ColorLowerBoundB { get { return __colorLowerBoundB__; } set { __colorLowerBoundB__ = value; } }
                    private float __colorLowerBoundB__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float ColorUpperBoundR { get { return __colorUpperBoundR__; } set { __colorUpperBoundR__ = value; } }
                    private float __colorUpperBoundR__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float ColorUpperBoundG { get { return __colorUpperBoundG__; } set { __colorUpperBoundG__ = value; } }
                    private float __colorUpperBoundG__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float ColorUpperBoundB { get { return __colorUpperBoundB__; } set { __colorUpperBoundB__ = value; } }
                    private float __colorUpperBoundB__;
                    /// <summary>
                    /// 8. StringID :: Offset: 0x1C (28)
                    /// </summary>
                    public StringID Name { get { return __name__; } set { __name__ = value; } }
                    private StringID __name__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PermutationsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PermutationsTagBlock(VehiLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PermutationsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PermutationsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 2. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ChangeColorsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ChangeColorsTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Permutations = new _PermutationsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ChangeColorsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ChangeColorsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 36. TagBlock :: Offset: 0xB4 (180)
            /// </summary>
            public _PredictedResourceTagBlock PredictedResource { get { return __predictedResource__; } set { __predictedResource__ = value; } }
            private _PredictedResourceTagBlock __predictedResource__;
            public class _PredictedResourceTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Int16 :: Offset: 0x0 (0)
                /// </summary>
                public short Type { get { return __type__; } set { __type__ = value; } }
                private short __type__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x2 (2)
                /// </summary>
                public short ResourceIndex { get { return __resourceIndex__; } set { __resourceIndex__ = value; } }
                private short __resourceIndex__;
                /// <summary>
                /// 3. ID :: Offset: 0x4 (4)
                /// </summary>
                public LoneID TagIndex { get { return __tagIndex__; } set { __tagIndex__ = value; } }
                private LoneID __tagIndex__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PredictedResourceTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PredictedResourceTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PredictedResourceTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PredictedResourceTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 37. Bitmask16 :: Offset: 0xBC (188)
            /// </summary>
            public OldFlags1Bitmask16 OldFlags1 { get { return __oldFlags1__; } set { __oldFlags1__ = value; } }
            private OldFlags1Bitmask16 __oldFlags1__;
            public class OldFlags1Bitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool CircularAiming { get { return __circularAiming__; } set { __circularAiming__ = value; } }
                private bool __circularAiming__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool DestroyedAfterDying { get { return __destroyedAfterDying__; } set { __destroyedAfterDying__ = value; } }
                private bool __destroyedAfterDying__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool HalfSpeedInterpolation { get { return __halfSpeedInterpolation__; } set { __halfSpeedInterpolation__ = value; } }
                private bool __halfSpeedInterpolation__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool FiresFromCamera { get { return __firesFromCamera__; } set { __firesFromCamera__ = value; } }
                private bool __firesFromCamera__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool EntranceInsideBoundingSphere { get { return __entranceInsideBoundingSphere__; } set { __entranceInsideBoundingSphere__ = value; } }
                private bool __entranceInsideBoundingSphere__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool WeaponNotVisibleIn { get { return __weaponNotVisibleIn__; } set { __weaponNotVisibleIn__ = value; } }
                private bool __weaponNotVisibleIn__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool CausesPassengerDialogue { get { return __causesPassengerDialogue__; } set { __causesPassengerDialogue__ = value; } }
                private bool __causesPassengerDialogue__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool ResistsPings { get { return __resistsPings__; } set { __resistsPings__ = value; } }
                private bool __resistsPings__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool MeleeAttackIsFatal { get { return __meleeAttackIsFatal__; } set { __meleeAttackIsFatal__ = value; } }
                private bool __meleeAttackIsFatal__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool DontRefaceDuringPings { get { return __dontRefaceDuringPings__; } set { __dontRefaceDuringPings__ = value; } }
                private bool __dontRefaceDuringPings__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool HasNoAiming { get { return __hasNoAiming__; } set { __hasNoAiming__ = value; } }
                private bool __hasNoAiming__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool SimpleCreature { get { return __simpleCreature__; } set { __simpleCreature__ = value; } }
                private bool __simpleCreature__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool ImpactMeleeAttachesToUnit { get { return __impactMeleeAttachesToUnit__; } set { __impactMeleeAttachesToUnit__ = value; } }
                private bool __impactMeleeAttachesToUnit__;
                /// <summary>
                /// 14. Option :: Value: 0xD (13)
                /// </summary>
                public bool ImpactMeleeDiesOnShields { get { return __impactMeleeDiesOnShields__; } set { __impactMeleeDiesOnShields__ = value; } }
                private bool __impactMeleeDiesOnShields__;
                /// <summary>
                /// 15. Option :: Value: 0xE (14)
                /// </summary>
                public bool CannotOpenDoorsAutomatically { get { return __cannotOpenDoorsAutomatically__; } set { __cannotOpenDoorsAutomatically__ = value; } }
                private bool __cannotOpenDoorsAutomatically__;
                /// <summary>
                /// 16. Option :: Value: 0xF (15)
                /// </summary>
                public bool MeleeAttackersCannotAttack { get { return __meleeAttackersCannotAttack__; } set { __meleeAttackersCannotAttack__ = value; } }
                private bool __meleeAttackersCannotAttack__;
                /// <summary>
                /// 17. Option :: Value: 0x10 (16)
                /// </summary>
                public bool NotInstantlyKilledByMelee { get { return __notInstantlyKilledByMelee__; } set { __notInstantlyKilledByMelee__ = value; } }
                private bool __notInstantlyKilledByMelee__;
                /// <summary>
                /// 18. Option :: Value: 0x11 (17)
                /// </summary>
                public bool ShieldSapping { get { return __shieldSapping__; } set { __shieldSapping__ = value; } }
                private bool __shieldSapping__;
                /// <summary>
                /// 19. Option :: Value: 0x12 (18)
                /// </summary>
                public bool RunsAroundFlaming { get { return __runsAroundFlaming__; } set { __runsAroundFlaming__ = value; } }
                private bool __runsAroundFlaming__;
                /// <summary>
                /// 20. Option :: Value: 0x13 (19)
                /// </summary>
                public bool Inconsequential { get { return __inconsequential__; } set { __inconsequential__ = value; } }
                private bool __inconsequential__;
                /// <summary>
                /// 21. Option :: Value: 0x14 (20)
                /// </summary>
                public bool SpecialCinematicUnit { get { return __specialCinematicUnit__; } set { __specialCinematicUnit__ = value; } }
                private bool __specialCinematicUnit__;
                /// <summary>
                /// 22. Option :: Value: 0x15 (21)
                /// </summary>
                public bool IgnoredByAutoaiming { get { return __ignoredByAutoaiming__; } set { __ignoredByAutoaiming__ = value; } }
                private bool __ignoredByAutoaiming__;
                /// <summary>
                /// 23. Option :: Value: 0x16 (22)
                /// </summary>
                public bool ShieldsFryInfectionForms { get { return __shieldsFryInfectionForms__; } set { __shieldsFryInfectionForms__ = value; } }
                private bool __shieldsFryInfectionForms__;
                /// <summary>
                /// 24. Option :: Value: 0x17 (23)
                /// </summary>
                public bool IntegratedLightControlsWeapon { get { return __integratedLightControlsWeapon__; } set { __integratedLightControlsWeapon__ = value; } }
                private bool __integratedLightControlsWeapon__;
                /// <summary>
                /// 25. Option :: Value: 0x18 (24)
                /// </summary>
                public bool IntegratedLightLastsForever { get { return __integratedLightLastsForever__; } set { __integratedLightLastsForever__ = value; } }
                private bool __integratedLightLastsForever__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    CircularAiming = ((bit & mask) == bit);
                    bit <<= 1;
                    DestroyedAfterDying = ((bit & mask) == bit);
                    bit <<= 1;
                    HalfSpeedInterpolation = ((bit & mask) == bit);
                    bit <<= 1;
                    FiresFromCamera = ((bit & mask) == bit);
                    bit <<= 1;
                    EntranceInsideBoundingSphere = ((bit & mask) == bit);
                    bit <<= 1;
                    WeaponNotVisibleIn = ((bit & mask) == bit);
                    bit <<= 1;
                    CausesPassengerDialogue = ((bit & mask) == bit);
                    bit <<= 1;
                    ResistsPings = ((bit & mask) == bit);
                    bit <<= 1;
                    MeleeAttackIsFatal = ((bit & mask) == bit);
                    bit <<= 1;
                    DontRefaceDuringPings = ((bit & mask) == bit);
                    bit <<= 1;
                    HasNoAiming = ((bit & mask) == bit);
                    bit <<= 1;
                    SimpleCreature = ((bit & mask) == bit);
                    bit <<= 1;
                    ImpactMeleeAttachesToUnit = ((bit & mask) == bit);
                    bit <<= 1;
                    ImpactMeleeDiesOnShields = ((bit & mask) == bit);
                    bit <<= 1;
                    CannotOpenDoorsAutomatically = ((bit & mask) == bit);
                    bit <<= 1;
                    MeleeAttackersCannotAttack = ((bit & mask) == bit);
                    bit <<= 1;
                    NotInstantlyKilledByMelee = ((bit & mask) == bit);
                    bit <<= 1;
                    ShieldSapping = ((bit & mask) == bit);
                    bit <<= 1;
                    RunsAroundFlaming = ((bit & mask) == bit);
                    bit <<= 1;
                    Inconsequential = ((bit & mask) == bit);
                    bit <<= 1;
                    SpecialCinematicUnit = ((bit & mask) == bit);
                    bit <<= 1;
                    IgnoredByAutoaiming = ((bit & mask) == bit);
                    bit <<= 1;
                    ShieldsFryInfectionForms = ((bit & mask) == bit);
                    bit <<= 1;
                    IntegratedLightControlsWeapon = ((bit & mask) == bit);
                    bit <<= 1;
                    IntegratedLightLastsForever = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 38. Bitmask16 :: Offset: 0xBE (190)
            /// </summary>
            public NewFlagsBitmask16 NewFlags { get { return __newFlags__; } set { __newFlags__ = value; } }
            private NewFlagsBitmask16 __newFlags__;
            public class NewFlagsBitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0xA (10)
                /// </summary>
                public bool ControllableByParentVehiDriver { get { return __controllableByParentVehiDriver__; } set { __controllableByParentVehiDriver__ = value; } }
                private bool __controllableByParentVehiDriver__;
                /// <summary>
                /// 2. Option :: Value: 0xC (12)
                /// </summary>
                public bool HasBoost { get { return __hasBoost__; } set { __hasBoost__ = value; } }
                private bool __hasBoost__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    ControllableByParentVehiDriver = ((bit & mask) == bit);
                    bit <<= 1;
                    HasBoost = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 39. Enum16 :: Offset: 0xC0 (192)
            /// </summary>
            public DefaultTeamEnum16 DefaultTeam { get { return __defaultTeam__; } set { __defaultTeam__ = value; } }
            private DefaultTeamEnum16 __defaultTeam__;
            public class DefaultTeamEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool None { get { return __none__; } set { __none__ = value; } }
                private bool __none__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Player { get { return __player__; } set { __player__ = value; } }
                private bool __player__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Human { get { return __human__; } set { __human__ = value; } }
                private bool __human__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Covenant { get { return __covenant__; } set { __covenant__ = value; } }
                private bool __covenant__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Flood { get { return __flood__; } set { __flood__ = value; } }
                private bool __flood__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool Sentinel { get { return __sentinel__; } set { __sentinel__ = value; } }
                private bool __sentinel__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool Heretic { get { return __heretic__; } set { __heretic__ = value; } }
                private bool __heretic__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool NewCovenant { get { return __newCovenant__; } set { __newCovenant__ = value; } }
                private bool __newCovenant__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    None = (cur == value);
                    cur++;
                    Player = (cur == value);
                    cur++;
                    Human = (cur == value);
                    cur++;
                    Covenant = (cur == value);
                    cur++;
                    Flood = (cur == value);
                    cur++;
                    Sentinel = (cur == value);
                    cur++;
                    Heretic = (cur == value);
                    cur++;
                    NewCovenant = (cur == value);
                }
            }
            /// <summary>
            /// 40. Enum16 :: Offset: 0xC2 (194)
            /// </summary>
            public ConstantSoundVolumeEnum16 ConstantSoundVolume { get { return __constantSoundVolume__; } set { __constantSoundVolume__ = value; } }
            private ConstantSoundVolumeEnum16 __constantSoundVolume__;
            public class ConstantSoundVolumeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Silent { get { return __silent__; } set { __silent__ = value; } }
                private bool __silent__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Medium { get { return __medium__; } set { __medium__ = value; } }
                private bool __medium__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Loud { get { return __loud__; } set { __loud__ = value; } }
                private bool __loud__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Shout { get { return __shout__; } set { __shout__ = value; } }
                private bool __shout__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Quiet { get { return __quiet__; } set { __quiet__ = value; } }
                private bool __quiet__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    Silent = (cur == value);
                    cur++;
                    Medium = (cur == value);
                    cur++;
                    Loud = (cur == value);
                    cur++;
                    Shout = (cur == value);
                    cur++;
                    Quiet = (cur == value);
                }
            }
            /// <summary>
            /// 41. Tag :: Offset: 0xC4 (196)
            /// </summary>
            public TagRef IntegratedLightToggle { get { return __integratedLightToggle__; } set { __integratedLightToggle__ = value; } }
            private TagRef __integratedLightToggle__;
            /// <summary>
            /// 42. Float :: Offset: 0xCC (204)
            /// </summary>
            public float CameraFieldOfView { get { return __cameraFieldOfView__; } set { __cameraFieldOfView__ = value; } }
            private float __cameraFieldOfView__;
            /// <summary>
            /// 43. Float :: Offset: 0xD0 (208)
            /// </summary>
            public float CameraStiffness { get { return __cameraStiffness__; } set { __cameraStiffness__ = value; } }
            private float __cameraStiffness__;
            /// <summary>
            /// 44. StringID :: Offset: 0xD4 (212)
            /// </summary>
            public StringID CameraMarkerName { get { return __cameraMarkerName__; } set { __cameraMarkerName__ = value; } }
            private StringID __cameraMarkerName__;
            /// <summary>
            /// 45. StringID :: Offset: 0xD8 (216)
            /// </summary>
            public StringID CameraSubmergedMarkerName { get { return __cameraSubmergedMarkerName__; } set { __cameraSubmergedMarkerName__ = value; } }
            private StringID __cameraSubmergedMarkerName__;
            /// <summary>
            /// 46. Float :: Offset: 0xDC (220)
            /// </summary>
            public float PitchAutoLevel { get { return __pitchAutoLevel__; } set { __pitchAutoLevel__ = value; } }
            private float __pitchAutoLevel__;
            /// <summary>
            /// 47. Float :: Offset: 0xE0 (224)
            /// </summary>
            public float PitchRange1 { get { return __pitchRange1__; } set { __pitchRange1__ = value; } }
            private float __pitchRange1__;
            /// <summary>
            /// 48. Float :: Offset: 0xE4 (228)
            /// </summary>
            public float PitchRange2 { get { return __pitchRange2__; } set { __pitchRange2__ = value; } }
            private float __pitchRange2__;
            /// <summary>
            /// 49. TagBlock :: Offset: 0xE8 (232)
            /// </summary>
            public _CameraTracksTagBlock CameraTracks { get { return __cameraTracks__; } set { __cameraTracks__ = value; } }
            private _CameraTracksTagBlock __cameraTracks__;
            public class _CameraTracksTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Track { get { return __track__; } set { __track__ = value; } }
                private TagRef __track__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _CameraTracksTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _CameraTracksTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_CameraTracksTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _CameraTracksTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 50. Float :: Offset: 0xF0 (240)
            /// </summary>
            public float SeatAccelerationScaleI { get { return __seatAccelerationScaleI__; } set { __seatAccelerationScaleI__ = value; } }
            private float __seatAccelerationScaleI__;
            /// <summary>
            /// 51. Float :: Offset: 0xF4 (244)
            /// </summary>
            public float SeatAccelerationScaleJ { get { return __seatAccelerationScaleJ__; } set { __seatAccelerationScaleJ__ = value; } }
            private float __seatAccelerationScaleJ__;
            /// <summary>
            /// 52. Float :: Offset: 0xF8 (248)
            /// </summary>
            public float SeatAccelerationScaleK { get { return __seatAccelerationScaleK__; } set { __seatAccelerationScaleK__ = value; } }
            private float __seatAccelerationScaleK__;
            /// <summary>
            /// 53. ByteArray :: Offset: 0xFC (252)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 54. Float :: Offset: 0x104 (260)
            /// </summary>
            public float SoftPingThreshold { get { return __softPingThreshold__; } set { __softPingThreshold__ = value; } }
            private float __softPingThreshold__;
            /// <summary>
            /// 55. Float :: Offset: 0x108 (264)
            /// </summary>
            public float SoftPingInterruptTime { get { return __softPingInterruptTime__; } set { __softPingInterruptTime__ = value; } }
            private float __softPingInterruptTime__;
            /// <summary>
            /// 56. Float :: Offset: 0x10C (268)
            /// </summary>
            public float HardPingThreshold { get { return __hardPingThreshold__; } set { __hardPingThreshold__ = value; } }
            private float __hardPingThreshold__;
            /// <summary>
            /// 57. Float :: Offset: 0x110 (272)
            /// </summary>
            public float HardPingInterruptTime { get { return __hardPingInterruptTime__; } set { __hardPingInterruptTime__ = value; } }
            private float __hardPingInterruptTime__;
            /// <summary>
            /// 58. Float :: Offset: 0x114 (276)
            /// </summary>
            public float HardPingDeathThreshold { get { return __hardPingDeathThreshold__; } set { __hardPingDeathThreshold__ = value; } }
            private float __hardPingDeathThreshold__;
            /// <summary>
            /// 59. Float :: Offset: 0x118 (280)
            /// </summary>
            public float FeignDeathThreshold { get { return __feignDeathThreshold__; } set { __feignDeathThreshold__ = value; } }
            private float __feignDeathThreshold__;
            /// <summary>
            /// 60. Float :: Offset: 0x11C (284)
            /// </summary>
            public float FeignDeathTime { get { return __feignDeathTime__; } set { __feignDeathTime__ = value; } }
            private float __feignDeathTime__;
            /// <summary>
            /// 61. Float :: Offset: 0x120 (288)
            /// </summary>
            public float DistOfEvadeAnim { get { return __distOfEvadeAnim__; } set { __distOfEvadeAnim__ = value; } }
            private float __distOfEvadeAnim__;
            /// <summary>
            /// 62. Float :: Offset: 0x124 (292)
            /// </summary>
            public float DistOfDiveAnim { get { return __distOfDiveAnim__; } set { __distOfDiveAnim__ = value; } }
            private float __distOfDiveAnim__;
            /// <summary>
            /// 63. Float :: Offset: 0x128 (296)
            /// </summary>
            public float StunnedMovementThreshold { get { return __stunnedMovementThreshold__; } set { __stunnedMovementThreshold__ = value; } }
            private float __stunnedMovementThreshold__;
            /// <summary>
            /// 64. Float :: Offset: 0x12C (300)
            /// </summary>
            public float FeignDeathChance { get { return __feignDeathChance__; } set { __feignDeathChance__ = value; } }
            private float __feignDeathChance__;
            /// <summary>
            /// 65. Float :: Offset: 0x130 (304)
            /// </summary>
            public float FeignRepeatChance { get { return __feignRepeatChance__; } set { __feignRepeatChance__ = value; } }
            private float __feignRepeatChance__;
            /// <summary>
            /// 66. Tag :: Offset: 0x134 (308)
            /// </summary>
            public TagRef SpawnedActor { get { return __spawnedActor__; } set { __spawnedActor__ = value; } }
            private TagRef __spawnedActor__;
            /// <summary>
            /// 67. Int16 :: Offset: 0x13C (316)
            /// </summary>
            public short SpawnedActorCountLower { get { return __spawnedActorCountLower__; } set { __spawnedActorCountLower__ = value; } }
            private short __spawnedActorCountLower__;
            /// <summary>
            /// 68. Int16 :: Offset: 0x13E (318)
            /// </summary>
            public short SpawnedActorCountUpper { get { return __spawnedActorCountUpper__; } set { __spawnedActorCountUpper__ = value; } }
            private short __spawnedActorCountUpper__;
            /// <summary>
            /// 69. Float :: Offset: 0x140 (320)
            /// </summary>
            public float SpawnedVelocity { get { return __spawnedVelocity__; } set { __spawnedVelocity__ = value; } }
            private float __spawnedVelocity__;
            /// <summary>
            /// 70. Float :: Offset: 0x144 (324)
            /// </summary>
            public float AimingVelocityMax { get { return __aimingVelocityMax__; } set { __aimingVelocityMax__ = value; } }
            private float __aimingVelocityMax__;
            /// <summary>
            /// 71. Float :: Offset: 0x148 (328)
            /// </summary>
            public float AimingAccelMax { get { return __aimingAccelMax__; } set { __aimingAccelMax__ = value; } }
            private float __aimingAccelMax__;
            /// <summary>
            /// 72. Float :: Offset: 0x14C (332)
            /// </summary>
            public float CasualAimingModifier { get { return __casualAimingModifier__; } set { __casualAimingModifier__ = value; } }
            private float __casualAimingModifier__;
            /// <summary>
            /// 73. Float :: Offset: 0x150 (336)
            /// </summary>
            public float LookingVelocityMax { get { return __lookingVelocityMax__; } set { __lookingVelocityMax__ = value; } }
            private float __lookingVelocityMax__;
            /// <summary>
            /// 74. Float :: Offset: 0x154 (340)
            /// </summary>
            public float LookingAccelMax { get { return __lookingAccelMax__; } set { __lookingAccelMax__ = value; } }
            private float __lookingAccelMax__;
            /// <summary>
            /// 75. StringID :: Offset: 0x158 (344)
            /// </summary>
            public StringID WeaponPlacementMarker { get { return __weaponPlacementMarker__; } set { __weaponPlacementMarker__ = value; } }
            private StringID __weaponPlacementMarker__;
            /// <summary>
            /// 76. StringID :: Offset: 0x15C (348)
            /// </summary>
            public StringID LeftHand { get { return __leftHand__; } set { __leftHand__ = value; } }
            private StringID __leftHand__;
            /// <summary>
            /// 77. StringID :: Offset: 0x160 (352)
            /// </summary>
            public StringID RightHand { get { return __rightHand__; } set { __rightHand__ = value; } }
            private StringID __rightHand__;
            /// <summary>
            /// 78. Tag :: Offset: 0x164 (356)
            /// </summary>
            public TagRef MeleeDamage { get { return __meleeDamage__; } set { __meleeDamage__ = value; } }
            private TagRef __meleeDamage__;
            /// <summary>
            /// 79. Tag :: Offset: 0x16C (364)
            /// </summary>
            public TagRef BoardingMeleeDamage { get { return __boardingMeleeDamage__; } set { __boardingMeleeDamage__ = value; } }
            private TagRef __boardingMeleeDamage__;
            /// <summary>
            /// 80. Tag :: Offset: 0x174 (372)
            /// </summary>
            public TagRef BoardingMeleeResponseDamage { get { return __boardingMeleeResponseDamage__; } set { __boardingMeleeResponseDamage__ = value; } }
            private TagRef __boardingMeleeResponseDamage__;
            /// <summary>
            /// 81. Tag :: Offset: 0x17C (380)
            /// </summary>
            public TagRef Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private TagRef __unknown1__;
            /// <summary>
            /// 82. Tag :: Offset: 0x184 (388)
            /// </summary>
            public TagRef Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private TagRef __unknown2__;
            /// <summary>
            /// 83. Tag :: Offset: 0x18C (396)
            /// </summary>
            public TagRef Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private TagRef __unknown3__;
            /// <summary>
            /// 84. Enum32 :: Offset: 0x194 (404)
            /// </summary>
            public MotionSensorBlipSizeEnum32 MotionSensorBlipSize { get { return __motionSensorBlipSize__; } set { __motionSensorBlipSize__ = value; } }
            private MotionSensorBlipSizeEnum32 __motionSensorBlipSize__;
            public class MotionSensorBlipSizeEnum32
            {
                /// <summary>
                /// 1. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Medium { get { return __medium__; } set { __medium__ = value; } }
                private bool __medium__;
                /// <summary>
                /// 2. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Small { get { return __small__; } set { __small__ = value; } }
                private bool __small__;
                /// <summary>
                /// 3. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Large { get { return __large__; } set { __large__ = value; } }
                private bool __large__;

                public void _Initialize(BitReader br)
                {
                    uint value = br.ReadUInt32();
                    uint cur = 0;

                    Medium = (cur == value);
                    cur++;
                    Small = (cur == value);
                    cur++;
                    Large = (cur == value);
                }
            }
            /// <summary>
            /// 85. TagBlock :: Offset: 0x198 (408)
            /// </summary>
            public _Unknown4TagBlock Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private _Unknown4TagBlock __unknown4__;
            public class _Unknown4TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private StringID __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown4TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown4TagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown4TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown4TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 86. TagBlock :: Offset: 0x1A0 (416)
            /// </summary>
            public _HudInterfacesTagBlock HudInterfaces { get { return __hudInterfaces__; } set { __hudInterfaces__ = value; } }
            private _HudInterfacesTagBlock __hudInterfaces__;
            public class _HudInterfacesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef UnitHudInterface { get { return __unitHudInterface__; } set { __unitHudInterface__ = value; } }
                private TagRef __unitHudInterface__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _HudInterfacesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _HudInterfacesTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_HudInterfacesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _HudInterfacesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 87. TagBlock :: Offset: 0x1A8 (424)
            /// </summary>
            public _Unknown5TagBlock Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private _Unknown5TagBlock __unknown5__;
            public class _Unknown5TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. ByteArray :: Offset: 0x0 (0)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 2. ByteArray :: Offset: 0x4 (4)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 3. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 4. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 5. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown5TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown5TagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown5TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown5TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 88. Float :: Offset: 0x1B0 (432)
            /// </summary>
            public float GrenadeVelocity { get { return __grenadeVelocity__; } set { __grenadeVelocity__ = value; } }
            private float __grenadeVelocity__;
            /// <summary>
            /// 89. Enum16 :: Offset: 0x1B4 (436)
            /// </summary>
            public GrenadeTypeEnum16 GrenadeType { get { return __grenadeType__; } set { __grenadeType__ = value; } }
            private GrenadeTypeEnum16 __grenadeType__;
            public class GrenadeTypeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool HumanFragmentation { get { return __humanFragmentation__; } set { __humanFragmentation__ = value; } }
                private bool __humanFragmentation__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool CovenantPlasma { get { return __covenantPlasma__; } set { __covenantPlasma__ = value; } }
                private bool __covenantPlasma__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    HumanFragmentation = (cur == value);
                    cur++;
                    CovenantPlasma = (cur == value);
                }
            }
            /// <summary>
            /// 90. UInt16 :: Offset: 0x1B6 (438)
            /// </summary>
            public ushort GrenadeCount { get { return __grenadeCount__; } set { __grenadeCount__ = value; } }
            private ushort __grenadeCount__;
            /// <summary>
            /// 91. TagBlock :: Offset: 0x1B8 (440)
            /// </summary>
            public _PoweredSeatsTagBlock PoweredSeats { get { return __poweredSeats__; } set { __poweredSeats__ = value; } }
            private _PoweredSeatsTagBlock __poweredSeats__;
            public class _PoweredSeatsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float DriverPowerupTime { get { return __driverPowerupTime__; } set { __driverPowerupTime__ = value; } }
                private float __driverPowerupTime__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float DriverPowerdownTime { get { return __driverPowerdownTime__; } set { __driverPowerdownTime__ = value; } }
                private float __driverPowerdownTime__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PoweredSeatsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PoweredSeatsTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PoweredSeatsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PoweredSeatsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 92. TagBlock :: Offset: 0x1C0 (448)
            /// </summary>
            public _WeaponTagBlock Weapon { get { return __weapon__; } set { __weapon__ = value; } }
            private _WeaponTagBlock __weapon__;
            public class _WeaponTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Weapon { get { return __weapon__; } set { __weapon__ = value; } }
                private TagRef __weapon__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WeaponTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WeaponTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WeaponTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WeaponTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 93. TagBlock :: Offset: 0x1C8 (456)
            /// </summary>
            public _SeatsTagBlock Seats { get { return __seats__; } set { __seats__ = value; } }
            private _SeatsTagBlock __seats__;
            public class _SeatsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool DriverVanishes { get { return __driverVanishes__; } set { __driverVanishes__ = value; } }
                    private bool __driverVanishes__;
                    /// <summary>
                    /// 2. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool DriverSeat { get { return __driverSeat__; } set { __driverSeat__ = value; } }
                    private bool __driverSeat__;
                    /// <summary>
                    /// 3. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool UseVehicleGun { get { return __useVehicleGun__; } set { __useVehicleGun__ = value; } }
                    private bool __useVehicleGun__;
                    /// <summary>
                    /// 4. Option :: Value: 0x5 (5)
                    /// </summary>
                    public bool UsePlayerWeapon { get { return __usePlayerWeapon__; } set { __usePlayerWeapon__ = value; } }
                    private bool __usePlayerWeapon__;
                    /// <summary>
                    /// 5. Option :: Value: 0x8 (8)
                    /// </summary>
                    public bool TurretFire { get { return __turretFire__; } set { __turretFire__ = value; } }
                    private bool __turretFire__;
                    /// <summary>
                    /// 6. Option :: Value: 0xB (11)
                    /// </summary>
                    public bool AwardBoardingMedal { get { return __awardBoardingMedal__; } set { __awardBoardingMedal__ = value; } }
                    private bool __awardBoardingMedal__;
                    /// <summary>
                    /// 7. Option :: Value: 0xC (12)
                    /// </summary>
                    public bool EnableHUD { get { return __enableHUD__; } set { __enableHUD__ = value; } }
                    private bool __enableHUD__;
                    /// <summary>
                    /// 8. Option :: Value: 0xF (15)
                    /// </summary>
                    public bool CloseDoor { get { return __closeDoor__; } set { __closeDoor__ = value; } }
                    private bool __closeDoor__;
                    /// <summary>
                    /// 9. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool N { get { return __n__; } set { __n__ = value; } }
                    private bool __n__;
                    /// <summary>
                    /// 10. Option :: Value: 0x9 (9)
                    /// </summary>
                    public bool DisableSeat { get { return __disableSeat__; } set { __disableSeat__ = value; } }
                    private bool __disableSeat__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        DriverVanishes = ((bit & mask) == bit);
                        bit <<= 1;
                        DriverSeat = ((bit & mask) == bit);
                        bit <<= 1;
                        UseVehicleGun = ((bit & mask) == bit);
                        bit <<= 1;
                        UsePlayerWeapon = ((bit & mask) == bit);
                        bit <<= 1;
                        TurretFire = ((bit & mask) == bit);
                        bit <<= 1;
                        AwardBoardingMedal = ((bit & mask) == bit);
                        bit <<= 1;
                        EnableHUD = ((bit & mask) == bit);
                        bit <<= 1;
                        CloseDoor = ((bit & mask) == bit);
                        bit <<= 1;
                        N = ((bit & mask) == bit);
                        bit <<= 1;
                        DisableSeat = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Name { get { return __name__; } set { __name__ = value; } }
                private StringID __name__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID SittingPositionMarker { get { return __sittingPositionMarker__; } set { __sittingPositionMarker__ = value; } }
                private StringID __sittingPositionMarker__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID EnterMarker { get { return __enterMarker__; } set { __enterMarker__ = value; } }
                private StringID __enterMarker__;
                /// <summary>
                /// 5. StringID :: Offset: 0x10 (16)
                /// </summary>
                public StringID GrenadeMarker { get { return __grenadeMarker__; } set { __grenadeMarker__ = value; } }
                private StringID __grenadeMarker__;
                /// <summary>
                /// 6. StringID :: Offset: 0x14 (20)
                /// </summary>
                public StringID Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private StringID __unknown1__;
                /// <summary>
                /// 7. StringID :: Offset: 0x18 (24)
                /// </summary>
                public StringID Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private StringID __unknown2__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float ForceOutAnglePossibly { get { return __forceOutAnglePossibly__; } set { __forceOutAnglePossibly__ = value; } }
                private float __forceOutAnglePossibly__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float AccelScaleI { get { return __accelScaleI__; } set { __accelScaleI__ = value; } }
                private float __accelScaleI__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float AccelScaleJ { get { return __accelScaleJ__; } set { __accelScaleJ__ = value; } }
                private float __accelScaleJ__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float AccelScaleK { get { return __accelScaleK__; } set { __accelScaleK__ = value; } }
                private float __accelScaleK__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float YawRate { get { return __yawRate__; } set { __yawRate__ = value; } }
                private float __yawRate__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float PitchRate { get { return __pitchRate__; } set { __pitchRate__ = value; } }
                private float __pitchRate__;
                /// <summary>
                /// 15. Int16 :: Offset: 0x38 (56)
                /// </summary>
                public short Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private short __unknown4__;
                /// <summary>
                /// 16. Int16 :: Offset: 0x3A (58)
                /// </summary>
                public short Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private short __unknown5__;
                /// <summary>
                /// 17. Int16 :: Offset: 0x3C (60)
                /// </summary>
                public short BitmaskPossibly { get { return __bitmaskPossibly__; } set { __bitmaskPossibly__ = value; } }
                private short __bitmaskPossibly__;
                /// <summary>
                /// 18. Int16 :: Offset: 0x3E (62)
                /// </summary>
                public short BoardWhichSeatPossibly { get { return __boardWhichSeatPossibly__; } set { __boardWhichSeatPossibly__ = value; } }
                private short __boardWhichSeatPossibly__;
                /// <summary>
                /// 19. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 20. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 21. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 22. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 23. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 24. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 25. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 26. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 27. StringID :: Offset: 0x60 (96)
                /// </summary>
                public StringID CameraMarkerName { get { return __cameraMarkerName__; } set { __cameraMarkerName__ = value; } }
                private StringID __cameraMarkerName__;
                /// <summary>
                /// 28. StringID :: Offset: 0x64 (100)
                /// </summary>
                public StringID CameraSubmergedMarkerName { get { return __cameraSubmergedMarkerName__; } set { __cameraSubmergedMarkerName__ = value; } }
                private StringID __cameraSubmergedMarkerName__;
                /// <summary>
                /// 29. Float :: Offset: 0x68 (104)
                /// </summary>
                public float PitchAutoLevel { get { return __pitchAutoLevel__; } set { __pitchAutoLevel__ = value; } }
                private float __pitchAutoLevel__;
                /// <summary>
                /// 30. Float :: Offset: 0x6C (108)
                /// </summary>
                public float PitchRangeLower { get { return __pitchRangeLower__; } set { __pitchRangeLower__ = value; } }
                private float __pitchRangeLower__;
                /// <summary>
                /// 31. Float :: Offset: 0x70 (112)
                /// </summary>
                public float PitchRangeUpper { get { return __pitchRangeUpper__; } set { __pitchRangeUpper__ = value; } }
                private float __pitchRangeUpper__;
                /// <summary>
                /// 32. TagBlock :: Offset: 0x74 (116)
                /// </summary>
                public _CameraTracksTagBlock CameraTracks { get { return __cameraTracks__; } set { __cameraTracks__ = value; } }
                private _CameraTracksTagBlock __cameraTracks__;
                public class _CameraTracksTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private VehiLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef Camera { get { return __camera__; } set { __camera__ = value; } }
                    private TagRef __camera__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _CameraTracksTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _CameraTracksTagBlock(VehiLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_CameraTracksTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _CameraTracksTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 33. TagBlock :: Offset: 0x7C (124)
                /// </summary>
                public _UnitHudInterfaceTagBlock UnitHudInterface { get { return __unitHudInterface__; } set { __unitHudInterface__ = value; } }
                private _UnitHudInterfaceTagBlock __unitHudInterface__;
                public class _UnitHudInterfaceTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private VehiLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Tag :: Offset: 0x0 (0)
                    /// </summary>
                    public TagRef Hud { get { return __hud__; } set { __hud__ = value; } }
                    private TagRef __hud__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _UnitHudInterfaceTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _UnitHudInterfaceTagBlock(VehiLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_UnitHudInterfaceTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _UnitHudInterfaceTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 34. StringID :: Offset: 0x84 (132)
                /// </summary>
                public StringID HudTextMessageIndex { get { return __hudTextMessageIndex__; } set { __hudTextMessageIndex__ = value; } }
                private StringID __hudTextMessageIndex__;
                /// <summary>
                /// 35. Float :: Offset: 0x88 (136)
                /// </summary>
                public float YawMin { get { return __yawMin__; } set { __yawMin__ = value; } }
                private float __yawMin__;
                /// <summary>
                /// 36. Float :: Offset: 0x8C (140)
                /// </summary>
                public float YawMax { get { return __yawMax__; } set { __yawMax__ = value; } }
                private float __yawMax__;
                /// <summary>
                /// 37. Tag :: Offset: 0x90 (144)
                /// </summary>
                public TagRef BuiltInGunner { get { return __builtInGunner__; } set { __builtInGunner__ = value; } }
                private TagRef __builtInGunner__;
                /// <summary>
                /// 38. Float :: Offset: 0x98 (152)
                /// </summary>
                public float SeatEnterRadius { get { return __seatEnterRadius__; } set { __seatEnterRadius__ = value; } }
                private float __seatEnterRadius__;
                /// <summary>
                /// 39. Float :: Offset: 0x9C (156)
                /// </summary>
                public float SeatEnterStartAngle { get { return __seatEnterStartAngle__; } set { __seatEnterStartAngle__ = value; } }
                private float __seatEnterStartAngle__;
                /// <summary>
                /// 40. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float SeatEnterEndAngle { get { return __seatEnterEndAngle__; } set { __seatEnterEndAngle__ = value; } }
                private float __seatEnterEndAngle__;
                /// <summary>
                /// 41. Float :: Offset: 0xA4 (164)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 42. StringID :: Offset: 0xA8 (168)
                /// </summary>
                public StringID Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private StringID __unknown15__;
                /// <summary>
                /// 43. ID :: Offset: 0xAC (172)
                /// </summary>
                public LoneID Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private LoneID __unknown16__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _SeatsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _SeatsTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                    CameraTracks = new _CameraTracksTagBlock(layout);
                    UnitHudInterface = new _UnitHudInterfaceTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_SeatsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _SeatsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 94. Float :: Offset: 0x1D0 (464)
            /// </summary>
            public float BoostSpeedMultiplier { get { return __boostSpeedMultiplier__; } set { __boostSpeedMultiplier__ = value; } }
            private float __boostSpeedMultiplier__;
            /// <summary>
            /// 95. Float :: Offset: 0x1D4 (468)
            /// </summary>
            public float TimeUntilNextBoost { get { return __timeUntilNextBoost__; } set { __timeUntilNextBoost__ = value; } }
            private float __timeUntilNextBoost__;
            /// <summary>
            /// 96. Float :: Offset: 0x1D8 (472)
            /// </summary>
            public float BoostLength { get { return __boostLength__; } set { __boostLength__ = value; } }
            private float __boostLength__;
            /// <summary>
            /// 97. Float :: Offset: 0x1DC (476)
            /// </summary>
            public float BoostTime { get { return __boostTime__; } set { __boostTime__ = value; } }
            private float __boostTime__;
            /// <summary>
            /// 98. Float :: Offset: 0x1E0 (480)
            /// </summary>
            public float BoostAccel { get { return __boostAccel__; } set { __boostAccel__ = value; } }
            private float __boostAccel__;
            /// <summary>
            /// 99. ByteArray :: Offset: 0x1E4 (484)
            /// </summary>
            public byte[] Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private byte[] __unused3__;
            /// <summary>
            /// 100. Bitmask16 :: Offset: 0x1EC (492)
            /// </summary>
            public OldFlags2Bitmask16 OldFlags2 { get { return __oldFlags2__; } set { __oldFlags2__ = value; } }
            private OldFlags2Bitmask16 __oldFlags2__;
            public class OldFlags2Bitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool SpeedWakesPhysics { get { return __speedWakesPhysics__; } set { __speedWakesPhysics__ = value; } }
                private bool __speedWakesPhysics__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool TurnWakesPhysics { get { return __turnWakesPhysics__; } set { __turnWakesPhysics__ = value; } }
                private bool __turnWakesPhysics__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool DriverPowerWakesPhysics { get { return __driverPowerWakesPhysics__; } set { __driverPowerWakesPhysics__ = value; } }
                private bool __driverPowerWakesPhysics__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool ControlOppositeSpeedSetsBrake { get { return __controlOppositeSpeedSetsBrake__; } set { __controlOppositeSpeedSetsBrake__ = value; } }
                private bool __controlOppositeSpeedSetsBrake__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool SlideWakesPhysics { get { return __slideWakesPhysics__; } set { __slideWakesPhysics__ = value; } }
                private bool __slideWakesPhysics__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool KillsRidersAtTerminalVelocity { get { return __killsRidersAtTerminalVelocity__; } set { __killsRidersAtTerminalVelocity__ = value; } }
                private bool __killsRidersAtTerminalVelocity__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool CausesCollisionDamage { get { return __causesCollisionDamage__; } set { __causesCollisionDamage__ = value; } }
                private bool __causesCollisionDamage__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool AIWeaponCannotRotate { get { return __aIWeaponCannotRotate__; } set { __aIWeaponCannotRotate__ = value; } }
                private bool __aIWeaponCannotRotate__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool AIDoesNotRequireDriver { get { return __aIDoesNotRequireDriver__; } set { __aIDoesNotRequireDriver__ = value; } }
                private bool __aIDoesNotRequireDriver__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool AIUnused { get { return __aIUnused__; } set { __aIUnused__ = value; } }
                private bool __aIUnused__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool AIDriverEnable { get { return __aIDriverEnable__; } set { __aIDriverEnable__ = value; } }
                private bool __aIDriverEnable__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool AIDriverFlying { get { return __aIDriverFlying__; } set { __aIDriverFlying__ = value; } }
                private bool __aIDriverFlying__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool AIDriverCanSidestep { get { return __aIDriverCanSidestep__; } set { __aIDriverCanSidestep__ = value; } }
                private bool __aIDriverCanSidestep__;
                /// <summary>
                /// 14. Option :: Value: 0xD (13)
                /// </summary>
                public bool AIDriverHovering { get { return __aIDriverHovering__; } set { __aIDriverHovering__ = value; } }
                private bool __aIDriverHovering__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    SpeedWakesPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    TurnWakesPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    DriverPowerWakesPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    ControlOppositeSpeedSetsBrake = ((bit & mask) == bit);
                    bit <<= 1;
                    SlideWakesPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    KillsRidersAtTerminalVelocity = ((bit & mask) == bit);
                    bit <<= 1;
                    CausesCollisionDamage = ((bit & mask) == bit);
                    bit <<= 1;
                    AIWeaponCannotRotate = ((bit & mask) == bit);
                    bit <<= 1;
                    AIDoesNotRequireDriver = ((bit & mask) == bit);
                    bit <<= 1;
                    AIUnused = ((bit & mask) == bit);
                    bit <<= 1;
                    AIDriverEnable = ((bit & mask) == bit);
                    bit <<= 1;
                    AIDriverFlying = ((bit & mask) == bit);
                    bit <<= 1;
                    AIDriverCanSidestep = ((bit & mask) == bit);
                    bit <<= 1;
                    AIDriverHovering = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 101. Bitmask16 :: Offset: 0x1EE (494)
            /// </summary>
            public Flags2Bitmask16 Flags2 { get { return __flags2__; } set { __flags2__ = value; } }
            private Flags2Bitmask16 __flags2__;
            public class Flags2Bitmask16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private bool __unknown1__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private bool __unknown2__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Handbrake { get { return __handbrake__; } set { __handbrake__ = value; } }
                private bool __handbrake__;

                public void _Initialize(BitReader br)
                {
                    ushort mask = br.ReadUInt16();
                    ushort bit = 1;

                    Unknown1 = ((bit & mask) == bit);
                    bit <<= 1;
                    Unknown2 = ((bit & mask) == bit);
                    bit <<= 1;
                    Handbrake = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 102. Enum16 :: Offset: 0x1F0 (496)
            /// </summary>
            public TypeEnum16 Type { get { return __type__; } set { __type__ = value; } }
            private TypeEnum16 __type__;
            public class TypeEnum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool HumanTank { get { return __humanTank__; } set { __humanTank__ = value; } }
                private bool __humanTank__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool HumanJeep { get { return __humanJeep__; } set { __humanJeep__ = value; } }
                private bool __humanJeep__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool HumanBoat { get { return __humanBoat__; } set { __humanBoat__ = value; } }
                private bool __humanBoat__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool HumanPlane { get { return __humanPlane__; } set { __humanPlane__ = value; } }
                private bool __humanPlane__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool CovieScout { get { return __covieScout__; } set { __covieScout__ = value; } }
                private bool __covieScout__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool CovieFighter { get { return __covieFighter__; } set { __covieFighter__ = value; } }
                private bool __covieFighter__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool Turret { get { return __turret__; } set { __turret__ = value; } }
                private bool __turret__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    HumanTank = (cur == value);
                    cur++;
                    HumanJeep = (cur == value);
                    cur++;
                    HumanBoat = (cur == value);
                    cur++;
                    HumanPlane = (cur == value);
                    cur++;
                    CovieScout = (cur == value);
                    cur++;
                    CovieFighter = (cur == value);
                    cur++;
                    Turret = (cur == value);
                }
            }
            /// <summary>
            /// 103. Enum16 :: Offset: 0x1F2 (498)
            /// </summary>
            public Unknown6Enum16 Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private Unknown6Enum16 __unknown6__;
            public class Unknown6Enum16
            {

                public void _Initialize(BitReader br)
                {
                    // Empty..
                    br.BaseStream.Position += 2;
                }
            }
            /// <summary>
            /// 104. Float :: Offset: 0x1F4 (500)
            /// </summary>
            public float CovieMaxForwardSpeed { get { return __covieMaxForwardSpeed__; } set { __covieMaxForwardSpeed__ = value; } }
            private float __covieMaxForwardSpeed__;
            /// <summary>
            /// 105. Float :: Offset: 0x1F8 (504)
            /// </summary>
            public float CovieMaxReverseSpeed { get { return __covieMaxReverseSpeed__; } set { __covieMaxReverseSpeed__ = value; } }
            private float __covieMaxReverseSpeed__;
            /// <summary>
            /// 106. Float :: Offset: 0x1FC (508)
            /// </summary>
            public float CovieSpeedAccel { get { return __covieSpeedAccel__; } set { __covieSpeedAccel__ = value; } }
            private float __covieSpeedAccel__;
            /// <summary>
            /// 107. Float :: Offset: 0x200 (512)
            /// </summary>
            public float CovieSpeedDecel { get { return __covieSpeedDecel__; } set { __covieSpeedDecel__ = value; } }
            private float __covieSpeedDecel__;
            /// <summary>
            /// 108. Float :: Offset: 0x204 (516)
            /// </summary>
            public float CovieMaxLeftTurn { get { return __covieMaxLeftTurn__; } set { __covieMaxLeftTurn__ = value; } }
            private float __covieMaxLeftTurn__;
            /// <summary>
            /// 109. Float :: Offset: 0x208 (520)
            /// </summary>
            public float CovieMaxRightTurn_negative { get { return __covieMaxRightTurn_negative__; } set { __covieMaxRightTurn_negative__ = value; } }
            private float __covieMaxRightTurn_negative__;
            /// <summary>
            /// 110. Float :: Offset: 0x20C (524)
            /// </summary>
            public float WheelCircumference_TurningCircleMbPossibly { get { return __wheelCircumference_TurningCircleMbPossibly__; } set { __wheelCircumference_TurningCircleMbPossibly__ = value; } }
            private float __wheelCircumference_TurningCircleMbPossibly__;
            /// <summary>
            /// 111. Float :: Offset: 0x210 (528)
            /// </summary>
            public float TurnRate { get { return __turnRate__; } set { __turnRate__ = value; } }
            private float __turnRate__;
            /// <summary>
            /// 112. Float :: Offset: 0x214 (532)
            /// </summary>
            public float BlurSpeed { get { return __blurSpeed__; } set { __blurSpeed__ = value; } }
            private float __blurSpeed__;
            /// <summary>
            /// 113. Enum16 :: Offset: 0x218 (536)
            /// </summary>
            public Unknown7Enum16 Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private Unknown7Enum16 __unknown7__;
            public class Unknown7Enum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private bool __unknown1__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private bool __unknown2__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private bool __unknown3__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private bool __unknown4__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private bool __unknown5__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private bool __unknown6__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private bool __unknown7__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    Unknown1 = (cur == value);
                    cur++;
                    Unknown2 = (cur == value);
                    cur++;
                    Unknown3 = (cur == value);
                    cur++;
                    Unknown4 = (cur == value);
                    cur++;
                    Unknown5 = (cur == value);
                    cur++;
                    Unknown6 = (cur == value);
                    cur++;
                    Unknown7 = (cur == value);
                }
            }
            /// <summary>
            /// 114. Enum16 :: Offset: 0x21A (538)
            /// </summary>
            public Unknown8Enum16 Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private Unknown8Enum16 __unknown8__;
            public class Unknown8Enum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool EverythingElse { get { return __everythingElse__; } set { __everythingElse__ = value; } }
                private bool __everythingElse__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Warthog { get { return __warthog__; } set { __warthog__ = value; } }
                private bool __warthog__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private bool __unknown__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool Ghost { get { return __ghost__; } set { __ghost__ = value; } }
                private bool __ghost__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool Banshee { get { return __banshee__; } set { __banshee__ = value; } }
                private bool __banshee__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    EverythingElse = (cur == value);
                    cur++;
                    Warthog = (cur == value);
                    cur++;
                    Unknown = (cur == value);
                    cur++;
                    Ghost = (cur == value);
                    cur++;
                    Banshee = (cur == value);
                }
            }
            /// <summary>
            /// 115. StringID :: Offset: 0x21C (540)
            /// </summary>
            public StringID FlipMessage { get { return __flipMessage__; } set { __flipMessage__ = value; } }
            private StringID __flipMessage__;
            /// <summary>
            /// 116. Float :: Offset: 0x220 (544)
            /// </summary>
            public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private float __unknown9__;
            /// <summary>
            /// 117. Float :: Offset: 0x224 (548)
            /// </summary>
            public float CovieMaxLeftSlide { get { return __covieMaxLeftSlide__; } set { __covieMaxLeftSlide__ = value; } }
            private float __covieMaxLeftSlide__;
            /// <summary>
            /// 118. Float :: Offset: 0x228 (552)
            /// </summary>
            public float CovieMaxRightSlide { get { return __covieMaxRightSlide__; } set { __covieMaxRightSlide__ = value; } }
            private float __covieMaxRightSlide__;
            /// <summary>
            /// 119. Float :: Offset: 0x22C (556)
            /// </summary>
            public float CovieVehiStrafeSpeedLeft { get { return __covieVehiStrafeSpeedLeft__; } set { __covieVehiStrafeSpeedLeft__ = value; } }
            private float __covieVehiStrafeSpeedLeft__;
            /// <summary>
            /// 120. Float :: Offset: 0x230 (560)
            /// </summary>
            public float CovieVehiStrafeSpeedRight { get { return __covieVehiStrafeSpeedRight__; } set { __covieVehiStrafeSpeedRight__ = value; } }
            private float __covieVehiStrafeSpeedRight__;
            /// <summary>
            /// 121. Float :: Offset: 0x234 (564)
            /// </summary>
            public float CovieVehiStrafeAccel { get { return __covieVehiStrafeAccel__; } set { __covieVehiStrafeAccel__ = value; } }
            private float __covieVehiStrafeAccel__;
            /// <summary>
            /// 122. Float :: Offset: 0x238 (568)
            /// </summary>
            public float CovieVehiStrafeDecel { get { return __covieVehiStrafeDecel__; } set { __covieVehiStrafeDecel__ = value; } }
            private float __covieVehiStrafeDecel__;
            /// <summary>
            /// 123. Float :: Offset: 0x23C (572)
            /// </summary>
            public float MinimumFlippingAngularVelocity { get { return __minimumFlippingAngularVelocity__; } set { __minimumFlippingAngularVelocity__ = value; } }
            private float __minimumFlippingAngularVelocity__;
            /// <summary>
            /// 124. Float :: Offset: 0x240 (576)
            /// </summary>
            public float MaximumFlippingAngularVelocity { get { return __maximumFlippingAngularVelocity__; } set { __maximumFlippingAngularVelocity__ = value; } }
            private float __maximumFlippingAngularVelocity__;
            /// <summary>
            /// 125. Enum16 :: Offset: 0x244 (580)
            /// </summary>
            public Unknown10Enum16 Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private Unknown10Enum16 __unknown10__;
            public class Unknown10Enum16
            {

                public void _Initialize(BitReader br)
                {
                    // Empty..
                    br.BaseStream.Position += 2;
                }
            }
            /// <summary>
            /// 126. Int16 :: Offset: 0x244 (580)
            /// </summary>
            public short Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private short __unknown11__;
            /// <summary>
            /// 127. ByteArray :: Offset: 0x246 (582)
            /// </summary>
            public byte[] Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private byte[] __unknown12__;
            /// <summary>
            /// 128. ByteArray :: Offset: 0x248 (584)
            /// </summary>
            public byte[] Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
            private byte[] __unused4__;
            /// <summary>
            /// 129. Float :: Offset: 0x24C (588)
            /// </summary>
            public float FixedGunYaw { get { return __fixedGunYaw__; } set { __fixedGunYaw__ = value; } }
            private float __fixedGunYaw__;
            /// <summary>
            /// 130. Float :: Offset: 0x250 (592)
            /// </summary>
            public float FixedGunPitch { get { return __fixedGunPitch__; } set { __fixedGunPitch__ = value; } }
            private float __fixedGunPitch__;
            /// <summary>
            /// 131. Float :: Offset: 0x254 (596)
            /// </summary>
            public float WheelWobble { get { return __wheelWobble__; } set { __wheelWobble__ = value; } }
            private float __wheelWobble__;
            /// <summary>
            /// 132. Float :: Offset: 0x258 (600)
            /// </summary>
            public float UnknownHumanMaxForwardSpeedMbPossibly { get { return __unknownHumanMaxForwardSpeedMbPossibly__; } set { __unknownHumanMaxForwardSpeedMbPossibly__ = value; } }
            private float __unknownHumanMaxForwardSpeedMbPossibly__;
            /// <summary>
            /// 133. Float :: Offset: 0x25C (604)
            /// </summary>
            public float UnknownHumanMaxReverseSpeedMbPossibly { get { return __unknownHumanMaxReverseSpeedMbPossibly__; } set { __unknownHumanMaxReverseSpeedMbPossibly__ = value; } }
            private float __unknownHumanMaxReverseSpeedMbPossibly__;
            /// <summary>
            /// 134. Float :: Offset: 0x260 (608)
            /// </summary>
            public float HumanHorsepower_Weight_GripSpeedAccelMbPossibly { get { return __humanHorsepower_Weight_GripSpeedAccelMbPossibly__; } set { __humanHorsepower_Weight_GripSpeedAccelMbPossibly__ = value; } }
            private float __humanHorsepower_Weight_GripSpeedAccelMbPossibly__;
            /// <summary>
            /// 135. Float :: Offset: 0x264 (612)
            /// </summary>
            public float HumanOverallSpeedReverseSpeedMbPossibly { get { return __humanOverallSpeedReverseSpeedMbPossibly__; } set { __humanOverallSpeedReverseSpeedMbPossibly__ = value; } }
            private float __humanOverallSpeedReverseSpeedMbPossibly__;
            /// <summary>
            /// 136. TagBlock :: Offset: 0x268 (616)
            /// </summary>
            public _Unknown13TagBlock Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private _Unknown13TagBlock __unknown13__;
            public class _Unknown13TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 7. ByteArray :: Offset: 0x18 (24)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 11. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 12. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 13. Float :: Offset: 0x30 (48)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 14. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 15. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 16. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 17. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown13TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown13TagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown13TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown13TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 137. Float :: Offset: 0x270 (624)
            /// </summary>
            public float AIOversteeringBounds { get { return __aIOversteeringBounds__; } set { __aIOversteeringBounds__ = value; } }
            private float __aIOversteeringBounds__;
            /// <summary>
            /// 138. Float :: Offset: 0x274 (628)
            /// </summary>
            public float To { get { return __to__; } set { __to__ = value; } }
            private float __to__;
            /// <summary>
            /// 139. Float :: Offset: 0x278 (632)
            /// </summary>
            public float AISteeringMax { get { return __aISteeringMax__; } set { __aISteeringMax__ = value; } }
            private float __aISteeringMax__;
            /// <summary>
            /// 140. Float :: Offset: 0x27C (636)
            /// </summary>
            public float AIThrottleMax { get { return __aIThrottleMax__; } set { __aIThrottleMax__ = value; } }
            private float __aIThrottleMax__;
            /// <summary>
            /// 141. Float :: Offset: 0x280 (640)
            /// </summary>
            public float AIMovePositionTime { get { return __aIMovePositionTime__; } set { __aIMovePositionTime__ = value; } }
            private float __aIMovePositionTime__;
            /// <summary>
            /// 142. Tag :: Offset: 0x284 (644)
            /// </summary>
            public TagRef SuspensionSound { get { return __suspensionSound__; } set { __suspensionSound__ = value; } }
            private TagRef __suspensionSound__;
            /// <summary>
            /// 143. Tag :: Offset: 0x28C (652)
            /// </summary>
            public TagRef CrashSound { get { return __crashSound__; } set { __crashSound__ = value; } }
            private TagRef __crashSound__;
            /// <summary>
            /// 144. Tag :: Offset: 0x294 (660)
            /// </summary>
            public TagRef MaterialEffects2 { get { return __materialEffects2__; } set { __materialEffects2__ = value; } }
            private TagRef __materialEffects2__;
            /// <summary>
            /// 145. Tag :: Offset: 0x29C (668)
            /// </summary>
            public TagRef Effect { get { return __effect__; } set { __effect__ = value; } }
            private TagRef __effect__;
            /// <summary>
            /// 146. Tag :: Offset: 0x2A4 (676)
            /// </summary>
            public TagRef Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
            private TagRef __unknown14__;
            /// <summary>
            /// 147. Float :: Offset: 0x2AC (684)
            /// </summary>
            public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
            private float __unknown15__;
            /// <summary>
            /// 148. Float :: Offset: 0x2B0 (688)
            /// </summary>
            public float HumanVehiHandling { get { return __humanVehiHandling__; } set { __humanVehiHandling__ = value; } }
            private float __humanVehiHandling__;
            /// <summary>
            /// 149. Float :: Offset: 0x2B4 (692)
            /// </summary>
            public float HumanVehiHeightFromGround_WeightPossibly { get { return __humanVehiHeightFromGround_WeightPossibly__; } set { __humanVehiHeightFromGround_WeightPossibly__ = value; } }
            private float __humanVehiHeightFromGround_WeightPossibly__;
            /// <summary>
            /// 150. Float :: Offset: 0x2B8 (696)
            /// </summary>
            public float HumanVehiBounce_Gravity_Weight { get { return __humanVehiBounce_Gravity_Weight__; } set { __humanVehiBounce_Gravity_Weight__ = value; } }
            private float __humanVehiBounce_Gravity_Weight__;
            /// <summary>
            /// 151. Float :: Offset: 0x2BC (700)
            /// </summary>
            public float HumanVehiTurningCorrectionForcePossibly { get { return __humanVehiTurningCorrectionForcePossibly__; } set { __humanVehiTurningCorrectionForcePossibly__ = value; } }
            private float __humanVehiTurningCorrectionForcePossibly__;
            /// <summary>
            /// 152. Float :: Offset: 0x2C0 (704)
            /// </summary>
            public float GroundContactUpper { get { return __groundContactUpper__; } set { __groundContactUpper__ = value; } }
            private float __groundContactUpper__;
            /// <summary>
            /// 153. Float :: Offset: 0x2C4 (708)
            /// </summary>
            public float GroundContactLower { get { return __groundContactLower__; } set { __groundContactLower__ = value; } }
            private float __groundContactLower__;
            /// <summary>
            /// 154. Float :: Offset: 0x2C8 (712)
            /// </summary>
            public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
            private float __unknown16__;
            /// <summary>
            /// 155. Float :: Offset: 0x2CC (716)
            /// </summary>
            public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
            private float __unknown17__;
            /// <summary>
            /// 156. Float :: Offset: 0x2D0 (720)
            /// </summary>
            public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
            private float __unknown18__;
            /// <summary>
            /// 157. Float :: Offset: 0x2D4 (724)
            /// </summary>
            public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
            private float __unknown19__;
            /// <summary>
            /// 158. Float :: Offset: 0x2D8 (728)
            /// </summary>
            public float CovieVehiTurnAngle_flipy { get { return __covieVehiTurnAngle_flipy__; } set { __covieVehiTurnAngle_flipy__ = value; } }
            private float __covieVehiTurnAngle_flipy__;
            /// <summary>
            /// 159. Float :: Offset: 0x2DC (732)
            /// </summary>
            public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
            private float __unknown20__;
            /// <summary>
            /// 160. Float :: Offset: 0x2E0 (736)
            /// </summary>
            public float DrivingGravity { get { return __drivingGravity__; } set { __drivingGravity__ = value; } }
            private float __drivingGravity__;
            /// <summary>
            /// 161. Float :: Offset: 0x2E4 (740)
            /// </summary>
            public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
            private float __unknown21__;
            /// <summary>
            /// 162. TagBlock :: Offset: 0x2E8 (744)
            /// </summary>
            public _AntigravTagBlock Antigrav { get { return __antigrav__; } set { __antigrav__ = value; } }
            private _AntigravTagBlock __antigrav__;
            public class _AntigravTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x4 (4)
                /// </summary>
                public short Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private short __unknown1__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x6 (6)
                /// </summary>
                public short Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private short __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 5. Float :: Offset: 0xC (12)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 6. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 7. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 8. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 9. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 10. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 11. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 12. Float :: Offset: 0x28 (40)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 13. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 14. Int16 :: Offset: 0x30 (48)
                /// </summary>
                public short Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private short __unknown13__;
                /// <summary>
                /// 15. Int16 :: Offset: 0x32 (50)
                /// </summary>
                public short Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private short __unknown14__;
                /// <summary>
                /// 16. StringID :: Offset: 0x34 (52)
                /// </summary>
                public StringID InitialPlacementMarker { get { return __initialPlacementMarker__; } set { __initialPlacementMarker__ = value; } }
                private StringID __initialPlacementMarker__;
                /// <summary>
                /// 17. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 18. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 19. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 20. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 21. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AntigravTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AntigravTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AntigravTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AntigravTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 163. TagBlock :: Offset: 0x2F0 (752)
            /// </summary>
            public _HumanShocksTagBlock HumanShocks { get { return __humanShocks__; } set { __humanShocks__ = value; } }
            private _HumanShocksTagBlock __humanShocks__;
            public class _HumanShocksTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. StringID :: Offset: 0x0 (0)
                /// </summary>
                public StringID Name { get { return __name__; } set { __name__ = value; } }
                private StringID __name__;
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float SuspensionBounce { get { return __suspensionBounce__; } set { __suspensionBounce__ = value; } }
                private float __suspensionBounce__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private float __unknown4__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Weight { get { return __weight__; } set { __weight__ = value; } }
                private float __weight__;
                /// <summary>
                /// 8. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private float __unknown5__;
                /// <summary>
                /// 9. Float :: Offset: 0x20 (32)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 10. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 11. ByteArray :: Offset: 0x28 (40)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;
                /// <summary>
                /// 12. StringID :: Offset: 0x3C (60)
                /// </summary>
                public StringID MaterialType { get { return __materialType__; } set { __materialType__ = value; } }
                private StringID __materialType__;
                /// <summary>
                /// 13. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 14. StringID :: Offset: 0x44 (68)
                /// </summary>
                public StringID MarkerName { get { return __markerName__; } set { __markerName__ = value; } }
                private StringID __markerName__;
                /// <summary>
                /// 15. Int16 :: Offset: 0x48 (72)
                /// </summary>
                public short Index1 { get { return __index1__; } set { __index1__ = value; } }
                private short __index1__;
                /// <summary>
                /// 16. Int16 :: Offset: 0x4A (74)
                /// </summary>
                public short Index2 { get { return __index2__; } set { __index2__ = value; } }
                private short __index2__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _HumanShocksTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _HumanShocksTagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_HumanShocksTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _HumanShocksTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 164. TagBlock :: Offset: 0x2F8 (760)
            /// </summary>
            public _Unknown22TagBlock Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
            private _Unknown22TagBlock __unknown22__;
            public class _Unknown22TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public VehiLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private VehiLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public Unknown1Bitmask32 Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private Unknown1Bitmask32 __unknown1__;
                public class Unknown1Bitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 2. Bitmask32 :: Offset: 0x4 (4)
                /// </summary>
                public Unknown2Bitmask32 Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private Unknown2Bitmask32 __unknown2__;
                public class Unknown2Bitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private float __unknown3__;
                /// <summary>
                /// 4. Bitmask32 :: Offset: 0xC (12)
                /// </summary>
                public Unknown4Bitmask32 Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private Unknown4Bitmask32 __unknown4__;
                public class Unknown4Bitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 5. Enum16 :: Offset: 0x10 (16)
                /// </summary>
                public Unknown5Enum16 Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private Unknown5Enum16 __unknown5__;
                public class Unknown5Enum16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }
                /// <summary>
                /// 6. Int16 :: Offset: 0x12 (18)
                /// </summary>
                public short Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private short __unknown6__;
                /// <summary>
                /// 7. Enum16 :: Offset: 0x14 (20)
                /// </summary>
                public Unknown7Enum16 Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private Unknown7Enum16 __unknown7__;
                public class Unknown7Enum16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }
                /// <summary>
                /// 8. Bitmask16 :: Offset: 0x16 (22)
                /// </summary>
                public Unknown8Bitmask16 Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private Unknown8Bitmask16 __unknown8__;
                public class Unknown8Bitmask16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }
                /// <summary>
                /// 9. Bitmask32 :: Offset: 0x18 (24)
                /// </summary>
                public Unknown9Bitmask32 Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private Unknown9Bitmask32 __unknown9__;
                public class Unknown9Bitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 10. Float :: Offset: 0x1C (28)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 11. Bitmask32 :: Offset: 0x20 (32)
                /// </summary>
                public Unknown11Bitmask32 Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private Unknown11Bitmask32 __unknown11__;
                public class Unknown11Bitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 12. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 13. Enum16 :: Offset: 0x28 (40)
                /// </summary>
                public Unknown13Enum16 Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private Unknown13Enum16 __unknown13__;
                public class Unknown13Enum16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }
                /// <summary>
                /// 14. Int16 :: Offset: 0x2A (42)
                /// </summary>
                public short Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private short __unknown14__;
                /// <summary>
                /// 15. Float :: Offset: 0x2C (44)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 16. Int16 :: Offset: 0x30 (48)
                /// </summary>
                public short Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private short __unknown16__;
                /// <summary>
                /// 17. Int16 :: Offset: 0x32 (50)
                /// </summary>
                public short Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private short __unknown17__;
                /// <summary>
                /// 18. Float :: Offset: 0x34 (52)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 19. Int16 :: Offset: 0x38 (56)
                /// </summary>
                public short Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private short __unknown19__;
                /// <summary>
                /// 20. Enum16 :: Offset: 0x3A (58)
                /// </summary>
                public Unknown20Enum16 Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private Unknown20Enum16 __unknown20__;
                public class Unknown20Enum16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }
                /// <summary>
                /// 21. Enum16 :: Offset: 0x3C (60)
                /// </summary>
                public Unknown21Enum16 Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                private Unknown21Enum16 __unknown21__;
                public class Unknown21Enum16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }
                /// <summary>
                /// 22. Int16 :: Offset: 0x3E (62)
                /// </summary>
                public short Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                private short __unknown22__;
                /// <summary>
                /// 23. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                private float __unknown23__;
                /// <summary>
                /// 24. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                private float __unknown24__;
                /// <summary>
                /// 25. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                private float __unknown25__;
                /// <summary>
                /// 26. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                private float __unknown26__;
                /// <summary>
                /// 27. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                private float __unknown27__;
                /// <summary>
                /// 28. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                private float __unknown28__;
                /// <summary>
                /// 29. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                private float __unknown29__;
                /// <summary>
                /// 30. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                private float __unknown30__;
                /// <summary>
                /// 31. Float :: Offset: 0x60 (96)
                /// </summary>
                public float Unknown31 { get { return __unknown31__; } set { __unknown31__ = value; } }
                private float __unknown31__;
                /// <summary>
                /// 32. Float :: Offset: 0x64 (100)
                /// </summary>
                public float Unknown32 { get { return __unknown32__; } set { __unknown32__ = value; } }
                private float __unknown32__;
                /// <summary>
                /// 33. Float :: Offset: 0x68 (104)
                /// </summary>
                public float Unknown33 { get { return __unknown33__; } set { __unknown33__ = value; } }
                private float __unknown33__;
                /// <summary>
                /// 34. Float :: Offset: 0x6C (108)
                /// </summary>
                public float Unknown34 { get { return __unknown34__; } set { __unknown34__ = value; } }
                private float __unknown34__;
                /// <summary>
                /// 35. Float :: Offset: 0x70 (112)
                /// </summary>
                public float Unknown35 { get { return __unknown35__; } set { __unknown35__ = value; } }
                private float __unknown35__;
                /// <summary>
                /// 36. Float :: Offset: 0x74 (116)
                /// </summary>
                public float Unknown36 { get { return __unknown36__; } set { __unknown36__ = value; } }
                private float __unknown36__;
                /// <summary>
                /// 37. Float :: Offset: 0x78 (120)
                /// </summary>
                public float Unknown37 { get { return __unknown37__; } set { __unknown37__ = value; } }
                private float __unknown37__;
                /// <summary>
                /// 38. Float :: Offset: 0x7C (124)
                /// </summary>
                public float Unknown38 { get { return __unknown38__; } set { __unknown38__ = value; } }
                private float __unknown38__;
                /// <summary>
                /// 39. Float :: Offset: 0x80 (128)
                /// </summary>
                public float Unknown39 { get { return __unknown39__; } set { __unknown39__ = value; } }
                private float __unknown39__;
                /// <summary>
                /// 40. Float :: Offset: 0x84 (132)
                /// </summary>
                public float Unknown40 { get { return __unknown40__; } set { __unknown40__ = value; } }
                private float __unknown40__;
                /// <summary>
                /// 41. Float :: Offset: 0x88 (136)
                /// </summary>
                public float Unknown41 { get { return __unknown41__; } set { __unknown41__ = value; } }
                private float __unknown41__;
                /// <summary>
                /// 42. Float :: Offset: 0x8C (140)
                /// </summary>
                public float Unknown42 { get { return __unknown42__; } set { __unknown42__ = value; } }
                private float __unknown42__;
                /// <summary>
                /// 43. Float :: Offset: 0x90 (144)
                /// </summary>
                public float Unknown43 { get { return __unknown43__; } set { __unknown43__ = value; } }
                private float __unknown43__;
                /// <summary>
                /// 44. Float :: Offset: 0x94 (148)
                /// </summary>
                public float Unknown44 { get { return __unknown44__; } set { __unknown44__ = value; } }
                private float __unknown44__;
                /// <summary>
                /// 45. Float :: Offset: 0x98 (152)
                /// </summary>
                public float Unknown45 { get { return __unknown45__; } set { __unknown45__ = value; } }
                private float __unknown45__;
                /// <summary>
                /// 46. Float :: Offset: 0x9C (156)
                /// </summary>
                public float Unknown46 { get { return __unknown46__; } set { __unknown46__ = value; } }
                private float __unknown46__;
                /// <summary>
                /// 47. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float Unknown47 { get { return __unknown47__; } set { __unknown47__ = value; } }
                private float __unknown47__;
                /// <summary>
                /// 48. Float :: Offset: 0xA4 (164)
                /// </summary>
                public float Unknown48 { get { return __unknown48__; } set { __unknown48__ = value; } }
                private float __unknown48__;
                /// <summary>
                /// 49. Float :: Offset: 0xA8 (168)
                /// </summary>
                public float Unknown49 { get { return __unknown49__; } set { __unknown49__ = value; } }
                private float __unknown49__;
                /// <summary>
                /// 50. Float :: Offset: 0xAC (172)
                /// </summary>
                public float Unknown50 { get { return __unknown50__; } set { __unknown50__ = value; } }
                private float __unknown50__;
                /// <summary>
                /// 51. Float :: Offset: 0xB0 (176)
                /// </summary>
                public float Unknown51 { get { return __unknown51__; } set { __unknown51__ = value; } }
                private float __unknown51__;
                /// <summary>
                /// 52. Float :: Offset: 0xB4 (180)
                /// </summary>
                public float Unknown52 { get { return __unknown52__; } set { __unknown52__ = value; } }
                private float __unknown52__;
                /// <summary>
                /// 53. Float :: Offset: 0xB8 (184)
                /// </summary>
                public float Unknown53 { get { return __unknown53__; } set { __unknown53__ = value; } }
                private float __unknown53__;
                /// <summary>
                /// 54. Float :: Offset: 0xBC (188)
                /// </summary>
                public float Unknown54 { get { return __unknown54__; } set { __unknown54__ = value; } }
                private float __unknown54__;
                /// <summary>
                /// 55. Float :: Offset: 0xC0 (192)
                /// </summary>
                public float Unknown55 { get { return __unknown55__; } set { __unknown55__ = value; } }
                private float __unknown55__;
                /// <summary>
                /// 56. Float :: Offset: 0xC4 (196)
                /// </summary>
                public float Unknown56 { get { return __unknown56__; } set { __unknown56__ = value; } }
                private float __unknown56__;
                /// <summary>
                /// 57. Float :: Offset: 0xC8 (200)
                /// </summary>
                public float Unknown57 { get { return __unknown57__; } set { __unknown57__ = value; } }
                private float __unknown57__;
                /// <summary>
                /// 58. Float :: Offset: 0xCC (204)
                /// </summary>
                public float Unknown58 { get { return __unknown58__; } set { __unknown58__ = value; } }
                private float __unknown58__;
                /// <summary>
                /// 59. Float :: Offset: 0xD0 (208)
                /// </summary>
                public float Unknown59 { get { return __unknown59__; } set { __unknown59__ = value; } }
                private float __unknown59__;
                /// <summary>
                /// 60. Float :: Offset: 0xD4 (212)
                /// </summary>
                public float Unknown60 { get { return __unknown60__; } set { __unknown60__ = value; } }
                private float __unknown60__;
                /// <summary>
                /// 61. Float :: Offset: 0xD8 (216)
                /// </summary>
                public float Unknown61 { get { return __unknown61__; } set { __unknown61__ = value; } }
                private float __unknown61__;
                /// <summary>
                /// 62. Float :: Offset: 0xDC (220)
                /// </summary>
                public float Unknown62 { get { return __unknown62__; } set { __unknown62__ = value; } }
                private float __unknown62__;
                /// <summary>
                /// 63. Float :: Offset: 0xE0 (224)
                /// </summary>
                public float Unknown63 { get { return __unknown63__; } set { __unknown63__ = value; } }
                private float __unknown63__;
                /// <summary>
                /// 64. Float :: Offset: 0xE4 (228)
                /// </summary>
                public float Unknown64 { get { return __unknown64__; } set { __unknown64__ = value; } }
                private float __unknown64__;
                /// <summary>
                /// 65. Float :: Offset: 0xE8 (232)
                /// </summary>
                public float Unknown65 { get { return __unknown65__; } set { __unknown65__ = value; } }
                private float __unknown65__;
                /// <summary>
                /// 66. Float :: Offset: 0xEC (236)
                /// </summary>
                public float Unknown66 { get { return __unknown66__; } set { __unknown66__ = value; } }
                private float __unknown66__;
                /// <summary>
                /// 67. Float :: Offset: 0xF0 (240)
                /// </summary>
                public float Unknown67 { get { return __unknown67__; } set { __unknown67__ = value; } }
                private float __unknown67__;
                /// <summary>
                /// 68. Float :: Offset: 0xF4 (244)
                /// </summary>
                public float Unknown68 { get { return __unknown68__; } set { __unknown68__ = value; } }
                private float __unknown68__;
                /// <summary>
                /// 69. Float :: Offset: 0xF8 (248)
                /// </summary>
                public float Unknown69 { get { return __unknown69__; } set { __unknown69__ = value; } }
                private float __unknown69__;
                /// <summary>
                /// 70. Float :: Offset: 0xFC (252)
                /// </summary>
                public float Unknown70 { get { return __unknown70__; } set { __unknown70__ = value; } }
                private float __unknown70__;
                /// <summary>
                /// 71. Float :: Offset: 0x100 (256)
                /// </summary>
                public float Unknown71 { get { return __unknown71__; } set { __unknown71__ = value; } }
                private float __unknown71__;
                /// <summary>
                /// 72. Float :: Offset: 0x104 (260)
                /// </summary>
                public float Unknown72 { get { return __unknown72__; } set { __unknown72__ = value; } }
                private float __unknown72__;
                /// <summary>
                /// 73. Float :: Offset: 0x108 (264)
                /// </summary>
                public float Unknown73 { get { return __unknown73__; } set { __unknown73__ = value; } }
                private float __unknown73__;
                /// <summary>
                /// 74. Float :: Offset: 0x10C (268)
                /// </summary>
                public float Unknown74 { get { return __unknown74__; } set { __unknown74__ = value; } }
                private float __unknown74__;
                /// <summary>
                /// 75. Float :: Offset: 0x110 (272)
                /// </summary>
                public float Unknown75 { get { return __unknown75__; } set { __unknown75__ = value; } }
                private float __unknown75__;
                /// <summary>
                /// 76. Float :: Offset: 0x114 (276)
                /// </summary>
                public float Unknown76 { get { return __unknown76__; } set { __unknown76__ = value; } }
                private float __unknown76__;
                /// <summary>
                /// 77. Float :: Offset: 0x118 (280)
                /// </summary>
                public float Unknown77 { get { return __unknown77__; } set { __unknown77__ = value; } }
                private float __unknown77__;
                /// <summary>
                /// 78. Float :: Offset: 0x11C (284)
                /// </summary>
                public float Unknown78 { get { return __unknown78__; } set { __unknown78__ = value; } }
                private float __unknown78__;
                /// <summary>
                /// 79. Float :: Offset: 0x120 (288)
                /// </summary>
                public float Unknown79 { get { return __unknown79__; } set { __unknown79__ = value; } }
                private float __unknown79__;
                /// <summary>
                /// 80. Float :: Offset: 0x124 (292)
                /// </summary>
                public float Unknown80 { get { return __unknown80__; } set { __unknown80__ = value; } }
                private float __unknown80__;
                /// <summary>
                /// 81. Float :: Offset: 0x128 (296)
                /// </summary>
                public float Unknown81 { get { return __unknown81__; } set { __unknown81__ = value; } }
                private float __unknown81__;
                /// <summary>
                /// 82. Float :: Offset: 0x12C (300)
                /// </summary>
                public float Unknown82 { get { return __unknown82__; } set { __unknown82__ = value; } }
                private float __unknown82__;
                /// <summary>
                /// 83. Float :: Offset: 0x130 (304)
                /// </summary>
                public float Unknown83 { get { return __unknown83__; } set { __unknown83__ = value; } }
                private float __unknown83__;
                /// <summary>
                /// 84. Float :: Offset: 0x134 (308)
                /// </summary>
                public float Unknown84 { get { return __unknown84__; } set { __unknown84__ = value; } }
                private float __unknown84__;
                /// <summary>
                /// 85. Float :: Offset: 0x138 (312)
                /// </summary>
                public float Unknown85 { get { return __unknown85__; } set { __unknown85__ = value; } }
                private float __unknown85__;
                /// <summary>
                /// 86. Float :: Offset: 0x13C (316)
                /// </summary>
                public float Unknown86 { get { return __unknown86__; } set { __unknown86__ = value; } }
                private float __unknown86__;
                /// <summary>
                /// 87. Float :: Offset: 0x140 (320)
                /// </summary>
                public float Unknown87 { get { return __unknown87__; } set { __unknown87__ = value; } }
                private float __unknown87__;
                /// <summary>
                /// 88. Float :: Offset: 0x144 (324)
                /// </summary>
                public float Unknown88 { get { return __unknown88__; } set { __unknown88__ = value; } }
                private float __unknown88__;
                /// <summary>
                /// 89. Float :: Offset: 0x148 (328)
                /// </summary>
                public float Unknown89 { get { return __unknown89__; } set { __unknown89__ = value; } }
                private float __unknown89__;
                /// <summary>
                /// 90. Float :: Offset: 0x14C (332)
                /// </summary>
                public float Unknown90 { get { return __unknown90__; } set { __unknown90__ = value; } }
                private float __unknown90__;
                /// <summary>
                /// 91. Float :: Offset: 0x150 (336)
                /// </summary>
                public float Unknown91 { get { return __unknown91__; } set { __unknown91__ = value; } }
                private float __unknown91__;
                /// <summary>
                /// 92. Float :: Offset: 0x154 (340)
                /// </summary>
                public float Unknown92 { get { return __unknown92__; } set { __unknown92__ = value; } }
                private float __unknown92__;
                /// <summary>
                /// 93. Float :: Offset: 0x158 (344)
                /// </summary>
                public float Unknown93 { get { return __unknown93__; } set { __unknown93__ = value; } }
                private float __unknown93__;
                /// <summary>
                /// 94. Float :: Offset: 0x15C (348)
                /// </summary>
                public float Unknown94 { get { return __unknown94__; } set { __unknown94__ = value; } }
                private float __unknown94__;
                /// <summary>
                /// 95. Float :: Offset: 0x160 (352)
                /// </summary>
                public float Unknown95 { get { return __unknown95__; } set { __unknown95__ = value; } }
                private float __unknown95__;
                /// <summary>
                /// 96. Float :: Offset: 0x164 (356)
                /// </summary>
                public float Unknown96 { get { return __unknown96__; } set { __unknown96__ = value; } }
                private float __unknown96__;
                /// <summary>
                /// 97. Float :: Offset: 0x168 (360)
                /// </summary>
                public float Unknown97 { get { return __unknown97__; } set { __unknown97__ = value; } }
                private float __unknown97__;
                /// <summary>
                /// 98. Float :: Offset: 0x16C (364)
                /// </summary>
                public float Unknown98 { get { return __unknown98__; } set { __unknown98__ = value; } }
                private float __unknown98__;
                /// <summary>
                /// 99. Float :: Offset: 0x170 (368)
                /// </summary>
                public float Unknown99 { get { return __unknown99__; } set { __unknown99__ = value; } }
                private float __unknown99__;
                /// <summary>
                /// 100. Float :: Offset: 0x174 (372)
                /// </summary>
                public float Unknown100 { get { return __unknown100__; } set { __unknown100__ = value; } }
                private float __unknown100__;
                /// <summary>
                /// 101. Float :: Offset: 0x178 (376)
                /// </summary>
                public float Unknown101 { get { return __unknown101__; } set { __unknown101__ = value; } }
                private float __unknown101__;
                /// <summary>
                /// 102. Float :: Offset: 0x17C (380)
                /// </summary>
                public float Unknown102 { get { return __unknown102__; } set { __unknown102__ = value; } }
                private float __unknown102__;
                /// <summary>
                /// 103. Float :: Offset: 0x180 (384)
                /// </summary>
                public float Unknown103 { get { return __unknown103__; } set { __unknown103__ = value; } }
                private float __unknown103__;
                /// <summary>
                /// 104. Float :: Offset: 0x184 (388)
                /// </summary>
                public float Unknown104 { get { return __unknown104__; } set { __unknown104__ = value; } }
                private float __unknown104__;
                /// <summary>
                /// 105. Float :: Offset: 0x188 (392)
                /// </summary>
                public float Unknown105 { get { return __unknown105__; } set { __unknown105__ = value; } }
                private float __unknown105__;
                /// <summary>
                /// 106. Float :: Offset: 0x18C (396)
                /// </summary>
                public float Unknown106 { get { return __unknown106__; } set { __unknown106__ = value; } }
                private float __unknown106__;
                /// <summary>
                /// 107. Float :: Offset: 0x190 (400)
                /// </summary>
                public float Unknown107 { get { return __unknown107__; } set { __unknown107__ = value; } }
                private float __unknown107__;
                /// <summary>
                /// 108. Float :: Offset: 0x194 (404)
                /// </summary>
                public float Unknown108 { get { return __unknown108__; } set { __unknown108__ = value; } }
                private float __unknown108__;
                /// <summary>
                /// 109. Float :: Offset: 0x198 (408)
                /// </summary>
                public float Unknown109 { get { return __unknown109__; } set { __unknown109__ = value; } }
                private float __unknown109__;
                /// <summary>
                /// 110. Float :: Offset: 0x19C (412)
                /// </summary>
                public float Unknown110 { get { return __unknown110__; } set { __unknown110__ = value; } }
                private float __unknown110__;
                /// <summary>
                /// 111. Float :: Offset: 0x1A0 (416)
                /// </summary>
                public float Unknown111 { get { return __unknown111__; } set { __unknown111__ = value; } }
                private float __unknown111__;
                /// <summary>
                /// 112. Float :: Offset: 0x1A4 (420)
                /// </summary>
                public float Unknown112 { get { return __unknown112__; } set { __unknown112__ = value; } }
                private float __unknown112__;
                /// <summary>
                /// 113. Float :: Offset: 0x1A8 (424)
                /// </summary>
                public float Unknown113 { get { return __unknown113__; } set { __unknown113__ = value; } }
                private float __unknown113__;
                /// <summary>
                /// 114. Float :: Offset: 0x1AC (428)
                /// </summary>
                public float Unknown114 { get { return __unknown114__; } set { __unknown114__ = value; } }
                private float __unknown114__;
                /// <summary>
                /// 115. Float :: Offset: 0x1B0 (432)
                /// </summary>
                public float Unknown115 { get { return __unknown115__; } set { __unknown115__ = value; } }
                private float __unknown115__;
                /// <summary>
                /// 116. Float :: Offset: 0x1B4 (436)
                /// </summary>
                public float Unknown116 { get { return __unknown116__; } set { __unknown116__ = value; } }
                private float __unknown116__;
                /// <summary>
                /// 117. Float :: Offset: 0x1B8 (440)
                /// </summary>
                public float Unknown117 { get { return __unknown117__; } set { __unknown117__ = value; } }
                private float __unknown117__;
                /// <summary>
                /// 118. Float :: Offset: 0x1BC (444)
                /// </summary>
                public float Unknown118 { get { return __unknown118__; } set { __unknown118__ = value; } }
                private float __unknown118__;
                /// <summary>
                /// 119. Float :: Offset: 0x1C0 (448)
                /// </summary>
                public float Unknown119 { get { return __unknown119__; } set { __unknown119__ = value; } }
                private float __unknown119__;
                /// <summary>
                /// 120. Float :: Offset: 0x1C4 (452)
                /// </summary>
                public float Unknown120 { get { return __unknown120__; } set { __unknown120__ = value; } }
                private float __unknown120__;
                /// <summary>
                /// 121. Float :: Offset: 0x1C8 (456)
                /// </summary>
                public float Unknown121 { get { return __unknown121__; } set { __unknown121__ = value; } }
                private float __unknown121__;
                /// <summary>
                /// 122. Float :: Offset: 0x1CC (460)
                /// </summary>
                public float Unknown122 { get { return __unknown122__; } set { __unknown122__ = value; } }
                private float __unknown122__;
                /// <summary>
                /// 123. Float :: Offset: 0x1D0 (464)
                /// </summary>
                public float Unknown123 { get { return __unknown123__; } set { __unknown123__ = value; } }
                private float __unknown123__;
                /// <summary>
                /// 124. Float :: Offset: 0x1D4 (468)
                /// </summary>
                public float Unknown124 { get { return __unknown124__; } set { __unknown124__ = value; } }
                private float __unknown124__;
                /// <summary>
                /// 125. Float :: Offset: 0x1D8 (472)
                /// </summary>
                public float Unknown125 { get { return __unknown125__; } set { __unknown125__ = value; } }
                private float __unknown125__;
                /// <summary>
                /// 126. Float :: Offset: 0x1DC (476)
                /// </summary>
                public float Unknown126 { get { return __unknown126__; } set { __unknown126__ = value; } }
                private float __unknown126__;
                /// <summary>
                /// 127. Float :: Offset: 0x1E0 (480)
                /// </summary>
                public float Unknown127 { get { return __unknown127__; } set { __unknown127__ = value; } }
                private float __unknown127__;
                /// <summary>
                /// 128. Float :: Offset: 0x1E4 (484)
                /// </summary>
                public float Unknown128 { get { return __unknown128__; } set { __unknown128__ = value; } }
                private float __unknown128__;
                /// <summary>
                /// 129. Float :: Offset: 0x1E8 (488)
                /// </summary>
                public float Unknown129 { get { return __unknown129__; } set { __unknown129__ = value; } }
                private float __unknown129__;
                /// <summary>
                /// 130. Float :: Offset: 0x1EC (492)
                /// </summary>
                public float Unknown130 { get { return __unknown130__; } set { __unknown130__ = value; } }
                private float __unknown130__;
                /// <summary>
                /// 131. Float :: Offset: 0x1F0 (496)
                /// </summary>
                public float Unknown131 { get { return __unknown131__; } set { __unknown131__ = value; } }
                private float __unknown131__;
                /// <summary>
                /// 132. Float :: Offset: 0x1F4 (500)
                /// </summary>
                public float Unknown132 { get { return __unknown132__; } set { __unknown132__ = value; } }
                private float __unknown132__;
                /// <summary>
                /// 133. Float :: Offset: 0x1F8 (504)
                /// </summary>
                public float Unknown133 { get { return __unknown133__; } set { __unknown133__ = value; } }
                private float __unknown133__;
                /// <summary>
                /// 134. Float :: Offset: 0x1FC (508)
                /// </summary>
                public float Unknown134 { get { return __unknown134__; } set { __unknown134__ = value; } }
                private float __unknown134__;
                /// <summary>
                /// 135. Float :: Offset: 0x200 (512)
                /// </summary>
                public float Unknown135 { get { return __unknown135__; } set { __unknown135__ = value; } }
                private float __unknown135__;
                /// <summary>
                /// 136. Float :: Offset: 0x204 (516)
                /// </summary>
                public float Unknown136 { get { return __unknown136__; } set { __unknown136__ = value; } }
                private float __unknown136__;
                /// <summary>
                /// 137. Float :: Offset: 0x208 (520)
                /// </summary>
                public float Unknown137 { get { return __unknown137__; } set { __unknown137__ = value; } }
                private float __unknown137__;
                /// <summary>
                /// 138. Float :: Offset: 0x20C (524)
                /// </summary>
                public float Unknown138 { get { return __unknown138__; } set { __unknown138__ = value; } }
                private float __unknown138__;
                /// <summary>
                /// 139. Float :: Offset: 0x210 (528)
                /// </summary>
                public float Unknown139 { get { return __unknown139__; } set { __unknown139__ = value; } }
                private float __unknown139__;
                /// <summary>
                /// 140. Float :: Offset: 0x214 (532)
                /// </summary>
                public float Unknown140 { get { return __unknown140__; } set { __unknown140__ = value; } }
                private float __unknown140__;
                /// <summary>
                /// 141. Float :: Offset: 0x218 (536)
                /// </summary>
                public float Unknown141 { get { return __unknown141__; } set { __unknown141__ = value; } }
                private float __unknown141__;
                /// <summary>
                /// 142. Float :: Offset: 0x21C (540)
                /// </summary>
                public float Unknown142 { get { return __unknown142__; } set { __unknown142__ = value; } }
                private float __unknown142__;
                /// <summary>
                /// 143. Float :: Offset: 0x220 (544)
                /// </summary>
                public float Unknown143 { get { return __unknown143__; } set { __unknown143__ = value; } }
                private float __unknown143__;
                /// <summary>
                /// 144. Float :: Offset: 0x224 (548)
                /// </summary>
                public float Unknown144 { get { return __unknown144__; } set { __unknown144__ = value; } }
                private float __unknown144__;
                /// <summary>
                /// 145. Float :: Offset: 0x228 (552)
                /// </summary>
                public float Unknown145 { get { return __unknown145__; } set { __unknown145__ = value; } }
                private float __unknown145__;
                /// <summary>
                /// 146. Float :: Offset: 0x22C (556)
                /// </summary>
                public float Unknown146 { get { return __unknown146__; } set { __unknown146__ = value; } }
                private float __unknown146__;
                /// <summary>
                /// 147. Float :: Offset: 0x230 (560)
                /// </summary>
                public float Unknown147 { get { return __unknown147__; } set { __unknown147__ = value; } }
                private float __unknown147__;
                /// <summary>
                /// 148. Float :: Offset: 0x234 (564)
                /// </summary>
                public float Unknown148 { get { return __unknown148__; } set { __unknown148__ = value; } }
                private float __unknown148__;
                /// <summary>
                /// 149. Float :: Offset: 0x238 (568)
                /// </summary>
                public float Unknown149 { get { return __unknown149__; } set { __unknown149__ = value; } }
                private float __unknown149__;
                /// <summary>
                /// 150. Float :: Offset: 0x23C (572)
                /// </summary>
                public float Unknown150 { get { return __unknown150__; } set { __unknown150__ = value; } }
                private float __unknown150__;
                /// <summary>
                /// 151. Float :: Offset: 0x240 (576)
                /// </summary>
                public float Unknown151 { get { return __unknown151__; } set { __unknown151__ = value; } }
                private float __unknown151__;
                /// <summary>
                /// 152. Float :: Offset: 0x244 (580)
                /// </summary>
                public float Unknown152 { get { return __unknown152__; } set { __unknown152__ = value; } }
                private float __unknown152__;
                /// <summary>
                /// 153. Float :: Offset: 0x248 (584)
                /// </summary>
                public float Unknown153 { get { return __unknown153__; } set { __unknown153__ = value; } }
                private float __unknown153__;
                /// <summary>
                /// 154. Float :: Offset: 0x24C (588)
                /// </summary>
                public float Unknown154 { get { return __unknown154__; } set { __unknown154__ = value; } }
                private float __unknown154__;
                /// <summary>
                /// 155. Float :: Offset: 0x250 (592)
                /// </summary>
                public float Unknown155 { get { return __unknown155__; } set { __unknown155__ = value; } }
                private float __unknown155__;
                /// <summary>
                /// 156. Float :: Offset: 0x254 (596)
                /// </summary>
                public float Unknown156 { get { return __unknown156__; } set { __unknown156__ = value; } }
                private float __unknown156__;
                /// <summary>
                /// 157. Float :: Offset: 0x258 (600)
                /// </summary>
                public float Unknown157 { get { return __unknown157__; } set { __unknown157__ = value; } }
                private float __unknown157__;
                /// <summary>
                /// 158. Float :: Offset: 0x25C (604)
                /// </summary>
                public float Unknown158 { get { return __unknown158__; } set { __unknown158__ = value; } }
                private float __unknown158__;
                /// <summary>
                /// 159. Float :: Offset: 0x260 (608)
                /// </summary>
                public float Unknown159 { get { return __unknown159__; } set { __unknown159__ = value; } }
                private float __unknown159__;
                /// <summary>
                /// 160. Float :: Offset: 0x264 (612)
                /// </summary>
                public float Unknown160 { get { return __unknown160__; } set { __unknown160__ = value; } }
                private float __unknown160__;
                /// <summary>
                /// 161. Float :: Offset: 0x268 (616)
                /// </summary>
                public float Unknown161 { get { return __unknown161__; } set { __unknown161__ = value; } }
                private float __unknown161__;
                /// <summary>
                /// 162. Float :: Offset: 0x26C (620)
                /// </summary>
                public float Unknown162 { get { return __unknown162__; } set { __unknown162__ = value; } }
                private float __unknown162__;
                /// <summary>
                /// 163. Float :: Offset: 0x270 (624)
                /// </summary>
                public float Unknown163 { get { return __unknown163__; } set { __unknown163__ = value; } }
                private float __unknown163__;
                /// <summary>
                /// 164. Float :: Offset: 0x274 (628)
                /// </summary>
                public float Unknown164 { get { return __unknown164__; } set { __unknown164__ = value; } }
                private float __unknown164__;
                /// <summary>
                /// 165. Float :: Offset: 0x278 (632)
                /// </summary>
                public float Unknown165 { get { return __unknown165__; } set { __unknown165__ = value; } }
                private float __unknown165__;
                /// <summary>
                /// 166. Float :: Offset: 0x27C (636)
                /// </summary>
                public float Unknown166 { get { return __unknown166__; } set { __unknown166__ = value; } }
                private float __unknown166__;
                /// <summary>
                /// 167. Float :: Offset: 0x280 (640)
                /// </summary>
                public float Unknown167 { get { return __unknown167__; } set { __unknown167__ = value; } }
                private float __unknown167__;
                /// <summary>
                /// 168. Float :: Offset: 0x284 (644)
                /// </summary>
                public float Unknown168 { get { return __unknown168__; } set { __unknown168__ = value; } }
                private float __unknown168__;
                /// <summary>
                /// 169. Float :: Offset: 0x288 (648)
                /// </summary>
                public float Unknown169 { get { return __unknown169__; } set { __unknown169__ = value; } }
                private float __unknown169__;
                /// <summary>
                /// 170. Float :: Offset: 0x28C (652)
                /// </summary>
                public float Unknown170 { get { return __unknown170__; } set { __unknown170__ = value; } }
                private float __unknown170__;
                /// <summary>
                /// 171. Float :: Offset: 0x290 (656)
                /// </summary>
                public float Unknown171 { get { return __unknown171__; } set { __unknown171__ = value; } }
                private float __unknown171__;
                /// <summary>
                /// 172. Float :: Offset: 0x294 (660)
                /// </summary>
                public float Unknown172 { get { return __unknown172__; } set { __unknown172__ = value; } }
                private float __unknown172__;
                /// <summary>
                /// 173. Float :: Offset: 0x298 (664)
                /// </summary>
                public float Unknown173 { get { return __unknown173__; } set { __unknown173__ = value; } }
                private float __unknown173__;
                /// <summary>
                /// 174. Int16 :: Offset: 0x29C (668)
                /// </summary>
                public short Unknown174 { get { return __unknown174__; } set { __unknown174__ = value; } }
                private short __unknown174__;
                /// <summary>
                /// 175. Bitmask16 :: Offset: 0x29E (670)
                /// </summary>
                public Unknown175Bitmask16 Unknown175 { get { return __unknown175__; } set { __unknown175__ = value; } }
                private Unknown175Bitmask16 __unknown175__;
                public class Unknown175Bitmask16
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 2;
                    }
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _Unknown22TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _Unknown22TagBlock(VehiLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Unknown1 = new Unknown1Bitmask32();
                    Unknown2 = new Unknown2Bitmask32();
                    Unknown4 = new Unknown4Bitmask32();
                    Unknown5 = new Unknown5Enum16();
                    Unknown7 = new Unknown7Enum16();
                    Unknown8 = new Unknown8Bitmask16();
                    Unknown9 = new Unknown9Bitmask32();
                    Unknown11 = new Unknown11Bitmask32();
                    Unknown13 = new Unknown13Enum16();
                    Unknown20 = new Unknown20Enum16();
                    Unknown21 = new Unknown21Enum16();
                    Unknown175 = new Unknown175Bitmask16();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_Unknown22TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _Unknown22TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }

            #endregion

            /// <summary>
            /// Initializes a new instance of the VehiLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public VehiLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags1 = new Flags1Bitmask32();
                AiProperties = new _AiPropertiesTagBlock(this);
                Functions = new _FunctionsTagBlock(this);
                Attachments = new _AttachmentsTagBlock(this);
                Widgets = new _WidgetsTagBlock(this);
                ChangeColors = new _ChangeColorsTagBlock(this);
                PredictedResource = new _PredictedResourceTagBlock(this);
                OldFlags1 = new OldFlags1Bitmask16();
                NewFlags = new NewFlagsBitmask16();
                DefaultTeam = new DefaultTeamEnum16();
                ConstantSoundVolume = new ConstantSoundVolumeEnum16();
                CameraTracks = new _CameraTracksTagBlock(this);
                MotionSensorBlipSize = new MotionSensorBlipSizeEnum32();
                Unknown4 = new _Unknown4TagBlock(this);
                HudInterfaces = new _HudInterfacesTagBlock(this);
                Unknown5 = new _Unknown5TagBlock(this);
                GrenadeType = new GrenadeTypeEnum16();
                PoweredSeats = new _PoweredSeatsTagBlock(this);
                Weapon = new _WeaponTagBlock(this);
                Seats = new _SeatsTagBlock(this);
                OldFlags2 = new OldFlags2Bitmask16();
                Flags2 = new Flags2Bitmask16();
                Type = new TypeEnum16();
                Unknown6 = new Unknown6Enum16();
                Unknown7 = new Unknown7Enum16();
                Unknown8 = new Unknown8Enum16();
                Unknown10 = new Unknown10Enum16();
                Unknown13 = new _Unknown13TagBlock(this);
                Antigrav = new _AntigravTagBlock(this);
                HumanShocks = new _HumanShocksTagBlock(this);
                Unknown22 = new _Unknown22TagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags1._Initialize(br);
                BoundingRadius = br.ReadSingle();
                BoundingOffsetX = br.ReadSingle();
                BoundingOffsetY = br.ReadSingle();
                BoundingOffsetZ = br.ReadSingle();
                AccelerationScale = br.ReadSingle();
                LightmapShadowMode = br.ReadSingle();
                SweetenerSize = br.ReadSingle();
                DynamicLightSphereRadius = br.ReadSingle();
                DynamicLightSphereOffsetX = br.ReadSingle();
                DynamicLightSphereOffsetY = br.ReadSingle();
                DynamicLightSphereOffsetZ = br.ReadSingle();
                DefaultModelVariant._Initialize(br);
                Model._Initialize(br);
                CrateObject._Initialize(br);
                ModifierShader._Initialize(br);
                CreationEffect._Initialize(br);
                MaterialEffects1._Initialize(br);
                AiProperties._Initialize(br);
                Functions._Initialize(br);
                RenderBoundingRadius = br.ReadSingle();
                Lower1 = br.ReadSingle();
                Upper1 = br.ReadSingle();
                Lower2 = br.ReadSingle();
                Upper2 = br.ReadSingle();
                Lower3 = br.ReadSingle();
                Upper3 = br.ReadSingle();
                Lower4 = br.ReadSingle();
                Upper4 = br.ReadSingle();
                HudTextMessageIndex = br.ReadInt16();
                ForcedShaderPermutationIndex = br.ReadInt16();
                Attachments._Initialize(br);
                Widgets._Initialize(br);
                Unused1 = br.ReadBytes(8);
                ChangeColors._Initialize(br);
                PredictedResource._Initialize(br);
                OldFlags1._Initialize(br);
                NewFlags._Initialize(br);
                DefaultTeam._Initialize(br);
                ConstantSoundVolume._Initialize(br);
                IntegratedLightToggle._Initialize(br);
                CameraFieldOfView = br.ReadSingle();
                CameraStiffness = br.ReadSingle();
                CameraMarkerName._Initialize(br);
                CameraSubmergedMarkerName._Initialize(br);
                PitchAutoLevel = br.ReadSingle();
                PitchRange1 = br.ReadSingle();
                PitchRange2 = br.ReadSingle();
                CameraTracks._Initialize(br);
                SeatAccelerationScaleI = br.ReadSingle();
                SeatAccelerationScaleJ = br.ReadSingle();
                SeatAccelerationScaleK = br.ReadSingle();
                Unused2 = br.ReadBytes(8);
                SoftPingThreshold = br.ReadSingle();
                SoftPingInterruptTime = br.ReadSingle();
                HardPingThreshold = br.ReadSingle();
                HardPingInterruptTime = br.ReadSingle();
                HardPingDeathThreshold = br.ReadSingle();
                FeignDeathThreshold = br.ReadSingle();
                FeignDeathTime = br.ReadSingle();
                DistOfEvadeAnim = br.ReadSingle();
                DistOfDiveAnim = br.ReadSingle();
                StunnedMovementThreshold = br.ReadSingle();
                FeignDeathChance = br.ReadSingle();
                FeignRepeatChance = br.ReadSingle();
                SpawnedActor._Initialize(br);
                SpawnedActorCountLower = br.ReadInt16();
                SpawnedActorCountUpper = br.ReadInt16();
                SpawnedVelocity = br.ReadSingle();
                AimingVelocityMax = br.ReadSingle();
                AimingAccelMax = br.ReadSingle();
                CasualAimingModifier = br.ReadSingle();
                LookingVelocityMax = br.ReadSingle();
                LookingAccelMax = br.ReadSingle();
                WeaponPlacementMarker._Initialize(br);
                LeftHand._Initialize(br);
                RightHand._Initialize(br);
                MeleeDamage._Initialize(br);
                BoardingMeleeDamage._Initialize(br);
                BoardingMeleeResponseDamage._Initialize(br);
                Unknown1._Initialize(br);
                Unknown2._Initialize(br);
                Unknown3._Initialize(br);
                MotionSensorBlipSize._Initialize(br);
                Unknown4._Initialize(br);
                HudInterfaces._Initialize(br);
                Unknown5._Initialize(br);
                GrenadeVelocity = br.ReadSingle();
                GrenadeType._Initialize(br);
                GrenadeCount = br.ReadUInt16();
                PoweredSeats._Initialize(br);
                Weapon._Initialize(br);
                Seats._Initialize(br);
                BoostSpeedMultiplier = br.ReadSingle();
                TimeUntilNextBoost = br.ReadSingle();
                BoostLength = br.ReadSingle();
                BoostTime = br.ReadSingle();
                BoostAccel = br.ReadSingle();
                Unused3 = br.ReadBytes(8);
                OldFlags2._Initialize(br);
                Flags2._Initialize(br);
                Type._Initialize(br);
                Unknown6._Initialize(br);
                CovieMaxForwardSpeed = br.ReadSingle();
                CovieMaxReverseSpeed = br.ReadSingle();
                CovieSpeedAccel = br.ReadSingle();
                CovieSpeedDecel = br.ReadSingle();
                CovieMaxLeftTurn = br.ReadSingle();
                CovieMaxRightTurn_negative = br.ReadSingle();
                WheelCircumference_TurningCircleMbPossibly = br.ReadSingle();
                TurnRate = br.ReadSingle();
                BlurSpeed = br.ReadSingle();
                Unknown7._Initialize(br);
                Unknown8._Initialize(br);
                FlipMessage._Initialize(br);
                Unknown9 = br.ReadSingle();
                CovieMaxLeftSlide = br.ReadSingle();
                CovieMaxRightSlide = br.ReadSingle();
                CovieVehiStrafeSpeedLeft = br.ReadSingle();
                CovieVehiStrafeSpeedRight = br.ReadSingle();
                CovieVehiStrafeAccel = br.ReadSingle();
                CovieVehiStrafeDecel = br.ReadSingle();
                MinimumFlippingAngularVelocity = br.ReadSingle();
                MaximumFlippingAngularVelocity = br.ReadSingle();
                Unknown10._Initialize(br);
                Unknown11 = br.ReadInt16();
                Unknown12 = br.ReadBytes(2);
                Unused4 = br.ReadBytes(4);
                FixedGunYaw = br.ReadSingle();
                FixedGunPitch = br.ReadSingle();
                WheelWobble = br.ReadSingle();
                UnknownHumanMaxForwardSpeedMbPossibly = br.ReadSingle();
                UnknownHumanMaxReverseSpeedMbPossibly = br.ReadSingle();
                HumanHorsepower_Weight_GripSpeedAccelMbPossibly = br.ReadSingle();
                HumanOverallSpeedReverseSpeedMbPossibly = br.ReadSingle();
                Unknown13._Initialize(br);
                AIOversteeringBounds = br.ReadSingle();
                To = br.ReadSingle();
                AISteeringMax = br.ReadSingle();
                AIThrottleMax = br.ReadSingle();
                AIMovePositionTime = br.ReadSingle();
                SuspensionSound._Initialize(br);
                CrashSound._Initialize(br);
                MaterialEffects2._Initialize(br);
                Effect._Initialize(br);
                Unknown14._Initialize(br);
                Unknown15 = br.ReadSingle();
                HumanVehiHandling = br.ReadSingle();
                HumanVehiHeightFromGround_WeightPossibly = br.ReadSingle();
                HumanVehiBounce_Gravity_Weight = br.ReadSingle();
                HumanVehiTurningCorrectionForcePossibly = br.ReadSingle();
                GroundContactUpper = br.ReadSingle();
                GroundContactLower = br.ReadSingle();
                Unknown16 = br.ReadSingle();
                Unknown17 = br.ReadSingle();
                Unknown18 = br.ReadSingle();
                Unknown19 = br.ReadSingle();
                CovieVehiTurnAngle_flipy = br.ReadSingle();
                Unknown20 = br.ReadSingle();
                DrivingGravity = br.ReadSingle();
                Unknown21 = br.ReadSingle();
                Antigrav._Initialize(br);
                HumanShocks._Initialize(br);
                Unknown22._Initialize(br);

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Vehi class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Vehi(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new VehiLayout(map, identity);
        }

        #region Specific Methods of vehi

        #endregion
    }
}
