using System;
using System.IO;
using System.Extended;
using H2UtilityToolkit;

namespace H2UtilityToolkit.Structures
{
    /// <summary>
    /// Weapon
    /// </summary>
    public class Weap
    {
        #region Constants

        /// <summary>
        /// Size of the weap meta header.
        /// </summary>
        public const int HeaderSize = 796;

        #endregion

        #region Instance Data

        private Map map;
        private int identity;
        private WeapLayout layout;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return map; } set { map = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return identity; } set { identity = value; } }
        /// <summary>
        /// v0.72 XZodia: Added lockon bit, updated Trigger Type Info.
        /// v0.7 XZodia: Added loads of values.
        /// v0.52 turk645: Added the FP position info.
        /// v0.5 XZodia: Added multi proj spawn ability.
        /// v0.45 TfAv1228: Added More to Structure At 712.
        /// v0.4 Iron_Forge: Tested and added some unknowns...
        /// v0.3 Iron_Forge: Added some known values...
        /// v0.1 Iron_Forge: Added basic layout of plugin...
        /// </summary>
        public WeapLayout Layout { get { return layout; } set { layout = value; } }

        #endregion

        /// <summary>
        /// The layout of the tag meta.
        /// </summary>
        public class WeapLayout
        {
            #region Class Instance Data, Accessors

            /// <summary>
            /// Map reference, used when in need of other functions outside of this class.
            /// </summary>
            public Map _Map { get { return map; } set { map = value; } }
            private Map map;
            /// <summary>
            /// The identity of the tag to be used by this class.
            /// </summary>
            public int _Identity { get { return identity; } set { identity = value; } }
            private int identity;
            /// <summary>
            /// True if Initialize() has been called, and executed, without error.
            /// </summary>
            public bool _Initialized { get { return initialized; } protected set { initialized = value; } }
            private bool initialized;

            #endregion

            #region Layout Instance Data, Accessors, Classes

            /// <summary>
            /// 1. Bitmask32 :: Offset: 0x0 (0)
            /// </summary>
            public Flags1Bitmask32 Flags1 { get { return __flags1__; } set { __flags1__ = value; } }
            private Flags1Bitmask32 __flags1__;
            public class Flags1Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool DoesNotCastShadow { get { return __doesNotCastShadow__; } set { __doesNotCastShadow__ = value; } }
                private bool __doesNotCastShadow__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool SearchCardinalDirectionLightmaps { get { return __searchCardinalDirectionLightmaps__; } set { __searchCardinalDirectionLightmaps__ = value; } }
                private bool __searchCardinalDirectionLightmaps__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Unused { get { return __unused__; } set { __unused__ = value; } }
                private bool __unused__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool NotAPathfindingObstacle { get { return __notAPathfindingObstacle__; } set { __notAPathfindingObstacle__ = value; } }
                private bool __notAPathfindingObstacle__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool ExtensionOfParent { get { return __extensionOfParent__; } set { __extensionOfParent__ = value; } }
                private bool __extensionOfParent__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool DoesNotCauseCollisionDamage { get { return __doesNotCauseCollisionDamage__; } set { __doesNotCauseCollisionDamage__ = value; } }
                private bool __doesNotCauseCollisionDamage__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool EarlyMover { get { return __earlyMover__; } set { __earlyMover__ = value; } }
                private bool __earlyMover__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool EarlyMoverLocalizedPhysics { get { return __earlyMoverLocalizedPhysics__; } set { __earlyMoverLocalizedPhysics__ = value; } }
                private bool __earlyMoverLocalizedPhysics__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool UseStaticMassiveLightmapSample { get { return __useStaticMassiveLightmapSample__; } set { __useStaticMassiveLightmapSample__ = value; } }
                private bool __useStaticMassiveLightmapSample__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool ObjectScalesAttachments { get { return __objectScalesAttachments__; } set { __objectScalesAttachments__ = value; } }
                private bool __objectScalesAttachments__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool InheritsPlayersAppearance { get { return __inheritsPlayersAppearance__; } set { __inheritsPlayersAppearance__ = value; } }
                private bool __inheritsPlayersAppearance__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool DeadBipedsCantLocalize { get { return __deadBipedsCantLocalize__; } set { __deadBipedsCantLocalize__ = value; } }
                private bool __deadBipedsCantLocalize__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool AttachToClustersByDynamicSphere { get { return __attachToClustersByDynamicSphere__; } set { __attachToClustersByDynamicSphere__ = value; } }
                private bool __attachToClustersByDynamicSphere__;
                /// <summary>
                /// 14. Option :: Value: 0xD (13)
                /// </summary>
                public bool EffectsCreatedByThisObjectDoNotBlah { get { return __effectsCreatedByThisObjectDoNotBlah__; } set { __effectsCreatedByThisObjectDoNotBlah__ = value; } }
                private bool __effectsCreatedByThisObjectDoNotBlah__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    DoesNotCastShadow = ((bit & mask) == bit);
                    bit <<= 1;
                    SearchCardinalDirectionLightmaps = ((bit & mask) == bit);
                    bit <<= 1;
                    Unused = ((bit & mask) == bit);
                    bit <<= 1;
                    NotAPathfindingObstacle = ((bit & mask) == bit);
                    bit <<= 1;
                    ExtensionOfParent = ((bit & mask) == bit);
                    bit <<= 1;
                    DoesNotCauseCollisionDamage = ((bit & mask) == bit);
                    bit <<= 1;
                    EarlyMover = ((bit & mask) == bit);
                    bit <<= 1;
                    EarlyMoverLocalizedPhysics = ((bit & mask) == bit);
                    bit <<= 1;
                    UseStaticMassiveLightmapSample = ((bit & mask) == bit);
                    bit <<= 1;
                    ObjectScalesAttachments = ((bit & mask) == bit);
                    bit <<= 1;
                    InheritsPlayersAppearance = ((bit & mask) == bit);
                    bit <<= 1;
                    DeadBipedsCantLocalize = ((bit & mask) == bit);
                    bit <<= 1;
                    AttachToClustersByDynamicSphere = ((bit & mask) == bit);
                    bit <<= 1;
                    EffectsCreatedByThisObjectDoNotBlah = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 2. Float :: Offset: 0x4 (4)
            /// </summary>
            public float BoundingRadius { get { return __boundingRadius__; } set { __boundingRadius__ = value; } }
            private float __boundingRadius__;
            /// <summary>
            /// 3. Float :: Offset: 0x8 (8)
            /// </summary>
            public float BoundingOffsetX { get { return __boundingOffsetX__; } set { __boundingOffsetX__ = value; } }
            private float __boundingOffsetX__;
            /// <summary>
            /// 4. Float :: Offset: 0xC (12)
            /// </summary>
            public float BoundingOffsetY { get { return __boundingOffsetY__; } set { __boundingOffsetY__ = value; } }
            private float __boundingOffsetY__;
            /// <summary>
            /// 5. Float :: Offset: 0x10 (16)
            /// </summary>
            public float BoundingOffsetZ { get { return __boundingOffsetZ__; } set { __boundingOffsetZ__ = value; } }
            private float __boundingOffsetZ__;
            /// <summary>
            /// 6. Float :: Offset: 0x14 (20)
            /// </summary>
            public float AccelerationScale { get { return __accelerationScale__; } set { __accelerationScale__ = value; } }
            private float __accelerationScale__;
            /// <summary>
            /// 7. Float :: Offset: 0x18 (24)
            /// </summary>
            public float LightmapShadowMode { get { return __lightmapShadowMode__; } set { __lightmapShadowMode__ = value; } }
            private float __lightmapShadowMode__;
            /// <summary>
            /// 8. Float :: Offset: 0x1C (28)
            /// </summary>
            public float SweetenerSize { get { return __sweetenerSize__; } set { __sweetenerSize__ = value; } }
            private float __sweetenerSize__;
            /// <summary>
            /// 9. Float :: Offset: 0x20 (32)
            /// </summary>
            public float DynamicLightSphereRadius { get { return __dynamicLightSphereRadius__; } set { __dynamicLightSphereRadius__ = value; } }
            private float __dynamicLightSphereRadius__;
            /// <summary>
            /// 10. Float :: Offset: 0x24 (36)
            /// </summary>
            public float DynamicLightSphereOffsetX { get { return __dynamicLightSphereOffsetX__; } set { __dynamicLightSphereOffsetX__ = value; } }
            private float __dynamicLightSphereOffsetX__;
            /// <summary>
            /// 11. Float :: Offset: 0x28 (40)
            /// </summary>
            public float DynamicLightSphereOffsetY { get { return __dynamicLightSphereOffsetY__; } set { __dynamicLightSphereOffsetY__ = value; } }
            private float __dynamicLightSphereOffsetY__;
            /// <summary>
            /// 12. Float :: Offset: 0x2C (44)
            /// </summary>
            public float DynamicLightSphereOffsetZ { get { return __dynamicLightSphereOffsetZ__; } set { __dynamicLightSphereOffsetZ__ = value; } }
            private float __dynamicLightSphereOffsetZ__;
            /// <summary>
            /// 13. StringID :: Offset: 0x30 (48)
            /// </summary>
            public StringID DefaultModelVariant { get { return __defaultModelVariant__; } set { __defaultModelVariant__ = value; } }
            private StringID __defaultModelVariant__;
            /// <summary>
            /// 14. Tag :: Offset: 0x34 (52)
            /// </summary>
            public TagRef Model { get { return __model__; } set { __model__ = value; } }
            private TagRef __model__;
            /// <summary>
            /// 15. Tag :: Offset: 0x3C (60)
            /// </summary>
            public TagRef CrateObject { get { return __crateObject__; } set { __crateObject__ = value; } }
            private TagRef __crateObject__;
            /// <summary>
            /// 16. Tag :: Offset: 0x44 (68)
            /// </summary>
            public TagRef ModifierShader { get { return __modifierShader__; } set { __modifierShader__ = value; } }
            private TagRef __modifierShader__;
            /// <summary>
            /// 17. Tag :: Offset: 0x4C (76)
            /// </summary>
            public TagRef CreationEffect { get { return __creationEffect__; } set { __creationEffect__ = value; } }
            private TagRef __creationEffect__;
            /// <summary>
            /// 18. Tag :: Offset: 0x54 (84)
            /// </summary>
            public TagRef MaterialEffects1 { get { return __materialEffects1__; } set { __materialEffects1__ = value; } }
            private TagRef __materialEffects1__;
            /// <summary>
            /// 19. TagBlock :: Offset: 0x5C (92)
            /// </summary>
            public _AiPropertiesTagBlock AiProperties { get { return __aiProperties__; } set { __aiProperties__ = value; } }
            private _AiPropertiesTagBlock __aiProperties__;
            public class _AiPropertiesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Flags { get { return __flags__; } set { __flags__ = value; } }
                private float __flags__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID TypeName { get { return __typeName__; } set { __typeName__ = value; } }
                private StringID __typeName__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float Size { get { return __size__; } set { __size__ = value; } }
                private float __size__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float LeapJumpSpeed { get { return __leapJumpSpeed__; } set { __leapJumpSpeed__ = value; } }
                private float __leapJumpSpeed__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AiPropertiesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AiPropertiesTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AiPropertiesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AiPropertiesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 20. TagBlock :: Offset: 0x64 (100)
            /// </summary>
            public _FunctionsTagBlock Functions { get { return __functions__; } set { __functions__ = value; } }
            private _FunctionsTagBlock __functions__;
            public class _FunctionsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Flags { get { return __flags__; } set { __flags__ = value; } }
                private float __flags__;
                /// <summary>
                /// 2. StringID :: Offset: 0x4 (4)
                /// </summary>
                public StringID Function { get { return __function__; } set { __function__ = value; } }
                private StringID __function__;
                /// <summary>
                /// 3. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID ScaleFunctionBy { get { return __scaleFunctionBy__; } set { __scaleFunctionBy__ = value; } }
                private StringID __scaleFunctionBy__;
                /// <summary>
                /// 4. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID WobbleFunction { get { return __wobbleFunction__; } set { __wobbleFunction__ = value; } }
                private StringID __wobbleFunction__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float WobblePeriod { get { return __wobblePeriod__; } set { __wobblePeriod__ = value; } }
                private float __wobblePeriod__;
                /// <summary>
                /// 6. TagBlock :: Offset: 0x14 (20)
                /// </summary>
                public _Inputs_realNumbersTagBlock Inputs_realNumbers { get { return __inputs_realNumbers__; } set { __inputs_realNumbers__ = value; } }
                private _Inputs_realNumbersTagBlock __inputs_realNumbers__;
                public class _Inputs_realNumbersTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private WeapLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Byte :: Offset: 0x0 (0)
                    /// </summary>
                    public byte Data { get { return __data__; } set { __data__ = value; } }
                    private byte __data__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _Inputs_realNumbersTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _Inputs_realNumbersTagBlock(WeapLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_Inputs_realNumbersTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _Inputs_realNumbersTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 7. StringID :: Offset: 0x1C (28)
                /// </summary>
                public StringID TurnOffWith { get { return __turnOffWith__; } set { __turnOffWith__ = value; } }
                private StringID __turnOffWith__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FunctionsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FunctionsTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Inputs_realNumbers = new _Inputs_realNumbersTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FunctionsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FunctionsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 21. Float :: Offset: 0x6C (108)
            /// </summary>
            public float RenderBoundingRadius { get { return __renderBoundingRadius__; } set { __renderBoundingRadius__ = value; } }
            private float __renderBoundingRadius__;
            /// <summary>
            /// 22. Float :: Offset: 0x70 (112)
            /// </summary>
            public float Lower1 { get { return __lower1__; } set { __lower1__ = value; } }
            private float __lower1__;
            /// <summary>
            /// 23. Float :: Offset: 0x74 (116)
            /// </summary>
            public float Upper1 { get { return __upper1__; } set { __upper1__ = value; } }
            private float __upper1__;
            /// <summary>
            /// 24. Float :: Offset: 0x78 (120)
            /// </summary>
            public float Lower2 { get { return __lower2__; } set { __lower2__ = value; } }
            private float __lower2__;
            /// <summary>
            /// 25. Float :: Offset: 0x7C (124)
            /// </summary>
            public float Upper2 { get { return __upper2__; } set { __upper2__ = value; } }
            private float __upper2__;
            /// <summary>
            /// 26. Float :: Offset: 0x80 (128)
            /// </summary>
            public float Lower3 { get { return __lower3__; } set { __lower3__ = value; } }
            private float __lower3__;
            /// <summary>
            /// 27. Float :: Offset: 0x84 (132)
            /// </summary>
            public float Upper3 { get { return __upper3__; } set { __upper3__ = value; } }
            private float __upper3__;
            /// <summary>
            /// 28. Float :: Offset: 0x88 (136)
            /// </summary>
            public float Lower4 { get { return __lower4__; } set { __lower4__ = value; } }
            private float __lower4__;
            /// <summary>
            /// 29. Float :: Offset: 0x8C (140)
            /// </summary>
            public float Upper4 { get { return __upper4__; } set { __upper4__ = value; } }
            private float __upper4__;
            /// <summary>
            /// 30. Int16 :: Offset: 0x90 (144)
            /// </summary>
            public short HudTextMessageIndex { get { return __hudTextMessageIndex__; } set { __hudTextMessageIndex__ = value; } }
            private short __hudTextMessageIndex__;
            /// <summary>
            /// 31. Int16 :: Offset: 0x92 (146)
            /// </summary>
            public short ForcedShaderPermutationIndex { get { return __forcedShaderPermutationIndex__; } set { __forcedShaderPermutationIndex__ = value; } }
            private short __forcedShaderPermutationIndex__;
            /// <summary>
            /// 32. TagBlock :: Offset: 0x94 (148)
            /// </summary>
            public _AttachmentsTagBlock Attachments { get { return __attachments__; } set { __attachments__ = value; } }
            private _AttachmentsTagBlock __attachments__;
            public class _AttachmentsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Type { get { return __type__; } set { __type__ = value; } }
                private TagRef __type__;
                /// <summary>
                /// 2. StringID :: Offset: 0x8 (8)
                /// </summary>
                public StringID Marker { get { return __marker__; } set { __marker__ = value; } }
                private StringID __marker__;
                /// <summary>
                /// 3. StringID :: Offset: 0xC (12)
                /// </summary>
                public StringID ChangeColor { get { return __changeColor__; } set { __changeColor__ = value; } }
                private StringID __changeColor__;
                /// <summary>
                /// 4. StringID :: Offset: 0x10 (16)
                /// </summary>
                public StringID PrimaryScale { get { return __primaryScale__; } set { __primaryScale__ = value; } }
                private StringID __primaryScale__;
                /// <summary>
                /// 5. StringID :: Offset: 0x14 (20)
                /// </summary>
                public StringID SecondaryScale { get { return __secondaryScale__; } set { __secondaryScale__ = value; } }
                private StringID __secondaryScale__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _AttachmentsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _AttachmentsTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_AttachmentsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _AttachmentsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 33. TagBlock :: Offset: 0x9C (156)
            /// </summary>
            public _WidgetsTagBlock Widgets { get { return __widgets__; } set { __widgets__ = value; } }
            private _WidgetsTagBlock __widgets__;
            public class _WidgetsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Reference { get { return __reference__; } set { __reference__ = value; } }
                private TagRef __reference__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _WidgetsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _WidgetsTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_WidgetsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _WidgetsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 34. ByteArray :: Offset: 0xA4 (164)
            /// </summary>
            public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
            private byte[] __unused1__;
            /// <summary>
            /// 35. TagBlock :: Offset: 0xAC (172)
            /// </summary>
            public _ChangeColorsTagBlock ChangeColors { get { return __changeColors__; } set { __changeColors__ = value; } }
            private _ChangeColorsTagBlock __changeColors__;
            public class _ChangeColorsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. TagBlock :: Offset: 0x0 (0)
                /// </summary>
                public _PermutationsTagBlock Permutations { get { return __permutations__; } set { __permutations__ = value; } }
                private _PermutationsTagBlock __permutations__;
                public class _PermutationsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private WeapLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Float :: Offset: 0x0 (0)
                    /// </summary>
                    public float Weight { get { return __weight__; } set { __weight__ = value; } }
                    private float __weight__;
                    /// <summary>
                    /// 2. Float :: Offset: 0x4 (4)
                    /// </summary>
                    public float ColorLowerBoundR { get { return __colorLowerBoundR__; } set { __colorLowerBoundR__ = value; } }
                    private float __colorLowerBoundR__;
                    /// <summary>
                    /// 3. Float :: Offset: 0x8 (8)
                    /// </summary>
                    public float ColorLowerBoundG { get { return __colorLowerBoundG__; } set { __colorLowerBoundG__ = value; } }
                    private float __colorLowerBoundG__;
                    /// <summary>
                    /// 4. Float :: Offset: 0xC (12)
                    /// </summary>
                    public float ColorLowerBoundB { get { return __colorLowerBoundB__; } set { __colorLowerBoundB__ = value; } }
                    private float __colorLowerBoundB__;
                    /// <summary>
                    /// 5. Float :: Offset: 0x10 (16)
                    /// </summary>
                    public float ColorUpperBoundR { get { return __colorUpperBoundR__; } set { __colorUpperBoundR__ = value; } }
                    private float __colorUpperBoundR__;
                    /// <summary>
                    /// 6. Float :: Offset: 0x14 (20)
                    /// </summary>
                    public float ColorUpperBoundG { get { return __colorUpperBoundG__; } set { __colorUpperBoundG__ = value; } }
                    private float __colorUpperBoundG__;
                    /// <summary>
                    /// 7. Float :: Offset: 0x18 (24)
                    /// </summary>
                    public float ColorUpperBoundB { get { return __colorUpperBoundB__; } set { __colorUpperBoundB__ = value; } }
                    private float __colorUpperBoundB__;
                    /// <summary>
                    /// 8. StringID :: Offset: 0x1C (28)
                    /// </summary>
                    public StringID Name { get { return __name__; } set { __name__ = value; } }
                    private StringID __name__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _PermutationsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _PermutationsTagBlock(WeapLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_PermutationsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _PermutationsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }
                /// <summary>
                /// 2. ByteArray :: Offset: 0x8 (8)
                /// </summary>
                public byte[] Unused { get { return __unused__; } set { __unused__ = value; } }
                private byte[] __unused__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ChangeColorsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ChangeColorsTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Permutations = new _PermutationsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ChangeColorsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ChangeColorsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 36. TagBlock :: Offset: 0xB4 (180)
            /// </summary>
            public _PredictedResource1TagBlock PredictedResource1 { get { return __predictedResource1__; } set { __predictedResource1__ = value; } }
            private _PredictedResource1TagBlock __predictedResource1__;
            public class _PredictedResource1TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Int16 :: Offset: 0x0 (0)
                /// </summary>
                public short Type { get { return __type__; } set { __type__ = value; } }
                private short __type__;
                /// <summary>
                /// 2. Int16 :: Offset: 0x2 (2)
                /// </summary>
                public short ResourceIndex { get { return __resourceIndex__; } set { __resourceIndex__ = value; } }
                private short __resourceIndex__;
                /// <summary>
                /// 3. ID :: Offset: 0x4 (4)
                /// </summary>
                public LoneID TagIndex { get { return __tagIndex__; } set { __tagIndex__ = value; } }
                private LoneID __tagIndex__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PredictedResource1TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PredictedResource1TagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PredictedResource1TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PredictedResource1TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 37. Float :: Offset: 0xBC (188)
            /// </summary>
            public float ItemFlags { get { return __itemFlags__; } set { __itemFlags__ = value; } }
            private float __itemFlags__;
            /// <summary>
            /// 38. Int16 :: Offset: 0xC0 (192)
            /// </summary>
            public short MessageIndex { get { return __messageIndex__; } set { __messageIndex__ = value; } }
            private short __messageIndex__;
            /// <summary>
            /// 39. Int16 :: Offset: 0xC2 (194)
            /// </summary>
            public short SortOrder { get { return __sortOrder__; } set { __sortOrder__ = value; } }
            private short __sortOrder__;
            /// <summary>
            /// 40. Float :: Offset: 0xC4 (196)
            /// </summary>
            public float Scale { get { return __scale__; } set { __scale__ = value; } }
            private float __scale__;
            /// <summary>
            /// 41. Int16 :: Offset: 0xC8 (200)
            /// </summary>
            public short HudMessageValueScale { get { return __hudMessageValueScale__; } set { __hudMessageValueScale__ = value; } }
            private short __hudMessageValueScale__;
            /// <summary>
            /// 42. ByteArray :: Offset: 0xCA (202)
            /// </summary>
            public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
            private byte[] __unused2__;
            /// <summary>
            /// 43. StringID :: Offset: 0xCC (204)
            /// </summary>
            public StringID Pickup { get { return __pickup__; } set { __pickup__ = value; } }
            private StringID __pickup__;
            /// <summary>
            /// 44. StringID :: Offset: 0xD0 (208)
            /// </summary>
            public StringID Swap { get { return __swap__; } set { __swap__ = value; } }
            private StringID __swap__;
            /// <summary>
            /// 45. StringID :: Offset: 0xD4 (212)
            /// </summary>
            public StringID PickupOrDualWield { get { return __pickupOrDualWield__; } set { __pickupOrDualWield__ = value; } }
            private StringID __pickupOrDualWield__;
            /// <summary>
            /// 46. StringID :: Offset: 0xD8 (216)
            /// </summary>
            public StringID SwapOrDualWield { get { return __swapOrDualWield__; } set { __swapOrDualWield__ = value; } }
            private StringID __swapOrDualWield__;
            /// <summary>
            /// 47. StringID :: Offset: 0xDC (220)
            /// </summary>
            public StringID DualWield { get { return __dualWield__; } set { __dualWield__ = value; } }
            private StringID __dualWield__;
            /// <summary>
            /// 48. StringID :: Offset: 0xE0 (224)
            /// </summary>
            public StringID PickedUp { get { return __pickedUp__; } set { __pickedUp__ = value; } }
            private StringID __pickedUp__;
            /// <summary>
            /// 49. StringID :: Offset: 0xE4 (228)
            /// </summary>
            public StringID AmmoSingular { get { return __ammoSingular__; } set { __ammoSingular__ = value; } }
            private StringID __ammoSingular__;
            /// <summary>
            /// 50. StringID :: Offset: 0xE8 (232)
            /// </summary>
            public StringID AmmoPlural { get { return __ammoPlural__; } set { __ammoPlural__ = value; } }
            private StringID __ammoPlural__;
            /// <summary>
            /// 51. StringID :: Offset: 0xEC (236)
            /// </summary>
            public StringID SwitchedTo { get { return __switchedTo__; } set { __switchedTo__ = value; } }
            private StringID __switchedTo__;
            /// <summary>
            /// 52. StringID :: Offset: 0xF0 (240)
            /// </summary>
            public StringID SwapAI { get { return __swapAI__; } set { __swapAI__ = value; } }
            private StringID __swapAI__;
            /// <summary>
            /// 53. Tag :: Offset: 0xF4 (244)
            /// </summary>
            public TagRef MaterialEffects2 { get { return __materialEffects2__; } set { __materialEffects2__ = value; } }
            private TagRef __materialEffects2__;
            /// <summary>
            /// 54. Tag :: Offset: 0xFC (252)
            /// </summary>
            public TagRef CollisionSoundPossibly1 { get { return __collisionSoundPossibly1__; } set { __collisionSoundPossibly1__ = value; } }
            private TagRef __collisionSoundPossibly1__;
            /// <summary>
            /// 55. TagBlock :: Offset: 0x104 (260)
            /// </summary>
            public _FireEffectBitmapsTagBlock FireEffectBitmaps { get { return __fireEffectBitmaps__; } set { __fireEffectBitmaps__ = value; } }
            private _FireEffectBitmapsTagBlock __fireEffectBitmaps__;
            public class _FireEffectBitmapsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Bitmap { get { return __bitmap__; } set { __bitmap__ = value; } }
                private TagRef __bitmap__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FireEffectBitmapsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FireEffectBitmapsTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FireEffectBitmapsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FireEffectBitmapsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 56. Tag :: Offset: 0x10C (268)
            /// </summary>
            public TagRef CollisionSoundPossibly2 { get { return __collisionSoundPossibly2__; } set { __collisionSoundPossibly2__ = value; } }
            private TagRef __collisionSoundPossibly2__;
            /// <summary>
            /// 57. Float :: Offset: 0x114 (276)
            /// </summary>
            public float DetonationDelayLower { get { return __detonationDelayLower__; } set { __detonationDelayLower__ = value; } }
            private float __detonationDelayLower__;
            /// <summary>
            /// 58. Float :: Offset: 0x118 (280)
            /// </summary>
            public float DetonationDelayUpper { get { return __detonationDelayUpper__; } set { __detonationDelayUpper__ = value; } }
            private float __detonationDelayUpper__;
            /// <summary>
            /// 59. Tag :: Offset: 0x11C (284)
            /// </summary>
            public TagRef DetonatingEffect { get { return __detonatingEffect__; } set { __detonatingEffect__ = value; } }
            private TagRef __detonatingEffect__;
            /// <summary>
            /// 60. Tag :: Offset: 0x124 (292)
            /// </summary>
            public TagRef DetonationEffect { get { return __detonationEffect__; } set { __detonationEffect__ = value; } }
            private TagRef __detonationEffect__;
            /// <summary>
            /// 61. Bitmask32 :: Offset: 0x12C (300)
            /// </summary>
            public Flags2Bitmask32 Flags2 { get { return __flags2__; } set { __flags2__ = value; } }
            private Flags2Bitmask32 __flags2__;
            public class Flags2Bitmask32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool VerticalHeatDisplay { get { return __verticalHeatDisplay__; } set { __verticalHeatDisplay__ = value; } }
                private bool __verticalHeatDisplay__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool MutuallyExclusiveTriggers { get { return __mutuallyExclusiveTriggers__; } set { __mutuallyExclusiveTriggers__ = value; } }
                private bool __mutuallyExclusiveTriggers__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool AttacksAutomaticallyOnBump { get { return __attacksAutomaticallyOnBump__; } set { __attacksAutomaticallyOnBump__ = value; } }
                private bool __attacksAutomaticallyOnBump__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool MustBeReadied { get { return __mustBeReadied__; } set { __mustBeReadied__ = value; } }
                private bool __mustBeReadied__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool DoesntCountTowardMaximum { get { return __doesntCountTowardMaximum__; } set { __doesntCountTowardMaximum__ = value; } }
                private bool __doesntCountTowardMaximum__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool AimAssistsOnlyWhenZoomed { get { return __aimAssistsOnlyWhenZoomed__; } set { __aimAssistsOnlyWhenZoomed__ = value; } }
                private bool __aimAssistsOnlyWhenZoomed__;
                /// <summary>
                /// 7. Option :: Value: 0x6 (6)
                /// </summary>
                public bool PreventsGrenadeThrowing { get { return __preventsGrenadeThrowing__; } set { __preventsGrenadeThrowing__ = value; } }
                private bool __preventsGrenadeThrowing__;
                /// <summary>
                /// 8. Option :: Value: 0x7 (7)
                /// </summary>
                public bool MustBePickedUp { get { return __mustBePickedUp__; } set { __mustBePickedUp__ = value; } }
                private bool __mustBePickedUp__;
                /// <summary>
                /// 9. Option :: Value: 0x8 (8)
                /// </summary>
                public bool HoldsTriggersWhenDropped { get { return __holdsTriggersWhenDropped__; } set { __holdsTriggersWhenDropped__ = value; } }
                private bool __holdsTriggersWhenDropped__;
                /// <summary>
                /// 10. Option :: Value: 0x9 (9)
                /// </summary>
                public bool PreventsMeleeAttack { get { return __preventsMeleeAttack__; } set { __preventsMeleeAttack__ = value; } }
                private bool __preventsMeleeAttack__;
                /// <summary>
                /// 11. Option :: Value: 0xA (10)
                /// </summary>
                public bool DetonatesWhenDropped { get { return __detonatesWhenDropped__; } set { __detonatesWhenDropped__ = value; } }
                private bool __detonatesWhenDropped__;
                /// <summary>
                /// 12. Option :: Value: 0xB (11)
                /// </summary>
                public bool CannotFireAtMaximumRange { get { return __cannotFireAtMaximumRange__; } set { __cannotFireAtMaximumRange__ = value; } }
                private bool __cannotFireAtMaximumRange__;
                /// <summary>
                /// 13. Option :: Value: 0xC (12)
                /// </summary>
                public bool SecondaryTriggerOverridesGrenades { get { return __secondaryTriggerOverridesGrenades__; } set { __secondaryTriggerOverridesGrenades__ = value; } }
                private bool __secondaryTriggerOverridesGrenades__;
                /// <summary>
                /// 14. Option :: Value: 0xE (14)
                /// </summary>
                public bool EnablesIntegratedNightVision { get { return __enablesIntegratedNightVision__; } set { __enablesIntegratedNightVision__ = value; } }
                private bool __enablesIntegratedNightVision__;
                /// <summary>
                /// 15. Option :: Value: 0xF (15)
                /// </summary>
                public bool AIsUseWeaponMeleeDamage { get { return __aIsUseWeaponMeleeDamage__; } set { __aIsUseWeaponMeleeDamage__ = value; } }
                private bool __aIsUseWeaponMeleeDamage__;
                /// <summary>
                /// 16. Option :: Value: 0x16 (22)
                /// </summary>
                public bool DualWieldable { get { return __dualWieldable__; } set { __dualWieldable__ = value; } }
                private bool __dualWieldable__;

                public void _Initialize(BitReader br)
                {
                    uint mask = br.ReadUInt32();
                    uint bit = 1;

                    VerticalHeatDisplay = ((bit & mask) == bit);
                    bit <<= 1;
                    MutuallyExclusiveTriggers = ((bit & mask) == bit);
                    bit <<= 1;
                    AttacksAutomaticallyOnBump = ((bit & mask) == bit);
                    bit <<= 1;
                    MustBeReadied = ((bit & mask) == bit);
                    bit <<= 1;
                    DoesntCountTowardMaximum = ((bit & mask) == bit);
                    bit <<= 1;
                    AimAssistsOnlyWhenZoomed = ((bit & mask) == bit);
                    bit <<= 1;
                    PreventsGrenadeThrowing = ((bit & mask) == bit);
                    bit <<= 1;
                    MustBePickedUp = ((bit & mask) == bit);
                    bit <<= 1;
                    HoldsTriggersWhenDropped = ((bit & mask) == bit);
                    bit <<= 1;
                    PreventsMeleeAttack = ((bit & mask) == bit);
                    bit <<= 1;
                    DetonatesWhenDropped = ((bit & mask) == bit);
                    bit <<= 1;
                    CannotFireAtMaximumRange = ((bit & mask) == bit);
                    bit <<= 1;
                    SecondaryTriggerOverridesGrenades = ((bit & mask) == bit);
                    bit <<= 1;
                    EnablesIntegratedNightVision = ((bit & mask) == bit);
                    bit <<= 1;
                    AIsUseWeaponMeleeDamage = ((bit & mask) == bit);
                    bit <<= 1;
                    DualWieldable = ((bit & mask) == bit);
                }
            }
            /// <summary>
            /// 62. StringID :: Offset: 0x130 (304)
            /// </summary>
            public StringID Label { get { return __label__; } set { __label__ = value; } }
            private StringID __label__;
            /// <summary>
            /// 63. Enum32 :: Offset: 0x134 (308)
            /// </summary>
            public SecondaryTriggerModeEnum32 SecondaryTriggerMode { get { return __secondaryTriggerMode__; } set { __secondaryTriggerMode__ = value; } }
            private SecondaryTriggerModeEnum32 __secondaryTriggerMode__;
            public class SecondaryTriggerModeEnum32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Normal { get { return __normal__; } set { __normal__ = value; } }
                private bool __normal__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool SlavedToPrimary { get { return __slavedToPrimary__; } set { __slavedToPrimary__ = value; } }
                private bool __slavedToPrimary__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool InhibitsPrimary { get { return __inhibitsPrimary__; } set { __inhibitsPrimary__ = value; } }
                private bool __inhibitsPrimary__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool LoadsAlternateAmmunition { get { return __loadsAlternateAmmunition__; } set { __loadsAlternateAmmunition__ = value; } }
                private bool __loadsAlternateAmmunition__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool LoadsMultiplePrimaryAmmunition { get { return __loadsMultiplePrimaryAmmunition__; } set { __loadsMultiplePrimaryAmmunition__ = value; } }
                private bool __loadsMultiplePrimaryAmmunition__;

                public void _Initialize(BitReader br)
                {
                    uint value = br.ReadUInt32();
                    uint cur = 0;

                    Normal = (cur == value);
                    cur++;
                    SlavedToPrimary = (cur == value);
                    cur++;
                    InhibitsPrimary = (cur == value);
                    cur++;
                    LoadsAlternateAmmunition = (cur == value);
                    cur++;
                    LoadsMultiplePrimaryAmmunition = (cur == value);
                }
            }
            /// <summary>
            /// 64. Float :: Offset: 0x138 (312)
            /// </summary>
            public float MaxAlternateShotsLoaded { get { return __maxAlternateShotsLoaded__; } set { __maxAlternateShotsLoaded__ = value; } }
            private float __maxAlternateShotsLoaded__;
            /// <summary>
            /// 65. Float :: Offset: 0x13C (316)
            /// </summary>
            public float ReadyTime { get { return __readyTime__; } set { __readyTime__ = value; } }
            private float __readyTime__;
            /// <summary>
            /// 66. Tag :: Offset: 0x140 (320)
            /// </summary>
            public TagRef ReadyEffect { get { return __readyEffect__; } set { __readyEffect__ = value; } }
            private TagRef __readyEffect__;
            /// <summary>
            /// 67. Tag :: Offset: 0x148 (328)
            /// </summary>
            public TagRef Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
            private TagRef __unused3__;
            /// <summary>
            /// 68. Float :: Offset: 0x150 (336)
            /// </summary>
            public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
            private float __unknown1__;
            /// <summary>
            /// 69. Float :: Offset: 0x154 (340)
            /// </summary>
            public float HeatRecoveryThreshold { get { return __heatRecoveryThreshold__; } set { __heatRecoveryThreshold__ = value; } }
            private float __heatRecoveryThreshold__;
            /// <summary>
            /// 70. Float :: Offset: 0x158 (344)
            /// </summary>
            public float OverheatedThreshold { get { return __overheatedThreshold__; } set { __overheatedThreshold__ = value; } }
            private float __overheatedThreshold__;
            /// <summary>
            /// 71. Float :: Offset: 0x15C (348)
            /// </summary>
            public float HeatDetonationThreshold { get { return __heatDetonationThreshold__; } set { __heatDetonationThreshold__ = value; } }
            private float __heatDetonationThreshold__;
            /// <summary>
            /// 72. Float :: Offset: 0x160 (352)
            /// </summary>
            public float HeatDetonationFraction { get { return __heatDetonationFraction__; } set { __heatDetonationFraction__ = value; } }
            private float __heatDetonationFraction__;
            /// <summary>
            /// 73. Float :: Offset: 0x164 (356)
            /// </summary>
            public float HeatLossPerSecond { get { return __heatLossPerSecond__; } set { __heatLossPerSecond__ = value; } }
            private float __heatLossPerSecond__;
            /// <summary>
            /// 74. Float :: Offset: 0x168 (360)
            /// </summary>
            public float HeatIllumination { get { return __heatIllumination__; } set { __heatIllumination__ = value; } }
            private float __heatIllumination__;
            /// <summary>
            /// 75. Tag :: Offset: 0x16C (364)
            /// </summary>
            public TagRef OverheatEffect { get { return __overheatEffect__; } set { __overheatEffect__ = value; } }
            private TagRef __overheatEffect__;
            /// <summary>
            /// 76. Tag :: Offset: 0x174 (372)
            /// </summary>
            public TagRef Unused4 { get { return __unused4__; } set { __unused4__ = value; } }
            private TagRef __unused4__;
            /// <summary>
            /// 77. Tag :: Offset: 0x17C (380)
            /// </summary>
            public TagRef Unused5 { get { return __unused5__; } set { __unused5__ = value; } }
            private TagRef __unused5__;
            /// <summary>
            /// 78. Tag :: Offset: 0x184 (388)
            /// </summary>
            public TagRef Unused6 { get { return __unused6__; } set { __unused6__ = value; } }
            private TagRef __unused6__;
            /// <summary>
            /// 79. Tag :: Offset: 0x18C (396)
            /// </summary>
            public TagRef Unused7 { get { return __unused7__; } set { __unused7__ = value; } }
            private TagRef __unused7__;
            /// <summary>
            /// 80. Tag :: Offset: 0x194 (404)
            /// </summary>
            public TagRef Unused8 { get { return __unused8__; } set { __unused8__ = value; } }
            private TagRef __unused8__;
            /// <summary>
            /// 81. Float :: Offset: 0x19C (412)
            /// </summary>
            public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
            private float __unknown2__;
            /// <summary>
            /// 82. Float :: Offset: 0x1A0 (416)
            /// </summary>
            public float Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
            private float __unknown3__;
            /// <summary>
            /// 83. Float :: Offset: 0x1A4 (420)
            /// </summary>
            public float Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
            private float __unknown4__;
            /// <summary>
            /// 84. Float :: Offset: 0x1A8 (424)
            /// </summary>
            public float Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
            private float __unknown5__;
            /// <summary>
            /// 85. Float :: Offset: 0x1AC (428)
            /// </summary>
            public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
            private float __unknown6__;
            /// <summary>
            /// 86. Float :: Offset: 0x1B0 (432)
            /// </summary>
            public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
            private float __unknown7__;
            /// <summary>
            /// 87. Float :: Offset: 0x1B4 (436)
            /// </summary>
            public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
            private float __unknown8__;
            /// <summary>
            /// 88. Float :: Offset: 0x1B8 (440)
            /// </summary>
            public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
            private float __unknown9__;
            /// <summary>
            /// 89. Tag :: Offset: 0x1BC (444)
            /// </summary>
            public TagRef MeleeDamage1 { get { return __meleeDamage1__; } set { __meleeDamage1__ = value; } }
            private TagRef __meleeDamage1__;
            /// <summary>
            /// 90. Tag :: Offset: 0x1C4 (452)
            /// </summary>
            public TagRef MeleeResponseDamage1 { get { return __meleeResponseDamage1__; } set { __meleeResponseDamage1__ = value; } }
            private TagRef __meleeResponseDamage1__;
            /// <summary>
            /// 91. Tag :: Offset: 0x1CC (460)
            /// </summary>
            public TagRef MeleeDamage2 { get { return __meleeDamage2__; } set { __meleeDamage2__ = value; } }
            private TagRef __meleeDamage2__;
            /// <summary>
            /// 92. Tag :: Offset: 0x1D4 (468)
            /// </summary>
            public TagRef MeleeResponseDamage2 { get { return __meleeResponseDamage2__; } set { __meleeResponseDamage2__ = value; } }
            private TagRef __meleeResponseDamage2__;
            /// <summary>
            /// 93. Tag :: Offset: 0x1DC (476)
            /// </summary>
            public TagRef SmashMelee { get { return __smashMelee__; } set { __smashMelee__ = value; } }
            private TagRef __smashMelee__;
            /// <summary>
            /// 94. Tag :: Offset: 0x1E4 (484)
            /// </summary>
            public TagRef SmashMeleeResponse { get { return __smashMeleeResponse__; } set { __smashMeleeResponse__ = value; } }
            private TagRef __smashMeleeResponse__;
            /// <summary>
            /// 95. Tag :: Offset: 0x1EC (492)
            /// </summary>
            public TagRef DashMelee { get { return __dashMelee__; } set { __dashMelee__ = value; } }
            private TagRef __dashMelee__;
            /// <summary>
            /// 96. Tag :: Offset: 0x1F4 (500)
            /// </summary>
            public TagRef DashMeleeResponse { get { return __dashMeleeResponse__; } set { __dashMeleeResponse__ = value; } }
            private TagRef __dashMeleeResponse__;
            /// <summary>
            /// 97. Int16 :: Offset: 0x1FC (508)
            /// </summary>
            public short Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
            private short __unknown10__;
            /// <summary>
            /// 98. Int16 :: Offset: 0x1FE (510)
            /// </summary>
            public short ZoomLevels { get { return __zoomLevels__; } set { __zoomLevels__ = value; } }
            private short __zoomLevels__;
            /// <summary>
            /// 99. Float :: Offset: 0x200 (512)
            /// </summary>
            public float MinZoom { get { return __minZoom__; } set { __minZoom__ = value; } }
            private float __minZoom__;
            /// <summary>
            /// 100. Float :: Offset: 0x204 (516)
            /// </summary>
            public float MaxZoom { get { return __maxZoom__; } set { __maxZoom__ = value; } }
            private float __maxZoom__;
            /// <summary>
            /// 101. Float :: Offset: 0x208 (520)
            /// </summary>
            public float AutoAimAngle_Radians { get { return __autoAimAngle_Radians__; } set { __autoAimAngle_Radians__ = value; } }
            private float __autoAimAngle_Radians__;
            /// <summary>
            /// 102. Float :: Offset: 0x20C (524)
            /// </summary>
            public float AutoAimRange { get { return __autoAimRange__; } set { __autoAimRange__ = value; } }
            private float __autoAimRange__;
            /// <summary>
            /// 103. Float :: Offset: 0x210 (528)
            /// </summary>
            public float MagnetismAngle_Radians { get { return __magnetismAngle_Radians__; } set { __magnetismAngle_Radians__ = value; } }
            private float __magnetismAngle_Radians__;
            /// <summary>
            /// 104. Float :: Offset: 0x214 (532)
            /// </summary>
            public float MagnetismRange { get { return __magnetismRange__; } set { __magnetismRange__ = value; } }
            private float __magnetismRange__;
            /// <summary>
            /// 105. Float :: Offset: 0x218 (536)
            /// </summary>
            public float DeviationAngle_Radians { get { return __deviationAngle_Radians__; } set { __deviationAngle_Radians__ = value; } }
            private float __deviationAngle_Radians__;
            /// <summary>
            /// 106. Enum32 :: Offset: 0x21C (540)
            /// </summary>
            public MovementPenalizedEnum32 MovementPenalized { get { return __movementPenalized__; } set { __movementPenalized__ = value; } }
            private MovementPenalizedEnum32 __movementPenalized__;
            public class MovementPenalizedEnum32
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Always { get { return __always__; } set { __always__ = value; } }
                private bool __always__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool WhenZoomed { get { return __whenZoomed__; } set { __whenZoomed__ = value; } }
                private bool __whenZoomed__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool WhenZoomedOrReloading { get { return __whenZoomedOrReloading__; } set { __whenZoomedOrReloading__ = value; } }
                private bool __whenZoomedOrReloading__;

                public void _Initialize(BitReader br)
                {
                    uint value = br.ReadUInt32();
                    uint cur = 0;

                    Always = (cur == value);
                    cur++;
                    WhenZoomed = (cur == value);
                    cur++;
                    WhenZoomedOrReloading = (cur == value);
                }
            }
            /// <summary>
            /// 107. Float :: Offset: 0x220 (544)
            /// </summary>
            public float ForwardsMovementPenalty { get { return __forwardsMovementPenalty__; } set { __forwardsMovementPenalty__ = value; } }
            private float __forwardsMovementPenalty__;
            /// <summary>
            /// 108. Float :: Offset: 0x224 (548)
            /// </summary>
            public float BackwardsMovementPenalty { get { return __backwardsMovementPenalty__; } set { __backwardsMovementPenalty__ = value; } }
            private float __backwardsMovementPenalty__;
            /// <summary>
            /// 109. Float :: Offset: 0x228 (552)
            /// </summary>
            public float SidewaysMovementPenalty { get { return __sidewaysMovementPenalty__; } set { __sidewaysMovementPenalty__ = value; } }
            private float __sidewaysMovementPenalty__;
            /// <summary>
            /// 110. Float :: Offset: 0x22C (556)
            /// </summary>
            public float AIUnknown1 { get { return __aIUnknown1__; } set { __aIUnknown1__ = value; } }
            private float __aIUnknown1__;
            /// <summary>
            /// 111. Float :: Offset: 0x230 (560)
            /// </summary>
            public float AIUnknown2 { get { return __aIUnknown2__; } set { __aIUnknown2__ = value; } }
            private float __aIUnknown2__;
            /// <summary>
            /// 112. Float :: Offset: 0x234 (564)
            /// </summary>
            public float AIMinimumTargetRange { get { return __aIMinimumTargetRange__; } set { __aIMinimumTargetRange__ = value; } }
            private float __aIMinimumTargetRange__;
            /// <summary>
            /// 113. Float :: Offset: 0x238 (568)
            /// </summary>
            public float AILookingTimeModifier { get { return __aILookingTimeModifier__; } set { __aILookingTimeModifier__ = value; } }
            private float __aILookingTimeModifier__;
            /// <summary>
            /// 114. Float :: Offset: 0x23C (572)
            /// </summary>
            public float LightPowerOnTime { get { return __lightPowerOnTime__; } set { __lightPowerOnTime__ = value; } }
            private float __lightPowerOnTime__;
            /// <summary>
            /// 115. Float :: Offset: 0x240 (576)
            /// </summary>
            public float LightPowerOffTime { get { return __lightPowerOffTime__; } set { __lightPowerOffTime__ = value; } }
            private float __lightPowerOffTime__;
            /// <summary>
            /// 116. Tag :: Offset: 0x244 (580)
            /// </summary>
            public TagRef LightPowerOnEffect { get { return __lightPowerOnEffect__; } set { __lightPowerOnEffect__ = value; } }
            private TagRef __lightPowerOnEffect__;
            /// <summary>
            /// 117. Tag :: Offset: 0x24C (588)
            /// </summary>
            public TagRef LightPowerOffEffect { get { return __lightPowerOffEffect__; } set { __lightPowerOffEffect__ = value; } }
            private TagRef __lightPowerOffEffect__;
            /// <summary>
            /// 118. Float :: Offset: 0x254 (596)
            /// </summary>
            public float AgeHeatRecoveryPenalty { get { return __ageHeatRecoveryPenalty__; } set { __ageHeatRecoveryPenalty__ = value; } }
            private float __ageHeatRecoveryPenalty__;
            /// <summary>
            /// 119. Float :: Offset: 0x258 (600)
            /// </summary>
            public float AgeRateOfFirePenalty { get { return __ageRateOfFirePenalty__; } set { __ageRateOfFirePenalty__ = value; } }
            private float __ageRateOfFirePenalty__;
            /// <summary>
            /// 120. Float :: Offset: 0x25C (604)
            /// </summary>
            public float AgeMisfireStart { get { return __ageMisfireStart__; } set { __ageMisfireStart__ = value; } }
            private float __ageMisfireStart__;
            /// <summary>
            /// 121. Float :: Offset: 0x260 (608)
            /// </summary>
            public float AgeMisfireChance { get { return __ageMisfireChance__; } set { __ageMisfireChance__ = value; } }
            private float __ageMisfireChance__;
            /// <summary>
            /// 122. Tag :: Offset: 0x264 (612)
            /// </summary>
            public TagRef PickupSound { get { return __pickupSound__; } set { __pickupSound__ = value; } }
            private TagRef __pickupSound__;
            /// <summary>
            /// 123. Tag :: Offset: 0x26C (620)
            /// </summary>
            public TagRef ZoomInSound { get { return __zoomInSound__; } set { __zoomInSound__ = value; } }
            private TagRef __zoomInSound__;
            /// <summary>
            /// 124. Tag :: Offset: 0x274 (628)
            /// </summary>
            public TagRef ZoomOutSound { get { return __zoomOutSound__; } set { __zoomOutSound__ = value; } }
            private TagRef __zoomOutSound__;
            /// <summary>
            /// 125. Float :: Offset: 0x27C (636)
            /// </summary>
            public float ActiveCamoDing { get { return __activeCamoDing__; } set { __activeCamoDing__ = value; } }
            private float __activeCamoDing__;
            /// <summary>
            /// 126. ByteArray :: Offset: 0x280 (640)
            /// </summary>
            public byte[] Unused9 { get { return __unused9__; } set { __unused9__ = value; } }
            private byte[] __unused9__;
            /// <summary>
            /// 127. StringID :: Offset: 0x288 (648)
            /// </summary>
            public StringID WeaponType1 { get { return __weaponType1__; } set { __weaponType1__ = value; } }
            private StringID __weaponType1__;
            /// <summary>
            /// 128. StringID :: Offset: 0x28C (652)
            /// </summary>
            public StringID WeaponShortForm { get { return __weaponShortForm__; } set { __weaponShortForm__ = value; } }
            private StringID __weaponShortForm__;
            /// <summary>
            /// 129. Int16 :: Offset: 0x290 (656)
            /// </summary>
            public short Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
            private short __unknown11__;
            /// <summary>
            /// 130. Enum16 :: Offset: 0x292 (658)
            /// </summary>
            public WeaponType2Enum16 WeaponType2 { get { return __weaponType2__; } set { __weaponType2__ = value; } }
            private WeaponType2Enum16 __weaponType2__;
            public class WeaponType2Enum16
            {
                /// <summary>
                /// 1. Option :: Value: 0x0 (0)
                /// </summary>
                public bool Undefined { get { return __undefined__; } set { __undefined__ = value; } }
                private bool __undefined__;
                /// <summary>
                /// 2. Option :: Value: 0x1 (1)
                /// </summary>
                public bool Shotgun { get { return __shotgun__; } set { __shotgun__ = value; } }
                private bool __shotgun__;
                /// <summary>
                /// 3. Option :: Value: 0x2 (2)
                /// </summary>
                public bool Needler { get { return __needler__; } set { __needler__ = value; } }
                private bool __needler__;
                /// <summary>
                /// 4. Option :: Value: 0x3 (3)
                /// </summary>
                public bool PlasmaPistol { get { return __plasmaPistol__; } set { __plasmaPistol__ = value; } }
                private bool __plasmaPistol__;
                /// <summary>
                /// 5. Option :: Value: 0x4 (4)
                /// </summary>
                public bool PlasmaRifle { get { return __plasmaRifle__; } set { __plasmaRifle__ = value; } }
                private bool __plasmaRifle__;
                /// <summary>
                /// 6. Option :: Value: 0x5 (5)
                /// </summary>
                public bool RocketLauncher { get { return __rocketLauncher__; } set { __rocketLauncher__ = value; } }
                private bool __rocketLauncher__;

                public void _Initialize(BitReader br)
                {
                    ushort value = br.ReadUInt16();
                    ushort cur = 0;

                    Undefined = (cur == value);
                    cur++;
                    Shotgun = (cur == value);
                    cur++;
                    Needler = (cur == value);
                    cur++;
                    PlasmaPistol = (cur == value);
                    cur++;
                    PlasmaRifle = (cur == value);
                    cur++;
                    RocketLauncher = (cur == value);
                }
            }
            /// <summary>
            /// 131. Enum32 :: Offset: 0x294 (660)
            /// </summary>
            public Unknown12Enum32 Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
            private Unknown12Enum32 __unknown12__;
            public class Unknown12Enum32
            {

                public void _Initialize(BitReader br)
                {
                    // Empty..
                    br.BaseStream.Position += 4;
                }
            }
            /// <summary>
            /// 132. ByteArray :: Offset: 0x298 (664)
            /// </summary>
            public byte[] Unused10 { get { return __unused10__; } set { __unused10__ = value; } }
            private byte[] __unused10__;
            /// <summary>
            /// 133. TagBlock :: Offset: 0x2A8 (680)
            /// </summary>
            public _FPModelsTagBlock FPModels { get { return __fPModels__; } set { __fPModels__ = value; } }
            private _FPModelsTagBlock __fPModels__;
            public class _FPModelsTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Tag :: Offset: 0x0 (0)
                /// </summary>
                public TagRef Model { get { return __model__; } set { __model__ = value; } }
                private TagRef __model__;
                /// <summary>
                /// 2. Tag :: Offset: 0x8 (8)
                /// </summary>
                public TagRef Animation { get { return __animation__; } set { __animation__ = value; } }
                private TagRef __animation__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _FPModelsTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _FPModelsTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_FPModelsTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _FPModelsTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 134. Tag :: Offset: 0x2B0 (688)
            /// </summary>
            public TagRef HUD { get { return __hUD__; } set { __hUD__ = value; } }
            private TagRef __hUD__;
            /// <summary>
            /// 135. TagBlock :: Offset: 0x2B8 (696)
            /// </summary>
            public _PredictedResource2TagBlock PredictedResource2 { get { return __predictedResource2__; } set { __predictedResource2__ = value; } }
            private _PredictedResource2TagBlock __predictedResource2__;
            public class _PredictedResource2TagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Float :: Offset: 0x0 (0)
                /// </summary>
                public float Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                private float __unknown__;
                /// <summary>
                /// 2. ID :: Offset: 0x4 (4)
                /// </summary>
                public LoneID Resource { get { return __resource__; } set { __resource__ = value; } }
                private LoneID __resource__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _PredictedResource2TagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _PredictedResource2TagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_PredictedResource2TagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _PredictedResource2TagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 136. TagBlock :: Offset: 0x2C0 (704)
            /// </summary>
            public _MagazineTagBlock Magazine { get { return __magazine__; } set { __magazine__ = value; } }
            private _MagazineTagBlock __magazine__;
            public class _MagazineTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool WastesRoundsWhenReloaded { get { return __wastesRoundsWhenReloaded__; } set { __wastesRoundsWhenReloaded__ = value; } }
                    private bool __wastesRoundsWhenReloaded__;
                    /// <summary>
                    /// 2. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool EveryRoundMustBeChambered { get { return __everyRoundMustBeChambered__; } set { __everyRoundMustBeChambered__ = value; } }
                    private bool __everyRoundMustBeChambered__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        WastesRoundsWhenReloaded = ((bit & mask) == bit);
                        bit <<= 1;
                        EveryRoundMustBeChambered = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 2. Int16 :: Offset: 0x4 (4)
                /// </summary>
                public short RoundsRecharged_perSecond { get { return __roundsRecharged_perSecond__; } set { __roundsRecharged_perSecond__ = value; } }
                private short __roundsRecharged_perSecond__;
                /// <summary>
                /// 3. Int16 :: Offset: 0x6 (6)
                /// </summary>
                public short InitialRounds { get { return __initialRounds__; } set { __initialRounds__ = value; } }
                private short __initialRounds__;
                /// <summary>
                /// 4. Int16 :: Offset: 0x8 (8)
                /// </summary>
                public short MaxRounds { get { return __maxRounds__; } set { __maxRounds__ = value; } }
                private short __maxRounds__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xA (10)
                /// </summary>
                public short MaxRoundsLoaded { get { return __maxRoundsLoaded__; } set { __maxRoundsLoaded__ = value; } }
                private short __maxRoundsLoaded__;
                /// <summary>
                /// 6. Int32 :: Offset: 0xC (12)
                /// </summary>
                public int MaxRoundsHeld { get { return __maxRoundsHeld__; } set { __maxRoundsHeld__ = value; } }
                private int __maxRoundsHeld__;
                /// <summary>
                /// 7. Float :: Offset: 0x10 (16)
                /// </summary>
                public float ReloadTime_seconds { get { return __reloadTime_seconds__; } set { __reloadTime_seconds__ = value; } }
                private float __reloadTime_seconds__;
                /// <summary>
                /// 8. Int32 :: Offset: 0x14 (20)
                /// </summary>
                public int RoundsReloaded { get { return __roundsReloaded__; } set { __roundsReloaded__ = value; } }
                private int __roundsReloaded__;
                /// <summary>
                /// 9. Float :: Offset: 0x18 (24)
                /// </summary>
                public float ChamberTime_seconds { get { return __chamberTime_seconds__; } set { __chamberTime_seconds__ = value; } }
                private float __chamberTime_seconds__;
                /// <summary>
                /// 10. ByteArray :: Offset: 0x1C (28)
                /// </summary>
                public byte[] Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private byte[] __unused1__;
                /// <summary>
                /// 11. Tag :: Offset: 0x34 (52)
                /// </summary>
                public TagRef ReloadingEffect { get { return __reloadingEffect__; } set { __reloadingEffect__ = value; } }
                private TagRef __reloadingEffect__;
                /// <summary>
                /// 12. Tag :: Offset: 0x3C (60)
                /// </summary>
                public TagRef ChamberingEffect { get { return __chamberingEffect__; } set { __chamberingEffect__ = value; } }
                private TagRef __chamberingEffect__;
                /// <summary>
                /// 13. Tag :: Offset: 0x44 (68)
                /// </summary>
                public TagRef Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private TagRef __unused2__;
                /// <summary>
                /// 14. Tag :: Offset: 0x4C (76)
                /// </summary>
                public TagRef Unused3 { get { return __unused3__; } set { __unused3__ = value; } }
                private TagRef __unused3__;
                /// <summary>
                /// 15. TagBlock :: Offset: 0x54 (84)
                /// </summary>
                public _MagazinesTagBlock Magazines { get { return __magazines__; } set { __magazines__ = value; } }
                private _MagazinesTagBlock __magazines__;
                public class _MagazinesTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private WeapLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int32 :: Offset: 0x0 (0)
                    /// </summary>
                    public int Rounds { get { return __rounds__; } set { __rounds__ = value; } }
                    private int __rounds__;
                    /// <summary>
                    /// 2. Tag :: Offset: 0x4 (4)
                    /// </summary>
                    public TagRef Equipment { get { return __equipment__; } set { __equipment__ = value; } }
                    private TagRef __equipment__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _MagazinesTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _MagazinesTagBlock(WeapLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_MagazinesTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _MagazinesTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _MagazineTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _MagazineTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                    Magazines = new _MagazinesTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_MagazineTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _MagazineTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 137. TagBlock :: Offset: 0x2C8 (712)
            /// </summary>
            public _TriggersTagBlock Triggers { get { return __triggers__; } set { __triggers__ = value; } }
            private _TriggersTagBlock __triggers__;
            public class _TriggersTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {

                    public void _Initialize(BitReader br)
                    {
                        // Empty..
                        br.BaseStream.Position += 4;
                    }
                }
                /// <summary>
                /// 2. Enum16 :: Offset: 0x4 (4)
                /// </summary>
                public ButtonUsedEnum16 ButtonUsed { get { return __buttonUsed__; } set { __buttonUsed__ = value; } }
                private ButtonUsedEnum16 __buttonUsed__;
                public class ButtonUsedEnum16
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool R { get { return __r__; } set { __r__ = value; } }
                    private bool __r__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool L { get { return __l__; } set { __l__ = value; } }
                    private bool __l__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool B_SPOnly { get { return __b_SPOnly__; } set { __b_SPOnly__ = value; } }
                    private bool __b_SPOnly__;
                    /// <summary>
                    /// 4. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool AutomatedFire { get { return __automatedFire__; } set { __automatedFire__ = value; } }
                    private bool __automatedFire__;

                    public void _Initialize(BitReader br)
                    {
                        ushort value = br.ReadUInt16();
                        ushort cur = 0;

                        R = (cur == value);
                        cur++;
                        L = (cur == value);
                        cur++;
                        B_SPOnly = (cur == value);
                        cur++;
                        AutomatedFire = (cur == value);
                    }
                }
                /// <summary>
                /// 3. Enum16 :: Offset: 0x6 (6)
                /// </summary>
                public TypeEnum16 Type { get { return __type__; } set { __type__ = value; } }
                private TypeEnum16 __type__;
                public class TypeEnum16
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool Automatic { get { return __automatic__; } set { __automatic__ = value; } }
                    private bool __automatic__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool SemiAutomatic { get { return __semiAutomatic__; } set { __semiAutomatic__ = value; } }
                    private bool __semiAutomatic__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool N1 { get { return __n1__; } set { __n1__ = value; } }
                    private bool __n1__;
                    /// <summary>
                    /// 4. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool Unknown { get { return __unknown__; } set { __unknown__ = value; } }
                    private bool __unknown__;
                    /// <summary>
                    /// 5. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool N2 { get { return __n2__; } set { __n2__ = value; } }
                    private bool __n2__;
                    /// <summary>
                    /// 6. Option :: Value: 0x5 (5)
                    /// </summary>
                    public bool N3 { get { return __n3__; } set { __n3__ = value; } }
                    private bool __n3__;

                    public void _Initialize(BitReader br)
                    {
                        ushort value = br.ReadUInt16();
                        ushort cur = 0;

                        Automatic = (cur == value);
                        cur++;
                        SemiAutomatic = (cur == value);
                        cur++;
                        N1 = (cur == value);
                        cur++;
                        Unknown = (cur == value);
                        cur++;
                        N2 = (cur == value);
                        cur++;
                        N3 = (cur == value);
                    }
                }
                /// <summary>
                /// 4. Int16 :: Offset: 0x8 (8)
                /// </summary>
                public short Proj1 { get { return __proj1__; } set { __proj1__ = value; } }
                private short __proj1__;
                /// <summary>
                /// 5. Int16 :: Offset: 0xA (10)
                /// </summary>
                public short Proj2 { get { return __proj2__; } set { __proj2__ = value; } }
                private short __proj2__;
                /// <summary>
                /// 6. Float :: Offset: 0xC (12)
                /// </summary>
                public float Error_min { get { return __error_min__; } set { __error_min__ = value; } }
                private float __error_min__;
                /// <summary>
                /// 7. Float :: Offset: 0x10 (16)
                /// </summary>
                public float Error_max { get { return __error_max__; } set { __error_max__ = value; } }
                private float __error_max__;
                /// <summary>
                /// 8. Float :: Offset: 0x14 (20)
                /// </summary>
                public float AccelerationTime { get { return __accelerationTime__; } set { __accelerationTime__ = value; } }
                private float __accelerationTime__;
                /// <summary>
                /// 9. Float :: Offset: 0x18 (24)
                /// </summary>
                public float DecelerationTime { get { return __decelerationTime__; } set { __decelerationTime__ = value; } }
                private float __decelerationTime__;
                /// <summary>
                /// 10. Float :: Offset: 0x1C (28)
                /// </summary>
                public float ChargingTime { get { return __chargingTime__; } set { __chargingTime__ = value; } }
                private float __chargingTime__;
                /// <summary>
                /// 11. Float :: Offset: 0x20 (32)
                /// </summary>
                public float ChargedTime { get { return __chargedTime__; } set { __chargedTime__ = value; } }
                private float __chargedTime__;
                /// <summary>
                /// 12. Enum32 :: Offset: 0x24 (36)
                /// </summary>
                public OverchargedActionEnum32 OverchargedAction { get { return __overchargedAction__; } set { __overchargedAction__ = value; } }
                private OverchargedActionEnum32 __overchargedAction__;
                public class OverchargedActionEnum32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool None { get { return __none__; } set { __none__ = value; } }
                    private bool __none__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool Explode { get { return __explode__; } set { __explode__ = value; } }
                    private bool __explode__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool Discharge { get { return __discharge__; } set { __discharge__ = value; } }
                    private bool __discharge__;

                    public void _Initialize(BitReader br)
                    {
                        uint value = br.ReadUInt32();
                        uint cur = 0;

                        None = (cur == value);
                        cur++;
                        Explode = (cur == value);
                        cur++;
                        Discharge = (cur == value);
                    }
                }
                /// <summary>
                /// 13. Float :: Offset: 0x28 (40)
                /// </summary>
                public float ChargedIllumination { get { return __chargedIllumination__; } set { __chargedIllumination__ = value; } }
                private float __chargedIllumination__;
                /// <summary>
                /// 14. Float :: Offset: 0x2C (44)
                /// </summary>
                public float SpewTime { get { return __spewTime__; } set { __spewTime__ = value; } }
                private float __spewTime__;
                /// <summary>
                /// 15. Tag :: Offset: 0x30 (48)
                /// </summary>
                public TagRef ChargingEffect { get { return __chargingEffect__; } set { __chargingEffect__ = value; } }
                private TagRef __chargingEffect__;
                /// <summary>
                /// 16. Tag :: Offset: 0x38 (56)
                /// </summary>
                public TagRef OverchargeDamage { get { return __overchargeDamage__; } set { __overchargeDamage__ = value; } }
                private TagRef __overchargeDamage__;

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _TriggersTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _TriggersTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                    ButtonUsed = new ButtonUsedEnum16();
                    Type = new TypeEnum16();
                    OverchargedAction = new OverchargedActionEnum32();
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_TriggersTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _TriggersTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 138. TagBlock :: Offset: 0x2D0 (720)
            /// </summary>
            public _ProjectilesTagBlock Projectiles { get { return __projectiles__; } set { __projectiles__ = value; } }
            private _ProjectilesTagBlock __projectiles__;
            public class _ProjectilesTagBlock
            {
                #region Class Instance Data, Accessors, Classes

                /// <summary>
                /// The root class.
                /// </summary>
                public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                private WeapLayout __layout__;
                /// <summary>
                /// The offset at which this reflexive is declared in the tag meta.
                /// </summary>
                public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                private long __declarationOffset__;
                /// <summary>
                /// The total amount of chunks.
                /// </summary>
                public int _Count { get { return __count__; } protected set { __count__ = value; } }
                private int __count__;
                /// <summary>
                /// The initial pointer read from the tag meta.
                /// </summary>
                public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                private int __pointer__;
                /// <summary>
                /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                /// </summary>
                public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                private int __translation__;
                /// <summary>
                /// The currently selected chunk's index.
                /// </summary>
                public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                private int __selectedIndex__;
                /// <summary>
                /// True if the reflexive does not exist in the current tag meta.
                /// </summary>
                public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                private bool __null__;
                /// <summary>
                /// True if Initialize() has been called, and executed, without error.
                /// </summary>
                public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                private bool __initialized__;

                #endregion

                #region Reflexive Instance Data, Accessors, Classes

                /// <summary>
                /// 1. Bitmask32 :: Offset: 0x0 (0)
                /// </summary>
                public FlagsBitmask32 Flags { get { return __flags__; } set { __flags__ = value; } }
                private FlagsBitmask32 __flags__;
                public class FlagsBitmask32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0xD (13)
                    /// </summary>
                    public bool Lockon { get { return __lockon__; } set { __lockon__ = value; } }
                    private bool __lockon__;

                    public void _Initialize(BitReader br)
                    {
                        uint mask = br.ReadUInt32();
                        uint bit = 1;

                        Lockon = ((bit & mask) == bit);
                    }
                }
                /// <summary>
                /// 2. Float :: Offset: 0x4 (4)
                /// </summary>
                public float RoundsPerSecond_min { get { return __roundsPerSecond_min__; } set { __roundsPerSecond_min__ = value; } }
                private float __roundsPerSecond_min__;
                /// <summary>
                /// 3. Float :: Offset: 0x8 (8)
                /// </summary>
                public float RoundsPerSecond_max { get { return __roundsPerSecond_max__; } set { __roundsPerSecond_max__ = value; } }
                private float __roundsPerSecond_max__;
                /// <summary>
                /// 4. Float :: Offset: 0xC (12)
                /// </summary>
                public float AccelerationTime_sec { get { return __accelerationTime_sec__; } set { __accelerationTime_sec__ = value; } }
                private float __accelerationTime_sec__;
                /// <summary>
                /// 5. Float :: Offset: 0x10 (16)
                /// </summary>
                public float DecelerationTime_sec { get { return __decelerationTime_sec__; } set { __decelerationTime_sec__ = value; } }
                private float __decelerationTime_sec__;
                /// <summary>
                /// 6. Float :: Offset: 0x14 (20)
                /// </summary>
                public float BlurredRateOfFire { get { return __blurredRateOfFire__; } set { __blurredRateOfFire__ = value; } }
                private float __blurredRateOfFire__;
                /// <summary>
                /// 7. Float :: Offset: 0x18 (24)
                /// </summary>
                public float Unknown1 { get { return __unknown1__; } set { __unknown1__ = value; } }
                private float __unknown1__;
                /// <summary>
                /// 8. Int16 :: Offset: 0x1C (28)
                /// </summary>
                public short RoundsPerShot { get { return __roundsPerShot__; } set { __roundsPerShot__ = value; } }
                private short __roundsPerShot__;
                /// <summary>
                /// 9. Int16 :: Offset: 0x1E (30)
                /// </summary>
                public short MinimumRoundsLoaded { get { return __minimumRoundsLoaded__; } set { __minimumRoundsLoaded__ = value; } }
                private short __minimumRoundsLoaded__;
                /// <summary>
                /// 10. Float :: Offset: 0x20 (32)
                /// </summary>
                public float ChamberTime_sec { get { return __chamberTime_sec__; } set { __chamberTime_sec__ = value; } }
                private float __chamberTime_sec__;
                /// <summary>
                /// 11. Float :: Offset: 0x24 (36)
                /// </summary>
                public float Unknown2 { get { return __unknown2__; } set { __unknown2__ = value; } }
                private float __unknown2__;
                /// <summary>
                /// 12. Int16 :: Offset: 0x28 (40)
                /// </summary>
                public short Magazine_index { get { return __magazine_index__; } set { __magazine_index__ = value; } }
                private short __magazine_index__;
                /// <summary>
                /// 13. Int16 :: Offset: 0x2A (42)
                /// </summary>
                public short AmmoPerShot { get { return __ammoPerShot__; } set { __ammoPerShot__ = value; } }
                private short __ammoPerShot__;
                /// <summary>
                /// 14. Enum32 :: Offset: 0x2C (44)
                /// </summary>
                public FiringNoiseEnum32 FiringNoise { get { return __firingNoise__; } set { __firingNoise__ = value; } }
                private FiringNoiseEnum32 __firingNoise__;
                public class FiringNoiseEnum32
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool Silent { get { return __silent__; } set { __silent__ = value; } }
                    private bool __silent__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool Medium { get { return __medium__; } set { __medium__ = value; } }
                    private bool __medium__;
                    /// <summary>
                    /// 3. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool Loud { get { return __loud__; } set { __loud__ = value; } }
                    private bool __loud__;
                    /// <summary>
                    /// 4. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool Shout { get { return __shout__; } set { __shout__ = value; } }
                    private bool __shout__;
                    /// <summary>
                    /// 5. Option :: Value: 0x4 (4)
                    /// </summary>
                    public bool Quiet { get { return __quiet__; } set { __quiet__ = value; } }
                    private bool __quiet__;

                    public void _Initialize(BitReader br)
                    {
                        uint value = br.ReadUInt32();
                        uint cur = 0;

                        Silent = (cur == value);
                        cur++;
                        Medium = (cur == value);
                        cur++;
                        Loud = (cur == value);
                        cur++;
                        Shout = (cur == value);
                        cur++;
                        Quiet = (cur == value);
                    }
                }
                /// <summary>
                /// 15. StringID :: Offset: 0x30 (48)
                /// </summary>
                public StringID Unknown3 { get { return __unknown3__; } set { __unknown3__ = value; } }
                private StringID __unknown3__;
                /// <summary>
                /// 16. Int16 :: Offset: 0x34 (52)
                /// </summary>
                public short Unknown4 { get { return __unknown4__; } set { __unknown4__ = value; } }
                private short __unknown4__;
                /// <summary>
                /// 17. Enum16 :: Offset: 0x36 (54)
                /// </summary>
                public Unknown5Enum16 Unknown5 { get { return __unknown5__; } set { __unknown5__ = value; } }
                private Unknown5Enum16 __unknown5__;
                public class Unknown5Enum16
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x2 (2)
                    /// </summary>
                    public bool UnaffectedByGravityPossibly { get { return __unaffectedByGravityPossibly__; } set { __unaffectedByGravityPossibly__ = value; } }
                    private bool __unaffectedByGravityPossibly__;
                    /// <summary>
                    /// 2. Option :: Value: 0x3 (3)
                    /// </summary>
                    public bool AffectedByGravityPossibly { get { return __affectedByGravityPossibly__; } set { __affectedByGravityPossibly__ = value; } }
                    private bool __affectedByGravityPossibly__;

                    public void _Initialize(BitReader br)
                    {
                        ushort value = br.ReadUInt16();
                        ushort cur = 0;

                        UnaffectedByGravityPossibly = (cur == value);
                        cur++;
                        AffectedByGravityPossibly = (cur == value);
                    }
                }
                /// <summary>
                /// 18. Float :: Offset: 0x38 (56)
                /// </summary>
                public float Unknown6 { get { return __unknown6__; } set { __unknown6__ = value; } }
                private float __unknown6__;
                /// <summary>
                /// 19. Float :: Offset: 0x3C (60)
                /// </summary>
                public float Unknown7 { get { return __unknown7__; } set { __unknown7__ = value; } }
                private float __unknown7__;
                /// <summary>
                /// 20. Float :: Offset: 0x40 (64)
                /// </summary>
                public float Unknown8 { get { return __unknown8__; } set { __unknown8__ = value; } }
                private float __unknown8__;
                /// <summary>
                /// 21. Float :: Offset: 0x44 (68)
                /// </summary>
                public float Unknown9 { get { return __unknown9__; } set { __unknown9__ = value; } }
                private float __unknown9__;
                /// <summary>
                /// 22. Float :: Offset: 0x48 (72)
                /// </summary>
                public float Unknown10 { get { return __unknown10__; } set { __unknown10__ = value; } }
                private float __unknown10__;
                /// <summary>
                /// 23. Float :: Offset: 0x4C (76)
                /// </summary>
                public float Unknown11 { get { return __unknown11__; } set { __unknown11__ = value; } }
                private float __unknown11__;
                /// <summary>
                /// 24. Float :: Offset: 0x50 (80)
                /// </summary>
                public float Unknown12 { get { return __unknown12__; } set { __unknown12__ = value; } }
                private float __unknown12__;
                /// <summary>
                /// 25. Float :: Offset: 0x54 (84)
                /// </summary>
                public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
                private float __unknown13__;
                /// <summary>
                /// 26. Float :: Offset: 0x58 (88)
                /// </summary>
                public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
                private float __unknown14__;
                /// <summary>
                /// 27. Float :: Offset: 0x5C (92)
                /// </summary>
                public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
                private float __unknown15__;
                /// <summary>
                /// 28. Float :: Offset: 0x60 (96)
                /// </summary>
                public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
                private float __unknown16__;
                /// <summary>
                /// 29. Float :: Offset: 0x64 (100)
                /// </summary>
                public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
                private float __unknown17__;
                /// <summary>
                /// 30. Enum16 :: Offset: 0x68 (104)
                /// </summary>
                public DistributionFunctionEnum16 DistributionFunction { get { return __distributionFunction__; } set { __distributionFunction__ = value; } }
                private DistributionFunctionEnum16 __distributionFunction__;
                public class DistributionFunctionEnum16
                {
                    /// <summary>
                    /// 1. Option :: Value: 0x0 (0)
                    /// </summary>
                    public bool Point { get { return __point__; } set { __point__ = value; } }
                    private bool __point__;
                    /// <summary>
                    /// 2. Option :: Value: 0x1 (1)
                    /// </summary>
                    public bool Fan { get { return __fan__; } set { __fan__ = value; } }
                    private bool __fan__;

                    public void _Initialize(BitReader br)
                    {
                        ushort value = br.ReadUInt16();
                        ushort cur = 0;

                        Point = (cur == value);
                        cur++;
                        Fan = (cur == value);
                    }
                }
                /// <summary>
                /// 31. Int16 :: Offset: 0x6A (106)
                /// </summary>
                public short ProjectilesPerShot { get { return __projectilesPerShot__; } set { __projectilesPerShot__ = value; } }
                private short __projectilesPerShot__;
                /// <summary>
                /// 32. Float :: Offset: 0x6C (108)
                /// </summary>
                public float DistributionAngle { get { return __distributionAngle__; } set { __distributionAngle__ = value; } }
                private float __distributionAngle__;
                /// <summary>
                /// 33. Float :: Offset: 0x70 (112)
                /// </summary>
                public float MinimumError { get { return __minimumError__; } set { __minimumError__ = value; } }
                private float __minimumError__;
                /// <summary>
                /// 34. Float :: Offset: 0x74 (116)
                /// </summary>
                public float ErrorAngle_min { get { return __errorAngle_min__; } set { __errorAngle_min__ = value; } }
                private float __errorAngle_min__;
                /// <summary>
                /// 35. Float :: Offset: 0x78 (120)
                /// </summary>
                public float ErrorAngle_max { get { return __errorAngle_max__; } set { __errorAngle_max__ = value; } }
                private float __errorAngle_max__;
                /// <summary>
                /// 36. Float :: Offset: 0x7C (124)
                /// </summary>
                public float SpawnOffset_x { get { return __spawnOffset_x__; } set { __spawnOffset_x__ = value; } }
                private float __spawnOffset_x__;
                /// <summary>
                /// 37. Float :: Offset: 0x80 (128)
                /// </summary>
                public float SpawnOffset_y { get { return __spawnOffset_y__; } set { __spawnOffset_y__ = value; } }
                private float __spawnOffset_y__;
                /// <summary>
                /// 38. Float :: Offset: 0x84 (132)
                /// </summary>
                public float SpawnOffset_z { get { return __spawnOffset_z__; } set { __spawnOffset_z__ = value; } }
                private float __spawnOffset_z__;
                /// <summary>
                /// 39. Float :: Offset: 0x88 (136)
                /// </summary>
                public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
                private float __unknown18__;
                /// <summary>
                /// 40. Tag :: Offset: 0x8C (140)
                /// </summary>
                public TagRef Projectile { get { return __projectile__; } set { __projectile__ = value; } }
                private TagRef __projectile__;
                /// <summary>
                /// 41. Tag :: Offset: 0x94 (148)
                /// </summary>
                public TagRef Unused1 { get { return __unused1__; } set { __unused1__ = value; } }
                private TagRef __unused1__;
                /// <summary>
                /// 42. Float :: Offset: 0x9C (156)
                /// </summary>
                public float EjectionPortRecoveryTime { get { return __ejectionPortRecoveryTime__; } set { __ejectionPortRecoveryTime__ = value; } }
                private float __ejectionPortRecoveryTime__;
                /// <summary>
                /// 43. Float :: Offset: 0xA0 (160)
                /// </summary>
                public float IlluminationRecoveryTime { get { return __illuminationRecoveryTime__; } set { __illuminationRecoveryTime__ = value; } }
                private float __illuminationRecoveryTime__;
                /// <summary>
                /// 44. Float :: Offset: 0xA4 (164)
                /// </summary>
                public float HeatGeneratedPerRound { get { return __heatGeneratedPerRound__; } set { __heatGeneratedPerRound__ = value; } }
                private float __heatGeneratedPerRound__;
                /// <summary>
                /// 45. Float :: Offset: 0xA8 (168)
                /// </summary>
                public float AgeGeneratedPerRound { get { return __ageGeneratedPerRound__; } set { __ageGeneratedPerRound__ = value; } }
                private float __ageGeneratedPerRound__;
                /// <summary>
                /// 46. ByteArray :: Offset: 0xAC (172)
                /// </summary>
                public byte[] Unused2 { get { return __unused2__; } set { __unused2__ = value; } }
                private byte[] __unused2__;
                /// <summary>
                /// 47. Float :: Offset: 0xB4 (180)
                /// </summary>
                public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
                private float __unknown19__;
                /// <summary>
                /// 48. Float :: Offset: 0xB8 (184)
                /// </summary>
                public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
                private float __unknown20__;
                /// <summary>
                /// 49. Float :: Offset: 0xBC (188)
                /// </summary>
                public float Unknown21 { get { return __unknown21__; } set { __unknown21__ = value; } }
                private float __unknown21__;
                /// <summary>
                /// 50. Float :: Offset: 0xC0 (192)
                /// </summary>
                public float Unknown22 { get { return __unknown22__; } set { __unknown22__ = value; } }
                private float __unknown22__;
                /// <summary>
                /// 51. Float :: Offset: 0xC4 (196)
                /// </summary>
                public float Unknown23 { get { return __unknown23__; } set { __unknown23__ = value; } }
                private float __unknown23__;
                /// <summary>
                /// 52. Float :: Offset: 0xC8 (200)
                /// </summary>
                public float Unknown24 { get { return __unknown24__; } set { __unknown24__ = value; } }
                private float __unknown24__;
                /// <summary>
                /// 53. Float :: Offset: 0xCC (204)
                /// </summary>
                public float Unknown25 { get { return __unknown25__; } set { __unknown25__ = value; } }
                private float __unknown25__;
                /// <summary>
                /// 54. Float :: Offset: 0xD0 (208)
                /// </summary>
                public float Unknown26 { get { return __unknown26__; } set { __unknown26__ = value; } }
                private float __unknown26__;
                /// <summary>
                /// 55. Float :: Offset: 0xD4 (212)
                /// </summary>
                public float Unknown27 { get { return __unknown27__; } set { __unknown27__ = value; } }
                private float __unknown27__;
                /// <summary>
                /// 56. Float :: Offset: 0xD8 (216)
                /// </summary>
                public float Unknown28 { get { return __unknown28__; } set { __unknown28__ = value; } }
                private float __unknown28__;
                /// <summary>
                /// 57. Float :: Offset: 0xDC (220)
                /// </summary>
                public float Unknown29 { get { return __unknown29__; } set { __unknown29__ = value; } }
                private float __unknown29__;
                /// <summary>
                /// 58. Float :: Offset: 0xE0 (224)
                /// </summary>
                public float Unknown30 { get { return __unknown30__; } set { __unknown30__ = value; } }
                private float __unknown30__;
                /// <summary>
                /// 59. TagBlock :: Offset: 0xE4 (228)
                /// </summary>
                public _FiringEffectsTagBlock FiringEffects { get { return __firingEffects__; } set { __firingEffects__ = value; } }
                private _FiringEffectsTagBlock __firingEffects__;
                public class _FiringEffectsTagBlock
                {
                    #region Class Instance Data, Accessors, Classes

                    /// <summary>
                    /// The root class.
                    /// </summary>
                    public WeapLayout _Layout { get { return __layout__; } protected set { __layout__ = value; } }
                    private WeapLayout __layout__;
                    /// <summary>
                    /// The offset at which this reflexive is declared in the tag meta.
                    /// </summary>
                    public long _DeclarationOffset { get { return __declarationOffset__; } protected set { __declarationOffset__ = value; } }
                    private long __declarationOffset__;
                    /// <summary>
                    /// The total amount of chunks.
                    /// </summary>
                    public int _Count { get { return __count__; } protected set { __count__ = value; } }
                    private int __count__;
                    /// <summary>
                    /// The initial pointer read from the tag meta.
                    /// </summary>
                    public int _Pointer { get { return __pointer__; } protected set { __pointer__ = value; } }
                    private int __pointer__;
                    /// <summary>
                    /// The pointer to the beginning of the first chunk. Calculated by subtracting the Map's Secondary Magic from the initial pointer.
                    /// </summary>
                    public int _Translation { get { return __translation__; } protected set { __translation__ = value; } }
                    private int __translation__;
                    /// <summary>
                    /// The currently selected chunk's index.
                    /// </summary>
                    public int _SelectedIndex { get { return __selectedIndex__; } protected set { __selectedIndex__ = value; } }
                    private int __selectedIndex__;
                    /// <summary>
                    /// True if the reflexive does not exist in the current tag meta.
                    /// </summary>
                    public bool _Null { get { return __null__; } protected set { __null__ = value; } }
                    private bool __null__;
                    /// <summary>
                    /// True if Initialize() has been called, and executed, without error.
                    /// </summary>
                    public bool _Initialized { get { return __initialized__; } protected set { __initialized__ = value; } }
                    private bool __initialized__;

                    #endregion

                    #region Reflexive Instance Data, Accessors, Classes

                    /// <summary>
                    /// 1. Int16 :: Offset: 0x0 (0)
                    /// </summary>
                    public short ShotCountLowerBound { get { return __shotCountLowerBound__; } set { __shotCountLowerBound__ = value; } }
                    private short __shotCountLowerBound__;
                    /// <summary>
                    /// 2. Int16 :: Offset: 0x2 (2)
                    /// </summary>
                    public short ShotCountUpperBound { get { return __shotCountUpperBound__; } set { __shotCountUpperBound__ = value; } }
                    private short __shotCountUpperBound__;
                    /// <summary>
                    /// 3. Tag :: Offset: 0x4 (4)
                    /// </summary>
                    public TagRef FiringEffect { get { return __firingEffect__; } set { __firingEffect__ = value; } }
                    private TagRef __firingEffect__;
                    /// <summary>
                    /// 4. Tag :: Offset: 0xC (12)
                    /// </summary>
                    public TagRef MisfireEffect { get { return __misfireEffect__; } set { __misfireEffect__ = value; } }
                    private TagRef __misfireEffect__;
                    /// <summary>
                    /// 5. Tag :: Offset: 0x14 (20)
                    /// </summary>
                    public TagRef EmptyEffect { get { return __emptyEffect__; } set { __emptyEffect__ = value; } }
                    private TagRef __emptyEffect__;
                    /// <summary>
                    /// 6. Tag :: Offset: 0x1C (28)
                    /// </summary>
                    public TagRef FiringDamage { get { return __firingDamage__; } set { __firingDamage__ = value; } }
                    private TagRef __firingDamage__;
                    /// <summary>
                    /// 7. Tag :: Offset: 0x24 (36)
                    /// </summary>
                    public TagRef MisfireDamage { get { return __misfireDamage__; } set { __misfireDamage__ = value; } }
                    private TagRef __misfireDamage__;
                    /// <summary>
                    /// 8. Tag :: Offset: 0x2C (44)
                    /// </summary>
                    public TagRef EmptyDamage { get { return __emptyDamage__; } set { __emptyDamage__ = value; } }
                    private TagRef __emptyDamage__;

                    #endregion

                    #region Initialization

                    /// <summary>
                    /// Initializes a new instance of the _FiringEffectsTagBlock class based 
                    /// on the supplied layout reference.
                    /// </summary>
                    /// <param name="layout">The layout reference. </param>
                    public _FiringEffectsTagBlock(WeapLayout layout)
                    {
                        this._Layout = layout;
                        this._Initialized = false;

                    }

                    /// <summary>
                    /// Parses the declaration of the reflexive into the several class members.
                    /// </summary>
                    /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                    public void _Initialize(BitReader br)
                    {
                        _Initialized = false;
                        int cc = br.ReadInt32();
                        int tr = br.ReadInt32();

                        if (tr == -1)
                        {
                            _Null = true;
                        }
                        else
                        {
                            _DeclarationOffset = br.BaseStream.Position - 8;
                            _Count = cc;
                            _Pointer = tr;
                            _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                            _SelectedIndex = 0;
                            _Null = false;
                        }
                        _Initialized = true;
                    }

                    #endregion

                    #region New, Add, Delete, Remove Methods

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    public void _New()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with null values. If the reflexive 
                    /// does not exist in the current tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _New(FileStream fs)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write null, -1 for pointers
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(_FiringEffectsTagBlock template)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._New(fs, template);
                        fs.Close();
                    }

                    /// <summary>
                    /// Overwrites the current reflexive with the values provided
                    /// by the template. If the reflexive does not exist in the current 
                    /// tag meta, space will be made.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="template">The template containing the new values. </param>
                    public void _New(FileStream fs, _FiringEffectsTagBlock template)
                    {
                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        if (_Null)
                        {
                            // Make space
                        }

                        // Write template
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    public void _Add()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be made up of null values.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Add(FileStream fs)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make null space
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Add(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                    /// be based off of the values found within the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                    public void _Add(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Make space, fill with template
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    public void _Delete()
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Delete(fs);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the reflexive if it exists in the current tag meta.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    public void _Delete(FileStream fs)
                    {
                        if (_Null) return;

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Delete

                        _Null = true;
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(int index)
                    {
                        FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                        this._Remove(fs, index);
                        fs.Close();
                    }

                    /// <summary>
                    /// Removes the chunk at the given index.
                    /// </summary>
                    /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                    /// <param name="index">The index of the chunk to remove. </param>
                    public void _Remove(FileStream fs, int index)
                    {
                        if (_Null) throw new Exception("Reflexive does not exist.");

                        BitReader br = new BitReader(fs, Endian.Little);
                        BitWriter bw = new BitWriter(fs, Endian.Little);

                        // Remove

                        if (--_Count == 0)
                        {
                            // Write -1 pointer
                            _Null = true;
                        }
                    }

                    #endregion
                }

                #endregion

                #region Initialization

                /// <summary>
                /// Initializes a new instance of the _ProjectilesTagBlock class based 
                /// on the supplied layout reference.
                /// </summary>
                /// <param name="layout">The layout reference. </param>
                public _ProjectilesTagBlock(WeapLayout layout)
                {
                    this._Layout = layout;
                    this._Initialized = false;

                    Flags = new FlagsBitmask32();
                    FiringNoise = new FiringNoiseEnum32();
                    Unknown5 = new Unknown5Enum16();
                    DistributionFunction = new DistributionFunctionEnum16();
                    FiringEffects = new _FiringEffectsTagBlock(layout);
                }

                /// <summary>
                /// Parses the declaration of the reflexive into the several class members.
                /// </summary>
                /// <param name="br">The reader to use for reading operations. Note that the current position of the reader is expected to be just before the declaration of this reflexive in the tag meta. </param>
                public void _Initialize(BitReader br)
                {
                    _Initialized = false;
                    int cc = br.ReadInt32();
                    int tr = br.ReadInt32();

                    if (tr == -1)
                    {
                        _Null = true;
                    }
                    else
                    {
                        _DeclarationOffset = br.BaseStream.Position - 8;
                        _Count = cc;
                        _Pointer = tr;
                        _Translation = tr - _Layout._Map.Index.SecondaryMagic;
                        _SelectedIndex = 0;
                        _Null = false;
                    }
                    _Initialized = true;
                }

                #endregion

                #region New, Add, Delete, Remove Methods

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                public void _New()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with null values. If the reflexive 
                /// does not exist in the current tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _New(FileStream fs)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write null, -1 for pointers
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="template">The template containing the new values. </param>
                public void _New(_ProjectilesTagBlock template)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._New(fs, template);
                    fs.Close();
                }

                /// <summary>
                /// Overwrites the current reflexive with the values provided
                /// by the template. If the reflexive does not exist in the current 
                /// tag meta, space will be made.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="template">The template containing the new values. </param>
                public void _New(FileStream fs, _ProjectilesTagBlock template)
                {
                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    if (_Null)
                    {
                        // Make space
                    }

                    // Write template
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                public void _Add()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be made up of null values.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Add(FileStream fs)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make null space
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Add(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Append's a new chunk to the reflexive in the tag meta. New chunk will
                /// be based off of the values found within the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk containing the values in which to base the new chunk off of. </param>
                public void _Add(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Make space, fill with template
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                public void _Delete()
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Delete(fs);
                    fs.Close();
                }

                /// <summary>
                /// Removes the reflexive if it exists in the current tag meta.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                public void _Delete(FileStream fs)
                {
                    if (_Null) return;

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Delete

                    _Null = true;
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(int index)
                {
                    FileStream fs = new FileStream(_Layout._Map.FileName, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    this._Remove(fs, index);
                    fs.Close();
                }

                /// <summary>
                /// Removes the chunk at the given index.
                /// </summary>
                /// <param name="fs">The System.IO.FileStream that is attached to the map file. </param>
                /// <param name="index">The index of the chunk to remove. </param>
                public void _Remove(FileStream fs, int index)
                {
                    if (_Null) throw new Exception("Reflexive does not exist.");

                    BitReader br = new BitReader(fs, Endian.Little);
                    BitWriter bw = new BitWriter(fs, Endian.Little);

                    // Remove

                    if (--_Count == 0)
                    {
                        // Write -1 pointer
                        _Null = true;
                    }
                }

                #endregion
            }
            /// <summary>
            /// 139. Float :: Offset: 0x2D8 (728)
            /// </summary>
            public float Unknown13 { get { return __unknown13__; } set { __unknown13__ = value; } }
            private float __unknown13__;
            /// <summary>
            /// 140. Float :: Offset: 0x2DC (732)
            /// </summary>
            public float Unknown14 { get { return __unknown14__; } set { __unknown14__ = value; } }
            private float __unknown14__;
            /// <summary>
            /// 141. Float :: Offset: 0x2E0 (736)
            /// </summary>
            public float Unknown15 { get { return __unknown15__; } set { __unknown15__ = value; } }
            private float __unknown15__;
            /// <summary>
            /// 142. Float :: Offset: 0x2E4 (740)
            /// </summary>
            public float Unknown16 { get { return __unknown16__; } set { __unknown16__ = value; } }
            private float __unknown16__;
            /// <summary>
            /// 143. Float :: Offset: 0x2E8 (744)
            /// </summary>
            public float Unknown17 { get { return __unknown17__; } set { __unknown17__ = value; } }
            private float __unknown17__;
            /// <summary>
            /// 144. Float :: Offset: 0x2EC (748)
            /// </summary>
            public float Unknown18 { get { return __unknown18__; } set { __unknown18__ = value; } }
            private float __unknown18__;
            /// <summary>
            /// 145. Tag :: Offset: 0x2F0 (752)
            /// </summary>
            public TagRef AgedVehicle { get { return __agedVehicle__; } set { __agedVehicle__ = value; } }
            private TagRef __agedVehicle__;
            /// <summary>
            /// 146. Tag :: Offset: 0x2F8 (760)
            /// </summary>
            public TagRef AgedEffect { get { return __agedEffect__; } set { __agedEffect__ = value; } }
            private TagRef __agedEffect__;
            /// <summary>
            /// 147. Tag :: Offset: 0x300 (768)
            /// </summary>
            public TagRef AgedWeapon { get { return __agedWeapon__; } set { __agedWeapon__ = value; } }
            private TagRef __agedWeapon__;
            /// <summary>
            /// 148. Float :: Offset: 0x308 (776)
            /// </summary>
            public float FPPositionX { get { return __fPPositionX__; } set { __fPPositionX__ = value; } }
            private float __fPPositionX__;
            /// <summary>
            /// 149. Float :: Offset: 0x30C (780)
            /// </summary>
            public float FPPositionY { get { return __fPPositionY__; } set { __fPPositionY__ = value; } }
            private float __fPPositionY__;
            /// <summary>
            /// 150. Float :: Offset: 0x310 (784)
            /// </summary>
            public float FPPositionZ { get { return __fPPositionZ__; } set { __fPPositionZ__ = value; } }
            private float __fPPositionZ__;
            /// <summary>
            /// 151. Float :: Offset: 0x314 (788)
            /// </summary>
            public float Unknown19 { get { return __unknown19__; } set { __unknown19__ = value; } }
            private float __unknown19__;
            /// <summary>
            /// 152. Float :: Offset: 0x318 (792)
            /// </summary>
            public float Unknown20 { get { return __unknown20__; } set { __unknown20__ = value; } }
            private float __unknown20__;

            #endregion

            /// <summary>
            /// Initializes a new instance of the WeapLayout class based
            /// on the supplied map reference ans tag identity.
            /// </summary>
            /// <param name="map">The map reference. </param>
            /// <param name="identity">The identity of the tag to be used by this class. </param>
            public WeapLayout(Map map, int identity)
            {
                this._Map = map;
                this._Identity = identity;
                this._Initialized = false;

                Flags1 = new Flags1Bitmask32();
                AiProperties = new _AiPropertiesTagBlock(this);
                Functions = new _FunctionsTagBlock(this);
                Attachments = new _AttachmentsTagBlock(this);
                Widgets = new _WidgetsTagBlock(this);
                ChangeColors = new _ChangeColorsTagBlock(this);
                PredictedResource1 = new _PredictedResource1TagBlock(this);
                FireEffectBitmaps = new _FireEffectBitmapsTagBlock(this);
                Flags2 = new Flags2Bitmask32();
                SecondaryTriggerMode = new SecondaryTriggerModeEnum32();
                MovementPenalized = new MovementPenalizedEnum32();
                WeaponType2 = new WeaponType2Enum16();
                Unknown12 = new Unknown12Enum32();
                FPModels = new _FPModelsTagBlock(this);
                PredictedResource2 = new _PredictedResource2TagBlock(this);
                Magazine = new _MagazineTagBlock(this);
                Triggers = new _TriggersTagBlock(this);
                Projectiles = new _ProjectilesTagBlock(this);
            }

            /// <summary>
            /// Opens up the map and validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            public void _Initialize()
            {
                BitReader br = new BitReader(_Map.FileName, Endian.Little);
                this._Initialize(br);
                br.Close();
            }

            /// <summary>
            /// Using the supplied reader, validates all reflexive existances in the 
            /// current tag meta, as well as gets a value for all base properties.
            /// </summary>
            /// <param name="br">The reader to be used for reading the tag meta. </param>
            public void _Initialize(BitReader br)
            {
                Flags1._Initialize(br);
                BoundingRadius = br.ReadSingle();
                BoundingOffsetX = br.ReadSingle();
                BoundingOffsetY = br.ReadSingle();
                BoundingOffsetZ = br.ReadSingle();
                AccelerationScale = br.ReadSingle();
                LightmapShadowMode = br.ReadSingle();
                SweetenerSize = br.ReadSingle();
                DynamicLightSphereRadius = br.ReadSingle();
                DynamicLightSphereOffsetX = br.ReadSingle();
                DynamicLightSphereOffsetY = br.ReadSingle();
                DynamicLightSphereOffsetZ = br.ReadSingle();
                DefaultModelVariant._Initialize(br);
                Model._Initialize(br);
                CrateObject._Initialize(br);
                ModifierShader._Initialize(br);
                CreationEffect._Initialize(br);
                MaterialEffects1._Initialize(br);
                AiProperties._Initialize(br);
                Functions._Initialize(br);
                RenderBoundingRadius = br.ReadSingle();
                Lower1 = br.ReadSingle();
                Upper1 = br.ReadSingle();
                Lower2 = br.ReadSingle();
                Upper2 = br.ReadSingle();
                Lower3 = br.ReadSingle();
                Upper3 = br.ReadSingle();
                Lower4 = br.ReadSingle();
                Upper4 = br.ReadSingle();
                HudTextMessageIndex = br.ReadInt16();
                ForcedShaderPermutationIndex = br.ReadInt16();
                Attachments._Initialize(br);
                Widgets._Initialize(br);
                Unused1 = br.ReadBytes(8);
                ChangeColors._Initialize(br);
                PredictedResource1._Initialize(br);
                ItemFlags = br.ReadSingle();
                MessageIndex = br.ReadInt16();
                SortOrder = br.ReadInt16();
                Scale = br.ReadSingle();
                HudMessageValueScale = br.ReadInt16();
                Unused2 = br.ReadBytes(2);
                Pickup._Initialize(br);
                Swap._Initialize(br);
                PickupOrDualWield._Initialize(br);
                SwapOrDualWield._Initialize(br);
                DualWield._Initialize(br);
                PickedUp._Initialize(br);
                AmmoSingular._Initialize(br);
                AmmoPlural._Initialize(br);
                SwitchedTo._Initialize(br);
                SwapAI._Initialize(br);
                MaterialEffects2._Initialize(br);
                CollisionSoundPossibly1._Initialize(br);
                FireEffectBitmaps._Initialize(br);
                CollisionSoundPossibly2._Initialize(br);
                DetonationDelayLower = br.ReadSingle();
                DetonationDelayUpper = br.ReadSingle();
                DetonatingEffect._Initialize(br);
                DetonationEffect._Initialize(br);
                Flags2._Initialize(br);
                Label._Initialize(br);
                SecondaryTriggerMode._Initialize(br);
                MaxAlternateShotsLoaded = br.ReadSingle();
                ReadyTime = br.ReadSingle();
                ReadyEffect._Initialize(br);
                Unused3._Initialize(br);
                Unknown1 = br.ReadSingle();
                HeatRecoveryThreshold = br.ReadSingle();
                OverheatedThreshold = br.ReadSingle();
                HeatDetonationThreshold = br.ReadSingle();
                HeatDetonationFraction = br.ReadSingle();
                HeatLossPerSecond = br.ReadSingle();
                HeatIllumination = br.ReadSingle();
                OverheatEffect._Initialize(br);
                Unused4._Initialize(br);
                Unused5._Initialize(br);
                Unused6._Initialize(br);
                Unused7._Initialize(br);
                Unused8._Initialize(br);
                Unknown2 = br.ReadSingle();
                Unknown3 = br.ReadSingle();
                Unknown4 = br.ReadSingle();
                Unknown5 = br.ReadSingle();
                Unknown6 = br.ReadSingle();
                Unknown7 = br.ReadSingle();
                Unknown8 = br.ReadSingle();
                Unknown9 = br.ReadSingle();
                MeleeDamage1._Initialize(br);
                MeleeResponseDamage1._Initialize(br);
                MeleeDamage2._Initialize(br);
                MeleeResponseDamage2._Initialize(br);
                SmashMelee._Initialize(br);
                SmashMeleeResponse._Initialize(br);
                DashMelee._Initialize(br);
                DashMeleeResponse._Initialize(br);
                Unknown10 = br.ReadInt16();
                ZoomLevels = br.ReadInt16();
                MinZoom = br.ReadSingle();
                MaxZoom = br.ReadSingle();
                AutoAimAngle_Radians = br.ReadSingle();
                AutoAimRange = br.ReadSingle();
                MagnetismAngle_Radians = br.ReadSingle();
                MagnetismRange = br.ReadSingle();
                DeviationAngle_Radians = br.ReadSingle();
                MovementPenalized._Initialize(br);
                ForwardsMovementPenalty = br.ReadSingle();
                BackwardsMovementPenalty = br.ReadSingle();
                SidewaysMovementPenalty = br.ReadSingle();
                AIUnknown1 = br.ReadSingle();
                AIUnknown2 = br.ReadSingle();
                AIMinimumTargetRange = br.ReadSingle();
                AILookingTimeModifier = br.ReadSingle();
                LightPowerOnTime = br.ReadSingle();
                LightPowerOffTime = br.ReadSingle();
                LightPowerOnEffect._Initialize(br);
                LightPowerOffEffect._Initialize(br);
                AgeHeatRecoveryPenalty = br.ReadSingle();
                AgeRateOfFirePenalty = br.ReadSingle();
                AgeMisfireStart = br.ReadSingle();
                AgeMisfireChance = br.ReadSingle();
                PickupSound._Initialize(br);
                ZoomInSound._Initialize(br);
                ZoomOutSound._Initialize(br);
                ActiveCamoDing = br.ReadSingle();
                Unused9 = br.ReadBytes(8);
                WeaponType1._Initialize(br);
                WeaponShortForm._Initialize(br);
                Unknown11 = br.ReadInt16();
                WeaponType2._Initialize(br);
                Unknown12._Initialize(br);
                Unused10 = br.ReadBytes(16);
                FPModels._Initialize(br);
                HUD._Initialize(br);
                PredictedResource2._Initialize(br);
                Magazine._Initialize(br);
                Triggers._Initialize(br);
                Projectiles._Initialize(br);
                Unknown13 = br.ReadSingle();
                Unknown14 = br.ReadSingle();
                Unknown15 = br.ReadSingle();
                Unknown16 = br.ReadSingle();
                Unknown17 = br.ReadSingle();
                Unknown18 = br.ReadSingle();
                AgedVehicle._Initialize(br);
                AgedEffect._Initialize(br);
                AgedWeapon._Initialize(br);
                FPPositionX = br.ReadSingle();
                FPPositionY = br.ReadSingle();
                FPPositionZ = br.ReadSingle();
                Unknown19 = br.ReadSingle();
                Unknown20 = br.ReadSingle();

                _Initialized = true;
            }
        }

        /// <summary>
        /// Initializes a new instance of the H2UtilityToolkit.Structures.Weap class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Weap(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
            this.Layout = new WeapLayout(map, identity);
        }

        #region Specific Methods of weap

        #endregion
    }
}
